package com.humaorie.dollar;

/*
 * Dollar, http://bitbucket.org/dfa/dollar
 * (c) 2010 Davide Angelocola <davide.angelocola@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
import com.humaorie.dollar.Dollar.Wrapper;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;

// TODO: this class is still under construction
class DateRangeWrapper extends AbstractWrapper<Date> {

    private final Date begin;
    private final Date end;
    private final int step;

    public DateRangeWrapper(Date begin, Date end) {
        this(begin, end, 1);
    }

    public DateRangeWrapper(Date begin, Date end, int step) {
        if (begin == null) {
            begin = new Date();
        }

        if (end == null) {
            end = new Date();
        }

        this.begin = begin;
        this.end = end;
        this.step = step;
    }

    @Override
    public Iterator<Date> iterator() {
        return new DateIterator(begin, end);
    }

    @Override
    public Wrapper<Date> copy() {
        return new DateRangeWrapper(begin, end, step);
    }

    @Override
    public int size() {
        throw new UnsupportedOperationException("not implemented");
    }

    private static class DateIterator implements Iterator<Date> {

        private Date current;
        private Date end;

        public DateIterator(Date begin, Date end) {
            this.current = begin;
            this.end = end;
        }

        @Override
        public boolean hasNext() {
            return current.before(end);
        }

        @Override
        public Date next() {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(current);
            calendar.add(Calendar.DAY_OF_MONTH, 1);
            current = calendar.getTime();
            return current;
        }

        @Override
        public void remove() {
            throw new UnsupportedOperationException("remove() not supported for DateWrapper");
        }
    }

    @Override
    public boolean equals(Object object) {
        if (object instanceof DateRangeWrapper) {
            DateRangeWrapper dateRangeWrapper = (DateRangeWrapper) object;
            return dateRangeWrapper.begin.equals(begin) && dateRangeWrapper.end.equals(end);
        } else {
            return false;
        }
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash *= 53 + begin.hashCode();
        hash *= 53 + end.hashCode();
        return hash;
    }
}
