//***********************************************************************
//
//                              ConfigFIS.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.util.*;
import java.io.*;
import javax.swing.*;

//------------------------------------------------------------------------------
public class ConfigFIS
{
  private Properties p;
  private static String config_name;
  private String userHomeFisproPath;
//------------------------------------------------------------------------------
  public ConfigFIS() throws Throwable
  {
  userHomeFisproPath = System.getProperty("user.home") + System.getProperty("file.separator") + ".fispro";
  new File(userHomeFisproPath).mkdirs();
  config_name = userHomeFisproPath + System.getProperty("file.separator") + "fispro.cnf";
  p = new Properties();
  try
    {
    // ouverture du fichier de config
    FileInputStream f = new FileInputStream(config_name);
    p.load(f);
    // verifie que les chemins FISFilePath et DataFilePath existent
    try
      {
      File fisf = new File(p.getProperty("FISFilePath"));
      if (fisf.exists() == false)
        p.setProperty("FISFilePath", System.getProperty("user.home"));
      }
    catch( Exception e ) {}
    try
      {
      File dataf = new File(p.getProperty("DataFilePath"));
      if (dataf.exists() == false)
        p.setProperty("DataFilePath", System.getProperty("user.home"));
      }
    catch (Exception e) {}
    }
  catch( Exception e ) {}
  }
//------------------------------------------------------------------------------
  public void Save()
  {
  try { p.store(new FileOutputStream(config_name),"FIS Config File"); }
  catch( Exception e ) { MessageFIS.Error(null, e ); }
  }
//------------------------------------------------------------------------------
  public String Language() { return p.getProperty("Language", LocaleFIS.DefaultLanguage()); }
//------------------------------------------------------------------------------
  public String Country() { return p.getProperty("Country", LocaleFIS.DefaultCountry()); }
//------------------------------------------------------------------------------
  public String FISFilePath() { return p.getProperty("FISFilePath", System.getProperty("user.home")); }
//------------------------------------------------------------------------------
  public String DataFilePath() { return p.getProperty("DataFilePath", System.getProperty("user.home")); }
//------------------------------------------------------------------------------
  public String LookAndFeel() { return p.getProperty("LookAndFeel", LookFIS.DefaultLookAndFeel()); }
//------------------------------------------------------------------------------
  public void Language( String language ) { p.setProperty("Language", language); Save(); }
//------------------------------------------------------------------------------
  public void Country( String country ) { p.setProperty("Country", country); Save(); }
//------------------------------------------------------------------------------
  public void FISFilePath( String FIS_path_file ) { p.setProperty("FISFilePath", FIS_path_file); Save(); }
//------------------------------------------------------------------------------
  public void DataFilePath( String data_path_file ) throws Throwable
  {
  jnifis.SetDataFilePath(data_path_file + System.getProperty("file.separator"));
  p.setProperty("DataFilePath", data_path_file);
  Save();
  }
//------------------------------------------------------------------------------
  public void LookAndFeel( String laf ) { p.setProperty("LookAndFeel", laf); Save(); }
//------------------------------------------------------------------------------
  public String GetUserHomeFisproPath() { return this.userHomeFisproPath; }
}