//***********************************************************************
//
//                              DataChooserFrame.java
//
// Author(s) : Jean-luc LABLEE, Mathieu GRELIER
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.TitledBorder;
import javax.swing.plaf.basic.BasicComboBoxRenderer;

import org.freehep.j3d.plot.View3DScatterPlot;

//------------------------------------------------------------------------------
public class DataChooserFrame extends JFISDialog
{
	GridBagLayout gridBagLayout = new GridBagLayout();
	//panel histogram
	JPanel jPanelHistogram = new JPanel();
	GridBagLayout gridBLHistogram = new GridBagLayout();
	TitledBorder titledBorderHistogram;
	JButton jButtonAll = new JButton();
	Vector <JCheckBox> CheckBoxs = new Vector<JCheckBox>();
	// panel XY
	JPanel jPanelXY = new JPanel();
	JPanel jPanelXYZ = new JPanel(); //MG : Visu3D
	GridBagLayout gridBLXY = new GridBagLayout();
	GridBagLayout gridBLXYZ = new GridBagLayout();  //MG : Visu3D
	TitledBorder titledBorderXY;
	TitledBorder titledBorderXYZ; //MG : Visu3D
	JLabel jLabelX = new JLabel("X:");
	JLabel jLabelY = new JLabel("Y:");
	JComboBox jCBX = new JComboBox();
	JComboBox jCBY = new JComboBox();

	JLabel jLabel3DX = new JLabel("X:");//MG : Visu3D
	JLabel jLabel3DY = new JLabel("Y:");
	JLabel jLabel3DZ = new JLabel("Z:");
	JComboBox jCB3DX = new JComboBox();
	JComboBox jCB3DY = new JComboBox();
	JComboBox jCB3DZ = new JComboBox();

	// panel validation
	JPanel jPanelValidation = new JPanel();
	GridBagLayout gridBLValidation = new GridBagLayout();
	JButton jButtonApply = new JButton();
	JButton jButtonCancel = new JButton();

	private JFISFrame Parent; // fenetre parent
	JFIS sif;   // objet sif
	private JExtendedDataFile DataFile;   // objet fichier de donnees
	private JExtendedFileListener file_listener = null; //data file listener
	private View3DScatterPlot scatter ;
	private JPanel jPanelFrame = new JPanel(new GridBagLayout());

	//------------------------------------------------------------------------------
	public DataChooserFrame(JFISFrame parent, JFIS sif, JExtendedDataFile data_file)
	{
		this.Parent = parent;
		this.sif = sif;
		this.DataFile = data_file;
		try { jbInit(); }
		catch(Throwable e) { MessageFIS.Error(null, e); }
	}
	//------------------------------------------------------------------------------
	private void jbInit() throws Throwable
	{
		this.getContentPane().add(this.jPanelFrame);
		this.setTitle(LocaleFIS.GetString("DataChooser"));

		// panel histogram
		jPanelHistogram.setLayout(gridBLHistogram);
		titledBorderHistogram = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),LocaleFIS.GetString("Histogram"));
		jPanelHistogram.setBorder(titledBorderHistogram);
		final JPanel inner_panel = new JPanel(new GridBagLayout());
		JScrollPane jsp = new JScrollPane(inner_panel) { public Dimension getMinimumSize(){ return new Dimension(150, Math.min(150, inner_panel.getMinimumSize().height)); } };
		this.jPanelHistogram.add(jsp,       new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
		jButtonAll.setText(LocaleFIS.GetString("All"));
		jButtonAll.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonAll_actionPerformed(e); } });
		this.jPanelFrame.add(jPanelHistogram,       new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 0, 0), 0, 0));
		this.jPanelHistogram.add(jButtonAll, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 0, 0));
		// ajout des checkboxs dans le panel histogram
		for( int i=0 ; i<this.DataFile.VariableCount() ; i++ )
		{
			JCheckBox cb = new JCheckBox(this.VariableName(i));
			if( this.sif != null )
			{
				if( (i < this.sif.NbInputs()) && (this.sif.GetInput(i).GetActive() == false) )
					cb.setForeground(Color.gray);
				if( (i >= this.sif.NbInputs()) && (this.sif.GetOutput(i-this.sif.NbInputs()).GetActive() == false) )
					cb.setForeground(Color.gray);
			}
			this.CheckBoxs.add(cb);
			inner_panel.add(cb, new GridBagConstraints(0, i+1, 1, 1, 0.0, 0.0
					,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
		}

		// panel XY
		jPanelXY.setLayout(gridBLXY);
		titledBorderXY = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"XY");
		jPanelXY.setBorder(titledBorderXY);
		jPanelXY.add(jLabelX,  new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
		jPanelXY.add(jLabelY,  new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
		jPanelXY.add(jCBX, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
		jPanelXY.add(jCBY,  new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
		this.jPanelFrame.add(jPanelXY,   new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 0, 0, 5), 0, 0));
		// remplissage des comboboxs
		DefaultComboBoxModel dcbmx = new DefaultComboBoxModel();
		dcbmx.addElement(new String());   // premier element vide
		DefaultComboBoxModel dcbmy = new DefaultComboBoxModel();
		dcbmy.addElement(new String());   // premier element vide
		for( int i=0 ; i<this.DataFile.VariableCount() ; i++ )
		{
			dcbmx.addElement(this.VariableName(i));
			dcbmy.addElement(this.VariableName(i));
		}
		this.jCBX.setModel(dcbmx);
		this.jCBY.setModel(dcbmy);
		// renderer des combobox
		this.jCBX.setRenderer(new BasicComboBoxRenderer()
		{
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
			{
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				try
				{
					if( (index > 0) && (index <= sif.NbInputs()) && (sif.GetInput(index-1).GetActive() == false) )
						this.setForeground(Color.gray);
					if( (index > sif.NbInputs()) && (sif.GetOutput(index-1-sif.NbInputs()).GetActive() == false) )
						this.setForeground(Color.gray);
				}
				catch( Throwable t ) {}
				return this;
			}
		});
		this.jCBY.setRenderer(new BasicComboBoxRenderer()
		{
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
			{
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				try
				{
					if( (index > 0) && (index <= sif.NbInputs()) && (sif.GetInput(index-1).GetActive() == false) )
						this.setForeground(Color.gray);
					if( (index > sif.NbInputs()) && (sif.GetOutput(index-1-sif.NbInputs()).GetActive() == false) )
						this.setForeground(Color.gray);
				}
				catch( Throwable t ) {}
				return this;
			}
		});

		//panel XYZ
		jPanelXYZ.setLayout(gridBLXYZ);
		titledBorderXYZ = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"XYZ");
		jPanelXYZ.setBorder(titledBorderXYZ);
		jPanelXYZ.add(jLabel3DX,  new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
		jPanelXYZ.add(jLabel3DY,  new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
		jPanelXYZ.add(jLabel3DZ,  new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
		jPanelXYZ.add(jCB3DX, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
		jPanelXYZ.add(jCB3DY,  new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
		jPanelXYZ.add(jCB3DZ,  new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
		this.jPanelFrame.add(jPanelXYZ,   new GridBagConstraints(2, 1, 1, 1, 1.0, 1.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 0, 0, 5), 0, 0));
		// remplissage des comboboxs
		DefaultComboBoxModel dcbmX = new DefaultComboBoxModel();
		dcbmX.addElement(new String());   // premier element vide
		DefaultComboBoxModel dcbmY = new DefaultComboBoxModel();
		dcbmY.addElement(new String());   // premier element vide
		DefaultComboBoxModel dcbmZ = new DefaultComboBoxModel();
		dcbmZ.addElement(new String());   // premier element vide

		for( int i=0 ; i<this.DataFile.VariableCount() ; i++ )
		{
			dcbmX.addElement(this.VariableName(i));
			dcbmY.addElement(this.VariableName(i));
			dcbmZ.addElement(this.VariableName(i));
		}
		this.jCB3DX.setModel(dcbmX);
		this.jCB3DY.setModel(dcbmY);
		this.jCB3DZ.setModel(dcbmZ);
		// renderer des combobox
		this.jCB3DX.setRenderer(new BasicComboBoxRenderer()
		{
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
			{
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				try
				{
					if( (index > 0) && (index <= sif.NbInputs()) && (sif.GetInput(index-1).GetActive() == false) )
						this.setForeground(Color.gray);
					if( (index > sif.NbInputs()) && (sif.GetOutput(index-1-sif.NbInputs()).GetActive() == false) )
						this.setForeground(Color.gray);
				}
				catch( Throwable t ) {}
				return this;
			}
		});
		this.jCB3DY.setRenderer(new BasicComboBoxRenderer()
		{
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
			{
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				try
				{
					if( (index > 0) && (index <= sif.NbInputs()) && (sif.GetInput(index-1).GetActive() == false) )
						this.setForeground(Color.gray);
					if( (index > sif.NbInputs()) && (sif.GetOutput(index-1-sif.NbInputs()).GetActive() == false) )
						this.setForeground(Color.gray);
				}
				catch( Throwable t ) {}
				return this;
			}
		});
		this.jCB3DZ.setRenderer(new BasicComboBoxRenderer()
		{
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
			{
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				try
				{
					if( (index > 0) && (index <= sif.NbInputs()) && (sif.GetInput(index-1).GetActive() == false) )
						this.setForeground(Color.gray);
					if( (index > sif.NbInputs()) && (sif.GetOutput(index-1-sif.NbInputs()).GetActive() == false) )
						this.setForeground(Color.gray);
				}
				catch( Throwable t ) {}
				return this;
			}
		});



		// panel validation
		jPanelValidation.setLayout(gridBLValidation);
		jButtonApply.setText(LocaleFIS.GetString("Apply"));
		jButtonApply.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonApply_actionPerformed(e); } });
		jButtonCancel.setText(LocaleFIS.GetString("Cancel"));
		jButtonCancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonCancel_actionPerformed(e); } });
		this.jPanelFrame.add(jPanelValidation,    new GridBagConstraints(0, 2, 3, 1, 1.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
		jPanelValidation.add(jButtonApply,   new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		jPanelValidation.add(jButtonCancel,   new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));

		// affichage de la boite de dialogue
		this.pack();
		this.setModal(true);
		this.setLocation(JFISChildFrame.ChildPosition(this.Parent, this.getSize()));
		this.setVisible(true);
	}
	//------------------------------------------------------------------------------
	void jButtonCancel_actionPerformed(ActionEvent e) { this.dispose(); }
	//------------------------------------------------------------------------------
	void jButtonApply_actionPerformed(ActionEvent e)
	{
		try
		{
			// recherche des checkboxs selectionnees
			for( int i=0 ; i<this.CheckBoxs.size() ; i++ )
				if( ((JCheckBox)this.CheckBoxs.elementAt(i)).isSelected() == true )
				{
					// ouverture d'une fenetre DataHistogramFrame
					DataHistogramInputFrame dhf = new DataHistogramInputFrame(this.Parent, this.sif, i, i, this.DataFile);
				}

			// affichage de la fenetre XY si des variables sont selectionnees
			final int xi = this.jCBX.getSelectedIndex();
			final int yi = this.jCBY.getSelectedIndex();
			if( (xi != 0) && (yi != 0) )
			{
				// ouverture d'une fenetre XY
				final JDataFile jdf = new JDataFile(this.DataFile.ActiveFileName());
				final JFileListener jfl = new JExtendedFileListener()
				{
					public void Closed() { jdf.Close(); }
					public void ReLoaded() { try { jdf.Reload(DataChooserFrame.this.DataFile.ActiveFileName()); } catch( Throwable t ) { MessageFIS.Error(null, t); } }
					public void ActivationChanged(int item, boolean active) { try { jdf.Reload(DataChooserFrame.this.DataFile.ActiveFileName()); } catch(Throwable t) {}}
					public void ActivationChanged(int []items, boolean active) { try { jdf.Reload(DataChooserFrame.this.DataFile.ActiveFileName()); } catch(Throwable t) {}}
					public void ActivationChanged(int [][]items_and_status){}
				};
				this.DataFile.AddJFileListener(jfl);
				ExtendedDataXYFrame dxyf = new ExtendedDataXYFrame( this.Parent, xi-1, yi-1, this.sif, xi-1, yi-1, jdf, this.DataFile )
				{
					public void dispose()
					{
						super.dispose();
						jdf.RemoveJFileListener(jfl);
					}
				};
			}
			//visu 3D
			// affichage de la fenetre XYZ si des variables sont selectionnees
			final int x3Di = this.jCB3DX.getSelectedIndex();
			final int y3Di = this.jCB3DY.getSelectedIndex();
			final int z3Di = this.jCB3DZ.getSelectedIndex();

			if( (x3Di != 0) && (y3Di != 0) && (z3Di != 0) )
			{
				if(MainFIS.isJava3DIsInstalled())
					(new JFrame3DData(this.Parent)).show(this.DataFile, x3Di, y3Di, z3Di, (String)this.jCB3DX.getSelectedItem(), (String)this.jCB3DY.getSelectedItem(), (String)this.jCB3DZ.getSelectedItem());
			}
			this.dispose();
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	void jButtonAll_actionPerformed(ActionEvent e)
	{
		for( int i=0 ; i<this.CheckBoxs.size() ; i++ )
			((JCheckBox)this.CheckBoxs.elementAt(i)).setSelected(true);
	}
	//------------------------------------------------------------------------------
	private String VariableName( int index ) throws Throwable
	{
		if( this.sif == null ) {   // il n'y a pas de sif ouvert
			if( this.DataFile.getVariableName() == null )
				return LocaleFIS.GetString("Variable") + " " + Integer.toString(index+1); // retourne variable index
			else
				return this.DataFile.getVariableName()[index];
		}
		else                      // un sif est ouvert
			return this.GetInput(index).GetName();  // retourne le nom de l'entree
	}
	//------------------------------------------------------------------------------
	private JInput GetInput( int index ) throws Throwable
	{
		if( this.sif != null )
			if( index < this.sif.NbInputs() )   // index correspond a une entree
				return this.sif.GetInput(index);
			else                                // index correspond a une sortie
				return this.sif.GetOutput(index-this.sif.NbInputs());
		else
			return null;
	}
}


