//***********************************************************************
//
//                              DataHistogramFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;
import java.util.*;
import java.awt.print.*;
import ptolemy.plot.*;
//needed classes for exporting graphics
import org.freehep.graphics2d.*;
import org.freehep.graphicsio.*;
import org.freehep.graphicsio.exportchooser.*;
import org.freehep.graphicsio.ps.*;
import org.freehep.graphicsio.png.*;
import org.freehep.graphicsio.jpg.*;
import org.freehep.graphicsio.pdf.*;
import org.freehep.util.export.*;



//------------------------------------------------------------------------------
public class DataHistogramFrame extends JFISChildFrame
{
	private JPanel jPanelHisto = new JPanel();
	private JLabel jLabelNumberOfClass = new JLabel();
	private IntegerField jIFNumberOfClass = new IntegerField();
	private JButton jButtonApply = new JButton();
	protected HistogramPanel jGraphHistogram;
	private int DefaultNumberOfClasses = 15;
	protected JPanel jHistogramFramePanel;

	protected int DataIndex;
	public JFile DataFile;
	private JFileListener JFileListener;
	private String Title;
	private JMenu jMenuFile = new JMenu();
	private JMenuItem jMenuPrint = new JMenuItem();
	private JMenuItem jMenuExport = new JMenuItem();
	//------------------------------------------------------------------------------
	public DataHistogramFrame( JFISFrame parent, String title, int data_index, JFile data_file )
	{
		super(parent);
		try
		{
			jbInit(title, data_index, data_file);
			this.Show();
		}
		catch(Throwable t) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	public DataHistogramFrame( JFISFrame parent ) { super(parent); }
	//------------------------------------------------------------------------------
	protected void jbInit( String title, int data_index, JFile data_file ) throws Throwable
	{
		JMenuBar jmb = new JMenuBar();
		jmb.add(this.jMenuFile);
		this.jMenuFile.add(this.jMenuPrint);
		this.jMenuFile.add(this.jMenuExport);
		this.setJMenuBar(jmb);

		this.Title = title;
		this.DataIndex = data_index;
		this.DataFile = data_file;
		this.jHistogramFramePanel = new JPanel(new GridBagLayout());
		this.getContentPane().add(this.jHistogramFramePanel);

		// panel histogramme
		this.jPanelHisto.setLayout(new GridBagLayout());
		this.jHistogramFramePanel.add(jPanelHisto,     new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
		this.jPanelHisto.add(this.jLabelNumberOfClass,     new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
		this.jIFNumberOfClass.setValue(this.DefaultNumberOfClasses);
		this.jIFNumberOfClass.SetAction( new NumberFieldActionListener()
		{
			public void NumberFieldAction()
			{
				if( DataHistogramFrame.this.jIFNumberOfClass.getValue() <= 0 )
					DataHistogramFrame.this.jIFNumberOfClass.setValue(DataHistogramFrame.this.DefaultNumberOfClasses);
			}
		} );
		this.jPanelHisto.add(this.jIFNumberOfClass,     new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 30, 0));
		this.jButtonApply.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e) { DataHistogramFrame.this.ApplyNumberOfClasses(); }
		});
		this.jPanelHisto.add(this.jButtonApply,     new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
		this.jGraphHistogram = this.HistogramPanel();
		this.jGraphHistogram.SetNbBins(this.DefaultNumberOfClasses);
		this.jGraphHistogram.AddData(this.DataFile.VariableData(this.DataIndex));
		this.jPanelHisto.add(this.jGraphHistogram,     new GridBagConstraints(0, 1, 3, 1, 1.0, 1.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 0, 0, 0), 0, 0));

		// events
		this.JFileListener = new JFileListener()
		{
			public void Closed() { DataHistogramFrame.this.dispose(); }
			public void ReLoaded() { DataHistogramFrame.this.DataReLoaded(); }
		};
		this.DataFile.AddJFileListener(this.JFileListener);

		jMenuPrint.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuPrint_actionPerformed(e); } });

		// adding listener on export menu
		// action performed builds a new export dialog
		// wanted export file types are explicitely added
		// if not, dialog crashes

		jMenuExport.addActionListener( new ActionListener()
		{
			public void actionPerformed( ActionEvent e )
			{
				ExportDialog export = new ExportDialog();
				export.showExportDialog( jGraphHistogram, "Export view as ...", DataHistogramFrame.this.jHistogramFramePanel, "export" );
			}
		});
	}
	//------------------------------------------------------------------------------
	protected void ApplyNumberOfClasses() { DataHistogramFrame.this.jGraphHistogram.SetNbBins(DataHistogramFrame.this.jIFNumberOfClass.getValue()); }
	//------------------------------------------------------------------------------
	protected void DataReLoaded()
	{
		this.jGraphHistogram.AddData(this.DataFile.VariableData(this.DataIndex));
		this.repaint();
	}
	//------------------------------------------------------------------------------
	protected HistogramPanel HistogramPanel() throws Throwable { return new HistogramPanel(); }
	//------------------------------------------------------------------------------
	public void Show()
	{
		// affichage
		this.setTitle(this.Title);
		this.Translate();
		this.pack();
		this.setLocation(this.ChildPosition(this.getSize()));
		this.setVisible(true);
	}
	//------------------------------------------------------------------------------
	public void dispose()
	{
		super.dispose();
		this.DataFile.RemoveJFileListener(this.JFileListener);
	}
	//------------------------------------------------------------------------------
	public void Translate()
	{
		this.setTitle(this.Title);
		this.jMenuFile.setText(LocaleFIS.GetString("File"));
		this.jLabelNumberOfClass.setText(LocaleFIS.GetString("NumberOfClasses")+":");
		this.jButtonApply.setText(LocaleFIS.GetString("Apply"));
		this.jMenuPrint.setText(LocaleFIS.GetString("Print"));
		this.jMenuExport.setText(LocaleFIS.GetString("Export"));
		this.repaint();
	}
	//------------------------------------------------------------------------------
	void jMenuPrint_actionPerformed(ActionEvent e)
	{
		PrinterJob job = PrinterJob.getPrinterJob();
		if( job.printDialog() == true )
		{
			job.setPrintable(new Printable()
			{
				public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) throws PrinterException
				{
					if (pageIndex >= 1) { return Printable.NO_SUCH_PAGE; }
					else return DataHistogramFrame.this.print(graphics,pageFormat,pageIndex);
				}
			});
			try { job.print(); }
			catch (PrinterException ex) { ex.printStackTrace(); }
		}
	}
	//------------------------------------------------------------------------------
	public int print( java.awt.Graphics g, PageFormat pageFormat, int pageIndex ) throws PrinterException
	{
		java.awt.Graphics2D  g2 = ( java.awt.Graphics2D )g;
		double scalew=1;
		double scaleh=1;
		double pageHeight = pageFormat.getImageableHeight();
		// See how much we need to scale the image to fit in the page's width
		double pageWidth = pageFormat.getImageableWidth();
		if(  getWidth() >= pageWidth ) { scalew =  pageWidth / getWidth(); }
		if(  getHeight() >= pageHeight) { scaleh =  pageWidth / getHeight(); }
		g2.translate( pageFormat.getImageableX(), pageFormat.getImageableY() );
		g2.scale( scalew, scaleh );
		this.jHistogramFramePanel.print( g2 );
		return Printable.PAGE_EXISTS;
	}
}


//------------------------------------------------------------------------------
// Classe d'affichage des SEFs

class JPanelInput extends FISPlot implements PlotRectangleListener
{
	protected JInput Input;
	protected Rectangle listener_rect;
	//------------------------------------------------------------------------------
	public Dimension getPreferredSize() { return new Dimension(400,200); }
	public Dimension getMinimumSize() { return  new Dimension(100,100); }
	//------------------------------------------------------------------------------
	public JPanelInput( JInput input ) throws Throwable
	{
		this.setColor(false);
		this.setYRange(0, 1);
		this.AddYTicks();
		FISPlot.DisableZoom(this);
		if( input != null )
			this.SetInput(input);
	}
	//------------------------------------------------------------------------------
	protected void AddYTicks()
	{
		this.addYTick("0", 0);
		this.addYTick("", 0.5);
		this.addYTick("1", 1);
	}
	//------------------------------------------------------------------------------
	public void SetInput( JInput input ) throws Throwable
	{
		this.Input = input;
		this.setXRange(this.Input.GetRange()[0], this.Input.GetRange()[1]);
		this.repaint();
	}
	//------------------------------------------------------------------------------
	public void PlotRectangle( Rectangle rect )
	{
		this.listener_rect = rect;
		this.repaint();
	}
	//------------------------------------------------------------------------------
	public void paintComponent(Graphics g)
	{
		try
		{
			// affichage de chaque SEF
			this.clear(false);
			for (int i = 0; i < this.Input.GetNbMF(); i++)
				this.Input.GetMF(i).NewMF().Draw(this, i);
			super.paintComponent(g);

			// ajustement de la taille du rectangle plot au listener
			this.setPlotRectangle(this.listener_rect);
			super.paintComponent(g);
			this.setPlotRectangle(null);  // supprime la memorisation du setPlotRectangle

			// affichage des bornes du domaine de l'entree
			if( this.Input != null )
				FISPlot.DrawXInputRange(g, this, this.Input.GetRange(), this._xMin, this._xMax);
		}
		catch( Throwable t ) { MessageFIS.Error( null, t ); }
	}
}


//------------------------------------------------------------------------------
interface PlotRectangleListener
{
	public void PlotRectangle( Rectangle rect );
}


//------------------------------------------------------------------------------
// Classe d'affichage de l'histogramme

class HistogramPanel extends Histogram
{
	protected int dataset = 1;
	protected int nb_bins = 10;
	private Vector listeners = new Vector();
	private double [] data_range = new double[2];
	protected double [] adjust_range = new double[2];
	//------------------------------------------------------------------------------
	public Dimension getPreferredSize() { return new Dimension(400,200); }
	public Dimension getMinimumSize() { return  new Dimension(100,200); }
	//------------------------------------------------------------------------------
	public HistogramPanel() throws Throwable
	{
		this.setBars(0.9, 0);
		FISPlot.DisableZoom(this);
	}
	//------------------------------------------------------------------------------
	public void AdjustXRange() { this.setXRange(this.adjust_range[0], this.adjust_range[1]); }
	//------------------------------------------------------------------------------
	public void AddData( double data[] )
	{
		this.data_range[0] = this.data_range[1] = data[0];
		for( int i=1 ; i<data.length ; i++ )
		{
			this.data_range[0] = Math.min(this.data_range[0], data[i]);
			this.data_range[1] = Math.max(this.data_range[1], data[i]);
		}
		double binwidth = this.nb_bins == 1 ? (this.data_range[1]-this.data_range[0])*2 : (this.data_range[1]-this.data_range[0])/(this.nb_bins-1);  // recalcul du binwidth
		this.setBinWidth(binwidth);
		this.clear(false);
		for( int i=0 ; i<data.length ; i++ )
			if( !Double.isNaN((double)data[i]) ) // le cast en double de data[i] est indispensable pour pour une gestion correcte des valeurs NaN
				this.addPoint(this.dataset, data[i]);
		this.adjust_range[0] = this._xBottom-(binwidth/2);
		this.adjust_range[1] = this._xTop;
		this.AdjustXRange();
		this.repaint();
	}
	//------------------------------------------------------------------------------
	public void SetNbBins( int nb_bins )
	{
		this.nb_bins = nb_bins;
		if( this._points.size() != 0 )  // si l'histogram contient des points
		{
			double binwidth = this.nb_bins == 1 ? (this.data_range[1]-this.data_range[0])*2 : (this.data_range[1]-this.data_range[0])/(this.nb_bins-1);  // recalcul du binwidth
			this.setBinWidth(binwidth);
			Vector points = (Vector)this._points.elementAt(this.dataset);
			this.clear(false);
			for (int i = 0; i < points.size(); i++)
				this.addPoint(this.dataset, ( (Double) points.elementAt(i)).doubleValue());
			this.adjust_range[0] = this._xBottom-(binwidth/2);
			this.adjust_range[1] = this._xTop;
			this.AdjustXRange();
			this.repaint();
		}
	}
	//------------------------------------------------------------------------------
	public void AddPlotRectangleListener( PlotRectangleListener listener ) { this.listeners.add(listener); }
	//------------------------------------------------------------------------------
	public void paintComponent(Graphics g)
	{
		try
		{
			// ajustement de la taille du rectangle plot au panel
			super.paintComponent(g);
			FISPlot.AdjustPlotRectangleSize(g, this);
			super.paintComponent(g);
			this.setPlotRectangle(null);  // supprime la memorisation du setPlotRectangle
			// envoi de la taille du rectangle aux listeners
			for( int i=0 ; i<this.listeners.size() ; i++ )
				((PlotRectangleListener)this.listeners.elementAt(i)).PlotRectangle(this.getPlotRectangle());
		}
		catch( Throwable t ) { MessageFIS.Error( null, t ); }
	}
}


//------------------------------------------------------------------------------
class HistogramInputPanel extends HistogramPanel
{
	private JInput Input;
	//------------------------------------------------------------------------------
	public HistogramInputPanel( JInput input ) throws Throwable
	{
		super();
		this.SetInput(input);
	}
	//------------------------------------------------------------------------------
	public void SetInput( JInput input ) throws Throwable
	{
		this.Input = input;
		this.repaint();
	}
	//------------------------------------------------------------------------------
	public void paintComponent(Graphics g)
	{
		try
		{
			super.paintComponent(g);
			if( this.Input != null )
			{
				// affichage des bornes du domaine de l'entree
				FISPlot.DrawXInputRange(g, this, this.Input.GetRange(), this._xMin, this._xMax);
			}
		}
		catch( Throwable t ) {}
	}
}
