//***********************************************************************
//
//                              DataHistogramInputFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import static fis.JOutput.CRISP;
import static fis.util.JFISUtil.isFISOutputNature;
import static fis.util.JFISUtil.isOutput;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

//------------------------------------------------------------------------------
//fenetre data histogram avec affichage du panel de l'entree ou sortie correspondante
class DataHistogramInputFrame extends DataHistogramFrame
{
private JFIS sif;
private int FISIndex;
private JPanelInput jPanelInput;
private FISListener FISListener;
private double [] input_range;
private JPanelPartitionInfo panelPartitionInfo = new JPanelPartitionInfo(new PartitionInfo());
//------------------------------------------------------------------------------
public DataHistogramInputFrame( JFISFrame parent, JFIS fis, int fis_index, int data_index, JFile data_file ) throws Throwable
{
super(parent);
this.sif = fis;
this.FISIndex = fis_index;
this.DataFile = data_file; // for this.GetVariableName
super.jbInit( this.GetVariableName(this.FISIndex), data_index, data_file);
// panel entree
if( this.sif != null )
 {
 this.jPanelInput = new JPanelInput(null);
 this.jGraphHistogram.AddPlotRectangleListener(this.jPanelInput);
 this.jHistogramFramePanel.add(jPanelInput,     new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0
       ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 5, 0, 5), 0, 0));
 if( this.FISListener != null )
   this.sif.RemoveFISListener(this.FISListener);
 if(!(isOutput(fis, fis_index) && isFISOutputNature(fis, fis_index, CRISP)))
	 this.jHistogramFramePanel.add(panelPartitionInfo, new GridBagConstraints(0, 2, 1, 1, 1.0, 0.0 ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
 this.FISListener = new FISListener()
   {
   public void FISClosed() { DataHistogramInputFrame.this.dispose(); }
   public void InputReplaced( int input_number )       { DataHistogramInputFrame.this.InputChanged( input_number); }
   public void OutputReplaced( int output_number )     { DataHistogramInputFrame.this.OutputChanged( output_number); }
   public void InputNameChanged( int input_number )    { DataHistogramInputFrame.this.InputChanged( input_number); }
   public void OutputNameChanged( int output_number )  { DataHistogramInputFrame.this.OutputChanged( output_number); }
   public void InputRangeChanged( int input_number )   { DataHistogramInputFrame.this.InputChanged( input_number); }
   public void OutputRangeChanged( int output_number ) { DataHistogramInputFrame.this.OutputChanged( output_number); }
   public void MFRemovedInInput( int input_number )    { DataHistogramInputFrame.this.InputChanged( input_number); }
   public void MFRemovedInOutput( int output_number )  { DataHistogramInputFrame.this.OutputChanged( output_number); }
   public void MFAddedInInput( int input_number )      { DataHistogramInputFrame.this.InputChanged( input_number); }
   public void MFAddedInOutput( int output_number )    { DataHistogramInputFrame.this.OutputChanged( output_number); }
   public void MFReplacedInInput( int input_number )   { DataHistogramInputFrame.this.InputChanged( input_number); }
   public void MFReplacedInOutput( int output_number ) { DataHistogramInputFrame.this.OutputChanged( output_number); }
   public void InputActiveChanged( int input_number ) {}
   public void OutputActiveChanged( int output_number ) {}
   public void InputRemoved( int input_number ) {}
   public void OutputRemoved( int output_number ) {}
   public void InputAdded( int input_number ) {}
   public void OutputAdded( int output_number ) {}
   public void OutputDefuzChanged( int output_number ) {}
   public void OutputImpliChanged( int output_number ) {}
   public void OutputDisjChanged( int output_number ) {}
   public void OutputClassifChanged( int output_number ) {}
   public void OutputDefaultChanged( int output_number ) {}
   public void OutputAlarmThresChanged( int output_number ) {}
   public void RulesModified() {}
   public void ConjunctionChanged() {}
   };
 this.sif.AddFISListener(this.FISListener);
 }
this.InitObjectsWithInput();
this.addWindowListener(new WindowAdapter() {

	@Override
	public void windowOpened(WindowEvent e) {
		super.windowOpened(e);
		toFront();
	}});
this.Show();
}
//------------------------------------------------------------------------------
private void AdjustXRange()
{
this.jPanelInput.setXRange(this.input_range[0], this.input_range[1]);
this.jGraphHistogram.AdjustXRange();
this.jPanelInput.setXRange(Math.min(this.jPanelInput.getXRange()[0],this.jGraphHistogram.getXRange()[0]),Math.max(this.jPanelInput.getXRange()[1],this.jGraphHistogram.getXRange()[1]));
this.jPanelInput.repaint();
this.jGraphHistogram.setXRange(Math.min(this.jPanelInput.getXRange()[0],this.jGraphHistogram.getXRange()[0]),Math.max(this.jPanelInput.getXRange()[1],this.jGraphHistogram.getXRange()[1]));
this.jGraphHistogram.repaint();
}
//------------------------------------------------------------------------------
public void dispose()
{
super.dispose();
if( this.sif != null )
 this.sif.RemoveFISListener(this.FISListener);
}
//------------------------------------------------------------------------------
protected HistogramPanel HistogramPanel() throws Throwable { return new HistogramInputPanel(this.GetInput(this.FISIndex)/*, this.Histogram*/); }
//------------------------------------------------------------------------------
protected void DataReLoaded()
{
try
 {
 super.DataReLoaded();
 JInput input = this.GetInput(this.FISIndex);
 if( input != null )
   this.AdjustXRange();
 this.repaint();
 }
catch( Throwable t ) { MessageFIS.Error(null, t); }
}
//------------------------------------------------------------------------------
private void InitObjectsWithInput() throws Throwable
{
JInput input = this.GetInput(this.FISIndex);
if( input != null )
 {
 if( input instanceof JOutput )
   this.jPanelInput.setVisible( ((JOutput)input).GetNature() == JOutput.CRISP ? false : true);
 this.jPanelInput.SetInput(input);
 this.panelPartitionInfo.setPartitionInfo(new PartitionInfo(input.PcPe(this.DataFile.VariableData(this.DataIndex))));
 this.input_range = input.GetRange();
 this.setTitle(this.GetVariableName(this.FISIndex));
 ((HistogramInputPanel)this.jGraphHistogram).SetInput(input);
 this.AdjustXRange();
 }
}
//------------------------------------------------------------------------------
private void InputChanged( int input_number )
{
try
 {
 // si Index correspond a une entree et que l'entree modifiee correspond a Index
 if( (this.FISIndex < this.sif.NbInputs()) && (input_number == this.FISIndex) )
   this.InitObjectsWithInput();
 }
catch( Throwable t ) { MessageFIS.Error(null, t); }
}
//------------------------------------------------------------------------------
protected void OutputChanged( int output_number )
{
try
 {
 if( (this.FISIndex >= this.sif.NbInputs()) && ((output_number+this.sif.NbInputs()) == this.FISIndex) )
   {
   this.InitObjectsWithInput();
   this.validate();
   }
 }
catch( Throwable t ) { MessageFIS.Error(null, t); }
}
//------------------------------------------------------------------------------
protected JInput GetInput( int index ) throws Throwable
{
if( this.sif == null )
 return null;
if( index < this.sif.NbInputs() )     // index correspond a une entree
 return this.sif.GetInput(index);
else                                  // index correspond a une sortie
 return this.sif.GetOutput(index-this.sif.NbInputs());
}
//------------------------------------------------------------------------------
private String GetVariableName( int index ) throws Throwable
{
if( this.sif != null )
 {
 if( index < this.sif.NbInputs() )     // index correspond a une entree
   return this.sif.GetInput(index).GetName();
 else                                  // index correspond a une sortie
   return this.sif.GetOutput(index-this.sif.NbInputs()).GetName();
 }
else {
 if( ((JDataFile)this.DataFile).getVariableName() == null )
     return LocaleFIS.GetString("Variable") + " " + Integer.toString(index+1);
 else
     return ((JDataFile)this.DataFile).getVariableName()[index];
 }
}
//------------------------------------------------------------------------------
protected void ApplyNumberOfClasses()
{
super.ApplyNumberOfClasses();
this.AdjustXRange();
}
}
