//***********************************************************************
//
//                              DataXYFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.*;

import javax.swing.*;
import java.util.*;
import java.text.*; // for decimal format
import java.awt.event.*;
import javax.swing.border.*;
import javax.swing.plaf.basic.*;
import java.awt.geom.*;
import java.awt.font.*;
import java.awt.print.*;
import ptolemy.plot.*;
//needed classes for exporting graphics
import org.freehep.graphics2d.*;
import org.freehep.graphicsio.*;
import org.freehep.graphicsio.exportchooser.*;
import org.freehep.graphicsio.ps.*;
import org.freehep.graphicsio.png.*;
import org.freehep.graphicsio.jpg.*;
import org.freehep.graphicsio.pdf.*;
import org.freehep.util.export.*;

import util.regression.*;

//------------------------------------------------------------------------------
public class DataXYFrame extends JFISChildFrame
{
	private JFIS sif;
	protected JFile DataFile;
	protected int XIndex;
	protected int YIndex;
	private int XIndexFIS;
	private int YIndexFIS;
	protected double [] XData;
	protected double [] YData;
	protected double [] KMinData; //implicative data kernel
	protected double [] KMaxData;
	protected double [] SMinData; //implicative data support
	protected double [] SMaxData;
	protected XYPanel jPanelXY;
	private JFileListener JFileListener;
	private FISListener FISListener;
	private JMenu jMenuFile = new JMenu();
	private JMenuItem jMenuPrint = new JMenuItem();
	private JPanel jPanelXYFrame = new JPanel(new GridBagLayout());
	private JMenuItem jMenuExport = new JMenuItem();
	protected JMenu jMenuDisplay = new JMenu();
	private JCheckBoxMenuItem jMenuDisplayRegression = new JCheckBoxMenuItem();
	private JCheckBoxMenuItem jMenuDisplayYX = new JCheckBoxMenuItem();
	private JMenuItem jMenuDisplayResetZoom = new JMenuItem();
	//------------------------------------------------------------------------------
	public DataXYFrame( JFISFrame parent, int x_fis, int y_fis, JFIS sif, int x_index, int y_index, JFile data_file )
	{
		super(parent);
		try { this.Init(parent, x_fis, y_fis, sif, x_index, y_index, data_file); }
		catch(Throwable t) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	protected DataXYFrame( JFISFrame parent ) { super(parent); }
	//------------------------------------------------------------------------------
	protected void Init( JFISFrame parent, int x_fis, int y_fis, JFIS sif, int x_index, int y_index, JFile data_file ) throws Throwable
	{
		this.sif = sif;
		this.XIndexFIS = x_fis;
		this.YIndexFIS = y_fis;
		this.DataFile = data_file;
		this.XIndex = x_index;
		this.YIndex = y_index;
		this.LoadData();
		jbInit();
	}
	//------------------------------------------------------------------------------
	protected XYPanel CreateXYPanel( JInput x_input, double [] x_data, JInput y_input, double [] y_data ) throws Throwable
	{
		return new XYPanel(x_input, x_data, y_input, y_data)
		{
			public String getXLabel() { return super.getXLabel() == null ? ((JDataFile)DataXYFrame.this.DataFile).getVariableName() == null ? LocaleFIS.GetString("Variable") + " " + (DataXYFrame.this.XIndex + 1) : ((JDataFile)DataXYFrame.this.DataFile).getVariableName()[DataXYFrame.this.XIndex] : super.getXLabel(); }
			public String getYLabel() { return super.getYLabel() == null ? ((JDataFile)DataXYFrame.this.DataFile).getVariableName() == null ? LocaleFIS.GetString("Variable") + " " + (DataXYFrame.this.YIndex + 1) : ((JDataFile)DataXYFrame.this.DataFile).getVariableName()[DataXYFrame.this.YIndex] : super.getYLabel(); }
			public void AddData(int dataset, double [] x_data,  double [] y_data) throws Throwable 
			{
				super.AddData(dataset, x_data, y_data);
			}
		};
	}
	//------------------------------------------------------------------------------
	private void jbInit() throws Throwable
	{
		JMenuBar jmb = new JMenuBar();
		jmb.add(this.jMenuFile);
		this.jMenuFile.add(this.jMenuPrint);
		this.jMenuFile.add(this.jMenuExport);
		jmb.add(this.jMenuDisplay);
		this.jMenuDisplay.add(this.jMenuDisplayRegression);
		this.jMenuDisplay.add(this.jMenuDisplayYX);
		this.jMenuDisplay.add(this.jMenuDisplayResetZoom);
		this.setJMenuBar(jmb);

		this.getContentPane().add(this.jPanelXYFrame);
		this.jPanelXY = this.CreateXYPanel(null, this.XData, null, this.YData);

		this.jPanelXYFrame.add(jPanelXY, new GridBagConstraints(2, 0, 1, 1, 1.0, 1.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(10, 0, 0, 10), 0, 0));

		// evenements
		this.JFileListener = new JFileListener()
		{
			public void Closed() { DataXYFrame.this.dispose(); }
			public void ReLoaded() { DataXYFrame.this.DataReLoaded(); }
		};
		this.DataFile.AddJFileListener(this.JFileListener);
		if( this.sif != null )
		{
			this.FISListener = new FISListener()
			{
				public void FISClosed() { DataXYFrame.this.dispose(); }
				public void InputReplaced( int input_number )       { DataXYFrame.this.InputChanged( input_number); }
				public void OutputReplaced( int output_number )     { DataXYFrame.this.OutputChanged( output_number); }
				public void InputNameChanged( int input_number )    { DataXYFrame.this.InputChanged( input_number); }
				public void OutputNameChanged( int output_number )  { DataXYFrame.this.OutputChanged( output_number); }
				public void InputRangeChanged( int input_number )   { DataXYFrame.this.InputChanged( input_number); }
				public void OutputRangeChanged( int output_number ) { DataXYFrame.this.OutputChanged( output_number); }
				public void MFRemovedInInput( int input_number )    { DataXYFrame.this.InputChanged( input_number); }
				public void MFRemovedInOutput( int output_number )  { DataXYFrame.this.OutputChanged( output_number); }
				public void MFAddedInInput( int input_number )      { DataXYFrame.this.InputChanged( input_number); }
				public void MFAddedInOutput( int output_number )    { DataXYFrame.this.OutputChanged( output_number); }
				public void MFReplacedInInput( int input_number )   { DataXYFrame.this.InputChanged( input_number); }
				public void MFReplacedInOutput( int output_number ) { DataXYFrame.this.OutputChanged( output_number); }
				public void InputActiveChanged( int input_number ) {}
				public void OutputActiveChanged( int output_number ) {}
				public void InputRemoved( int input_number ) {}
				public void OutputRemoved( int output_number ) {}
				public void InputAdded( int input_number ) {}
				public void OutputAdded( int output_number ) {}
				public void OutputDefuzChanged( int output_number ) {}
				public void OutputImpliChanged( int output_number ) {}
				public void OutputDisjChanged( int output_number ) {}
				public void OutputClassifChanged( int output_number ) {}
				public void OutputDefaultChanged( int output_number ) {}
				public void OutputAlarmThresChanged( int output_number ) {}
				public void RulesModified() {}
				public void ConjunctionChanged() {}
			};
			this.sif.AddFISListener(this.FISListener);
		}
		this.jMenuDisplayRegression.setSelected(this.jPanelXY.ShowRegression());
		this.jMenuDisplayRegression.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { DataXYFrame.this.jPanelXY.ShowRegression(DataXYFrame.this.jMenuDisplayRegression.isSelected()); } });
		this.jMenuDisplayYX.setSelected(this.jPanelXY.ShowYX());
		this.jMenuDisplayYX.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { DataXYFrame.this.jPanelXY.ShowYX(DataXYFrame.this.jMenuDisplayYX.isSelected()); } });
		this.jMenuDisplayResetZoom.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { /*DataXYFrame.this.jPanelXY.resetAxes();*/ try { DataXYFrame.this.jPanelXY.setXRange(); DataXYFrame.this.jPanelXY.setYRange(); DataXYFrame.this.jPanelXY.repaint(); } catch( Throwable t ) { MessageFIS.Error(null, t); } } });
		jMenuPrint.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuPrint_actionPerformed(e); } });

		this.InitObjectsWithInput();

		// adding listener on export menu
		// action performed builds a new export dialog
		// wanted export file types are explicitely added
		// if not, dialog crashes

		jMenuExport.addActionListener( new ActionListener()
		{
			public void actionPerformed( ActionEvent e )
			{
				ExportDialog export = new ExportDialog();
				export.showExportDialog( jPanelXY, "Export view as ...", jPanelXY, "export" );
			}
		});

		// affichage de la fenetre
		this.Translate();
		this.pack();
		this.setLocation(this.ChildPosition(this.getSize()));
		this.setSize(450,450);
		this.setVisible(true);
	}
	//------------------------------------------------------------------------------
	public void dispose()
	{
		super.dispose();
		this.DataFile.RemoveJFileListener(this.JFileListener);
		if( this.sif != null )
			this.sif.RemoveFISListener(this.FISListener);
	}
	//------------------------------------------------------------------------------
    protected void LoadData()
    {
	try
	    {
		this.XData = this.DataFile.VariableData(this.XIndex);
		this.YData = this.DataFile.VariableData(this.YIndex);
		if ((this.DataFile instanceof PerformanceFile) && ((PerformanceFile)this.DataFile).IsPerfFileImpli())
		    this.LoadIntervalImpli();
	    }
	catch( Throwable t ) { MessageFIS.Error(null, t); }	
    }
    //------------------------------------------------------------------------------
	protected void LoadIntervalImpli() 
	{
		try
		{	
			this.KMinData = this.DataFile.VariableData(((PerformanceFile)this.DataFile).IndexMinKernel());
			this.KMaxData = this.DataFile.VariableData(((PerformanceFile)this.DataFile).IndexMaxKernel());
			this.SMinData = this.DataFile.VariableData(((PerformanceFile)this.DataFile).IndexMinSupport());
			this.SMaxData = this.DataFile.VariableData(((PerformanceFile)this.DataFile).IndexMaxSupport());	
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	protected void DataReLoaded()
	{
		try
		{
			this.LoadData();
			this.jPanelXY.Reload(this.XData, this.YData);
			this.repaint();
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	private void InitObjectsWithInput() throws Throwable
	{
		if( this.sif != null )
		{
			this.jPanelXY.SetXInput(this.GetInput(this.XIndexFIS));
			this.jPanelXY.SetYInput(this.GetInput(this.YIndexFIS));
		}
		this.jPanelXY.setXLabel(this.GetXLabel());
		this.jPanelXY.setYLabel(this.GetYLabel());
		this.setTitle(this.GetYLabel() + " / " + this.GetXLabel());
	}
	//------------------------------------------------------------------------------
	private void InputChanged( int input_number )
	{
		try
		{
			if( ((this.XIndexFIS < this.sif.NbInputs()) && (input_number == this.XIndexFIS)) ||
					((this.YIndexFIS < this.sif.NbInputs()) && (input_number == this.YIndexFIS)) )
				this.InitObjectsWithInput();
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	private void OutputChanged( int output_number )
	{
		try
		{
			if( ((this.XIndexFIS >= this.sif.NbInputs()) && ((output_number+this.sif.NbInputs()) == this.XIndexFIS)) ||
					((this.YIndexFIS >= this.sif.NbInputs()) && ((output_number+this.sif.NbInputs()) == this.YIndexFIS)) )
				this.InitObjectsWithInput();
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	public void Translate() throws Throwable
	{
		this.jMenuFile.setText(LocaleFIS.GetString("File"));
		this.jMenuDisplay.setText(LocaleFIS.GetString("Display"));
		this.jMenuDisplayRegression.setText(LocaleFIS.GetString("Regression"));
		this.jMenuDisplayYX.setText(LocaleFIS.GetString("YX"));
		this.jMenuDisplayResetZoom.setText(LocaleFIS.GetString("ResetZoom"));
		this.jMenuPrint.setText(LocaleFIS.GetString("Print"));
		this.jMenuExport.setText(LocaleFIS.GetString("Export"));
		this.repaint();
	}
	//------------------------------------------------------------------------------
	private JInput GetInput( int index ) throws Throwable
	{
		if( (this.sif == null) || (index == -1) )
			return null;
		if( index < this.sif.NbInputs() )     // index correspond a une entree
			return this.sif.GetInput(index);
		else                                  // index correspond a une sortie
			return this.sif.GetOutput(index-this.sif.NbInputs());
	}
	//------------------------------------------------------------------------------
	protected String GetXLabel() throws Throwable { return this.jPanelXY.getXLabel(); }
	//------------------------------------------------------------------------------
	protected String GetYLabel() throws Throwable { return this.jPanelXY.getYLabel(); }
	//------------------------------------------------------------------------------
	public XYPanel XYPanel() { return this.jPanelXY; }
	//------------------------------------------------------------------------------
	void jMenuPrint_actionPerformed(ActionEvent e)
	{
		PrinterJob job = PrinterJob.getPrinterJob();
		if( job.printDialog() == true )
		{
			job.setPrintable(new Printable()
			{
				public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) throws PrinterException
				{
					if (pageIndex >= 1) { return Printable.NO_SUCH_PAGE; }
					else return DataXYFrame.this.print(graphics,pageFormat,pageIndex);
				}
			});
			try { job.print(); }
			catch (PrinterException ex) { ex.printStackTrace(); }
		}
	}
	//------------------------------------------------------------------------------
	public int print( java.awt.Graphics g, PageFormat pageFormat, int pageIndex ) throws PrinterException
	{
		java.awt.Graphics2D  g2 = ( java.awt.Graphics2D )g;
		double scalew=1;
		double scaleh=1;
		double pageHeight = pageFormat.getImageableHeight();
		// See how much we need to scale the image to fit in the page's width
		double pageWidth = pageFormat.getImageableWidth();
		if(  getWidth() >= pageWidth ) { scalew =  pageWidth / getWidth(); }
		if(  getHeight() >= pageHeight) { scaleh =  pageWidth / getHeight(); }
		g2.translate( pageFormat.getImageableX(), pageFormat.getImageableY() );
		g2.scale( scalew, scaleh );
		this.jPanelXYFrame.print( g2 );
		return Printable.PAGE_EXISTS;
	}
}


//------------------------------------------------------------------------------
class XYPanel extends FISPlot
{
	private JInput XInput;    // entree associe au x
	private JInput YInput;    // entree associe au y
	private double [] x_data_range = new double[2];
	private double [] y_data_range = new double[2];
	private boolean ShowRegression = false;
	private boolean ShowYX = false;
	private RegressionCalculator regression;
	static final int DefaultDataSet = 1;

	//------------------------------------------------------------------------------
	XYPanel( JInput x_input, double [] x_data, JInput y_input, double [] y_data ) throws Throwable
	{
		this.x_data_range[0] = Double.MAX_VALUE;
		this.y_data_range[0] = Double.MAX_VALUE;
		this.x_data_range[1] = Double.MIN_VALUE;
		this.y_data_range[1] = Double.MIN_VALUE;
		this.setMarksStyle("various");
		this.AddData(XYPanel.DefaultDataSet, x_data, y_data);
		this.SetXInput(x_input);
		this.SetYInput(y_input);
	}
	//------------------------------------------------------------------------------
	public void SetXInput( JInput x_input ) throws Throwable
	{
		this.XInput = x_input;
		this.setXRange();
		this.repaint();
	}
	//------------------------------------------------------------------------------
	public void SetYInput( JInput y_input ) throws Throwable
	{
		this.YInput = y_input;
		this.setYRange();
		this.repaint();
	}
	//------------------------------------------------------------------------------
	public void setXRange() throws Throwable
	{
		if( this.XInput != null )
			super.setXRange(Math.min(this.x_data_range[0], this.XInput.GetRange()[0]), Math.max(this.x_data_range[1], this.XInput.GetRange()[1]));
		else
			super.setXRange(this.x_data_range[0], this.x_data_range[1]);
	}
	//------------------------------------------------------------------------------
	public void setYRange() throws Throwable
	{
		if( this.YInput != null )
			super.setYRange(Math.min(this.y_data_range[0], this.YInput.GetRange()[0]), Math.max(this.y_data_range[1], this.YInput.GetRange()[1]));
		else
			super.setYRange(this.y_data_range[0], this.y_data_range[1]);
	}
	//------------------------------------------------------------------------------
	public String getXLabel() { try { return this.XInput.GetName(); } catch(Throwable t ) { return null; } }
	//------------------------------------------------------------------------------
	public String getYLabel() { try { return this.YInput.GetName(); } catch(Throwable t ) { return null; } }
	//------------------------------------------------------------------------------
	public void AddData( int dataset, double [] x_data,  double [] y_data ) throws Throwable
	{
		if( (x_data == null) || (y_data == null) ) return;
		if( x_data.length != y_data.length ) throw new Exception("XYPanel.AddData(): x_data.length != y_data.length");
		// recherche du min et max des datas
		for( int i=0 ; i<x_data.length ; i++ )
		{
			this.x_data_range[0] = Math.min(this.x_data_range[0], x_data[i]);
			this.x_data_range[1] = Math.max(this.x_data_range[1], x_data[i]);
		}
		for( int i=0 ; i<y_data.length ; i++ )
		{
			this.y_data_range[0] = Math.min(this.y_data_range[0], y_data[i]);
			this.y_data_range[1] = Math.max(this.y_data_range[1], y_data[i]);
		}
		// affiche les data dans le Plot
		this.clear(dataset);
		for( int i=0 ; i<x_data.length ; i++ )
		{
			this.addPoint(dataset, x_data[i], y_data[i], false);
		}
		// calcul de la regression
		if( dataset == XYPanel.DefaultDataSet )
			this.regression = new RegressionCalculator(x_data, y_data);
		// affichage
		this.repaint();
	}
	//------------------------------------------------------------------------------
	public void AddData( int implidataset, double [] x_data,  double [] y_data, double [] i_minData, double [] i_maxData  ) throws Throwable
	{
		if( (x_data == null) || (y_data == null) ) return;
		if( x_data.length != y_data.length ) throw new Exception("XYPanel.AddData(): x_data.length != y_data.length");
	
		// affiche les intervalles dans le Plot
		this.clear(implidataset);
		
		//interval
		if((i_minData == null) || (i_maxData == null)) return;
		Color[] tabc = new Color [1];
		//tabc[0]=Color.RED;
		//this.setColors(tabc);
		// affichage des segments
		for( int i=0 ; i<x_data.length ; i++ )
		{
			this.addPoint(implidataset, x_data[i], i_minData[i], false);
			this.addPoint(implidataset, x_data[i], i_maxData[i], true);
		}				
		// affichage
		this.repaint();
	}
	
	//------------------------------------------------------------------------------
	public void Reload( double []x_data, double []y_data) throws Throwable
	{
		this.AddData(XYPanel.DefaultDataSet, x_data, y_data);
	}
	//------------------------------------------------------------------------------
	public void paintComponent(Graphics g)
	{
		try
		{
			super.paintComponent(g);

			// ajustement de la taille du rectangle plot au panel
			FISPlot.AdjustPlotRectangleSize(g, this);
			super.paintComponent(g);
			this.setPlotRectangle(null);  // supprime la memorisation du setPlotRectangle

			// affichage des bornes du domaine des entrees
			if( this.XInput != null ) FISPlot.DrawXInputRange(g, this, this.XInput.GetRange(), this._xMin, this._xMax);
			if( this.YInput != null ) this.DrawYInputRange(g, this.YInput.GetRange());

			// affichage de la droite de regression
			if( (this.ShowRegression == true) && (this.regression != null) )
			{
				Graphics2D g2d = (Graphics2D)g;
				g2d.setColor(Color.gray);
				BasicStroke bs = new BasicStroke((float)1);
				g2d.setStroke(bs);
				// calcul du min et max et affichage
				double a = this.regression.getSlope();
				double b = this.regression.getIntercept();
				double y_min = a * this._xMin + b;
				double y_max = a * this._xMax + b;
				this._drawLine(g, 0, this.XtoPixel(this._xMin), this.YtoPixel(y_min), this.XtoPixel(this._xMax), this.YtoPixel(y_max), true, (float)1);
				// affichage du r2
				g2d.setColor(Color.black);
				g2d.drawString("r2=" + DecimalFormat.getInstance().format(Math.pow(this.regression.getPearsonR(), 2)), this.XtoPixel(this._xMin + (this._xMax-this._xMin)*1/8), this.YtoPixel(this._yMin + (this._yMax-this._yMin)*7/8));
			}
			// affichage de la droite y=x
			if( this.ShowYX == true )
			{
				Graphics2D g2d = (Graphics2D)g;
				g2d.setColor(Color.lightGray);
				BasicStroke bs = new BasicStroke((float)1);
				g2d.setStroke(bs);
				// calcul du min et max et affichage
				double min = Math.min(this._xMin, this._yMin);
				double max = Math.max(this._xMax, this._yMax);
				this._drawLine(g, 0, this.XtoPixel(min), this.YtoPixel(min), this.XtoPixel(max), this.YtoPixel(max), true, (float)1);
			}
		}
		catch( Throwable t ) { MessageFIS.Error( null, t ); }
	}
	//------------------------------------------------------------------------------
	public void ShowRegression( boolean show ) { this.ShowRegression = show; this.repaint(); }
	//------------------------------------------------------------------------------
	public void ShowYX( boolean show ) { this.ShowYX = show; this.repaint(); }
	//------------------------------------------------------------------------------
	public boolean ShowRegression() { return this.ShowRegression; }
	//------------------------------------------------------------------------------
	public boolean ShowYX() { return this.ShowYX; }
}


