//***********************************************************************
//
//                              ExtendedDataXYFrame.java
//
// Author(s) : Jean-luc LABLEE, Mathieu GRELIER
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;



public class ExtendedDataXYFrame extends DataXYFrame {
  protected JExtendedDataFile extend_data_file;
  private JCheckBoxMenuItem jMenuDisplayInactives = new JCheckBoxMenuItem();
//------------------------------------------------------------------------------
  public ExtendedDataXYFrame( JFISFrame parent, int x_fis, int y_fis, JFIS sif, int x_index, int y_index, JFile active_data_file, JExtendedDataFile extend_data_file )
    {
    super(parent);
    try
      {
      this.extend_data_file = extend_data_file;
      this.Init(parent, x_fis, y_fis, sif, x_index, y_index, active_data_file);
      }
    catch(Throwable t) { MessageFIS.Error(null, t); }
    }
//------------------------------------------------------------------------------
  public ExtendedDataXYFrame( JFISFrame parent ) { super(parent); }
//------------------------------------------------------------------------------
  protected void Init( JFISFrame parent, int x_fis, int y_fis, JFIS sif, int x_index, int y_index, JFile data_file ) throws Throwable
  {
  this.addWindowListener(new WindowAdapter() {

	@Override
	public void windowOpened(WindowEvent e) {
		super.windowOpened(e);
		toFront();
	}});
  super.Init(parent, x_fis, y_fis, sif, x_index, y_index, data_file);
  this.jMenuDisplayInactives.setSelected(((ExtendedXYPanel)this.jPanelXY).ShowInactive());
  this.jMenuDisplay.add(this.jMenuDisplayInactives);
  this.jMenuDisplayInactives.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { try { ((ExtendedXYPanel)ExtendedDataXYFrame.this.jPanelXY).ShowInactive(ExtendedDataXYFrame.this.jMenuDisplayInactives.isSelected()); } catch(Throwable t) { MessageFIS.Error(null, t); } } });
  }
  //------------------------------------------------------------------------------
  protected XYPanel CreateXYPanel( JInput x_input, double [] x_data, JInput y_input, double [] y_data ) throws Throwable
  {
  return new ExtendedXYPanel(x_input, x_data, y_input, y_data, this.extend_data_file, this.XIndex, this.YIndex)
    {
      public String getXLabel() { return super.getXLabel() == null ? ((JDataFile)ExtendedDataXYFrame.this.DataFile).getVariableName() == null ? LocaleFIS.GetString("Variable") + " " + (ExtendedDataXYFrame.this.XIndex + 1) : ((JDataFile)ExtendedDataXYFrame.this.DataFile).getVariableName()[ExtendedDataXYFrame.this.XIndex] : super.getXLabel(); }
      public String getYLabel() { return super.getYLabel() == null ? ((JDataFile)ExtendedDataXYFrame.this.DataFile).getVariableName() == null ? LocaleFIS.GetString("Variable") + " " + (ExtendedDataXYFrame.this.YIndex + 1) : ((JDataFile)ExtendedDataXYFrame.this.DataFile).getVariableName()[ExtendedDataXYFrame.this.YIndex] : super.getYLabel(); }
    };
  }
  //------------------------------------------------------------------------------
  public void Translate() throws Throwable
  {
  super.Translate();
  this.jMenuDisplayInactives.setText(LocaleFIS.GetString("InactiveItems"));
  }
}
//------------------------------------------------------------------------------

class InferedDataXYFrame extends ExtendedDataXYFrame
{
  private String inact_perf_result;
  private LinksItemsRules items_rules;
  private JCheckBoxMenuItem jMenuDisplayBlank = new JCheckBoxMenuItem();
//------------------------------------------------------------------------------
  public InferedDataXYFrame( JFISFrame parent, int x_fis, int y_fis, JFIS sif, int x_index, int y_index, JFile data_file, JExtendedDataFile extend_data_file, String inact_perf_result, LinksItemsRules items_rules )
    {
    super(parent);
    try
      {
      this.extend_data_file = extend_data_file;
      this.inact_perf_result = inact_perf_result;
      this.items_rules = items_rules;
      this.Init(parent, x_fis, y_fis, sif, x_index, y_index, data_file);
      }
    catch(Throwable t) { MessageFIS.Error(null, t); }
    }
//------------------------------------------------------------------------------
  protected void Init( JFISFrame parent, int x_fis, int y_fis, JFIS sif, int x_index, int y_index, JFile data_file ) throws Throwable
  {
  super.Init(parent, x_fis, y_fis, sif, x_index, y_index, data_file);
  this.jMenuDisplayBlank.setSelected(((InferedXYPanel)this.jPanelXY).ShowBlank());
  this.jMenuDisplay.add(this.jMenuDisplayBlank);
  this.jMenuDisplayBlank.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { try { ((InferedXYPanel)InferedDataXYFrame.this.jPanelXY).ShowBlank(InferedDataXYFrame.this.jMenuDisplayBlank.isSelected()); } catch(Throwable t) { MessageFIS.Error(null, t); } } });
  }
//------------------------------------------------------------------------------
  public void Translate() throws Throwable
  {
  super.Translate();
  this.jMenuDisplayBlank.setText(LocaleFIS.GetString("BlankItems"));
  }
  //------------------------------------------------------------------------------
  protected XYPanel CreateXYPanel( JInput x_input, double [] x_data, JInput y_input, double [] y_data ) throws Throwable
  {
      // implicative case - check if there is a mink field in the result file
      boolean impli;
      impli=((PerformanceFile)this.DataFile).IsPerfFileImpli();
     if (!impli)
	return new InferedXYPanel(x_input, null, y_input, null, this.extend_data_file, this.DataFile, this.inact_perf_result, this.items_rules)
    {
		public void AddData(int dataset, double [] x_data,  double [] y_data) throws Throwable 
		{
		    
		    //super.AddData(dataset, ExtendedXYPanel.this.ImpliDataSet, x_data, y_data, InferedDataXYFrame.this.KMinData, InferedDataXYFrame.this.KMaxData);
		    super.AddData(dataset, x_data, y_data);
			
		}
    };
    else
	return new ImpliInferedXYPanel(InferedDataXYFrame.this.KMinData, InferedDataXYFrame.this.KMaxData,x_input, null, y_input, null, this.extend_data_file, this.DataFile, this.inact_perf_result, this.items_rules)
    {
	/*public void AddData(int dataset, double [] x_data,  double [] y_data) throws Throwable 
		{
		    super.AddData(dataset, x_data, y_data);
		    }*/
    };
  }
}

