//***********************************************************************
//
//                              ExtendedXYPanel.java
//
// Author(s) : Jean-luc LABLEE, Mathieu GRELIER
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import javax.swing.*;
import java.text.*;
import java.awt.*;
import java.util.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.io.*;
import ptolemy.plot.*;

//------------------------------------------------------------------------------
public class ExtendedXYPanel extends XYPanel {

	protected JExtendedDataFile extend_data_file;
	protected int x_index;
	protected int y_index;
	protected int which_ex ;
	protected double[][] PointsTable;
	protected boolean point_located = false ;
	static final int InactiveDataSet = 9;
	static final int ActiveBlankDataSet = 8;
        static final int ImpliDataSet=7;
	static final int ActiveDataSet = XYPanel.DefaultDataSet;
	private boolean show_inactive = true;
	//------------------------------------------------------------------------------
	public ExtendedXYPanel(JInput x_input, double[] x_data, JInput y_input,
			double[] y_data, JExtendedDataFile extend_data_file,
			int x_index, int y_index) throws Throwable {
		super(x_input, x_data, y_input, y_data);

		this.extend_data_file = extend_data_file;
		this.x_index = x_index;
		this.y_index = y_index;
		this.ShowInactiveItems();
		this.setXRange();
		this.setYRange();

		this.addMouseMotionListener(new MouseMotionAdapter() {
			public void mouseMoved(MouseEvent e) {
				ExtendedXYPanel_mouseMoved(e);
			}
		});

		//to change the activation status of the examples by clicking on them
		this.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				ExtendedXYPanel_mousePressed(e);
			}
		});
	}


	//------------------------------------------------------------------------------
	/* force les points a s'afficher en croix quel que soit le dataset d'origine
	protected void _drawPoint(Graphics graphics, int dataset, long xpos, long ypos, boolean clip) {
		super._drawPoint(graphics, 1, xpos, ypos, clip);
	}*/

	//------------------------------------------------------------------------------
	public void Reload(double[] x_data, double[] y_data) throws Throwable {
		super.Reload(x_data, y_data);
		this.ShowInactiveItems();
	}

	//------------------------------------------------------------------------------
	protected void ShowInactiveItems() throws Throwable {
		if( this.show_inactive == false ) return;
		File f = new File(this.extend_data_file.InactiveFileName());
		if (f.exists() == false) {
			return;
		}
		JDataFile jdf = new JDataFile(this.extend_data_file.InactiveFileName());
		double[] inact_x = jdf.VariableData(this.x_index);
		double[] inact_y = jdf.VariableData(this.y_index);
		this.AddData(ExtendedXYPanel.InactiveDataSet, inact_x, inact_y);
	}

	//------------------------------------------------------------------------------
	public void ShowInactive( boolean show_inactive ) throws Throwable
	{
		this.show_inactive = show_inactive;
		if( this.show_inactive == true )
			this.ShowInactiveItems();
		else
			this.clear(ExtendedXYPanel.InactiveDataSet);
	}

	//------------------------------------------------------------------------------
	public boolean ShowInactive() { return this.show_inactive; }

	//------------------------------------------------------------------------------
	protected void ExtendedXYPanel_mousePressed(MouseEvent e)
	{
		if (SwingUtilities.isLeftMouseButton(e))
		{
			if (ExtendedXYPanel.this.point_located == true)
			{
				boolean bvalue = this.extend_data_file.GetActive(this.which_ex-1) ;
				this.extend_data_file.SetActive(this.which_ex, !bvalue);
			}
		}

	}
	//-----------------------------------------------------------------------------
	protected void ExtendedXYPanel_mouseMoved(MouseEvent e)  {
		this.point_located = false ;
		double distance;
		int mouse_X = e.getX();
		int mouse_Y = e.getY();

		Vector va = (Vector)this._points.get(this.ActiveDataSet);
		Vector vi = null;
		try{
			vi = (Vector)this._points.get(this.InactiveDataSet);
		} catch (Throwable t) {vi = null ; }
		PlotPoint p = (PlotPoint) va.get(0);

		double min_dist = CalculateDistance(p.x, p.y, mouse_X, mouse_Y);

		which_ex = 0; //the example we have to find
		int tmp_min = 1; //temporary nearest example : its number in its dataset ; initialized with the first active example
		int whichvector = this.ActiveDataSet ; //to mark in which dataset the point is ; the first point for initialisation is from the active data set
		for (int i = 1; i < va.size(); i++) {   //we look for a nearer example in the active dataset
			p = (PlotPoint) va.get(i);
			distance = CalculateDistance(p.x, p.y, mouse_X, mouse_Y);
			if (distance < min_dist) {
				min_dist = distance;
				tmp_min = i + 1;
				whichvector = this.ActiveDataSet;
			}
		}
		if (vi != null)
		{
			for (int i = 1; i < vi.size(); i++) {   //now we look in the inactive dataset
				p = (PlotPoint) vi.get(i);
				distance = CalculateDistance(p.x, p.y, mouse_X, mouse_Y);
				if (distance < min_dist) {
					min_dist = distance;
					tmp_min = i + 1;
					whichvector = this.InactiveDataSet;
				}
			}
		}
		if (min_dist < 0.02) { //retrieving of the example number, knowing that it's the nth active or inactive example
			boolean b = false;
			if (whichvector == this.ActiveDataSet) {
				b = true;
			}
			int compt = 0;
			int i;
			for (i = 0; i < this.extend_data_file.ItemCount() ; i++) {
				if (this.extend_data_file.GetActive(i) == b) {
					compt++;
				}
				if (compt == tmp_min) {
					break;
				}
			}
			which_ex = i + 1;
			this.point_located = true ; //left clicks for activate/inactivate can be done
			ToolTipManager.sharedInstance().setEnabled(true);   //tooltip display
			this.setToolTipText("No " + which_ex);
			ToolTipManager.sharedInstance().setInitialDelay(1000);
			ToolTipManager.sharedInstance().setReshowDelay(10);
		}
		else {
			ToolTipManager.sharedInstance().setEnabled(false);
		}

	}
	//------------------------------------------------------------------------------
	protected double CalculateDistance(double x, double y, int mouse_X, int mouse_Y) {
		double distance;
		//with standardization
		distance = Math.sqrt(Math.pow( (double) ((x - this.PixelToX(mouse_X))/(this._xMax - this._xMin )), 2.0) +
				Math.pow( (double) ((y - this.PixelToY(mouse_Y))/(this._yMax - this._yMin )), 2.0));
		return distance;
	}

} //end of ExtendedXYPanel class


class ImpliInferedXYPanel extends InferedXYPanel{
    double [] KMinData, KMaxData;
    public ImpliInferedXYPanel(double [] KMinData, double [] KMaxData, JInput x_input, double[] x_data, JInput y_input, double[] y_data, JExtendedDataFile extend_data_file,
			       JFile act_perf_file, String inact_perf_result, LinksItemsRules items_rules) throws
				   Throwable {
	super( x_input, x_data,  y_input,  y_data,  extend_data_file,
	       act_perf_file,  inact_perf_result,  items_rules);
	this.KMinData = KMinData;
	this.KMaxData = KMaxData;
	
	this.ShowImpliIntervals();
    }
	
    protected void ShowImpliIntervals() throws Throwable {
	double[] act_x = this.act_perf_file.VariableData( ( (PerformanceFile)this.
				act_perf_file).IndexObserved());
	double[] act_y = this.act_perf_file.VariableData( ( (PerformanceFile)this.
				act_perf_file).IndexInferred());
	
	this.AddData(ExtendedXYPanel.ImpliDataSet,act_x, act_y,KMinData,KMaxData);
    }
    
    public void Reload(double[] x_data, double[] y_data) throws Throwable {
		super.Reload(null, null);
		this.ShowImpliIntervals();

    }

    protected void _drawPoint(Graphics graphics, int dataset, long xpos, long ypos, boolean clip) {
	// comment out to display segments only
	super._drawPoint(graphics, 1, xpos, ypos, clip);
    }
	
}
// end of ImpliInferedXYPanel class
//------------------------------------------------------------------------------

class InferedXYPanel extends ExtendedXYPanel {

	private String inact_perf_result;
	protected JFile act_perf_file;
	private LinksItemsRules items_rules;
	protected double[] act_noblank_x, act_noblank_y, blank_x, blank_y,  blancs;
	private boolean show_blank = true;
	//------------------------------------------------------------------------------
	public InferedXYPanel(JInput x_input, double[] x_data, JInput y_input, double[] y_data, JExtendedDataFile extend_data_file,
			JFile act_perf_file, String inact_perf_result, LinksItemsRules items_rules) throws
			Throwable {
		super(x_input, x_data, y_input, y_data, extend_data_file, 0, 0);
		this.inact_perf_result = inact_perf_result;
		this.act_perf_file = act_perf_file;
		this.ShowActive();
		this.ShowInactiveItems();
		this.setXRange();
		this.setYRange();
		this.items_rules = items_rules;
	}
    
	//------------------------------------------------------------------------------
	protected void _drawPoint(Graphics graphics, int dataset, long xpos, long ypos, boolean clip) {
		super._drawPoint(graphics, 1, xpos, ypos, clip);
	}
	//------------------------------------------------------------------------------
	protected void ShowInactiveItems() throws Throwable {
		if( this.ShowInactive() == false ) return;
		if (this.inact_perf_result == null) {
			return;
		}
		File f = new File(this.inact_perf_result);
		if (f.exists() == false) {
			return;
		}
		PerformanceFile pf = new PerformanceFile(this.inact_perf_result);
		double[] inact_x = pf.VariableData(pf.IndexObserved());
		double[] inact_y = pf.VariableData(pf.IndexInferred());
		this.AddData(ExtendedXYPanel.InactiveDataSet, inact_x, inact_y);
	}

	//------------------------------------------------------------------------------
	private void ShowActive() throws Throwable {
		double[] act_x = this.act_perf_file.VariableData( ( (PerformanceFile)this.
				act_perf_file).IndexObserved());
		double[] act_y = this.act_perf_file.VariableData( ( (PerformanceFile)this.
				act_perf_file).IndexInferred());
		blancs = this.act_perf_file.VariableData( ( (PerformanceFile)this.act_perf_file).IndexBlank());
		int nbrBlancs = BlankCount(blancs);
		CreateActiveAndBlankArrays(nbrBlancs, blancs, act_x, act_y);
		if( this.show_blank == true ) this.AddData(ExtendedXYPanel.ActiveBlankDataSet, blank_x, blank_y);
		this.AddData(ExtendedXYPanel.ActiveDataSet, act_noblank_x, act_noblank_y);
		// implicative data
		
	}
	//----------------------------------------------------------------------------
	private int BlankCount(double[] blancs) {
		int count = 0;
		for (int i = 0; i < blancs.length; i++) {
			if (blancs[i] == 1) {
				count++;
			}
		}
		return count;
	}

	//------------------------------------------------------------------------------
	private void CreateActiveAndBlankArrays(int nbrBlancs, double[] blancs, double[] act_x, double[] act_y ) {
		act_noblank_x = new double[act_x.length - nbrBlancs];
		act_noblank_y = new double[act_x.length - nbrBlancs];
		blank_x = new double[nbrBlancs];
		blank_y = new double[nbrBlancs];
		int j = 0;
		int k = 0;
		for (int i = 0; i < act_x.length; i++) {
			if (blancs[i] == 0) {
				act_noblank_x[j] = act_x[i];
				act_noblank_y[j] = act_y[i];
				j++;
			}
			else {
				blank_x[k] = act_x[i];
				blank_y[k] = act_y[i];
				k++;
			}
		}
	}
	//-----------------------------------------------------------------------------
	protected void ExtendedXYPanel_mouseMoved(MouseEvent e) {

		this.point_located = false ;
		boolean is_active_example = false ;
		double distance;
		int mouse_X = e.getX();
		int mouse_Y = e.getY();
		//use of three datasets with 3 different colors : active ex that are not blank, blank ex and inactive ex
		Vector va = (Vector)this._points.get(this.ActiveDataSet);
		Vector vi = null;
		Vector vb = null;
		// these two can be null
		try{
			vi = (Vector)this._points.get(this.InactiveDataSet);
		} catch (Throwable t) {vi = null ; }
		try{
			vb = (Vector)this._points.get(this.ActiveBlankDataSet);
		} catch (Throwable t) {vb = null ; }

		//initialisation of the distance
		PlotPoint p = (PlotPoint) va.get(0);
		double min_dist = CalculateDistance(p.x, p.y, mouse_X, mouse_Y); // min_dist is the current minimum distance

		which_ex = 0;   // the ex number that we have to retrieve
		int which_ex_in_its_dataset  = 0 ; // its number in its own dataset
		int tmp_min = 0; //temporary value
		int whichvector = this.ActiveDataSet; //to mark in which dataset the point is ; the first point for initialisation is from the active data set
		//we calculate the distances between the mouse pointer and all the points in the three datasets
		for (int i = 1; i < va.size(); i++)
		{
			p = (PlotPoint) va.get(i);
			distance = CalculateDistance(p.x, p.y, mouse_X, mouse_Y);
			if (distance < min_dist) {
				min_dist = distance;
				tmp_min = i ;
				whichvector = this.ActiveDataSet;
			}
		}

		if (vb != null)
		{
			for (int i = 0; i < vb.size(); i++) {
				p = (PlotPoint) vb.get(i);
				distance = CalculateDistance(p.x, p.y, mouse_X, mouse_Y);
				if (distance < min_dist) {
					min_dist = distance;
					tmp_min = i ;
					whichvector = this.ActiveBlankDataSet;
				}
			}
		}


		if (vi != null)
		{
			for (int i = 0; i < vi.size(); i++) {
				p = (PlotPoint) vi.get(i);
				distance = CalculateDistance(p.x, p.y, mouse_X, mouse_Y);
				if (distance < min_dist) {
					min_dist = distance;
					tmp_min = i ;
					whichvector = this.InactiveDataSet;
				}
			}
		}

		which_ex_in_its_dataset = tmp_min + 1 ; // here we know the ex number in its dataset

		if (min_dist < 0.02) {      // if the distance is small enough, the tool tip is displayed
			boolean b = false ; //used for the final retrieving of the example number in the total dataset
			double d ; //used for the retrieving of the active example number if it is an active example
			int compt = 0; // a counter
			int i = 0; // two indexes
			int j = 0 ;
			int which_act_or_inact_ex = 0 ; //the  active or inactive example number. We refer to that because the only thing containing all the examples is the extended data file, with the ActiveValuesArray, and it just has a active/inactive information : there is no blank example info.
			if (whichvector == this.ActiveDataSet || whichvector == this.ActiveBlankDataSet)
			{
				is_active_example = true ;
				b = true ; // it is an active example
				if (whichvector == this.ActiveDataSet) //it's not a blank example, marked by the double 0.0 in the performance file
				{
					d = 0.0;
					for (j = 0; j < this.blancs.length ; j++)
					{
						if (this.blancs[j] == d)
							compt++;        // retrieving of the active example number
						if (compt == which_ex_in_its_dataset)
							break;
					}
				}

				if (whichvector == this.ActiveBlankDataSet)
				{
					d = 1.0; //it's a blank example : marked by 1.0
					for (j = 0; j < this.blancs.length ; j++)
					{
						if (this.blancs[j] == d)
							compt++;            // retrieving of the active example number
						if (compt == which_ex_in_its_dataset)
							break;
					}
				}
				which_act_or_inact_ex = j+1 ;      // the active example number
			}


			if (whichvector == this.InactiveDataSet)
				which_act_or_inact_ex = which_ex_in_its_dataset ; // in this case, we know the inactive example number directly

			compt = 0 ;  // reinitialisation
			//now, retrieving of the example number
			for (i = 0; i < this.extend_data_file.ItemCount(); i++) {
				if (this.extend_data_file.GetActive(i) == b) {
					compt++;
				}
				if (compt == which_act_or_inact_ex) {
					break;
				}
			}

			which_ex = i + 1; // the example number
			this.point_located = true ;
			//display
			ToolTipManager.sharedInstance().setEnabled(true);
			if (is_active_example == true)
			{
				String example = "No " + which_ex ;
				String activated_rules = new String(InferedXYPanel.this.items_rules.GetRules(which_act_or_inact_ex-1) );
				if(activated_rules.length() != 0) {
					String text ;
					if (activated_rules.equals(new String(LocaleFIS.GetString("NoActivatedRules"))))
					{
						text = "<HTML>" + example + "<BR>" + activated_rules + "</HTML>";
					}
					else
					{
						text = "<HTML>" + example + "<BR>" + LocaleFIS.GetString("ActivatedRules") + " :" + activated_rules + "</HTML>"  ;
					}
					this.setToolTipText(text);
				}
				else this.setToolTipText(example);
			}
			else
				this.setToolTipText("No " + which_ex);

			ToolTipManager.sharedInstance().setInitialDelay(1000);
			ToolTipManager.sharedInstance().setReshowDelay(10);
		}
		else {
			ToolTipManager.sharedInstance().setEnabled(false);
		}
	}

	//------------------------------------------------------------------------------
	public void Reload(double[] x_data, double[] y_data) throws Throwable {
		super.Reload(null, null);
		this.ShowActive();
	}
	//------------------------------------------------------------------------------
	public void ShowBlank( boolean show_blank ) throws Throwable
	{
		this.show_blank = show_blank;
		if( this.show_blank == true )
			this.ShowActive();
		else
			this.clear(ExtendedXYPanel.ActiveBlankDataSet);
	}

	//------------------------------------------------------------------------------
	public boolean ShowBlank() { return this.show_blank; }
}

