//***********************************************************************
//
//                              FISPlot.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.Line2D;

import ptolemy.plot.Plot;
import ptolemy.plot.PlotBox;

//------------------------------------------------------------------------------
//extension de la classe Plot

//les fonctions static sont destinees a etre utilisees par les classes Histogram qui ne peuvent pas heriter de FISPlot.

public class FISPlot extends Plot
{
	protected boolean showTicks = true;    

	public double GetXMin() { return this._xMin; }
	public double GetXMax() { return this._xMax; }
	public static void DisableZoom( PlotBox pb ) { pb.removeMouseListener(pb.getMouseListeners()[0]); }

	public FISPlot() {
		super();
	}

	/**
	 * Permet d'afficher ou de cacher les ticks
	 * @param showTicks showTicks a definir.
	 */
	public void setShowTicks(boolean showTicks) {
		this.showTicks = showTicks;
	}

	//------------------------------------------------------------------------------
	public void paintComponent(Graphics g) {
		// ajustement de la taille du rectangle plot au panel
		super.paintComponent(g);
		if( this.showTicks == true )
			FISPlot.AdjustPlotRectangleSize(g, this);
		else         
			FISPlot.fillPlotRectangleSize(g, this);
		super.paintComponent(g);
		this.setPlotRectangle(null);  // supprime la memorisation du setPlotRectangle
	}

	/* (non-Javadoc)
	 * @see javax.swing.JComponent#setPreferredSize(java.awt.Dimension)
	 * surcharge car PlotBox surcharge getPreferredSize() sans surcharger setPreferredSize()
	 */
	/*public void setPreferredSize(Dimension preferredSize) {
    this._preferredWidth = preferredSize.width;
    this._preferredHeight = preferredSize.height;
}

public synchronized Dimension getMinimumSize() {
    return new Dimension(10,10);
}*/

	//ajustement de la taille du rectangle plot au panel(il faut 1 paintComponent avant !!!!)
	public static void AdjustPlotRectangleSize( Graphics g, PlotBox pb )
	{
		Rectangle r1 = pb.getPlotRectangle();
		r1.height = r1.height + r1.y;
		r1.y = 0;
		r1.width = pb.getSize().width - r1.x -10;
		pb.setPlotRectangle(r1);
	}
	//rempli le rectangle plot au panel(il faut 1 paintComponent avant !!!!)
	public static void fillPlotRectangleSize( Graphics g, PlotBox pb )
	{
		Rectangle r = pb.getPlotRectangle();
		r.x = 0;
		r.y = 0;
		r.width = pb.getSize().width -1;
		r.height = pb.getSize().height -1;
		pb.setPlotRectangle(r);
	}
	// dessine le domaine de l'entree sur l'axe X
	public static void DrawXInputRange( Graphics g, PlotBox pb, double [] input_range, double x_min, double x_max )
	{
		Graphics2D g2d = (Graphics2D)g;
		g2d.setColor(Color.red);
		Rectangle rect = pb.getPlotRectangle();
		if( rect.height <= 0 ) return;
		float [] dash = new float[2];
		dash[0] = (float)rect.height/21;
		dash[1] = dash[0];
		BasicStroke bs_dash = new BasicStroke( (float)1, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_ROUND, 0, dash, 0 );
		g2d.setStroke(bs_dash);
		if( (input_range[0] >= x_min) && (input_range[0] <= x_max) )
		{
			int x = FISPlot.XtoPixel(pb, input_range[0], x_min, x_max);
			g2d.draw(new Line2D.Double(x, rect.y, x, rect.y+rect.height) );
		}
		if( (input_range[1] >= x_min) && (input_range[1] <= x_max) )
		{
			int x = FISPlot.XtoPixel(pb, input_range[1], x_min, x_max);
			g2d.draw(new Line2D.Double(x, rect.y, x, rect.y+rect.height) );
		}
	}
	//------------------------------------------------------------------------------
	/**
	 * Dessine les bornes du domaine de l'entree sur l'axe Y\n
	 * dessine 2 traits pointilles rouge sur les bornes
	 * @param g objet Graphics
	 * @param input_range bornes a afficher
	 */
	public void DrawYInputRange( Graphics g, double [] input_range )
	{
		Graphics2D g2d = (Graphics2D)g;
		g2d.setColor(Color.red);
		Rectangle rect = this.getPlotRectangle();
		if( rect.width <= 0 ) return;
		float [] dash = new float[2];
		dash[0] = (float)rect.width/21;
		dash[1] = dash[0];
		BasicStroke bs_dash = new BasicStroke( (float)1, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_ROUND, 0, dash, 0 );
		g2d.setStroke(bs_dash);
		if( (input_range[0] >= this._yMin) && (input_range[0] <= this._yMax) )
		{
			int y = this.YtoPixel(input_range[0]);
			g2d.draw(new Line2D.Double(rect.x, y, rect.x+rect.width, y) );
		}
		if( (input_range[1] >= this._yMin) && (input_range[1] <= this._yMax) )
		{
			int y = this.YtoPixel(input_range[1]);
			g2d.draw(new Line2D.Double(rect.x, y, rect.x+rect.width, y) );
		}
	}
	//------------------------------------------------------------------------------
	/**
	 * Conversion d'une abscisse x en abscisse pixel dans le panel
	 * @param pb objet PlotBox
	 * @param x abscisse a convertir
	 * @param x_min abscisse min de l'objet PlotBox (passer PlotBox._xMin)
	 * @param x_max abscisse max de l'objet PlotBox (passer PlotBox._xMax)
	 * @return abscisse pixel dans le panel
	 */
	public static int XtoPixel( PlotBox pb, double x, double x_min, double x_max )
	{
		Rectangle rect = pb.getPlotRectangle();
		return (int)(rect.getX() + (((x-x_min)*rect.getWidth())/(x_max-x_min)));
	}
	//------------------------------------------------------------------------------
	/**
	 * Conversion d'une abscisse x en pixel en abscisse dans le panel
	 * @param x abscisse en pixel a convertir
	 * @return abscisse dans le panel
	 */
	public double PixelToX( int x )
	{
		Rectangle rect = this.getPlotRectangle();
		return this._xMin + ((((double)x - (double)rect.x)/(double)rect.width)*(this._xMax-this._xMin));
	}
	//------------------------------------------------------------------------------
	/**
	 * Conversion d'une ordonnee y en pixel en ordonnnee dans le panel
	 * @param x abscisse en pixel a convertir
	 * @return abscisse dans le panel
	 */
	public double PixelToY( int y )
	{
		Rectangle rect = this.getPlotRectangle();

		y-- ; //there's an one-pixel offset
		return ((this._yMin)*2 + ((this._yMax - this._yMin) - (this._yMin + ((((double)y - (double)rect.y)/(double)rect.height)*(this._yMax-this._yMin)))));
	}

	//------------------------------------------------------------------------------
	/**
	 * Conversion d'une abscisse x en abscisse pixel dans le panel\n
	 * @param x abscisse a convertir
	 */
	public int XtoPixel( double x ) { return FISPlot.XtoPixel(this, x, this._xMin, this._xMax); }
	//------------------------------------------------------------------------------
	/**
	 * Conversion d'une ordonnee y en ordonnee pixel dans le panel
	 * @param y ordonnee a convertir
	 * @return ordonnee pixel dans le panel
	 */
	public int YtoPixel( double y )
	{
		Rectangle rect = this.getPlotRectangle();
		return (int)(rect.getY() + rect.getHeight() - (((y-_yMin)*rect.getHeight())/(_yMax-_yMin))) + 1;
	}
}
