//***********************************************************************
//
//                              JAdeqationFrame.java
//
// Author(s) : Anne TIREAU
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.Point;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collections;

import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JViewport;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumnModel;

import util.tableheader.AttributiveCellTableModel;
import util.tableheader.CellSpan;
import util.tableheader.ColumnGroup;
import util.tableheader.GroupableTableHeader;
import util.tableheader.MultiSpanCellTable;

public class JAdequationFrame extends JFISChildFrame 
{
	private Object[][] row_header_name; /**<Tableau des noms des entetes de ligne de la table. (voir MultipleRowHeaderExample.java pour exemple d'utilisation)*/
	private Object[] adequation_header;
	private String[]row_header; /**<Tableau des noms des entetes de ligne de la table. (voir MultipleRowHeaderExample.java pour exemple d'utilisation)*/
	private MultiSpanCellTable fixedTable; /**<Objet MultiSpanCellTable. (voir MultipleRowHeaderExample.java pour exemple d'utilisation)*/
	protected JTable table; /**<Objet table de visualisation des classes inferees.*/
	protected JFIS fis; /**<Objet FIS. */
	protected DefaultTableModel model; /**<Modele de donnees pour la table JClassifInfFrame#table. */
	protected String[] columnsLabels;
	protected int nb_Mf; /**<Nombre de mf */
	protected int nb_Item;
	protected int output_number; /**<Index de la sortie inferee du FIS */
	protected JFile perf_file; /**<Objet fis#PerformanceFile. Fichier performance calcule par la fonction JPerformanceFrame#Compute. */
	protected int mf_index;
	protected int match_index;
	protected Object[][] adequation_data; 
	protected FISListener FISListener = null; 
	protected JFileListener JFileListener = null; 
	private JScrollPane scroll = null;


	protected void ReadAdequationData() throws Throwable
	{
		this.nb_Item = this.perf_file.ItemCount();
		Double [] maxItem =	new Double[this.nb_Item];
		double[][] adeq_data = 	new double[this.nb_Mf][];
		double [] match = new double[this.nb_Item];

		// valeurs de la colonne match
		match = this.perf_file.VariableData(this.match_index);
		
		// valeurs observees de chaque sef
		for(int i=0; i<this.nb_Mf; i++)
		{
			adeq_data[i] = this.perf_file.VariableData(this.mf_index + i);
			//recherche si max dans les nouvelles valeurs du sef
			for(int j=0; j< this.nb_Item; j++)
			{
				if(maxItem[j] != null)
					maxItem[j] = Math.max(adeq_data[i][j],maxItem[j].doubleValue());
				else maxItem[j] = adeq_data[i][j];
			}
		}
		// initialize array 
	for(int i=0; i<this.nb_Mf; i++)
			for(int j=0; j<this.nb_Item; j++)
			    {
				this.adequation_data[i][0]=0;
				this.adequation_data[i][1]=0;
			    }
		
		//construction du tableau de donnees a afficher
		for(int i=0; i<this.nb_Mf; i++)
			for(int j=0; j<this.nb_Item; j++)
				if(adeq_data[i][j] == maxItem[j])
				{
					this.adequation_data[i][0] = Double.valueOf(this.adequation_data[i][0].toString()) + match[j]; 
					this.adequation_data[i][1] = 1 + Integer.parseInt(this.adequation_data[i][1].toString());
				}
	}

	//------------------------------------------------------------------------------
	/**
	 * Initialisation des objets de la fenetre.
	 * @throws Throwable 
	 */
	protected void Init() throws Throwable
	{
		// header des lignes
		this.nb_Mf = this.fis.GetOutput(this.output_number).GetNbMF();

		// noms des headers de ligne
		row_header_name = new Object[this.nb_Mf][2];
		row_header_name[0][0] = "infered";
		for( int i=0 ; i<this.nb_Mf ; i++ )
			row_header_name[i][1] = new String ("MF"+(i+1));

		// header des columns
		this.adequation_header = new Object[]{LocaleFIS.GetString("Match"),LocaleFIS.GetString("items")};
		
		//initialisation 
		this.adequation_data = new Object[this.nb_Mf][2];
		for(int i=0; i<this.nb_Mf; i++)
			for(int j=0; j<2; j++)
				this.adequation_data [i][j] = 0;
		
				
		//lecture donnees fichier et algo
		this.ReadAdequationData();	
		
		AttributiveCellTableModel fixedModel = new AttributiveCellTableModel(this.row_header_name, this.adequation_header) {
			public boolean isCellEditable(int row, int col) { return false; } };

			CellSpan cellAtt =(CellSpan)fixedModel.getCellAttribute();
			// combinaisons de headers
			int[] cell_mf = new int[this.nb_Mf];
			for( int i=0 ; i<cell_mf.length ; i++ )
				cell_mf[i] = i;
			cellAtt.combine(cell_mf, new int[] {0});
			cellAtt.combine(new int[] {this.nb_Mf}, new int[] {0,1});
			
			
			model = new DefaultTableModel(row_header_name.length, this.adequation_header.length) 
			{
				public boolean isCellEditable(int row, int col) { return false; }
				public Object getValueAt(int row, int column) { return adequation_data[row][column]; } };

				fixedTable = new MultiSpanCellTable( fixedModel );
				table = new JTable( model );
				table.getColumnModel().getColumn(0).setCellRenderer(new DefaultTableCellRenderer() {
					public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
						super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
						this.setHorizontalAlignment(SwingConstants.CENTER);
						return this; }
				} );
				table.getTableHeader().setReorderingAllowed(false);
				table.getTableHeader().setResizingAllowed(false);
				table.getTableHeader().setDefaultRenderer(new ColumnHeaderRenderer(table));
				fixedTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
				fixedTable.setDefaultRenderer(Object.class, new RowHeaderRenderer(fixedTable));
				fixedTable.setGridColor(table.getTableHeader().getBackground());
				
				//suite
				model.setDataVector(this.adequation_data, adequation_header);
				
				
				table = new JTable( model )
				{
					protected JTableHeader createDefaultTableHeader() { return new GroupableTableHeader(columnModel); }
				};

				// centrage des valeurs dans la table
				for( int i=0 ; i<table.getColumnModel().getColumnCount() ; i++ )
				{
					table.getColumnModel().getColumn(i).setCellRenderer(new DefaultTableCellRenderer()
					{
						public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
						{
							super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
							this.setHorizontalAlignment(SwingConstants.CENTER);
							return this;
						}
					} );
				}

				this.Translate();
		
				// listener du FIS
				if( this.FISListener != null )
					this.fis.RemoveFISListener(this.FISListener);
				this.FISListener = new FISListener() {
					public void FISClosed() {}
					public void InputReplaced(int input_number) {}
					public void OutputReplaced(int output_number)
					{
						try
						{
							// la sortie a ete change en floue
							//if( JAdequationFrame.this.fis.GetOutput(output_number).GetNature() == JOutput.FUZZY )
								//JAdequationFrame.this.dispose(); // on ferme la fenetre
						}
						catch( Throwable t ) {}
					}
					public void InputNameChanged(int input_number) {}
					public void OutputNameChanged(int output_number) { try { JAdequationFrame.this.Translate(); } catch (Throwable t) {} }
					public void InputRangeChanged(int input_number) {}
					public void OutputRangeChanged(int output_number) {}
					public void MFRemovedInInput(int input_number) {}
					public void MFRemovedInOutput(int output_number) {}
					public void MFAddedInInput(int input_number) {}
					public void MFAddedInOutput(int output_number) {}
					public void MFReplacedInInput(int input_number) {}
					public void MFReplacedInOutput(int output_number) {}
					public void InputActiveChanged(int input_number) {}
					public void OutputActiveChanged(int output_number) {}
					public void InputRemoved(int input_number) {}
					public void OutputRemoved(int output_number) {}
					public void InputAdded(int input_number) {}
					public void OutputAdded(int output_number) {}
					public void OutputDefuzChanged(int output_number) {}
					public void OutputImpliChanged( int output_number ) 
					{
						try
						{
							// la sortie n'est plus impli
							if( JAdequationFrame.this.fis.GetOutput(output_number).GetImplicatif() == false )
								JAdequationFrame.this.dispose(); // on ferme la fenetre
						}
						catch( Throwable t ) {}
					}
					public void OutputDisjChanged(int output_number) {}
					public void OutputClassifChanged(int output_number) {}
					public void OutputDefaultChanged(int output_number) {}
					public void OutputAlarmThresChanged(int output_number) {}
					public void RulesModified() {}
					public void ConjunctionChanged() {}
				};
				this.fis.AddFISListener(this.FISListener);

				// listener du fichier de donnees
				if( this.JFileListener != null )
					this.perf_file.RemoveJFileListener(this.JFileListener);
				this.JFileListener = new JFileListener() {
					public void Closed() { JAdequationFrame.this.dispose(); }
					public void ReLoaded() { JAdequationFrame.this.DataReLoaded(); }
				};
				this.perf_file.AddJFileListener(this.JFileListener);
	}
	//------------------------------------------------------------------------------
	/**
	 * Initialistion des objets de la fenetre et Affichage de la fenetre.
	 * @throws Throwable 
	 */
	public void Show() throws Throwable
	{
		this.Init();
		
		Point position = null;
		if( this.scroll != null )
		{
			this.getContentPane().remove(this.scroll);
			position = new Point(this.getX(), this.getY());
		}
		this.scroll = new JScrollPane( this.table );
		JViewport viewport = new JViewport();
		viewport.setView(this.fixedTable);
		viewport.setPreferredSize(this.fixedTable.getPreferredSize());
		this.scroll.setRowHeaderView(viewport);

		// ajuste la largeur des colonnes
		TableColumnModel columns = this.table.getColumnModel();
		for( int i=0 ; i<columns.getColumnCount() ; i++ )
			columns.getColumn(i).setPreferredWidth(100);
		table.setPreferredScrollableViewportSize(new Dimension(this.table.getPreferredSize().width, this.table.getPreferredSize().height));

		// affichage de la fenetre
		getContentPane().add(this.scroll, BorderLayout.CENTER);
		this.pack();
		if( position != null )
			this.setLocation(position);
		else
			this.setLocation(this.ChildPosition(this.getSize()));
		this.setResizable(false);
		super.setVisible(true);
		
	}
	//------------------------------------------------------------------------------
	/**
	 * Fonction appelee lorsque les donnees sont reinferees\n
	 * appelee par le listener JClassifInfFrame#JFileListener lors de la modification du fichier JClassifInfFrame#perf_file.\n
	 */
	protected void DataReLoaded()
	{
		try { this.Show(); }
		catch (Throwable t) {}
	}
	//------------------------------------------------------------------------------
	/**
	 * Fermeture de la fenetre
	 */
	public void dispose()
	{
		super.dispose();
		this.fis.RemoveFISListener(this.FISListener);
		this.perf_file.RemoveJFileListener(this.JFileListener);
	}
	//------------------------------------------------------------------------------
	public JAdequationFrame( JFISFrame parent, int output_number, JFIS fis, int obs_index, int match_index, JFile perf_file) throws Throwable
	{
		super(parent);
		this.fis = fis;
		this.output_number = output_number;
		this.mf_index = obs_index + 1; //juste apres en impli
		this.match_index = match_index;
		this.perf_file = perf_file;
		this.Show();
	}
	
	//------------------------------------------------------------------------------
	public void Translate() throws Throwable {
		//super.Translate();
		String output_name = this.fis.GetOutput(this.output_number).GetName();
		this.setTitle(output_name + " " + LocaleFIS.GetString("infered") + " / " + output_name + " " + LocaleFIS.GetString("observed"));
		this.repaint();

	}
	//------------------------------------------------------------------------------
	/**
	 * "Renderer" des entetes de colonne pour la fenetre fis#JClassifInfFrame.
	 */
	class ColumnHeaderRenderer extends JLabel implements TableCellRenderer {
		ColumnHeaderRenderer(JTable table) {
			JTableHeader header = table.getTableHeader();
			setOpaque(true);
			setBorder(UIManager.getBorder("TableHeader.cellBorder"));
			setHorizontalAlignment(CENTER);
			setForeground(header.getForeground());
			setBackground(header.getBackground());
			setFont(header.getFont());
		}
		//------------------------------------------------------------------------------
		/**
		 * Traduit l'entete de la colonne.
		 */
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
			setText(LocaleFIS.GetString("items"));
			return this;
		}
	}
	
	//------------------------------------------------------------------------------
	/**
	 * "Renderer" des entetes de ligne pour la fenetre fis#JClassifInfFrame.
	 */
	class RowHeaderRenderer extends JLabel implements TableCellRenderer {
		RowHeaderRenderer(JTable table) {
			JTableHeader header = table.getTableHeader();
			setOpaque(true);
			setBorder(UIManager.getBorder("TableHeader.cellBorder"));
			setHorizontalAlignment(CENTER);
			setForeground(header.getForeground());
			setBackground(header.getBackground());
			setFont(header.getFont());
		}
		//------------------------------------------------------------------------------
		/**
		 * Traduit les entetes de ligne.
		 */
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
			setText((value == null) ? "" : LocaleFIS.GetStringNoException(value.toString()) + ((value.toString().equals("mf") == true) ? " " + NumberFormat.getInstance().format(columnsLabels[row]) : ""));
			return this;
		}
	}
}
