//***********************************************************************
//
//                              JClassifInfFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;
import util.tableheader.*;
import java.text.*;
import java.lang.reflect.Array;

//------------------------------------------------------------------------------
/**
 * Fenetre de visualisation des classes inferees depuis un fichier de donnees.\n
 * cette fenetre est appelee depuis la fenetre fis#JPerformanceFrame (menu JPerformanceFrame#jMenuClassif_actionPerformed) dans le cas ou le fichier de donnees ne contient pas de valeurs observees, et que la sortie est classif.\n
 * on visualise les classes inferees sous forme d'une table (nombre d'exemples inferes dans chaque classe):
 * - nombre d'exemples inferes en colonne
 * - une classe par ligne
 * - la derniere ligne contient les exemples non classes
 *
 */
class JClassifInfFrame extends JFISChildFrame
{
	private Object[][] row_header_name; /**<Tableau des noms des entetes de ligne de la table. (voir MultipleRowHeaderExample.java pour exemple d'utilisation)*/
	private MultiSpanCellTable fixedTable; /**<Objet MultiSpanCellTable. (voir MultipleRowHeaderExample.java pour exemple d'utilisation)*/
	protected JTable table; /**<Objet table de visualisation des classes inferees.*/
	protected JFIS fis; /**<Objet FIS. */
	protected DefaultTableModel model; /**<Modele de donnees pour la table JClassifInfFrame#table. */
	protected double[] classes; /**<Tableau des labels des classes. */
	protected int nb_class; /**<Nombre de classes:\n - dans les donnees si le fichier de donnees contient des donnees observees\n - dans les conclusions des regles si pas de donnees observees */
	protected int output_number; /**<Index de la sortie inferee du FIS JClassifInfFrame#fis. */
	protected JFile perf_file; /**<Objet fis#PerformanceFile. Fichier performance calcule par la fonction JPerformanceFrame#Compute. */
	protected int inf_index; /**<Index sur les donnees inferees dans le fichier performance JClassifInfFrame#perf_file. */
	protected Object[][] classif_data; /**<Tableau nombre d'exemples inferes par classes\n - classes inferees en ligne / exemples en colonne dans la fenetre JClassifInfFrame \n - classes inferees en ligne / classes observees en colonne dans la fenetre JClassifInfObsFrame \n - la derniere ligne contient les exemples non classes */
	protected FISListener FISListener = null; /**<Listener du FIS JClassifInfFrame#fis.*/
	protected JFileListener JFileListener = null; /**<Listener du fichier JClassifInfFrame#perf_file.*/
	private JScrollPane scroll = null; /**<Objet JScrollPane pour affichage de la table JClassifInfFrame#table. */
	//------------------------------------------------------------------------------
	/**
	 * Lecture des donnees inferees dans le fichier performance JClassifInfFrame#perf_file.\n
	 * - Classement des donnees inferees dans le tableau JClassifInfFrame#classif_data.\n
	 * - En cas d'exception, le tableau JClassifInfFrame#classif_data est reinitialise a 0.
	 */
	protected void ReadClassifData()
	{
		this.classif_data = new Object[this.nb_class+1][1];
		ArrayList list = new ArrayList();
		try
		{
			double[] inf_data  = this.perf_file.VariableData(this.inf_index);
			for( int i=0 ; i<inf_data.length ; i++ )
				list.add(new Double(inf_data[i]));  // ajout des donnees inferees dans une liste
			Collections.sort(list); // tri de la liste
			// extrait le nombre d'exemples par classes inferees depuis la liste et remplissage du tableau classif_data
			for( int i=0 ; i<this.classes.length ; i++ )
				this.classif_data[i][0] = new Integer(list.indexOf(new Double(this.classes[i])) == -1 ? 0 : list.lastIndexOf(new Double(this.classes[i])) - list.indexOf(new Double(this.classes[i])) + 1);
			double default_value = this.fis.GetOutput(this.output_number).GetDefaultValue();
			this.classif_data[this.nb_class][0] = new Integer(list.indexOf(new Double(default_value)) == -1 ? 0 : list.lastIndexOf(new Double(default_value)) - list.indexOf(new Double(default_value)) + 1);
		}
		catch( Throwable t )
		{
			// il y a eu une exception lors de la lecture des donnees performance > init de classif_data a 0 et non classes au nombre d'exemple
			for( int i=0 ; i<this.nb_class ; i++ )
				this.classif_data[i][0] = new Integer(0);
			this.classif_data[this.nb_class][0] = new Integer(this.perf_file.GetData().length);
		}
	}
	//------------------------------------------------------------------------------
	/**
	 * Initialisation des objets de la fenetre.
	 */
	protected void Init() throws Throwable
	{
		// lecture des classes
		this.classes = this.fis.GetOutput(this.output_number).GetClasses();
		this.nb_class = this.classes.length;

		// lecture du fichier de performance
		this.ReadClassifData();

		// noms des headers de ligne
		row_header_name = new Object[this.nb_class+1][2];
		row_header_name[0][0] = "infered";
		for( int i=0 ; i<this.nb_class ; i++ )
			row_header_name[i][1] = "class";
		row_header_name[this.nb_class][0] = "NotClassified";

		AttributiveCellTableModel fixedModel = new AttributiveCellTableModel(row_header_name, new Object[]{"",""}) {
			public boolean isCellEditable(int row, int col) { return false; } };

			CellSpan cellAtt =(CellSpan)fixedModel.getCellAttribute();
			// combinaisons de headers
			int[] cell_class = new int[this.nb_class];
			for( int i=0 ; i<cell_class.length ; i++ )
				cell_class[i] = i;
			cellAtt.combine(cell_class, new int[] {0});
			cellAtt.combine(new int[] {this.nb_class}, new int[] {0,1});

			model = new DefaultTableModel(row_header_name.length, 1) {
				public boolean isCellEditable(int row, int col) { return false; }
				public Object getValueAt(int row, int column) { return classif_data[row][column]; } };

				fixedTable = new MultiSpanCellTable( fixedModel );
				table = new JTable( model );
				table.getColumnModel().getColumn(0).setCellRenderer(new DefaultTableCellRenderer() {
					public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
						super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
						this.setHorizontalAlignment(SwingConstants.CENTER);
						return this; }
				} );
				table.getTableHeader().setReorderingAllowed(false);
				table.getTableHeader().setResizingAllowed(false);
				table.getTableHeader().setDefaultRenderer(new ColumnHeaderRenderer(table));
				fixedTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
				fixedTable.setDefaultRenderer(Object.class, new RowHeaderRenderer(fixedTable));
				fixedTable.setGridColor(table.getTableHeader().getBackground());
				this.Translate();

				// listener du FIS
				if( this.FISListener != null )
					this.fis.RemoveFISListener(this.FISListener);
				this.FISListener = new FISListener() {
					public void FISClosed() {}
					public void InputReplaced(int input_number) {}
					public void OutputReplaced(int output_number)
					{
						try
						{
							// la sortie a ete change en floue
							if( JClassifInfFrame.this.fis.GetOutput(output_number).GetNature() == JOutput.FUZZY )
								JClassifInfFrame.this.dispose(); // on ferme la fenetre
						}
						catch( Throwable t ) {}
					}
					public void InputNameChanged(int input_number) {}
					public void OutputNameChanged(int output_number) { try { JClassifInfFrame.this.Translate(); } catch (Throwable t) {} }
					public void InputRangeChanged(int input_number) {}
					public void OutputRangeChanged(int output_number) {}
					public void MFRemovedInInput(int input_number) {}
					public void MFRemovedInOutput(int output_number) {}
					public void MFAddedInInput(int input_number) {}
					public void MFAddedInOutput(int output_number) {}
					public void MFReplacedInInput(int input_number) {}
					public void MFReplacedInOutput(int output_number) {}
					public void InputActiveChanged(int input_number) {}
					public void OutputActiveChanged(int output_number) {}
					public void InputRemoved(int input_number) {}
					public void OutputRemoved(int output_number) {}
					public void InputAdded(int input_number) {}
					public void OutputAdded(int output_number) {}
					public void OutputDefuzChanged(int output_number) {}
					public void OutputImpliChanged( int output_number ) {}
					public void OutputDisjChanged(int output_number) {}
					public void OutputClassifChanged(int output_number)
					{
						try
						{
							// la sortie n'est plus classif
							if( JClassifInfFrame.this.fis.GetOutput(output_number).GetClassif() == false )
								JClassifInfFrame.this.dispose(); // on ferme la fenetre
						}
						catch( Throwable t ) {}
					}
					public void OutputDefaultChanged(int output_number) {}
					public void OutputAlarmThresChanged(int output_number) {}
					public void RulesModified() {}
					public void ConjunctionChanged() {}
				};
				this.fis.AddFISListener(this.FISListener);

				// listener du fichier de donnees
				if( this.JFileListener != null )
					this.perf_file.RemoveJFileListener(this.JFileListener);
				this.JFileListener = new JFileListener() {
					public void Closed() { JClassifInfFrame.this.dispose(); }
					public void ReLoaded() { JClassifInfFrame.this.DataReLoaded(); }
				};
				this.perf_file.AddJFileListener(this.JFileListener);
	}
	//------------------------------------------------------------------------------
	public JClassifInfFrame( JFISFrame parent, int output_number, JFIS fis, int inf_index, JFile perf_file) throws Throwable
	{
		super(parent);
		this.fis = fis;
		this.output_number = output_number;
		this.inf_index = inf_index;
		this.perf_file = perf_file;
		this.Show();
	}
	//------------------------------------------------------------------------------
	/**
	 * Fermeture de la fenetre
	 */
	public void dispose()
	{
		super.dispose();
		this.fis.RemoveFISListener(this.FISListener);
		this.perf_file.RemoveJFileListener(this.JFileListener);
	}
	//------------------------------------------------------------------------------
	/**
	 * Fonction appelee lorsque les donnees sont reinferees\n
	 * appelee par le listener JClassifInfFrame#JFileListener lors de la modification du fichier JClassifInfFrame#perf_file.\n
	 */
	protected void DataReLoaded()
	{
		try { this.Show(); }
		catch (Throwable t) {}
	}
	//------------------------------------------------------------------------------
	/**
	 * Initialistion des objets de la fenetre et Affichage de la fenetre.
	 */
	public void Show() throws Throwable
	{
		this.Init();
		Point position = null;
		if( this.scroll != null )
		{
			this.getContentPane().remove(this.scroll);
			position = new Point(this.getX(), this.getY());
		}
		this.scroll = new JScrollPane( this.table );
		JViewport viewport = new JViewport();
		viewport.setView(this.fixedTable);
		viewport.setPreferredSize(this.fixedTable.getPreferredSize());
		this.scroll.setRowHeaderView(viewport);

		// ajuste la largeur des colonnes
		TableColumnModel columns = this.table.getColumnModel();
		for( int i=0 ; i<columns.getColumnCount() ; i++ )
			columns.getColumn(i).setPreferredWidth(100);
		table.setPreferredScrollableViewportSize(new Dimension(this.table.getPreferredSize().width, this.table.getPreferredSize().height));

		// affichage de la fenetre
		getContentPane().add(this.scroll, BorderLayout.CENTER);
		this.pack();
		if( position != null )
			this.setLocation(position);
		else
			this.setLocation(this.ChildPosition(this.getSize()));
		this.setResizable(false);
		super.setVisible(true);
	}
	//------------------------------------------------------------------------------
	/**
	 * Traduction de la fenetre
	 */
	public void Translate() throws Throwable
	{
		this.setTitle(this.fis.GetOutput(this.output_number).GetName() + " " + LocaleFIS.GetString("infered"));
		this.repaint();
	}
	//------------------------------------------------------------------------------
	/**
	 * "Renderer" des entetes de ligne pour la fenetre fis#JClassifInfFrame.
	 */
	class RowHeaderRenderer extends JLabel implements TableCellRenderer {
		RowHeaderRenderer(JTable table) {
			JTableHeader header = table.getTableHeader();
			setOpaque(true);
			setBorder(UIManager.getBorder("TableHeader.cellBorder"));
			setHorizontalAlignment(CENTER);
			setForeground(header.getForeground());
			setBackground(header.getBackground());
			setFont(header.getFont());
		}
		//------------------------------------------------------------------------------
		/**
		 * Traduit les entetes de ligne.
		 */
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
			setText((value == null) ? "" : LocaleFIS.GetStringNoException(value.toString()) + ((value.toString().equals("class") == true) ? " " + NumberFormat.getInstance().format(classes[row]) : ""));
			return this;
		}
	}
	//------------------------------------------------------------------------------
	/**
	 * "Renderer" des entetes de colonne pour la fenetre fis#JClassifInfFrame.
	 */
	class ColumnHeaderRenderer extends JLabel implements TableCellRenderer {
		ColumnHeaderRenderer(JTable table) {
			JTableHeader header = table.getTableHeader();
			setOpaque(true);
			setBorder(UIManager.getBorder("TableHeader.cellBorder"));
			setHorizontalAlignment(CENTER);
			setForeground(header.getForeground());
			setBackground(header.getBackground());
			setFont(header.getFont());
		}
		//------------------------------------------------------------------------------
		/**
		 * Traduit l'entete de la colonne.
		 */
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
			setText(LocaleFIS.GetString("items"));
			return this;
		}
	}
}



//------------------------------------------------------------------------------
/**
 * Fenetre de visualisation des classes inferees depuis un fichier de donnees.\n
 * cette fenetre est appelee depuis la fenetre fis#JPerformanceFrame (menu JPerformanceFrame#jMenuClassif_actionPerformed) dans le cas ou le fichier de donnees contient des valeurs observees, et que la sortie est classif.\n
 * on visualise les classes inferees sous forme d'une table (nombre d'exemples inferes dans chaque classe):
 * - les classes observees en colonne
 * - les classes inferees en ligne
 * - la derniere ligne contient les exemples non classes
 */
class JClassifInfObsFrame extends JClassifInfFrame {
	protected int obs_index; /**<Index sur les donnees observees dans le fichier performance JClassifInfFrame#perf_file. */
	//------------------------------------------------------------------------------
	public JClassifInfObsFrame( JFISFrame parent, int fis_index, JFIS fis, int inf_index, int obs_index, JFile perf_file) throws Throwable
	{
		super(parent, fis_index, fis, inf_index, perf_file);
		this.obs_index = obs_index;
	}
	//------------------------------------------------------------------------------
	/**
	 * Initialisation des objets de la fenetre.
	 */
	protected void Init() throws Throwable
	{
		super.Init();

		// header de la table
		Object[] class_header = new Object[this.nb_class];
		for( int i=0 ; i<class_header.length ; i++ )
			class_header[i] = "class";

		model.setDataVector(this.classif_data, class_header);
		
		table = new JTable( model )
		{
			protected JTableHeader createDefaultTableHeader() { return new GroupableTableHeader(columnModel); }
		};

		// centrage des valeurs dans la table
		for( int i=0 ; i<table.getColumnModel().getColumnCount() ; i++ )
		{
			table.getColumnModel().getColumn(i).setCellRenderer(new DefaultTableCellRenderer()
			{
				public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
				{
					super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
					this.setHorizontalAlignment(SwingConstants.CENTER);
					return this;
				}
			} );
		}

		// groupe de colonne
		TableColumnModel cm = table.getColumnModel();
		ColumnGroup g_name = new ColumnGroup("observed");
		for( int i=0 ; i<this.nb_class ; i++ )
			g_name.add(cm.getColumn(i));
		final GroupableTableHeader header = (GroupableTableHeader)table.getTableHeader();
		header.addColumnGroup(g_name);
		header.setDefaultRenderer(new JClassifInfObsFrame.ColumnHeaderRenderer(this.table));
		g_name.setHeaderRenderer(new JClassifInfObsFrame.ColumnHeaderRenderer(this.table));
	}
	//------------------------------------------------------------------------------
	/**
	 * Lecture des donnees inferees dans le fichier performance JClassifInfFrame#perf_file.\n
	 * - Classement des donnees inferees dans le tableau JClassifInfFrame#classif_data.\n
	 * - En cas d'exception, le tableau JClassifInfFrame#classif_data est reinitialise a 0.
	 */
	protected void ReadClassifData()
	{
		/*
		 * Classe contenant un couple de donnee infere et observe\n
		 * cette classe implemente l'interface Comparable pour permettre de trier les objets dans un tableau de type Arrays.
		 * tri des objets par donnee observee puis par donnee inferee
		 */
		class DataObsInf implements Comparable
		{
			double data_obs; /* donnee observee */
			double data_inf; /* donnee inferee */
			public DataObsInf( double data_obs, double data_inf ) { this.data_obs = data_obs; this.data_inf = data_inf; }
			/* Teste l'egalite des objets (necessaire pour les fonctions IndexOf et LastIndexOf de la classe java.util.List) @param obj objet a tester @return true si les objets sont egaux, false sinon. */
			public boolean equals(Object obj) { return (((DataObsInf)obj).data_obs == this.data_obs) && (((DataObsInf)obj).data_inf == this.data_inf) ? true : false; }
			/* compare les objets (interface Comparable): @param obj objet a comparer @return -1 si obj superieur a this, 0 si egaux , 1 si this superieur a obj */
			public int compareTo(Object obj)
			{
				double diff_obs = this.data_obs - ((DataObsInf)obj).data_obs;
				if( diff_obs > 0 )
					return 1;
				else if( diff_obs < 0 )
					return -1;
				else
				{
					double diff_inf = this.data_inf - ((DataObsInf)obj).data_inf;
					if( diff_inf > 0 ) return 1;
					if( diff_inf < 0 ) return -1;
				}
				return 0;
			}
		};

		try
		{
			this.classif_data = new Object[this.nb_class+1][this.nb_class];
			for( int i=0 ; i<classif_data.length ; i++ )
				for( int j=0 ; j<classif_data[i].length ; j++ )
					this.classif_data[i][j] = new Integer(0);
			// lecture du fichier performance
			double[] inf_data  = this.perf_file.VariableData(this.inf_index);
			double[] obs_data  = this.perf_file.VariableData(this.obs_index);

			// creation du tableau d'objets DataObsInf et tri de ce tableau
			DataObsInf []obs_inf = new DataObsInf[obs_data.length];
			for( int i=0 ; i<obs_data.length ; i++ )
				obs_inf[i] = new DataObsInf(obs_data[i], inf_data[i]);
			Arrays.sort(obs_inf);

			// conversion du tableau obs_inf en java.util.List
			java.util.List list_obs_inf = Arrays.asList(obs_inf);

			// extraction du nombre d'exemple par classe
			for( int i_obs=0 ; i_obs<this.nb_class ; i_obs++ )
				for( int i_inf=0 ; i_inf<this.nb_class ; i_inf++ )
				{
					DataObsInf search_obj = new DataObsInf(this.classes[i_obs], this.classes[i_inf]);
					this.classif_data[i_inf][i_obs] = new Integer(list_obs_inf.indexOf(search_obj) == -1 ? 0 : list_obs_inf.lastIndexOf(search_obj) - list_obs_inf.indexOf(search_obj) + 1 );
				}
			// recherche des non classes
			for( int i_obs=0 ; i_obs<this.nb_class ; i_obs++ )
			{
				double default_value = this.fis.GetOutput(this.output_number).GetDefaultValue();
				DataObsInf search_obj = new DataObsInf(this.classes[i_obs], default_value);
				this.classif_data[this.nb_class][i_obs] = new Integer(list_obs_inf.indexOf(search_obj) == -1 ? 0 : list_obs_inf.lastIndexOf(search_obj) - list_obs_inf.indexOf(search_obj) + 1 );
			}
		}
		catch( Throwable t )
		{
			for( int i=0 ; i<this.classif_data.length ; i++ )
				for( int j=0 ; j<this.classif_data[i].length ; j++ )
					this.classif_data[i][j] = new Integer(0);
		}
	}
	//------------------------------------------------------------------------------
	/**
	 * Traduction de la fenetre
	 */
	public void Translate() throws Throwable
	{
		super.Translate();
		String output_name = this.fis.GetOutput(this.output_number).GetName();
		this.setTitle(output_name + " " + LocaleFIS.GetString("infered") + " / " + output_name + " " + LocaleFIS.GetString("observed"));
		this.repaint();
	}
	//------------------------------------------------------------------------------
	/**
	 * "Renderer" des entetes de colonne pour la fenetre fis#JClassifInfObsFrame.
	 */
	class ColumnHeaderRenderer extends JClassifInfFrame.ColumnHeaderRenderer
	{
		ColumnHeaderRenderer(JTable table) { super(table); }
		//------------------------------------------------------------------------------
		/**
		 * Traduit les entetes de colonne.
		 */
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
		{
			setText((value == null) ? "" : LocaleFIS.GetStringNoException(value.toString()) + ((value.toString().equals("class") == true) ? " " + NumberFormat.getInstance().format(classes[column]) : ""));
			return this;
		}
	}
}










