//***********************************************************************
//
//                              JComboBoxInputs.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.event.EventListenerList;
import javax.swing.plaf.basic.BasicComboBoxRenderer;

// TODO voir comment gerer les modifications dynamiques sur les entrees
/*
 * Classe de selection des entrees (ou sorties) d'un FIS en JComboBox.
 * - entrees inactives non selectionnables
 * - possibilite de grouper des objets JComboBoxInputs dans un JComboBoxInputsGroup pour rendre les selections exclusives entre plusieurs JComboBoxInputs
 */
public class JComboBoxInputs extends JComboBox {

    private JInput[] inputs = null;

    // memorisation de l'index selectionne
    private int previousSelectedIndex = -1;

    // redirection des evenements de type ActionListenner exterieurs a la classe vers un EventListenerList local
    // la classe utilisant deja en interne un listener ActionListener pour la gestion des elements desactives
    private EventListenerList localListenerList = new EventListenerList();

    private JComboBoxInputsGroup group = null;

    /**
     * 
     */
    public JComboBoxInputs() {
        super();
        initialize();
        this.setRenderer(new BasicComboBoxRenderer() {
            public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
                try {
                    if (JComboBoxInputs.this.isIndexEnabled(index) == false) {
                        // element desactive grise
                        this.setForeground(Color.gray);
                        setBackground(list.getBackground());
                    }
                } catch (Throwable e) {
                    this.setForeground(Color.gray);
                    setBackground(list.getBackground());
                    // TODO Bloc catch auto-genere
                    e.printStackTrace();
                }
                return this;
            }
        });
    }

    /**
     * This method initializes this
     * 
     */
    private void initialize() {
        // ActionListener pour empecher la selection d'un element desactive
        super.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent e) {
                int selectedIndex = ((JComboBox) e.getSource()).getSelectedIndex();
                try {
                    if (JComboBoxInputs.this.isIndexEnabled(selectedIndex) == false) {
                        // element desactive, on selectionne de dernier element selectionne
                        ((JComboBox) e.getSource()).setSelectedIndex(previousSelectedIndex);
                        clickOnDisabledItem(selectedIndex);
                    } else {
                        previousSelectedIndex = selectedIndex;
                        fireLocalActionEvent();
                    }
                } catch (Throwable e1) {
                    // TODO Bloc catch auto-genere
                    e1.printStackTrace();
                }
            }
        });

    }

    /**
     * 
     */
    public void setInputs(JInput[] inputs) throws Throwable {
        this.inputs = inputs;
        // modele de selection: contient toutes les entrees
        DefaultComboBoxModel model = new DefaultComboBoxModel();
        for (int i = 0; i < inputs.length; i++)
            model.addElement(inputs[i].GetName());
        this.setModel(model);
        // selectionne la premiere entree active
        for (int i = 0; i < inputs.length; i++)
            if (this.isIndexEnabled(i) == true) {
                this.setSelectedIndex(i);
                this.previousSelectedIndex = i;
                return;
            }
        // aucune entree selectionnable
        this.setSelectedIndex(-1);
    }

    /* (non-Javadoc)
     * @see javax.swing.JComboBox#addActionListener(java.awt.event.ActionListener)
     */
    public void addActionListener(ActionListener al) {
        // redirection des listeners de type ActionListener vers le localListenerList
        this.localListenerList.add(ActionListener.class, al);
    }

    /* (non-Javadoc)
     * @see javax.swing.JComboBox#removeActionListener(java.awt.event.ActionListener)
     */
    public void removeActionListener(ActionListener al) {
        this.localListenerList.remove(ActionListener.class, al);
    }

    /* (non-Javadoc)
     * @see javax.swing.JComboBox#getActionListeners()
     */
    public ActionListener[] getActionListeners() {
        return (ActionListener[]) this.localListenerList.getListeners(ActionListener.class);
    }

    public void setGroup(JComboBoxInputsGroup group) {
        this.group = group;
    }

    private void fireLocalActionEvent() {
        ActionListener[] listeners = this.getActionListeners();
        for (int i = 0; i < listeners.length; i++)
            listeners[i].actionPerformed(new ActionEvent(this, 0, null));
    }

    protected boolean isIndexEnabled(int index) throws Throwable {
        return index == -1 ? false : this.inputs[index].GetActive() && (this.group == null ? true : this.group.isIndexAvailable(this, index));
    }

    protected void clickOnDisabledItem(int index) {}

}
