//***********************************************************************
//
//                              JDataTableFrame.java
//
// Author(s) : Mathieu GRELIER
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import javax.swing.*;
//import java.text.*;
import java.awt.*;
import java.util.*;
//import javax.swing.border.*;
import java.awt.event.*;
import javax.swing.table.*;
//import javax.swing.event.*;
//import javax.swing.plaf.basic.*;
//import java.text.NumberFormat;
//import java.awt.datatransfer.*;
//import java.awt.print.*;
import util.tableheader.*;
import util.sortabletable.*;
import java.io.*;
import util.*;

public class JDataTableFrame
    extends JFISChildFrame
    implements SortableHeaderListener.SortableObject {

  private JTable jDataTable;
  private JFIS sif;
  private int InputCount, OutputCount;
  private JExtendedDataFile extend_data_file; /**< Fichier de donnees etendu*/
  private SortableTableModel DataFrameModel;
      /**< Modele de donnees regles triable */
  private JTable ExamplesTable; //this table is a field, for value settings convenience
  private JTable InputsTable;
  private JTable OutputsTable;
  private SortButtonRenderer renderer; //triangular button indicating sorting order
  private TableColumnModel columnModel;
  private ListSelectionModel rowSM;
  private JScrollPane InputsScrollPane;
  private JSplitPane jspData = null;
      /**< Objet conteneur des tables entrees et sorties */
  private int hlenght, vlenght; // totlenght; //dimensions in the JDataTableFrame used many times
  double[][] TmpDataFile;
  public Object[] VectorName; //the names in the columns headers
  private Vector tables_headers; /**< Vector containing the JTableHeader objects of the (maximum) three tables used in the JDataTableFrame : examples/active, inputs and outputs */
  private JPopupMenu jPopupRowsSelected; //right button click popup menu
  private JMenuItem RowsSelection;
  private JMenuItem RowsNegativeSelection;
  private JMenuItem RowsSwitch;
  private JMenuItem ActiveAll;
  private JMenuItem ActiveNothing;
  private JSemaphore JDataFrameOpen;
  private FISListener fis_listener = null;
      /**< Listener du FIS JInferenceFrame#fis */
  private JFileListener file_listener = null;
  private int sort_col = -1;
      /**< Memorisation du numero de colonne triees. -1 pour aucune colonne triee */
  private boolean is_ascent; /**< Memorisation ordre de tri */
  private JMenuBar jMenuBarRules;
  private JMenu jMenuDisplay;
  private JCheckBoxMenuItem jMenuDisplayInactive;
  private boolean ShowInactive = false;


  public JDataTableFrame(JFISFrame parent, JFIS fis,
			 JExtendedDataFile data_file) {
    super(parent);
    this.extend_data_file = data_file;
    this.sif = fis;
    try {
      jDataTableInit(); //creation of the JDataTableFrame
      this.pack();
      this.setLocation(this.ChildPosition(this.getSize())); //display
      this.setVisible(true);
    }
    catch (Throwable t) {
      MessageFIS.Error(null, t);
    }
  }

//------------------------------------------------------------------------------
  public void dispose() {
    super.dispose();
    JExtendedDataFile.jhtTableFrames.remove(this.extend_data_file.FileName());   //this data table is not dispalyed now, so its key is removed from the jhtTableFrames hash table
    this.extend_data_file.RemoveJFileListener(this.file_listener);
    if (this.sif != null) {
      this.sif.RemoveFISListener(this.fis_listener);
    }
  }

//------------------------------------------------------------------------------
  private void jDataTableInit() throws Throwable {

    this.DataFrameModel = new SortableTableModel() { // necessary to display boolean values as checkboxes :
      public Class getColumnClass(int col) {
	return getValueAt(0, col).getClass();
      }

      public boolean isCellEditable(int row, int col) {
	if (col == 1) { // only the second column can be edited (the checkboxes)
	  return true;
	}
	else {
	  return false;
	}
      }

    };

    if (sif != null) {
      // listener de SIF //FIS listener :
      if (this.fis_listener != null) {
	this.sif.RemoveFISListener(this.fis_listener);
      }
      this.fis_listener = new FISListener() {
	public void FISClosed() {
	  JDataTableFrame.this.dispose();
	}

	public void InputReplaced(int input_number) {
	  JDataTableFrame.this.ReInit();
	}

	public void OutputReplaced(int output_number) {
	  JDataTableFrame.this.ReInit();
	}

	public void InputActiveChanged(int input_number) {
	  JDataTableFrame.this.ReInit();
	}

	public void OutputActiveChanged(int output_number) {
	  JDataTableFrame.this.ReInit();
	}

	public void InputNameChanged(int input_number) {
	  JDataTableFrame.this.ReInit();
	}

	public void OutputNameChanged(int output_number) {
	  JDataTableFrame.this.ReInit();
	}

	public void InputRangeChanged(int input_number) {}

	public void OutputRangeChanged(int output_number) {}

	public void InputRemoved(int input_number) {}

	public void OutputRemoved(int output_number) {}

	public void InputAdded(int input_number) {}

	public void OutputAdded(int output_number) {}

	public void MFRemovedInInput(int input_number) {}

	public void MFRemovedInOutput(int output_number) {}

	public void MFReplacedInInput(int input_number) {}

	public void MFReplacedInOutput(int output_number) {}

	public void OutputDefuzChanged(int output_number) {}
	
	public void OutputImpliChanged( int output_number ) {}

	public void OutputDisjChanged(int output_number) {}

	public void OutputClassifChanged(int output_number) {}

	public void MFAddedInInput(int input_number) {}

	public void MFAddedInOutput(int output_number) {}

	public void OutputDefaultChanged(int output_number) {}

	public void OutputAlarmThresChanged(int output_number) {}

	public void RulesModified() {}

	public void ConjunctionChanged() {}
      };
      this.sif.AddFISListener(this.fis_listener);
    }

    //extended data file listener
    if (this.file_listener != null) {
      this.extend_data_file.RemoveJFileListener(this.file_listener);
    }
    this.file_listener = new JExtendedFileListener() {
      public void Closed() {
	JDataTableFrame.this.dispose();
      }

      public void ReLoaded() {}
     //the activation status of one example was modified
      public void ActivationChanged(int item, boolean active)
        {
	int row = JDataTableFrame.this.ConvertExampleToIndex(item) ; //because rows can be sorted
	JDataTableFrame.this.DataFrameModel.setValueAt(new Boolean(active), row, 1); //use of the data model's setValueAt // the table's one is just used for clicks on it
	JDataTableFrame.this.ExamplesTable.repaint() ; JDataTableFrame.this.InputsTable.repaint() ;
	if (JDataTableFrame.this.sif != null)
	   JDataTableFrame.this.OutputsTable.repaint() ; //to update the gray/black colors
        }

      //the activation status of an array of examples was modified
      public void ActivationChanged(int []items, boolean active)
      {
      int[] rows = JDataTableFrame.this.ConvertExamplesToIndices(items) ; //because rows can be sorted
      for (int i=0 ; i<rows.length ; i++)
        {
        int row = rows[i] ;
	JDataTableFrame.this.DataFrameModel.setValueAt(new Boolean(active), row, 1);  //use of the data model's setValueAt // the table's one is just used for clicks on it
        }
      JDataTableFrame.this.ExamplesTable.repaint() ; JDataTableFrame.this.InputsTable.repaint() ;
      if (JDataTableFrame.this.sif != null)
	 JDataTableFrame.this.OutputsTable.repaint() ; //to update the gray/black colors
      }

    };
    this.extend_data_file.AddJFileListener(this.file_listener);

    if (sif != null) {
      // menus
      jMenuBarRules = new JMenuBar();
      jMenuDisplay = new JMenu(LocaleFIS.GetString("Display"));
      jMenuDisplayInactive = new JCheckBoxMenuItem(LocaleFIS.GetString(
	  "InactiveVariables"));

      this.setJMenuBar(this.jMenuBarRules);
      jMenuBarRules.add(jMenuDisplay);
      jMenuDisplay.add(jMenuDisplayInactive);
      jMenuDisplayInactive.setSelected(this.ShowInactive);

      jMenuDisplayInactive.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  jMenuDisplayInactive_actionPerformed(e);
	}
      });
    }

    // building of the table model
    // affecte les vecteurs de donnees et de titre au modele de table
    TmpDataFile = this.extend_data_file.GetData();
    hlenght = TmpDataFile[0].length;


    VectorName = new Object[hlenght + 2];
    int i;
    VectorName[0] = new String(LocaleFIS.GetString("items"));
    VectorName[1] = new String(LocaleFIS.GetString("Active"));
    if (sif == null) {
      for (i = 2; i < hlenght + 2; i++) {
	VectorName[i] = new String(LocaleFIS.GetString("Variable") + " " + (i - 1));
      }
    }
    //sinon prend le nombre de variables actives. On doit recuperer leur
    //indices et leurs noms
    else {

      //Filling of the VectorName Vector
      InputCount = sif.NbInputs();
      for (i = 0; i < InputCount; i++) { // au max on explore jusqu'a l'avant-dernier
	JInput CurrentInput = sif.GetInput(i);
	VectorName[i + 2] = new String(CurrentInput.GetName());
      }
      OutputCount = Math.min(sif.NbOutputs(), this.hlenght-this.InputCount); // cas sorties sans donnees
      for (int j = 0; j < OutputCount; j++) { //
	JOutput CurrentOutput = sif.GetOutput(j);
	VectorName[i + 2] = new String(CurrentOutput.GetName());
	i++;
      }
    }

    //Table height is given by the lenght of the first dimension
    vlenght = TmpDataFile.length;
    //on doit rajouter deux nouvelles colonnes, exemples et actifs
    //Columns 'Example' and 'Active' must be added
    Object[][] FormatedDataFile = new Object[vlenght][hlenght + 2];
    for (i = 0; i < vlenght; i++) {
      FormatedDataFile[i][0] = new Integer(i + 1); //Example number
      //retreiving of the active examples
      FormatedDataFile[i][1] = new Boolean(this.extend_data_file.GetActive(i));

      for (int j = 0; j < hlenght; j++) {
	FormatedDataFile[i][j + 2] = new Double(TmpDataFile[i][j]);
      }
    }

    //retreiving of the active examples
    //Building of the table model
    DataFrameModel.setDataVector(FormatedDataFile, VectorName);
    this.tables_headers = new Vector(); //vector for go through table headers
    renderer = new SortButtonRenderer() { // triangular button, this method is overiddeb with an anonymous class to have the headers colored in gray for inactive inputs
      public Component getTableCellRendererComponent(JTable table, Object value,
	  boolean isSelected, boolean hasFocus, int row, int column) {
	Component comp = super.getTableCellRendererComponent(table, value,
	    isSelected, hasFocus, row, column);
	( (JButton) comp).setEnabled(true);
	try {
	  if (JDataTableFrame.this.ShowInactive == true) {
	    int col_mod = table.convertColumnIndexToModel(column);

	    if (col_mod == 0) {
	      ( (JButton) comp).setText(LocaleFIS.GetString("items")); ( (
		  JButton) comp).setToolTipText(null); return comp;
	    }

	    if (col_mod == 1) {
	      ( (JButton) comp).setText(LocaleFIS.GetString("Active")); ( (
		  JButton) comp).setToolTipText(null); return comp;
	    }

	    if ( (col_mod > 1) &&
		(col_mod < JDataTableFrame.this.InputCount + 2)) {
	      if (JDataTableFrame.this.sif.GetInput(col_mod - 2).GetActive() == false) {
		( (JButton) comp).setEnabled(false);
	      }
	    }
	    if (col_mod >= JDataTableFrame.this.InputCount + 2) {
	      if (JDataTableFrame.this.sif.GetOutput(col_mod -
		  JDataTableFrame.this.InputCount - 2).GetActive() == false) {
		( (JButton) comp).setEnabled(false);
	      }
	    }
	  }
	}
	catch (Throwable t) {}
	return comp;
      }
    };

    //Inputs JScrollPane Creation
    rowSM = new DefaultListSelectionModel(); // one selection model for all the tables
    ExamplesTable = new JDataTablePart(this.DataFrameModel,
				       new ExamplesColumnModel(), rowSM); // the "example/active" table

    /* Now, again, we have to consider 2 cases : a fis is open or not.
	 Case no : we will just create one table for all the inputs. Its column model
     is defined by the NoFisInputsColumnModel. We will put this table
     with the "examples table" just created in a JScrollPane.
     Case yes : we will create two others tables. One for the inputs (model defined by
     the InputsColumnModel class) and another for the outputs (see OutputsColumnModel)
     We will put the "examples table" with the "inputs table" in a JScrollPane. We will
	 also put the "outputs table" in a JScrollPane (actually, we put those tables in
	 JPanels before to use layout managers). Those two JScrollPanes are finally put
     in a JSplitPane. */
    //case no : two tables only, in a JScrollPane
    InputsTable = new JDataTablePart(this.DataFrameModel,
				     new InputsColumnModel(), rowSM);

    JPanel InputsPanel = new JPanel(new BorderLayout());
    InputsPanel.add(InputsTable, BorderLayout.CENTER);
    if (this.InputsScrollPane != null) {
      this.getContentPane().remove(this.InputsScrollPane);
    }
    InputsScrollPane = new JScrollPane(InputsPanel);
    InputsScrollPane.setColumnHeaderView(InputsTable.getTableHeader());
    InputsScrollPane.setRowHeaderView(this.ExamplesTable);
    JPanel ExampleHeaderPanel = new JPanel(new BorderLayout());
    ExampleHeaderPanel.add(ExamplesTable.getTableHeader(), BorderLayout.NORTH);
    InputsScrollPane.setCorner(JScrollPane.UPPER_LEFT_CORNER,
			       ExampleHeaderPanel);

    // tri des colonnes de la table
    if (this.sort_col != -1) {
      this.renderer.setPressedColumn(this.sort_col);
      this.renderer.setSelectedColumn(this.sort_col);
      if (this.is_ascent == false) { // fait un appel supplementaire a SortButtonRenderer.setSelectedColumn pour mettre la variable SortButtonRenderer.state dans le bon etat
	this.renderer.setSelectedColumn(this.sort_col);
      }
      this.DataFrameModel.sortByColumn(this.sort_col, this.is_ascent);
      this.renderer.setPressedColumn( -1);
    }

    if (sif == null) {
      SetUpColors();
      this.setTitle(this.extend_data_file.GetFileName());
      this.getContentPane().add(InputsScrollPane);
    }

    else {
      //Outputs JScrollPane Creation
      OutputsTable = new JDataTablePart(this.DataFrameModel,
					new OutputsColumnModel(), rowSM);
      //here, the three tables have been created, we have to set up the color of their columns and their rows
      SetUpColors();
      JPanel OutputsPanel = new JPanel(new BorderLayout());
      OutputsPanel.add(OutputsTable, BorderLayout.CENTER);
      JScrollPane OutputsTableScrollPane = new JScrollPane(OutputsPanel);
      OutputsTableScrollPane.setColumnHeaderView(OutputsTable.getTableHeader());

      // asservi la scrollbar verticale de la table sortie sur la scrollbar verticale de la table entree
      //allows the vertical scrollbar of the outputs table to be controlled by the vertical scrollbar of the inputs table
      JScrollBar inputs_Bar = InputsScrollPane.getVerticalScrollBar();
      JScrollBar outputs_Bar = OutputsTableScrollPane.getVerticalScrollBar();
      OutputsTableScrollPane.setVerticalScrollBarPolicy(JScrollPane.
	  VERTICAL_SCROLLBAR_NEVER); // cache la scrollbar verticale //hides the vertical scrollbar
      inputs_Bar.getModel().addChangeListener(new
					      BoundedRangeModelChangeListener(
	  outputs_Bar.getModel()));

      //Affichage du JSplitPane //JSplitPane display
      if (this.jspData != null) {
	this.getContentPane().remove(this.jspData);
      }
      this.jspData = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true,
				    InputsScrollPane, OutputsTableScrollPane) {
	protected boolean continuousLayout = true;
	public int getDividerSize() {
	  return 2;
	}
      };

      this.setTitle(this.extend_data_file.GetFileName());
      this.jspData.setResizeWeight( (1.0 /
				     (this.InputsTable.getColumnCount() +
				      this.OutputsTable.getColumnCount())) *
				   this.InputsTable.getColumnCount());
      this.getContentPane().add(jspData);
    }
    //right click button popup menu building
    CreateMenuItems();
    this.jPopupRowsSelected = new JPopupMenu();
    this.jPopupRowsSelected.add(RowsSelection);
    this.jPopupRowsSelected.add(RowsNegativeSelection);
    this.jPopupRowsSelected.add(RowsSwitch);
    this.jPopupRowsSelected.add(ActiveAll);
    this.RowsSelection.addMouseListener(new MouseAdapter() {
      public void mousePressed(MouseEvent e) {
	JDataTablePopup_selection_mousePressed(e);
      }
    });
    this.RowsNegativeSelection.addMouseListener(new MouseAdapter() {
      public void mousePressed(MouseEvent e) {
	JDataTablePopup_negativeselection_mousePressed(e);
      }
    });
    this.RowsSwitch.addMouseListener(new MouseAdapter() {
      public void mousePressed(MouseEvent e) {
	JDataTablePopup_switch_mousePressed(e);
      }
    });
    this.ActiveAll.addMouseListener(new MouseAdapter() {
      public void mousePressed(MouseEvent e) {
	JDataTablePopup_activeall_mousePressed(e);
      }
    });


  } //end of jDataTablebInit

//------------------------------------------------------------------------------
  private void SetUpColors() throws Throwable {
    for (int i = 0; i < (ExamplesTable.getColumnModel().getColumnCount()) - 1;
	 i++) { // only the example number column must be colored in light gray
      ExamplesTable.getColumnModel().getColumn(i).setCellRenderer(new
	  DefaultTableCellRenderer() {
	public Component getTableCellRendererComponent(JTable table,
	    Object value, boolean isSelected, boolean hasFocus, int row,
	    int column) {
	  super.getTableCellRendererComponent(table, value, isSelected,
					      hasFocus, row, column);
	  try {
	    // grise les exemples inactifs // Inactive examples are colored in light gray
	    this.setForeground(Color.black);
	    if ( ( (Boolean) table.getModel().getValueAt(row, 1)).booleanValue() == false) {
	      super.setForeground(Color.lightGray);
	    }
	  }
	  catch (Throwable t) {}
	  return this;
	}
      });
    }

    for (int i = 0; i < InputsTable.getColumnModel().getColumnCount(); i++) {
      InputsTable.getColumnModel().getColumn(i).setCellRenderer(new
	  DefaultTableCellRenderer() {
	public Component getTableCellRendererComponent(JTable table,
	    Object value, boolean isSelected, boolean hasFocus, int row,
	    int column) {
	  super.getTableCellRendererComponent(table, value, isSelected,
					      hasFocus, row, column);
	  // grise les colonnes correspondant a des entrees ou sorties inactives //Columns corresponding to inactive inputs are colored in lighyt gray
	  this.setForeground(Color.black);
	  try {
	    if (JDataTableFrame.this.ShowInactive == true) {
	      if (JDataTableFrame.this.sif.GetInput(column).GetActive() == false) {
		this.setForeground(Color.lightGray);
	      }
	    }
	    // grise les exemples inactifs // Inactive examples are colored in light gray
	    if ( ( (Boolean) table.getModel().getValueAt(row, 1)).booleanValue() == false) {
	      super.setForeground(Color.lightGray);
	    }
	  }
	  catch (Throwable t) {}
	  return this;
	}
      });
    }

    if (this.sif != null)
    {
    for (int i = 0; i < OutputsTable.getColumnModel().getColumnCount(); i++) {
      OutputsTable.getColumnModel().getColumn(i).setCellRenderer(new
	  DefaultTableCellRenderer() {
	public Component getTableCellRendererComponent(JTable table,
	    Object value, boolean isSelected, boolean hasFocus, int row,
	    int column) {
	  super.getTableCellRendererComponent(table, value, isSelected,
					      hasFocus, row, column);
	  // grise les colonnes correspondant a des entrees ou sorties inactives //Columns corresponding to inactive inputs are colored in lighyt gray
	  this.setForeground(Color.black);
	  try {
	    if (JDataTableFrame.this.ShowInactive == true) {
	      if (JDataTableFrame.this.sif.GetOutput(column).GetActive() == false) {
		this.setForeground(Color.lightGray);
	      }
	    }
	    // grise les exemples inactifs // Inactive examples are colored in light gray
	    if ( ( (Boolean) table.getModel().getValueAt(row, 1)).booleanValue() == false) {
	      super.setForeground(Color.lightGray);
	    }
	  }
	  catch (Throwable t) {}
	  return this;
	}
      });
    }
   }
  }

  private void ColorExamples (Component comp, JTable table, int row)
	{
	comp.setForeground(Color.black);
	 if ( ( (Boolean) table.getModel().getValueAt(row, 1)).booleanValue() == false) {
	super.setForeground(Color.lightGray);}
	}

//------------------------------------------------------------------------------

  class HeaderListenerBak
      extends MouseAdapter {
    JTableHeader header;
    SortButtonRenderer renderer;

    HeaderListenerBak(JTableHeader header, SortButtonRenderer renderer) {
      this.header = header;
      this.renderer = renderer;
    }

    public void mousePressed(MouseEvent e) {
      int col = header.columnAtPoint(e.getPoint());
      int sortCol = header.getTable().convertColumnIndexToModel(col);
      renderer.setPressedColumn(sortCol);
      renderer.setSelectedColumn(sortCol);
      for (int i = 0; i < JDataTableFrame.this.tables_headers.size(); i++) {
	( (JTableHeader) JDataTableFrame.this.tables_headers.elementAt(i)).
	    repaint();
      }

      if (header.getTable().isEditing()) {
	header.getTable().getCellEditor().stopCellEditing();
      }

      boolean isAscent;
      if (SortButtonRenderer.DOWN == renderer.getState(col)) {
	isAscent = true;
      }
      else {
	isAscent = false;
      }

      ( (SortableTableModel) header.getTable().getModel())
	  .sortByColumn(sortCol, isAscent);
    }

    public void mouseReleased(MouseEvent e) {
      int col = header.columnAtPoint(e.getPoint());
      renderer.setPressedColumn( -1); // clear
      for (int i = 0; i < JDataTableFrame.this.tables_headers.size(); i++) {
	( (JTableHeader) JDataTableFrame.this.tables_headers.elementAt(i)).
	    repaint();
      }
    }
  }

//------------------------------------------------------------------------------
  /**
   * Column model for the inputs table
   */
  class InputsColumnModel
      extends DefaultTableColumnModel {
    /**
     *  Overriding of DefaultTableColumnModel.addColumn to display the inputs columns only
     * */
    public void addColumn(TableColumn tc) {
      int index = tc.getModelIndex();
      if (sif == null) {
	if (index > 1) {
	  super.addColumn(tc);
	}
      }
      else {
	if (ShowInactive == true) {
	  if ( (index > 1) && (index < JDataTableFrame.this.InputCount + 2)) {
	    super.addColumn(tc);
	  }
	}
	else {
	  if ( (index > 1) && (index < JDataTableFrame.this.InputCount + 2)) {
	    try {
	      JInput CurrentInput = sif.GetInput(index - 2);
	      if (CurrentInput.GetActive() == true) {
		super.addColumn(tc);
	      }
	    }
	    catch (Throwable t) {
	      MessageFIS.Error(null, t);
	    }
	    ;
	  }
	}
      }
    }
  }

  //------------------------------------------------------------------------------
  /**
   * Column model for the outputs table
   */
  class OutputsColumnModel
      extends DefaultTableColumnModel {
    /**
     *  Overriding of DefaultTableColumnModel.addColumn to display the outputs columns only
     * */
    public void addColumn(TableColumn tc) {
      int index = tc.getModelIndex();

      if (ShowInactive == true) {
	if (index >= JDataTableFrame.this.InputCount + 2) {
	  super.addColumn(tc);
	}
      }
      else {
	if (index >= JDataTableFrame.this.InputCount + 2) {
	  try {
	    JOutput CurrentOutput = sif.GetOutput(index -
						  JDataTableFrame.this.InputCount -
						  2);
	    if (CurrentOutput.GetActive() == true) {
	      super.addColumn(tc);
	    }
	  }
	  catch (Throwable t) {
	    MessageFIS.Error(null, t);
	  }
	  ;
	}
      }
    }
  }

//------------------------------------------------------------------------------

   /**
    *  Column model for the examples table
    */
   class ExamplesColumnModel
       extends DefaultTableColumnModel {
     /**
      *  Overriding of DefaultTableColumnModel.addColumn to display the examples + active columns only
      * */
     public void addColumn(TableColumn tc) {
       int index = tc.getModelIndex();
       if (index <= 1) {
	 super.addColumn(tc);
       }
     }
   }

//------------------------------------------------------------------------------

  /**
   * Table class used for the different tables displayed in the frame
   */
  private class JDataTablePart
      extends JTable {
    public JDataTablePart(SortableTableModel dm, TableColumnModel cm,
			  ListSelectionModel rsm) {
      super(dm, cm);
      this.createDefaultColumnsFromModel(); // construit la table //building of the table
      this.setSelectionModel(rsm);
      this.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
      this.setColumnSelectionAllowed(false);
      this.SetSortableHeaders();

      this.addMouseListener(new MouseAdapter() {
	public void mousePressed(MouseEvent e) {
	  JDataTablePart_mousePressed(e);
	}
	public void mouseReleased(MouseEvent e) {
          JDataTablePart_mouseReleased(e);
        }

	public void mouseClicked (MouseEvent e)
	{
	  if ((e.getModifiers() & InputEvent.BUTTON1_MASK) != 0 && e.getClickCount() == 2)
	    {
	      Point p = e.getPoint();
	      int row = rowAtPoint(p);
	      int column = columnAtPoint(p); // This is the view column! but it corresponds to the model's one
	      if (column == 1 && getColumnModel() instanceof ExamplesColumnModel)
		    return ; //won't work on the checkboxes column

	      int clickedExample = ConvertIndiceToExample(row);
	      double[] row_data = JDataTableFrame.this.extend_data_file.GetData()[clickedExample-1] ;
	      try
	      {
          if( sif != null ) {
		    parent.showInferenceFrame() ;
	         //we set the inference frame's sliders values one by one. Setting the sliders double value is sufficient to have the correct inference
	         for (int i=0 ; i<sif.NbInputs() ; i++)  //
		       parent.setInferenceFrameInputValue(i, row_data[i]) ;
          }

	      } catch (Throwable t) {t.printStackTrace();}

	    }
	}
      });

    } //end of constructor

    public void setValueAt(Object data, int row, int column)
    { //to manage the activation of one example by clicking on the table and only in this case
        int example_at_i = ( (Integer) JDataTableFrame.this.ExamplesTable.getModel().getValueAt(row, 0)).intValue();
	JDataTableFrame.this.extend_data_file.SetActive(example_at_i, !(( (Boolean)this.getValueAt(row, 1)).booleanValue())) ;
    }

//wil be used by this.pack to set the cells dimensions
    public Dimension getPreferredScrollableViewportSize() {
      Dimension size = super.getPreferredScrollableViewportSize();
      return new Dimension(Math.min(getPreferredSize().width, size.width),
			   size.height);
    }

//for headers tooltips : overriding of createDefaultTableHeader method
    protected JTableHeader createDefaultTableHeader() {
      return new JTableHeader(columnModel) {
	public String getToolTipText(MouseEvent e) {
	  String tip = null;
	  java.awt.Point p = e.getPoint();
	  int index = columnModel.getColumnIndexAtX(p.x);
	  int realIndex =
	      columnModel.getColumn(index).getModelIndex();
	  return (String) JDataTableFrame.this.VectorName[realIndex];
	}
      };

    }

    // SortButtonRenderer renderer = new SortButtonRenderer();
    private void SetSortableHeaders() {
      TableColumnModel model = JDataTablePart.this.getColumnModel();
      for (int i = 0; i < model.getColumnCount(); i++) {
	model.getColumn(i).setHeaderRenderer(JDataTableFrame.this.renderer);
      }
      this.getTableHeader().setReorderingAllowed(false);
      this.getTableHeader().setResizingAllowed(false);
      JTableHeader header = JDataTablePart.this.getTableHeader();
      header.addMouseListener(new SortableHeaderListener(header,
	  JDataTableFrame.this.renderer, JDataTableFrame.this));
      JDataTableFrame.this.tables_headers.add(header); // ajout du header dans le vecteurs des headers

    }

   /*
    Mouse events on the table
   */
    private void JDataTablePart_mousePressed(MouseEvent e) {
      if (SwingUtilities.isRightMouseButton(e)) {
	JDataTableFrame.this.jPopupRowsSelected.setEnabled(true);
	if (rowSM.getMinSelectionIndex() == -1) {
	  EnablePopupItems(false);
	}

	SwingUtilities.updateComponentTreeUI(JDataTableFrame.this.
					     jPopupRowsSelected); // affecte le look actuel au composant jPopupMenuInputs
	jPopupRowsSelected.show(this, e.getX(), e.getY()); // affichage du popup
      }

      if (SwingUtilities.isLeftMouseButton(e)) {
	if (rowSM.getMinSelectionIndex() != -1) { //if one line is selected, the popup items can be enabled
	  EnablePopupItems(true);
	}
      }
    }

    private void JDataTablePart_mouseReleased(MouseEvent e) {   //disable switch when all rows are selected
      MenuElement[] me = jPopupRowsSelected.getSubElements();
      int[] tmp = GetSelectionArray() ;
      if (tmp.length == JDataTableFrame.this.vlenght || rowSM.getMinSelectionIndex() == -1)
            me[2].getComponent().setEnabled(false);
      else
	    me[2].getComponent().setEnabled(true);
    }

  }; //end of JDataTablePart nested private class
//------------------------------------------------------------------------------
  //action fired by the JCheckBoxMenuItem button : display or not the inactive inputs. If they are shown, they will appear in light gray
  void jMenuDisplayInactive_actionPerformed(ActionEvent e) {
    this.ShowInactive = this.jMenuDisplayInactive.isSelected();
    this.ReInit();
  }

//------------------------------------------------------------------------------
  /*
   Some functions for Popup menu handling
   */

  private void EnablePopupItems(boolean b_enable) {
    MenuElement[] me = jPopupRowsSelected.getSubElements();
    for (int i = 0; i < me.length - 1; i++) { //active all and active nothing stay enabled
      me[i].getComponent().setEnabled(b_enable);
    }
  }

  private void JDataTablePopup_selection_mousePressed(MouseEvent e) {
    if (SwingUtilities.isLeftMouseButton(e)) {
      DoSelection(true);
    }
  }

  private void JDataTablePopup_negativeselection_mousePressed(MouseEvent e) {
    if (SwingUtilities.isLeftMouseButton(e)) {
      DoSelection(false);
    }
  }

  private void DoSelection(boolean bvalue)
  {
    int[] SelectedIndices = this.GetSelectionArray();
    int[] SelectedItems = ConvertIndicesToExamples(SelectedIndices);
   this.extend_data_file.SetActive(SelectedItems, bvalue) ;

}


  private void JDataTablePopup_switch_mousePressed(MouseEvent e) {
    if (SwingUtilities.isLeftMouseButton(e)) {
      DoSwitch();
    }
  }

  private void JDataTablePopup_inverseswitch_mousePressed(MouseEvent e) {
    if (SwingUtilities.isLeftMouseButton(e)) { // le bouton gauche a ete presse
      DoSwitch();
    }
  }

  private boolean stat_bvalue = false;
  private void DoSwitch() {

    int[] SelectedIndices = this.GetSelectionArray();
    int[] SelectedItems = ConvertIndicesToExamples(SelectedIndices);
    boolean[] SwitchValues = new boolean[SelectedIndices.length] ;
    this.extend_data_file.GlobalSetActive(stat_bvalue, true);
    for (int i = 0; i < this.vlenght ; i++)
          this.DataFrameModel.setValueAt(new Boolean(stat_bvalue), i, 1);
    this.extend_data_file.SetActive(SelectedItems, !stat_bvalue);

    stat_bvalue = !stat_bvalue; //to inverse the next switch

  }

  private void JDataTablePopup_activeall_mousePressed(MouseEvent e) {
    if (SwingUtilities.isLeftMouseButton(e)) {
      DoGlobalSelection(true);
      this.rowSM.clearSelection();
      stat_bvalue = true; //to ensure that DoSwitch will work correctly : if we active all the examples and then we do a switch, the result must be : no examples activated. This is avoided directly in the menu
    }
  }

  private void DoGlobalSelection(boolean bvalue) {
        this.extend_data_file.GlobalSetActive(bvalue, false);

  }

 //this function doen't exist in the row selection model class. So we create it.
  protected int[] GetSelectionArray() {
    Vector SelectionVector = new Vector();
    int i;
    for (i = this.rowSM.getMinSelectionIndex();
	 i <= (this.rowSM.getMaxSelectionIndex()); i++) {
      if (this.rowSM.isSelectedIndex(i)) {
	SelectionVector.add(new Integer(i));
      }
    }
    int lenght = SelectionVector.size();
    int[] SelectionArray = new int[lenght];
    for (i = 0; i < lenght; i++) {
      Integer value = (Integer) SelectionVector.get(i);
      SelectionArray[i] = value.intValue();
    }
    return SelectionArray;
  }

//necessary function, because rows can be sorted
private int[] ConvertIndicesToExamples (int[] SelectedIndices)
  {
   int[] SelectedItems = new int[SelectedIndices.length] ;
   int example_at_i ;
   for (int i=0 ; i< SelectedItems.length ; i++)
     {
     example_at_i = ( (Integer) JDataTableFrame.this.ExamplesTable.getModel().getValueAt(SelectedIndices[i], 0)).intValue();
     SelectedItems[i] = example_at_i ;
     }
   return SelectedItems ;
  }

//necessary function, because rows can be sorted
private int ConvertIndiceToExample (int selectedIndice)
  {
   int example_at_i ;
   example_at_i = ( (Integer) JDataTableFrame.this.ExamplesTable.getModel().getValueAt(selectedIndice, 0)).intValue();
   return example_at_i ;
  }

//necessary function, because rows can be sorted
private int ConvertExampleToIndex (int example)
  {
   int index ;
   int example_at_index ;
   for (index=0 ; index<this.vlenght ; index++)
     {
     example_at_index = ( (Integer) JDataTableFrame.this.ExamplesTable.getModel().getValueAt(index, 0)).intValue();
     if (example_at_index == example)
         break ;
     }
    return index ;
  }

//necessary function, because rows can be sorted
 private int[] ConvertExamplesToIndices (int[] examples)
  {
  int[] indices = new int [examples.length] ;
  int example_at_index ;
  for (int i=0 ; i <examples.length ; i++)
     {
     int j ;
     for (j=0 ; j<this.vlenght ; j++)
	 {
	 example_at_index = ( (Integer) JDataTableFrame.this.ExamplesTable.getModel().getValueAt(j, 0)).intValue();
	 if (example_at_index == examples[i])
	     break ;
	 }
     indices[i] = j ;
     }
  return indices ;
  }
//------------------------------------------------------------------------------
  /**
   * ReInit of the frame. Called by the fis listener JDataTableFrame#fis_listener to reinit and display the frame after the FIS have been modified
   */
  private void ReInit() {
    try {
      int divider_location = 0;
      if (this.jspData != null) {
	divider_location = this.jspData.getDividerLocation();
      }
      this.jDataTableInit();
      if (this.jspData != null) {
	this.jspData.setDividerLocation(divider_location);
      }
      this.setVisible(true);
    }
    catch (Throwable t) {
      MessageFIS.Error(null, t);
    }
  }

//------------------------------------------------------------------------------
  public void Translate() throws Throwable {}

//------------------------------------------------------------------------------

  private void CreateMenuItems() {
    RowsSelection = new JMenuItem(new String(LocaleFIS.GetString("Activate")));
    RowsNegativeSelection = new JMenuItem(new String(LocaleFIS.GetString(
	"Desactivate")));
    RowsSwitch = new JMenuItem(new String(LocaleFIS.GetString("Switch")));
    ActiveAll = new JMenuItem(new String(LocaleFIS.GetString("ActivateAll")));
    ActiveNothing = new JMenuItem(new String(LocaleFIS.GetString(
	"ActivateNothing")));

  }

//------------------------------------------------------------------------------
  public void SetSortedColumn(int column) {
    this.sort_col = column;
  }

  public int GetSortedColumn() {
    return this.sort_col;
  }

  public void SetSortAscent(boolean is_ascent) {
    this.is_ascent = is_ascent;
  }

  public Vector GetTablesHeaders() {
    return this.tables_headers;
  }

  public Dimension getMinimumSize() { return  new Dimension(300,200); }
//------------------------------------------------------------------------------
  public void setSize(Dimension d)
    {
    super.setSize(d);
    if( (this.jspData != null) && (this.jspData.getDividerLocation() > d.width) )
      this.jspData.setDividerLocation(d.width-100);
    }
} // end of JDataFrameTable class

