//***********************************************************************
//
//                              JDoubleSlider.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;


/**
 * Classe curseur\n
 * surcharge de la classe JSlider pour gestion de la position du curseur en double. la classe JSlider gere la position du curseur en int.
 */
 public class JDoubleSlider extends JSlider
  {
  final int resolution = 100; /**< resolution du curseur */
  protected double value = 0; /**< valeur du curseur */
  protected double min = 0; /**< valeur minimum du curseur */
  protected double max = this.resolution; /**< valeur maximum du curseur */
  protected boolean manual_adjust = false; /**< ajustement manuel du curseur */
//------------------------------------------------------------------------------
  public JDoubleSlider()
    {
    // ajuste les limites du curseur
    this.setMinimum(0);
    this.setMaximum(this.resolution);
    // ajoute un listener pour detecter un ajustement manuel du curseur
    this.addChangeListener(new ChangeListener()
      {
      public void stateChanged(ChangeEvent e)
	{
	JDoubleSlider.this.manual_adjust = JDoubleSlider.this.getValueIsAdjusting();
	JDoubleSlider.this.value = JDoubleSlider.this.min + (((double)JDoubleSlider.this.getValue() / (double)JDoubleSlider.this.resolution) * (JDoubleSlider.this.max - JDoubleSlider.this.min));
	} } );
    }
//------------------------------------------------------------------------------
    /**
     * Initialise les valeurs (en double) minimum et maximum du curseur
     * @param min valeur minimum
     * @param max valeur maximum
     */
  public void setDoubleMinMax( double min, double max ) { this.min = min; this.max = max; }
//------------------------------------------------------------------------------
  /**
   * Retourne la valeur double du curseur
   * @return la valeur du curseur
   */
  public double getDoubleValue()
    {
    // si le curseur n'a pas ete ajuste manuellement: renvoit this.value, sinon recalcule la valeur du curseur en fonction des limites et de la resolution
    return this.manual_adjust == false ? this.value : this.min + (((double)super.getValue() / (double)this.resolution) * (this.max - this.min));
    }
//------------------------------------------------------------------------------
    /**
     * Fixe la valeur double du curseur
     * @param value nouvelle valeur du curseur
     */
  public void setDoubleValue( double value )
    {
    this.value = value;
    super.setValue((int)(((value - this.min)/(this.max - this.min))*this.resolution));
    super.fireStateChanged(); // force une changement d'etat pour reaffichage du curseur
    }
  }

