//***********************************************************************
//
//                              JFIS.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import static fis.MainFIS.config;
import static org.apache.commons.io.FilenameUtils.concat;
import static org.apache.commons.lang.StringUtils.EMPTY;

import java.util.Vector;

import fis.optimization.crossvalidation.CrossValidation;

//------------------------------------------------------------------------------
public class JFIS
{
	private long ptr;
	String FISFile = null;
	private Vector InputsSemaphore = new Vector();
	private Vector OutputsSemaphore = new Vector();
	private long CopyFISPtr = 0;
	private Vector Listeners = new Vector();

	//------------------------------------------------------------------------------
	public JFIS() throws Throwable
	{
		this.ptr = jnifis.NewFIS();
		this.SetName(LocaleFIS.GetString("NewFIS"));
		this.SetConjunction(1);
		this.CopyFISPtr = jnifis.NewFIS(this.ptr);
	}
	//------------------------------------------------------------------------------
	public JFIS( String FISFile, boolean cover ) throws Throwable
	{
		this.ptr = jnifis.NewFIS(FISFile, cover);
		this.FISFile = FISFile;
		for( int i=0 ; i<this.NbInputs() ; i++ )
			this.InputsSemaphore.add(new JSemaphore());
		for( int i=0 ; i<this.NbOutputs() ; i++ )
			this.OutputsSemaphore.add(new JSemaphore());
		this.CopyFISPtr = jnifis.NewFIS(this.ptr);
	}
	//------------------------------------------------------------------------------
	public JFIS( JFIS fis ) throws Throwable
	{
		this.ptr = jnifis.NewFIS(fis.ptr);
		for( int i=0 ; i<this.NbInputs() ; i++ )
			this.InputsSemaphore.add(new JSemaphore());
		for( int i=0 ; i<this.NbOutputs() ; i++ )
			this.OutputsSemaphore.add(new JSemaphore());
		this.CopyFISPtr = jnifis.NewFIS(this.ptr);
	}
	//------------------------------------------------------------------------------
	protected JFIS( long ptr ) throws Throwable
	{
		this.ptr = ptr;
		for( int i=0 ; i<this.NbInputs() ; i++ )
			this.InputsSemaphore.add(new JSemaphore());
		for( int i=0 ; i<this.NbOutputs() ; i++ )
		{
			this.OutputsSemaphore.add(new JSemaphore());
			if( this.GetOutput(i).GetName().equals("") == true )
				this.GetOutput(i).SetName(LocaleFIS.GetString("Output") + " " + Integer.toString(i+1) );
		}
	}
	//------------------------------------------------------------------------------
	public long getPtr() {
		return this.ptr;
	}
	//------------------------------------------------------------------------------
	protected long NewFISFPA( JExtendedDataFile data, int strategy, int min_card, double min_deg ) throws Throwable
	{
		return jnifis.NewFISFPA(this.ptr, data.ActiveFileName(), strategy, min_card, min_deg );
	}
	//------------------------------------------------------------------------------
	protected long NewGENFIS( JExtendedDataFile data, boolean sort, double thres ) throws Throwable
	{
		String data_file = null;
		if( data != null )
			data_file = data.ActiveFileName();
		return jnifis.NewGENFIS(this.ptr, data_file, sort, thres );
	}
	//------------------------------------------------------------------------------
	protected JFIS NewFISSIMPLE( JExtendedDataFile data, String resultFile, double homogeneity_threshold, double loss_performance, int output_number, double min_matching_degree, double loss_coverage, double coverage_min, boolean keep_last, double max_err, boolean ruleRemoval, boolean variableRemoval, String validationFile) throws Throwable
	{
		return new JFIS(jnifis.NewFISSIMPLE(this.ptr, data.ActiveFileName(), resultFile, homogeneity_threshold, loss_performance, output_number, min_matching_degree, loss_coverage, coverage_min, keep_last, max_err, ruleRemoval, variableRemoval, validationFile));
	}
	//------------------------------------------------------------------------------
	public static JFIS NewFISOLS(JFIS fis, JExtendedDataFile data, int outputNumber, boolean keepExistingRules, double standardDeviation, String conjunction, double totalUnexplainedVariance, double maximumNumberOfRules, boolean reduceVocabulary, double degradationAllowed, double numberOfConc, boolean useRuleConc, boolean outputFuzzification, boolean removeTemporaryFiles, String testDataFile) throws Throwable
	{
		return new JFIS(jnifis.NewFISOLS(fis == null ? 0 : fis.getPtr(), data.ActiveFileName(), outputNumber, keepExistingRules, standardDeviation, conjunction, totalUnexplainedVariance, maximumNumberOfRules, reduceVocabulary, degradationAllowed, numberOfConc, useRuleConc, outputFuzzification, removeTemporaryFiles, testDataFile));
	}
	//------------------------------------------------------------------------------
	public void outputVocabularyReduction(JExtendedDataFile  data, int outputNumber, double blankThreshold, double degradationAllowed, double numberOfConc, boolean useRuleConc) throws Throwable {
		jnifis.outputVocabularyReduction(this.ptr, data.ActiveFileName(), outputNumber, blankThreshold, degradationAllowed, numberOfConc, useRuleConc );
	}
	//------------------------------------------------------------------------------
	protected JFIS NewCustomFISOPT( JExtendedDataFile data, String key, int iterations, double std_gauss, int constraints, int fails, double center_thres, int output_number, boolean without_redondancy, int seed_value, double sw1, double sw2, double sw3, int cardinal_vocabulary, double mu_thres, double loss_coverage ) throws Throwable
	{
		long new_fis = jnifis.NewCustomFISOPT(this.ptr, data.ActiveFileName(), key, iterations, std_gauss, constraints, fails, center_thres, output_number, without_redondancy, seed_value, sw1, sw2, sw3, cardinal_vocabulary, mu_thres, loss_coverage);
		return new_fis == 0 ? null : new JFIS(new_fis);
	}
	protected JFIS NewFISOPT( JExtendedDataFile data, String optimization_order, boolean output, boolean rules, int number_of_loops, double loss_coverage, int iterations, double std_gauss, int constraints, int fails, int output_number, int seed_value, double sw1, double sw2, double sw3, double mu_thres, CrossValidation crossValidation, double minBetweenMF) throws Throwable
	{
		long new_fis = jnifis.NewFISOPT(this.ptr, concat(config.FISFilePath(), EMPTY), data.ActiveFileName(), optimization_order, output, rules, number_of_loops, loss_coverage, iterations, std_gauss, constraints, fails, output_number, seed_value, sw1, sw2, sw3, mu_thres, crossValidation.isEnabled(), crossValidation.getPairSample().getLearningRatio(), crossValidation.getClassifSample().isClassif(), crossValidation.getClassifSample().getPrecision(), crossValidation.getClassifSample().getClassifVariable(), crossValidation.isEnabled() ? crossValidation.getPairSample().getPairNumber() : 0, minBetweenMF);
		return new_fis == 0 ? null : new JFIS(new_fis);
	}
	//------------------------------------------------------------------------------
	protected JFIS [] NewTreeFIS( JExtendedDataFile data, String tree_file,  String valid_file, String resultFistreeFile, String perfFistreeFile, int maxtreedepth, double min_ed_gain, double min_significant, int min_leaf_card, double ambiguity_thres, int outputNumber, boolean prune, boolean split, double relperfloss, boolean relgain, boolean display, double covThresh ) throws Throwable
	{
		String data_file = null;
		if( data != null )
			data_file = data.ActiveFileName();
		JFIS [] result = new JFIS[2];
		result[0] = result[1] = null;
		long [] long_fis = jnifis.NewTreeFIS(this.ptr, data_file, tree_file, valid_file, resultFistreeFile, perfFistreeFile, maxtreedepth, min_ed_gain, min_significant, ambiguity_thres,outputNumber,
				prune, split,relperfloss,relgain, display, covThresh, min_leaf_card);
		if( long_fis[0] != 0 )
			result[0] = new JFIS(long_fis[0]);
		if( long_fis[1] != 0 )
			result[1] = new JFIS(long_fis[1]);
		return result;
	}
	//------------------------------------------------------------------------------
	public void Links( JExtendedDataFile data, double threshold, boolean sort, String file_prefix, LinksItemsRules items_rules ) throws Throwable
	{
		jnifis.Links(this.ptr, data.ActiveFileName(), threshold, sort, file_prefix);
		if( items_rules != null )
			items_rules.LoadFile(file_prefix+"."+jnifis.LinksItemsRulesExtension());
	}
	//------------------------------------------------------------------------------

	public void Delete()
	{
		try {
			jnifis.DeleteFIS( this.ptr );
			this.ptr = 0;
			jnifis.DeleteFIS( this.CopyFISPtr );
			this.CopyFISPtr = 0; }
		catch ( Throwable t ) {}
	}
	//------------------------------------------------------------------------------
	public void Save( String FISFile ) throws Throwable
	{
		jnifis.SaveFIS( ptr, FISFile );
		this.FISFile = FISFile;
		this.CopyFISPtr = jnifis.NewFIS(this.ptr);
	}
	//------------------------------------------------------------------------------
	public void Save() throws Throwable { this.Save(this.FISFile); }
	//------------------------------------------------------------------------------
	public void SetConjunction( int index ) throws Throwable
	{
		jnifis.SetConjunction(ptr, TypeConjunction()[index]);
		// dispatch l'evenement ConjunctionChanged
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).ConjunctionChanged();

	}
	//------------------------------------------------------------------------------
	public void SetName( String name ) throws Throwable { jnifis.SetFISName(ptr, name); }
	//------------------------------------------------------------------------------
	public String GetName() throws Throwable { return jnifis.GetFISName(ptr); }
	//------------------------------------------------------------------------------
	public int GetConjunction() throws Throwable
	{
		String conj = jnifis.GetConjunction(ptr);
		String [] type_conjs = JFIS.TypeConjunction();
		for( int i=0 ; i<type_conjs.length ; i++ )
			if( conj.equals(type_conjs[i]) == true )
				return i;
		throw new Exception(LocaleFIS.GetString("UnknownConjunction") + ": " + conj);
	}
	//------------------------------------------------------------------------------
	static public String [] TypeConjunction() throws Throwable { return jnifis.TypeConjunction(); }
	//------------------------------------------------------------------------------
	static public String [] TypeMissingValues() throws Throwable { return jnifis.TypeMissingValues(); }
	//------------------------------------------------------------------------------
	public boolean Modified() throws Throwable
	{
		if( this.CopyFISPtr == 0 )
			return true;
		return !jnifis.EqualFIS(this.ptr, this.CopyFISPtr);
	}
	//------------------------------------------------------------------------------
	public int NbInputs() throws Throwable { return jnifis.NbInputs( ptr ); }
	//------------------------------------------------------------------------------
	public int NbActiveInputs() throws Throwable
	{
		int nb_active = 0;
		for( int i=0 ; i<this.NbInputs() ; i++ )
			if( this.GetInput(i).GetActive() == true )
				nb_active++;
		return nb_active;
	}
	//------------------------------------------------------------------------------
	public int NbOutputs() throws Throwable { return jnifis.NbOutputs( ptr ); }
	//------------------------------------------------------------------------------
	public int NbActiveOutputs() throws Throwable
	{
		int nb_active = 0;
		for( int i=0 ; i<this.NbOutputs() ; i++ )
			if( this.GetOutput(i).GetActive() == true )
				nb_active++;
		return nb_active;
	}
	//------------------------------------------------------------------------------
	public int NbRules() throws Throwable { return jnifis.NbRules( ptr ); }
	//------------------------------------------------------------------------------
	public int NbActiveRules() throws Throwable
	{
		int nb_active = 0;
		for( int i=0 ; i<this.NbRules() ; i++ )
			if( this.GetRule(i).GetActive() == true )
				nb_active++;
		return nb_active;
	}
	//------------------------------------------------------------------------------
	public JInput GetInput( int input_number ) throws Throwable { return new JInput( jnifis.GetInput( ptr, input_number ) ); }
	//------------------------------------------------------------------------------
	public JInput GetActiveInput( int active_number ) throws Throwable
	{
		for( int i=0, active=0 ; i<this.NbInputs() ; i++ )
		{
			JInput input = this.GetInput(i);
			if( input.GetActive() == true )
			{
				if( active == active_number )
					return input;
				else
					active++;
			}
		}
		return null;
	}
	//------------------------------------------------------------------------------
	public JInput []GetInputs() throws Throwable {
		JInput []inputs = new JInput[this.NbInputs()];  
		for( int i=0 ; i<this.NbInputs() ; i++ )
			inputs[i] = this.GetInput(i);
		return inputs;
	}
	//------------------------------------------------------------------------------
	public JOutput []GetOutputs() throws Throwable {
		JOutput []outputs = new JOutput[this.NbOutputs()];  
		for( int i=0 ; i<this.NbOutputs() ; i++ )
			outputs[i] = this.GetOutput(i);
		return outputs;
	}
	//------------------------------------------------------------------------------
	public JOutput GetOutput( int output_number ) throws Throwable { return new JOutput( jnifis.GetOutput( ptr, output_number ) ); }
	//------------------------------------------------------------------------------
	public JOutput GetActiveOutput( int active_number ) throws Throwable
	{
		for( int i=0, active=0 ; i<this.NbOutputs() ; i++ )
		{
			JOutput output = this.GetOutput(i);
			if( output.GetActive() == true )
			{
				if( active == active_number )
					return output;
				else
					active++;
			}
		}
		return null;
	}
	//------------------------------------------------------------------------------
	public int GetBuildFuzIn(int input_number, double [] kw, double [] sw, double [] inputs_values, Vector<Double> fuzval ) throws Throwable { return jnifis.GetBuildFuzIn(this.ptr, input_number, kw, sw, inputs_values, fuzval); }  
	//------------------------------------------------------------------------------
	public JRule GetRule( int rule_number ) throws Throwable { return new JRule( jnifis.GetRule( ptr, rule_number ) ); }
	//------------------------------------------------------------------------------
	public void AddInput( JInput input ) throws Throwable
	{
		jnifis.AddInput( ptr, input.Ptr() );
		this.InputsSemaphore.add(new JSemaphore());
		// dispatch l'evenement InputAdded
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).InputAdded(this.NbInputs()-1);
	}
	//------------------------------------------------------------------------------
	public void AddOutput( JOutput output ) throws Throwable
	{
		jnifis.AddOutput( ptr, output.Ptr() );
		this.OutputsSemaphore.add(new JSemaphore());
		// dispatch l'evenement OutputAdded
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).OutputAdded(this.NbOutputs()-1);
	}
	//------------------------------------------------------------------------------
	public void RemoveInput( int input_number ) throws Throwable
	{
		this.InputsSemaphore.removeElementAt(input_number);
		jnifis.RemoveInput( this.ptr, input_number );
		// dispatch l'evenement InputRemoved
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).InputRemoved(input_number);
	}
	//------------------------------------------------------------------------------
	public void ReplaceInput( int input_number, JInput new_input ) throws Throwable
	{
		jnifis.ReplaceInput( ptr, input_number, new_input.Ptr() );
		// dispatch l'evenement InputChanged
		// dans la structure actuelle du programme une entree d'un sif est modifie lors de l'appel a ReplaceInput
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).InputReplaced(input_number);
	}
	//------------------------------------------------------------------------------
	public void RemoveOutput( int output_number ) throws Throwable
	{
		this.OutputsSemaphore.removeElementAt(output_number);
		jnifis.RemoveOutput( this.ptr, output_number );
		// dispatch l'evenement OutputRemoved
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).OutputRemoved(output_number);
	}
	//------------------------------------------------------------------------------
	public void ReplaceOutput( int output_number, JOutput new_output ) throws Throwable
	{
		jnifis.ReplaceOutput( ptr, output_number, new_output.Ptr() );
		// dispatch l'evenement OutputChanged
		// dans la structure actuelle du programme une sortie d'un sif est modifie lors de l'appel a ReplaceOutput
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).OutputReplaced(output_number);
	}
	//------------------------------------------------------------------------------
	public JRule NewRule( boolean active, int [] facteurs, double [] actions )  throws Throwable { return new JRule( jnifis.NewRule(this.ptr, active, facteurs, actions) ); }
	//------------------------------------------------------------------------------
	public void AddRule( JRule rule ) throws Throwable
	{
		jnifis.AddRule( ptr, rule.Ptr() );
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).RulesModified();
	}
	//------------------------------------------------------------------------------
	public void RemoveRule( int rule_number ) throws Throwable
	{
		jnifis.RemoveRule( ptr, rule_number );
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).RulesModified();
	}
	//------------------------------------------------------------------------------
	public void Infer( double [] values ) throws Throwable { jnifis.Infer(ptr, values); }
	//------------------------------------------------------------------------------
	public void InferFuzzyInput(double [] kw, double [] sw, int nbAlphaCut, double [] values ) throws Throwable
	{ jnifis.InferFuzzyInput(ptr,kw, sw, nbAlphaCut, values); }  
	//------------------------------------------------------------------------------
	public Object[] Infer3DSurface (SurfacePlotParameters Parameters) throws Throwable { return jnifis.Infer3DSurface(ptr, Parameters); }
	//------------------------------------------------------------------------------
	public double[] getRulesWeight (double [] input_values) throws Throwable { return jnifis.GetRulesWeight(ptr, input_values);}
	//------------------------------------------------------------------------------
	public double[][] Infer2DSurface (SurfacePlotParameters Parameters) throws Throwable { return jnifis.Infer2DSurface(ptr, Parameters); }
	//------------------------------------------------------------------------------
	public double [] Infer( int output_number, String data_file, String result_file, double blank_thres, boolean display ) throws Throwable
	{ return jnifis.DataFileInfer(ptr, output_number, data_file, result_file, blank_thres, display); }
	//------------------------------------------------------------------------------
	public double [] InferErrorRegression() throws Throwable { return jnifis.InferErrorRegression(ptr); }
	//------------------------------------------------------------------------------
	public double [] SortiesObtenues() throws Throwable { return jnifis.SortiesObtenues(ptr); }
	//------------------------------------------------------------------------------
	public JSemaphore GetInputSemaphore( int input_number ) { return (JSemaphore)this.InputsSemaphore.elementAt(input_number); }
	//------------------------------------------------------------------------------
	public JSemaphore GetOutputSemaphore( int output_number ) { return (JSemaphore)this.OutputsSemaphore.elementAt(output_number); }
	//------------------------------------------------------------------------------
	public void AddFISListener( FISListener listener ) { this.Listeners.add(listener); }
	//------------------------------------------------------------------------------
	public void RemoveFISListener( FISListener listener ) { this.Listeners.remove(listener); }
	//------------------------------------------------------------------------------
	public void Close()
	{
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).FISClosed();
	}
	//------------------------------------------------------------------------------
	public void InputActiveChanged( int input_number )
	{
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).InputActiveChanged(input_number);
	}
	//------------------------------------------------------------------------------
	public void OutputActiveChanged( int output_number )
	{
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).OutputActiveChanged(output_number);
	}
	//------------------------------------------------------------------------------
	public void InputNameChanged( int input_number )
	{
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).InputNameChanged(input_number);
	}
	//------------------------------------------------------------------------------
	public void OutputNameChanged( int output_number )
	{
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).OutputNameChanged(output_number);
	}
	//------------------------------------------------------------------------------
	public void InputRangeChanged( int input_number )
	{
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).InputRangeChanged(input_number);
	}
	//------------------------------------------------------------------------------
	public void OutputRangeChanged( int output_number )
	{
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).OutputRangeChanged(output_number);
	}
	//------------------------------------------------------------------------------
	public void RemoveMFInInput( int input_number, int mf_number ) throws Throwable
	{
		jnifis.RemoveMFInInput(this.ptr, input_number, mf_number);
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).MFRemovedInInput(input_number);
	}
	//------------------------------------------------------------------------------
	public void RemoveMFInOutput( int output_number, int mf_number ) throws Throwable
	{
		jnifis.RemoveMFInOutput(this.ptr, output_number, mf_number);
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).MFRemovedInOutput(output_number);
	}
	//------------------------------------------------------------------------------
	public void MFAddedInInput( int input_number )
	{
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).MFAddedInInput(input_number);
	}
	//------------------------------------------------------------------------------
	public void MFAddedInOutput( int output_number )
	{
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).MFAddedInOutput(output_number);
	}
	//------------------------------------------------------------------------------
	public void MFReplacedInInput( int input_number )
	{
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).MFReplacedInInput(input_number);
	}
	//------------------------------------------------------------------------------
	public void MFReplacedInOutput( int output_number )
	{
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).MFReplacedInOutput(output_number);
	}
	//------------------------------------------------------------------------------
	public void SetOutputDefuz( int output_number, String defuz ) throws Throwable
	{
		jnifis.SetOutputDefuz(this.ptr, output_number, defuz);
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).OutputDefuzChanged(output_number);
	}
	//------------------------------------------------------------------------------
	public void SetOutputDisjunction( int output_number, String disj ) throws Throwable
	{
		jnifis.SetOutputDisjunction(this.ptr, output_number, disj);
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).OutputDisjChanged(output_number);
	}
	//------------------------------------------------------------------------------
	public void SetOutputClassif( int output_number, boolean classif ) throws Throwable
	{
		jnifis.SetOutputClassif(this.ptr, output_number, classif);
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).OutputClassifChanged(output_number);
	}
	//------------------------------------------------------------------------------
	public void SetOutputImpliFuzzy( int output_number, boolean impli ) throws Throwable
	{
		jnifis.SetOutputImplicatifFuzzy(this.ptr, output_number, impli);
		Object [] listeners = this.Listeners.toArray(); 
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).OutputImpliChanged(output_number);
	}
	//------------------------------------------------------------------------------
	public void SetOutputDefaultValue( int output_number, double default_value ) throws Throwable
	{
		this.GetOutput(output_number).SetDefaultValue(default_value);
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).OutputDefaultChanged(output_number);
	}
	//------------------------------------------------------------------------------
	public void SetAlarmThres( int output_number, double default_value ) throws Throwable
	{
		this.GetOutput(output_number).SetDefuzThres(default_value);
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).OutputAlarmThresChanged(output_number);
	}
	//------------------------------------------------------------------------------
	public double [] GetClassesLabels( int output_number ) throws Throwable { return jnifis.GetOutputClassesLabels( this.ptr, output_number ); }
	//------------------------------------------------------------------------------
	public void ReplaceRules( JRule [] new_rules ) throws Throwable
	{
		// supression des regles existantes
		while( this.NbRules() != 0 )
			this.RemoveRule(0);
		// ajout des nouvelles regles au sif
		for( int i=0 ; i<new_rules.length ; i++ )
			this.AddRule(new_rules[i]);

		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).RulesModified();
	}
	//------------------------------------------------------------------------------
	public void Crisp2Fuz( String defuz, int output_number ) throws Throwable
	{
		jnifis.Crisp2Fuz(this.ptr, defuz, output_number);
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).OutputReplaced(output_number);
	}
	//------------------------------------------------------------------------------
	public void Fuz2Crisp( int output_number ) throws Throwable
	{
		jnifis.Fuz2Crisp(this.ptr, output_number);
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).OutputReplaced(output_number);
	}
	//------------------------------------------------------------------------------
	public int Conj2Imp( int output_number, boolean transfPart ) throws Throwable
	{
		int error = 0;
		error = jnifis.Conj2Imp(this.ptr, output_number, transfPart);
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
		    {
			((FISListener)listeners[i]).OutputReplaced(output_number);
			((FISListener)listeners[i]).OutputImpliChanged(output_number);
		    }
		return error;
	}
	//------------------------------------------------------------------------------
	public void Imp2Conj( int output_number, boolean transfPart ) throws Throwable
	{
		jnifis.Imp2Conj(this.ptr, output_number, transfPart );
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
		    {
			((FISListener)listeners[i]).OutputReplaced(output_number);
			((FISListener)listeners[i]).OutputImpliChanged(output_number);
		    }
	}
	//------------------------------------------------------------------------------
	public void SetRuleActive( int rule_number, boolean active ) throws Throwable
	{
		this.GetRule(rule_number).SetActive(active);
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).RulesModified();
	}
	//------------------------------------------------------------------------------
	public void SetRuleExpertWeight( int rule_number, double weight ) throws Throwable
	{
		this.GetRule(rule_number).SetExpertWeight(weight);
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).RulesModified();
	}
	//------------------------------------------------------------------------------
	public void SetRuleProp( int rule_number, int input_number, int prop ) throws Throwable
	{
		this.GetRule(rule_number).SetProp(input_number, prop);
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).RulesModified();
	}
	//------------------------------------------------------------------------------
	public void SetRuleConc( int rule_number, int output_number, double conc ) throws Throwable
	{
		this.GetRule(rule_number).SetConc(output_number, conc);
		Object [] listeners = this.Listeners.toArray();
		for( int i=0 ; i<listeners.length ; i++ )
			((FISListener)listeners[i]).RulesModified();
	}
	//------------------------------------------------------------------------------
	public int GetActiveInputNumber( int active_number ) throws Throwable
	{
		for( int i=0, active=0 ; i<this.NbInputs() ; i++ )
			if( this.GetInput(i).GetActive() == true )
			{
				if( active == active_number )
					return i;
				else
					active++;
			}
		throw new Exception("No active input number: " + active_number);
	}
	//------------------------------------------------------------------------------
	public int GetActiveOutputNumber( int active_number ) throws Throwable
	{
		for( int i=0, active=0 ; i<this.NbOutputs() ; i++ )
			if( this.GetOutput(i).GetActive() == true )
			{
				if( active == active_number )
					return i;
				else
					active++;
			}
		throw new Exception("No active output number: " + active_number);
	}
	//------------------------------------------------------------------------------
	public void InitClasses( String data_file ) throws Throwable
	{
		for( int i=0 ; i<this.NbOutputs() ; i++ )
			jnifis.InitClasses(this.ptr, i, data_file);
	}
	//------------------------------------------------------------------------------
	public void InitPossibles() throws Throwable { jnifis.InitPossibles(this.ptr); }
	//------------------------------------------------------------------------------
	public double [][] AgregationResult(int output_number) throws Throwable { return jnifis.AgregationResult(this.ptr, output_number); }
	//------------------------------------------------------------------------------
	public int AgregationImpliResult(int output_number, Vector<Double> result) throws Throwable { return jnifis.AgregationImpliResult(this.ptr, output_number, result); }
	//------------------------------------------------------------------------------
	public int GetImpliConc(int output_number, int rule_number, Vector<Double> result) throws Throwable { return jnifis.GetImpliConc(this.ptr, output_number, rule_number, result); }

	public boolean hasExpertWeights() throws Throwable {
		for(int i=0 ; i<NbRules() ; i++)
			if(GetRule(i).GetExpertWeight() != 1.0)
				return true;
		return false;
	}

	public boolean hasImplicativeOutput() throws Throwable {
		for(int i=0 ; i<NbOutputs() ; i++ )
			if(GetOutput(i).GetImplicatif())
				return true;
		return false;
	}
}


//------------------------------------------------------------------------------
class JFISFPA extends JFIS
{
	public JFISFPA( JFIS fis, JExtendedDataFile data, int strategy, int min_card, double min_deg  ) throws Throwable
	{
		super( fis.NewFISFPA(data, strategy, min_card, min_deg) );
	}
}


//------------------------------------------------------------------------------
class JGENFIS extends JFIS
{
	public JGENFIS( JFIS fis, JExtendedDataFile data, boolean sort, double thres  ) throws Throwable
	{
		super( fis.NewGENFIS(data, sort, thres) );
	}
}



//------------------------------------------------------------------------------
interface FISListener
{
	public void FISClosed();
	public void InputReplaced( int input_number );
	public void OutputReplaced( int output_number );
	public void InputActiveChanged( int input_number );
	public void OutputActiveChanged( int output_number );
	public void InputNameChanged( int input_number );
	public void OutputNameChanged( int output_number );
	public void InputRangeChanged( int input_number );
	public void OutputRangeChanged( int output_number );
	public void InputRemoved( int input_number );
	public void OutputRemoved( int output_number );
	public void InputAdded( int input_number );
	public void OutputAdded( int output_number );
	public void MFRemovedInInput( int input_number );
	public void MFRemovedInOutput( int output_number );
	public void MFAddedInInput( int input_number );
	public void MFAddedInOutput( int output_number );
	public void MFReplacedInInput( int input_number );
	public void MFReplacedInOutput( int output_number );
	public void OutputDefuzChanged( int output_number );
	public void OutputImpliChanged( int output_number );
	public void OutputDisjChanged( int output_number );
	public void OutputClassifChanged( int output_number );
	public void OutputDefaultChanged( int output_number );
	public void OutputAlarmThresChanged( int output_number );
	public void RulesModified();
	public void ConjunctionChanged();
}



