//***********************************************************************
//
//                              JFISConsole.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

//------------------------------------------------------------------------------
public class JFISConsole extends JFISChildFrame
{
private JTextArea textArea;
private JButton button = new JButton();
//------------------------------------------------------------------------------
public JFISConsole( JFISFrame parent, String file_name ) throws Throwable
  {
  super(parent);
  this.setTitle(file_name);
  this.Init();
  this.button.setVisible(false);
  this.ShowFile(new File(file_name));
  }
//------------------------------------------------------------------------------
public JFISConsole( JFISFrame parent ) throws Throwable
  {
  super(parent);
  this.setTitle("Java Console");
  this.Init();
  }
//------------------------------------------------------------------------------
public void Init() throws Throwable
  {
  Dimension screenSize=Toolkit.getDefaultToolkit().getScreenSize();
  Dimension frameSize=new Dimension((int)(screenSize.width/2),(int)(screenSize.height/2));
  int x=(int)(frameSize.width/2);
  int y=(int)(frameSize.height/2);
  this.setBounds(x,y,frameSize.width,frameSize.height);

  textArea=new JTextArea();
  textArea.setEditable(false);

  this.getContentPane().setLayout(new BorderLayout());
  this.getContentPane().add(new JScrollPane(textArea),BorderLayout.CENTER);
  this.getContentPane().add(button,BorderLayout.SOUTH);

  button.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { JFISConsole.this.textArea.setText(""); } });

  this.Translate();
  this.setLocation(this.ChildPosition(this.getSize()));
  this.setVisible(true);
  }
//------------------------------------------------------------------------------
public void dispose()
  {
  super.dispose();
  MainFIS.Console = null;
  }
//------------------------------------------------------------------------------
public void Translate() throws Throwable { button.setText(LocaleFIS.GetString("Clear")); this.repaint(); ;}
//------------------------------------------------------------------------------
public void ShowFile( File file )
  {
  try
    {
    FileInputStream pin= new FileInputStream(file);
    while( pin.available() !=0 )
      {
      String input = this.readLine(pin);
      textArea.append(input);
      }
    }
  catch( Exception e ) { this.textArea.append(e.getLocalizedMessage()); }
  }
//------------------------------------------------------------------------------
public String readLine(FileInputStream in) throws IOException
  {
  String input="";
  do
    {
    int available=in.available();
    if (available==0) break;
    byte b[]=new byte[available];
    in.read(b);
    input=input+new String(b,0,b.length);
    } while( !input.endsWith("\n") &&  !input.endsWith("\r\n") );
  return input;
  }
}