//***********************************************************************
//
//                              JFISFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import static fis.LocaleFIS.GetString;
import static fis.optimization.crossvalidation.CrossValidation.LABEL_CROSS_VALIDATION;
import static javax.swing.JOptionPane.OK_CANCEL_OPTION;
import static javax.swing.JOptionPane.PLAIN_MESSAGE;
import static javax.swing.JOptionPane.showOptionDialog;
import static util.swing.SwingUtilities.getWindowAncestor;

import java.awt.*;
import java.awt.event.*;

import javax.swing.border.*;
import javax.swing.*;

import java.util.*;
import java.io.*;

import javax.swing.event.*;

import com.jgoodies.binding.value.ValueHolder;

import util.jgoodies.validation.ValidationResultAdapter;

import fis.data.sample.generator.block.BlockSampleGenerator;
import fis.data.sample.generator.block.BlockSampleGeneratorModel;
import fis.data.sample.generator.pair.PairSampleGenerator;
import fis.data.sample.generator.pair.PairSampleGeneratorModel;
import fis.swing.data.distance.DistanceAction;
import fis.swing.data.sample.generator.block.BlockSampleGeneratorAdapter;
import fis.swing.data.sample.generator.block.JPanelBlockSampleGenerator;
import fis.swing.data.sample.generator.pair.JPanelPairSampleGenerator;
import fis.swing.data.sample.generator.pair.PairSampleGeneratorAdapter;

public class JFISFrame extends JFrame implements Translatable
{
	private static Vector JFISFrames = new Vector();    // vecteur des JFISFRame ouvertes
	private static Vector Translatables = new Vector();    // vector des frames traduisibles qui implemente l'interface Translatable
	private static Vector JFramesLook = new Vector();   // vecteur des JFrames dont il faut gerer le look
	private JFIS fis = null;             // objet sif
	private ValueHolder fisHolder = new ValueHolder();
	private JExtendedDataFile DataFile = null;   // objet fichier de donnees
	private ValueHolder dataHolder = new ValueHolder();
	private JSemaphore JRuleFrameOpen = new JSemaphore();  // semaphore fenetre regles ouverte
	private JSemaphore JInferenceFrameOpen = new JSemaphore();  // semaphore fenetre inference ouverte
	private JSemaphore JPerformanceFrameOpen = new JSemaphore();  // semaphore fenetre performance ouverte
	private JRuleFrame jRuleFrame = null;  // fenetre regles
	private JDataTableFrame jDataTableFrame = null;  // fenetre de donnees
	public JInferenceFrame jInferenceFrame = null;  // fenetre inference
	private JPerformanceFrame jPerformanceFrame = null;  // fenetre performance
	private FISListener FISListener;          // listener de FIS

	// gestion des menus
	private JMenuBar jMenuBarFIS = new JMenuBar();
	// menu sif
	private JMenu jMenuFIS = new JMenu();
	private JMenuItem jMenuNew = new JMenuItem();
	private JMenuItem jMenuOpen = new JMenuItem();
	private JMenuItem jMenuClose = new JMenuItem();
	private JMenuItem jMenuSave = new JMenuItem();
	private JMenuItem jMenuSaveAs = new JMenuItem();
	private JMenu jMenuInputs = new JMenu();
	private JMenuItem jMenuNewInput = new JMenuItem();
	private JMenuItem jMenuEditInput = new JMenuItem();
	private JMenuItem jMenuRemoveInput = new JMenuItem();
	private JMenu jMenuOutputs = new JMenu();
	private JMenuItem jMenuNewOutput = new JMenuItem();
	private JMenuItem jMenuEditOutput = new JMenuItem();
	private JMenuItem jMenuRemoveOutput = new JMenuItem();
	private JMenuItem jMenuInfere = new JMenuItem();
	private JMenu jMenuSystemResponse = new JMenu();
	private JMenuItem jMenu2DInfer = new JMenuItem();
	private JMenuItem jMenu3DInfer = new JMenuItem();
	private JMenuItem jMenuHFPSR1 = new JMenuItem();
	private JMenuItem jMenuGenerateRules = new JMenuItem();
	private JMenuItem jMenuGenerateConcs = new JMenuItem();
	private JMenuItem jMenuExit = new JMenuItem();
	// menu data
	private JMenu jMenuData = new JMenu();
	private JMenuItem jMenuDataOpen = new JMenuItem();
	private JMenuItem jMenuDataClose = new JMenuItem();
	//private JMenuItem jMenuDataGenerateSample = new JMenuItem();
	private JMenu jMenuDataGenerateSample = new JMenu();
	private JMenuItem jMenuDataGenerateSample1 = new JMenuItem();
	private JMenuItem jMenuDataGenerateSample2 = new JMenuItem();

	private JMenuItem jMenuDataView = new JMenuItem();
	private JMenuItem jMenuDataTable = new JMenuItem();
	private JMenuItem jMenuDataInfer = new JMenuItem();
	private JMenuItem jMenuDataLinks = new JMenuItem();
	private JMenuItem jMenuDataDistance = new JMenuItem();
	// menu apprentissage
	private JMenu jMenuLearning = new JMenu();
	private JMenu jMenuLearningPartition = new JMenu();
	private JMenuItem jMenuHFPSR2 = new JMenuItem();
	private JMenu jMenuHFPMF = new JMenu();
	private JMenuItem jMenuHFPGenerate = new JMenuItem();
	private JMenuItem jMenuHFPEdit = new JMenuItem();
	private JMenuItem jMenuHFPGenerateVertices = new JMenuItem();
	private JMenuItem jMenuHFPViewVertices = new JMenuItem();
	private JMenuItem jMenuHFPSelectPartition = new JMenuItem();
	private JMenu jMenuLearningRules = new JMenu();
	private JMenuItem jMenuFPAInduction = new JMenuItem();
	private JMenuItem jMenuWangMendelInduction = new JMenuItem();
	private JMenuItem jMenuOLS = new JMenuItem();
	private JMenu jMenuTree = new JMenu();
	private JMenuItem jMenuTreeGenerate = new JMenuItem();
	private JMenuItem jMenuTreeView = new JMenu();
	private JMenuItem jMenuTreeTable = new JMenuItem();
	private JMenuItem jMenuTreeGraphe = new JMenuItem();
	private JMenuItem jMenuHFPGenerateFIS = new JMenuItem();
	private JMenuItem jMenuSimple = new JMenuItem();
	private JMenu jMenuOptimization = new JMenu();
	private JMenuItem jMenuSolisWets = new JMenuItem();
	private JMenuItem jMenuCustomSolisWets = new JMenuItem();
	private JMenuItem jMenuConclusionsOptimization = new JMenuItem();
	// menu options
	private JMenu jMenuOptions = new JMenu();
	private JMenu jMenuLanguage = new JMenu();
	private JRadioButtonMenuItem jRBMenuFrench = new JRadioButtonMenuItem();
	private JRadioButtonMenuItem jRBMenuEnglish = new JRadioButtonMenuItem();
	private JRadioButtonMenuItem jRBMenuSpanish = new JRadioButtonMenuItem();
	private JRadioButtonMenuItem jRBMenuPortugues = new JRadioButtonMenuItem();
	private JMenu jMenuLook = new JMenu();
	// menu aide
	private JMenu jMenuHelp = new JMenu();
	private JMenuItem jMenuItemHelp = new JMenuItem();
	private JMenuItem jMenuAbout = new JMenuItem();

	// menus popup
	private JPopupMenu jPopupMenuInputs = new JPopupMenu();
	private JMenuItem jPopupNewInput = new JMenuItem();
	private JMenuItem jPopupEditInput = new JMenuItem();
	private JMenuItem jPopupRemoveInput = new JMenuItem();
	private JPopupMenu jPopupMenuOutputs = new JPopupMenu();
	private JMenuItem jPopupNewOutput = new JMenuItem();
	private JMenuItem jPopupEditOutput = new JMenuItem();
	private JMenuItem jPopupRemoveOutput = new JMenuItem();

	// Listes
	private JList jListInputs = new JList()
	{
		// selectionneur de liste
		protected ListSelectionModel createSelectionModel() { return new DefaultListSelectionModel()
		{
			// element selectionne
			public boolean isSelectedIndex(int index)
			{
				boolean retour = super.isSelectedIndex(index);
				// l'element index de la liste n'est pas selectionnable si la fenetre correspondant a l'entree index est deja ouverte
				if( ((JSemaphore)JFISFrame.this.fis.GetInputSemaphore(index)).Permits() == 0 )
					return false;
				return retour;
			}
		}; }
	};
	private JList jListOutputs = new JList()
	{
		// selectionneur de liste
		protected ListSelectionModel createSelectionModel() { return new DefaultListSelectionModel()
		{
			// element selectionne
			public boolean isSelectedIndex(int index)
			{
				boolean retour = super.isSelectedIndex(index);
				// l'element index de la liste n'est pas selectionnable si la fenetre correspondant a la sortie index est deja ouverte
				if( ((JSemaphore)JFISFrame.this.fis.GetOutputSemaphore(index)).Permits() == 0 )
					return false;
				return retour;
			}
		}; }
	};
	private DefaultListModel jListInputsModel;
	private DefaultListModel jListOutputsModel;

	// ComboBox conjunction
	private JComboBox jCBConjunction = new JComboBox();
	private DefaultComboBoxModel jCBConjunctionModel = new DefaultComboBoxModel()
	{
		// c'est cette methode qui assure la traduction des types de conjonction
		public Object getElementAt(int index)
		{
			try { return LocaleFIS.GetString(JFIS.TypeConjunction()[index]); }
			catch( Throwable t ) { return null; }
		}
	};

	// objets graphique de la fenetre
	private JPanel jPHeader = new JPanel(new GridBagLayout());
	// nom
	private JPanel jPName = new JPanel(new GridBagLayout());
	private TitledBorder titledBorderName;
	private JTextField jTFName = new JTextField();
	// conjunction
	private JPanel jPConjunction = new JPanel();
	private TitledBorder titledBorderConjunction;
	// fichier de donnees
	private JPanel jPDataFile = new JPanel(new GridBagLayout());
	private TitledBorder titledBorderDataFile;
	private JTextField jTFDataFile = new JTextField();
	// entrees
	private JScrollPane jScrollPaneInputs = new JScrollPane()
	{
		public Dimension getMinimumSize() { return  new Dimension(150,100); }
	};
	private TitledBorder titledBorderInputs;
	// regles
	private JButton jButtonRules = new JButton();
	// sorties
	private JScrollPane jScrollPaneOutputs = new JScrollPane()
	{
		public Dimension getMinimumSize() { return  new Dimension(150,100); }
	};
	private TitledBorder titledBorderOutputs;
	// fichier HFP, Vertex
	private String HFPFile;
	private String VertexFile;
	// warning sur nombre de regles
	private int WarningNumberOfRules = 10000;

	//------------------------------------------------------------------------------
	public JFISFrame()
	{
		try { jbInit(); }
		catch(Throwable t)
		{
			MessageFIS.Error(null, t);
			this.dispose();
		}
	}
	//------------------------------------------------------------------------------
	public JFISFrame( JFIS fis, JExtendedDataFile data_file, JFrame parent )
	{
		try
		{
			jbInit();
			this.fis = fis;
			this.fisHolder.setValue(this.fis);
			if( data_file != null )
			{
				this.DataFile = JExtendedDataFile.Open(data_file.FileName(), false);
				this.dataHolder.setValue(this.DataFile);
				this.jTFDataFile.setText(this.DataFile.FileName());
			}
			this.InitJFISFrameWithFIS();
			// decalage de la fenetre par rapport a la fenetre parent
			if( parent != null )
				this.setLocation(parent.getLocation().x + this.getInsets().top, parent.getLocation().y + this.getInsets().top);
	    	SwingUtilities.invokeLater(new Runnable() {

				@Override
				public void run() {
			    	requestFocus();
			}});
		}
		catch(Throwable t)
		{
			MessageFIS.Error(null, t);
			this.dispose();
		}
	}
	//------------------------------------------------------------------------------
	private void jbInit() throws Throwable
	{
		// creation du menu
		// menu sif
		jMenuBarFIS.add(jMenuFIS);
		jMenuFIS.add(jMenuNew);
		jMenuFIS.add(jMenuOpen);
		jMenuFIS.add(jMenuClose);
		jMenuFIS.add(jMenuSave);
		jMenuFIS.add(jMenuSaveAs);
		jMenuFIS.addSeparator();
		jMenuFIS.add(jMenuInputs);
		jMenuInputs.add(jMenuNewInput);
		jMenuInputs.add(jMenuEditInput);
		jMenuInputs.add(jMenuRemoveInput);
		jMenuFIS.add(jMenuOutputs);
		jMenuOutputs.add(jMenuNewOutput);
		jMenuOutputs.add(jMenuEditOutput);
		jMenuOutputs.add(jMenuRemoveOutput);
		jMenuFIS.addSeparator();
		jMenuFIS.add(jMenuInfere);
		jMenuFIS.add(jMenuSystemResponse);
		jMenuSystemResponse.add(jMenu2DInfer);
		jMenuSystemResponse.add(jMenu3DInfer);
		jMenuFIS.addSeparator();
		jMenuFIS.add(this.jMenuHFPSR1);
		jMenuFIS.add(jMenuGenerateRules);
		jMenuFIS.add(this.jMenuGenerateConcs);
		jMenuFIS.addSeparator();
		jMenuFIS.add(jMenuExit);
		// menu data
		jMenuBarFIS.add(jMenuData);
		jMenuData.add(jMenuDataOpen);
		jMenuData.add(jMenuDataClose);
		jMenuData.add(jMenuDataGenerateSample);
		// submenu
		this.jMenuDataGenerateSample.add(this.jMenuDataGenerateSample1);
		this.jMenuDataGenerateSample.add(this.jMenuDataGenerateSample2);
		//
		jMenuData.addSeparator();
		jMenuData.add(jMenuDataView);
		jMenuData.add(jMenuDataTable);
		jMenuData.addSeparator();
		jMenuData.add(jMenuDataInfer);
		jMenuData.add(jMenuDataLinks);
		jMenuData.add(jMenuDataDistance);
		// menu apprentissage
		jMenuBarFIS.add(jMenuLearning);
		this.jMenuLearning.add(this.jMenuLearningPartition);
		this.jMenuLearningPartition.add(this.jMenuHFPSR2);
		this.jMenuLearningPartition.add(this.jMenuHFPMF);
		this.jMenuHFPMF.add(this.jMenuHFPGenerate);
		this.jMenuHFPMF.add(this.jMenuHFPEdit);
		this.jMenuHFPMF.add(this.jMenuHFPGenerateVertices);
		this.jMenuHFPMF.add(this.jMenuHFPViewVertices);
		this.jMenuHFPMF.add(this.jMenuHFPSelectPartition);
		this.jMenuLearning.add(this.jMenuLearningRules);
		this.jMenuLearningRules.add(this.jMenuFPAInduction);
		this.jMenuLearningRules.add(this.jMenuWangMendelInduction);
		this.jMenuLearningRules.add(this.jMenuOLS);
		this.jMenuLearningRules.add(this.jMenuTree);
		this.jMenuTree.add(this.jMenuTreeGenerate);
		this.jMenuTree.add(this.jMenuTreeView);
		this.jMenuTreeView.add(this.jMenuTreeTable);
		this.jMenuTreeView.add(this.jMenuTreeGraphe);
		this.jMenuLearningRules.add(this.jMenuHFPGenerateFIS);
		jMenuLearning.addSeparator();
		jMenuLearning.add(jMenuSimple);
		jMenuLearning.add(jMenuOptimization);
		jMenuOptimization.add(jMenuSolisWets);
		jMenuOptimization.add(jMenuCustomSolisWets);
		jMenuOptimization.add(jMenuConclusionsOptimization);
		// menu options
		jMenuBarFIS.add(jMenuOptions);
		jMenuOptions.add(jMenuLanguage);
		jMenuLanguage.add(jRBMenuFrench);
		jMenuLanguage.add(jRBMenuEnglish);
		jMenuLanguage.add(jRBMenuSpanish);
		jMenuLanguage.add(jRBMenuPortugues);
		jMenuOptions.add(jMenuLook);
		// menu aide
		jMenuBarFIS.add(jMenuHelp);
		jMenuHelp.add(jMenuItemHelp);
		jMenuHelp.add(jMenuAbout);

		// menus popup
		// popup inputs
		jPopupMenuInputs.add(jPopupNewInput);
		jPopupMenuInputs.add(jPopupEditInput);
		jPopupMenuInputs.add(jPopupRemoveInput);
		// popup outputs
		jPopupMenuOutputs.add(jPopupNewOutput);
		jPopupMenuOutputs.add(jPopupEditOutput);
		jPopupMenuOutputs.add(jPopupRemoveOutput);

		this.setJMenuBar(jMenuBarFIS);

		// creation du panel du SIF
		this.getContentPane().setLayout(new GridBagLayout());

		// Header
		this.getContentPane().add(jPHeader,  new GridBagConstraints(0, 0, 3, 1, 1.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));

		// nom
		titledBorderName = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"");
		jPName.setBorder(titledBorderName);
		jPHeader.add(jPName,         new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 10, 0, 10), 0, 0));
		jPName.add(jTFName,    new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 0, 5), 0, 0));

		// conjonction
		for( int i=0 ; i<JFIS.TypeConjunction().length ; i++ )
			this.jCBConjunctionModel.addElement(new String());  // on met des String vide dans le modele, c'est la methode getElementAt qui fait la traduction
		this.jCBConjunction.setModel(this.jCBConjunctionModel);
		titledBorderConjunction = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"");
		jPConjunction.setBorder(titledBorderConjunction);
		jPHeader.add(jPConjunction,          new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 0, 0, 10), 0, 0));
		jPConjunction.add(jCBConjunction, null);

		// fichier de donnees
		jTFDataFile.setEditable(false);
		jPDataFile.add(jTFDataFile,   new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 0, 5), 0, 0));
		titledBorderDataFile = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"");
		jPDataFile.setBorder(titledBorderDataFile);
		jPHeader.add(jPDataFile,         new GridBagConstraints(2, 0, 2, 1, 1.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 0, 0, 10), 0, 0));

		// entrees
		this.getContentPane().add(jScrollPaneInputs,           new GridBagConstraints(0, 1, 1, 2, 1.0, 1.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(10, 10, 10, 10), 0, 0));
		//jListInputs.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		jScrollPaneInputs.getViewport().add(jListInputs, null);
		titledBorderInputs = new TitledBorder("");
		jScrollPaneInputs.setBorder(titledBorderInputs);

		// icone
		this.getContentPane().add(new JLabel(MainFIS.fisproIcon),       new GridBagConstraints(1, 1, 1, 1, 0.0, 1.0
				,GridBagConstraints.SOUTH, GridBagConstraints.NONE, new Insets(5, 0, 5, 0), 0, 0));

		// regles
		this.getContentPane().add(jButtonRules,       new GridBagConstraints(1, 2, 1, 1, 0.0, 1.0
				,GridBagConstraints.NORTH, GridBagConstraints.NONE, new Insets(5, 0, 5, 0), 0, 0));

		// sorties
		this.getContentPane().add(jScrollPaneOutputs,            new GridBagConstraints(2, 1, 1, 2, 1.0, 1.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(10, 10, 10, 10), 0, 0));
		jScrollPaneOutputs.getViewport().add(jListOutputs, null);
		titledBorderOutputs = new TitledBorder("");
		jScrollPaneOutputs.setBorder(titledBorderOutputs);

		// traduction
		Translate();

		// gestion des evenements
		Events();
		this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

		this.InitJFISFrameWithFIS();

		// affectation du look et affichage
		MainFIS.look.ChangeLook( this, MainFIS.config.LookAndFeel() );
		this.JFISFrames.addElement(this);   // ajoute la fenetre a la liste des fenetres JFISFrame ouvertes
		JFISFrame.AddTranslatable(this);    // ajoute la fenetre a la liste des fenetres traduisibles
		JFISFrame.AddJFrameLook(this);      // ajoute la fenetre a la liste des fenetres lookable
		this.pack();

		this.setIconImages(MainFIS.icons);
		
		// affichage de la fenetre au centre de l'ecran
		Dimension dim = getToolkit().getScreenSize();
		this.setLocation(dim.width/2 - this.getWidth()/2, dim.height/2 - this.getHeight()/2);
		this.setVisible(true);
	}
	//------------------------------------------------------------------------------
	public void Translate()
	{
		this.TranslateTitle();
		this.TranslateMenu();
		this.TranslateFileChooser();

		// traduction de la conjonction
		try { this.jCBConjunction.setSelectedIndex(this.fis.GetConjunction()); }
		catch( Throwable t ) {}

		this.titledBorderName.setTitle(LocaleFIS.GetString("Name"));
		this.titledBorderConjunction.setTitle(LocaleFIS.GetString("Conjunction"));
		this.titledBorderDataFile.setTitle(LocaleFIS.GetString("DataFile"));
		this.titledBorderInputs.setTitle(LocaleFIS.GetString("Inputs"));
		this.jButtonRules.setText(LocaleFIS.GetString("Rules"));
		this.titledBorderOutputs.setTitle(LocaleFIS.GetString("Outputs"));
		this.repaint();
	}
	//------------------------------------------------------------------------------
	private void TranslateTitle()
	{
		this.setTitle(LocaleFIS.GetString("FIS") + ": ");
		if( this.fis != null )
			if( this.fis.FISFile != null )
				this.setTitle(LocaleFIS.GetString("FIS") + ": " + this.fis.FISFile);
	}
	//------------------------------------------------------------------------------
	private void TranslateMenu()
	{
		// menu SIF
		this.jMenuFIS.setText(LocaleFIS.GetString("FIS"));
		this.jMenuNew.setText(LocaleFIS.GetString("New"));
		this.jMenuOpen.setText(LocaleFIS.GetString("Open"));
		this.jMenuClose.setText(LocaleFIS.GetString("Close"));
		this.jMenuSave.setText(LocaleFIS.GetString("Save"));
		this.jMenuSaveAs.setText(LocaleFIS.GetString("SaveAs"));
		this.jMenuInputs.setText(LocaleFIS.GetString("Inputs"));
		this.jMenuNewInput.setText(LocaleFIS.GetString("NewInput"));
		this.jMenuEditInput.setText(LocaleFIS.GetString("Edit"));
		this.jMenuRemoveInput.setText(LocaleFIS.GetString("Remove"));
		this.jMenuOutputs.setText(LocaleFIS.GetString("Outputs"));
		this.jMenuNewOutput.setText(LocaleFIS.GetString("NewOutput"));
		this.jMenuEditOutput.setText(LocaleFIS.GetString("Edit"));
		this.jMenuRemoveOutput.setText(LocaleFIS.GetString("Remove"));
		this.jMenuInfere.setText(LocaleFIS.GetString("Infer"));
		this.jMenuHFPSR1.setText(LocaleFIS.GetString("GenerateFisWithoutRules"));
		this.jMenuGenerateRules.setText(LocaleFIS.GetString("GenerateRules"));
		this.jMenuGenerateConcs.setText(LocaleFIS.GetString("GenerateConcs"));
		this.jMenuExit.setText(LocaleFIS.GetString("Exit"));
		// menu Data
		this.jMenuData.setText(LocaleFIS.GetString("Data"));
		this.jMenuDataOpen.setText(LocaleFIS.GetString("Open"));
		this.jMenuDataClose.setText(LocaleFIS.GetString("Close"));
		this.jMenuDataGenerateSample.setText(LocaleFIS.GetString("GenerateSample"));
		// submenu sample
		this.jMenuDataGenerateSample1.setText(LocaleFIS.GetString("GenerateSamplePairs"));
		this.jMenuDataGenerateSample2.setText(LocaleFIS.GetString("GenerateSampleBlocks"));
		//
		this.jMenuDataView.setText(LocaleFIS.GetString("View"));
		this.jMenuDataTable.setText(LocaleFIS.GetString("Table"));
		this.jMenuDataInfer.setText(LocaleFIS.GetString("Infer"));
		this.jMenuSystemResponse.setText(LocaleFIS.GetString("SystemResponse"));
		this.jMenu2DInfer.setText(LocaleFIS.GetString("Section"));
		this.jMenu3DInfer.setText(LocaleFIS.GetString("Surface"));
		this.jMenuDataLinks.setText(LocaleFIS.GetString("Links"));
		this.jMenuDataDistance.setAction(new DistanceAction(fisHolder, dataHolder));
		// menu apprentissage
		this.jMenuLearning.setText(LocaleFIS.GetString("Learning"));
		this.jMenuLearningPartition.setText(LocaleFIS.GetString("Partitions"));
		this.jMenuHFPSR2.setText(LocaleFIS.GetString("GenerateFisWithoutRules"));
		this.jMenuHFPMF.setText(LocaleFIS.GetString("HFP") + " " + LocaleFIS.GetString("MF"));
		this.jMenuHFPGenerate.setText(LocaleFIS.GetString("GenerateHFPFile"));
		this.jMenuHFPEdit.setText(LocaleFIS.GetString("EditHFPFile"));
		this.jMenuHFPGenerateVertices.setText(LocaleFIS.GetString("GenerateVertices"));
		this.jMenuHFPViewVertices.setText(LocaleFIS.GetString("ViewVertices"));
		this.jMenuHFPSelectPartition.setText(LocaleFIS.GetString("SelectPartition"));
		this.jMenuLearningRules.setText(LocaleFIS.GetString("RuleInduction"));
		this.jMenuFPAInduction.setText("FPA");
		this.jMenuWangMendelInduction.setText("Wang & Mendel");
		this.jMenuOLS.setText("OLS");
		this.jMenuTree.setText(LocaleFIS.GetString("Tree"));
		this.jMenuTreeGenerate.setText(LocaleFIS.GetString("Generate"));
		this.jMenuTreeView.setText(LocaleFIS.GetString("View"));
		this.jMenuTreeTable.setText(LocaleFIS.GetString("Table"));
		this.jMenuTreeGraphe.setText(LocaleFIS.GetString("Graphe"));
		this.jMenuHFPGenerateFIS.setText(LocaleFIS.GetString("HFP") + " " + LocaleFIS.GetString("FIS"));
		this.jMenuSimple.setText(LocaleFIS.GetString("Simple"));
		this.jMenuOptimization.setText(LocaleFIS.GetString("Optimization"));
		this.jMenuSolisWets.setText(LocaleFIS.GetString("SolisWets"));
		this.jMenuCustomSolisWets.setText(LocaleFIS.GetString("CustomSolisWets"));
		this.jMenuConclusionsOptimization.setText(LocaleFIS.GetString("OptimizeConclusions"));
		// menu options
		this.jMenuOptions.setText(LocaleFIS.GetString("Options"));
		this.jMenuLanguage.setText(LocaleFIS.GetString("Language"));
		this.jRBMenuFrench.setText(LocaleFIS.GetString("Langfr"));
		this.jRBMenuEnglish.setText(LocaleFIS.GetString("Langen"));
		this.jRBMenuSpanish.setText(LocaleFIS.GetString("Langes"));
		this.jRBMenuPortugues.setText(LocaleFIS.GetString("Langpt"));
		this.jMenuLook.setText(LocaleFIS.GetString("Look"));
		// menu aide
		this.jMenuHelp.setText(LocaleFIS.GetString("Help"));
		this.jMenuItemHelp.setText(LocaleFIS.GetString("Help"));
		this.jMenuAbout.setText(LocaleFIS.GetString("About"));

		// menus popup
		this.jPopupNewInput.setText(LocaleFIS.GetString("NewInput"));
		this.jPopupEditInput.setText(LocaleFIS.GetString("Edit"));
		this.jPopupRemoveInput.setText(LocaleFIS.GetString("Remove"));
		this.jPopupNewOutput.setText(LocaleFIS.GetString("NewOutput"));
		this.jPopupEditOutput.setText(LocaleFIS.GetString("Edit"));
		this.jPopupRemoveOutput.setText(LocaleFIS.GetString("Remove"));
	}
	//------------------------------------------------------------------------------
	private void TranslateFileChooser()
	{
		UIManager.put("FileChooser.acceptAllFileFilterText",LocaleFIS.GetString("FCAcceptAllFileFilterText"));
		UIManager.put("FileChooser.cancelButtonMnemonic",new Integer(LocaleFIS.GetString("FCCancelButtonMnemonic").charAt(0)));
		UIManager.put("FileChooser.cancelButtonText",LocaleFIS.GetString("FCCancelButtonText"));
		UIManager.put("FileChooser.cancelButtonToolTipText",LocaleFIS.GetString("FCCancelButtonToolTipText"));
		//  UIManager.put("FileChooser.detailsViewButtonAccessibleName","");
		UIManager.put("FileChooser.detailsViewButtonToolTipText",LocaleFIS.GetString("FCDetailsViewButtonToolTipText"));
		//  UIManager.put("FileChooser.directoryDescriptionText","");
		//  UIManager.put("FileChooser.fileDescriptionText","");
		UIManager.put("FileChooser.fileNameLabelMnemonic",new Integer(LocaleFIS.GetString("FCFileNameLabelMnemonic").charAt(0)));
		UIManager.put("FileChooser.fileNameLabelText",LocaleFIS.GetString("FCFileNameLabelText"));
		UIManager.put("FileChooser.filesOfTypeLabelMnemonic",new Integer(LocaleFIS.GetString("FCFilesOfTypeLabelMnemonic").charAt(0)));
		UIManager.put("FileChooser.filesOfTypeLabelText",LocaleFIS.GetString("FCFilesOfTypeLabelText"));
		//  UIManager.put("FileChooser.helpButtonMnemonic",new Integer(''));
		//  UIManager.put("FileChooser.helpButtonText","");
		//  UIManager.put("FileChooser.helpButtonToolTipText","");
		//  UIManager.put("FileChooser.homeFolderAccessibleName","");
		UIManager.put("FileChooser.homeFolderToolTipText",LocaleFIS.GetString("FCHomeFolderToolTipText"));
		//  UIManager.put("FileChooser.listViewButtonAccessibleName","");
		UIManager.put("FileChooser.listViewButtonToolTipText",LocaleFIS.GetString("FCListViewButtonToolTipText"));
		UIManager.put("FileChooser.lookInLabelMnemonic",new Integer(LocaleFIS.GetString("FCLookInLabelMnemonic").charAt(0)));
		UIManager.put("FileChooser.lookInLabelText",LocaleFIS.GetString("FCLookInLabelText"));
		//  UIManager.put("FileChooser.newFolderAccessibleNam","Nouveau Dossier");
		//  UIManager.put("FileChooser.newFolderErrorSeparator","");
		//  UIManager.put("FileChooser.newFolderErrorText","");
		UIManager.put("FileChooser.newFolderToolTipText",LocaleFIS.GetString("FCNewFolderToolTipText"));
		UIManager.put("FileChooser.openButtonMnemonic",new Integer(LocaleFIS.GetString("FCOpenButtonMnemonic").charAt(0)));
		UIManager.put("FileChooser.openButtonText",LocaleFIS.GetString("FCOpenButtonText"));
		UIManager.put("FileChooser.openButtonToolTipText",LocaleFIS.GetString("FCOpenButtonToolTipText"));
		UIManager.put("FileChooser.saveButtonMnemonic",new Integer(LocaleFIS.GetString("FCSaveButtonMnemonic").charAt(0)));
		UIManager.put("FileChooser.saveButtonText",LocaleFIS.GetString("FCSaveButtonText"));
		UIManager.put("FileChooser.saveButtonToolTipText",LocaleFIS.GetString("FCSaveButtonToolTipText"));
		//  UIManager.put("FileChooser.upFolderAccessibleName","");
		UIManager.put("FileChooser.upFolderToolTipText",LocaleFIS.GetString("FCUpFolderToolTipText"));
		//  UIManager.put("FileChooser.updateButtonMnemonic",new Integer(''));
		//  UIManager.put("FileChooser.updateButtonText","");
		//  UIManager.put("FileChooser.updateButtonToolTipText","");

		// proprietes du FileChooser CDE/Motif
		UIManager.put("FileChooser.pathLabelText",LocaleFIS.GetString("FCLookInLabelText"));
		UIManager.put("FileChooser.filterLabelText",LocaleFIS.GetString("FCFilesOfTypeLabelText"));
		UIManager.put("FileChooser.foldersLabelText",LocaleFIS.GetString("FCFolderLabelText"));
		UIManager.put("FileChooser.filesLabelText",LocaleFIS.GetString("FCFilesLabelText"));
		UIManager.put("FileChooser.enterFileNameLabelText",LocaleFIS.GetString("FCFileNameLabelText"));
	}
	//------------------------------------------------------------------------------
	private void Events()
	{
		// events de la fenetre principale
		this.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e) { this_windowClosing(e); }
			public void windowActivated(WindowEvent e) { this_windowActivated(e); }
		});

		// redimensionnement de la fenetre
		this.addComponentListener(new ComponentAdapter()
		{
			public void componentResized(ComponentEvent e)
			{
				Dimension min_dim = JFISFrame.this.getMinimumSize();
				Dimension d = JFISFrame.this.getSize();
				if (d.height < min_dim.height || d.width < min_dim.width)
				{
					if (d.height < min_dim.height) d.height = min_dim.height;
					if (d.width < min_dim.width) d.width = min_dim.width;
					JFISFrame.this.setSize(d);
				}
			}
		});

		// evenements de menus
		// menu sif
		jMenuFIS.addMenuListener(new MenuListener()
		{
			public void menuSelected(MenuEvent e) { jMenuFIS_menuSelected(e); }
			public void menuDeselected(MenuEvent e) {}
			public void menuCanceled(MenuEvent e) {}
		});
		jMenuNew.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuNew_actionPerformed(e); }} );
		jMenuOpen.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuOpen_actionPerformed(e); }} );
		jMenuClose.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuClose_actionPerformed(e); }} );
		jMenuSave.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuSave_actionPerformed(e); }} );
		jMenuSaveAs.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuSaveAs_actionPerformed(e); }} );
		jMenuInputs.addMenuListener(new MenuListener()
		{
			public void menuSelected(MenuEvent e) { jMenuInputs_menuSelected(e); }
			public void menuDeselected(MenuEvent e) {}
			public void menuCanceled(MenuEvent e) {}
		});
		jMenuNewInput.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuNewInput_actionPerformed(e); }} );
		jMenuEditInput.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuEditInput_actionPerformed(e); }} );
		jMenuRemoveInput.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuRemoveInput_actionPerformed(e); }} );
		jMenuOutputs.addMenuListener(new MenuListener()
		{
			public void menuSelected(MenuEvent e) { jMenuOutputs_menuSelected(e); }
			public void menuDeselected(MenuEvent e) {}
			public void menuCanceled(MenuEvent e) {}
		});
		jMenuNewOutput.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuNewOutput_actionPerformed(e); }} );
		jMenuEditOutput.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuEditOutput_actionPerformed(e); }} );
		jMenuRemoveOutput.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuRemoveOutput_actionPerformed(e); }} );
		jMenuInfere.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuInfere_actionPerformed(e); }} );
		jMenu2DInfer.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenu2DInfer_actionPerformed(e); }} );
		jMenu3DInfer.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenu3DInfer_actionPerformed(e); }} );
		jMenuExit.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuExit_actionPerformed(e); }} );
		// menu data
		jMenuData.addMenuListener(new MenuListener()
		{
			public void menuSelected(MenuEvent e) { jMenuData_menuSelected(e); }
			public void menuDeselected(MenuEvent e) {}
			public void menuCanceled(MenuEvent e) {}
		});
		jMenuDataOpen.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuDataOpen_actionPerformed(e); }} );
		jMenuDataClose.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuDataClose_actionPerformed(e); }} );
		jMenuDataGenerateSample1.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuDataGenerateSample1_actionPerformed(e); }} );
		jMenuDataGenerateSample2.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuDataGenerateSample2_actionPerformed(e); }} );
		jMenuDataView.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuDataView_actionPerformed(e); }} );
		jMenuDataTable.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuDataTable_actionPerformed(e); }} );
		jMenuDataInfer.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuDataInfer_actionPerformed(e); }} );
		jMenuDataLinks.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuDataLinks_actionPerformed(e); }} );
		// menu apprentissage
		jMenuLearning.addMenuListener(new MenuListener()
		{
			public void menuSelected(MenuEvent e) { jMenuLearning_menuSelected(e); }
			public void menuDeselected(MenuEvent e) {}
			public void menuCanceled(MenuEvent e) {}
		});
		this.jMenuLearningRules.addMenuListener(new MenuListener()
		{
			public void menuSelected(MenuEvent e) { jMenuLearningRules_menuSelected(e); }
			public void menuDeselected(MenuEvent e) {}
			public void menuCanceled(MenuEvent e) {}
		});
		jMenuTree.addMenuListener(new MenuListener()
		{
			public void menuSelected(MenuEvent e) { jMenuTree_menuSelected(e); }
			public void menuDeselected(MenuEvent e) {}
			public void menuCanceled(MenuEvent e) {}
		});
		jMenuTreeTable.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuTreeTable_actionPerformed(e); }} );
		jMenuTreeGraphe.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuTreeGraphe_actionPerformed(e); }} );

		jMenuTreeGenerate.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuTreeGenerate_actionPerformed(e); }} );
		jMenuGenerateConcs.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuGenerateConcs_actionPerformed(e); }} );
		jMenuGenerateRules.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuGenerateRules_actionPerformed(e); }} );
		jMenuHFPGenerate.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuHFPGenerate_actionPerformed(e); }} );
		jMenuHFPEdit.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuHFPEdit_actionPerformed(e); }} );
		jMenuHFPGenerateVertices.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuHFPGenerateVertices_actionPerformed(e); }} );
		jMenuHFPViewVertices.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuHFPViewVertices_actionPerformed(e); }} );
		jMenuHFPSelectPartition.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuHFPSelectPartition_actionPerformed(e); }} );
		jMenuHFPGenerateFIS.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuHFPGenerateFIS_actionPerformed(e); }} );
		jMenuSimple.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuSimple_actionPerformed(e); }} );
		jMenuSolisWets.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuSolisWets_actionPerformed(e); }} );
		jMenuCustomSolisWets.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuCustomSolisWets_actionPerformed(e); }} );
		jMenuConclusionsOptimization.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuConclusionsOptimization_actionPerformed(e); }} );
		jMenuHFPSR1.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuHFPSR_actionPerformed(e); }} );
		this.jMenuFPAInduction.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuFPAInduction_actionPerformed(e); }} );
		jMenuWangMendelInduction.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuWangMendel_actionPerformed(e); }} );
		jMenuOLS.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuOLS_actionPerformed(e); }} );
		jMenuHFPSR2.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuHFPSR_actionPerformed(e); }} );
		// menu options
		jMenuLanguage.addMenuListener(new MenuListener()
		{
			public void menuSelected(MenuEvent e) { jMenuLanguage_menuSelected(e); }
			public void menuDeselected(MenuEvent e) {}
			public void menuCanceled(MenuEvent e) {}
		} );
		jRBMenuFrench.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jRBMenuFrench_actionPerformed(e); }} );
		jRBMenuEnglish.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jRBMenuEnglish_actionPerformed(e); }} );
		jRBMenuSpanish.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jRBMenuSpanish_actionPerformed(e); }} );
		jRBMenuPortugues.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jRBMenuPortugues_actionPerformed(e); }} );
		jMenuLook.addMenuListener(new MenuListener()
		{
			public void menuSelected(MenuEvent e) { jMenuLook_menuSelected(e); }
			public void menuDeselected(MenuEvent e) { jMenuLook_menuDeselected(e); }
			public void menuCanceled(MenuEvent e) {}
		} );
		// menu aide
		jMenuItemHelp.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuItemHelp_actionPerformed(e); } });
		jMenuAbout.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuAbout_actionPerformed(e); } });

		// evenements de menus popup
		// popup inputs
		jListInputs.addMouseListener(new MouseAdapter() { public void mousePressed(MouseEvent e) { jListInputs_mousePressed(e); }} );
		jPopupNewInput.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jPopupNewInput_actionPerformed(e); }} );
		jPopupRemoveInput.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jPopupRemoveInput_actionPerformed(e); }} );
		jPopupEditInput.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jPopupEditInput_actionPerformed(e); }} );
		// popup outputs
		jListOutputs.addMouseListener(new MouseAdapter() { public void mousePressed(MouseEvent e) { jListOutputs_mousePressed(e); }} );
		jPopupNewOutput.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jPopupNewOutput_actionPerformed(e); }} );
		jPopupEditOutput.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jPopupEditOutput_actionPerformed(e); }} );
		jPopupRemoveOutput.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jPopupRemoveOutput_actionPerformed(e); }} );

		// autres evenements
		jTFName.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jTFName_actionPerformed(e); }} );
		jTFName.addFocusListener(new FocusAdapter() { public void focusLost(FocusEvent e) { jTFName_focusLost(e); }} );
		jCBConjunction.addItemListener(new ItemListener() { public void itemStateChanged(ItemEvent e) { jCBConjunction_itemStateChanged(e); }} );
		jButtonRules.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonRules_actionPerformed(e); }} );
	}

	//------------------------------------------------------------------------------
	// Initialisation de la fenetre JFISFrame
	private void InitJFISFrame()
	{
		this.TranslateTitle();    // affichage du titre de la fenetre

		// vide les listes
		this.jCBConjunction.setSelectedIndex(-1);
		this.jListInputsModel = new DefaultListModel();
		this.jListInputs.setModel(this.jListInputsModel);
		this.jListOutputsModel = new DefaultListModel();
		this.jListOutputs.setModel(this.jListOutputsModel);

		// tous les elements de la fenetre sont desactives
		this.jTFName.setEnabled(false);
		this.jTFName.setText("");
		this.jButtonRules.setEnabled(false);
		this.jListInputs.setEnabled(false);
		this.jListOutputs.setEnabled(false);
		this.jCBConjunction.setEnabled(false);
	}
	//------------------------------------------------------------------------------
	// Initialisation de la fenetre JFISFrame avec le sif courant
	private void InitJFISFrameWithFIS() throws Throwable
	{
		this.InitJFISFrame();

		// affichage des elements du sif
		if( this.fis != null )
		{
			// active les elements de la fenetre
			this.jTFName.setEnabled(true);
			this.jListInputs.setEnabled(true);
			this.jListOutputs.setEnabled(true);
			this.jCBConjunction.setEnabled(true);

			// affectation des renderers des listes inputs et outputs
			jListInputs.setCellRenderer(new DefaultListCellRenderer()
			{
				public Component getListCellRendererComponent( JList list, Object value, int index, boolean selected, boolean hasFocus )
				{
					super.getListCellRendererComponent( list, value, index, selected, hasFocus);
					try
					{
						// affichage du nom de l'entree
						super.setText(JFISFrame.this.fis.GetInput(index).GetName());
						// affichage en gris des entees non actives
						if( JFISFrame.this.fis.GetInput(index).GetActive() == false )
							super.setForeground(Color.gray);
					}
					catch( Throwable e ) {}   // on n'affiche pas l'erreur
					return this;
				}
			});
			jListOutputs.setCellRenderer(new DefaultListCellRenderer()
			{
				public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
				{
					super.getListCellRendererComponent( list, value, index, isSelected, cellHasFocus);
					try
					{
						// affichage du nom de la sortie
						super.setText(JFISFrame.this.fis.GetOutput(index).GetName());
						// affichage en gris des sorties non actives
						if( JFISFrame.this.fis.GetOutput(index).GetActive() == false )
							super.setForeground(Color.gray);
					}
					catch( Throwable e ) {}   // on n'affiche pas l'erreur
					return this;
				}
			});

			// affichage du nom du fis
			this.jTFName.setText(this.fis.GetName());

			// affichage de la conjunction
			this.jCBConjunction.setSelectedIndex(this.fis.GetConjunction());

			// affichage des entrees
			this.jListInputsModel = new DefaultListModel();
			this.jListInputs.setModel(this.jListInputsModel);
			for( int i=0 ; i<this.fis.NbInputs() ; i++ )
			{
				this.jListInputsModel.addElement(new String()); // on met une chaine vide dans le model de la liste, c'est le renderer de la liste qui affiche le nom de l'entree
				JInput input = this.fis.GetInput(i);
				// affecte un nom aux entrees sans nom
				if( input.GetName().equals("") )
					input.SetName(LocaleFIS.GetString("Input") + " " + String.valueOf(i+1));
				// affecte un nom aux SEF sans nom
				for( int j=0 ; j<input.GetNbMF() ; j++ )
				{
					JMF sef = input.GetMF(j);
					if( sef.GetName().equals("") )
						sef.SetName( LocaleFIS.GetString("MF") + String.valueOf(j+1) );
				}
			}

			// affichage des sorties
			this.jListOutputsModel = new DefaultListModel();
			this.jListOutputs.setModel(this.jListOutputsModel);
			for( int i=0 ; i<this.fis.NbOutputs() ; i++ )
			{
				this.jListOutputsModel.addElement(new String()); // on met une chaine vide dans le model de la liste, c'est le renderer de la liste qui affiche le nom de la sortie
				JOutput output = this.fis.GetOutput(i);
				// affecte un nom aux sorties sans nom
				if( output.GetName().equals("") )
					output.SetName(LocaleFIS.GetString("Output") + " " + String.valueOf(i+1));
				// affecte un nom aux SEF sans nom
				for( int j=0 ; j<output.GetNbMF() ; j++ )
				{
					JMF sef = output.GetMF(j);
					if( sef.GetName().equals("") )
						sef.SetName( LocaleFIS.GetString("MF") + String.valueOf(j+1) );
				}
			}

			// activation du bouton regles
			if( (this.fis.NbInputs() != 0) && (this.fis.NbOutputs() != 0) ) // bouton regle active si au moins une entree et une sortie
				this.jButtonRules.setEnabled(true);

			// listeners de FIS
			this.fis.RemoveFISListener(this.FISListener);
			this.FISListener = new FISListener()
			{
				public void InputActiveChanged( int input_number )   { JFISFrame.this.repaint(); }
				public void OutputActiveChanged( int output_number ) { JFISFrame.this.repaint(); }
				public void InputNameChanged( int input_number )     { JFISFrame.this.repaint(); }
				public void OutputNameChanged( int output_number )   { JFISFrame.this.repaint(); }
				public void FISClosed() {}
				public void InputReplaced( int input_number ) {}
				public void OutputReplaced( int output_number ) {}
				public void InputRangeChanged( int input_number ) {}
				public void OutputRangeChanged( int output_number ) {}
				public void InputRemoved( int input_number ) {}
				public void OutputRemoved( int output_number ) {}
				public void InputAdded( int input_number ) {}
				public void OutputAdded( int output_number ) {}
				public void MFRemovedInInput( int input_number ) {}
				public void MFRemovedInOutput( int output_number ) {}
				public void MFAddedInInput( int input_number ) {}
				public void MFAddedInOutput( int output_number ) {}
				public void MFReplacedInInput( int input_number ) {}
				public void MFReplacedInOutput( int output_number ) {}
				public void OutputDefuzChanged( int output_number ) {}
				public void OutputImpliChanged( int output_number ) {}
				public void OutputDisjChanged( int output_number ) {}
				public void OutputClassifChanged( int output_number ) {}
				public void OutputDefaultChanged( int output_number ) {}
				public void OutputAlarmThresChanged( int output_number ) {}
				public void RulesModified() {}
				public void ConjunctionChanged() {}
			};
			this.fis.AddFISListener(this.FISListener);
		}
	}

	//------------------------------------------------------------------------------
	// ouverture d'un nouveau SIF
	void jMenuNew_actionPerformed(ActionEvent e)
	{
		// ferme le SIF courant
		if( this.CloseFIS() == false )
			return;
		// ferme le fichier de donnees
		this.CloseDataFile();
		try
		{
			// ouverture du nouveau sif
			this.fis = new JFIS();
			this.fisHolder.setValue(this.fis);
			this.InitJFISFrameWithFIS();
		}
		catch( Throwable t )
		{
			MessageFIS.Error(null, t);
			if( this.fis != null ) this.fis.Delete();
			this.fis = null;
			this.fisHolder.setValue(this.fis);
			this.InitJFISFrame();
		}
	}
	//------------------------------------------------------------------------------
	void jMenuOpen_actionPerformed(ActionEvent e)
	{
		JFIS new_fis = null;
		JOpenFileChooser file_chooser = new JOpenFileChooser(MainFIS.config.FISFilePath());
		file_chooser.setAcceptAllFileFilterUsed(true);
		JFileFilter filter = new JFileFilter("fis", LocaleFIS.GetString("FilterFISFile"));
		//filter.addExtension("sd");
		//filter.addExtension("sif");
		file_chooser.addChoosableFileFilter(filter);

		// ajout de la case a cocher dans la fenetre d'ouverture
		JCheckBox jcb = new JCheckBox(LocaleFIS.GetString("OpenInNewWindow"), false);
		file_chooser.AddComponent(jcb);
		JCheckBox jcbcdf = new JCheckBox(LocaleFIS.GetString("CloseDataFile"), false);
		file_chooser.AddComponent(jcbcdf);
		JCheckBox jcbcover = new JCheckBox(LocaleFIS.GetString("EnsureOutputLimitInference"), false);
		file_chooser.AddComponent(jcbcover);

		if( file_chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION )
		{
			MainFIS.config.FISFilePath(file_chooser.getSelectedFile().getParent());
			if( jcb.isSelected() == true )  // ouverture du sif dans une nouvelle fenetre
			{
				try {				 
				JFISFrame jsf = new JFISFrame( new JFIS(file_chooser.getSelectedFile().getAbsolutePath(), jcbcover.isSelected()), null, this ); 
				
				} catch( Throwable t ) { MessageFIS.Error(null, t); } }
			else  // modification du sif courant
			{ 
				try
				{
					if( jcbcdf.isSelected() == true )
						this.CloseDataFile();
					// ouverture du nouveau sif
					new_fis = new JFIS( file_chooser.getSelectedFile().getAbsolutePath(), jcbcover.isSelected() );
					// verifie que le fichier de donnees a un nombre correct de colonnes
					if( this.DataFile != null )
						if( (this.DataFile.VariableCount() > (new_fis.NbInputs() + new_fis.NbOutputs())) || (this.DataFile.VariableCount() < new_fis.NbInputs()) )
							this.ThrowDataFileError(new_fis, this.DataFile);
					// fermeture du sif courant
					if( this.CloseFIS() == false )
						return;
					this.fis = new_fis;
					this.fisHolder.setValue(this.fis);
					this.InitJFISFrameWithFIS();
				}
				catch( Exception exception )
				{
					MessageFIS.Error(null, exception);
					if( new_fis != null ) new_fis.Delete();
					new_fis = null;
				}
				catch( Throwable t )
				{
					MessageFIS.Error(null, t);
					if( this.fis != null ) this.fis.Delete();
					if( new_fis != null ) new_fis.Delete();
					this.fis = null;
					this.fisHolder.setValue(this.fis);
					new_fis = null;
					this.InitJFISFrame();
				}
			}
		}
	}
	//------------------------------------------------------------------------------
	void jMenuExit_actionPerformed(ActionEvent e) { this.dispose(); }
	//------------------------------------------------------------------------------
	// ferme le SIF courant
	// retourne true si le SIF a ete ferme (SIF null ou SIF correctement ferme)
	// retourne false si le SIF n'a pas ete ferme (annulation ou exception)
	private boolean CloseFIS()
	{
		if( this.fis != null )
		{
			try
			{
				if( this.fis.Modified() == true )   // si le SIF a ete modifie
				{
					String inact_rules = new String(this.fis.NbRules() != this.fis.NbActiveRules() ? "\n" + LocaleFIS.GetString("OnlyActiveRulesWillBeSaved") : "");
					int confirm = JOptionPane.showConfirmDialog( this, LocaleFIS.GetString("FIS") + " " + this.fis.GetName() + " " + LocaleFIS.GetString("IsNotSaved") + "\n" + LocaleFIS.GetString("DoYouWantToSaveIt") + inact_rules, LocaleFIS.GetString("Confirmation"), JOptionPane.YES_NO_CANCEL_OPTION , JOptionPane.QUESTION_MESSAGE );
					if( confirm == JOptionPane.CANCEL_OPTION ) return false;
					if( confirm == JOptionPane.YES_OPTION )
					{
						// sauvegarde du SIF
						if( this.fis.FISFile == null )   // le SIF n'a pas encore ete sauvegarde
						{
							if( this.SaveAs() == false )      // ouverture de SaveAs
								return false;                   // abandon de SaveAs, le SIF n'a pas ete sauvegarde
						}
						else
							this.fis.Save(); // sauvegarde du SIF
					}

				}
				// suppression du SIF
				this.fis.Close();
				this.fis.Delete();
				this.fis = null;
				this.fisHolder.setValue(this.fis);
				this.InitJFISFrame();
			}
			catch( Throwable except )
			{
				MessageFIS.Error( null, except );
				return false;     // le SIF n'a pas ete sauvegarde
			}
		}
		return true;
	}
	//------------------------------------------------------------------------------
	private void CloseDataFile()
	{
		// fermeture du fichier de donnees
		if( this.DataFile != null )
			this.DataFile.Close();
		this.DataFile = null;
		this.dataHolder.setValue(this.DataFile);
		this.jTFDataFile.setText("");
	}
	//------------------------------------------------------------------------------
	public void dispose()
	{
		if( this.CloseFIS() == false )             // ferme le SIF courant
			return;
		this.CloseDataFile();                   // ferme le fichier de donnees
		this.JFISFrames.removeElement(this);    // retire la fenetre de la liste des fenetres JSIFFRAmes
		JFISFrame.RemoveTranslatable(this);     // retire la fenetre de la liste des fenetres traduisibles
		JFISFrame.RemoveJFrameLook(this);       // retire la fenetee de la liste des fenetres lookable
		super.dispose();                        // ferme la fenetre
		if( this.JFISFrames.size() == 0 )       // si c'est la derniere fenetre ouverte
			System.exit(0);                       // quitte l'application
	}
	//------------------------------------------------------------------------------
	void jRBMenuFrench_actionPerformed(ActionEvent e)
	{
		try
		{
			MainFIS.locale = new LocaleFIS( "fr", "FR" );
			MainFIS.config.Language("fr");
			MainFIS.config.Country("FR");
			JFISFrame.TranslateTranslatables();
		}
		catch( Throwable ex ) { MessageFIS.Error( null, ex ); }
	}
	//------------------------------------------------------------------------------
	void jRBMenuEnglish_actionPerformed(ActionEvent e)
	{
		try
		{
			MainFIS.locale = new LocaleFIS( "en", "GB" );
			MainFIS.config.Language("en");
			MainFIS.config.Country("GB");
			JFISFrame.TranslateTranslatables();
		}
		catch( Throwable ex ) { MessageFIS.Error( null, ex ); }
	}
	//------------------------------------------------------------------------------
	void jRBMenuSpanish_actionPerformed(ActionEvent e)
	{
		try
		{
			MainFIS.locale = new LocaleFIS( "es", "ES" );
			MainFIS.config.Language("es");
			MainFIS.config.Country("ES");
			JFISFrame.TranslateTranslatables();
		}
		catch( Throwable ex ) { MessageFIS.Error( null, ex ); }
	}
	//------------------------------------------------------------------------------
	void jRBMenuPortugues_actionPerformed(ActionEvent e)
	{
		try
		{
			MainFIS.locale = new LocaleFIS( "pt", "BR" );
			MainFIS.config.Language("pt");
			MainFIS.config.Country("BR");
			JFISFrame.TranslateTranslatables();
		}
		catch( Throwable ex ) { MessageFIS.Error( null, ex ); }
	}
	//------------------------------------------------------------------------------
	void jMenuClose_actionPerformed(ActionEvent e) { this.CloseFIS(); }
	//------------------------------------------------------------------------------
	void jMenuSave_actionPerformed(ActionEvent e)
	{
		try
		{
			if( this.fis.NbRules() != this.fis.NbActiveRules() )
				if( JOptionPane.showConfirmDialog( this, LocaleFIS.GetString("OnlyActiveRulesWillBeSaved"), LocaleFIS.GetString("Information"), JOptionPane.OK_CANCEL_OPTION , JOptionPane.INFORMATION_MESSAGE ) == JOptionPane.CANCEL_OPTION )
					return;
			this.fis.Save();
		}
		catch( Throwable except ) { MessageFIS.Error( null, except ); }
	}
	//------------------------------------------------------------------------------
	private boolean SaveAs() throws Throwable
	{
		if( this.fis.NbRules() != this.fis.NbActiveRules() )
			if( JOptionPane.showConfirmDialog( this, LocaleFIS.GetString("OnlyActiveRulesWillBeSaved"), LocaleFIS.GetString("Information"), JOptionPane.OK_CANCEL_OPTION , JOptionPane.INFORMATION_MESSAGE ) == JOptionPane.CANCEL_OPTION )
				return false;
		JFileChooser file_chooser = new JFileChooser(MainFIS.config.FISFilePath())
		{
			protected JDialog createDialog(Component parent) throws HeadlessException
			{
				JDialog dialog = super.createDialog(parent);
				dialog.setResizable(false);
				return dialog;
			}
		};
		file_chooser.setAcceptAllFileFilterUsed(false);
		JFileFilter filter = new JFileFilter("fis", LocaleFIS.GetString("FilterFISFile"));
		file_chooser.addChoosableFileFilter(filter);
		file_chooser.setSelectedFile(new File(MainFIS.config.FISFilePath() + System.getProperty("file.separator") + this.fis.GetName().toLowerCase() + ".fis"));
		if( file_chooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION )
		{
			String file_name = file_chooser.getSelectedFile().getAbsolutePath();
			// ajoute l'extension si necessaire
			if( file_name.lastIndexOf('.') == -1 )
				file_name += ".fis";

			// teste si le fichier existe
			File f = new File( file_name );
			if( f.exists() )
			{
				String message = LocaleFIS.GetString("TheFISFile") + " : " + file_name + " " + LocaleFIS.GetString("AlreadyExist") + "\n" + LocaleFIS.GetString("DoYouWantToReplaceIt");
				if( MessageFIS.Confirm(this, message) == JOptionPane.NO_OPTION )
					return false;
			}

			try
			{
				MainFIS.config.FISFilePath(file_chooser.getSelectedFile().getParent());
				fis.Save(file_name);
				this.TranslateTitle();
				return true;
			}
			catch( Throwable t ) { MessageFIS.Error( null, t ); }
		}
		return false;
	}
	//------------------------------------------------------------------------------
	void jMenuSaveAs_actionPerformed(ActionEvent e)
	{
		try { SaveAs(); }
		catch( Throwable t ) { MessageFIS.Error( null, t ); }
	}
	//------------------------------------------------------------------------------
	void jMenuLook_menuSelected(MenuEvent e)
	{
		final JFISFrame adaptee = this;

		// groupe de bouton
		ButtonGroup groupLook = new ButtonGroup();
		// pour chaque style
		for( int i=0 ; i<MainFIS.look.Length() ; i++ )
		{
			// creation d'un menu look
			JRadioButtonMenuItemLook m = new JRadioButtonMenuItemLook( MainFIS.look.LookAndFeelInfo(i) );

			groupLook.add(m);
			m.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					JRadioButtonMenuItemLook source = (JRadioButtonMenuItemLook)(e.getSource());
					// modifie le style des fenetres ouvertes
					for( int i=0 ; i<adaptee.JFramesLook.size() ; i++ )
						MainFIS.look.ChangeLook((Component)adaptee.JFramesLook.elementAt(i), source.class_name);
					// sauve le style actuel dans la config
					MainFIS.config.LookAndFeel(source.class_name);
				}
			} );
			jMenuLook.add(m);
		}
	}
	//------------------------------------------------------------------------------
	void jMenuLook_menuDeselected(MenuEvent e) { jMenuLook.removeAll(); }
	//------------------------------------------------------------------------------
	void jMenuLanguage_menuSelected(MenuEvent e)
	{
		jRBMenuFrench.setSelected(false);
		jRBMenuEnglish.setSelected(false);
		jRBMenuSpanish.setSelected(false);
		jRBMenuPortugues.setSelected(false);
		if( MainFIS.locale.Language().equals("fr") )
			jRBMenuFrench.setSelected(true);
		if( MainFIS.locale.Language().equals("en") )
			jRBMenuEnglish.setSelected(true);
		if( MainFIS.locale.Language().equals("es") )
			jRBMenuSpanish.setSelected(true);
		if( MainFIS.locale.Language().equals("pt") )
			jRBMenuPortugues.setSelected(true);
	}
	//------------------------------------------------------------------------------
	void jPopupNewInput_actionPerformed(ActionEvent e) { this.NewInput(); }
	//------------------------------------------------------------------------------
	private void EditInput()
	{
		try
		{
			JInputFrame previous = null;
			int [] selected = this.jListInputs.getSelectedIndices();
			for( int i=0 ; i<selected.length ; i++ )
			{
				JInputFrame jif = new JInputFrame(this, this.fis, selected[i] );
				jif.Show(previous);
				previous = jif;
			}
		}
		catch( Throwable t ) { MessageFIS.Error( null, t ); }
	}
	//------------------------------------------------------------------------------
	void jPopupEditInput_actionPerformed(ActionEvent e) { this.EditInput(); }
	//------------------------------------------------------------------------------
	private void RemoveInput()
	{
		try
		{
			int [] selected = this.jListInputs.getSelectedIndices();
			for( int i=selected.length-1 ; i>=0 ; i-- )
				this.fis.RemoveInput(selected[i]);
			this.InitJFISFrameWithFIS();
		}
		catch( Throwable en ) { MessageFIS.Error( null, en ); }
	}
	//------------------------------------------------------------------------------
	void jPopupRemoveInput_actionPerformed(ActionEvent e) { this.RemoveInput(); }
	//------------------------------------------------------------------------------
	void jTFName_actionPerformed(ActionEvent e)
	{
		try { this.fis.SetName(this.jTFName.getText()); }
		catch( Throwable except ) { MessageFIS.Error( null, except ); }
	}
	//------------------------------------------------------------------------------
	void jTFName_focusLost(FocusEvent e)
	{
		try { if( this.fis!= null ) this.fis.SetName(this.jTFName.getText()); }
		catch( Throwable except ) { MessageFIS.Error( null, except ); }
	}
	//------------------------------------------------------------------------------
	void jCBConjunction_itemStateChanged(ItemEvent e)
	{
		if( e.getStateChange() == ItemEvent.SELECTED )
			if( this.fis != null )
			{
				try { this.fis.SetConjunction(this.jCBConjunction.getSelectedIndex()); }
				catch( Throwable except ) { MessageFIS.Error( null, except ); }
			}
	}
	//------------------------------------------------------------------------------
	private void NewOutput()
	{
		try
		{
			// creation d'une nouvelle sortie
			JOutput new_output = new JOutputFloue();  // sortie floue par defaut
			// numerotation de la nouvelle sortie en fonction du nombre de sorties actuellement dans le sif
			new_output.SetName(LocaleFIS.GetString("Output") + " " + String.valueOf(this.fis.NbOutputs()+1));
			new_output.AddMF(new JMFUniversal(LocaleFIS.GetString("MF")+ "1",new_output.GetRange()));
			this.fis.AddOutput( new_output );
			this.InitJFISFrameWithFIS();
			// ouverture d'une fenetre JOutputFrame avec la derniere sortie cree
			JOutputFrame jof = new JOutputFrame(this, this.fis, this.fis.NbOutputs()-1);
			jof.Show(null);
		}
		catch( Throwable en ) { MessageFIS.Error( null, en ); }
	}
	//------------------------------------------------------------------------------
	void jPopupNewOutput_actionPerformed(ActionEvent e) { this.NewOutput(); }
	//------------------------------------------------------------------------------
	private void RemoveOutput()
	{
		try
		{
			int [] selected = this.jListOutputs.getSelectedIndices();
			for( int i=selected.length-1 ; i>=0 ; i-- )
				this.fis.RemoveOutput(selected[i]);
			this.InitJFISFrameWithFIS();
		}
		catch( Throwable en ) { MessageFIS.Error( null, en ); }
	}
	//------------------------------------------------------------------------------
	void jPopupRemoveOutput_actionPerformed(ActionEvent e) { this.RemoveOutput(); }
	//------------------------------------------------------------------------------
	private void EditOutput()
	{
		try
		{
			JInputFrame previous = null;
			int [] selected = this.jListOutputs.getSelectedIndices();
			for( int i=0 ; i<selected.length ; i++ )
			{
				JOutputFrame jof = new JOutputFrame(this, this.fis, selected[i] );
				jof.Show(previous);
				previous = jof;
			}
		}
		catch( Throwable en ) { MessageFIS.Error( null, en ); }
	}
	//------------------------------------------------------------------------------
	void jPopupEditOutput_actionPerformed(ActionEvent e) { this.EditOutput(); }
	//------------------------------------------------------------------------------
	void jButtonRules_actionPerformed(ActionEvent e)
	{
		try
		{
			if( this.fis.NbRules() > this.WarningNumberOfRules )
			{
				MessageFIS.Information(this, LocaleFIS.GetString("TooManyRules"));
				return;
			}
			if( this.JRuleFrameOpen.Permits() == 1 )
				this.jRuleFrame = new JRuleFrame( this, this.fis, this.JRuleFrameOpen );
			this.jRuleFrame.setVisible(true);
		}
		catch( Throwable en ) { MessageFIS.Error( null, en ); }
	}
	//------------------------------------------------------------------------------
	void jMenuDataView_actionPerformed(ActionEvent e)
	{
		DataChooserFrame dcf = new DataChooserFrame(this, this.fis, this.DataFile);
	}
	//------------------------------------------------------------------------------
	void jMenuDataTable_actionPerformed(ActionEvent e)
	{
		JDataTableFrame dtf = (JDataTableFrame)JExtendedDataFile.jhtTableFrames.get(this.DataFile.FileName()); //the jhtTableFrames hash table is used to check if this data table frame is already opened
		if( dtf == null ) //if not, it is created
		{
			dtf = new JDataTableFrame( this, this.fis, this.DataFile);
			JExtendedDataFile.jhtTableFrames.put(this.DataFile.FileName(), dtf);
		}
		dtf.setVisible(true);   //in both cases, the data table frame is shown
	}
	//------------------------------------------------------------------------------
	void jMenuDataInfer_actionPerformed(ActionEvent e)
	{
		if( this.JPerformanceFrameOpen.Permits() == 1 )
			this.jPerformanceFrame = new JPerformanceFrame(this, this.fis, this.DataFile, this.JPerformanceFrameOpen);
		this.jPerformanceFrame.setVisible(true);
	}
	//------------------------------------------------------------------------------
	void jMenuDataLinks_actionPerformed(ActionEvent e)
	{
		try { JLinksFrame jlf = new JLinksFrame(this, this.fis, this.DataFile); }
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	void this_windowClosing(WindowEvent e)
	{
		// demande confirmation pour fermer la fenetre quand pas de sif ouvert ou sif non modifie
		// la fonction closeFIS() dans dispose() demandera confirmation de sauvegarde du sif s'il a ete modifie
		try
		{
			if( this.fis == null ? true : !this.fis.Modified() )
			{
				if( MessageFIS.Confirm(this, LocaleFIS.GetString("Exit") + "?") == JOptionPane.NO_OPTION )
					return;
			}
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); return; }
		this.dispose();
	}
	//------------------------------------------------------------------------------
	void jMenuFIS_menuSelected(MenuEvent e)
	{
		if( this.fis == null )  // il n'y a pas de SIF ouvert
		{
			this.jMenuClose.setEnabled(false);
			this.jMenuSave.setEnabled(false);
			this.jMenuSaveAs.setEnabled(false);
			this.jMenuInputs.setEnabled(false);
			this.jMenuOutputs.setEnabled(false);
			this.jMenuInfere.setEnabled(false);
			this.jMenuSystemResponse.setEnabled(false);
			this.jMenu2DInfer.setEnabled(false);
			this.jMenu3DInfer.setEnabled(false);
			this.jMenuGenerateRules.setEnabled(false);
			this.jMenuGenerateConcs.setEnabled(false);
		}
		else  // un SIF est ouvert
		{
			this.jMenuClose.setEnabled(true);
			if( this.fis.FISFile == null )      // le SIF n'est pas associe a un fichier
				this.jMenuSave.setEnabled(false);
			else
				this.jMenuSave.setEnabled(true);
			this.jMenuSaveAs.setEnabled(true);
			this.jMenuInputs.setEnabled(true);
			this.jMenuOutputs.setEnabled(true);
			this.jMenuInfere.setEnabled(false);
			this.jMenuSystemResponse.setEnabled(false);
			this.jMenu2DInfer.setEnabled(false);
			this.jMenu3DInfer.setEnabled(false);
			this.jMenuGenerateRules.setEnabled(false);
			this.jMenuGenerateConcs.setEnabled(false);
			try
			{
				if( this.fis.NbActiveRules() != 0 )  // active le menu Infere et Generer conclusions si au moins une regle active
				{
					this.jMenuInfere.setEnabled(true);
					if( this.fis.NbActiveInputs() >= 1 ) {
						this.jMenuSystemResponse.setEnabled(true);
						this.jMenu2DInfer.setEnabled(true);
					}
					if( this.fis.NbActiveInputs() >= 2 )
						this.jMenu3DInfer.setEnabled(true);
					this.jMenuGenerateConcs.setEnabled(this.DataFile != null ? true: false);
				}
				if ( (this.fis.NbActiveInputs() != 0) && (this.fis.NbActiveOutputs() != 0)) // active le menu generer regle si au moins 1 entree et sortie
					this.jMenuGenerateRules.setEnabled(true);
				else
					this.jMenuGenerateRules.setEnabled(false);
			}
			catch( Throwable t ) { MessageFIS.Error(null, t); }
		}
		this.jMenuHFPSR1.setEnabled(this.DataFile != null ? true: false);
	}
	//------------------------------------------------------------------------------
	void jMenuData_menuSelected(MenuEvent e)
	{
		if( this.DataFile == null ) // il n'y a pas de fichier Data d'ouvert
		{
			this.jMenuDataClose.setEnabled(false);
			this.jMenuDataGenerateSample.setEnabled(false);
			this.jMenuDataView.setEnabled(false);
			this.jMenuDataTable.setEnabled(false);
			this.jMenuDataInfer.setEnabled(false);
			this.jMenuDataLinks.setEnabled(false);
			this.jMenuDataDistance.setEnabled(false);
		}
		else  // un fichier Data est ouvert
		{
			this.jMenuDataClose.setEnabled(true);
			this.jMenuDataGenerateSample.setEnabled(true);
			this.jMenuDataView.setEnabled(true);
			this.jMenuDataTable.setEnabled(true);
			// si aucun sif d'ouvert
			if( this.fis == null )
			{
				this.jMenuDataInfer.setEnabled(false);
				this.jMenuDataLinks.setEnabled(false);
				this.jMenuDataDistance.setEnabled(false);
			}
			else
			{
				try
				{
					this.jMenuDataInfer.setEnabled(false);
					if( (this.fis.NbActiveRules() != 0) && (this.fis.NbActiveInputs() != 0) && (this.fis.NbActiveOutputs() != 0) )
						this.jMenuDataInfer.setEnabled(true); // infere si regles, entrees et sorties actives
				}
				catch( Throwable t ) { MessageFIS.Error(null, t); }
				this.jMenuDataLinks.setEnabled(true);
				this.jMenuDataDistance .setEnabled(true);
			}
		}
		// fenetre inference est ouverte: on interdit l'ouverture ou la fermeture du fichier de donnees
		if( this.JInferenceFrameOpen.Permits() == 0 )
		{
			this.jMenuDataOpen.setEnabled(false);
			this.jMenuDataClose.setEnabled(false);
		}
		else
			this.jMenuDataOpen.setEnabled(true);
	}
	//------------------------------------------------------------------------------
	void jMenuInputs_menuSelected(MenuEvent e)
	{
		// on ne peut pas ajouter une entree si un fichier de donnees est ouvert
		if( this.DataFile != null )
			this.jMenuNewInput.setEnabled(false);
		else
			this.jMenuNewInput.setEnabled(true);
		// aucune entree n'est selectionnee
		if( this.jListInputs.getSelectedIndices().length == 0 )
		{
			this.jMenuEditInput.setEnabled(false);
			this.jMenuRemoveInput.setEnabled(false);
		}
		else
		{
			this.jMenuEditInput.setEnabled(true);
			if( this.DataFile != null )
				this.jMenuRemoveInput.setEnabled(false);  // on ne peut pas supprimer une entree si un fichier de donnees est ouvert
			else
				this.jMenuRemoveInput.setEnabled(true);
		}
	}
	//------------------------------------------------------------------------------
	void jMenuOutputs_menuSelected(MenuEvent e)
	{
		// on ne peut pas ajouter une sortie si un fichier de donnees est ouvert
		if( this.DataFile != null )
			this.jMenuNewOutput.setEnabled(false);
		else
			this.jMenuNewOutput.setEnabled(true);

		// aucune sortie n'est selectionnee
		if( this.jListOutputs.getSelectedIndices().length == 0 )
		{
			this.jMenuEditOutput.setEnabled(false);
			this.jMenuRemoveOutput.setEnabled(false);
		}
		else
		{
			this.jMenuEditOutput.setEnabled(true);
			if( this.DataFile != null )
				this.jMenuRemoveOutput.setEnabled(false); // on ne peut pas supprimer une sortie si un fichier de donnees est ouvert
			else
				this.jMenuRemoveOutput.setEnabled(true);
		}
	}
	//------------------------------------------------------------------------------
	void jMenuInfere_actionPerformed(ActionEvent e)
	{
		this.showInferenceFrame() ;
	}
	//------------------------------------------------------------------------------
	void jMenu2DInfer_actionPerformed(ActionEvent e)
	{
		try {
			(new JFrame2DInfer(this)).show(this.fis);
		} catch (Throwable e1) {
			MessageFIS.Error( null, e1 );
		}
	}
	//------------------------------------------------------------------------------
	void jMenu3DInfer_actionPerformed(ActionEvent e)
	{
		try {
			if(MainFIS.isJava3DIsInstalled())
				(new JFrame3DInfer(this)).show(this.fis);
		} 
		catch (Throwable e1) {
			MessageFIS.Error( null, e1 );
		}
	}
	//------------------------------------------------------------------------------
	private void NewInput()
	{
		try
		{
			// creation d'une nouvelle entree
			JInput new_input = new JInput();
			// numerotation de la nouvelle entree en fonction du nombre d'entree actuellement dans le sif
			new_input.SetName(LocaleFIS.GetString("Input") + " " + String.valueOf(this.fis.NbInputs()+1));
			this.fis.AddInput( new_input );
			this.InitJFISFrameWithFIS();
			// ouverture d'une fenetre JInputFrame avec la derniere entree cree
			JInputFrame jif = new JInputFrame(this, this.fis, this.fis.NbInputs()-1);
			jif.Show(null);
		}
		catch( Throwable en ) { MessageFIS.Error( null, en ); }
	}
	//------------------------------------------------------------------------------
	void jMenuNewInput_actionPerformed(ActionEvent e) { this.NewInput(); }
	//------------------------------------------------------------------------------
	void jListInputs_mousePressed(MouseEvent e)
	{
		if( this.jListInputs.isEnabled() == false )   // return si jListInputs est disable
			return;

		if (SwingUtilities.isRightMouseButton(e))     // le bouton droit a ete presse
		{
			// on ne peut pas ajouter une entree si un fichier de donnees est ouvert
			if( this.DataFile != null )
				this.jPopupNewInput.setEnabled(false);
			else
				this.jPopupNewInput.setEnabled(true);

			// aucune entree n'est selectionnee
			if( this.jListInputs.getSelectedIndices().length == 0 )
			{
				this.jPopupEditInput.setEnabled(false);
				this.jPopupRemoveInput.setEnabled(false);
			}
			else
			{
				this.jPopupEditInput.setEnabled(true);
				if( this.DataFile != null )
					this.jPopupRemoveInput.setEnabled(false); // on ne peut pas supprimer une entree si un fichier de donnees est ouvert
				else
					this.jPopupRemoveInput.setEnabled(true);
			}
			SwingUtilities.updateComponentTreeUI(this.jPopupMenuInputs);   // affecte le look actuel au composant jPopupMenuInputs
			jPopupMenuInputs.show(this.jListInputs, e.getX(), e.getY());   // affichage du popup
		}
		// double click
		if( SwingUtilities.isLeftMouseButton(e) && (e.getClickCount() == 2) && (jListInputs.isSelectionEmpty() == false) )
			this.EditInput();
	}
	//------------------------------------------------------------------------------
	void jListOutputs_mousePressed(MouseEvent e)
	{
		if( this.jListOutputs.isEnabled() == false )   // return si jListOutputs est disable
			return;

		if (SwingUtilities.isRightMouseButton(e))     // le bouton droit a ete presse
		{
			// on ne peut pas ajouter une sortie si un fichier de donnees est ouvert
			if( this.DataFile != null )
				this.jPopupNewOutput.setEnabled(false);
			else
				this.jPopupNewOutput.setEnabled(true);

			// aucune sortie n'est selectionnee
			if( this.jListOutputs.getSelectedIndices().length == 0 )
			{
				this.jPopupEditOutput.setEnabled(false);
				this.jPopupRemoveOutput.setEnabled(false);
			}
			else
			{
				this.jPopupEditOutput.setEnabled(true);
				if( this.DataFile != null )
					this.jPopupRemoveOutput.setEnabled(false); // on ne peut pas supprimer une sortie si un fichier de donnees est ouvert
				else
					this.jPopupRemoveOutput.setEnabled(true);
			}
			SwingUtilities.updateComponentTreeUI(this.jPopupMenuOutputs);    // affecte le look actuel au composant jPopupMenuOutputs
			jPopupMenuOutputs.show(this.jListOutputs, e.getX(), e.getY());   // affichage du popup
		}
		// double click
		if( SwingUtilities.isLeftMouseButton(e) && (e.getClickCount() == 2) && (jListOutputs.isSelectionEmpty() == false) )
			this.EditOutput();
	}
	//------------------------------------------------------------------------------
	void jMenuEditInput_actionPerformed(ActionEvent e) { this.EditInput(); }
	//------------------------------------------------------------------------------
	void this_windowActivated(WindowEvent e) { this.repaint(); }
	//------------------------------------------------------------------------------
	void jMenuRemoveInput_actionPerformed(ActionEvent e) { this.RemoveInput(); }
	//------------------------------------------------------------------------------
	void jMenuNewOutput_actionPerformed(ActionEvent e) { this.NewOutput(); }
	//------------------------------------------------------------------------------
	void jMenuEditOutput_actionPerformed(ActionEvent e) { this.EditOutput(); }
	//------------------------------------------------------------------------------
	void jMenuRemoveOutput_actionPerformed(ActionEvent e) { this.RemoveOutput(); }
	//------------------------------------------------------------------------------
	public static void AddJFrameLook( Component frame )  { JFISFrame.JFramesLook.add(frame); }
	//------------------------------------------------------------------------------
	public static void RemoveJFrameLook( Component frame )  { JFISFrame.JFramesLook.remove(frame); }
	//------------------------------------------------------------------------------
	public static void AddTranslatable( Translatable t ) { JFISFrame.Translatables.add(t); }
	//------------------------------------------------------------------------------
	public static void RemoveTranslatable( Translatable t ) { JFISFrame.Translatables.remove(t); }
	//------------------------------------------------------------------------------
	private static void TranslateTranslatables() throws Throwable
	{
		for( int i=0 ; i<JFISFrame.Translatables.size() ; i++ )
			((Translatable)JFISFrame.Translatables.elementAt(i)).Translate();
	}
	//------------------------------------------------------------------------------
	void jMenuDataOpen_actionPerformed(ActionEvent e)
	{
		try
		{
			JOpenFileChooser file_chooser = new JOpenFileChooser(MainFIS.config.DataFilePath());

			// ajout du panel d'option d'ouverture
			JPanel jp = new JPanel(new GridBagLayout());
			JCheckBox jcbActiveAllItems = new JCheckBox(LocaleFIS.GetString("ActivateAllItems"));
			jcbActiveAllItems.setSelected(true);
			jp.add(jcbActiveAllItems, new GridBagConstraints(0, 0, 2, 1, 1.0, 0.0
					,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
			file_chooser.AddComponent(jp);

			if( file_chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION )
			{
				MainFIS.config.DataFilePath(file_chooser.getSelectedFile().getParent());
				// ouverture du fichier de donnees
				JExtendedDataFile new_data_file = JExtendedDataFile.Open(file_chooser.getSelectedFile().getAbsolutePath(), jcbActiveAllItems.isSelected());
				// verifie que le fichier de donnees a un nombre correct de colonnes
				if( this.fis != null )
					if( (new_data_file.VariableCount() > (this.fis.NbInputs() + this.fis.NbOutputs())) || (new_data_file.VariableCount() < this.fis.NbInputs()) )
						this.ThrowDataFileError(this.fis, new_data_file);
				// remplacement du fichier de donnees actuel
				if( this.DataFile != null )
					this.DataFile.Close();
				this.DataFile = new_data_file;
				this.dataHolder.setValue(this.DataFile);
				this.jTFDataFile.setText(this.DataFile.FileName());      
			}
		}
		catch( Throwable t ) { MessageFIS.Error(null, t);}
	}
	//------------------------------------------------------------------------------
	private void ThrowDataFileError( JFIS fis, JDataFile data_file ) throws Throwable
	{
		String message = LocaleFIS.GetString("UnexpectedNumberOfColumnInDataFile") + ": " + data_file.FileName();
		message += "\n" + Integer.toString(data_file.VariableCount()) + " " + LocaleFIS.GetString("ColumnsInDataFile");
		message += "\n" + Integer.toString(fis.NbInputs()) + " " + LocaleFIS.GetString("Inputs").toLowerCase() + " " + LocaleFIS.GetString("AND").toLowerCase() + " "+ Integer.toString(fis.NbOutputs()) + " " + LocaleFIS.GetString("Outputs").toLowerCase() + " " + LocaleFIS.GetString("InFIS");
		throw new Exception(message);
	}
	//------------------------------------------------------------------------------
	void jMenuDataClose_actionPerformed(ActionEvent e)
	{
		this.DataFile.Close();
		this.DataFile = null;
		this.dataHolder.setValue(this.DataFile);
		this.jTFDataFile.setText("");
	}
	//------------------------------------------------------------------------------
	/*void jMenuDataGenerateSample1_actionPerformed(ActionEvent e)
	{
		JPanelGenerateSample1 panel = new JPanelGenerateSample1(this.DataFile, this.fis);
		if( JOptionPane.showOptionDialog(this, panel, LocaleFIS.GetString("GenerateSample"), JOptionPane.YES_OPTION | JOptionPane.CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE, null, null, null) == JOptionPane.YES_OPTION ) {
			try {
				jnifis.GenerateSample(this.DataFile.FileName(), panel.getSampleNumber(), panel.getValidationDataRatio(), panel.getSeed(), panel.getClassif(), panel.getColumnNumber(), panel.getPrecision(),panel.getReplace(),panel.getTruncate());
			} catch (Throwable e1) {
				MessageFIS.Error("", e1);
			}
		}
	}*/
	//------------------------------------------------------------------------------
	void jMenuDataGenerateSample1_actionPerformed(ActionEvent e)
	{
		PairSampleGeneratorModel pairSampleGenerator = new PairSampleGeneratorModel(DataFile, fis);
		PairSampleGeneratorAdapter pairSampleGeneratorAdapter = new PairSampleGeneratorAdapter(pairSampleGenerator, DataFile, fis);
		JPanelPairSampleGenerator panelPairSampleGenerator = new JPanelPairSampleGenerator();
		panelPairSampleGenerator.bind(pairSampleGeneratorAdapter);
		ValidationResultAdapter validationResultAdapter = new ValidationResultAdapter(panelPairSampleGenerator, pairSampleGeneratorAdapter);
		showOptionDialog(getWindowAncestor(e.getSource()), panelPairSampleGenerator, LocaleFIS.GetString("GenerateSamplePairs"), OK_CANCEL_OPTION, PLAIN_MESSAGE, null, pairSampleGeneratorAdapter.getOptions(), pairSampleGeneratorAdapter.getInitialOption());
		validationResultAdapter.release();
		pairSampleGeneratorAdapter.release();
		pairSampleGenerator.release();
		/*System.out.println("listeners " + pairSampleGenerator.getPropertyChangeListeners().length);
		System.out.println("listeners " + pairSampleGenerator.getPairSample().getPropertyChangeListeners().length);
		System.out.println("listeners " + pairSampleGenerator.getClassifSample().getPropertyChangeListeners().length);
		System.out.println("listeners " + pairSampleGenerator.getSeed().getPropertyChangeListeners().length);
*/	}
	//------------------------------------------------------------------------------
	/*void jMenuDataGenerateSample2_actionPerformed(ActionEvent e)
	{
		JPanelGenerateSample2 panel = new JPanelGenerateSample2(this.DataFile, this.fis);
		if( JOptionPane.showOptionDialog(this, panel, LocaleFIS.GetString("GenerateSample"), JOptionPane.YES_OPTION | JOptionPane.CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE, null, null, null) == JOptionPane.YES_OPTION ) {
			try {
				jnifis.GenerateSample(this.DataFile.FileName(), panel.getSampleNumber(), panel.getValidationDataRatio(), panel.getSeed(), panel.getClassif(), panel.getColumnNumber(), panel.getPrecision(),panel.getReplace(),panel.getTruncate());
			} catch (Throwable e1) {
				MessageFIS.Error("", e1);
			}
		}
	}*/
	//------------------------------------------------------------------------------
	void jMenuDataGenerateSample2_actionPerformed(ActionEvent e)
	{
		BlockSampleGeneratorModel blockSampleGenerator = new BlockSampleGeneratorModel(DataFile, fis);
		BlockSampleGeneratorAdapter blockSampleGeneratorAdapter = new BlockSampleGeneratorAdapter(blockSampleGenerator, DataFile, fis);
		JPanelBlockSampleGenerator panelBlockSampleGenerator = new JPanelBlockSampleGenerator();
		panelBlockSampleGenerator.bind(blockSampleGeneratorAdapter);
		ValidationResultAdapter validationResultAdapter = new ValidationResultAdapter(panelBlockSampleGenerator, blockSampleGeneratorAdapter);
		showOptionDialog(getWindowAncestor(e.getSource()), panelBlockSampleGenerator, LocaleFIS.GetString("GenerateSampleBlocks"), OK_CANCEL_OPTION, PLAIN_MESSAGE, null, blockSampleGeneratorAdapter.getOptions(), blockSampleGeneratorAdapter.getInitialOption());
		validationResultAdapter.release();
		blockSampleGeneratorAdapter.release();
		blockSampleGenerator.release();
		/*System.out.println("listeners " + blockSampleGenerator.getPropertyChangeListeners().length);
		System.out.println("listeners " + blockSampleGenerator.getBlockSample().getPropertyChangeListeners().length);
		System.out.println("listeners " + blockSampleGenerator.getClassifSample().getPropertyChangeListeners().length);
		System.out.println("listeners " + blockSampleGenerator.getSeed().getPropertyChangeListeners().length);
*/	}
	//------------------------------------------------------------------------------
	void jMenuTree_menuSelected(MenuEvent e)
	{
		try { this.jMenuTreeGenerate.setEnabled( (this.DataFile != null) && (this.fis != null) && (this.fis.NbActiveOutputs() != 0) && (this.fis.NbActiveInputs() != 0));  }
		catch (Throwable t) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	void jMenuLearningRules_menuSelected(MenuEvent e)
	{
		try
		{
			this.jMenuFPAInduction.setEnabled( (this.fis != null) && (this.DataFile != null) && (this.fis.NbActiveOutputs() != 0) && (this.fis.NbActiveInputs() != 0) );
			this.jMenuWangMendelInduction.setEnabled( (this.fis != null) && (this.DataFile != null) && (this.fis.NbActiveOutputs() != 0) && (this.fis.NbActiveInputs() != 0)  );
			this.jMenuOLS.setEnabled(this.DataFile != null);
			this.jMenuHFPGenerateFIS.setEnabled(this.DataFile != null);
		}
		catch (Throwable t) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	void jMenuLearning_menuSelected(MenuEvent e)
	{
		try
		{
			this.jMenuLearningPartition.setEnabled(this.DataFile != null);
			this.jMenuSimple.setEnabled((this.fis != null) && (this.DataFile != null) && (this.fis.NbActiveRules() != 0) && (this.fis.NbActiveOutputs() != 0)  && (this.fis.NbActiveInputs() != 0) );
			this.jMenuOptimization.setEnabled((this.fis != null) && (this.DataFile != null) && (this.fis.NbActiveRules() != 0) && (this.fis.NbActiveOutputs() != 0)  && (this.fis.NbActiveInputs() != 0)  );
		}
		catch (Throwable t) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	void jMenuFPAInduction_actionPerformed(ActionEvent e)
	{
		try
		{
			// verifie que le fichier de donnees a un nombre correct de colonnes
			if( this.DataFile.VariableCount() != (this.fis.NbInputs() + this.fis.NbOutputs()) )
				this.ThrowDataFileError(this.fis, this.DataFile);

			JPanelFPA panel = new JPanelFPA();
			if( JOptionPane.showOptionDialog(this, panel, "FPA", JOptionPane.YES_OPTION | JOptionPane.CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE, null, null, null) == JOptionPane.YES_OPTION ) {
				JFIS gen_fis = this.fis;
				if(this.fis.NbRules() == 0)
					gen_fis = new JGENFIS(this.fis, this.DataFile, true, panel.getMinDegree() );
				new JFISFrame(new JFISFPA(gen_fis, this.DataFile, panel.getStrategy(), panel.getMinCardinality(), panel.getMinDegree() ), this.DataFile, this);
			}
		}
		catch (Throwable t) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	void jMenuTreeTable_actionPerformed(ActionEvent e)
	{
		JOpenFileChooser file_chooser = new JOpenFileChooser(MainFIS.config.FISFilePath());
		file_chooser.setAcceptAllFileFilterUsed(true);
		JFileFilter filter = new JFileFilter(new String("tree"), LocaleFIS.GetString("FilterTreeFile"));
		filter.addExtension("pruned");
		file_chooser.addChoosableFileFilter(filter);
		if( file_chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION )
		{
			MainFIS.config.FISFilePath(file_chooser.getSelectedFile().getParent());
			JFuzzyTreeFrame jftf = new JFuzzyTreeFrame( this, file_chooser.getSelectedFile().getAbsolutePath() );
		}
	}
	//------------------------------------------------------------------------------
	void jMenuTreeGraphe_actionPerformed(ActionEvent e)
	{
		JOpenFileChooser file_chooser = new JOpenFileChooser(MainFIS.config.FISFilePath());
		file_chooser.setAcceptAllFileFilterUsed(true);
		JFileFilter filter = new JFileFilter(new String("tree"), LocaleFIS.GetString("FilterTreeFile"));
		filter.addExtension("pruned");
		file_chooser.addChoosableFileFilter(filter);
		if( file_chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION )
		{
			MainFIS.config.FISFilePath(file_chooser.getSelectedFile().getParent());
			JFuzzyGraphFrame jftf = new JFuzzyGraphFrame( this, file_chooser.getSelectedFile().getAbsolutePath() );

		}
	}
	//------------------------------------------------------------------------------
	void jMenuTreeGenerate_actionPerformed(ActionEvent e)
	{
		try
		{
			// verifie que le fichier de donnees a un nombre correct de colonnes
			if( this.DataFile.VariableCount() != (this.fis.NbInputs() + this.fis.NbOutputs()) )
				this.ThrowDataFileError(this.fis, this.DataFile);
			JTreeGenerateFrame jtgf = new JTreeGenerateFrame(this, this.fis, this.DataFile);
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	void jMenuGenerateConcs_actionPerformed(ActionEvent e) { 
		JFIS newFIS = null;
		JPanelGenerateConclusions panel = new JPanelGenerateConclusions();
		if( JOptionPane.showOptionDialog(this, panel, LocaleFIS.GetString("GenerateConcs"), JOptionPane.YES_OPTION | JOptionPane.CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE, null, null, null) == JOptionPane.YES_OPTION ) {
			try {
				if( panel.getMethod() == JPanelGenerateConclusions.FPA) {
					newFIS = new JFISFPA(this.fis, this.DataFile, 1, 1, 0 );
					if( panel.getReduceVocabulary() )
						for (int i=0 ; i<newFIS.NbOutputs() ; i++)
							newFIS.outputVocabularyReduction(this.DataFile, i, panel.getJPanelOutputVocabularyReduction().getBlankThresh(), panel.getJPanelOutputVocabularyReduction().getDegradationAllowed(), panel.getJPanelOutputVocabularyReduction().getNumberOfConc(), panel.getJPanelOutputVocabularyReduction().getUseRuleConc());
				} else {
					for (int i=0 ; i<this.fis.NbOutputs() ; i++)
						if( this.fis.GetOutput(i).GetNature() == JOutput.FUZZY) {
							MessageFIS.Information(this, LocaleFIS.GetString("ApplyOLSOnlyWithNoClassifCrispOutput"));
							return;
						}
					for (int i=0 ; i<this.fis.NbOutputs() ; i++)
						newFIS = JFIS.NewFISOLS(this.fis, this.DataFile, i, true, 0, JFIS.TypeConjunction()[this.fis.GetConjunction()], 0, 0, panel.getReduceVocabulary(), panel.getJPanelOutputVocabularyReduction().getDegradationAllowed(), panel.getJPanelOutputVocabularyReduction().getNumberOfConc(), panel.getJPanelOutputVocabularyReduction().getUseRuleConc(), false, true, this.DataFile.ActiveFileName());
				}
				new JFISFrame(newFIS, this.DataFile, this );
			} catch (Throwable e1) {
				MessageFIS.Error(null, e1);
			}
		}
	}
	//------------------------------------------------------------------------------
	void GenerateFIS( boolean sort, double threshold )
	{
		try { JFISFrame jff = new JFISFrame(new JGENFIS(this.fis, this.DataFile, sort, threshold ), this.DataFile, this); }
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	public void showInferenceFrame()
	{
		try
		{
			if( this.JInferenceFrameOpen.Permits() == 1 )
				this.jInferenceFrame = new JInferenceFrame( this, this.fis, this.JInferenceFrameOpen, this.DataFile );

			this.jInferenceFrame.setVisible(true);
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }

	}
	//------------------------------------------------------------------------------
	public void setInferenceFrameInputValue(int input_number, double new_value)
	{
		if (jInferenceFrame != null && jInferenceFrame.IsOpen())
			jInferenceFrame.setSliderInputValue(input_number, new_value);
	}
	//------------------------------------------------------------------------------
	void jMenuGenerateRules_actionPerformed(ActionEvent e)
	{
		try
		{
			int nb_rules_to_compute = this.fis.GetActiveInput(0).GetNbMF();
			for( int i=1 ; i<this.fis.NbActiveInputs() ; i++ )
				nb_rules_to_compute = nb_rules_to_compute * this.fis.GetActiveInput(i).GetNbMF();
			if( nb_rules_to_compute > this.WarningNumberOfRules )
				if( MessageFIS.Confirm(this, LocaleFIS.GetString("NumberOfRules") + " = " + nb_rules_to_compute + "\n" + LocaleFIS.GetString("DoYouWantToContinue")) == JOptionPane.NO_OPTION )
					return;

			if( this.DataFile == null )
				this.GenerateFIS(false, 0);
			else
			{
				final JDialog jd = new JDialog();
				jd.setTitle(LocaleFIS.GetString("GenerateRules"));
				jd.getContentPane().setLayout(new GridBagLayout());
				final JCheckBox jcb = new JCheckBox(LocaleFIS.GetString("SortByWeight"));
				jd.getContentPane().add(jcb,  new GridBagConstraints(0, 0, 2, 1, 0.0, 0.0
						,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
				jd.getContentPane().add(new JLabel(LocaleFIS.GetString("Threshold") + ":"),  new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
						,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 0, 5, 5, 5), 0, 0));
				final DoubleField df = new DoubleField();
				df.setValue(0.1);
				jd.getContentPane().add(df,  new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
						,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 5, 5), 30, 0));
				JPanel jp_validation = new JPanel();
				JButton apply_button = new JButton(LocaleFIS.GetString("Apply"));
				apply_button.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e)
				{
					JFISFrame.this.GenerateFIS(jcb.isSelected(), df.getValue());
					jd.dispose();
				} } );
				jp_validation.add(apply_button);
				JButton cancel_button = new JButton(LocaleFIS.GetString("Cancel"));
				cancel_button.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jd.dispose(); } } );
				jp_validation.add(cancel_button);
				jd.getContentPane().add(jp_validation,  new GridBagConstraints(0, 2, 2, 1, 0.0, 0.0
						,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 5, 5), 0, 0));
				jd.setModal(true);
				jd.pack();
				jd.setLocation(JFISChildFrame.ChildPosition(this, jd.getSize()));
				jd.setVisible(true);
			}
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	void jMenuHFPGenerate_actionPerformed(ActionEvent e)
	{
		try
		{
			JHFPConfigFrame jhfpf = new JHFPConfigFrame(this, this.DataFile);
			this.HFPFile = jhfpf.GetHFPFile();
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	void jMenuHFPEdit_actionPerformed(ActionEvent e)
	{
		JOpenFileChooser file_chooser = new JOpenFileChooser(MainFIS.config.FISFilePath());
		if( this.HFPFile != null )
			file_chooser.setSelectedFile(new File(this.HFPFile));
		file_chooser.setAcceptAllFileFilterUsed(true);
		// ajout du filtre *.hfp
		JFileFilter filter = new JFileFilter(new String("hfp"), LocaleFIS.GetString("HFPFile"));
		file_chooser.addChoosableFileFilter(filter);
		if( file_chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION )
		{
			try
			{
				JHFPConfigFrame jhfpf = new JHFPConfigFrame(this, this.DataFile, file_chooser.getSelectedFile().getAbsolutePath());
				this.HFPFile = jhfpf.GetHFPFile();
			}
			catch( Throwable t ) { MessageFIS.Error(null, t); }
		}
	}
	//------------------------------------------------------------------------------
	void jMenuHFPGenerateVertices_actionPerformed(ActionEvent e)
	{
		try
		{
			JHFPVertexFrame jvf = new JHFPVertexFrame(this, this.DataFile, this.HFPFile);
			this.HFPFile = jvf.GetHFPFile();
			this.VertexFile = jvf.GetVertexFile();
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	void jMenuHFPViewVertices_actionPerformed(ActionEvent e)
	{
		try
		{
			JViewVertexFrame jvvf = new JViewVertexFrame(this, this.VertexFile, this.HFPFile, this.DataFile);
			this.HFPFile = jvvf.GetHFPFile();
			this.VertexFile = jvvf.GetVertexFile();
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	void jMenuHFPSelectPartition_actionPerformed(ActionEvent e)
	{
		try
		{
			JHFPSelectFrame jsf = new JHFPSelectFrame( this, this.DataFile, this.HFPFile, this.VertexFile );
			this.HFPFile = jsf.GetHFPFile();
			this.VertexFile = jsf.GetVertexFile();
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	void jMenuHFPGenerateFIS_actionPerformed(ActionEvent e)
	{
		try
		{
			JHFPFISFrame jsf = new JHFPFISFrame( this, this.DataFile, this.HFPFile, this.VertexFile );
			this.HFPFile = jsf.GetHFPFile();
			this.VertexFile = jsf.GetVertexFile();
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	void jMenuSimple_actionPerformed(ActionEvent e)
	{
		try
		{
			// verifie que le fichier de donnees a un nombre correct de colonnes
			if( this.DataFile.VariableCount() != (this.fis.NbInputs() + this.fis.NbOutputs()) )
				this.ThrowDataFileError(this.fis, this.DataFile);
			JSimpleFrame jsf = new JSimpleFrame( this, this.fis, this.DataFile );
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	protected void jMenuSolisWets_actionPerformed(ActionEvent e) {
		try
		{
			// verifie que le fichier de donnees a un nombre correct de colonnes
			if( this.DataFile.VariableCount() != (this.fis.NbInputs() + this.fis.NbOutputs()) )
				this.ThrowDataFileError(this.fis, this.DataFile);
			JFrameSolisWets jof = new JFrameSolisWets(this, this.fis, this.DataFile);
			jof.setLocation(JFISChildFrame.ChildPosition(this, jof.getSize()));
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	void jMenuCustomSolisWets_actionPerformed(ActionEvent e)
	{
		try
		{
			// verifie que le fichier de donnees a un nombre correct de colonnes
			if( this.DataFile.VariableCount() != (this.fis.NbInputs() + this.fis.NbOutputs()) )
				this.ThrowDataFileError(this.fis, this.DataFile);
			JOptFrame jof = new JOptFrame(this, this.fis, this.DataFile);
			jof.setLocation(JFISChildFrame.ChildPosition(this, jof.getSize()));
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	void jMenuConclusionsOptimization_actionPerformed(ActionEvent e)
	{
		try
		{
			if( this.DataFile.VariableCount() != (this.fis.NbInputs() + this.fis.NbOutputs()) )
				this.ThrowDataFileError(this.fis, this.DataFile);
			JFIS newFIS = null;
			JPanelOptimizeConclusions panel = new JPanelOptimizeConclusions(this.fis);
			if( JOptionPane.showOptionDialog(this, panel, LocaleFIS.GetString("OptimizeConclusions"), JOptionPane.YES_OPTION | JOptionPane.CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE, null, null, null) == JOptionPane.YES_OPTION ) {
				try {
					newFIS = JFIS.NewFISOLS(this.fis, this.DataFile, panel.getSelectedOutput(), true, 0, JFIS.TypeConjunction()[this.fis.GetConjunction()], 0, 0, panel.getReduceVocabulary(), panel.getJPanelOutputVocabularyReduction().getDegradationAllowed(), panel.getJPanelOutputVocabularyReduction().getNumberOfConc(), panel.getJPanelOutputVocabularyReduction().getUseRuleConc(), false, true, this.DataFile.ActiveFileName());
					new JFISFrame(newFIS, this.DataFile, this );
				} catch (Throwable e1) {
					MessageFIS.Error(null, e1);
				}
			}
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------

	boolean helpExist(String language) {
		return language.equals("fr") || language.equals("es") || language.equals("en");
	}
	
	void jMenuItemHelp_actionPerformed(ActionEvent e)
	{
		if(Desktop.isDesktopSupported()) {
			Desktop desktop = Desktop.getDesktop();
			if(desktop.isSupported(Desktop.Action.BROWSE)) {
				try {
					desktop.browse(getClass().getResource("/" + (helpExist(MainFIS.config.Language()) ? MainFIS.config.Language() : "en") + "/fisprodoc.html").toURI());
				} catch (Exception exception) {
					JOptionPane.showMessageDialog(this, "help not found", "error", JOptionPane.ERROR_MESSAGE);
				} 
			}
		}
	}
	//------------------------------------------------------------------------------
	void jMenuAbout_actionPerformed(ActionEvent e) { JAboutFrame jaf = new JAboutFrame(this); }
	//------------------------------------------------------------------------------
	void jMenuWangMendel_actionPerformed(ActionEvent e)
	{
		File temp_fis = null;
		try
		{
			// verifie que le fichier de donnees a un nombre correct de colonnes
			if( this.DataFile.VariableCount() != (this.fis.NbInputs() + this.fis.NbOutputs()) )
				this.ThrowDataFileError(this.fis, this.DataFile);
			temp_fis = File.createTempFile("temp", ".fis", new File(MainFIS.config.FISFilePath()));
			String fisfile = this.fis.FISFile;
			this.fis.Save(temp_fis.getAbsolutePath());
			this.fis.FISFile = fisfile;
			JFIS new_fis = JFISHFP.WangMendel(temp_fis.getAbsolutePath(), this.DataFile.ActiveFileName());
			new_fis.SetName(new_fis.GetName() + ".wm");
			JFISFrame jsf = new JFISFrame(new_fis, this.DataFile, this);
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
		finally{ if( temp_fis != null ) temp_fis.delete(); }
	}
	//------------------------------------------------------------------------------
	void jMenuOLS_actionPerformed(ActionEvent e)
	{
		try {
			// verifie que le fichier de donnees a un nombre correct de colonnes
			if( fis != null ) {
				if( this.DataFile.VariableCount() != (this.fis.NbInputs() + this.fis.NbOutputs()) )
					this.ThrowDataFileError(this.fis, this.DataFile);
			}
			JPanelOLS panel = new JPanelOLS(this.fis, this.DataFile);
			if( (this.fis == null) || (panel.getOutputNumber() != -1) ) {
				while( true ) {
					if( JOptionPane.showOptionDialog(this, panel, "OLS", JOptionPane.YES_OPTION | JOptionPane.CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE, null, null, null) == JOptionPane.YES_OPTION ) {
						JFIS olsFIS = this.fis;
						if( (this.fis == null) && (panel.getGenerateFISWithoutRules() == true) ) {
							olsFIS = (new JHFPSRFrame(this, this.DataFile, false)).newFIS;
							if( olsFIS == null ) // annulation du fis sans regles
								continue;
						}
						new JFISFrame(JFIS.NewFISOLS(olsFIS, this.DataFile, panel.getOutputNumber() == -1 ? 0 : panel.getOutputNumber(), panel.keepExistingRules(), panel.getStandardDeviation(), panel.getConjunction(), panel.getTotalUnexplainedVariance(), panel.getMaximumNumberOfRules(), panel.getReduceVocabulary(), panel.getJPanelOutputVocabularyReduction().getDegradationAllowed(), panel.getJPanelOutputVocabularyReduction().getNumberOfConc(), panel.getJPanelOutputVocabularyReduction().getUseRuleConc(), panel.getOutputFuzzification(), panel.getRemoveTemporaryFiles(), panel.getTestDataFile()), this.DataFile, this);
						return;
					} else { return; }
				}
			} else {
				MessageFIS.Information(this, LocaleFIS.GetString("ApplyOLSOnlyWithNoClassifCrispOutput"));
			}
		} catch (Throwable t) {
			MessageFIS.Error(null, t);    }
	}
	//------------------------------------------------------------------------------
	void jMenuHFPSR_actionPerformed(ActionEvent e)
	{
		try
		{
			// verifie que le fichier de donnees a un nombre correct de colonnes
			if( this.DataFile.VariableCount() < 2 )
				throw new Exception(LocaleFIS.GetString("OnlyOneVariableInDataFile"));
			JHFPSRFrame jsf = new JHFPSRFrame( this, this.DataFile, true );
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
}



//------------------------------------------------------------------------------
// la classe JFileChooser est surchargee:
// - pour une meilleure gestion de l'affichage lors de l'ajout de composants par la methode add
//   Utiliser la methode AddComponent qui ajoute le composant dans un GridBagLayout plutot que
//   la methode add qui ajoute le composant dans un BoxLayout avec des problemes d'affichage
class JOpenFileChooser extends JFileChooser
{
	private int Option = JFileChooser.CANCEL_OPTION;
	private JDialog Dialog;
	private Vector AddedComponents = new Vector();
	//------------------------------------------------------------------------------
	public JOpenFileChooser( String currentDirectoryPath )
	{
		super(currentDirectoryPath);
		try
		{
			// Disable the create new folder button
			((JPanel)this.getComponent(1)).getAccessibleContext().getAccessibleChild(9).getAccessibleContext().getAccessibleComponent().setEnabled(false);
			// Hide the create new folder button
			((JPanel)this.getComponent(1)).getAccessibleContext().getAccessibleChild(9).getAccessibleContext().getAccessibleComponent().setVisible(false);
		}
		catch( Exception e ) {}
	}
	//------------------------------------------------------------------------------
	public int showOpenDialog(Component parent)
	{
		this.Dialog = new JFISDialog();
		this.Dialog.setTitle(LocaleFIS.GetString("Open"));
		this.Dialog.getContentPane().setLayout(new GridBagLayout());
		this.Dialog.getContentPane().add(this, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
		for( int i=0 ; i<this.AddedComponents.size() ; i++ )
			this.Dialog.getContentPane().add((Component)this.AddedComponents.elementAt(i), new GridBagConstraints(0, i+1, 1, 1, 1.0, 0.0
					,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 0, 5), 0, 0));

		this.Dialog.setModal(true);
		this.Dialog.pack();
		this.Dialog.setLocation(JFISChildFrame.ChildPosition(parent, this.Dialog.getSize()));
		this.Dialog.setResizable(false);
		this.Dialog.setVisible(true);
		return this.Option;
	}
	//------------------------------------------------------------------------------
	public void approveSelection()
	{
		if( this.getSelectedFile() != null )
		{
			this.Option = JFileChooser.APPROVE_OPTION;
			this.Dialog.dispose();
		}
	}
	//------------------------------------------------------------------------------
	public void cancelSelection() { this.Dialog.dispose(); }
	//------------------------------------------------------------------------------
	public void AddComponent( JComponent comp ) { this.AddedComponents.add(comp); }
}


