//***********************************************************************
//
//                              JFile.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.util.Vector;

//------------------------------------------------------------------------------
abstract class JFile
{
  private double [][] Data;
  private String FileName;
  protected Vector Listeners = new Vector();
//------------------------------------------------------------------------------
  public JFile( String file_name ) throws Throwable
  {
  this.FileName = file_name;
  this.Data = this.LoadData();
  }
//------------------------------------------------------------------------------
  public void Reload( String file_name ) throws Throwable
  {
  this.FileName = file_name;
  this.Data = this.LoadData();
  Object [] listeners = this.Listeners.toArray();
  for( int i=0 ; i<listeners.length ; i++ )
    ((JFileListener)listeners[i]).ReLoaded();
  }
//------------------------------------------------------------------------------
  public String FileName() { return this.FileName; }
//------------------------------------------------------------------------------
  abstract protected double [][] LoadData() throws Throwable;
//------------------------------------------------------------------------------
  public double [][] GetData() { return this.Data; }
//------------------------------------------------------------------------------
  public double[] getColumnData (int column_no)
  {
    double[][] tmp_wholedata = this.GetData() ;
    double[] column_data = new double[tmp_wholedata.length] ;
    for (int i=0 ;i<column_data.length ;i++)
    {
      column_data[i] = tmp_wholedata[i][column_no-1] ;
    }
    return column_data ;
  }
//------------------------------------------------------------------------------
  public int VariableCount() { return this.Data[0].length; }
//------------------------------------------------------------------------------
  public int ItemCount() { return this.Data.length; }
//------------------------------------------------------------------------------
  public double [] VariableData( int column )
    {
    double [] result = new double[this.Data.length];
    for( int i=0 ; i<result.length ; i++ )
      result[i] = this.Data[i][column];
    return result;
    }
//------------------------------------------------------------------------------
  public void AddJFileListener( JFileListener listener ) { this.Listeners.add(listener); }
//------------------------------------------------------------------------------
  public void RemoveJFileListener( JFileListener listener ) { this.Listeners.remove(listener); }
//------------------------------------------------------------------------------
  public int ListenersSize() { return this.Listeners.size(); }
//------------------------------------------------------------------------------
  public void Close()
  {
  Object [] listeners = this.Listeners.toArray();
  for( int i=0 ; i<listeners.length ; i++ )
    ((JFileListener)listeners[i]).Closed();
  }
}
