//***********************************************************************
//
//                              JFrame3DData.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import javax.media.j3d.Node;
import javax.swing.JFileChooser;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.ToolTipManager;
import javax.swing.JPanel;
import javax.vecmath.Point3f;

import java.awt.GridBagLayout;
import org.freehep.j3d.plot.View3DScatterPlot;
import org.freehep.j3d.plot.ViewUnbinned3DData;
import org.freehep.util.export.ExportDialog;

import util.JDoubleRound;

import com.sun.j3d.utils.picking.PickResult;

import java.awt.GridBagConstraints;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

// TODO ajouter un listener de fis sur les noms des entrees pour maj des labels des axes
public class JFrame3DData extends JFISChildFrame {

    private JMenuBar jJMenuBar = null;

    private JMenu jMenuFile = null;

    private JMenuItem jMenuItemExport = null;

    private JCheckBoxMenuItem jCheckBoxMenuItemRecord = null;

    private JSeparator jSeparator = null;

    private JMenuItem jMenuItemResetView = null;

    private JPanel jContentPane = null;

    private View3DScatterPlot view3DScatterPlot = null;

    private JExtendedDataFile dataFile = null;

    private JExtendedFileListener file_listener = null;

    /**
     * This method initializes this
     * 
     */
    private void initialize() {
        this.setJMenuBar(getJJMenuBar());
        this.setContentPane(getJContentPane());

    }

    public void Translate() throws Throwable {
    // TODO Raccord de methode auto-genere

    }

    /**
     * @param parent
     */
    public JFrame3DData(JFISFrame parent) {
        super(parent);
        //A crucial setting to make visible the JMenuItems :
        //InferenceSurfacePlot is a HeavyWeight object whereas the JMenuBar is a lightweight object
        //this make the JMenuItems invisible because they use the rendering system of the frame (JFrame fPlot).
        //Canvas3D anf thus InferenceSurfacePlot renders its contents in its own native window, which is actually a child of the frame's native window, so the JMenuItems are hidden
        //A discussion on "Mixing heavy and light components" can be found at : http://java.sun.com/products/jfc/tsc/articles/mixing/
        //this next setting forces the JMenuBar to create an AWT heavyweight panel3D to display the JMenuItems
        JPopupMenu.setDefaultLightWeightPopupEnabled(false);
        ToolTipManager.sharedInstance().setLightWeightPopupEnabled(false);
        initialize();
    }

    public void show(JExtendedDataFile dataFile, int xSelected, int ySelected, int zSelected, String xLabel, String yLabel, String zLabel) throws Throwable {
        this.setTitle(zLabel + " / " + yLabel + " / " + xLabel);
        this.dataFile = dataFile;

        // send datas to view3DScatterPlot
        this.view3DScatterPlot.setAxes(new String[] { xLabel, yLabel, zLabel });
        this.view3DScatterPlot.setData(new ViewUnbinned3DData(this.dataFile.getColumnData(xSelected), this.dataFile.getColumnData(ySelected), this.dataFile.getColumnData(zSelected), this.dataFile.GetActiveValuesArray()));

        //extended data file listener, to listen to activation status changes:  items must change their color (color<->gray)
        if (this.dataFile != null) {
            final JDataFile jdf = new JDataFile(this.dataFile.ActiveFileName());

            this.file_listener = new JExtendedFileListener() {
                public void Closed() {
                    jdf.Close();
                }

                public void ReLoaded() {
                    try {
                        jdf.Reload(JFrame3DData.this.dataFile.ActiveFileName());
                    } catch (Throwable t) {
                        MessageFIS.Error(null, t);
                    }
                }

                //       the activation status of one example was modified
                public void ActivationChanged(int item, boolean active) {
                    JFrame3DData.this.view3DScatterPlot.setItemActive(item - 1, active);
                }

                //       the activation status of an array of examples was modified
                public void ActivationChanged(int[] items, boolean active) {
                    for (int i = 0; i < JFrame3DData.this.dataFile.GetActiveValuesArray().length; i++) {
                        if (JFrame3DData.this.dataFile.GetActive(i))
                            JFrame3DData.this.view3DScatterPlot.setItemActive(i, true);
                        else
                            JFrame3DData.this.view3DScatterPlot.setItemActive(i, false);
                    }
                }
            };
            JFrame3DData.this.dataFile.AddJFileListener(this.file_listener);
        }

        this.addWindowListener(new WindowAdapter() {

			@Override
			public void windowOpened(WindowEvent e) {
				super.windowOpened(e);
				toFront();
			}});
        
        this.pack();
        this.setLocation(JFISChildFrame.ChildPosition(this.parent, this.getSize()));
        super.setVisible(true);
    }

    /**
     * This method initializes jJMenuBar	
     * 	
     * @return javax.swing.JMenuBar	
     */
    private JMenuBar getJJMenuBar() {
        if (jJMenuBar == null) {
            jJMenuBar = new JMenuBar();
            jJMenuBar.add(getJMenuFile());
        }
        return jJMenuBar;
    }

    /**
     * This method initializes jMenuFile	
     * 	
     * @return javax.swing.JMenu	
     */
    private JMenu getJMenuFile() {
        if (jMenuFile == null) {
            jMenuFile = new JMenu();
            jMenuFile.setText(LocaleFIS.GetString("File"));
            jMenuFile.add(getJMenuItemExport());
            jMenuFile.add(getJCheckBoxMenuItemRecord());
            jMenuFile.add(getJSeparator());
            jMenuFile.add(getJMenuItemResetView());
        }
        return jMenuFile;
    }

    /**
     * This method initializes jMenuItemExport	
     * 	
     * @return javax.swing.JMenuItem	
     */
    private JMenuItem getJMenuItemExport() {
        if (jMenuItemExport == null) {
            jMenuItemExport = new JMenuItem();
            jMenuItemExport.setText(LocaleFIS.GetString("Export"));
            jMenuItemExport.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    ExportDialog export = new ExportDialog();
                    export.showExportDialog(JFrame3DData.this, LocaleFIS.GetString("ExportViewAs"), view3DScatterPlot, "export");
                }
            });
        }
        return jMenuItemExport;
    }

    /**
     * This method initializes jCheckBoxMenuItemRecord	
     * 	
     * @return javax.swing.JCheckBoxMenuItem	
     */
    private JCheckBoxMenuItem getJCheckBoxMenuItemRecord() {
        if (jCheckBoxMenuItemRecord == null) {
            jCheckBoxMenuItemRecord = new JCheckBoxMenuItem();
            jCheckBoxMenuItemRecord.setText(LocaleFIS.GetString("Record"));
            jCheckBoxMenuItemRecord.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(java.awt.event.ItemEvent e) {
                    String recordFileName = null;
                    if (jCheckBoxMenuItemRecord.isSelected() == true) {
                        JFileChooser jfc = new JFileChooser(System.getProperty("user.dir"));
                        jfc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                        jfc.setAcceptAllFileFilterUsed(false);
                        jfc.addChoosableFileFilter(new JFileFilter("jpg", LocaleFIS.GetString("JPEGFile")));
                        jfc.setSelectedFile(new File("capture"));
                        if (jfc.showSaveDialog(JFrame3DData.this) == JFileChooser.APPROVE_OPTION)
                            recordFileName = jfc.getSelectedFile().getAbsolutePath();
                        else {
                            jCheckBoxMenuItemRecord.setSelected(false);
                            return;
                        }
                    }
                    view3DScatterPlot.record(jCheckBoxMenuItemRecord.isSelected(), recordFileName);
                }
            });
        }
        return jCheckBoxMenuItemRecord;
    }

    /**
     * This method initializes jSeparator	
     * 	
     * @return javax.swing.JSeparator	
     */
    private JSeparator getJSeparator() {
        if (jSeparator == null) {
            jSeparator = new JSeparator();
        }
        return jSeparator;
    }

    /**
     * This method initializes jMenuItemResetView	
     * 	
     * @return javax.swing.JMenuItem	
     */
    private JMenuItem getJMenuItemResetView() {
        if (jMenuItemResetView == null) {
            jMenuItemResetView = new JMenuItem();
            jMenuItemResetView.setText(LocaleFIS.GetString("ResetView"));
            jMenuItemResetView.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    view3DScatterPlot.resetView();
                }
            });
        }
        return jMenuItemResetView;
    }

    /**
     * This method initializes jContentPane	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getJContentPane() {
        if (jContentPane == null) {
            GridBagConstraints gridBagConstraints = new GridBagConstraints();
            gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
            gridBagConstraints.gridy = 0;
            gridBagConstraints.weightx = 1.0;
            gridBagConstraints.weighty = 1.0;
            gridBagConstraints.gridx = 0;
            jContentPane = new JPanel();
            jContentPane.setLayout(new GridBagLayout());
            jContentPane.add(getView3DScatterPlot(), gridBagConstraints);
        }
        return jContentPane;
    }

    /**
     * This method initializes view3DScatterPlot	
     * 	
     * @return org.freehep.j3d.plot.View3DScatterPlot	
     */
    private View3DScatterPlot getView3DScatterPlot() {
        if (view3DScatterPlot == null) {
            view3DScatterPlot = new View3DScatterPlot() {

                public void processIntersection(PickResult result) {
                    try {
                        //  intersection.get
                        Node n = result.getObject();
                        Integer item_number = (Integer) n.getUserData(); //!!! we started at zero
                        Point3f p = this.getItemData(item_number.intValue());
                        this.getToolTip().setToolTipText(LocaleFIS.GetString("Item").toLowerCase() + " " + (item_number.intValue() + 1) + " <n> " + "x= " + JDoubleRound.round(p.x, 4) + ", y= " + JDoubleRound.round(p.y, 4) + ", z= " + JDoubleRound.round(p.z, 4)); //wait before showing     
                    } catch (Throwable t) {
                        System.out.println(t.getMessage());
                    }
                }

                public void processNoIntersection() {
                    this.getToolTip().hideToolTip();
                }

                public void processLeftClick(PickResult result) {
                    Node n = result.getObject();
                    Integer item_number = (Integer) n.getUserData(); //!!! we started at zero
                    boolean activation = !dataFile.GetActive(item_number.intValue());
                    setItemActive(item_number.intValue(), activation);
                    //update data table;  before:  remove file listener to avoid double events ;
                    JExtendedFileListener jefl = file_listener;
                    dataFile.RemoveJFileListener(jefl);
                    dataFile.SetActive(item_number.intValue() + 1, activation); //!!! setActive and getActive works differently, item_number or item_number+1 !
                    dataFile.AddJFileListener(jefl);
                }

            };
        }
        return view3DScatterPlot;
    }

}
