//***********************************************************************
//
//                              JFrame3DInfer.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.io.File;
import java.lang.ref.WeakReference;

import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.ToolTipManager;
import javax.vecmath.Point3f;

import org.freehep.j3d.plot.OutputInferBinned2DData;
import org.freehep.j3d.plot.SurfacePlot2;
import org.freehep.util.export.ExportDialog;

import util.JDoubleRound;

import com.sun.j3d.utils.picking.PickResult;

public class JFrame3DInfer extends JFISChildFrame {

    private JPanel jContentPane = null;

    private JPanel3DInferParameters jPanel3DInferParameters = null;

    private JFIS fis = null;

    private SurfacePlot2 surfacePlot2 = null; //  @jve:decl-index=0:visual-constraint="505,27"

    private SurfacePlotParameters surfacePlotParameters = new SurfacePlotParameters();

    private double rulesMatchingDegree = 0.00d;

    private JMenuBar jJMenuBar = null;

    private JMenu jMenuFile = null;

    private JMenuItem jMenuItemExport = null;

    private JCheckBoxMenuItem jCheckBoxMenuItemRecord = null;

    private JMenu jMenuSection = null;

    private JMenu jMenuOptions = null;

    private JMenuItem jMenuItemDisplay = null;

    private JMenuItem jMenuItemResetView = null;

    private JSeparator jSeparator = null;

    private JFrameSectionInfer jFrameSectionInfer = null;

    private JMenuItem jMenuItemBlankThreshold = null;

    private JPanel jPanelOptionBlankThresold = null; //  @jve:decl-index=0:visual-constraint="559,168"

    private JLabel jLabelBlankThres = null;

    private DoubleField doubleFieldBlankThres = null;

    /**
     * This is the default constructor
     */
    public JFrame3DInfer(JFISFrame parent) {
        super(parent);
        //A crucial setting to make visible the JMenuItems :
        //InferenceSurfacePlot is a HeavyWeight object whereas the JMenuBar is a lightweight object
        //this make the JMenuItems invisible because they use the rendering system of the frame (JFrame fPlot).
        //Canvas3D anf thus InferenceSurfacePlot renders its contents in its own native window, which is actually a child of the frame's native window, so the JMenuItems are hidden
        //A discussion on "Mixing heavy and light components" can be found at : http://java.sun.com/products/jfc/tsc/articles/mixing/
        //this next setting forces the JMenuBar to create an AWT heavyweight panel3D to display the JMenuItems
        JPopupMenu.setDefaultLightWeightPopupEnabled(false);
        ToolTipManager.sharedInstance().setLightWeightPopupEnabled(false);
        initialize();
    }

    public void show(JFIS fis) throws Throwable {
        getSurfacePlot2();
        this.fis = fis;
        this.jPanel3DInferParameters.init(this.fis, true);
        this.addSurfacePlotInContentPane();
        this.showSurfacePlot();
        this.pack();
        // limitation de la largeur de la fenetre (a cause du jPanelPlot2DInfer qui n'est pas limite en largeur) 
        this.setSize(this.getMinimumSize().width, this.getSize().height);
        this.setLocation(JFISChildFrame.ChildPosition(this.parent, this.getSize()));
        this.setModal(true);
        super.setVisible(true);
    }

    void addSurfacePlotInContentPane() {
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.gridx = 0;
        this.jContentPane.add(this.surfacePlot2, gridBagConstraints);
    }

    /* (non-Javadoc)
     * @see fis.Translatable#Translate()
     */
    public void Translate() throws Throwable {
    // TODO Raccord de methode auto-genere

    }

    /**
     * This method initializes this
     * 
     * @return void
     */
    private void initialize() {
        this.setContentPane(getJContentPane());
        this.setJMenuBar(getJJMenuBar());
        this.setTitle(LocaleFIS.GetString("Surface"));
    }

    /**
     * This method initializes jContentPane
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getJContentPane() {
        if (jContentPane == null) {
            GridBagConstraints gridBagConstraints = new GridBagConstraints();
            gridBagConstraints.gridx = 0;
            gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
            gridBagConstraints.weightx = 1.0D;
            gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
            gridBagConstraints.gridy = 0;
            jContentPane = new JPanel();
            jContentPane.setLayout(new GridBagLayout());
            jContentPane.add(getJPanel3DInferParameters(), gridBagConstraints);
        }
        return jContentPane;
    }

    /**
     * This method initializes jPanel3DInferParameters	
     * 	
     * @return fis.JPanel3DInferParameters	
     */
    private JPanel3DInferParameters getJPanel3DInferParameters() {
        if (jPanel3DInferParameters == null) {
            jPanel3DInferParameters = new JPanel3DInferParameters();
            jPanel3DInferParameters.addChangeListener(new javax.swing.event.ChangeListener() {
                public void stateChanged(javax.swing.event.ChangeEvent e) {
                    try {
                        showSurfacePlot();
                    } catch (Throwable t) {
                        // TODO Bloc catch auto-genere
                        t.printStackTrace();
                    }
                }
            });
        }
        return jPanel3DInferParameters;
    }

    /**
     * Affiche les donnees dans l'objet SurfacePlot2
     * 
     * @throws Throwable
     */
    void showSurfacePlot() throws Throwable {
        Object[] surfaceplotInput = compute3DInfer(jPanel3DInferParameters.selectedXInput(), jPanel3DInferParameters.xResolution(), jPanel3DInferParameters.xMin(), jPanel3DInferParameters.xMax(), jPanel3DInferParameters.selectedYInput(), jPanel3DInferParameters.yResolution(), jPanel3DInferParameters.yMin(), jPanel3DInferParameters.yMax(), jPanel3DInferParameters.selectedOutput(), jPanel3DInferParameters.getInputsValues());
        String[] Axislabels = { fis.GetInput(jPanel3DInferParameters.selectedXInput()).GetName(), fis.GetInput(jPanel3DInferParameters.selectedYInput()).GetName(), fis.GetOutput(jPanel3DInferParameters.selectedOutput()).GetName() };
        surfacePlot2.setAxes(Axislabels);
        surfacePlot2.setData(new OutputInferBinned2DData(((double[]) surfaceplotInput[0]), jPanel3DInferParameters.xMin(), jPanel3DInferParameters.xMax(), jPanel3DInferParameters.yMin(), jPanel3DInferParameters.yMax()), Axislabels, false);
    }

    /**
     * TODO a reprendre quand Infer acceptera les entrees inactives
     * @param selectedXInput
     * @param xResolution
     * @param xMin
     * @param xMax
     * @param selectedYInput
     * @param yResolution
     * @param yMin
     * @param yMax
     * @param selectedOutput
     * @param inputsValues
     * @return the Z values array to send to SurfacePlot2. Two first additional values must be the X and Y resolutions.
     * @throws Throwable
     */
    private Object[] compute3DInfer(int selectedXInput, int xResolution, double xMin, double xMax, int selectedYInput, int yResolution, double yMin, double yMax, int selectedOutput, double[] inputsValues) throws Throwable {
        surfacePlotParameters.Xresolution = xResolution;
        surfacePlotParameters.Yresolution = xResolution; // y resolution = x resolution
        surfacePlotParameters.Xmax = xMax;
        surfacePlotParameters.Xmin = xMin;
        surfacePlotParameters.Ymax = yMax;
        surfacePlotParameters.Ymin = yMin;

        //output number
        surfacePlotParameters.output_number = selectedOutput;

        //fixed inputs :
        surfacePlotParameters.fixedinputs_numbers = new int[this.fis.NbActiveInputs() - 2];
        surfacePlotParameters.fixedinputs_values = new double[this.fis.NbActiveInputs() - 2];

        // modifie la selection selectedXInput en numero d'entree active
        surfacePlotParameters.selected_X_input_number = selectedXInput;
        for (int i = 0; i < selectedXInput; i++)
            if (this.fis.GetInput(i).GetActive() == false)
                surfacePlotParameters.selected_X_input_number--;

        // modifie la selection selectedYInput en numero d'entree active
        surfacePlotParameters.selected_Y_input_number = selectedYInput;
        for (int i = 0; i < selectedYInput; i++)
            if (this.fis.GetInput(i).GetActive() == false)
                surfacePlotParameters.selected_Y_input_number--;

        for (int i = 0, active = 0, active_count = 0; i < this.fis.NbInputs(); i++) {
            if (this.fis.GetInput(i).GetActive() == true) //we retrieve all actives values except the X one
            {
                if ((i != selectedXInput) && (i != selectedYInput)) {
                    surfacePlotParameters.fixedinputs_numbers[active_count] = active;
                    surfacePlotParameters.fixedinputs_values[active_count] = inputsValues[i];
                    active_count++;
                }
                active++;
            }
        }

        //SurfacePlotParameters is ready. Now, go to jni
        return this.fis.Infer3DSurface(surfacePlotParameters);
    }

    /**
     * This method initializes surfacePlot2	
     * 	
     * @return org.freehep.j3d.plot.SurfacePlot2	
     */
    private SurfacePlot2 getSurfacePlot2() {
        if (surfacePlot2 == null) {
            surfacePlot2 = new SurfacePlot2() {

                public void processNoIntersection() {
                    this.getToolTip().hideToolTip();
                }

                public void processLeftClick(PickResult result) {
                    Point3f p = getPickIntersectionPoint(result);
                    this.getToolTip().setToolTipText("x: " + JDoubleRound.round(p.x, 4) + "<n> y: " + JDoubleRound.round(p.y, 4) + "<n> z: " + JDoubleRound.round(p.z, 4));
                }

                public void processRightClick(PickResult result) {
                    Point3f p = getPickIntersectionPoint(result);
                    try {
                        int nb_act_inputs = fis.NbActiveInputs();
                        double[] input_values = new double[nb_act_inputs];
                        input_values[surfacePlotParameters.selected_X_input_number] = p.x;
                        input_values[surfacePlotParameters.selected_Y_input_number] = p.y;
                        for (int i = 0; i < nb_act_inputs - 2; i++) {
                            input_values[surfacePlotParameters.fixedinputs_numbers[i]] = surfacePlotParameters.fixedinputs_values[i];
                        }
                        double[] rules_weight = fis.getRulesWeight(input_values);
                        String tip = new String(LocaleFIS.GetString("BlankThres").toLowerCase() + ": " + rulesMatchingDegree + "<n> " + LocaleFIS.GetString("Rules").toLowerCase() + ": <n> ");
                        boolean at_least_one_rule = false;
                        for (int i = 0; i < rules_weight.length; i++) {
                            if (rules_weight[i] > rulesMatchingDegree) {
                                tip = tip + (i + 1) + " : " + JDoubleRound.round(rules_weight[i], 4) + " <n> ";
                                at_least_one_rule = true;
                            }
                        }
                        if (at_least_one_rule == false)
                            tip = new String(LocaleFIS.GetString("BlankThres").toLowerCase() + ": " + rulesMatchingDegree + "<n> " + LocaleFIS.GetString("NoRulesActivated"));
                        this.getToolTip().setToolTipText(tip);
                    } catch (Throwable t) {
                        // TODO a modifier
                        t.printStackTrace();
                    }
                }

                public void processLeftDoubleClick(PickResult result) {
                    Point3f p = getPickIntersectionPoint(result);
                    try {
                        parent.showInferenceFrame();

                        //we set the inference frame's sliders values one by one. Setting the sliders double value is sufficient to have the correct inference
                        int Xinput_number = fis.GetActiveInputNumber(surfacePlotParameters.selected_X_input_number);
                        int Yinput_number = fis.GetActiveInputNumber(surfacePlotParameters.selected_Y_input_number);
                        parent.setInferenceFrameInputValue(Xinput_number, p.x);
                        parent.setInferenceFrameInputValue(Yinput_number, p.y);
                        for (int i = 0; i < surfacePlotParameters.fixedinputs_numbers.length; i++) {
                            int fixed_input_number = fis.GetActiveInputNumber(surfacePlotParameters.fixedinputs_numbers[i]);
                            parent.setInferenceFrameInputValue(fixed_input_number, surfacePlotParameters.fixedinputs_values[i]);
                        }
                    } catch (Throwable t) {
                        // TODO a modifier
                        t.printStackTrace();
                    }
                }

                public Dimension getPreferredSize() {
                    // objet carre de la taille du panel de parametrage 3D
                    return new Dimension(jPanel3DInferParameters.getMinimumSize().width, jPanel3DInferParameters.getMinimumSize().width);
                }
            };
        }
        return surfacePlot2;
    }

    /**
     * This method initializes jJMenuBar	
     * 	
     * @return javax.swing.JMenuBar	
     */
    private JMenuBar getJJMenuBar() {
        if (jJMenuBar == null) {
            jJMenuBar = new JMenuBar();
            jJMenuBar.add(getJMenuFile());
            jJMenuBar.add(getJMenuOptions());
            jJMenuBar.add(getJMenuSection());
        }
        return jJMenuBar;
    }

    /**
     * This method initializes jMenuFile	
     * 	
     * @return javax.swing.JMenu	
     */
    private JMenu getJMenuFile() {
        if (jMenuFile == null) {
            jMenuFile = new JMenu();
            jMenuFile.setText(LocaleFIS.GetString("File"));
            jMenuFile.add(getJMenuItemExport());
            jMenuFile.add(getJCheckBoxMenuItemRecord());
            jMenuFile.add(getJSeparator());
            jMenuFile.add(getJMenuItemResetView());
        }
        return jMenuFile;
    }

    /**
     * This method initializes jMenuItemExport	
     * 	
     * @return javax.swing.JMenuItem	
     */
    private JMenuItem getJMenuItemExport() {
        if (jMenuItemExport == null) {
            jMenuItemExport = new JMenuItem();
            jMenuItemExport.setText(LocaleFIS.GetString("Export"));
            jMenuItemExport.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    ExportDialog export = new ExportDialog();
                    export.showExportDialog(JFrame3DInfer.this, LocaleFIS.GetString("ExportViewAs"), surfacePlot2, "export");
                }
            });
        }
        return jMenuItemExport;
    }

    /**
     * This method initializes jCheckBoxMenuItemRecord	
     * 	
     * @return javax.swing.JCheckBoxMenuItem	
     */
    private JCheckBoxMenuItem getJCheckBoxMenuItemRecord() {
        if (jCheckBoxMenuItemRecord == null) {
            jCheckBoxMenuItemRecord = new JCheckBoxMenuItem();
            jCheckBoxMenuItemRecord.setText(LocaleFIS.GetString("Record"));
            jCheckBoxMenuItemRecord.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(java.awt.event.ItemEvent e) {
                    String recordFileName = null;
                    if (jCheckBoxMenuItemRecord.isSelected() == true) {
                        JFileChooser jfc = new JFileChooser(System.getProperty("user.dir"));
                        jfc.setAcceptAllFileFilterUsed(false);
                        jfc.addChoosableFileFilter(new JFileFilter("jpg", LocaleFIS.GetString("JPEGFile")));
                        jfc.setSelectedFile(new File("capture"));
                        if (jfc.showSaveDialog(JFrame3DInfer.this) == JFileChooser.APPROVE_OPTION)
                            recordFileName = jfc.getSelectedFile().getAbsolutePath();
                        else {
                            jCheckBoxMenuItemRecord.setSelected(false);
                            return;
                        }
                    }
                    surfacePlot2.record(jCheckBoxMenuItemRecord.isSelected(), recordFileName);
                }
            });
        }
        return jCheckBoxMenuItemRecord;
    }

    /**
     * This method initializes jMenuSection	
     * 	
     * @return javax.swing.JMenu	
     */
    private JMenu getJMenuSection() {
        if (jMenuSection == null) {
            jMenuSection = new JMenu();
            jMenuSection.setText("Section");
            jMenuSection.add(getJMenuItemDisplay());
        }
        return jMenuSection;
    }

    /**
     * This method initializes jMenuOptions	
     * 	
     * @return javax.swing.JMenu	
     */
    private JMenu getJMenuOptions() {
        if (jMenuOptions == null) {
            jMenuOptions = new JMenu();
            jMenuOptions.setText("Options");
            jMenuOptions.add(getJMenuItemBlankThreshold());
        }
        return jMenuOptions;
    }

    /**
     * This method initializes jMenuItemDisplay	
     * 	
     * @return javax.swing.JMenuItem	
     */
    private JMenuItem getJMenuItemDisplay() {
        if (jMenuItemDisplay == null) {
            jMenuItemDisplay = new JMenuItem();
            jMenuItemDisplay.setText(LocaleFIS.GetString("JFrame3DInfer.Display"));
            jMenuItemDisplay.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    try {
                        if ((jFrameSectionInfer == null) || (jFrameSectionInfer.IsOpen() == false)) {
                            jFrameSectionInfer = new JFrameSectionInfer(JFrame3DInfer.this.parent, JFrame3DInfer.this) {
                                // changement d'etat de la fenetre section
                                protected void jFrameSectionStateChanged(boolean showSectionSelected, boolean isXAxisSelected, double sectionValue, double sectionMin, double sectionMax) {
                                    surfacePlot2.setSectionVisible(showSectionSelected);
                                    double normalizedValue;
                                    if (isXAxisSelected == false)
                                        normalizedValue = ((sectionValue - sectionMin) / (sectionMax - sectionMin)) - 0.5; //-0.5 because x minimum = -0.5 in the 3D plot
                                    else
                                        normalizedValue = ((sectionMax - sectionValue) / (sectionMax - sectionMin)) - 0.5; //-0.5 because x minimum = -0.5 in the 3D plot   invert y axis
                                    surfacePlot2.setSectionCoordinates(showSectionSelected == true ? (float) normalizedValue : 0.5f, isXAxisSelected == true ? 0 : 1);
                                }
                            };
                            jFrameSectionInfer.show(fis, jPanel3DInferParameters);
                        } else
                            jFrameSectionInfer.setVisible(true);
                    } catch (Throwable e1) {
                        // TODO Bloc catch auto-genere
                        e1.printStackTrace();
                    }
                }
            });
        }
        return jMenuItemDisplay;
    }

    /**
     * This method initializes jMenuItemResetView	
     * 	
     * @return javax.swing.JMenuItem	
     */
    private JMenuItem getJMenuItemResetView() {
        if (jMenuItemResetView == null) {
            jMenuItemResetView = new JMenuItem();
            jMenuItemResetView.setText(LocaleFIS.GetString("ResetView"));
            jMenuItemResetView.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    surfacePlot2.resetView();
                }
            });
        }
        return jMenuItemResetView;
    }

    /**
     * This method initializes jSeparator	
     * 	
     * @return javax.swing.JSeparator	
     */
    private JSeparator getJSeparator() {
        if (jSeparator == null) {
            jSeparator = new JSeparator();
        }
        return jSeparator;
    }

    /**
     * This method initializes jMenuItemBlankThreshold	
     * 	
     * @return javax.swing.JMenuItem	
     */
    private JMenuItem getJMenuItemBlankThreshold() {
        if (jMenuItemBlankThreshold == null) {
            jMenuItemBlankThreshold = new JMenuItem();
            jMenuItemBlankThreshold.setText(LocaleFIS.GetString("BlankThres"));
            jMenuItemBlankThreshold.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    if (doubleFieldBlankThres != null)
                        doubleFieldBlankThres.setValue(rulesMatchingDegree);
                    if (JOptionPane.showOptionDialog(JFrame3DInfer.this, getJPanelOptionBlankThresold(), LocaleFIS.GetString("Options"), JOptionPane.YES_OPTION | JOptionPane.CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE, null, null, null) == JOptionPane.YES_OPTION) {
                        rulesMatchingDegree = doubleFieldBlankThres.getValue();
                    }
                }
            });
        }
        return jMenuItemBlankThreshold;
    }

    /**
     * This method initializes jPanelOptionBlankThresold	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getJPanelOptionBlankThresold() {
        if (jPanelOptionBlankThresold == null) {
            GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
            gridBagConstraints2.gridx = 0;
            gridBagConstraints2.insets = new java.awt.Insets(0, 0, 0, 5);
            gridBagConstraints2.gridy = 0;
            GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
            gridBagConstraints1.gridx = 1;
            gridBagConstraints1.gridy = 0;
            gridBagConstraints1.ipadx = 40;
            jLabelBlankThres = new JLabel();
            jLabelBlankThres.setText(LocaleFIS.GetString("BlankThres") + ":");
            jPanelOptionBlankThresold = new JPanel();
            jPanelOptionBlankThresold.setLayout(new GridBagLayout());
            jPanelOptionBlankThresold.setSize(new java.awt.Dimension(75, 26));
            jPanelOptionBlankThresold.add(jLabelBlankThres, gridBagConstraints2);
            jPanelOptionBlankThresold.add(getDoubleField(), gridBagConstraints1);
        }
        return jPanelOptionBlankThresold;
    }

    /**
     * This method initializes doubleField	
     * 	
     * @return fis.DoubleField	
     */
    private DoubleField getDoubleField() {
        if (doubleFieldBlankThres == null) {
            doubleFieldBlankThres = new DoubleField();
        }
        doubleFieldBlankThres.setValue(this.rulesMatchingDegree);
        System.out.println("setValue");
        return doubleFieldBlankThres;
    }
} //  @jve:decl-index=0:visual-constraint="4,3"
