package fis;

import static fis.LocaleFIS.GetString;
import static org.apache.commons.lang.StringUtils.EMPTY;
import static util.swing.SwingUtilities.getWindowAncestor;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import org.apache.commons.lang.StringUtils;

import com.google.common.collect.Iterables;
import com.jgoodies.validation.ValidationResult;
import com.jgoodies.validation.ValidationResultModel;
import com.jgoodies.validation.Validator;
import com.jgoodies.validation.util.DefaultValidationResultModel;

import util.swing.JFormattedTextFieldDouble;
import util.swing.SwingUtilities;
import util.swing.WindowUtilities;
import util.swing.dialog.validation.JOptionPaneValidationResult;

import fis.optimization.crossvalidation.CrossValidationModel;
import fis.swing.optimization.crossvalidation.CrossValidationAdapter;
import fis.swing.optimization.crossvalidation.JPanelCrossValidation;

public class JFrameSolisWets extends JFISDialog {

	private static final long serialVersionUID = 1L;
	private JButton jButtonApply = new JButton();
	private JButton jButtonCancel = new JButton();
	private JFIS fis;
	private JFISFrame Parent;
	private JExtendedDataFile DataFile;
	private ButtonGroup jBGOutput;
	private JPanel jPanelOutput;
	private JLabel jLabelMaxOfIter = new JLabel();
	private IntegerField jIFMaxOfIter = new IntegerField();
	private JLabel jLabelLossOfCoverage = new JLabel();
	private DoubleField jDFLossOfCoverage = new DoubleField();
	private JLabel jLabelDistanceThres = new JLabel();
	private JLabel jLabelBlankThres = new JLabel();
	private DoubleField jDFBlankThres = new DoubleField();
	private JLabel jLabelNumberOfLoops = new JLabel();
	private IntegerField jIFNumberOfLoops = new IntegerField();
	private JLabel jLabelMinBetweenMF = new JLabel();
	private JFormattedTextFieldDouble fieldMinBetweenMF = new JFormattedTextFieldDouble();
	private JButton jbAdvanced = new JButton();
	private int MaxOfConstraints = 1000;
	private int MaxOfFailures = 1000;
	private int SeedValue = 1;
	private double l1 = 0.4, l2 = 0.2, l3 = 0.5;
	private double StdGaussianNoise = 0.005;
	private JPanelInputs jPanelInputs = null;
	private JCheckBox jCheckBoxRules = new JCheckBox();
	private JCheckBox jCheckBoxOutput = new JCheckBox();
	private JPanelCrossValidation panelCrossValidation = new JPanelCrossValidation();
	private CrossValidationModel crossValidation;
	private CrossValidationAdapter crossValidationAdapter;

	public JFrameSolisWets(JFISFrame parent, JFIS fis, JExtendedDataFile data) {
	    this.fis = fis;
	    this.Parent = parent;
	    this.DataFile = data;
	    this.crossValidation = new CrossValidationModel(this.DataFile, this.fis);
	    this.crossValidationAdapter = new CrossValidationAdapter(crossValidation, this.DataFile, this.fis);
	    try
	      {
	      if( this.fis.NbRules() == 0 )
	        {
	        MessageFIS.Error(this,LocaleFIS.GetString("Error"),LocaleFIS.GetString("OptimizationImpossibleTheFISHasNoRules"));
	        return;
	        }

	      this.jBGOutput = new ButtonGroup();
	      this.jPanelOutput = new JPanel(new GridBagLayout());
	      this.jPanelOutput.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),LocaleFIS.GetString("OutputToBeOptimized")));
	      for( int i=0, x=0, y=0 ; i<this.fis.NbOutputs() ; i++ )
	        {
	        JOutput output = this.fis.GetOutput(i);
	        if( output.GetActive() == true )
	          {
	          JRadioButton jrb = new JRadioButton(output.GetName());
	          jrb.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e)
	            {
	            try { jbInit(); }
	            catch(Throwable t) { MessageFIS.Error(null, t); }
	            } });
	          this.jBGOutput.add(jrb);
	          this.jPanelOutput.add(jrb,  new GridBagConstraints(x, y, 1, 1, 0.0, 0.0
	                ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 0, 0), 0, 0));
	          //y = (x == 2) ? y+1 : y;
	          //x = (x == 2) ? 0 : x+1;
	          x++;
	          }
	        }
	      ((JRadioButton)this.jBGOutput.getElements().nextElement()).setSelected(true); // active la premiere sortie
	      jbInit();
	      this.setLocation(JFISChildFrame.ChildPosition(this.Parent, this.getSize()));
	      this.setVisible(true);
	      }
	    catch(Throwable t) { MessageFIS.Error(null, t); }
	}

	private void jbInit()  throws Throwable {
		  
		this.setLayout(new GridBagLayout());
		
	    // panel sortie
	    this.getContentPane().add(jPanelOutput,  new GridBagConstraints(0, 0, 5, 1, 1.0, 0.0
	            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 0, 5), 0, 0));

	    // nombre d'iterations
	    this.jIFMaxOfIter.setValue(100);

	    // loss of coverage
	    this.getContentPane().add(this.jLabelLossOfCoverage, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
	    		,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
		this.jDFLossOfCoverage.setValue(10);
		this.getContentPane().add(this.jDFLossOfCoverage, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));
		this.getContentPane().add(new JLabel("%"), new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 40, 0));
	    
	    // seuil blanc
	    this.getContentPane().add(this.jLabelBlankThres, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
	            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
	    this.jDFBlankThres.setValue(0.1);
	    this.getContentPane().add(this.jDFBlankThres, new GridBagConstraints(1, 2, 2, 1, 0.0, 0.0
	            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));

	    // nombre de boucles
	    this.getContentPane().add(this.jLabelNumberOfLoops, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
	            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
	    enableNumberOfLoop(false);
	    this.getContentPane().add(this.jIFNumberOfLoops, new GridBagConstraints(1, 3, 2, 1, 0.0, 0.0
	            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));	    
	    
	    // min between MF
	    this.getContentPane().add(this.jLabelMinBetweenMF, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
	            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
	    this.fieldMinBetweenMF.setValue(1d);
	    this.getContentPane().add(this.fieldMinBetweenMF, new GridBagConstraints(1, 4, 2, 1, 0.0, 0.0
	            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));	    
		this.getContentPane().add(new JLabel("%"), new GridBagConstraints(2, 4, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 40, 0));
		enableMinBetweenMF(false);
	    
	    // options avancees
	    this.getContentPane().add(this.jbAdvanced, new GridBagConstraints(0, 5, 3, 1, 0.0, 0.0
	            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
	    this.jbAdvanced.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jbAdvanced_actionPerformed(e); } });

	    // panel input
	    jPanelInputs = new JPanelInputs(this.fis, new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				enableNumberOfLoop(((Boolean)e.getSource()).booleanValue());
			}}, new ActionListener() {

				@Override
				public void actionPerformed(ActionEvent e) {
					enableMinBetweenMF(((Boolean)e.getSource()).booleanValue() || jCheckBoxOutput.isSelected());
				}});
	    this.getContentPane().add(this.jPanelInputs, new GridBagConstraints(0, 6, 3, 1, 1.0, 0.0
	            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 0, 5), 0, 0));
	    
	    // rgles
	    this.getContentPane().add(this.jCheckBoxRules, new GridBagConstraints(0, 7, 3, 1, 1.0, 0.0
	            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 5, 0, 0), 0, 0));
	    
	    // sortie
	    this.getContentPane().add(this.jCheckBoxOutput, new GridBagConstraints(0, 8, 3, 1, 1.0, 0.0
	            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 5, 0, 0), 0, 0));
	    this.jCheckBoxOutput.setVisible(this.fis.GetOutput(this.SelectedOutput()).GetNature() == JOutput.FUZZY);
	    this.jCheckBoxOutput.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				enableMinBetweenMF(((JCheckBox)e.getSource()).isSelected() || jPanelInputs.isInputSelected());
			}
		});
	    
	    // cross validation
	    panelCrossValidation.bind(crossValidationAdapter);
	    this.getContentPane().add(panelCrossValidation, new GridBagConstraints(0, 9, 5, 1, 1.0, 0.0 ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
	    
	    // panel validation
	    JPanel jPanelValidation = new JPanel(new GridBagLayout());
	    //apply button
	    jButtonApply.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonApply_actionPerformed(e); } });
	    jPanelValidation.add(jButtonApply,  new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
	            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));

	    // cancel button
	    jButtonCancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { JFrameSolisWets.this.dispose(); } });
	    jPanelValidation.add(jButtonCancel,  new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
	            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
	    this.getContentPane().add(jPanelValidation, new GridBagConstraints(0, 10, 5, 1, 1.0, 0.0
	            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
	  
	    // affichage
	    this.Translate();
	    this.setModal(true);
	    this.pack();
	  }

	@Override
	public void dispose() {
	    this.crossValidationAdapter.release();
	    this.crossValidation.release();
	    super.dispose();
	}

	private void enableNumberOfLoop(boolean enable) {
		jLabelNumberOfLoops.setEnabled(enable);
		jIFNumberOfLoops.setEnabled(enable);
		jIFNumberOfLoops.setValue(enable ? jIFNumberOfLoops.getValue() : 1);
	}
	
	private void enableMinBetweenMF(boolean enable) {
		jLabelMinBetweenMF.setEnabled(enable);
		fieldMinBetweenMF.setEnabled(enable);
	}
	
	protected void jbAdvanced_actionPerformed(ActionEvent e) {
		  final JDialog dialog = new JFISDialog();
		  dialog.setTitle(LocaleFIS.GetString("AdvancedParameters"));
		  dialog.getContentPane().setLayout(new GridBagLayout());

		  // seed value
		  JLabel jLabelSeedValue = new JLabel(LocaleFIS.GetString("SeedValue"));
		  final IntegerField jIFSeedValue = new IntegerField();
		  jIFSeedValue.setValue(this.SeedValue);
		  dialog.getContentPane().add(jLabelSeedValue, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
		            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
		  dialog.getContentPane().add(jIFSeedValue, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
		            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));

		  // Solis Wets constants
		  JPanel jPanelSolisWets = new JPanel(new GridBagLayout());
		  jPanelSolisWets.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),LocaleFIS.GetString("SolisWetsConstants")));
		  dialog.getContentPane().add(jPanelSolisWets, new GridBagConstraints(0, 1, 2, 1, 1.0, 0.0
		            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 0, 5), 0, 0));
		  final DoubleField jDFl1 = new DoubleField();
		  jDFl1.setValue(this.l1);
		  jDFl1.setEditable(false);
		  final DoubleField jDFl2 = new DoubleField();
		  jDFl2.setValue(this.l2);
		  jDFl2.setEditable(false);
		  final DoubleField jDFl3 = new DoubleField();
		  jDFl3.setValue(this.l3);
		  jDFl3.setEditable(false);
		  jPanelSolisWets.add(new JLabel("l1:"), new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
		            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 0), 0, 0));
		  jPanelSolisWets.add(jDFl1, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
		            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 0), 20, 0));
		  jPanelSolisWets.add(new JLabel("l2:"), new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
		            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 0), 0, 0));
		  jPanelSolisWets.add(jDFl2, new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0
		            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 0), 20, 0));
		  jPanelSolisWets.add(new JLabel("l3:"), new GridBagConstraints(4, 0, 1, 1, 0.0, 0.0
		            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 0), 0, 0));
		  jPanelSolisWets.add(jDFl3, new GridBagConstraints(5, 0, 1, 1, 0.0, 0.0
		            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 20, 0));

		  // ecart type bruit gaussien
		  JLabel jLabelStdGaussianNoise = new JLabel(LocaleFIS.GetString("StdGaussianNoise"));
		  final DoubleField jDFStdGaussianNoise = new DoubleField();
		  dialog.getContentPane().add(jLabelStdGaussianNoise, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
		          ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
		  jDFStdGaussianNoise.setValue(this.StdGaussianNoise);
		  dialog.getContentPane().add(jDFStdGaussianNoise, new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
		          ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));

		  // nombre de contraintes
		  JLabel jLabelMaxOfConstraints = new JLabel(LocaleFIS.GetString("MaximumNumberOfConstraints"));
		  final IntegerField jIFMaxOfConstraints = new IntegerField();
		  dialog.getContentPane().add(jLabelMaxOfConstraints, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
		          ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
		  jIFMaxOfConstraints.setValue(this.MaxOfConstraints);
		  dialog.getContentPane().add(jIFMaxOfConstraints, new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0
		          ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));

		  // nombre d'iterations
		  dialog.getContentPane().add(this.jLabelMaxOfIter, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
			  ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
		  dialog.getContentPane().add(this.jIFMaxOfIter, new GridBagConstraints(1, 4, 1, 1, 0.0, 0.0
			  ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));
		  
		  // panel validation
		  JPanel jPanelValidation = new JPanel();
		  JButton jbApply = new JButton(LocaleFIS.GetString("Apply"));
		  jbApply.addActionListener(new ActionListener()
		    {
		    public void actionPerformed(ActionEvent e)
		      {
		      SeedValue = jIFSeedValue.getValue();
		      StdGaussianNoise = jDFStdGaussianNoise.getValue();
		      MaxOfConstraints = jIFMaxOfConstraints.getValue();
		      dialog.dispose();
		      }
		    });
		  JButton jbCancel = new JButton(LocaleFIS.GetString("Cancel"));
		  jbCancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { dialog.dispose(); } });
		  jPanelValidation.add(jbApply);
		  jPanelValidation.add(jbCancel);
		  dialog.getContentPane().add(jPanelValidation, new GridBagConstraints(0, 5, 2, 1, 1.0, 0.0
		            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));

		  // affichage
		  dialog.pack();
		  dialog.setModal(true);
		  dialog.setResizable(false);
		  dialog.setLocation(JFISChildFrame.ChildPosition(JFrameSolisWets.this, dialog.getSize()));
		  dialog.setVisible(true);
	}

	//------------------------------------------------------------------------------
	  private void Translate()
	  {
	    this.setTitle(LocaleFIS.GetString("Optimization") + " " + LocaleFIS.GetString("SolisWets"));
	    this.jLabelMaxOfIter.setText(LocaleFIS.GetString("MaximumNumberOfIterations"));
	    this.jLabelLossOfCoverage.setText(LocaleFIS.GetString("LossOfCoverage"));
	    this.jLabelDistanceThres.setText(LocaleFIS.GetString("CenterDistanceThreshold"));
	    this.jLabelBlankThres.setText(LocaleFIS.GetString("BlankThres"));
	    this.jLabelNumberOfLoops.setText(LocaleFIS.GetString("NumberOfLoops"));
	    this.jLabelMinBetweenMF.setText(LocaleFIS.GetString("MinBetweenMF"));
	    this.jbAdvanced.setText(LocaleFIS.GetString("AdvancedParameters"));
	    this.jCheckBoxRules.setText(LocaleFIS.GetString("Rules"));
	    this.jCheckBoxOutput.setText(LocaleFIS.GetString("Output"));
	    this.jButtonApply.setText(LocaleFIS.GetString("Apply"));
	    this.jButtonCancel.setText(LocaleFIS.GetString("Cancel"));
	    ((TitledBorder)this.jPanelInputs.getBorder()).setTitle(LocaleFIS.GetString("Inputs"));
	   }
	  
	protected void jButtonApply_actionPerformed(ActionEvent e) {
		  ValidationResult validationResult = new ValidationResult();
		  try
		    {
			validation(validationResult);
			if(validationResult.isEmpty()) {
				WindowUtilities.setCursor(this, Cursor.WAIT_CURSOR);
			    JFIS new_fis = this.fis.NewFISOPT(this.DataFile, this.jPanelInputs.getOptimizationOrder(), this.jCheckBoxOutput.isSelected(), this.jCheckBoxRules.isSelected(), this.jIFNumberOfLoops.getValue(), this.jDFLossOfCoverage.getValue() / 100.0, this.jIFMaxOfIter.getValue(), this.StdGaussianNoise, this.MaxOfConstraints, this.MaxOfFailures, this.SelectedOutput(), this.SeedValue, this.l1, this.l2, this.l3, this.jDFBlankThres.getValue(), this.crossValidation, (Double)this.fieldMinBetweenMF.getValue() / 100d);
			    if( new_fis != null ) { JFISFrame jf = new JFISFrame(new_fis, this.DataFile, this.Parent); }
			    else { MessageFIS.Information(this, LocaleFIS.GetString("NoPerformanceImprovement")); }
			    crossValidation.release();
			    this.dispose();
			} else {
				JOptionPaneValidationResult.show(getWindowAncestor(e.getSource()), validationResult, EMPTY);
			}
		    }
		  catch(Throwable t) { MessageFIS.Error(null, t); }
		  finally {
				WindowUtilities.setCursor(this, Cursor.DEFAULT_CURSOR);
		  }
	}
	
	private void validation(ValidationResult validationResult) {
		if(!jCheckBoxRules.isSelected() && !jPanelInputs.isInputSelected())
			validationResult.addWarning(GetString("SelectElementsToOptimize"));
	}

	//------------------------------------------------------------------------------
	  private int SelectedOutput() throws Throwable
	  {
	  Enumeration items = this.jBGOutput.getElements();
	  for( int i=0 ; i<this.fis.NbOutputs() ; i++ )
	    if( this.fis.GetOutput(i).GetActive() == true )
	      if( ((JRadioButton)items.nextElement()).isSelected() )
	        return i;
	  throw new Exception("NoOutputFound");
	  }
}

class JPanelInputs extends JPanel {

	private static final long serialVersionUID = 1L;
	private Vector<InputCheckBox> inputsCheckBox = new Vector<InputCheckBox>();
	private Vector<InputCheckBox> selectedInputsCheckBox = new Vector<InputCheckBox>();
	private JTextField jTextFieldOptimizationOrder = new JTextField();
	
	public JPanelInputs(JFIS fis, final ActionListener enableNumberOfLoop, final ActionListener enableMinBetweenMF) throws Throwable {
		super();
		this.setBorder(BorderFactory.createTitledBorder(null, "Inputs", TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, new Font("Dialog", Font.BOLD, 12), new Color(51, 51, 51)));
		this.setLayout(new GridBagLayout());
		
		JButton jButtonAllInputs = new JButton(LocaleFIS.GetString("SelectallInputs"));
	    jButtonAllInputs.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonAllInputs_actionPerformed(e); } });
		this.add(jButtonAllInputs, new GridBagConstraints(0, GridBagConstraints.RELATIVE, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		
		for(int inputNumber=0 ; inputNumber<fis.NbInputs() ; inputNumber++) {
			InputCheckBox inputCheckBox = new InputCheckBox(fis, inputNumber);
			inputCheckBox.addItemListener(new ItemListener() {

				@Override
				public void itemStateChanged(ItemEvent itemEvent) {
					InputCheckBox inputCheckBox = (InputCheckBox) itemEvent.getSource();
					if(inputCheckBox.isSelected())
						selectedInputsCheckBox.add(inputCheckBox);
					else
						selectedInputsCheckBox.remove(inputCheckBox);
					showOptimizationOrder();
					enableNumberOfLoop.actionPerformed(new ActionEvent(selectedInputsCheckBox.size() >= 2 ? Boolean.TRUE : Boolean.FALSE, 0, null));
					enableMinBetweenMF.actionPerformed(new ActionEvent(!selectedInputsCheckBox.isEmpty(), 0, null));
				}});
			inputsCheckBox.add(inputCheckBox);
			this.add(inputCheckBox, new GridBagConstraints(0, GridBagConstraints.RELATIVE, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		}
		
		JPanel jPanelOptimizationOrder = new JPanel(new GridBagLayout());
		jPanelOptimizationOrder.add(new JLabel(LocaleFIS.GetString("OptimizationOrder")), new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		jTextFieldOptimizationOrder.setEditable(false);
		jPanelOptimizationOrder.add(jTextFieldOptimizationOrder, new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 0, 0), 0, 0));
		this.add(jPanelOptimizationOrder, new GridBagConstraints(0, GridBagConstraints.RELATIVE, 1, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 0, 5), 0, 0));
	}

	public String getOptimizationOrder() {
		return jTextFieldOptimizationOrder.getText();
	}

	protected void jButtonAllInputs_actionPerformed(ActionEvent e) {
		for(JCheckBox inputCheckBox : inputsCheckBox) 
			inputCheckBox.setSelected(true);
		Collections.sort(selectedInputsCheckBox);
		showOptimizationOrder();
	}	
	
	private void showOptimizationOrder() {
		jTextFieldOptimizationOrder.setText("");
		for(InputCheckBox inputCheckBox : selectedInputsCheckBox) 
			jTextFieldOptimizationOrder.setText((jTextFieldOptimizationOrder.getText().isEmpty() ? "" : jTextFieldOptimizationOrder.getText() + " ") + (inputCheckBox.getInputNumber() + 1));
	}
	
	public boolean isInputSelected() {
		return !selectedInputsCheckBox.isEmpty();
	}
}

class InputCheckBox extends JCheckBox implements Comparable<InputCheckBox> {

	private static final long serialVersionUID = 1L;
	private final int inputNumber;
	
	InputCheckBox(JFIS fis, int inputNumber) throws Throwable {
		super(fis.GetInput(inputNumber).GetName());
		this.inputNumber = inputNumber;
	}
	
	@Override
	public int compareTo(InputCheckBox other) {
		return new Integer(inputNumber).compareTo(other.inputNumber);
	}
	
	int getInputNumber() {
		return inputNumber;
	}
}
