//***********************************************************************
//
//                              JFuzzyGraphFrame.java
//
// Author(s) : Anne Tireau
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.SpinnerListModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.JLabel;

import org.freehep.util.export.ExportDialog;



public class JFuzzyGraphFrame extends JFISChildFrame // implements ChangeListener
{
	
	//attributs
	private JScrollPane jScrollPaneTree;
	private GridBagLayout gridBagLayout = new GridBagLayout();
	private JPanel jPanelDisplay = new JPanel();
	private GridBagLayout gridBLDisplay = new GridBagLayout();
	private TitledBorder titledBorderDisplay;
	private JSpinner jSpinnerDeep = new JSpinner();
	private JSpinner jSpinnerFont = new JSpinner();
	private JSpinner jSpinnerScale = new JSpinner();	
	private JLabel jLabelSpinnerDeep = new JLabel();
	private JLabel jLabelSpinnerFont = new JLabel();
	private JLabel jLabelSpinnerScale = new JLabel();
	private JMenuItem jMenuItemExport = null;
	private JMenuItem jMenuItemPrint = null;
	private JMenuBar jJMenuBar = null;
	private JMenu jMenuFile = null;
	private ButtonGroup buttonGroupDisplayNodes = new ButtonGroup();
	private JRadioButton jRBAllNodes = new JRadioButton();
	private JRadioButton jRBLeafs = new JRadioButton();
	
	private String FileName;
	private FuzzyGraph jFuzzyGraph;
	
//---------------------------------------------------------------------------	
//constructeur
public 	JFuzzyGraphFrame(JFISFrame parent, String file_name)
{
	super(parent);
	this.FileName = file_name;
	try
	{
		this.jFuzzyGraph = new FuzzyGraph(this.FileName);
		
		jFuzzyGraph.setEditable(false);
		jFuzzyGraph.setGridEnabled(true);
		jFuzzyGraph.setGridVisible(true);
		
		jbInit();
	}
	catch(Throwable t) { MessageFIS.Error(null, t); }
}


//------------------------------------------------------------------------------
private void jbInit() throws Throwable
{
  //JMenuBar jmb = new JMenuBar();
  this.setJMenuBar(getJJMenuBar());
  //jmb.add(getJMenuItemExport());
  //this.setJMenuBar(jmb);

  titledBorderDisplay = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"");
  this.getContentPane().setLayout(gridBagLayout);
  this.jScrollPaneTree = new JScrollPane(this.jFuzzyGraph)
  {
      public Dimension getMinimumSize() { return  new Dimension(300,100); }
      };
  jPanelDisplay.setLayout(gridBLDisplay);
  jPanelDisplay.setBorder(titledBorderDisplay);
  this.getContentPane().add(jScrollPaneTree,      new GridBagConstraints(0, 1, 5, 1, 1.0, 1.0
         ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
  this.getContentPane().add(jPanelDisplay,       new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
          ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 0, 5), 0, 0));
  // boutons radio
  buttonGroupDisplayNodes.add(jRBAllNodes);
  buttonGroupDisplayNodes.add(jRBLeafs);
  jPanelDisplay.add(jRBAllNodes,  new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
          ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
  jPanelDisplay.add(jRBLeafs,   new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
          ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
  class DisplayActionListener implements ActionListener
  {
	  public void actionPerformed(ActionEvent e)
	    {
		  JFuzzyGraphFrame.this.jFuzzyGraph.DisplayAllCell = JFuzzyGraphFrame.this.jRBAllNodes.isSelected();
		  JFuzzyGraphFrame.this.jFuzzyGraph.InfoGraph();
	    }
  }
  this.jRBAllNodes.addActionListener(new DisplayActionListener());
  this.jRBLeafs.addActionListener(new DisplayActionListener());
  if( JFuzzyGraphFrame.this.jFuzzyGraph.DisplayAllCell == true )
	  this.jRBAllNodes.setSelected(true);
  else
	  this.jRBLeafs.setSelected(true);

  // affichage des checkBoxs
  for( int i=0 ; i<this.jFuzzyGraph.VariablesGroups.size() ; i++ )
    {	  
    final int index = i;
    final DisplayVariableGroup dv = (DisplayVariableGroup)this.jFuzzyGraph.VariablesGroups.elementAt(i);
    JCheckBox cb = new JCheckBox()
      {
      public String getText() // retourne le texte de la check box traduit
        {
        if( dv != null )
          return LocaleFIS.GetString(dv.GroupName());
        return null;
        }
      };     
	  cb.setSelected(dv.Display());   // selectionne la check box si le groupe est affichable
      cb.addActionListener(new ActionListener() // ActionListener de la check box
      {
      public void actionPerformed(ActionEvent e)
        {
	        DisplayVariableGroup dv = (DisplayVariableGroup)JFuzzyGraphFrame.this.jFuzzyGraph.VariablesGroups.elementAt(index);
	        dv.Display(((JCheckBox)e.getSource()).isSelected());
	        JFuzzyGraphFrame.this.jFuzzyGraph.InfoGraph();
        }
      });	  
	  jPanelDisplay.add(cb,  new GridBagConstraints(0, i+1, 2, 1, 0.0, 0.0
	            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    }
  //spinner
  jPanelDisplay.add(jLabelSpinnerDeep,  new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
          ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 20, 0, 0), 0, 0));

  SpinnerNumberModel spiM = new SpinnerNumberModel(this.jFuzzyGraph.getDeepGraph(),0,this.jFuzzyGraph.getDeepGraph(),1);
  jSpinnerDeep.setModel(spiM);
  jSpinnerDeep.addChangeListener(new ChangeListener()
  	{
	  public void stateChanged(ChangeEvent e) 
		{	
			JSpinner source = (JSpinner)e.getSource();
			Number valeur = (Number)source.getValue();
			int v = valeur.intValue();
			JFuzzyGraphFrame.this.jFuzzyGraph.DisplayDeepGraph(v);			
		}
  	});
  jPanelDisplay.add(jSpinnerDeep,  new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
		  ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
  
  jPanelDisplay.add(jLabelSpinnerFont,  new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0
          ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 20, 0, 0), 0, 0));
  SpinnerNumberModel spinM = new SpinnerNumberModel(FuzzyGraph.FONTSIZE,8,16,1);
  jSpinnerFont.setModel(spinM);
  jSpinnerFont.addChangeListener(new ChangeListener()
  	{
	  public void stateChanged(ChangeEvent e) 
		{	
			JSpinner source = (JSpinner)e.getSource();
			Number valeur = (Number)source.getValue();
			int v = valeur.intValue();
			JFuzzyGraphFrame.this.jFuzzyGraph.setFontSizeGraph(v);		
		}
  	});
  jPanelDisplay.add(jSpinnerFont,  new GridBagConstraints(2, 3, 1, 1, 0.0, 0.0
		  ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
  
  jPanelDisplay.add(jLabelSpinnerScale,  new GridBagConstraints(1, 4, 1, 1, 0.0, 0.0
          ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 20, 0, 0), 0, 0));
  SpinnerNumberModel spinM1 = new SpinnerNumberModel(100,50,120,10);
  jSpinnerScale.setModel(spinM1);
  jSpinnerScale.addChangeListener(new ChangeListener()
  	{
	  public void stateChanged(ChangeEvent e) 
		{	
			JSpinner source = (JSpinner)e.getSource();
			Number valeur = (Number)source.getValue();
			Double v = valeur.doubleValue();
			JFuzzyGraphFrame.this.jFuzzyGraph.setScaleGraph(v);		
		}
  	});
  jPanelDisplay.add(jSpinnerScale,  new GridBagConstraints(2, 4, 1, 1, 0.0, 0.0
		  ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
  
  // affichage
  this.Translate();
  this.pack();
  this.setLocation(this.ChildPosition(this.getSize()));
  this.setVisible(true);
  
}

//---------------------------------------------------------------------------	
	public void Translate() throws Throwable {
		// TODO Raccord de mthode auto-gnr
		  this.setTitle(LocaleFIS.GetString("Tree") + ": " + this.FileName);
		  this.titledBorderDisplay.setTitle(LocaleFIS.GetString("Display"));
		  this.jLabelSpinnerDeep.setText(LocaleFIS.GetString("GraphProfondeur") + " ");
		  this.jLabelSpinnerFont.setText(LocaleFIS.GetString("GraphFont") + " ");
		  this.jLabelSpinnerScale.setText(LocaleFIS.GetString("GraphEchelle") + " ");
		  this.jRBLeafs.setText(LocaleFIS.GetString("Leafs"));
		  this.jRBAllNodes.setText(LocaleFIS.GetString("AllNodes"));		  
		  this.repaint();
	}
//	---------------------------------------------------------------------------
	private JMenuItem getJMenuItemExport() {
        if (jMenuItemExport == null) {
            jMenuItemExport = new JMenuItem();
            jMenuItemExport.setText(LocaleFIS.GetString("Export"));
            jMenuItemExport.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    ExportDialog export = new ExportDialog();
                    //export.showExportDialog(JFuzzyGraphFrame.this, LocaleFIS.GetString("ExportViewAs"), jScrollPaneTree, "export");
                    export.showExportDialog(JFuzzyGraphFrame.this, LocaleFIS.GetString("ExportViewAs"), jScrollPaneTree, "export");
                }
            });
        }
        return jMenuItemExport;
    }
//	---------------------------------------------------------------------------
	private JMenuItem getJMenuItemPrint() {
        if (jMenuItemPrint == null) {
        	jMenuItemPrint = new JMenuItem();
        	jMenuItemPrint.setText(LocaleFIS.GetString("Print"));
        	jMenuItemPrint.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                	  PrinterJob job = PrinterJob.getPrinterJob();
                	  if( job.printDialog() == true )
                	    {
                	    job.setPrintable(new Printable()
                	      {
                	      public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) throws PrinterException
                	        {
                	        if (pageIndex >= 1) { return Printable.NO_SUCH_PAGE; }
                	        else return JFuzzyGraphFrame.this.print(graphics,pageFormat,pageIndex);
                	        }
                	      });
                	    try { job.print(); }
                	    catch (PrinterException ex) { ex.printStackTrace(); }
                	    }
                }
            });
        }
        return jMenuItemPrint;
    }
//	------------------------------------------------------------------------------
	  public int print( java.awt.Graphics g, PageFormat pageFormat, int pageIndex ) throws PrinterException
	 {
	     java.awt.Graphics2D  g2 = ( java.awt.Graphics2D )g;
	     double scalew=1;
	     double scaleh=1;
	     double pageHeight = pageFormat.getImageableHeight();
	     // See how much we need to scale the image to fit in the page's width
	     double pageWidth = pageFormat.getImageableWidth();
	     if(  getWidth() >= pageWidth ) { scalew =  pageWidth / getWidth(); }
	     if(  getHeight() >= pageHeight) { scaleh =  pageWidth / getHeight(); }
	     g2.translate( pageFormat.getImageableX(), pageFormat.getImageableY() );
	     g2.scale( scalew, scaleh );
	     this.jScrollPaneTree.print( g2 );
	     return Printable.PAGE_EXISTS;
	 }
// ---------------------------------------------------------------------------	  
	private JMenuBar getJJMenuBar() {
        if (jJMenuBar == null) {
            jJMenuBar = new JMenuBar();
            jJMenuBar.add(getJMenuFile());
        }
        return jJMenuBar;
    }
//	---------------------------------------------------------------------------  
    private JMenu getJMenuFile() {
        if (jMenuFile == null) {
            jMenuFile = new JMenu();
            jMenuFile.setText(LocaleFIS.GetString("File"));
            jMenuFile.add(getJMenuItemExport());
            jMenuFile.add(getJMenuItemPrint());
        }
        return jMenuFile;
    }
}


