//***********************************************************************
//
//                              JFuzzyTreeFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import javax.swing.*;
import java.awt.*;
import javax.swing.tree.*;
import javax.swing.plaf.basic.*;
import java.util.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.border.*;
import javax.swing.event.*;
import util.treetable.*;
import util.tableheader.*;
import javax.swing.table.*;
import java.awt.print.*;

//------------------------------------------------------------------------------
class FuzzyJTreeTable extends JTreeTable
{
//------------------------------------------------------------------------------
  public FuzzyJTreeTable(FuzzyTreeTableModel treeTableModel)
  {
  super(treeTableModel);
  this.setAutoCreateColumnsFromModel(true);
  // dissine les branches de l'arbre
  this.tree.putClientProperty("JTree.lineStyle", "Angled");
  // suppression des icones de l'arbre
  DefaultTreeCellRenderer dtr = (DefaultTreeCellRenderer)this.tree.getCellRenderer();
  dtr.setLeafIcon(null);
  dtr.setIcon(null);
  dtr.setDisabledIcon(null);
  dtr.setClosedIcon(null);
  dtr.setOpenIcon(null);
  // deploie les branches de l'arbre
  for( int i=0 ; i<this.tree.getRowCount() ; i++ )
    this.tree.expandRow(i);
  // entete des colonnes de type multi-ligne
  for( int i=0 ; i<this.getColumnModel().getColumnCount() ; i++ )
    this.getColumnModel().getColumn(i).setHeaderRenderer(new MultiLineHeaderRenderer()
      {
      public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
        {
        return super.getTableCellRendererComponent(table, LocaleFIS.GetStringNoException(FuzzyJTreeTable.this.getModel().getColumnName(column)), isSelected, hasFocus, row, column);
        }
      });
  // centrage des colonnes
  for( int i=1 ; i<this.getColumnModel().getColumnCount() ; i++ )
    {
    this.getColumnModel().getColumn(i).setCellRenderer(new DefaultTableCellRenderer()
      {
      public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
        {
        super.getTableCellRendererComponent( table, value, isSelected, hasFocus, row, column);
        // centrage des elements
        super.setHorizontalAlignment(SwingConstants.CENTER);
        return this;
        }
      } );
    }

  // affichage des groupes de colonnes
  for( int i=0 ; i<treeTableModel.fuzzy_tree.VariablesGroups.size() ; i++ )
    {
    final DisplayVariableGroup dvg = (DisplayVariableGroup)treeTableModel.fuzzy_tree.VariablesGroups.elementAt(i);
    if( (dvg.VariablesNamesSize() > 1) && (dvg.Display() == true) )   // c'est un groupe de colonnes affichables
      {
      TableColumnModel cm = this.getColumnModel();
      ColumnGroup cg = new ColumnGroup(new String()); // chaine vide, c'est le renderer qui fait la traduction et affiche le nom de la colonne
      // recherche des colonnes appartenant au groupe en comparant les noms de colonnes
      for( int j=0 ; j<dvg.VariablesNamesSize() ; j++ )
        for( int k=0 ; k<cm.getColumnCount() ; k++ )
          if( dvg.VariableName(j).equals(cm.getColumn(k).getHeaderValue()) )
            cg.add(cm.getColumn(k));
      // ajout du groupe au header
      GroupableTableHeader header = (GroupableTableHeader)this.getTableHeader();
      header.addColumnGroup(cg);
      // renderer du groupe de colonne
      cg.setHeaderRenderer(new DefaultTableCellRenderer()
        {
        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
          {
          super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
          this.setHorizontalAlignment(SwingConstants.CENTER);
          this.setForeground(table.getTableHeader().getForeground());
          this.setBackground(table.getTableHeader().getBackground());
          this.setBorder(UIManager.getBorder("TableHeader.cellBorder"));
          this.setText(LocaleFIS.GetString(dvg.GroupName()));
          return this;
          }
        });
      }
    }

  // largeur de la premiere colonne
  TableColumnModel cm = this.getColumnModel();
  cm.getColumn(0).setPreferredWidth(treeTableModel.fuzzy_tree.getPreferredSize().width);
  }
//------------------------------------------------------------------------------
  // l'objet TableHeader par defaut est de type GroupableTableHeader
  protected JTableHeader createDefaultTableHeader() { return new GroupableTableHeader(columnModel); }
}

//------------------------------------------------------------------------------
public class JFuzzyTreeFrame extends JFISChildFrame
{
  private JScrollPane jScrollPaneTree;
  private GridBagLayout gridBagLayout = new GridBagLayout();
  private JPanel jPanelDisplay = new JPanel();
  private GridBagLayout gridBLValidation = new GridBagLayout();
  private GridBagLayout gridBLDisplay = new GridBagLayout();
  private TitledBorder titledBorderDisplay;
  private ButtonGroup buttonGroupDisplayNodes = new ButtonGroup();
  private JRadioButton jRBAllNodes = new JRadioButton();
  private JRadioButton jRBLeafs = new JRadioButton();
  private JMenuItem jMenuPrint = new JMenuItem();

  private FuzzyTree jFuzzyTree;           // arbre flou
  private JTreeTable jFuzzyTreeGraph;     // representation graphique de l'arbre flou
  private String FileName;
//------------------------------------------------------------------------------
  public JFuzzyTreeFrame( JFISFrame parent, String file_name )
  {
    super(parent);
    this.FileName = file_name;
    try
    {
      jFuzzyTree = new FuzzyTree(file_name);
      jFuzzyTreeGraph = new FuzzyJTreeTable(new FuzzyTreeTableModel(jFuzzyTree));

      jbInit();
    }
    catch(Throwable t) { MessageFIS.Error(null, t); }
  }
//------------------------------------------------------------------------------
  private void jbInit() throws Throwable
  {
    JMenuBar jmb = new JMenuBar();
    jmb.add(this.jMenuPrint);
    this.setJMenuBar(jmb);

    titledBorderDisplay = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"");
    this.getContentPane().setLayout(gridBagLayout);
    this.jScrollPaneTree = new JScrollPane(this.jFuzzyTreeGraph)
      {
      public Dimension getMinimumSize() { return  new Dimension(300,100); }
      };
    this.jFuzzyTreeGraph.setPreferredScrollableViewportSize(this.jFuzzyTreeGraph.getPreferredSize());

    jPanelDisplay.setLayout(gridBLDisplay);
    jPanelDisplay.setBorder(titledBorderDisplay);
    buttonGroupDisplayNodes.add(jRBAllNodes);
    buttonGroupDisplayNodes.add(jRBLeafs);
    this.getContentPane().add(jScrollPaneTree,      new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
    this.getContentPane().add(jPanelDisplay,       new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 0, 5), 0, 0));
    jMenuPrint.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuPrint_actionPerformed(e); } });
    jPanelDisplay.add(jRBAllNodes,  new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    jPanelDisplay.add(jRBLeafs,   new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));

    // radio bouton all nodes /leafs
    class DisplayActionListener implements ActionListener
      {
      public void actionPerformed(ActionEvent e)
        {
        JFuzzyTreeFrame.this.jFuzzyTree.DisplayAllNodes = JFuzzyTreeFrame.this.jRBAllNodes.isSelected();
        JFuzzyTreeFrame.this.jFuzzyTreeGraph.repaint();
        }
      }
    this.jRBAllNodes.addActionListener(new DisplayActionListener());
    this.jRBLeafs.addActionListener(new DisplayActionListener());
    if( JFuzzyTreeFrame.this.jFuzzyTree.DisplayAllNodes == true )
      this.jRBAllNodes.setSelected(true);
    else
      this.jRBLeafs.setSelected(true);

    // affichage des checkBoxs
    for( int i=0 ; i<this.jFuzzyTree.VariablesGroups.size() ; i++ )
      {
      final int index = i;
      final DisplayVariableGroup dv = (DisplayVariableGroup)this.jFuzzyTree.VariablesGroups.elementAt(i);
      JCheckBox cb = new JCheckBox()
        {
        public String getText() // retourne le texte de la check box traduit
          {
          if( dv != null )
            return LocaleFIS.GetString(dv.GroupName());
          return null;
          }
        };
      cb.setSelected(dv.Display());   // selectionne la check box si le groupe est affichable
      cb.addActionListener(new ActionListener() // ActionListener de la check box
        {
        public void actionPerformed(ActionEvent e)
          {
          DisplayVariableGroup dv = (DisplayVariableGroup)JFuzzyTreeFrame.this.jFuzzyTree.VariablesGroups.elementAt(index);
          dv.Display(((JCheckBox)e.getSource()).isSelected());
          JFuzzyTreeFrame.this.jScrollPaneTree.getViewport().remove(JFuzzyTreeFrame.this.jFuzzyTreeGraph);
          JFuzzyTreeFrame.this.jFuzzyTreeGraph = new FuzzyJTreeTable(new FuzzyTreeTableModel(jFuzzyTree));
          JFuzzyTreeFrame.this.jScrollPaneTree.getViewport().add(JFuzzyTreeFrame.this.jFuzzyTreeGraph);
          JFuzzyTreeFrame.this.repaint();
          }
        });
      jPanelDisplay.add(cb,  new GridBagConstraints(0, i+1, 2, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
      }

    // affichage
    this.Translate();
    this.pack();
    this.setLocation(this.ChildPosition(this.getSize()));
    this.setVisible(true);
  }
//------------------------------------------------------------------------------
public void Translate() throws Throwable
  {
  this.setTitle(LocaleFIS.GetString("Tree") + ": " + this.FileName);
  this.titledBorderDisplay.setTitle(LocaleFIS.GetString("Display"));
  this.jRBLeafs.setText(LocaleFIS.GetString("Leafs"));
  this.jRBAllNodes.setText(LocaleFIS.GetString("AllNodes"));
  this.jMenuPrint.setText(LocaleFIS.GetString("Print"));
  this.repaint();
  }
//------------------------------------------------------------------------------
  void jMenuPrint_actionPerformed(ActionEvent e)
  {
  PrinterJob job = PrinterJob.getPrinterJob();
  if( job.printDialog() == true )
    {
    job.setPrintable(new Printable()
      {
      public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) throws PrinterException
        {
        if (pageIndex >= 1) { return Printable.NO_SUCH_PAGE; }
        else return JFuzzyTreeFrame.this.print(graphics,pageFormat,pageIndex);
        }
      });
    try { job.print(); }
    catch (PrinterException ex) { ex.printStackTrace(); }
    }
  }
//------------------------------------------------------------------------------
  public int print( java.awt.Graphics g, PageFormat pageFormat, int pageIndex ) throws PrinterException
 {
     java.awt.Graphics2D  g2 = ( java.awt.Graphics2D )g;
     double scalew=1;
     double scaleh=1;
     double pageHeight = pageFormat.getImageableHeight();
     // See how much we need to scale the image to fit in the page's width
     double pageWidth = pageFormat.getImageableWidth();
     if(  getWidth() >= pageWidth ) { scalew =  pageWidth / getWidth(); }
     if(  getHeight() >= pageHeight) { scaleh =  pageWidth / getHeight(); }
     g2.translate( pageFormat.getImageableX(), pageFormat.getImageableY() );
     g2.scale( scalew, scaleh );
     this.jScrollPaneTree.print( g2 );
     return Printable.PAGE_EXISTS;
 }
}

