//***********************************************************************
//
//                              JHFPConfigFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.*;
import java.io.*;
import javax.swing.*;

import java.awt.event.*;
import javax.swing.border.*;

//------------------------------------------------------------------------------
public class JHFPConfigFrame extends JFISDialog
{
  private JLabel jLabelHierarchy = new JLabel();
  private JComboBox jCBHierarchy = new JComboBox();
  private DefaultComboBoxModel jCBHierarchyModel = new DefaultComboBoxModel()
    {
    // c'est cette methode qui assure la traduction des types de hierarchie
    public Object getElementAt(int index)
      {
      try { return LocaleFIS.GetString(JFISHFP.HierarchyType()[index]); }
      catch( Throwable t ) { return null; }
      }
    };
  private JComboBox jCBDistance = new JComboBox();
  private DefaultComboBoxModel jCBDistanceModel = new DefaultComboBoxModel()
    {
    // c'est cette methode qui assure la traduction des types de hierarchie
    public Object getElementAt(int index)
      {
      try { return LocaleFIS.GetString(JFISHFP.DistanceType()[index]); }
      catch( Throwable t ) { return null; }
      }
    };
  private JFISHFP hfp;
  private JPanel jPanelInitialPartition = new JPanel(new GridBagLayout());
  private ButtonGroup jbgInitialPartition = new ButtonGroup();
  private JRadioButton jbNbGroups;
  private JRadioButton jbToleranceThreshold;
  private IntegerField jIFNbGroups = new IntegerField();
  private DoubleField jDFToleranceThres = new DoubleField();
  private JFISFrame Parent;
  private JExtendedDataFile DataFile;
  private JPanel jPanelVariables = new JPanel(new GridBagLayout());
  private JScrollPane jScrollPaneVariables;
  private JPanel jPanelValidation = new JPanel(new GridBagLayout());
  private JButton jButtonSave = new JButton();
  private JButton jButtonSaveAs = new JButton();
  private JButton jButtonCancel = new JButton();
  private String HFPFile = null;
  private JCheckBox jCBOutput = new JCheckBox(LocaleFIS.GetString("Output"));
  private DefaultComboBoxModel jCBConjunctionModel = new DefaultComboBoxModel()
    {
    // c'est cette methode qui assure la traduction des types de conjonction
    public Object getElementAt(int index)
      {
      try { return LocaleFIS.GetString(JFIS.TypeConjunction()[index]); }
      catch( Throwable t ) { return null; }
      }
    };
  private JComboBox jCBConjunction = new JComboBox(this.jCBConjunctionModel);
//------------------------------------------------------------------------------
  public JHFPConfigFrame( JFISFrame parent, JExtendedDataFile data ) throws Throwable
  {
    this.Parent = parent;
    this.DataFile = data;
    try
      {
      this.hfp = JFISHFP.HFPConfig(this.DataFile.ActiveFileName(), 1, "temp.hfp");
      jbInit();
      }
    catch(Throwable t)
      {
      MessageFIS.Error(null, t);
      this.hfp.Delete();
      }
  }
//------------------------------------------------------------------------------
  public JHFPConfigFrame( JFISFrame parent, JExtendedDataFile data, String hfp_file ) throws Throwable
  {
    this.Parent = parent;
    this.DataFile = data;
    this.HFPFile = hfp_file;
    try
      {
      this.hfp = new JFISHFP(this.DataFile.ActiveFileName(), hfp_file);
      jbInit();
      }
    catch(Throwable t)
      {
      MessageFIS.Error(null, t);
      this.hfp.Delete();
      }
  }
//------------------------------------------------------------------------------
  private void jbInit() throws Throwable
  {
    this.getContentPane().setLayout(new GridBagLayout());

    // type de hierarchie
    this.getContentPane().add(jLabelHierarchy,      new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    for( int i=0 ; i<JFISHFP.HierarchyType().length ; i++ )
      this.jCBHierarchyModel.addElement(new String());  // on met des String vide dans le modele, c'est la methode getElementAt qui fait la traduction
    this.jCBHierarchy.setModel(this.jCBHierarchyModel);
    this.getContentPane().add(jCBHierarchy,       new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, 0));
    for( int i=0 ; i<JFISHFP.DistanceType().length ; i++ )
        this.jCBDistanceModel.addElement(new String());  // on met des String vide dans le modele, c'est la methode getElementAt qui fait la traduction
    this.jCBDistance.setModel(this.jCBDistanceModel);
    Object [] params = this.hfp.GetParameters();
    String [] types = JFISHFP.DistanceType();
    for( int i=0 ; i<types.length ; i++ )
      if( params[0].equals(types[i]) )
        this.jCBDistance.setSelectedIndex(i);
   this.getContentPane().add(jCBDistance,      new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, 0));

    // partition initiale
    this.jPanelInitialPartition.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)), LocaleFIS.GetString("InitialPartition")));
    this.getContentPane().add(this.jPanelInitialPartition, new GridBagConstraints(0, 1, 3, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 0, 5), 0, 0));
    this.jbNbGroups = new JRadioButton(LocaleFIS.GetString("NbGroups") + ":");
    this.jbNbGroups.addItemListener(new java.awt.event.ItemListener() { public void itemStateChanged(ItemEvent e) { jIFNbGroups.setEnabled(jbNbGroups.isSelected()); } });
    this.jbToleranceThreshold = new JRadioButton(LocaleFIS.GetString("ToleranceThreshold") + ":");
    this.jbToleranceThreshold.addItemListener(new java.awt.event.ItemListener() { public void itemStateChanged(ItemEvent e) { jDFToleranceThres.setEnabled(jbToleranceThreshold.isSelected()); } });
    this.jbgInitialPartition.add(jbNbGroups);
    this.jbgInitialPartition.add(jbToleranceThreshold);
    //this.jbNbGroups.setSelected(true);
    this.jPanelInitialPartition.add(this.jbNbGroups, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
    this.jPanelInitialPartition.add(this.jbToleranceThreshold, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    double tol_value = 100; // valeur par defaut de nb_groups
    if( this.HFPFile != null )  // lecture du seuil depuis le fichier hfp
      tol_value = this.hfp.GetToleranceThreshold();
    if( tol_value >= 1 ) { this.jIFNbGroups.setValue((int)tol_value); this.jDFToleranceThres.setValue(0.01); this.jbNbGroups.setSelected(true); }
    else { this.jIFNbGroups.setValue(100); this.jDFToleranceThres.setValue(tol_value); this.jbToleranceThreshold.setSelected(true); }
    this.jPanelInitialPartition.add(this.jIFNbGroups, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 40, 0));
    this.jPanelInitialPartition.add(this.jDFToleranceThres, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 40, 0));

    // chekbox sortie
    this.jCBOutput.setSelected(this.hfp.GetNbOutput() == 0 ? false : true);
    this.getContentPane().add(this.jCBOutput,      new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, 0));

    // conjunction
    JPanel panel_conj = new JPanel(new GridBagLayout());
    for( int i=0 ; i<JFIS.TypeConjunction().length ; i++ )
      this.jCBConjunctionModel.addElement(new String());  // on met des String vide dans le modele, c'est la methode getElementAt qui fait la traduction
    panel_conj.add(new JLabel(LocaleFIS.GetString("Conjunction")+":"),   new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 0, 5), 0, 0));
    panel_conj.add(this.jCBConjunction,   new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
    this.getContentPane().add(panel_conj,   new GridBagConstraints(1, 3, 2, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));

    // init des variables
    this.InitVariables();

    // panel validation
    if( this.HFPFile == null )
      this.jButtonSave.setEnabled(false);
    jPanelValidation.add(jButtonSave,  new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    jPanelValidation.add(jButtonSaveAs,  new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
    jPanelValidation.add(jButtonCancel,  new GridBagConstraints(2, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
    this.getContentPane().add(jPanelValidation,         new GridBagConstraints(0, 6, 4, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));

    // events
    this.jCBHierarchy.addItemListener(new ItemListener() { public void itemStateChanged(ItemEvent e) { jCBHierarchy_itemStateChanged(e); }});
    jButtonSave.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonSave_actionPerformed(e); } });
    jButtonSaveAs.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonSaveAs_actionPerformed(e); } });
    jButtonCancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonCancel_actionPerformed(e); } });
    this.jCBOutput.addItemListener(new java.awt.event.ItemListener() { public void itemStateChanged(ItemEvent e) { jCBOutput_itemStateChanged(e); } });
    this.addWindowListener(new WindowAdapter() { public void windowClosing(WindowEvent e) { this_windowClosing(e); } });

    // affichage
    this.jCBHierarchy.setSelectedIndex(this.HFPFile == null ? JFISHFP.KMEANS : this.hfp.GetHierarchy());
    this.jCBConjunction.setSelectedIndex(this.HFPFile == null ? JFISHFP.MINIMUM : this.hfp.GetConjunction());
    this.Translate();
    this.pack();
    this.setModal(true);
    this.setLocation(JFISChildFrame.ChildPosition(this.Parent, this.getSize()));
    this.setVisible(true);
  }
//------------------------------------------------------------------------------
  private void InitVariables() throws Throwable
  {
    if( this.jScrollPaneVariables != null )
      this.getContentPane().remove(this.jScrollPaneVariables);
    this.jPanelVariables = new JPanel(new GridBagLayout());
    this.jScrollPaneVariables = new JScrollPane(this.jPanelVariables) { public Dimension getMinimumSize() { return new Dimension(0, Math.min(200, JHFPConfigFrame.this.jPanelVariables.getMinimumSize().height)); } };
    this.getContentPane().add(this.jScrollPaneVariables,      new GridBagConstraints(0, 5, 4, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 0, 0), 0, 0));
    int x = 0;
    int y = 0;
    for( int i=0 ; i<this.hfp.GetNbInput() ; i++ )
      {
      final int input = i;
      JButton button = new JButton()
        {
        public String getText()
          {
          try
            {
            JInput hfp_input = JHFPConfigFrame.this.hfp.GetInput(input);
            this.setText(hfp_input.GetName());
            if( hfp_input.GetActive() == false )
              this.setForeground(Color.gray);
            else
              this.setForeground(Color.black);
            }
          catch( Throwable t ) {}
          return super.getText();
          }
        };
      button.addActionListener(new ActionListener()
        {
        public void actionPerformed(ActionEvent e)
          {
          try { JHFPInputFrame jif = new JHFPInputFrame(JHFPConfigFrame.this.Parent, JHFPConfigFrame.this.hfp, input); }
          catch( Throwable t ) { MessageFIS.Error( null, t ); }
          }
        });
      this.jPanelVariables.add(button, new GridBagConstraints(x, y, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(y==0 ? 5 : 0, 0, 5, 5), 0, 0));
      x++;
      if( x == 3 )
        {
        x = 0;
        y++;
        }
      }
    for( int i=0 ; i<this.hfp.GetNbOutput() ; i++ )
      {
      final int output = i;
      JButton button = new JButton(this.hfp.GetOutput(i).GetName())
        {
        public String getText()
          {
          try
            {
            if( JHFPConfigFrame.this.hfp.GetOutput(output).GetActive() == false )
              this.setForeground(Color.gray);
            else
              this.setForeground(Color.black);
            }
          catch( Throwable t ) {}
          return super.getText();
          }
        };
      button.addActionListener(new ActionListener()
        {
        public void actionPerformed(ActionEvent e)
          {
          try
            {
            JHFPOutputFrame jif = new JHFPOutputFrame(JHFPConfigFrame.this.Parent, JHFPConfigFrame.this.hfp, output);
            jif.Show(null);
            }
          catch( Throwable t ) { MessageFIS.Error( null, t ); }
          }
        });
      this.jPanelVariables.add(button, new GridBagConstraints(x, y, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(y==0 ? 5 : 0, 0, 5, 5), 0, 0));
      x++;
      if( x == 3 )
        {
        x = 0;
        y++;
        }
      }
  }
//------------------------------------------------------------------------------
  private void Translate()
  {
    this.setTitle(LocaleFIS.GetString("HFP") + (this.HFPFile == null ? "" : ": " + this.HFPFile));
    jLabelHierarchy.setText(LocaleFIS.GetString("Hierarchy") + ":");
    jButtonSave.setText(LocaleFIS.GetString("Save"));
    jButtonSaveAs.setText(LocaleFIS.GetString("SaveAs"));
    jButtonCancel.setText(LocaleFIS.GetString("Cancel"));
  }
//------------------------------------------------------------------------------
  void jCBHierarchy_itemStateChanged(ItemEvent e)
  {
  if( e.getStateChange() == ItemEvent.SELECTED )
    {
    if( this.jCBHierarchy.getSelectedIndex() != 0 )
      {
        this.jCBDistance.setEnabled(false);
        this.jbNbGroups.setEnabled(false);
        this.jbToleranceThreshold.setEnabled(false);
        this.jIFNbGroups.setEnabled(false);
        this.jDFToleranceThres.setEnabled(false);
      }
    else
      {
        this.jCBDistance.setEnabled(true);
        this.jbNbGroups.setEnabled(true);
        this.jbToleranceThreshold.setEnabled(true);
        this.jIFNbGroups.setEnabled(this.jbNbGroups.isSelected());
        this.jDFToleranceThres.setEnabled(this.jbToleranceThreshold.isSelected());
      }
    }
  }
//------------------------------------------------------------------------------
  void jCBOutput_itemStateChanged(ItemEvent e)
  {
  try
    {
    this.hfp = JFISHFP.HFPConfig(this.DataFile.ActiveFileName(), this.jCBOutput.isSelected() == true ? 1 : 0, "temp.hfp");
    this.InitVariables();
    this.pack();
    }
  catch(Throwable t) { MessageFIS.Error(null, t); }
  }
//------------------------------------------------------------------------------
  void jButtonCancel_actionPerformed(ActionEvent e)
  {
  this.dispose();

  // suppression du fichier HFP temporaire
  File f = new File("temp.hfp");
  f.delete();

  try { this.hfp.Delete(); }
  catch( Throwable except ) { MessageFIS.Error( null, except ); }
  this.HFPFile = null;
  }
//------------------------------------------------------------------------------
  void jButtonSave_actionPerformed(ActionEvent e)
  {
  try
    {
    // sauvegarde du fichier hfp
	this.hfp.SetParameters(this.jCBDistance.getSelectedIndex()); 
    this.hfp.SetHierarchy(this.jCBHierarchy.getSelectedIndex());
    if( this.jCBHierarchy.getSelectedIndex() == JFISHFP.HFP )
    {
      if ( (this.jbNbGroups.isSelected() == true) && (this.jIFNbGroups.getValue() < 1))
        throw new Exception(LocaleFIS.GetString("NbGroups") + " " + LocaleFIS.GetString("MustBe") + " >= 1");
      if ( (this.jbToleranceThreshold.isSelected() == true) && (this.jDFToleranceThres.getValue() > 1))
        throw new Exception(LocaleFIS.GetString("ToleranceThreshold") + " " + LocaleFIS.GetString("MustBe") + " <= 1");
      this.hfp.SetToleranceThreshold(this.jbNbGroups.isSelected() == true ? this.jIFNbGroups.getValue() : this.jDFToleranceThres.getValue());
    }
    this.hfp.SetConjunction(this.jCBConjunction.getSelectedIndex());
    this.hfp.Save(this.DataFile.ActiveFileName(), this.HFPFile);
    this.hfp.Delete();

    // suppression du fichier HFP temporaire
    File f = new File("temp.hfp");
    f.delete();

    this.dispose();
    }
  catch( Throwable except ) { MessageFIS.Error( null, except ); }
  }
//------------------------------------------------------------------------------
  void jButtonSaveAs_actionPerformed(ActionEvent e)
  {
  try
    {
    JFileChooser file_chooser = new JFileChooser(MainFIS.config.DataFilePath())
      {
      protected JDialog createDialog(Component parent) throws HeadlessException
        {
        JDialog dialog = super.createDialog(parent);
        dialog.setResizable(false);
        return dialog;
        }
      };
    file_chooser.setAcceptAllFileFilterUsed(false);
    // ajout du filtre *.hfp
    JFileFilter filter = new JFileFilter(new String("hfp"), LocaleFIS.GetString("HFPFile"));
    file_chooser.addChoosableFileFilter(filter);
    File df = new File(this.DataFile.FileName());
    file_chooser.setSelectedFile(new File(MainFIS.config.DataFilePath() + System.getProperty("file.separator") + df.getName().toLowerCase() + "." + LocaleFIS.GetString("hfp")));
    if( file_chooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION )
      {
      String file_name = file_chooser.getSelectedFile().getAbsolutePath();
      // ajoute l'extension si necessaire
      if( file_name.lastIndexOf('.') == -1 )
       file_name += '.' + LocaleFIS.GetString("hfp");

      // teste si le fichier existe
      File f = new File( file_name );
      if( f.exists() )
        {
        String message = LocaleFIS.GetString("HFPFile") + " : " + file_name + " " + LocaleFIS.GetString("AlreadyExist") + "\n" + LocaleFIS.GetString("DoYouWantToReplaceIt");
        if( MessageFIS.Confirm(this, message) == JOptionPane.NO_OPTION )
          return;
        }
      this.HFPFile = file_name;

      // sauvegarde du fichier hfp
     this.hfp.SetParameters(this.jCBDistance.getSelectedIndex()); 
     this.hfp.SetHierarchy(this.jCBHierarchy.getSelectedIndex());
      if( this.jCBHierarchy.getSelectedIndex() == JFISHFP.HFP )
      {
        if ( (this.jbNbGroups.isSelected() == true) && (this.jIFNbGroups.getValue() < 1))
          throw new Exception(LocaleFIS.GetString("NbGroups") + " " + LocaleFIS.GetString("MustBe") + " >= 1");
        if ( (this.jbToleranceThreshold.isSelected() == true) && (this.jDFToleranceThres.getValue() > 1))
          throw new Exception(LocaleFIS.GetString("ToleranceThreshold") + " " + LocaleFIS.GetString("MustBe") + " <= 1");
        this.hfp.SetToleranceThreshold(this.jbNbGroups.isSelected() == true ? this.jIFNbGroups.getValue() : this.jDFToleranceThres.getValue());
      }
      this.hfp.SetConjunction(this.jCBConjunction.getSelectedIndex());
      this.hfp.Save(this.DataFile.ActiveFileName(), this.HFPFile);
      this.hfp.Delete();

      // suppression du fichier HFP temporaire
      File temp = new File("temp.hfp");
      temp.delete();

      this.dispose();
      }
    }
  catch( Throwable except ) { MessageFIS.Error( null, except ); }
  }
//------------------------------------------------------------------------------
  void this_windowClosing(WindowEvent e)
  {
  // suppression du fichier HFP temporaire
  File f = new File("temp.hfp");
  f.delete();
  try { this.hfp.Delete(); }
  catch( Throwable except ) { MessageFIS.Error( null, except ); }
  }
//------------------------------------------------------------------------------
  String GetHFPFile() { return this.HFPFile; }
}





//------------------------------------------------------------------------------
class JFISHFP
{
private long ptr;
static int HFP = 0;
static int KMEANS = 1;
static int REGULAR = 2;

static int MINIMUM = 1;
//------------------------------------------------------------------------------
public JFISHFP(String data_file, String hfp_file) throws Throwable { this.ptr = jnifis.HFPOpen(data_file, hfp_file); }
//------------------------------------------------------------------------------
protected JFISHFP( long ptr ) { this.ptr = ptr; }
//------------------------------------------------------------------------------
static String [] HierarchyType() throws Throwable { return jnifis.HFPHierarchyType(); }
//------------------------------------------------------------------------------
static JFISHFP HFPConfig(String data_file, int column_not_process, String hfp_file) throws Throwable { return new JFISHFP(jnifis.HFPConfig(data_file, column_not_process, hfp_file)); }
//------------------------------------------------------------------------------
int GetHierarchy() throws Throwable
  {
  String hierarchy = jnifis.GetHFPHierarchy(this.ptr);
  String [] type_hierarchy = JFISHFP.HierarchyType();
  for( int i=0 ; i<type_hierarchy.length ; i++ )
    if( hierarchy.equals(type_hierarchy[i]) == true )
      return i;
  throw new Exception(LocaleFIS.GetString("UnknownHierarchyType") + ": " + hierarchy);
  }
//------------------------------------------------------------------------------
void SetHierarchy( int index ) throws Throwable { jnifis.SetHFPHierarchy(this.ptr, JFISHFP.HierarchyType()[index]); }
//------------------------------------------------------------------------------
int GetConjunction() throws Throwable
  {
  String conjunction = jnifis.GetHFPConjunction(this.ptr);
  String [] type_conjunction = JFIS.TypeConjunction();
  for( int i=0 ; i<type_conjunction.length ; i++ )
    if( conjunction.equals(type_conjunction[i]) == true )
      return i;
  throw new Exception(LocaleFIS.GetString("UnknownConjunctionType") + ": " + conjunction);
  }
//------------------------------------------------------------------------------
void SetConjunction( int index ) throws Throwable { jnifis.SetHFPConjunction(this.ptr, JFIS.TypeConjunction()[index]); }
//------------------------------------------------------------------------------
double GetToleranceThreshold() throws Throwable { return jnifis.GetHFPToleranceThreshold(this.ptr); }
//------------------------------------------------------------------------------
void SetToleranceThreshold( double tolerance ) throws Throwable { jnifis.SetHFPToleranceThreshold(this.ptr, tolerance); }
//------------------------------------------------------------------------------
static String [] DistanceType() throws Throwable { return jnifis.HFPDistanceType(); }
//------------------------------------------------------------------------------
Object [] GetParameters() throws Throwable { return jnifis.GetHFPParameters(this.ptr); }
//------------------------------------------------------------------------------
void SetParameters(int distance) throws Throwable
  {
  jnifis.SetHFPParameters(this.ptr, DistanceType()[distance]);
  }
//------------------------------------------------------------------------------
int GetNbInput() throws Throwable { return jnifis.GetHFPNbInput(this.ptr); }
//------------------------------------------------------------------------------
int GetNbOutput() throws Throwable { return jnifis.GetHFPNbOutput(this.ptr); }
//------------------------------------------------------------------------------
JInputHFP GetInput( int input_number ) throws Throwable { return new JInputHFP(jnifis.GetHFPInput(this.ptr, input_number)); }
//------------------------------------------------------------------------------
JOutput GetOutput( int output_number ) throws Throwable { return new JOutput(jnifis.GetHFPOutput(this.ptr, output_number)); }
//------------------------------------------------------------------------------
void Delete() throws Throwable { jnifis.HFPDelete(this.ptr); }
//------------------------------------------------------------------------------
void Save( String data_file, String hfp_file ) throws Throwable { jnifis.HFPSave(this.ptr, data_file, hfp_file); }
//------------------------------------------------------------------------------
static void Vertex( String data_file, String hfp_file, String vertex_file ) throws Throwable { jnifis.HFPVertex(data_file, hfp_file, vertex_file); }
//------------------------------------------------------------------------------
static void Select(String data_file, String hfp_file, boolean ruleInductionMethodFPA, double min_cumul_weight, int cardinality_strategy, double min_matching_degree, int min_cardinality, String result_file_name, double loss_coverage, int initial_number_of_mf, int max_number_of_iter, String vertex_file, int output_number, String validation_file) throws Throwable
  { jnifis.HFPSelect(data_file, hfp_file, ruleInductionMethodFPA, min_cumul_weight, cardinality_strategy, min_matching_degree, min_cardinality, result_file_name, loss_coverage, initial_number_of_mf, max_number_of_iter, vertex_file, output_number, validation_file); }
//------------------------------------------------------------------------------
static JFIS GenerateFIS(String data_file, String hfp_file, boolean ruleInductionMethodFPA, double min_cumul_weight, int cardinality_strategy, double min_matching_degree, int min_cardinality, int output_number, String vertex_file) throws Throwable
  { return new JFIS(jnifis.HFPFIS(data_file, hfp_file, ruleInductionMethodFPA, min_cumul_weight, cardinality_strategy, min_matching_degree, min_cardinality, output_number, vertex_file)); }
//------------------------------------------------------------------------------
void ReplaceOutput( int output_number, JOutput new_output ) throws Throwable { jnifis.ReplaceHFPOutput(this.ptr, output_number, new_output.Ptr()); }
//------------------------------------------------------------------------------
static void SetHFPFIS(String hfp_in, String cfg, String hfp_out) throws Throwable { jnifis.SetHFPFIS(hfp_in, cfg, hfp_out); }
//------------------------------------------------------------------------------
static JFIS WangMendel(String fis, String data_file) throws Throwable
  {
  File temp = File.createTempFile("tempwm", ".fis", new File(MainFIS.config.FISFilePath()));
  JFIS new_fis = new JFIS(jnifis.WangMendel(fis, data_file, temp.getAbsolutePath()));
  temp.delete();
  return new_fis;
  }
//------------------------------------------------------------------------------
static JFIS HFPSR( String data_file, String inputs_mfs, int input_hierarchy, double input_tol, int output_mfs, int output_hierarchy, String output_defuz, String output_disj, double output_tol, String conjunction, boolean classif ) throws Throwable { return new JFIS(jnifis.HFPSR(data_file, inputs_mfs, input_hierarchy, input_tol, output_mfs, output_hierarchy, output_defuz, output_disj, output_tol, conjunction, classif));}
}


//------------------------------------------------------------------------------
class JInputHFP extends JInput
{
//------------------------------------------------------------------------------
JInputHFP( long ptr ) { super(ptr); }
//------------------------------------------------------------------------------
int GetNumberOfMF() throws Throwable { return jnifis.GetHFPInputNumberOfMF(this.ptr); }
//------------------------------------------------------------------------------
void SetNumberOfMF( int number ) throws Throwable { jnifis.SetHFPInputNumberOfMF(this.ptr, number); }
}
