//***********************************************************************
//
//                              JHFPFISFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import javax.swing.*;
import javax.swing.border.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

//------------------------------------------------------------------------------
public class JHFPFISFrame extends JFISDialog
{
  private JExtendedDataFile DataFile;
  private String HFPFile;
  private String VertexFile;
  private JFISFrame Parent;
  private JLabel jLabelHFPFile = new JLabel();
  private JTextField jTFHFPFile = new JTextField();
  private JButton jButtonSelectHFPFile = new JButton("...");
  private JLabel jLabelVertexFile = new JLabel();
  private JTextField jTFVertexFile = new JTextField();
  private JButton jButtonSelectVertexFile = new JButton("...");
  private JLabel jLabelMinCumulatedWeight = new JLabel();
  private DoubleField jDFMinCumulatedWeight = new DoubleField();
  private JLabel jLabelCardinalityStrategy = new JLabel();
  private JComboBox jCBCardinalityStrategy = new JComboBox();
  private JLabel jLabelMinMatchingDegree = new JLabel();
  private DoubleField jDFMinMatchingDegree = new DoubleField();
  private JLabel jLabelMinCardinality = new JLabel();
  private IntegerField jIFMinCardinality = new IntegerField();
  private JLabel jLabelOutputNumber = new JLabel();
  private IntegerField jIFOutputNumber = new IntegerField();
  private JLabel jLabelFISResultFile = new JLabel();
  private JTextField jTFFISResultFile = new JTextField();
  private JPanel jPanelValidation = new JPanel(new GridBagLayout());
  private JButton jButtonApply = new JButton();
  private JButton jButtonCancel = new JButton();
  private JPanel jPanelNumberOfMF = null;
  private Vector jVectNumberOfMF = new Vector();
  private ButtonGroup buttonGroupRuleInductionMethod = null;  
  private JRadioButton jRadioButtonFPA = null;  
  private JRadioButton jRadioButtonWangMendel = null;  
  //------------------------------------------------------------------------------
  public JHFPFISFrame( JFISFrame parent, JExtendedDataFile data_file, String hfp_file, String vertex_file )
  {
    this.Parent = parent;
    this.DataFile = data_file;
    this.HFPFile = hfp_file;
    this.VertexFile = vertex_file;
    try { jbInit(); }
    catch(Throwable t) { MessageFIS.Error(null, t); }
  }
//------------------------------------------------------------------------------
  private void jbInit() throws Throwable
  {
    this.getContentPane().setLayout(new GridBagLayout());
    
    // nom du fichier hfp
    this.jTFHFPFile.setEditable(false);
    this.getContentPane().add(jLabelHFPFile,   new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    if( this.HFPFile != null )
      {
      this.jTFHFPFile.setText(this.HFPFile);
      this.OpenHFPFile();
      }
    this.getContentPane().add(jTFHFPFile,  new GridBagConstraints(1, 0, 3, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 0, 0, 5), 40, 0));
    this.getContentPane().add(jButtonSelectHFPFile,       new GridBagConstraints(4, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, -10));

    // nom du fichier vertex
    this.jTFVertexFile.setEditable(false);
    this.getContentPane().add(jLabelVertexFile,   new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    if( this.VertexFile != null )
      this.jTFVertexFile.setText(this.VertexFile);
    this.getContentPane().add(jTFVertexFile,  new GridBagConstraints(1, 1, 3, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 0, 0, 5), 40, 0));
    this.getContentPane().add(jButtonSelectVertexFile,    new GridBagConstraints(4, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, -10));

    // panel methode d'induction des regles
    JPanel jPanelRuleInductionMethod = new JPanel();
    jPanelRuleInductionMethod.setLayout(new GridBagLayout());
    jPanelRuleInductionMethod.setBorder(javax.swing.BorderFactory.createTitledBorder(null, LocaleFIS.GetString("RuleInductionMethod"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
    this.getButtonGroupRuleInductionMethod().add(this.getJRadioButtonFPA());
    this.getButtonGroupRuleInductionMethod().add(this.getJRadioButtonWangMendel());
    jPanelRuleInductionMethod.add(jRadioButtonFPA,   new GridBagConstraints(0, 1, 2, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
    jPanelRuleInductionMethod.add(jRadioButtonWangMendel,   new GridBagConstraints(0, 2, 2, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
    jPanelRuleInductionMethod.add(jLabelCardinalityStrategy,   new GridBagConstraints(0, 4, 2, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jCBCardinalityStrategy.addItem(LocaleFIS.GetString("min"));
    this.jCBCardinalityStrategy.addItem(LocaleFIS.GetString("decrease"));
    this.jCBCardinalityStrategy.setSelectedIndex(1);
    jPanelRuleInductionMethod.add(jCBCardinalityStrategy,  new GridBagConstraints(2, 4, 2, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, 0));

    jPanelRuleInductionMethod.add(jLabelMinMatchingDegree,   new GridBagConstraints(0, 5, 2, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jDFMinMatchingDegree.setValue(0.3);
    jPanelRuleInductionMethod.add(jDFMinMatchingDegree,  new GridBagConstraints(2, 5, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));

    jPanelRuleInductionMethod.add(jLabelMinCardinality,   new GridBagConstraints(0, 6, 2, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jIFMinCardinality.setValue(3);
    jPanelRuleInductionMethod.add(jIFMinCardinality,  new GridBagConstraints(2, 6, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 5, 5), 40, 0));
    this.getContentPane().add(jPanelRuleInductionMethod,   new GridBagConstraints(0, 3, 5, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 0, 5), 0, 0));

    // parametres
    this.getContentPane().add(jLabelMinCumulatedWeight,   new GridBagConstraints(0, 4, 2, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jDFMinCumulatedWeight.setValue(0.0);
    this.getContentPane().add(jDFMinCumulatedWeight,  new GridBagConstraints(2, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));

    this.jIFOutputNumber.setValue(0);

    // panel validation
    jPanelValidation.add(jButtonApply,  new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    jPanelValidation.add(jButtonCancel,  new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    this.getContentPane().add(jPanelValidation,         new GridBagConstraints(0, 12, 5, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));

    // evenements
    jButtonSelectHFPFile.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonSelectHFPFile_actionPerformed(e); } });
    jButtonSelectVertexFile.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonSelectVertexFile_actionPerformed(e); } });
    jButtonApply.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonApply_actionPerformed(e); } });
    jButtonCancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonCancel_actionPerformed(e); } });

    // affichage
    this.Translate();
    this.pack();
    this.setModal(true);
    this.setLocation(JFISChildFrame.ChildPosition(this.Parent, this.getSize()));
    this.setVisible(true);
  }
//------------------------------------------------------------------------------
  private void OpenHFPFile() throws Throwable
  {
  if( this.HFPFile.equals("") == true )
    return;
  if( this.jPanelNumberOfMF != null )
    this.getContentPane().remove(this.jPanelNumberOfMF);
  this.jPanelNumberOfMF = new JPanel(new GridBagLayout());
  final JPanel inner_panel = new JPanel(new GridBagLayout());
  JScrollPane jsp = new JScrollPane(inner_panel) { public Dimension getMinimumSize(){ return new Dimension(0, Math.min(200, inner_panel.getMinimumSize().height)); } };
  this.jPanelNumberOfMF.add(jsp, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
          ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
  this.jPanelNumberOfMF.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)), LocaleFIS.GetString("NumberOfMF")));
  this.getContentPane().add(this.jPanelNumberOfMF,   new GridBagConstraints(0, 2, 5, 1, 1.0, 1.0
          ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 0, 5), 0, 0));
  this.jVectNumberOfMF = new Vector();
  JFISHFP hfp = new JFISHFP(this.DataFile.ActiveFileName(), this.HFPFile);
  for( int i=0 ; i<hfp.GetNbInput() ; i++ )
    {
    inner_panel.add(new JLabel(hfp.GetInput(i).GetName()),   new GridBagConstraints(i%2==0?0:2, i/2, 1, 1, 0.0, 0.0
          ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(i/2==0 ? 5 : 0, 5, 5, 5), 0, 0));
    IntegerField ifield = new IntegerField();
    this.jVectNumberOfMF.add(ifield);
    ifield.setValue(hfp.GetInput(i).GetNumberOfMF());
    inner_panel.add(ifield,   new GridBagConstraints(i%2==0?1:3, i/2, 1, 1, 0.0, 0.0
          ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(i/2==0 ? 5 : 0, 5, 5, 5), 40, 0));
    }
  }
//------------------------------------------------------------------------------
  private void Translate()
  {
    this.setTitle(LocaleFIS.GetString("GenerateFIS"));
    this.jLabelHFPFile.setText(LocaleFIS.GetString("HFPFile") + ":");
    this.jLabelVertexFile.setText(LocaleFIS.GetString("VerticesFile") + ":");
    this.jLabelMinCumulatedWeight.setText(LocaleFIS.GetString("MinimumCumulatedWeight") + ":");
    this.jLabelCardinalityStrategy.setText(LocaleFIS.GetString("CardinalityStrategy") + ":");
    this.jLabelMinMatchingDegree.setText(LocaleFIS.GetString("MinimumMatchingDegree") + ":");
    this.jLabelMinCardinality.setText(LocaleFIS.GetString("MinimumCardinality") + ":");
    this.jLabelOutputNumber.setText(LocaleFIS.GetString("OutputNumber") + ":");
    this.jLabelFISResultFile.setText(LocaleFIS.GetString("FISResultFile") + ":");
    jButtonApply.setText(LocaleFIS.GetString("Apply"));
    jButtonCancel.setText(LocaleFIS.GetString("Cancel"));
  }
//------------------------------------------------------------------------------
  void jButtonSelectHFPFile_actionPerformed(ActionEvent e)
  {
  JOpenFileChooser file_chooser = new JOpenFileChooser(MainFIS.config.FISFilePath());
  file_chooser.setSelectedFile(new File(this.jTFHFPFile.getText()));
  file_chooser.setAcceptAllFileFilterUsed(true);
  // ajout du filtre *.hfp
  JFileFilter filter = new JFileFilter(new String("hfp"), LocaleFIS.GetString("HFPFile"));
  file_chooser.addChoosableFileFilter(filter);
  if( file_chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION )
    {
    this.jTFHFPFile.setText(file_chooser.getSelectedFile().getAbsolutePath());
    this.HFPFile = file_chooser.getSelectedFile().getAbsolutePath();
    try { this.OpenHFPFile(); }
    catch(Throwable t) { MessageFIS.Error(null, t); }
    this.pack();
    }
  }
//------------------------------------------------------------------------------
  void jButtonSelectVertexFile_actionPerformed(ActionEvent e)
  {
  JOpenFileChooser file_chooser = new JOpenFileChooser(MainFIS.config.FISFilePath());
  file_chooser.setSelectedFile(new File(this.jTFVertexFile.getText()));
  file_chooser.setAcceptAllFileFilterUsed(true);
  // ajout du filtre *.vertices
  JFileFilter filter = new JFileFilter(new String("vertices"), LocaleFIS.GetString("VerticesFile"));
  file_chooser.addChoosableFileFilter(filter);
  if( file_chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION )
    {
    this.jTFVertexFile.setText(file_chooser.getSelectedFile().getAbsolutePath());
    this.VertexFile = file_chooser.getSelectedFile().getAbsolutePath();
    }
  }
//------------------------------------------------------------------------------
  void jButtonCancel_actionPerformed(ActionEvent e) { this.dispose(); }
//------------------------------------------------------------------------------
  void jButtonApply_actionPerformed(ActionEvent e)
  {
  try
    {
    if( this.jTFHFPFile.getText().equals("") )
      {
      JOptionPane.showConfirmDialog(this, LocaleFIS.GetString("SelectAHFPFile"), null, JOptionPane.CLOSED_OPTION , JOptionPane.INFORMATION_MESSAGE );
      return;
      }
    if( this.jTFVertexFile.getText().equals("") )
      {
      JOptionPane.showConfirmDialog(this, LocaleFIS.GetString("SelectAVerticesFile"), null, JOptionPane.CLOSED_OPTION , JOptionPane.INFORMATION_MESSAGE );
      return;
      }

    String cfg = new String();
    for( int i=0 ; i<this.jVectNumberOfMF.size() ; i++ )
      cfg += ((IntegerField)this.jVectNumberOfMF.elementAt(i)).getValue() + " ";
    JFISHFP.SetHFPFIS(this.HFPFile, cfg, "temp.hfp");
    JFIS new_fis = JFISHFP.GenerateFIS(this.DataFile.ActiveFileName(), "temp.hfp", this.jRadioButtonFPA.isSelected(), this.jDFMinCumulatedWeight.getValue(), this.jCBCardinalityStrategy.getSelectedIndex(), this.jDFMinMatchingDegree.getValue(), this.jIFMinCardinality.getValue(), this.jIFOutputNumber.getValue(), this.jTFVertexFile.getText());
    JFISFrame jff = new JFISFrame(new_fis, this.DataFile, this.Parent);
    // suppression du fichier HFP temporaire
    File f = new File("temp.hfp");
    f.delete();
    this.dispose();
    }
  catch( Throwable except ) { MessageFIS.Error( null, except ); }
  }
//------------------------------------------------------------------------------
  String GetHFPFile() { return this.jTFHFPFile.getText(); }
//------------------------------------------------------------------------------
  String GetVertexFile() { return this.jTFVertexFile.getText(); }
  
  /**
   * This method initializes buttonGroupRuleInductionMethod 
   *    
   * @return javax.swing.ButtonGroup    
   */
  private ButtonGroup getButtonGroupRuleInductionMethod() {
      if (buttonGroupRuleInductionMethod == null) {
          buttonGroupRuleInductionMethod = new ButtonGroup();
      }
      return buttonGroupRuleInductionMethod;
  }
  /**
   * This method initializes jRadioButtonFPA    
   *    
   * @return javax.swing.JRadioButton   
   */
  private JRadioButton getJRadioButtonFPA() {
      if (jRadioButtonFPA == null) {
          jRadioButtonFPA = new JRadioButton();
          jRadioButtonFPA.setSelected(true);
          jRadioButtonFPA.setText("FPA");
          jRadioButtonFPA.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(java.awt.event.ActionEvent e) {
                  jLabelCardinalityStrategy.setEnabled(true);
                  jCBCardinalityStrategy.setEnabled(true);
                  jLabelMinCardinality.setEnabled(true);
                  jIFMinCardinality.setEnabled(true);
                  jLabelMinMatchingDegree.setEnabled(true);
                  jDFMinMatchingDegree.setEnabled(true);
              }
          });
      }
      return jRadioButtonFPA;
  }
  /**
   * This method initializes jRadioButtonWangMendel 
   *    
   * @return javax.swing.JRadioButton   
   */
  private JRadioButton getJRadioButtonWangMendel() {
      if (jRadioButtonWangMendel == null) {
          jRadioButtonWangMendel = new JRadioButton();
          jRadioButtonWangMendel.setText("Wang & Mendel");
          jRadioButtonWangMendel.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(java.awt.event.ActionEvent e) {
                  jLabelCardinalityStrategy.setEnabled(false);
                  jCBCardinalityStrategy.setEnabled(false);
                  jLabelMinCardinality.setEnabled(false);
                  jIFMinCardinality.setEnabled(false);
                  jLabelMinMatchingDegree.setEnabled(false);
                  jDFMinMatchingDegree.setEnabled(false);
              }
          });
      }
      return jRadioButtonWangMendel;
  }
  
}
