//***********************************************************************
//
//                              JHFPSRFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import javax.swing.*;
import javax.swing.border.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.JRadioButton;
import javax.swing.JCheckBox;

//------------------------------------------------------------------------------
public class JHFPSRFrame extends JFISDialog
{
  private static int DEFAULT_NUMBER_OF_MF = 3;
	
  private JExtendedDataFile DataFile;
  private File HFPFile;
  private JFISFrame Parent;
  private JLabel jLabelVertexFile = new JLabel();
  private JPanel jPanelValidation = new JPanel(new GridBagLayout());
  private JButton jButtonApply = new JButton();
  private JButton jButtonCancel = new JButton();
  private JPanel jPanelInputs = new JPanel(new GridBagLayout());
  private JPanel jPanelNumberOfMF = null;
  private Vector jVectNumberOfMF = new Vector();
  private JPanel jPanelOutput = new JPanel(new GridBagLayout());
  private JLabel jLOutputMFs = new JLabel();
  private IntegerField jIFOutputMFs = new IntegerField();
  private JLabel jLabelOHierarchy = new JLabel();
  private JLabel jLabelIHierarchy = new JLabel();
  private DefaultComboBoxModel jCBOHierarchyModel = new DefaultComboBoxModel()
    {
    // c'est cette methode qui assure la traduction des types de hierarchie
    public Object getElementAt(int index)
      {
      try { return LocaleFIS.GetString(JFISHFP.HierarchyType()[index]); }
      catch( Throwable t ) { return null; }
      }
    };
  private DefaultComboBoxModel jCBIHierarchyModel = new DefaultComboBoxModel()
    {
    // c'est cette methode qui assure la traduction des types de hierarchie
    public Object getElementAt(int index)
      {
      try { return LocaleFIS.GetString(JFISHFP.HierarchyType()[index]); }
      catch( Throwable t ) { return null; }
      }
    };
  private JComboBox jCBOHierarchy = new JComboBox(this.jCBOHierarchyModel);
  private JComboBox jCBIHierarchy = new JComboBox(this.jCBIHierarchyModel);
  private JPanel jPanelOInitialPartition = new JPanel(new GridBagLayout());
  private ButtonGroup jbgOInitialPartition = new ButtonGroup();
  private JRadioButton jbONbGroups;
  private JRadioButton jbOToleranceThreshold;
  private IntegerField jIFONbGroups = new IntegerField();
  private DoubleField jDFOToleranceThres = new DoubleField();
  private JPanel jPanelIInitialPartition = new JPanel(new GridBagLayout());
  private ButtonGroup jbgIInitialPartition = new ButtonGroup();
  private JRadioButton jbINbGroups;
  private JRadioButton jbIToleranceThreshold;
  private IntegerField jIFINbGroups = new IntegerField();
  private DoubleField jDFIToleranceThres = new DoubleField();
  private JLabel jLDeffuzzication = new JLabel();
  private DefaultComboBoxModel jCBFuzzyDefuzzificationModel = new DefaultComboBoxModel()
  {
  // c'est cette methode qui assure la traduction des types de defuzzification
  public Object getElementAt(int index)
    {
      //try { return LocaleFIS.GetString(JOutput.TypeFuzzyDefuzzification()[index]); }
    try { return LocaleFIS.GetString(jnifis.TypeDefuzzificationFloue()[index]); }
    catch( Throwable t ) { return null; }
    }
  };
  private JComboBox jCBFuzzyDefuzzification = new JComboBox(this.jCBFuzzyDefuzzificationModel);  
  private DefaultComboBoxModel jCBCrispDefuzzificationModel = new DefaultComboBoxModel()
  {
  // c'est cette methode qui assure la traduction des types de defuzzification
  public Object getElementAt(int index)
    {
    try { return LocaleFIS.GetString(jnifis.TypeDefuzzificationNette()[index]); }
    catch( Throwable t ) { return null; }
    }
  };
  private JComboBox jCBCrispDefuzzification = new JComboBox(this.jCBCrispDefuzzificationModel);
  private JLabel jLDisjunction = new JLabel();
  private DefaultComboBoxModel jCBDisjunctionModel = new DefaultComboBoxModel()
    {
    // c'est cette methode qui assure la traduction des types de defuzzification
    public Object getElementAt(int index)
      {
      try { return LocaleFIS.GetString(JOutput.TypeFuzzyDisjunction()[index]); }
      catch( Throwable t ) { return null; }
      }
    };
  private JComboBox jCBDisjunction = new JComboBox(this.jCBDisjunctionModel);
  private DefaultComboBoxModel jCBConjunctionModel = new DefaultComboBoxModel()
    {
    // c'est cette methode qui assure la traduction des types de conjonction
    public Object getElementAt(int index)
      {
      try { return LocaleFIS.GetString(JFIS.TypeConjunction()[index]); }
      catch( Throwable t ) { return null; }
      }
    };
  private JComboBox jCBConjunction = new JComboBox(this.jCBConjunctionModel);
  private boolean openFISInJFISFrame;
  public JFIS newFIS = null;
private JRadioButton jRadioButtonFuzzyOutput = null;  //  @jve:decl-index=0:visual-constraint="469,189"
private JRadioButton jRadioButtonCrispOutput = null;  //  @jve:decl-index=0:visual-constraint="277,207"
private JCheckBox jCheckBoxClassif = null;  //  @jve:decl-index=0:visual-constraint="477,199"
//------------------------------------------------------------------------------
  public JHFPSRFrame( JFISFrame parent, JExtendedDataFile data_file, boolean openFISInJFISFrame )
  {
    this.Parent = parent;
    this.DataFile = data_file;
    this.openFISInJFISFrame = openFISInJFISFrame;
    try { jbInit(); }
    catch(Throwable t) { MessageFIS.Error(null, t); }
  }
//------------------------------------------------------------------------------
  private void jbInit() throws Throwable
  {
    this.getContentPane().setLayout(new GridBagLayout());

    // fichier HFP
    this.HFPFile = File.createTempFile("temp", ".hfp", new File(MainFIS.config.DataFilePath()));
    JFISHFP.HFPConfig(this.DataFile.ActiveFileName(),1,this.HFPFile.getAbsolutePath());
    // following call will display input panel
    //this.OpenHFPFile();

    // parametres entrees
    this.jPanelInputs.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)), LocaleFIS.GetString("Inputs")));
    this.getContentPane().add(this.jPanelInputs,   new GridBagConstraints(0, 0, 1, 2, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 0, 5), 0, 0));
    // nombre de SEFS
    this.jPanelNumberOfMF = new JPanel(new GridBagLayout());
    JPanel outer_panel = new JPanel(new GridBagLayout());
    outer_panel.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)), LocaleFIS.GetString("NumberOfMF")));
    final IntegerField jifAllInputs = new IntegerField();
    jifAllInputs.setValue(DEFAULT_NUMBER_OF_MF);
    outer_panel.add(jifAllInputs,   new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 40, 0));
    JButton jbAllInputs = new JButton(LocaleFIS.GetString("AllInputs"));
    jbAllInputs.addActionListener(new ActionListener()
      {
      public void actionPerformed(ActionEvent e)
        {
        for( int i=0 ; i<JHFPSRFrame.this.jVectNumberOfMF.size() ; i++ )
          ((IntegerField)JHFPSRFrame.this.jVectNumberOfMF.elementAt(i)).setValue(jifAllInputs.getValue());
        }
      });
    outer_panel.add(jbAllInputs,   new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 5, 0), 0, 0));
    outer_panel.add(new JScrollPane(this.jPanelNumberOfMF),   new GridBagConstraints(0, 1, 2, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
    this.jPanelInputs.add(outer_panel,   new GridBagConstraints(0, 0, 2, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 5, 0, 5), 0, 0));
    this.jVectNumberOfMF = new Vector();
    JFISHFP hfp = new JFISHFP(this.DataFile.ActiveFileName(), this.HFPFile.getAbsolutePath());
    for( int i=0 ; i<hfp.GetNbInput() ; i++ )
      {
      this.jPanelNumberOfMF.add(new JLabel(hfp.GetInput(i).GetName()),   new GridBagConstraints(i%2==0?0:2, i/2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
      IntegerField ifield = new IntegerField();
      this.jVectNumberOfMF.add(ifield);
      ifield.setValue(DEFAULT_NUMBER_OF_MF);
      this.jPanelNumberOfMF.add(ifield,   new GridBagConstraints(i%2==0?1:3, i/2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 40, 0));
      }
    // hierarchy type
    for( int i=0 ; i<JFISHFP.HierarchyType().length ; i++ )
      this.jCBIHierarchyModel.addElement(new String());  // on met des String vide dans le modele, c'est la methode getElementAt qui fait la traduction
    this.jPanelInputs.add(this.jLabelIHierarchy,  new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
                ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
    this.jPanelInputs.add(this.jCBIHierarchy,  new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, 0));
    // partition initiale entree
    this.jPanelIInitialPartition.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)), LocaleFIS.GetString("InitialPartition")));
    this.jPanelInputs.add(this.jPanelIInitialPartition,  new GridBagConstraints(0, 2, 2, 1, 0.0, 0.0
              ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
    this.jbINbGroups = new JRadioButton(LocaleFIS.GetString("NbGroups") + ":");
    this.jbINbGroups.addItemListener(new java.awt.event.ItemListener() { public void itemStateChanged(ItemEvent e) { jIFINbGroups.setEnabled(jbINbGroups.isSelected()); } });
    this.jbIToleranceThreshold = new JRadioButton(LocaleFIS.GetString("ToleranceThreshold") + ":");
    this.jbIToleranceThreshold.addItemListener(new java.awt.event.ItemListener() { public void itemStateChanged(ItemEvent e) { jDFIToleranceThres.setEnabled(jbIToleranceThreshold.isSelected()); } });
    this.jbgIInitialPartition.add(jbINbGroups);
    this.jbgIInitialPartition.add(jbIToleranceThreshold);
    this.jbINbGroups.setSelected(true);              // force le changement de boutton
    this.jbIToleranceThreshold.setSelected(true);
    this.jbINbGroups.setSelected(true);
    this.jPanelIInitialPartition.add(this.jbINbGroups, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
          ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
    this.jPanelIInitialPartition.add(this.jbIToleranceThreshold, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
          ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jIFINbGroups.setValue(100);
    this.jDFIToleranceThres.setValue(0.01);
    this.jPanelIInitialPartition.add(this.jIFINbGroups, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
          ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 40, 0));
    this.jPanelIInitialPartition.add(this.jDFIToleranceThres, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
          ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 40, 0));

    // conjunction
    JPanel panel_conj = new JPanel(new GridBagLayout());
    for( int i=0 ; i<JFIS.TypeConjunction().length ; i++ )
      this.jCBConjunctionModel.addElement(new String());  // on met des String vide dans le modele, c'est la methode getElementAt qui fait la traduction
    this.jCBConjunction.setSelectedIndex(JFISHFP.MINIMUM);
    panel_conj.add(new JLabel(LocaleFIS.GetString("Conjunction")+":"),   new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 0, 5), 0, 0));
    panel_conj.add(this.jCBConjunction,   new GridBagConstraints(1, 0, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
    this.getContentPane().add(panel_conj,   new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));

    // parametres sortie
    this.jPanelOutput.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)), LocaleFIS.GetString("Output")));
    this.getContentPane().add(this.jPanelOutput,   new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 0, 5), 0, 0));
    // type de sortie
    ButtonGroup buttonGroupOutput = new ButtonGroup();
    buttonGroupOutput.add(this.getJRadioButtonFuzzyOutput());
    buttonGroupOutput.add(this.getJRadioButtonCrispOutput());
    this.jPanelOutput.add(this.getJRadioButtonFuzzyOutput(),  new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    this.jPanelOutput.add(this.getJRadioButtonCrispOutput(),  new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    // number of MF
    this.jIFOutputMFs.setValue(DEFAULT_NUMBER_OF_MF);
    this.jPanelOutput.add(this.jLOutputMFs,  new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
                ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
    this.jPanelOutput.add(this.jIFOutputMFs,  new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 5, 5), 40, 0));
    // hierarchy type
    for( int i=0 ; i<JFISHFP.HierarchyType().length ; i++ )
      this.jCBOHierarchyModel.addElement(new String());  // on met des String vide dans le modele, c'est la methode getElementAt qui fait la traduction
    this.jPanelOutput.add(this.jLabelOHierarchy,  new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
                ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 5, 5), 0, 0));
    this.jPanelOutput.add(this.jCBOHierarchy,  new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
    // partition initiale sortie
    this.jPanelOInitialPartition.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)), LocaleFIS.GetString("InitialPartition")));
    this.jPanelOutput.add(this.jPanelOInitialPartition,  new GridBagConstraints(0, 4, 2, 1, 0.0, 0.0
                ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 5), 0, 0));
    this.jbONbGroups = new JRadioButton(LocaleFIS.GetString("NbGroups") + ":");
    this.jbONbGroups.addItemListener(new java.awt.event.ItemListener() { public void itemStateChanged(ItemEvent e) { jIFONbGroups.setEnabled(jbONbGroups.isSelected()); } });
    this.jbOToleranceThreshold = new JRadioButton(LocaleFIS.GetString("ToleranceThreshold") + ":");
    this.jbOToleranceThreshold.addItemListener(new java.awt.event.ItemListener() { public void itemStateChanged(ItemEvent e) { jDFOToleranceThres.setEnabled(jbOToleranceThreshold.isSelected()); } });
    this.jbgOInitialPartition.add(jbONbGroups);
    this.jbgOInitialPartition.add(jbOToleranceThreshold);
    this.jbONbGroups.setSelected(true);
    this.jPanelOInitialPartition.add(this.jbONbGroups, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
    this.jPanelOInitialPartition.add(this.jbOToleranceThreshold, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jIFONbGroups.setValue(100);
    this.jDFOToleranceThres.setValue(0.01);
    this.jPanelOInitialPartition.add(this.jIFONbGroups, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 40, 0));
    this.jPanelOInitialPartition.add(this.jDFOToleranceThres, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 40, 0));
    // defuzzification type
    for( int i=0 ; i<jnifis.TypeDefuzzificationFloue().length ; i++ )
        this.jCBFuzzyDefuzzificationModel.addElement(new String());  // ajout de chaines vide, c'est la methode getElementAt du modele qui assure la traduction
    this.jCBFuzzyDefuzzification.setSelectedIndex(0);
    for( int i=0 ; i<jnifis.TypeDefuzzificationNette().length ; i++ )
        this.jCBCrispDefuzzificationModel.addElement(new String());  // ajout de chaines vide, c'est la methode getElementAt du modele qui assure la traduction
    this.jCBCrispDefuzzification.setSelectedIndex(0);
    this.jCBCrispDefuzzification.setVisible(false);
    this.jPanelOutput.add(this.jLDeffuzzication,  new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0
                ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 5, 5), 0, 0));
    this.jPanelOutput.add(this.jCBFuzzyDefuzzification,  new GridBagConstraints(1, 5, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
    this.jPanelOutput.add(this.jCBCrispDefuzzification,  new GridBagConstraints(1, 5, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
    // disjunction type
    for( int i=0 ; i<JOutput.TypeFuzzyDisjunction().length ; i++ )
      this.jCBDisjunctionModel.addElement(new String());  // ajout de chaines vide, c'est la methode getElementAt du modele qui assure la traduction
    this.jCBDisjunction.setSelectedIndex(0);
    this.jPanelOutput.add(this.jLDisjunction,  new GridBagConstraints(0, 6, 1, 1, 0.0, 0.0
                ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 5, 5), 0, 0));
    this.jPanelOutput.add(this.jCBDisjunction,  new GridBagConstraints(1, 6, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
        // flag classif
    this.jPanelOutput.add(this.getJCheckBoxClassif(),  new GridBagConstraints(0, 7, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));

    // panel validation
    jPanelValidation.add(jButtonApply,  new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    jPanelValidation.add(jButtonCancel,  new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    this.getContentPane().add(jPanelValidation,         new GridBagConstraints(0, 2, 2, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));

    // evenements
    jCBOHierarchy.addItemListener(new ItemListener() { public void itemStateChanged(ItemEvent e) { jCBOHierarchy_itemStateChanged(e); }});
    jCBIHierarchy.addItemListener(new ItemListener() { public void itemStateChanged(ItemEvent e) { jCBIHierarchy_itemStateChanged(e); }});
    jButtonApply.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonApply_actionPerformed(e); } });
    jButtonCancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonCancel_actionPerformed(e); } });

    // affichage
    this.jCBOHierarchy.setSelectedIndex(JFISHFP.REGULAR);
    this.jCBIHierarchy.setSelectedIndex(JFISHFP.KMEANS);
    this.Translate();
    this.pack();
    this.setModal(true);
    this.setLocation(JFISChildFrame.ChildPosition(this.Parent, this.getSize()));
    this.setVisible(true);
  }
//------------------------------------------------------------------------------
  private void Translate()
  {
    this.setTitle(LocaleFIS.GetString("GenerateFisWithoutRules"));
    this.jLabelVertexFile.setText(LocaleFIS.GetString("VerticesFile") + ":");
    this.jLOutputMFs.setText(LocaleFIS.GetString("NumberOfMF") + ":");
    this.jLabelIHierarchy.setText(LocaleFIS.GetString("Hierarchy") + ":");
    this.jLabelOHierarchy.setText(LocaleFIS.GetString("Hierarchy") + ":");
    this.jLDeffuzzication.setText(LocaleFIS.GetString("Defuzzification")+":");
    this.jLDisjunction.setText(LocaleFIS.GetString("Disjunction")+":");
    this.jButtonApply.setText(LocaleFIS.GetString("Apply"));
    this.jButtonCancel.setText(LocaleFIS.GetString("Cancel"));
  }
//------------------------------------------------------------------------------
  void jButtonCancel_actionPerformed(ActionEvent e) { this.dispose(); this.HFPFile.delete(); }
//------------------------------------------------------------------------------
  void jButtonApply_actionPerformed(ActionEvent e)
  {
  try
    {
    String cfg = new String();
    for( int i=0 ; i<this.jVectNumberOfMF.size() ; i++ )
      cfg += ((IntegerField)this.jVectNumberOfMF.elementAt(i)).getValue() + " ";

    if( this.jCBIHierarchy.getSelectedIndex() == JFISHFP.HFP )
    {
      if ( (this.jbINbGroups.isSelected() == true) && (this.jIFINbGroups.getValue() < 1))
        throw new Exception(LocaleFIS.GetString("NbGroups") + " " + LocaleFIS.GetString("MustBe") + " >= 1");
      if ( (this.jbIToleranceThreshold.isSelected() == true) && (this.jDFIToleranceThres.getValue() > 1))
        throw new Exception(LocaleFIS.GetString("ToleranceThreshold") + " " + LocaleFIS.GetString("MustBe") + " <= 1");
    }
    if( this.jCBOHierarchy.getSelectedIndex() == JFISHFP.HFP )
    {
      if ( (this.jbONbGroups.isSelected() == true) && (this.jIFONbGroups.getValue() < 1))
        throw new Exception(LocaleFIS.GetString("NbGroups") + " " + LocaleFIS.GetString("MustBe") + " >= 1");
      if ( (this.jbOToleranceThreshold.isSelected() == true) && (this.jDFOToleranceThres.getValue() > 1))
        throw new Exception(LocaleFIS.GetString("ToleranceThreshold") + " " + LocaleFIS.GetString("MustBe") + " <= 1");
    }

    newFIS = JFISHFP.HFPSR(this.DataFile.ActiveFileName(), cfg, this.jCBIHierarchy.getSelectedIndex()+1, this.jbINbGroups.isSelected() == true ? this.jIFINbGroups.getValue() : this.jDFIToleranceThres.getValue(), this.jRadioButtonFuzzyOutput.isSelected() ? this.jIFOutputMFs.getValue() : 0, this.jCBOHierarchy.getSelectedIndex()+1, this.jRadioButtonFuzzyOutput.isSelected() ? jnifis.TypeDefuzzificationFloue()[this.jCBFuzzyDefuzzification.getSelectedIndex()] : jnifis.TypeDefuzzificationNette()[this.jCBCrispDefuzzification.getSelectedIndex()], JOutput.TypeFuzzyDisjunction()[this.jCBDisjunction.getSelectedIndex()], this.jbONbGroups.isSelected() == true ? this.jIFONbGroups.getValue() : this.jDFOToleranceThres.getValue(), JFIS.TypeConjunction()[this.jCBConjunction.getSelectedIndex()], this.jCheckBoxClassif.isSelected() );
    newFIS.SetName(newFIS.GetName().split(JExtendedDataFile.ActiveExtension)[0]);
    if( this.openFISInJFISFrame == true ) {
        JFISFrame jff = new JFISFrame(newFIS, this.DataFile, this.Parent);
    }
    this.dispose();

    this.HFPFile.delete();
    }
  catch( Throwable except ) { MessageFIS.Error( null, except ); }
  }
//------------------------------------------------------------------------------
  void jCBOHierarchy_itemStateChanged(ItemEvent e)
  {
  if( e.getStateChange() == ItemEvent.SELECTED )
    {
    if( this.jCBOHierarchy.getSelectedIndex() != 0 )
      {
        this.jbONbGroups.setEnabled(false);
        this.jbOToleranceThreshold.setEnabled(false);
        this.jIFONbGroups.setEnabled(false);
        this.jDFOToleranceThres.setEnabled(false);
      }
    else
      {
        this.jbONbGroups.setEnabled(true);
        this.jbOToleranceThreshold.setEnabled(true);
        this.jIFONbGroups.setEnabled(this.jbONbGroups.isSelected());
        this.jDFOToleranceThres.setEnabled(this.jbOToleranceThreshold.isSelected());
      }
    }
  }
//------------------------------------------------------------------------------
  void jCBIHierarchy_itemStateChanged(ItemEvent e)
  {
  if( e.getStateChange() == ItemEvent.SELECTED )
    {
    if( this.jCBIHierarchy.getSelectedIndex() != 0 )
      {
        this.jbINbGroups.setEnabled(false);
        this.jbIToleranceThreshold.setEnabled(false);
        this.jIFINbGroups.setEnabled(false);
        this.jDFIToleranceThres.setEnabled(false);
      }
    else
      {
        this.jbINbGroups.setEnabled(true);
        this.jbIToleranceThreshold.setEnabled(true);
        this.jIFINbGroups.setEnabled(this.jbINbGroups.isSelected());
        this.jDFIToleranceThres.setEnabled(this.jbIToleranceThreshold.isSelected());
      }
    }
  }
/**
 * This method initializes jRadioButtonFuzzyOutput	
 * 	
 * @return javax.swing.JRadioButton	
 */
private JRadioButton getJRadioButtonFuzzyOutput() {
    if (jRadioButtonFuzzyOutput == null) {
        jRadioButtonFuzzyOutput = new JRadioButton();
        try {
            jRadioButtonFuzzyOutput.setText(LocaleFIS.GetString(JOutput.TypeNature()[JOutput.FUZZY]));
        } catch (Throwable e) {
            jRadioButtonFuzzyOutput.setText("Fuzzy");
        }
        jRadioButtonFuzzyOutput.setSelected(true);
        jRadioButtonFuzzyOutput.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent e) {
                setOutputFuzzy(true);
            }
        });
    }
    return jRadioButtonFuzzyOutput;
}
/**
 * This method initializes jRadioButtonCrispOutput	
 * 	
 * @return javax.swing.JRadioButton	
 */
private JRadioButton getJRadioButtonCrispOutput() {
    if (jRadioButtonCrispOutput == null) {
        jRadioButtonCrispOutput = new JRadioButton();
        try {
            jRadioButtonCrispOutput.setText(LocaleFIS.GetString(JOutput.TypeNature()[JOutput.CRISP]));
        } catch (Throwable e) {
            jRadioButtonCrispOutput.setText("Crisp");
        }
        jRadioButtonCrispOutput.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent e) {
                setOutputFuzzy(false);
            }
        });
    }
    return jRadioButtonCrispOutput;
}

private void setOutputFuzzy(boolean isFuzzy) {
    jLOutputMFs.setEnabled(isFuzzy);
    jIFOutputMFs.setEnabled(isFuzzy);
    jLabelOHierarchy.setEnabled(isFuzzy);
    jCBOHierarchy.setEnabled(isFuzzy);
    ((TitledBorder) jPanelOInitialPartition.getBorder()).setTitleColor(isFuzzy == false ? UIManager.getColor("Label.disabledForeground") : UIManager.getColor("Label.foreground"));
    repaint();
    jbONbGroups.setEnabled(isFuzzy == false ? false : this.jCBOHierarchy.getSelectedIndex() == 0);
    jbOToleranceThreshold.setEnabled(isFuzzy == false ? false : this.jCBOHierarchy.getSelectedIndex() == 0);
    jIFONbGroups.setEnabled(isFuzzy == false ? false : this.jCBOHierarchy.getSelectedIndex() == 0);
    jDFOToleranceThres.setEnabled(isFuzzy == false ? false : this.jCBOHierarchy.getSelectedIndex() == 0);
    jCBFuzzyDefuzzification.setVisible(isFuzzy);
    jCBCrispDefuzzification.setVisible(!isFuzzy);
    jCheckBoxClassif.setEnabled(!isFuzzy);
}
/**
 * This method initializes jCheckBoxClassif	
 * 	
 * @return javax.swing.JCheckBox	
 */
private JCheckBox getJCheckBoxClassif() {
    if (jCheckBoxClassif == null) {
        jCheckBoxClassif = new JCheckBox();
        jCheckBoxClassif.setText(LocaleFIS.GetString("Classif"));
        jCheckBoxClassif.setEnabled(false);
    }
    return jCheckBoxClassif;
}
}
