//***********************************************************************
//
//                              JHFPSelectFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.table.*;
import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;

//------------------------------------------------------------------------------
public class JHFPSelectFrame extends JFISDialog
{
  private JExtendedDataFile DataFile;
  private String HFPFile;
  private String VertexFile;
  private JFISFrame Parent;
  private JLabel jLabelHFPFile = new JLabel();
  private JTextField jTFHFPFile = new JTextField();
  private JButton jButtonSelectHFPFile = new JButton("...");
  private JLabel jLabelVertexFile = new JLabel();
  private JTextField jTFVertexFile = new JTextField();
  private JButton jButtonSelectVertexFile = new JButton("...");
  private JLabel jLabelMinCumulatedWeight = new JLabel();
  private DoubleField jDFMinCumulatedWeight = new DoubleField();
  private JLabel jLabelCardinalityStrategy = new JLabel();
  private JComboBox jCBCardinalityStrategy = new JComboBox();
  private JLabel jLabelMinMatchingDegree = new JLabel();
  private DoubleField jDFMinMatchingDegree = new DoubleField();
  private JLabel jLabelMinCardinality = new JLabel();
  private IntegerField jIFMinCardinality = new IntegerField();
  private JLabel jLabelCoverage = new JLabel();
  private DoubleField jDFCoverage = new DoubleField();
  private JLabel jLabelInitialNumberOfMF = new JLabel();
  private IntegerField jIFInitialNumberOfMF = new IntegerField();
  private JLabel jLabelMaximumNumberOfIterations = new JLabel();
  private IntegerField jIFMaximumNumberOfIterations = new IntegerField();
  private JLabel jLabelOutputNumber = new JLabel();
  private IntegerField jIFOutputNumber = new IntegerField();
  private JPanel jPanelValidation = new JPanel(new GridBagLayout());
  private JButton jButtonApply = new JButton();
  private JButton jButtonCancel = new JButton();
  protected JPanelValidationFile jpValidationFile;
private ButtonGroup buttonGroupRuleInductionMethod = null;  //  @jve:decl-index=0:visual-constraint="316,169"
private JRadioButton jRadioButtonFPA = null;  //  @jve:decl-index=0:visual-constraint="225,212"
private JRadioButton jRadioButtonWangMendel = null;  //  @jve:decl-index=0:visual-constraint="327,236"
//------------------------------------------------------------------------------
  public JHFPSelectFrame( JFISFrame parent, JExtendedDataFile data_file, String hfp_file, String vertex_file )
  {
    this.Parent = parent;
    this.DataFile = data_file;
    this.HFPFile = hfp_file;
    this.VertexFile = vertex_file;
    try { jbInit(); }
    catch(Throwable t) { MessageFIS.Error(null, t); }
  }
//------------------------------------------------------------------------------
  private void jbInit() throws Exception
  {
    this.getContentPane().setLayout(new GridBagLayout());

    // nom du fichier hfp
    this.jTFHFPFile.setEditable(false);
    this.getContentPane().add(jLabelHFPFile,   new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    if( this.HFPFile != null )
      this.jTFHFPFile.setText(this.HFPFile);
    this.getContentPane().add(jTFHFPFile,  new GridBagConstraints(1, 0, 3, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 0, 0, 5), 40, 0));
    this.getContentPane().add(jButtonSelectHFPFile,       new GridBagConstraints(4, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, -10));

    // nom du fichier vertex
    this.jTFVertexFile.setEditable(false);
    this.getContentPane().add(jLabelVertexFile,   new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    if( this.VertexFile != null )
      this.jTFVertexFile.setText(this.VertexFile);
    this.getContentPane().add(jTFVertexFile,  new GridBagConstraints(1, 1, 3, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 0, 0, 5), 40, 0));
    this.getContentPane().add(jButtonSelectVertexFile,    new GridBagConstraints(4, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, -10));

    // panel methode d'induction des regles
    JPanel jPanelRuleInductionMethod = new JPanel();
    jPanelRuleInductionMethod.setLayout(new GridBagLayout());
    jPanelRuleInductionMethod.setBorder(javax.swing.BorderFactory.createTitledBorder(null, LocaleFIS.GetString("RuleInductionMethod"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
    this.getButtonGroupRuleInductionMethod().add(this.getJRadioButtonFPA());
    this.getButtonGroupRuleInductionMethod().add(this.getJRadioButtonWangMendel());
    jPanelRuleInductionMethod.add(jRadioButtonFPA,   new GridBagConstraints(0, 1, 2, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
    jPanelRuleInductionMethod.add(jRadioButtonWangMendel,   new GridBagConstraints(0, 2, 2, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
    jPanelRuleInductionMethod.add(jLabelCardinalityStrategy,   new GridBagConstraints(0, 3, 2, 1, 1.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jCBCardinalityStrategy.addItem(LocaleFIS.GetString("min"));
    this.jCBCardinalityStrategy.addItem(LocaleFIS.GetString("decrease"));
    this.jCBCardinalityStrategy.setSelectedIndex(1);
    jPanelRuleInductionMethod.add(jCBCardinalityStrategy,  new GridBagConstraints(2, 3, 2, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, 0));

    jPanelRuleInductionMethod.add(jLabelMinMatchingDegree,   new GridBagConstraints(0, 4, 2, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jDFMinMatchingDegree.setValue(0.3);
    jPanelRuleInductionMethod.add(jDFMinMatchingDegree,  new GridBagConstraints(2, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));

    jPanelRuleInductionMethod.add(jLabelMinCardinality,   new GridBagConstraints(0, 5, 2, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jIFMinCardinality.setValue(3);
    jPanelRuleInductionMethod.add(jIFMinCardinality,  new GridBagConstraints(2, 5, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 5, 5), 40, 0));
    this.getContentPane().add(jPanelRuleInductionMethod,   new GridBagConstraints(0, 2, 5, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 0, 5), 0, 0));

    // parametres
    this.getContentPane().add(jLabelMinCumulatedWeight,   new GridBagConstraints(0, 3, 2, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jDFMinCumulatedWeight.setValue(0.0);
    this.getContentPane().add(jDFMinCumulatedWeight,  new GridBagConstraints(2, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));

    this.getContentPane().add(jLabelCoverage,   new GridBagConstraints(0, 6, 2, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jDFCoverage.setValue(100);
    this.getContentPane().add(jDFCoverage,  new GridBagConstraints(2, 6, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));
    this.getContentPane().add(new JLabel("%"),  new GridBagConstraints(3, 6, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, 0));

    this.getContentPane().add(jLabelInitialNumberOfMF,   new GridBagConstraints(0, 7, 2, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jIFInitialNumberOfMF.setValue(1);
    this.getContentPane().add(jIFInitialNumberOfMF,  new GridBagConstraints(2, 7, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));

    this.getContentPane().add(jLabelMaximumNumberOfIterations,   new GridBagConstraints(0, 8, 2, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jIFMaximumNumberOfIterations.setValue(10);
    this.getContentPane().add(jIFMaximumNumberOfIterations,  new GridBagConstraints(2, 8, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));

    this.jIFOutputNumber.setValue(0);

    // fichier de validation
    this.jpValidationFile = new JPanelValidationFile(this.DataFile);
    this.getContentPane().add(this.jpValidationFile,     new GridBagConstraints(0, 11, 5, 1, 1.0, 0.0
             ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));

    // panel validation
    jPanelValidation.add(jButtonApply,  new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    jPanelValidation.add(jButtonCancel,  new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    this.getContentPane().add(jPanelValidation,         new GridBagConstraints(0, 12, 5, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));

    // evenements
    jButtonSelectHFPFile.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonSelectHFPFile_actionPerformed(e); } });
    jButtonSelectVertexFile.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonSelectVertexFile_actionPerformed(e); } });
    jButtonApply.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonApply_actionPerformed(e); } });
    jButtonCancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonCancel_actionPerformed(e); } });

    // affichage
    this.Translate();
    this.pack();
    this.setModal(true);
    this.setLocation(JFISChildFrame.ChildPosition(this.Parent, this.getSize()));
    this.setVisible(true);
  }
//------------------------------------------------------------------------------
  private void Translate()
  {
    this.setTitle(LocaleFIS.GetString("SelectPartition"));
    this.jLabelHFPFile.setText(LocaleFIS.GetString("HFPFile") + ":");
    this.jLabelVertexFile.setText(LocaleFIS.GetString("VerticesFile") + ":");
    this.jLabelMinCumulatedWeight.setText(LocaleFIS.GetString("MinimumCumulatedWeight") + ":");
    this.jLabelCardinalityStrategy.setText(LocaleFIS.GetString("CardinalityStrategy") + ":");
    this.jLabelMinMatchingDegree.setText(LocaleFIS.GetString("MinimumMatchingDegree") + ":");
    this.jLabelMinCardinality.setText(LocaleFIS.GetString("MinimumCardinality") + ":");
    this.jLabelCoverage.setText(LocaleFIS.GetString("Coverage") + ":");
    this.jLabelInitialNumberOfMF.setText(LocaleFIS.GetString("InitialNumberOfMF") + ":");
    this.jLabelMaximumNumberOfIterations.setText(LocaleFIS.GetString("MaximumNumberOfIterations") + ":");
    this.jLabelOutputNumber.setText(LocaleFIS.GetString("OutputNumber") + ":");
    jButtonApply.setText(LocaleFIS.GetString("Apply"));
    jButtonCancel.setText(LocaleFIS.GetString("Cancel"));
  }
//------------------------------------------------------------------------------
  void jButtonSelectHFPFile_actionPerformed(ActionEvent e)
  {
  JOpenFileChooser file_chooser = new JOpenFileChooser(MainFIS.config.FISFilePath());
  file_chooser.setSelectedFile(new File(this.jTFHFPFile.getText()));
  file_chooser.setAcceptAllFileFilterUsed(true);
  // ajout du filtre *.hfp
  JFileFilter filter = new JFileFilter(new String("hfp"), LocaleFIS.GetString("HFPFile"));
  file_chooser.addChoosableFileFilter(filter);
  if( file_chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION )
    this.jTFHFPFile.setText(file_chooser.getSelectedFile().getAbsolutePath());
  }
//------------------------------------------------------------------------------
  void jButtonSelectVertexFile_actionPerformed(ActionEvent e)
  {
  JOpenFileChooser file_chooser = new JOpenFileChooser(MainFIS.config.FISFilePath());
  file_chooser.setSelectedFile(new File(this.jTFVertexFile.getText()));
  file_chooser.setAcceptAllFileFilterUsed(true);
  // ajout du filtre *.vertices
  JFileFilter filter = new JFileFilter(new String("vertices"), LocaleFIS.GetString("VerticesFile"));
  file_chooser.addChoosableFileFilter(filter);
  if( file_chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION )
    this.jTFVertexFile.setText(file_chooser.getSelectedFile().getAbsolutePath());
  }
//------------------------------------------------------------------------------
  void jButtonCancel_actionPerformed(ActionEvent e) { this.jTFHFPFile.setText(null); this.jTFVertexFile.setText(null); this.dispose(); }
//------------------------------------------------------------------------------
  void jButtonApply_actionPerformed(ActionEvent e)
  {
  try
    {
    if( this.jTFHFPFile.getText().equals("") )
      {
      JOptionPane.showConfirmDialog(this, LocaleFIS.GetString("SelectAHFPFile"), null, JOptionPane.CLOSED_OPTION , JOptionPane.INFORMATION_MESSAGE );
      return;
      }
    if( this.jTFVertexFile.getText().equals("") )
      {
      JOptionPane.showConfirmDialog(this, LocaleFIS.GetString("SelectAVerticesFile"), null, JOptionPane.CLOSED_OPTION , JOptionPane.INFORMATION_MESSAGE );
      return;
      }
    JFISHFP.Select(this.DataFile.ActiveFileName(), this.jTFHFPFile.getText(), this.jRadioButtonFPA.isSelected(), this.jDFMinCumulatedWeight.getValue(), this.jCBCardinalityStrategy.getSelectedIndex(), this.jDFMinMatchingDegree.getValue(), this.jIFMinCardinality.getValue(), /*this.jTFFISResultFile.getText()*/"temp.fis", this.jDFCoverage.getValue()/100.0, this.jIFInitialNumberOfMF.getValue(), this.jIFMaximumNumberOfIterations.getValue(), this.jTFVertexFile.getText(), this.jIFOutputNumber.getValue(), this.jpValidationFile.GetValidationFile());

    // suppression du fichier temp.fis
    File f = new File("temp.fis");
    f.delete();

    // deplacement des fichiers resultat vers MainFIS.config.FISFilePath() et affichage
    JMoveFile result = new JMoveFile("result");
    String new_name = MainFIS.config.FISFilePath() + System.getProperty("file.separator") + "result";
    result.MoveTo(new_name);
    JFISConsole jc = new JFISConsole(this.Parent, new_name);

    JMoveFile result_min = new JMoveFile("result.min");
    new_name = MainFIS.config.FISFilePath() + System.getProperty("file.separator") + "result.min";
    result_min.MoveTo(new_name);
    JFISConsole jcmin = new JFISConsole(this.Parent, new_name);

    this.dispose();
    }
  catch( Throwable except ) { MessageFIS.Error( null, except ); }
  }
//------------------------------------------------------------------------------
  String GetHFPFile() { return this.jTFHFPFile.getText(); }
//------------------------------------------------------------------------------
  String GetVertexFile() { return this.jTFVertexFile.getText(); }
/**
 * This method initializes buttonGroupRuleInductionMethod	
 * 	
 * @return javax.swing.ButtonGroup	
 */
private ButtonGroup getButtonGroupRuleInductionMethod() {
    if (buttonGroupRuleInductionMethod == null) {
        buttonGroupRuleInductionMethod = new ButtonGroup();
    }
    return buttonGroupRuleInductionMethod;
}
/**
 * This method initializes jRadioButtonFPA	
 * 	
 * @return javax.swing.JRadioButton	
 */
private JRadioButton getJRadioButtonFPA() {
    if (jRadioButtonFPA == null) {
        jRadioButtonFPA = new JRadioButton();
        jRadioButtonFPA.setSelected(true);
        jRadioButtonFPA.setText("FPA");
        jRadioButtonFPA.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent e) {
                jLabelCardinalityStrategy.setEnabled(true);
                jCBCardinalityStrategy.setEnabled(true);
                jLabelMinCardinality.setEnabled(true);
                jIFMinCardinality.setEnabled(true);
                jLabelMinMatchingDegree.setEnabled(true);
                jDFMinMatchingDegree.setEnabled(true);
            }
        });
    }
    return jRadioButtonFPA;
}
/**
 * This method initializes jRadioButtonWangMendel	
 * 	
 * @return javax.swing.JRadioButton	
 */
private JRadioButton getJRadioButtonWangMendel() {
    if (jRadioButtonWangMendel == null) {
        jRadioButtonWangMendel = new JRadioButton();
        jRadioButtonWangMendel.setText("Wang & Mendel");
        jRadioButtonWangMendel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent e) {
                jLabelCardinalityStrategy.setEnabled(false);
                jCBCardinalityStrategy.setEnabled(false);
                jLabelMinCardinality.setEnabled(false);
                jIFMinCardinality.setEnabled(false);
                jLabelMinMatchingDegree.setEnabled(false);
                jDFMinMatchingDegree.setEnabled(false);
            }
        });
    }
    return jRadioButtonWangMendel;
}
}

// classe File modifiee
class JMoveFile extends File
{
public JMoveFile( String pathname ) { super(pathname); }
//------------------------------------------------------------------------------
public void MoveTo( String new_pathname ) throws Exception
  {
  if( this.getAbsolutePath().equals(new_pathname) ) return;
  BufferedInputStream bis = new BufferedInputStream( new FileInputStream(this));
  BufferedOutputStream bos = new BufferedOutputStream( new FileOutputStream(new File(new_pathname)));
  int byteRead = 0;
  while((byteRead = bis.read()) != -1)
    bos.write(byteRead);
  bos.flush();
  bos.close();
  bis.close();
  this.delete();
  }
}



class JResultViewer extends JFrame
{
  private String FileName;
  private Vector ResultData = new Vector();
  private DefaultTableModel dataModel;
  private JTable table;
  //------------------------------------------------------------------------------
  public JResultViewer( String file_name ) throws Exception
  {
  super();
  this.FileName = file_name;
  this.OpenFile();
  this.dataModel = new DefaultTableModel()
    {
    public boolean isCellEditable(int row, int column) { return false; }
    public int getColumnCount() { return ((Vector)ResultData.elementAt(0)).size(); }
    public int getRowCount()
    {
    //return 11;
    int toto = JResultViewer.this.ResultData.size();
    return toto;
    }
    public Object getValueAt(int row, int col) { return ((Vector)ResultData.elementAt(row)).elementAt(col); }
    };
  this.table = new JTable(dataModel);
  this.getContentPane().add(this.table);
  this.pack();
  this.setVisible(true);
  }
  //------------------------------------------------------------------------------
  private void OpenFile() throws Exception
  {
  File f = new File(this.FileName);
  BufferedReader br = new BufferedReader( new InputStreamReader( new FileInputStream(f) ) );
  String buffer;
  while( (buffer = br.readLine()) != null ) // tant que le fichier n'est pas entierement lu
    {
    StringTokenizer st = new StringTokenizer( buffer, " " );
    Vector line = new Vector();
    while( st.countTokens() != 0 )
      {
      String token = st.nextToken();
      if( (token.equals("&") == false) && (token.charAt(0) != '(') )
        line.add(token);
      }
    this.ResultData.add(line);
    }
  }
}
