//***********************************************************************
//
//                              JInferenceFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.*;

import javax.swing.*;

import java.awt.event.*;
import java.util.*;
import javax.swing.table.*;
import javax.swing.event.*;

import java.awt.geom.*;
import java.awt.geom.Point2D.*;
import util.polygon2d.*;

import javax.swing.border.*;
import ptolemy.plot.*;
import util.tableheader.*;
import util.sortabletable.*;

import java.io.File;
import java.text.*;

/**
 * Fenetre d'inference manuelle\n
 * Realise l'inference a partir des valeurs d'entrees saisies (manuellement ou par curseur) puis affiche les resultats graphiquement.\n
 * la fenetre est composee d'un objet JSplitPane coupe horizontalement en 2 parties:
 * -# les entrees actives a gauche affichees dans un objet JTable JInferenceFrame#inputs_table construite par la fonction JInferenceFrame#InputsTable
 * -# les sorties actives a droite affichees dans un objet JTable JInferenceFrame#outputs_table construite par la fonction JInferenceFrame#OutputsTable
 *
 * ces 2 tables sont incluses dans un objet JScrollPane.
 * -# le JScrollPane de la table entree contient:
 *   - une table numero des regles construite par JInferenceFrame#RulesTable dans le JScrollPane.rowHeader
 *   - une table entete des regles construite par JInferenceFrame#RulesTableHeader dans le JScrollPane.upperLeft
 *   - une table entete des entrees construite par JInferenceFrame#InputsTableHeader dans le JScrollPane.columnHeader
 * -# le JScrollPane de la table sortie contient:
 *   - une table entete des sorties construite par JInferenceFrame#OutputsTableHeader dans le JScrollPane.columnHeader
 *
 * Toutes les tables de la fenetre partagent le meme modele de donnees triable JInferenceFrame#rules_model.
 * Ce sont les modeles de colonnes RulesColumnModel, InputsColumnModel et OutputsColumnModel qui permettent a chaque table d'afficher les donnees qui la concerne.
 */
public class JInferenceFrame extends JFISChildFrame implements SortableHeaderListener.SortableObject
{
	private JFIS fis; /**< Objet FIS */
	private int nb_inputs; /**< Nombre d'entrees actives dans le FIS */
	private int nb_outputs; /**< Nombre de sorties actives dans le FIS */
	private int nb_rules; /**< Nombre de regles actives dans le FIS */
	private double [] kw;
	private double [] sw;
	private int nbAlphaCut = 5;
	private final int RowHeight = 50; /** Hauteur des cellules de la table (pour affichage correct d'un objet FISPlot) */
	private final int FieldRowHeight = 16; /** Hauteur des objets champ (pour affichage correct d'un objet DoubleFieldInput) */
	private SortableTableModel rules_model; /**< Modele de donnees regles triable */
	private FISListener fis_listener = null; /**< Listener du FIS JInferenceFrame#fis */
	private Component split_pane = null; /**< Objet conteneur des tables entrees et sorties */
	private SortButtonRenderer table_header_renderer; /**< Renderer des headers de table triable\n les 3 tables regles, entrees et sorties utilisent le meme objet renderer */
	private Vector tables_headers; /**< Vecteur contenant les objets JTableHeader des 3 tables regles, entrees et sorties */
	private JSemaphore frame_open;   /**< Semaphore d'ouverture de la fenetre JInferenceFrame */
	private JMenuBar jMenuBarInference; /**< Barre de menu de la fenetre */
	private JMenu jMenuDisplay; /**< Menu configuration de l'affichage */
	private JMenu jMenuRange; /**< Menu de selection des domaines d'affichage */
	private JMenu jMenuEntreeFloue;
	private JMenu jMenuGabarit;
	private JMenuItem jMenuAlphaCut;
	private Vector infer_range; /**< Domaine d'inference des entrees */
	private JTable inputs_table; /**< Table des entrees */
	private JTable outputs_table; /**< Table des sorties */
	private Vector sliders_value; /**< Valeur des curseurs pour chaque entree */
	private double []outputs_infer; /**< Valeur des sorties inferees */
	private JExtendedDataFile data_file; /**< Fichier de donnees */
	private int sort_col = -1; /**< Memorisation du numero de colonne triees. -1 pour aucune colonne triee */
	private boolean is_ascent; /**< Memorisation ordre de tri */
	private JDoubleSlider []sliders; /**< Objets curseurs */
	private DefaultBoundedRangeModel scroll_model; /**< Modele des scrollbar verticales des tables entrees et sorties */
	//------------------------------------------------------------------------------
	/**
	 * Classe modele de colonne pour table regle
	 */
	class RulesColumnModel extends DefaultTableColumnModel
	{
		public void addColumn(TableColumn tc) { if( tc.getModelIndex() == 0 ) super.addColumn(tc); } /**< Surcharge de DefaultTableColumnModel.addColumn pour afficher la premiere colonne du modele (numero des regles)*/
	}
	//------------------------------------------------------------------------------
	/**
	 * Classe modele de colonne pour table entrees
	 *
	 */
	class InputsColumnModel extends DefaultTableColumnModel
	{
		/**
		 *  Surcharge de DefaultTableColumnModel.addColumn pour afficher les colonnes des entrees actives
		 * */
		public void addColumn(TableColumn tc)
		{
			int index = tc.getModelIndex();
			if( (index > 0) && (index <= JInferenceFrame.this.nb_inputs) )
			{
				boolean active = false;
				try{ active = JInferenceFrame.this.fis.GetInput(index-1).GetActive(); } catch( Throwable t ) {}
				if( active == true )
					super.addColumn(tc);
			}
		}
	}
	//------------------------------------------------------------------------------
	/**
	 * Classe modele de colonne pour table sortiees
	 */
	class OutputsColumnModel extends DefaultTableColumnModel
	{
		/**
		 *  Surcharge de DefaultTableColumnModel.addColumn pour afficher les colonnes des sorties actives
		 * */
		public void addColumn(TableColumn tc)
		{
			int index = tc.getModelIndex();
			if( index > JInferenceFrame.this.nb_inputs )
			{
				boolean active = false;
				try{ active = JInferenceFrame.this.fis.GetOutput(index-JInferenceFrame.this.nb_inputs-1).GetActive(); } catch( Throwable t ) {}
				if( active == true )
					super.addColumn(tc);
			}
		}
	}
	//------------------------------------------------------------------------------
	/**
	 * Classe table utilisee pour les tables de la fenetre
	 */
	class JInferenceTable extends JTable
	{
		public JInferenceTable(TableModel dm, TableColumnModel cm)
		{
			super(dm, cm);
			this.createDefaultColumnsFromModel(); // construit la table
			this.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
			this.setBackground(UIManager.getColor("Button.background"));
			this.setColumnSelectionAllowed(false);
			this.setCellSelectionEnabled(false);
		}
		public void changeSelection(int rowIndex, int columnIndex, boolean toggle, boolean extend) {} // table non selectionnable
		public Dimension getPreferredScrollableViewportSize() { return new Dimension(this.getPreferredSize().width, this.getPreferredSize().height); }
		public boolean isCellEditable(int row, int column) { return false; }  // cellules non editables
	}

	//------------------------------------------------------------------------------
	/**
	 * Classe table utilisee pour les tables entete de la fenetre
	 */
	class JInferenceTableHeader extends JInferenceTable
	{
		public JInferenceTableHeader(TableModel dm, TableColumnModel cm)
		{
			super( dm, cm );
			for( int i=0 ; i<this.getColumnCount() ; i++ )
				this.getColumnModel().getColumn(i).setHeaderRenderer(JInferenceFrame.this.table_header_renderer); // entete triable
			this.getTableHeader().setReorderingAllowed(false);
			this.getTableHeader().setResizingAllowed(false);
			this.getTableHeader().addMouseListener(new SortableHeaderListener(this.getTableHeader(),JInferenceFrame.this.table_header_renderer, JInferenceFrame.this)); // listener de header triable
			JInferenceFrame.this.tables_headers.add(this.getTableHeader()); // ajout du header dans le vecteurs des headers
		}
	}
	//------------------------------------------------------------------------------
	/**
	 * Construction de la Table entete de regles
	 * @return un objet JPanel contenant la table entete regle et son entete triable
	 */
	private Component RulesTableHeader() throws Throwable
	{
		// construit la table a partir du modele de donnees et du modele de colonne
		// les donnees de cette table sont independante du modele this.rules_model.
		// le modele this.rules_model est passe au constructeur uniquement pour recuperer les JTable.tableHeader
		// surcharge de la fonction getRowCount et getValueAt qui renvoit une chaine vide
		JTable table = new JInferenceTableHeader(this.rules_model, new RulesColumnModel())
		{
			public int getRowCount() { return 1; } // une seule ligne
			public Object getValueAt(int row, int column) { return new String(); }
		};
		table.setShowGrid(false);
		// construction du panel de retour avec la table et son entete
		JPanel panel = new JPanel(new BorderLayout());
		panel.add(table.getTableHeader(), BorderLayout.NORTH);
		panel.add(table, BorderLayout.SOUTH);
		return panel;
	}
	//------------------------------------------------------------------------------
	/**
	 * Construction de la table entete des entrees\n
	 * - les champs de saisie sur la premiere ligne
	 * - les curseurs sur la seconde ligne
	 *
	 * @return un objet JPanel contenant la table entete entrees et son entete triable
	 */
	private Component InputsTableHeader() throws Throwable
	{
		this.sliders = new JDoubleSlider[this.nb_inputs]; // tableau des objets curseurs
		// donnees utilisees par la table: objets de saisie DoubleFieldInput et curseur JDoubleSlider
		final Object [][]data = new Object[2][this.nb_inputs];
		for( int i=0 ; i<this.nb_inputs ; i++ )
		{
			data[0][i] = new DoubleFieldInput(this.fis.GetInput(i));  // champ de saisie
			JDoubleSlider slider = new JDoubleSlider();  // curseur
			data[1][i] = slider;
			this.sliders[i] = slider;
		}

		// construit la table a partir du modele de donnees et du modele de colonne
		// les donnees de cette table sont independante du modele this.rules_model.
		// le modele this.rules_model est passe au constructeur uniquement pour recuperer les JTable.tableHeader
		// surcharge de la fonction getRowCount et getValueAt
		final JTable table = new JInferenceTableHeader(this.rules_model, new InputsColumnModel())
		{
			public int getRowCount() { return 3; } // 3 lignes: 2 premiere lignes: objets data, 3eme ligne vide: correspond a la 3eme ligne de la table OutputsTableHeader
			// surcharge de JTable.getValueAt renvoit les objets du tableau data pour les 2 premieres lignes, un JPanel vide pour la troisieme ligne
			public Object getValueAt(int row, int column) { return row < 2 ? data[row][this.convertColumnIndexToModel(column)-1] : new JPanel(); }
			public void setValueAt(Object aValue, int row, int column) {} // ne fait rien
			public boolean isCellEditable(int row, int column) { return true; }  // cellules editables
			// surcharge de JTable.tableChanged car apres un appel a JTable.tableChanged les hauteurs de lignes sont perdues
			public void tableChanged(TableModelEvent e)
			{
				super.tableChanged(e);
				if(JInferenceFrame.this != null) {
					this.setRowHeight(0, JInferenceFrame.this.FieldRowHeight);
					this.setRowHeight(2, JInferenceFrame.this.RowHeight);
				}
			}
		};

		// configuration des listeners
		final double[] infer_values = new double[this.fis.NbInputs()]; // valeur des entrees pour l'inference
		for( int i=0 ; i<this.fis.NbInputs() ; i++ )
		{

			final int fi = i;
			final DoubleFieldInput dfi = (DoubleFieldInput)data[0][i];
			final JDoubleSlider slider = (JDoubleSlider)data[1][i];

			// listener de curseur
			slider.addChangeListener(new ChangeListener()
			{
				// changement d'etat du curseur
				public void stateChanged(ChangeEvent e)
				{
					double input_value = slider.getDoubleValue(); // lecture de la valeur du curseur
					JInferenceFrame.this.sliders_value.setElementAt(new java.lang.Double(input_value), fi);
					// mise a jour du champ de saisie
					dfi.setValue(input_value);
					((DefaultTableModel)table.getModel()).fireTableCellUpdated(0, fi+1);
					// inference
					infer_values[fi] = input_value;
					try
					{	//si on est en entrees floues ie kw et sw d'une des entrees est != 0					
						if(!JInferenceFrame.this.IsFuzzyInputModeActive()) 
							JInferenceFrame.this.fis.Infer(infer_values);							
						else //FATI
						{
							JInferenceFrame.this.fis.InferFuzzyInput(JInferenceFrame.this.kw, JInferenceFrame.this.sw, JInferenceFrame.this.nbAlphaCut, infer_values);
						}
						JInferenceFrame.this.outputs_infer = JInferenceFrame.this.fis.SortiesObtenues();
					} catch(Throwable t) {}
					// mise a jour des panels graphiques de l'entree modifiee
					for( int row=0 ; row<JInferenceFrame.this.inputs_table.getRowCount() ; row++ )
						((DefaultTableModel)JInferenceFrame.this.inputs_table.getModel()).fireTableCellUpdated(row, /*table.convertColumnIndexToModel(fi)-1*/ fi+1);
					// mise a jour des sorties
					if( JInferenceFrame.this.outputs_table != null )
						for( int i=0 ; i<JInferenceFrame.this.outputs_table.getColumnCount() ; i++ )
							for( int j=0 ; j<JInferenceFrame.this.outputs_table.getRowCount() ; j++ )
								((DefaultTableModel)JInferenceFrame.this.outputs_table.getModel()).fireTableCellUpdated(j, JInferenceFrame.this.outputs_table.convertColumnIndexToModel(i));
				}
			} );
			// listener de champ de saisie
			dfi.SetAction(new NumberFieldActionListener()
			{
				public void NumberFieldAction()
				{
					// limite la saisie au domaine d'inference
					double []range = (double [])JInferenceFrame.this.infer_range.elementAt(fi);
					if( dfi.getValue() < range[0] ) dfi.setValue(range[0]);
					if( dfi.getValue() > range[1] ) dfi.setValue(range[1]);
					// mise a jour du curseur
					slider.setDoubleValue(dfi.getValue());
					((DefaultTableModel)table.getModel()).fireTableCellUpdated(1, fi+1);
				}
			} );
			// valeur par defaut des listeners
			double []range = (double [])this.infer_range.elementAt(i); // domaine d'inference
			slider.setDoubleMinMax(range[0], range[1]);
			double value = ((java.lang.Double)this.sliders_value.elementAt(i)).doubleValue();
			slider.setDoubleValue(value);
			dfi.setValue(value);
		}

		// configuration des renderers et editeurs de cellule
		for( int i=0 ; i<table.getColumnCount() ; i++ )
		{
			// renderers
			table.getColumnModel().getColumn(i).setCellRenderer(new DefaultTableCellRenderer()
			{
				public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) { return (Component)value; }
			} );
			// editeurs
			table.getColumnModel().getColumn(i).setCellEditor(new DefaultCellEditor(new JTextField()) // JTextField virtuel (ne correspond pas au type d'editeur utilise)
			{
				public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) { return (Component)table.getValueAt(row, column); }
				// Active l'editeur de cellule au premier clic de souris\n
				// par defaut les editeurs de cellules sont actives apres un double clic
				public boolean isCellEditable(EventObject evt)
				{
					if( evt instanceof MouseEvent ) { return ((MouseEvent)evt).getClickCount() >= 1; }
					return true;
				}
			} );
		}
		// construction du panel de retour avec la table et son entete
		JPanel panel = new JPanel(new BorderLayout());
		panel.add(table.getTableHeader(), BorderLayout.NORTH);
		panel.add(table, BorderLayout.SOUTH);
		return panel;
	}
	//------------------------------------------------------------------------------
	/**
	 * Construction de la table entete des sorties\n
	 * - la valeur de la sortie sur la premiere ligne
	 * - la valeur de l'alarme sur la seconde ligne
	 * - le panel graphique agregation sur la 3eme ligne
	 *
	 * @return un objet JPanel contenant la table entete sorties et son entete triable
	 */
	private Component OutputsTableHeader() throws Throwable
	{
		// construit la table a partir du modele de donnees et du modele de colonne
		// les donnees de cette table sont independante du modele this.rules_model.
		// le modele this.rules_model est passe au constructeur uniquement pour recuperer les JTable.tableHeader
		// surcharge de la fonction getRowCount et getValueAt
		JTable table = new JInferenceTableHeader(this.rules_model, new OutputsColumnModel())
		{
			public int getRowCount() { return 3; } // 3 lignes
			public Object getValueAt(int row, int column) { return new String(); } // chaine vide
			public Class getColumnClass(int column) { return String.class; }
			// surcharge de JTable.tableChanged car apres un appel a JTable.tableChanged les hauteurs de lignes sont perdues
			public void tableChanged(TableModelEvent e)
			{
				super.tableChanged(e);
				if(JInferenceFrame.this != null) {
					this.setRowHeight(0, JInferenceFrame.this.FieldRowHeight);
					this.setRowHeight(2, JInferenceFrame.this.RowHeight);
				}
			}
		};
		((DefaultTableModel)table.getModel()).fireTableDataChanged(); // force le changement pour reaffichage de la table

		// configuration des renderers
		for( int i=0 ; i<table.getColumnCount() ; i++ )
		{
			final int output_number = table.convertColumnIndexToModel(i)-JInferenceFrame.this.nb_inputs-1;
			final JOutput output = JInferenceFrame.this.fis.GetOutput(output_number);
			table.getColumnModel().getColumn(i).setCellRenderer(new DefaultTableCellRenderer()
			{
				public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
				{
					// premiere ligne: valeur de la sortie
					if( (row == 0) && (JInferenceFrame.this.outputs_infer != null) )
					{
						double out = JInferenceFrame.this.outputs_infer[output_number];
						try
						{
							// affiche la sortie inferee dans un objet DoubleFieldInput
							DoubleFieldInput dfi = new DoubleFieldInput(output);
							dfi.setValue(out);
							return dfi;
						} catch( Throwable t ) {}
						return this;
					}
					// 2eme ligne: valeur de l'alarme
					if( row == 1 )
					{
						try
						{
							super.getTableCellRendererComponent(table, value, isSelected,hasFocus, row, column);
							switch(output.GetAlarm())
							{
							case 0: this.setText(""); break;
							case 1: this.setText("No rule"); break;
							case 2: this.setText("Ambiguity"); break;
							case 3: this.setText("No connex"); break;
							case 4: this.setText("No connex"); break;
							}
							this.setForeground(Color.red);
							this.setHorizontalAlignment(SwingConstants.CENTER);
							this.setBackground(UIManager.getColor("Button.background"));
						} catch( Throwable t ) {}
						return this;
					}
					// 3eme ligne: affichage du panel agregation
					if( row == 2 )
					{
						try
						{
							if( output.GetNature() == JOutput.FUZZY )
								return new FuzzyAgregationRenderer(output, output_number); // sortie floue
							else
								return new CrispAgregationRenderer(output, output_number); // sortie nette
						} catch( Throwable t ) {} }
					return this;
				}
			} );
		}
		// construction du panel de retour avec la table et son entete
		JPanel panel = new JPanel(new BorderLayout());
		panel.add(table.getTableHeader(), BorderLayout.NORTH);
		panel.add(table, BorderLayout.SOUTH);
		return panel;
	}
	//------------------------------------------------------------------------------
	/**
	 * Construction de la table des numeros des regles
	 * @return un objet JTable contenant les une colonne avec les numeros de regles
	 */
	private JTable RulesTable() throws Throwable
	{
		JTable table = new JInferenceTable(this.rules_model, new RulesColumnModel() );
		table.setRowHeight(this.RowHeight); // hauteur des lignes
		// configuration du renderer
		table.getColumnModel().getColumn(0).setCellRenderer(new DefaultTableCellRenderer()
		{
			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
			{
				super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
				super.setHorizontalAlignment(SwingConstants.CENTER);
				return this;
			}
		} );
		return table;
	}
	//------------------------------------------------------------------------------
	/**
	 * Construction de la table des entrees actives
	 * @return un objet JScrollPane contenant la table des entrees actives et:
	 * - la table JInferenceFrame#RulesTable en JScrollPane.rowHeader
	 * - la table JInferenceFrame#InputsTableHeader en JScrollPane.columnHeader
	 * - la table JInferenceFrame#RulesTableHeader en JScrollPane.upperLeft
	 */
	private JScrollPane InputsTable() throws Throwable
	{
		JTable table = new JInferenceTable(this.rules_model, new InputsColumnModel())
		{
			protected void configureEnclosingScrollPane() {} // supprime l'insertion du JTable.tableHeader dans le JScrollPane
		};
		this.inputs_table = table;
		// configuration des renderers
		for( int i=0 ; i<table.getColumnCount() ; i++ )
		{
			int input_number = table.convertColumnIndexToModel(i)-1;
			table.getColumnModel().getColumn(i).setCellRenderer(new InputRenderer(this.fis.GetInput(input_number), input_number));
		}
		table.setRowHeight(this.RowHeight); // hauteur des lignes
		// construction du panel de retour
		JPanel panel = new JPanel(new BorderLayout());
		panel.add(table, BorderLayout.CENTER);
		JScrollPane scroll = new JScrollPane(panel);
		scroll.setColumnHeaderView(this.InputsTableHeader());
		scroll.setRowHeaderView(this.RulesTable());
		scroll.setCorner(JScrollPane.UPPER_LEFT_CORNER, this.RulesTableHeader());
		return scroll;
	}
	//------------------------------------------------------------------------------
	/**
	 * Construction de la table des sorties actives
	 * @return un objet JScrollPane contenant la table des sorties actives et:
	 * - la table JInferenceFrame#OutputsTableHeader en JScrollPane.columnHeader
	 */
	private JScrollPane OutputsTable() throws Throwable
	{
		JTable table = new JInferenceTable(this.rules_model, new OutputsColumnModel())
		{
			protected void configureEnclosingScrollPane() {} // supprime l'insertion du JTable.tableHeader dans le JScrollPane
		};
		this.outputs_table = table;
		// configuration des renderers
		for( int i=0 ; i<table.getColumnCount() ; i++ )
		{
			int output_number = table.convertColumnIndexToModel(i)-JInferenceFrame.this.nb_inputs-1;
			JOutput output = this.fis.GetOutput(output_number);
			if( output.GetNature() == JOutput.CRISP )
				table.getColumnModel().getColumn(i).setCellRenderer(new CrispRenderer());
			else
				table.getColumnModel().getColumn(i).setCellRenderer(new FuzzyRenderer(output, output_number));
		}
		table.setRowHeight(this.RowHeight); // hauteur des lignes
		// construction du panel de retour
		JPanel panel = new JPanel(new BorderLayout());
		panel.add(table, BorderLayout.CENTER);
		JScrollPane scroll = new JScrollPane(panel);
		scroll.setColumnHeaderView(this.OutputsTableHeader());
		return scroll;
	}
	//------------------------------------------------------------------------------
	/**
	 * Reinitialistion de la fenetre\n
	 * fonction appellee par le listener de fis JInferenceFrame#fis_listener pour reafficher la fenetre apres une modification du FIS JInferenceFrame#fis
	 */
	private void ReInit()
	{
		try
		{
			this.Init();
			this.repaint();
		}
		catch( Throwable t )
		{
			if( this.split_pane != null )
			{
				this.getContentPane().remove(this.split_pane);
				this.split_pane = new JPanel();
				this.getContentPane().add(this.split_pane);
				this.repaint();
			}
			MessageFIS.Error(this, LocaleFIS.GetStringNoException(t.getClass().getName()), t.getLocalizedMessage());
		}
	}
	//------------------------------------------------------------------------------
	/**
	 * Fonction pour tester si on est dans les bonnes conditions pour avoir des entrees floues
	 * @throws Throwable 
	 */	
	public boolean isFuzzyInputAvailable() throws Throwable
	{
		//si pas d'entree
		if(this.fis.NbInputs() <= 0) return false;
		//pour une sortie
		if(this.fis.NbOutputs() > 1) return false;
		//si on est en impli = sortie
		if(this.fis.GetOutput(0).GetImplicatif() && this.fis.NbInputs() > 2)
			return false;
		
		return true;
	}
	//------------------------------------------------------------------------------
	/**
	 * Initialisation de la fenetre
	 */
	private void Init() throws Throwable
	{
		this.nb_inputs = this.fis.NbInputs();
		this.nb_outputs = this.fis.NbOutputs();
		this.nb_rules = this.fis.NbActiveRules();
		if( this.nb_rules == 0 )
			throw new Exception(LocaleFIS.GetString("NoActiveRule"));
		if( this.fis.NbActiveInputs() == 0 )
			throw new Exception(LocaleFIS.GetString("NoActiveInput"));
		if( this.fis.NbActiveOutputs() == 0 )
			throw new Exception(LocaleFIS.GetString("NoActiveOutput"));

		// init des classes
		if( this.data_file != null )
			this.fis.InitClasses(this.data_file.ActiveFileName());
		this.fis.InitPossibles();
		
		// init tableau pour gabarit
		this.MakeDoubleKWSW();
		if(!this.isFuzzyInputAvailable())
			for(int i = 0; i < this.fis.NbInputs(); i++)
				this.fis.GetInput(i).SetTemplate(0.0, 0.0); //on reinitialise les gabarits pour enlever les EF
		
		// construction des menus
		this.jMenuBarInference = new JMenuBar();
		this.setJMenuBar(this.jMenuBarInference);
		this.jMenuDisplay = new JMenu();
		this.jMenuRange = new JMenu();
		this.jMenuRange.addMenuListener(new MenuListener()
		{
			public void menuSelected(MenuEvent e)
			{
				JInferenceFrame.this.jMenuRange.removeAll();
				try
				{
					for( int i=0 ; i<JInferenceFrame.this.fis.NbInputs() ; i++ )
					{
						final int fi = i;
						final JInput input = JInferenceFrame.this.fis.GetInput(i);
						if( input.GetActive() == true )
						{
							JMenuItem item = new JMenuItem(input.GetName());
							JInferenceFrame.this.jMenuRange.add(item);
							item.addActionListener(new ActionListener()
							{
								public void actionPerformed(ActionEvent e)
								{
									JDisplayRangeFrame jdrf = new JDisplayRangeFrame(input, (double [])JInferenceFrame.this.infer_range.elementAt(fi), "InputRange");
									jdrf.setLocation(JInferenceFrame.this.ChildPosition(jdrf.getSize()));
									double []new_range = jdrf.Show();
									JInferenceFrame.this.infer_range.setElementAt(new_range, fi);
									JInferenceFrame.this.sliders[fi].setDoubleMinMax(new_range[0], new_range[1]);
									JInferenceFrame.this.inputs_table.repaint();
								}
							} );
						}
					}
				}
				catch( Throwable t ) { MessageFIS.Error(null, t); }
			}
			public void menuDeselected(MenuEvent e) {}
			public void menuCanceled(MenuEvent e) {}
		});
		this.jMenuBarInference.add(this.jMenuDisplay);
		this.jMenuDisplay.add(this.jMenuRange);
		this.jMenuEntreeFloue = new JMenu();
		this.jMenuGabarit = new JMenu();
		this.jMenuAlphaCut = new JMenuItem();
		this.jMenuGabarit.addMenuListener(new MenuListener()
		{
			public void menuSelected(MenuEvent e)
			{
				JInferenceFrame.this.jMenuGabarit.removeAll();
				try
				{
					for( int i=0 ; i<JInferenceFrame.this.fis.NbInputs() ; i++ )
					{
						final int fi = i;
						final JInput input = JInferenceFrame.this.fis.GetInput(i);			
						if( input.GetActive() == true )
						{
							JMenuItem item = new JMenuItem(input.GetName());
							JInferenceFrame.this.jMenuGabarit.add(item);
							item.addActionListener(new ActionListener()
							{
								public void actionPerformed(ActionEvent e)
								{									
									try {
										JGabaritFrame jgf = new JGabaritFrame(JInferenceFrame.this.fis, fi);
										jgf.setLocation(JInferenceFrame.this.ChildPosition(jgf.getSize()));
										jgf.Show();	
										
										// Preparation Inference
										double[] infer_values = new double[JInferenceFrame.this.nb_inputs];
										for(int i=0; i<JInferenceFrame.this.nb_inputs; i++)
											infer_values[i] = JInferenceFrame.this.sliders[i].getDoubleValue();
											
										// si on est en entree floue -> reinferer avec 
										if(JInferenceFrame.this.IsFuzzyInputModeActive())
										{
											//constitution des tableaux kw et sw
											JInferenceFrame.this.MakeDoubleKWSW();
											JInferenceFrame.this.fis.InferFuzzyInput(JInferenceFrame.this.kw, JInferenceFrame.this.sw, JInferenceFrame.this.nbAlphaCut, infer_values);
											
											if(JInferenceFrame.this.fis.GetOutput(0).GetImplicatif())
												// Nb Alpha-cut accessible
												JInferenceFrame.this.jMenuAlphaCut.setEnabled(true);
											else // Nb Alpha-cut inaccessible										
												JInferenceFrame.this.jMenuAlphaCut.setEnabled(false);
										}
										else
											JInferenceFrame.this.fis.Infer(infer_values);

										// mise a jour des panels graphiques de l'entree modifiee 
										for( int row=0 ; row<JInferenceFrame.this.inputs_table.getRowCount() ; row++ )
											((DefaultTableModel)JInferenceFrame.this.inputs_table.getModel()).fireTableCellUpdated(row, fi+1);
										// mise a jour des sorties
										if( JInferenceFrame.this.outputs_table != null )
											for( int i=0 ; i<JInferenceFrame.this.outputs_table.getColumnCount() ; i++ )
												for( int j=0 ; j<JInferenceFrame.this.outputs_table.getRowCount() ; j++ )
													((DefaultTableModel)JInferenceFrame.this.outputs_table.getModel()).fireTableCellUpdated(j, JInferenceFrame.this.outputs_table.convertColumnIndexToModel(i));							
									}catch( Throwable t ) { MessageFIS.Error(null, t); }
								}
							} );
						}
					}
				}
				catch( Throwable t ) { MessageFIS.Error(null, t); }				
			}
			public void menuDeselected(MenuEvent e) {}
			public void menuCanceled(MenuEvent e) {}
		});
		this.jMenuAlphaCut.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JNbAlphaCutFrame jacf = new JNbAlphaCutFrame(JInferenceFrame.this.nbAlphaCut);		
				jacf.setLocation(JInferenceFrame.this.ChildPosition(jacf.getSize()));
				JInferenceFrame.this.nbAlphaCut = jacf.GetAlphaCutFrame(); 
				//inference
				double[] infer_values = new double[JInferenceFrame.this.nb_inputs];
				try {
					for(int i=0; i<JInferenceFrame.this.nb_inputs; i++)
						infer_values[i] = JInferenceFrame.this.sliders[i].getDoubleValue();
					JInferenceFrame.this.MakeDoubleKWSW();
					JInferenceFrame.this.fis.InferFuzzyInput(JInferenceFrame.this.kw, JInferenceFrame.this.sw, JInferenceFrame.this.nbAlphaCut, infer_values);
					// mise a jour des sorties
					if( JInferenceFrame.this.outputs_table != null )
						for( int i=0 ; i<JInferenceFrame.this.outputs_table.getColumnCount() ; i++ )
							for( int j=0 ; j<JInferenceFrame.this.outputs_table.getRowCount() ; j++ )
								((DefaultTableModel)JInferenceFrame.this.outputs_table.getModel()).fireTableCellUpdated(j, JInferenceFrame.this.outputs_table.convertColumnIndexToModel(i));				
				} catch (Throwable e1) { MessageFIS.Error(null, e1); }
			} });		
		this.jMenuEntreeFloue.add(this.jMenuGabarit);
		this.jMenuEntreeFloue.add(this.jMenuAlphaCut);
		this.jMenuBarInference.add(this.jMenuEntreeFloue);
		//On permet l'utilisation d'entrees floues :
		// - pour une ou deux entree et une seule sortie impli 
		// - pour plusieurs entrees et une seule sortie conj
		if(this.fis.NbOutputs() != 1) 
		{
			this.jMenuEntreeFloue.setEnabled(false);
			this.jMenuAlphaCut.setEnabled(false);
		}
		else 
		{
			if(this.fis.GetOutput(0).GetImplicatif() && this.fis.NbInputs() > 2 ) 
				this.jMenuEntreeFloue.setEnabled(false);
			//si on a une sortie et qu'elle est conj
			if (!this.IsFuzzyInputModeActive() || !this.fis.GetOutput(0).GetImplicatif()) this.jMenuAlphaCut.setEnabled(false);
		}

		// construction du modele de donnees
		Object [][]data = new Object[this.nb_rules][this.nb_inputs + this.nb_outputs + 1];
		Object []column_names = new Object[this.nb_inputs + this.nb_outputs + 1];
		final Class []column_class = new Class[this.nb_inputs + this.nb_outputs + 1];
		column_names[0] = new String("Rules");
		column_class[0] = Integer.class;
		// construction des tableaux d'entrees et sorties
		for( int i=0 ; i<this.fis.NbInputs() ; i++ )
		{
			column_names[i+1] = new String(this.fis.GetInput(i).GetName());
			column_class[i+1] = Integer.class;
		}
		for( int i=0 ; i<this.fis.NbOutputs() ; i++ )
		{
			column_names[this.nb_inputs+i+1] = new String(this.fis.GetOutput(i).GetName());
			column_class[this.nb_inputs+i+1] = java.lang.Double.class;
		}
		// construction du tableau data
		for( int i=0, i_active=0 ; i<this.fis.NbRules() ; i++ )
		{
			JRule rule = this.fis.GetRule(i);
			if( rule.GetActive() == true )
			{
				data[i_active][0] = new Integer(i+1); // numero des regles actives
				for( int j=0 ; j<this.fis.NbInputs() ; j++ )
					data[i_active][j+1] = new Integer(this.fis.GetRule(i).Facteurs()[j]);
				for( int j=0 ; j<this.fis.NbOutputs() ; j++ )
					data[i_active][this.nb_inputs+j+1] = new java.lang.Double(this.fis.GetRule(i).Actions()[j]);
				i_active++;
			}
		}

		// construction du modele triable
		this.rules_model = new SortableTableModel()
		{
			public Class getColumnClass(int col) { return column_class[col]; }
			public boolean isCellEditable(int row, int column) { return false; }
		};
		this.rules_model.setDataVector(data, column_names);
		this.rules_model.setSorter(new TableSorter(this.rules_model) // modification de la classe de triage
		{
			// surcharge de la fonction de tri de la classe TableSorter pour implementer les regles de tri suivantes:
			// - si 2 cellules sont egales, on tri sur les numeros de regles ascendants (colonne 0)
			// - sur les colonnes d'entrees:
			//     - si une cellule est nulle (pas de SEF), elle est toujours triee en fin de tableau
			public void sort(final int column, final boolean isAscent)
			{
				int n = model.getRowCount();
				int[] indexes = model.getIndexes();

				// affichage test du tri
				/*for( int y=0 ; y<model.getRowCount() ; y++ )
        			System.out.print(model.getValueAt(y,column));
      				System.out.println("");*/
				for (int i=0; i<n-1; i++)
				{
					int k = i;
					for (int j=i+1; j<n; j++)
					{
						if (isAscent)  // tri ascendant
						{
							if( column < JInferenceFrame.this.nb_inputs+1 ) // colonnes d'entrees
							{
								if( this.Zero(column, j, k) == true ) // la cellule j est nulle et k > 0: on force le tri j > k
									k = j;
								else if( (compare(column, j, k) < 0) && (((Number)model.getValueAt(j, column)).doubleValue() != 0) ) // k > j et j != 0
									k = j;
								else if ((compare(column, j, k) == 0) && (compare(0, j, k) < 0)) // les cellules sont egales, tri sur les numeros de regles
									k = j;
							}
							else // colonnes de sorties
							{
								if( compare(column, j, k) < 0 )
									k = j;
								else if ((compare(column, j, k) == 0) && (compare(0, j, k) < 0)) // les cellules sont egales, tri sur les numeros de regles
									k = j;
							}
						}
						else // tri descendant
						{
							if (compare(column, j, k) > 0)
								k = j;
							else if ((compare(column, j, k) == 0) && (compare(0, j, k) < 0)) // les cellules sont egales, tri sur les numeros de regles
								k = j;
						}
					}
					int tmp = indexes[i];
					indexes[i] = indexes[k];
					indexes[k] = tmp;
					// affichage test du tri
					/*for( int y=0 ; y<model.getRowCount() ; y++ )
          				System.out.print(model.getValueAt(y,column));
        				System.out.println("");*/
				}
			}
			// fonction utilisee par la fonction sort ci-dessus pour implementer le tri particulier des cellules d'entrees nulles
			// return true si row2 est nulle et row1 non nulle, false sinon
			public boolean Zero(int column, int row1, int row2)
			{
				Object o1 = model.getValueAt(row1, column);
				Object o2 = model.getValueAt(row2, column);
				if( (o1 == null) || (o2 == null) ) { return  false; }
				else if( (((Number)o2).doubleValue() == 0) && (((Number)o1).doubleValue() != 0) ) { return true; }
				return false;
			}
		} );

		// construction des tables entrees et sorties
		this.tables_headers = new Vector();
		this.table_header_renderer = new SortButtonRenderer() // renderer des entetes des tables
		{
			// surcharge de getTableCellRendererComponent pour traduction de la colonne regle et ajout des tooltips
			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
			{
				Component comp = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
				if( table.convertColumnIndexToModel(column) == 0 )
				{
					((JButton)comp).setText(LocaleFIS.GetString(value.toString()));
					((JButton)comp).setToolTipText(null);
				}
				try
				{
					if( (table.convertColumnIndexToModel(column) > 0) && (table.convertColumnIndexToModel(column) <= JInferenceFrame.this.nb_inputs) )
						((JButton)comp).setToolTipText(JInferenceFrame.this.fis.GetInput(table.convertColumnIndexToModel(column)-1).GetName());
					if( table.convertColumnIndexToModel(column) > JInferenceFrame.this.nb_inputs )
						((JButton)comp).setToolTipText(JInferenceFrame.this.fis.GetOutput(table.convertColumnIndexToModel(column)-JInferenceFrame.this.nb_inputs-1).GetName());
				} catch( Throwable t ) {}
				return comp;
			}
		};
		JScrollPane jsp_inputs = this.InputsTable();
		JScrollPane jsp_outputs = this.OutputsTable();

		// tri des colonnes de la table
		if( this.sort_col != -1 )
		{
			this.table_header_renderer.setPressedColumn(this.sort_col);
			this.table_header_renderer.setSelectedColumn(this.sort_col);
			if( this.is_ascent == false ) // fait un appel supplementaire a SortButtonRenderer.setSelectedColumn pour mettre la variable SortButtonRenderer.state dans le bon etat
				this.table_header_renderer.setSelectedColumn(this.sort_col);
			((SortableTableModel)this.inputs_table.getModel()).sortByColumn(this.sort_col, this.is_ascent);
			this.table_header_renderer.setPressedColumn(-1);
		}

		// asservi la scrollbar verticale de la table sortie sur la scrollbar verticale de la table entree
		JScrollBar inputs_Bar = jsp_inputs.getVerticalScrollBar();
		JScrollBar outputs_Bar = jsp_outputs.getVerticalScrollBar();
		jsp_outputs.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER); // cache la scrollbar verticale
		// modele de scroll bar
		this.scroll_model = new DefaultBoundedRangeModel();
		inputs_Bar.setModel( this.scroll_model );
		outputs_Bar.setModel( this.scroll_model );

		// tables dans un JSplitPane
		if( this.split_pane != null )
			this.getContentPane().remove(this.split_pane);
		this.split_pane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, jsp_inputs, jsp_outputs )
		{
			public int getDividerSize() { return 2; }
			public Dimension getMinimumSize() { return  new Dimension(300,200); }
		};
		// configure le redimensionnement du JSplitPane en fonction du nombres d'entrees et sorties actives
		int nb_active_inputs = this.fis.NbActiveInputs();
		int nb_active_outputs = this.fis.NbActiveOutputs();
		((JSplitPane)this.split_pane).setResizeWeight((1.0/(nb_active_inputs+nb_active_outputs))*nb_active_inputs);

		// listener de SIF
		if( this.fis_listener != null )
			this.fis.RemoveFISListener(this.fis_listener);
		this.fis_listener = new FISListener()
		{
			public void FISClosed() { JInferenceFrame.this.dispose(); }
			public void InputReplaced( int input_number )   { JInferenceFrame.this.ReInit(); }
			public void OutputReplaced( int output_number ) { JInferenceFrame.this.ReInit(); }
			public void InputActiveChanged( int input_number )   { JInferenceFrame.this.ReInit(); }
			public void OutputActiveChanged( int output_number ) { JInferenceFrame.this.ReInit(); }
			public void InputNameChanged( int input_number )   { JInferenceFrame.this.ReInit(); }
			public void OutputNameChanged( int output_number ) { JInferenceFrame.this.ReInit(); }
			public void InputRangeChanged( int input_number )   { JInferenceFrame.this.ReInit(); }
			public void OutputRangeChanged( int output_number ) { JInferenceFrame.this.ReInit(); }
			public void InputRemoved( int input_number )
			{
				JInferenceFrame.this.sliders_value.remove(input_number);
				JInferenceFrame.this.infer_range.remove(input_number);
				if( JInferenceFrame.this.sort_col == input_number+1) // l'entree supprimee est la colonne triee
					JInferenceFrame.this.sort_col = -1;
				if( JInferenceFrame.this.sort_col > input_number+1) // la colonne triee est apres l'entree supprimee
					JInferenceFrame.this.sort_col--; // decalage de la colonne triee
				JInferenceFrame.this.ReInit();
			}
			public void OutputRemoved( int output_number )
			{
				if( JInferenceFrame.this.sort_col == JInferenceFrame.this.nb_inputs+output_number+1) // la sortie supprimee est la colonne triee
					JInferenceFrame.this.sort_col = -1;
				if( JInferenceFrame.this.sort_col > JInferenceFrame.this.nb_inputs+output_number+1) // la colonne triee est apres la sortie supprimee
					JInferenceFrame.this.sort_col--; // decalage de la colonne triee
				JInferenceFrame.this.ReInit();
			}
			public void InputAdded( int input_number )
			{
				double []range = new double [] {0, 1};
				try  { range = JInferenceFrame.this.fis.GetInput(input_number).GetRange(); }
				catch( Throwable t ) { MessageFIS.Error(null, t); }
				JInferenceFrame.this.sliders_value.add(new java.lang.Double(range[0] + ((range[1] - range[0])/2)));
				JInferenceFrame.this.infer_range.add(range);
				if( JInferenceFrame.this.sort_col >= JInferenceFrame.this.nb_inputs+1)
					JInferenceFrame.this.sort_col++;
				JInferenceFrame.this.ReInit();
			}
			public void OutputAdded( int output_number ) { JInferenceFrame.this.ReInit(); }
			public void MFRemovedInInput( int input_number )    { JInferenceFrame.this.ReInit(); }
			public void MFRemovedInOutput( int output_number )  { JInferenceFrame.this.ReInit(); }
			public void MFReplacedInInput( int input_number )   { JInferenceFrame.this.ReInit(); }
			public void MFReplacedInOutput( int output_number ) { JInferenceFrame.this.ReInit(); }
			public void OutputDefuzChanged( int output_number ) { JInferenceFrame.this.ReInit(); }
			public void OutputImpliChanged( int output_number ) { JInferenceFrame.this.ReInit(); }
			public void OutputDisjChanged( int output_number )  { JInferenceFrame.this.ReInit(); }
			public void OutputClassifChanged( int output_number ) { JInferenceFrame.this.ReInit(); }
			public void MFAddedInInput( int input_number ) {}
			public void MFAddedInOutput( int output_number ) {}
			public void OutputDefaultChanged( int output_number ) { JInferenceFrame.this.ReInit(); }
			public void OutputAlarmThresChanged( int output_number ) { JInferenceFrame.this.ReInit(); }
			public void RulesModified() { JInferenceFrame.this.ReInit(); }
			public void ConjunctionChanged() { JInferenceFrame.this.ReInit(); }
		};
		this.fis.AddFISListener(this.fis_listener);

		// affichage
		this.Translate();
		this.getContentPane().add(this.split_pane);
	}
	//------------------------------------------------------------------------------
	/**
	 * Fermeture de la fenetre
	 */
	public void dispose()
	{
		super.dispose();
		this.frame_open.Release();
		this.fis.RemoveFISListener(this.fis_listener);
	}
	//------------------------------------------------------------------------------
	public JInferenceFrame( JFISFrame parent, JFIS fis, JSemaphore open, JExtendedDataFile data_file ) throws Throwable
	{
		super(parent);
		this.fis = fis;
		this.frame_open = open;
		this.data_file = data_file;

		// construction des valeurs initiales des curseurs et domaines d'affichage
		this.sliders_value = new Vector();
		this.infer_range = new Vector();
		for( int i=0 ; i<this.fis.NbInputs() ; i++ )
		{
			double []range = this.fis.GetInput(i).GetRange();
			this.sliders_value.add(new java.lang.Double(range[0] + ((range[1] - range[0])/2)));
			this.infer_range.add(range);
		}

		this.Init();
		this.pack();
		this.frame_open.Acquire();
		this.setLocation(this.ChildPosition(this.getSize()));
	}
	//------------------------------------------------------------------------------	
	/**
	 * Traduction de la fenetre
	 */
	public void Translate() throws Throwable
	{
		this.setTitle(LocaleFIS.GetString("Inference"));
		this.jMenuDisplay.setText(LocaleFIS.GetString("Display"));
		this.jMenuRange.setText(LocaleFIS.GetString("Range"));
		this.jMenuEntreeFloue.setText(LocaleFIS.GetString("EntreeFloue"));
		this.jMenuGabarit.setText(LocaleFIS.GetString("Gabarit"));
		this.jMenuAlphaCut.setText(LocaleFIS.GetString("AlphaCut"));
		this.repaint();
	}
	//------------------------------------------------------------------------------
	/**
	 * Construction des tableaux de gabarits
	 * @throws Throwable 
	 */	
	private void MakeDoubleKWSW() throws Throwable
	{
		int nbInput = JInferenceFrame.this.nb_inputs;
		this.kw = new double[nbInput];
		this.sw = new double[nbInput];
		for(int i = 0; i < nbInput ; i++)
		{
			this.kw[i] = JInferenceFrame.this.fis.GetInput(i).GetKernelWeightTemplate();
			this.sw[i] = JInferenceFrame.this.fis.GetInput(i).GetSupportWeightTemplate();
		}
	}
	//------------------------------------------------------------------------------
	/**
	 * Verifie que l'on est en mode entrees floues
	 * ie au moins un des gabarits est actif : sw et kw != 0
	 * @throws Throwable 
	 */	
	private boolean IsFuzzyInputModeActive() throws Throwable
	{
		for(int i = 0; i < JInferenceFrame.this.nb_inputs; i++)
			if(JInferenceFrame.this.fis.GetInput(i).GetKernelWeightTemplate() != 0.0 ||
					JInferenceFrame.this.fis.GetInput(i).GetSupportWeightTemplate() != 0.0 ) 
				return true;
		return false;
	}	

	//------------------------------------------------------------------------------
	/**
	 * Classe renderer des cellules de la table des sorties nette\n
	 * affiche le poids des regles et entre parentheses la conclusion des regles
	 */
	class CrispRenderer extends JPanel implements TableCellRenderer
	{
		JLabel label_weight = new JLabel(); /**< label poids */
		JLabel label_conc = new JLabel(); /**< label conclusion */
		//------------------------------------------------------------------------------
		public CrispRenderer()
		{
			this.setLayout(new GridBagLayout());
			this.add(this.label_weight, new GridBagConstraints(0, 0, 1, 1, 0.0, 1.0
					,GridBagConstraints.CENTER, GridBagConstraints.VERTICAL, new Insets(0, 0, 0, 0), 0, 0));
			this.add(this.label_conc,  new GridBagConstraints(0, 1, 1, 1, 0.0, 1.0
					,GridBagConstraints.CENTER, GridBagConstraints.VERTICAL, new Insets(0, 0, 0, 0), 0, 0));
		}
		//------------------------------------------------------------------------------
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
		{
			int rule_number = ((Integer)((DefaultTableModel)table.getModel()).getValueAt(row, 0)).intValue()-1;
			double poids = 0;
			try { poids = JInferenceFrame.this.fis.GetRule(rule_number).Poids(); } catch( Throwable t ) {}
			this.label_weight.setText(DecimalFormat.getInstance().format(poids)); // affichage poids
			this.label_conc.setText("(" + DecimalFormat.getInstance().format(value) + ")"); // affichage conclusion
			return this;
		}
	}

	//------------------------------------------------------------------------------
	/**
	 * Classe renderer des cellules agregation de la table des sorties nettes\n
	 * affiche les resultats d'agregation dans un objet FISPlot
	 */
	class CrispAgregationRenderer extends FuzzyAgregationRenderer
	{
		//------------------------------------------------------------------------------
		/**
		 * Constructeur
		 * @param output objet JOutput de la sortie
		 * @param output_number numero d'ordre de la sortie dans le SIF
		 */
		public CrispAgregationRenderer( JOutput output, int output_number ) throws Throwable { super(output, output_number); }
		//------------------------------------------------------------------------------
		/**
		 * Affichage des resultats d'agregation sous la forme de SEFs discret
		 */
		protected void ShowMF(Graphics g) throws Throwable
		{
			this.clear(false);
			double [][] result = JInferenceFrame.this.fis.AgregationResult(this.number);
			for( int i=0 ; i<result[0].length ; i++ )
			{
				double [] param = { result[0][i] };
				JMF mf = new JMFDiscrete("", param );
				mf.Draw(this, 9); // data set 9 pour affichage en gris
				mf.Delete();
			}
		}
		//------------------------------------------------------------------------------
		/**
		 * Affichage des resultats d'agregation
		 */
		protected void ShowAlphaKernel(Graphics g) throws Throwable
		{
			double [][] result = JInferenceFrame.this.fis.AgregationResult(this.number);
			for( int i=0 ; i<result[0].length ; i++ )
			{
				double [] param = { result[0][i] };
				JMF mf = new JMFDiscrete("", param );
				if( java.lang.Double.isNaN((double)result[1][i]) == false )
					mf.DrawAlphaKernel(this, (Graphics2D)g, result[1][i]);
				mf.Delete();
			}
		}
		//------------------------------------------------------------------------------
		protected void ShowMFGlob(Graphics g) throws Throwable {}
		//------------------------------------------------------------------------------
		protected void ShowMFConc(Graphics g) throws Throwable {}
	}

	//------------------------------------------------------------------------------
	/**
	 * Classe renderer des cellules agregation de la table des sorties floues\n
	 * affiche les resultats d'agregation dans un objet FISPlot
	 */
	class FuzzyAgregationRenderer extends FuzzyRenderer
	{
		//------------------------------------------------------------------------------
		/**
		 * Constructeur
		 * @param output objet JOutput de la sortie
		 * @param output_number numero d'ordre de la sortie dans le SIF
		 */
		public FuzzyAgregationRenderer( JOutput output, int output_number ) throws Throwable { super(output, output_number); }
		//------------------------------------------------------------------------------
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) { return this; }
		//------------------------------------------------------------------------------
		/**
		 * Affichage des SEF de la sortie
		 */
		protected void ShowMF(Graphics g) throws Throwable
		{			
			this.clear(false);
			for( int i=0 ; i<this.input.GetNbMF() ; i++ )
				this.input.GetMF(i).NewMF().Draw(this, 9); // data set 9 pour affichage en gris
		}
		//------------------------------------------------------------------------------
		/**
		 * Affichage des resultats d'agregation
		 */
		protected void ShowAlphaKernel(Graphics g) throws Throwable
		{
			double [][] result = JInferenceFrame.this.fis.AgregationResult(this.number);			
			for( int i=0 ; i<result[0].length ; i++ )
				if( ((int)result[0][i] != 0) && (java.lang.Double.isNaN((double)result[1][i]) == false) )
					this.input.GetMF((int)result[0][i]-1).NewMF().DrawAlphaKernel(this, (Graphics2D)g, result[1][i]);

		}
		//------------------------------------------------------------------------------
		/**
		 * Affichage des resultats d'agregation implicatif
		 */		
		protected void ShowMFGlob(Graphics g) throws Throwable
		{
			Vector<java.lang.Double> dPoss = new Vector<java.lang.Double>();
			int nbPoss;

			nbPoss = JInferenceFrame.this.fis.AgregationImpliResult(this.number, dPoss);

			if (nbPoss >= 2)
				this.input.GetMFGlob().NewMF().DrawMFDPoss(this, (Graphics2D)g, dPoss);

		}
		//------------------------------------------------------------------------------
		protected void ShowMFConc(Graphics g) throws Throwable {}

	}
	//------------------------------------------------------------------------------
	/**
	 * Classe renderer des cellules de la table des sorties floues\n
	 * affiche le SEF et le poids d'appartenance au SEF dans un objet FISPlot
	 */
	class FuzzyRenderer extends InputRenderer
	{
		int rule_number; /**< numero de la regle */
		//------------------------------------------------------------------------------
		/**
		 * Constructeur
		 * @param output objet JOutput de la sortie
		 * @param output_number numero d'ordre de la sortie dans le SIF
		 */
		public FuzzyRenderer( JOutput output, int output_number ) throws Throwable { super(output, output_number); }
		//------------------------------------------------------------------------------
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
		{
			// lecture du numero de regle et du numero de SEF dans la table
			this.rule_number = ((Integer)((DefaultTableModel)table.getModel()).getValueAt(row, 0)).intValue()-1;
			this.sef_number = ((java.lang.Double)value).intValue();
			try
			{
				// ajout d'un tooltip avec le nom du SEF
				if( this.sef_number != 0 ) this.setToolTipText(this.input.GetMF(this.sef_number-1).GetName());
			} catch( Throwable t ) {}
			return ((java.lang.Double)value).intValue() == 0 ? new JPanel() : this; // retourne un JPanel vide si pas de SEF
		}
		//------------------------------------------------------------------------------
		/**
		 * Fixe le domaine d'affichage en abscisse de l'objet FISPlot\n
		 * domaine d'affichage = domaine de la sortie
		 */
		protected void SetXRange() throws Throwable
		{
			double []range = this.input.GetRange();
			this.setXRange(range[0], range[1]);
		}
		//------------------------------------------------------------------------------
		/**
		 * Lecture du poids de la regle
		 * @return le poids de la regle FuzzyRenderer#rule_number
		 */
		protected double Alpha() throws Throwable {
			if(JInferenceFrame.this.fis.GetOutput(this.number).GetImplicatif())
				return 0.0;
			else return JInferenceFrame.this.fis.GetRule(this.rule_number).Poids(); }
		//------------------------------------------------------------------------------
		protected void ShowRange(Graphics g) throws Throwable {} /**< pas d'affichage du domaine de la sortie */
		//------------------------------------------------------------------------------
		protected void ShowSlider(Graphics g) {} /**< pas d'affichage du curseur */

		//------------------------------------------------------------------------------
		/**
		 * Affichage des conclusions de regle cas implicatif
		 */		
		protected void ShowMFConc(Graphics g) throws Throwable
		{		
			Vector<java.lang.Double> dPoss = new Vector<java.lang.Double>();
			int nbPoss;

			nbPoss = JInferenceFrame.this.fis.GetImpliConc(this.number,this.rule_number, dPoss);

			if (nbPoss >= 2) //modif AT 8/4/09 ?
				this.input.GetMFConc(this.rule_number).NewMF().DrawMFDPoss(this, (Graphics2D)g, dPoss);	
			//this.input.GetMFGlob().NewMF().DrawMFDPoss(this, (Graphics2D)g, dPoss);	
		}
	}

	//------------------------------------------------------------------------------
	/**
	 * Classe renderer des cellules de la table des entrees\n
	 * affiche le SEF et le degre d'appartenance au SEF dans un objet FISPlot
	 */
	class InputRenderer extends FISPlot implements TableCellRenderer
	{
		protected JInput input;  /**< objet entree */
		protected int sef_number; /**< numero du SEF a afficher */
		protected int number; /**< numero d'ordre de l'entree dans le SIF */
		//------------------------------------------------------------------------------
		/**
		 * Constructeur
		 * @param input objet JInput de l'entree
		 * @param input_number numero d'ordre de l'entree dans le SIF
		 */
		private InputRenderer( JInput input,  int input_number ) throws Throwable
		{
			this.input = input;
			this.number = input_number;
			this.setYRange(0, 1);
			// ajout de ticks invisible
			this.addXTick("", java.lang.Double.MAX_VALUE);
			this.addYTick("", java.lang.Double.MAX_VALUE);
			FISPlot.DisableZoom(this);
			this.setShowTicks(false);
		}
		//------------------------------------------------------------------------------
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
		{
			this.sef_number = ((Integer)value).intValue(); // le numero du SEF a afficher est lu dans la table
			// ajout d'un tooltip avec le nom du SEF
			try { if( this.sef_number != 0 ) this.setToolTipText(this.input.GetMF(this.sef_number-1).GetName()); } catch( Throwable t ) {}
			return ((Integer)value).intValue() == 0 ? new JPanel() : this; // retourne un JPanel vide si pas de SEF
		}
		//------------------------------------------------------------------------------
		/**
		 * Surcharge de Plot._drawLine pour affichage en epaisseur 0.1
		 */
		protected void _drawLine(java.awt.Graphics graphics, int dataset, long startx, long starty, long endx, long endy, boolean clip, float width)
		{super._drawLine(graphics, dataset, startx, starty, endx, endy, clip, (float)0.1);}
		//------------------------------------------------------------------------------
		/**
		 * Fixe le domaine d'affichage en abscisse de l'objet FISPlot\n
		 * domaine d'affichage = domaine d'inference de l'entree
		 */
		protected void SetXRange() throws Throwable
		{
			double []range = (double [])JInferenceFrame.this.infer_range.elementAt(this.number);
			this.setXRange(range[0], range[1]);
		}
		//------------------------------------------------------------------------------
		/**
		 * Lecture du degre d'appartenance au SEF
		 * @return degre d'appartenance au SEF
		 */
	    protected double Alpha() throws Throwable 
	    { 
		// bch - July 16- for implicative FIS do not fill MF if fuzzy input
		// first check implicative outputs
		boolean implicative=false;
		for (int i=0;i<JInferenceFrame.this.fis.NbOutputs();i++)
		    if(JInferenceFrame.this.fis.GetOutput(i).GetImplicatif())
			implicative=true;
		// then check if fuzzy input
		if (implicative)
		    if(JInferenceFrame.this.fis.GetInput(this.number).GetKernelWeightTemplate()!=0.0 || 
		       JInferenceFrame.this.fis.GetInput(this.number).GetSupportWeightTemplate()!=0.0 )
			return 0.0;
		return this.input.Appartenance()[sef_number-1]; 
	    }
		//------------------------------------------------------------------------------
		/**
		 * Affichage du domaine de l'entree en pointille rouge si le domaine d'inference est superieur au domaine de l'entree
		 */
		protected void ShowRange(Graphics g) throws Throwable
		{
			double []input_range = this.input.GetRange();
			double []infer_range = (double [])JInferenceFrame.this.infer_range.elementAt(this.number);
			if( (input_range[0] != infer_range[0]) || (input_range[1] != infer_range[1]) )
				FISPlot.DrawXInputRange(g, this, input_range, this._xMin, this._xMax);
		}
		//------------------------------------------------------------------------------
		/**
		 * Affichage du curseur
		 */
		protected void ShowSlider(Graphics g)
		{
			Rectangle rect = this.getPlotRectangle();
			if( rect.height > 0 )
			{
				Graphics2D g2d = (Graphics2D)g;
				g2d.setColor(Color.black);
				g2d.setStroke(new BasicStroke());
				int x = this.XtoPixel(((java.lang.Double)JInferenceFrame.this.sliders_value.elementAt(this.number)).doubleValue());
				g2d.draw(new Line2D.Double(x, rect.y, x, rect.y+rect.height) );

				//FuzzyInput
				try {
					if(JInferenceFrame.this.fis.GetInput(this.number).GetKernelWeightTemplate()!=0.0 || 
							JInferenceFrame.this.fis.GetInput(this.number).GetSupportWeightTemplate()!=0.0 )
					{
						g2d.setColor(Color.blue);
						g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.8f ));
						double[] infer_values = new double[JInferenceFrame.this.nb_inputs];
						Vector<java.lang.Double> fuzval = new Vector<java.lang.Double>();
						int nbPoss;
						int j = 0;
						for(int i=0; i<JInferenceFrame.this.nb_inputs; i++)
							infer_values[i] = JInferenceFrame.this.sliders[i].getDoubleValue();
						// Build DPOSS

						nbPoss = JInferenceFrame.this.fis.GetBuildFuzIn(this.number, JInferenceFrame.this.kw, JInferenceFrame.this.sw, infer_values, fuzval);
						if (nbPoss >= 2)
						{
							Polygon2D.Double poly = new Polygon2D.Double(this.XtoPixel(fuzval.elementAt(j).doubleValue()),this.YtoPixel(fuzval.elementAt(j+1).doubleValue()));
							j++;
							for( int i = j+1 ; i<fuzval.size() ; i = i+2 )
								poly.lineTo(this.XtoPixel(fuzval.elementAt(i).doubleValue()),this.YtoPixel(fuzval.elementAt(i+1).doubleValue()));
							g2d.fill( poly );
						}

					}
				} catch( Throwable t ) { MessageFIS.Error( null, t ); }

			}
		}
		//------------------------------------------------------------------------------
		/**
		 * Affichage du SEF
		 */
		protected void ShowMF(Graphics g) throws Throwable
		{
			this.clear(false);
			this.input.GetMF(this.sef_number-1).NewMF().Draw(this, 9); // data set 9 pour affichage en gris
		}
		//------------------------------------------------------------------------------
		/**
		 * Affichage du degre d'appartenance au SEF, ShowAlphaKernel : conjounctif
		 * Affichage de distribution : implicatif
		 */
		protected void ShowAlphaKernel(Graphics g) throws Throwable  { this.input.GetMF(this.sef_number-1).NewMF().DrawAlphaKernel(this, (Graphics2D)g, this.Alpha()); }

		//------------------------------------------------------------------------------
		/**
		 * Affichage de distribution : implicatif
		 */		
		protected void ShowMFGlob(Graphics g) throws Throwable {}

		//------------------------------------------------------------------------------
		/**
		 * Affichage de distribution : implicatif
		 */		
		protected void ShowMFConc(Graphics g) throws Throwable {}	

		//------------------------------------------------------------------------------
		/**
		 * Affichage de l'objet FISPlot
		 */
		public void paintComponent(Graphics g)
		{
			try
			{
				// domaine d'affichage
				this.SetXRange();

				// affichage du SEF
				this.ShowMF(g);
				super.paintComponent(g);

				// ajustement de la taille du rectangle plot au panel
				Rectangle r = this.getPlotRectangle();
				r.x = 0;
				r.y = 0;
				r.width = this.getSize().width -1;
				r.height = this.getSize().height -1;
				this.setPlotRectangle(r);
				super.paintComponent(g);
				this.setPlotRectangle(null);  // supprime la memorisation du setPlotRectangle

				// affichage noyau
				this.ShowAlphaKernel(g);

				// affiche MFGlob
				this.ShowMFGlob(g);

				//affiche conclusion des regles
				this.ShowMFConc(g);

				// affichage des bornes du domaine de l'entree
				this.ShowRange(g);

				// affichage du curseur
				this.ShowSlider(g);
			}
			catch( Throwable t ) { MessageFIS.Error( null, t ); }
		}
	};

	public void setSliderInputValue (int input_number, double new_value)
	{
		sliders[input_number].setDoubleValue(new_value);
	}
	//------------------------------------------------------------------------------
	public void SetSortedColumn( int column ) { this.sort_col = column; }
	public int GetSortedColumn() { return this.sort_col; }
	public void SetSortAscent( boolean is_ascent ) { this.is_ascent = is_ascent; }
	public Vector GetTablesHeaders() { return this.tables_headers; }
};




//------------------------------------------------------------------------------
/**
 * Classe listener des headers de table triable\n
 * intercepte les actions sur la souris et declenche le tri du modele de la table
 * Ce listener est a utiliser quand le tri est realise sur plusieurs table qui partagent un meme modele de donnees.
 */
class SortableHeaderListener extends MouseAdapter
{
	private JTableHeader header; /**< Header de la table */
	private SortButtonRenderer renderer; /**< Renderer du header de table */
	private SortableObject sort_obj; /**< Reference de l'interface SortableObject */

	/**
	 * Interface de lien avec l'objet triable
	 */
	interface SortableObject {
		void SetSortedColumn(int column); /**< Memorisation du numero de colonne triee */
		int GetSortedColumn(); /**< Recuperation du numero de colonne triee */
		void SetSortAscent(boolean ascent); /**< Memorisation de l'ordre de tri */
		Vector GetTablesHeaders(); /**< Vecteur des objets JTableHeader des differentes tables utilisant le modele triable */
	}
	//------------------------------------------------------------------------------
	/**
	 * @param header objet JTableHeader de la table
	 * @param renderer objet Renderer du header de table
	 * @param sort_obj Reference a l'interface SortableObject
	 */
	SortableHeaderListener(JTableHeader header, SortButtonRenderer renderer, SortableObject sort_obj)
	{
		this.header   = header;
		this.renderer = renderer;
		this.sort_obj = sort_obj;
	}
	//------------------------------------------------------------------------------
	/**
	 * Fonction appelee lors du clic sur le header de la table\n
	 * declenche le tri du modele de la table
	 */
	public void mousePressed(MouseEvent e)
	{
		// pas de tri si on est en cours de redimensionnement de colonne
		if( ((JTableHeader)e.getSource()).getCursor().getType() == Cursor.E_RESIZE_CURSOR ) return;
		int col = header.columnAtPoint(e.getPoint());
		int sortCol = header.getTable().convertColumnIndexToModel(col); // colonne du modele cliquee
		this.sort_obj.SetSortedColumn( sortCol ); // memorisation de la colonne triee
		renderer.setPressedColumn(sortCol);
		renderer.setSelectedColumn(sortCol);
		// redessine l'ensemble des headers de table
		for( int i=0 ; i<this.sort_obj.GetTablesHeaders().size() ; i++ )
			((JTableHeader)this.sort_obj.GetTablesHeaders().elementAt(i)).repaint();
		if (header.getTable().isEditing()) { header.getTable().getCellEditor().stopCellEditing(); }
		boolean isAscent;
		if (SortButtonRenderer.DOWN == renderer.getState(sortCol)) { isAscent = true; }
		else { isAscent = false; }
		this.sort_obj.SetSortAscent(isAscent); // memorisation de l'ordre de tri
		((SortableTableModel)header.getTable().getModel()).sortByColumn(sortCol, isAscent); // tri de la colonne
	}
	//------------------------------------------------------------------------------
	/**
	 * Fonction appelee quand la souris est relachee
	 */
	public void mouseReleased(MouseEvent e) {
		renderer.setPressedColumn(-1);                // clear
		// redessine l'ensemble des headers de table
		for( int i=0 ; i<this.sort_obj.GetTablesHeaders().size() ; i++ )
			((JTableHeader)this.sort_obj.GetTablesHeaders().elementAt(i)).repaint();
	}	
}


class JNbAlphaCutFrame extends JFISDialog
{
	private JPanel jPanelAlphaCutFrame = new JPanel();
	private GridBagLayout gridBagLayoutAlphaCutFrame = new GridBagLayout();
	private JLabel jLabelACut = new JLabel();
	private IntegerField ifACut = new IntegerField();
	private JButton jButtonApply = new JButton();
	private JButton jButtonCancel = new JButton();
	private int nbAC;

	public JNbAlphaCutFrame (int nbac)
	{
		try
		{
			this.jLabelACut.setText(LocaleFIS.GetString("nbACut")+":");
			this.jButtonApply.setText(LocaleFIS.GetString("Apply"));
			this.jButtonCancel.setText(LocaleFIS.GetString("Cancel"));
			this.ifACut.setValue(nbac);
			this.nbAC = nbac;

			this.jButtonCancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonCancel_actionPerformed(e); } });
			this.jButtonApply.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { try { jButtonApply_actionPerformed(e); } catch (Exception e1) { MessageFIS.Error(null, e1); } } });

			this.getContentPane().add(this.jPanelAlphaCutFrame);
			this.jPanelAlphaCutFrame.setLayout(this.gridBagLayoutAlphaCutFrame);

			this.jPanelAlphaCutFrame.add(this.jLabelACut,    new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
					,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
			this.jPanelAlphaCutFrame.add(this.ifACut,    new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
					,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 50, 0));
			this.jPanelAlphaCutFrame.add(this.jButtonCancel, new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0
					,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
			this.jPanelAlphaCutFrame.add(this.jButtonApply,    new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
					,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));

			this.setModal(true);
			this.setResizable(false);
			this.pack();		

		}catch( Exception ex ) { MessageFIS.Error(null, ex);}
	}
	//------------------------------------------------------------------------------
	private void jButtonApply_actionPerformed(ActionEvent e) throws Exception
	{
		if(this.ifACut.getValue() <= 0 )
			throw new Exception(LocaleFIS.GetString("ACsupZero"));
		else
		{
			this.nbAC = this.ifACut.getValue();
			this.dispose();
		}
	} 	
	//------------------------------------------------------------------------------
	private void jButtonCancel_actionPerformed(ActionEvent e) { this.dispose(); }
	//------------------------------------------------------------------------------
	public int GetAlphaCutFrame()
	{
		this.setVisible(true);
		return this.nbAC;
	}	
}

class JGabaritFrame extends JFISDialog
{
	private JPanel jPanelGabaritFrame = new JPanel();
	private JPanel jPanelSaisie = new JPanel();
	private JPanel jPanelValidation = new JPanel();
	private GridBagLayout gridBagLayoutGabaritFrame = new GridBagLayout();
	private GridBagLayout gridBagLayoutValidation = new GridBagLayout();
	private GridBagLayout gridBLSaisie = new GridBagLayout();
	private JLabel jLabelLSupport = new JLabel();
	private JLabel jLabelLNoyau = new JLabel();
	private DoubleField dfLNoyau = new DoubleField();
	private DoubleField dfLSupport = new DoubleField();
	private IntegerField ifACut = new IntegerField();
	private JButton jButtonCancel = new JButton();
	private JButton jButtonApply = new JButton();
	private JMenuBar jMenuBarGabarit = new JMenuBar();
	private JMenu jMenuGabarit = new JMenu();
	protected JMenuItem jMenuOpenGabarit = new JMenuItem();
	protected JMenuItem jMenuSaveGabarit = new JMenuItem();
	private double kw;
	private double sw;
	private JFIS fis;
	private int num_input;

	public JGabaritFrame (JFIS fis, int in) throws Throwable
	{
		this.num_input = in;
		this.fis = fis;
		this.kw = fis.GetInput(this.num_input).GetKernelWeightTemplate();
		this.sw = fis.GetInput(this.num_input).GetSupportWeightTemplate();
		this.jbInit();
	}

	//------------------------------------------------------------------------------
	private void jMenuOpenGabarit_actionPerformed(ActionEvent e)
	{
		JOpenFileChooser file_chooser = new JOpenFileChooser(MainFIS.config.FISFilePath());
		file_chooser.setAcceptAllFileFilterUsed(true);
		JFileFilter filter = new JFileFilter("tpl", LocaleFIS.GetString("FilterFISFile"));
		file_chooser.addChoosableFileFilter(filter);
		if( file_chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION )
		{
			MainFIS.config.FISFilePath(file_chooser.getSelectedFile().getParent());
			try {
				fis.GetInput(this.num_input).ReadTemplate(file_chooser.getSelectedFile().getAbsolutePath());
				this.dfLNoyau.setValue(fis.GetInput(this.num_input).GetKernelWeightTemplate());
				this.dfLSupport.setValue(fis.GetInput(this.num_input).GetSupportWeightTemplate());
			} catch( Throwable ex ) { MessageFIS.Error(null, ex);}
		}
	}
	//------------------------------------------------------------------------------
	private void jMenuSaveGabarit_actionPerformed(ActionEvent e)
	{
		try { SaveAs(); }
		catch( Throwable t ) { MessageFIS.Error( null, t ); }
	}
	//------------------------------------------------------------------------------
	private void jButtonCancel_actionPerformed(ActionEvent e) { this.dispose(); }
	//------------------------------------------------------------------------------
	private void jButtonApply_actionPerformed(ActionEvent e) throws Throwable
	{
		this.kw = this.dfLNoyau.getValue();
		this.sw = this.dfLSupport.getValue();
		this.ReadGabarit();
		this.dispose();
	} 
	//------------------------------------------------------------------------------
	private void ReadGabarit() throws Throwable
	{		
		if((this.kw > this.sw || this.sw <= 0.0 || this.kw < 0) && !(this.sw == 0.0 && this.kw == 0.0) )
			throw new Exception(LocaleFIS.GetString("SWSupZero")+" "+LocaleFIS.GetString("SWSupKW"));
		fis.GetInput(this.num_input).SetTemplate(this.kw, this.sw);
	}
	//------------------------------------------------------------------------------
	private void jbInit() throws Exception
	{
		//translate
		this.jMenuGabarit.setText(LocaleFIS.GetString("Gabarit"));
		this.jMenuOpenGabarit.setText(LocaleFIS.GetString("Open"));
		this.jMenuSaveGabarit.setText(LocaleFIS.GetString("Save"));
		this.jLabelLNoyau.setText(LocaleFIS.GetString("largNoyau")+":");
		this.jLabelLSupport.setText(LocaleFIS.GetString("largSupport")+":");
		this.jButtonCancel.setText(LocaleFIS.GetString("Cancel"));
		this.jButtonApply.setText(LocaleFIS.GetString("Apply"));	

		//events
		this.jMenuGabarit.addMenuListener(new MenuListener()
		{
			public void menuSelected(MenuEvent e) {}
			public void menuDeselected(MenuEvent e) {}
			public void menuCanceled(MenuEvent e) {}
		});
		this.jMenuOpenGabarit.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuOpenGabarit_actionPerformed(e); } });
		this.jMenuSaveGabarit.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuSaveGabarit_actionPerformed(e); } });
		//boutons
		this.jButtonCancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonCancel_actionPerformed(e); } });
		this.jButtonApply.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) {
			try {jButtonApply_actionPerformed(e);} catch (Throwable e1) {MessageFIS.Error(null, e1);}}});

		//init val
		this.dfLNoyau.setValue(this.kw);
		this.dfLSupport.setValue(this.sw);	

		this.jPanelValidation.setLayout(gridBagLayoutValidation);
		this.jPanelSaisie.setLayout(gridBLSaisie);
		this.jPanelGabaritFrame.setLayout(this.gridBagLayoutGabaritFrame);

		this.setTitle(LocaleFIS.GetString("EntreeFloue")+" "+(this.num_input + 1));

		// menus
		this.setJMenuBar(this.jMenuBarGabarit);
		this.jMenuBarGabarit.add(this.jMenuGabarit);
		this.jMenuGabarit.add(this.jMenuOpenGabarit);
		this.jMenuGabarit.add(this.jMenuSaveGabarit);

		// affichage des objets
		this.getContentPane().add(this.jPanelGabaritFrame);

		this.jPanelGabaritFrame.add(jPanelSaisie,    new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
		this.jPanelGabaritFrame.add(jPanelValidation,   new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 5, 5), 0, 0));

		jPanelValidation.add(jButtonApply,    new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
		jPanelValidation.add(jButtonCancel, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));

		jPanelSaisie.add(this.jLabelLNoyau,         new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
		jPanelSaisie.add(this.jLabelLSupport,         new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));	    

		jPanelSaisie.add(this.dfLNoyau,   new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 50, 0));   
		jPanelSaisie.add(this.dfLSupport,    new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 50, 0));

		this.setModal(true);
		this.setResizable(false);
		this.pack();
	}

	public void Show()
	{
		this.setVisible(true);
	}
	//------------------------------------------------------------------------------
	private boolean SaveAs() throws Throwable
	{
		JFileChooser file_chooser = new JFileChooser(MainFIS.config.FISFilePath())
		{
			protected JDialog createDialog(Component parent) throws HeadlessException
			{
				JDialog dialog = super.createDialog(parent);
				dialog.setResizable(false);
				return dialog;
			}
		};
		file_chooser.setAcceptAllFileFilterUsed(false);
		JFileFilter filter = new JFileFilter("tpl", LocaleFIS.GetString("FilterFISFile"));
		file_chooser.addChoosableFileFilter(filter);
		if( file_chooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION )
		{
			String file_name = file_chooser.getSelectedFile().getAbsolutePath();
			// ajoute l'extension si necessaire
			if( file_name.lastIndexOf('.') == -1 )
				file_name += ".tpl";

			// teste si le fichier existe
			File f = new File( file_name );
			if( f.exists() )
			{
				String message = LocaleFIS.GetString("TheFISFile") + " : " + file_name + " " + LocaleFIS.GetString("AlreadyExist") + "\n" + LocaleFIS.GetString("DoYouWantToReplaceIt");
				if( MessageFIS.Confirm(this, message) == JOptionPane.NO_OPTION )
					return false;
			}

			try
			{
				MainFIS.config.FISFilePath(file_chooser.getSelectedFile().getParent());
				fis.GetInput(this.num_input).WriteTemplate(file_name, this.kw, this.sw);
				return true;
			}
			catch( Throwable t ) {  MessageFIS.Error( null, t ); }
		}
		return false;
	}

}

