//***********************************************************************
//
//                              JInput.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.util.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

//------------------------------------------------------------------------------
public class JInput
{
	protected long ptr;

	//------------------------------------------------------------------------------
	public JInput() throws Throwable
	{
		this.ptr = jnifis.NewInput();
		this.SetName("new input");
		this.SetRange(0, 1);
		this.SetActive(true);
	}
	//------------------------------------------------------------------------------
	public JInput( JInput input ) throws Throwable { this.ptr = jnifis.NewInput( input.Ptr() ); }
	//------------------------------------------------------------------------------
	public JInput( long ptr ) { this.ptr = ptr; }
	//------------------------------------------------------------------------------
	public JInput( double [] t, double min, double max, boolean sort ) throws Throwable
	{
		this.ptr = jnifis.NewInput(t, min, max, sort);
		this.SetName("new input");
		this.SetActive(true);
	}
	//------------------------------------------------------------------------------
	public JInput NewRegular( int nb_sef, double min, double max) throws Throwable
	{
		JInput new_input = new JInput( jnifis.NewRegularInput(nb_sef, min, max) );
		new_input.SetName(this.GetName());
		new_input.SetActive(this.GetActive());
		// les SEF sont crees sans nom -> affectation d'un nom par defaut
		for( int i=0 ; i<new_input.GetNbMF() ; i++ )
			((JMF)new_input.GetMF(i)).SetName(LocaleFIS.GetString("MF") + String.valueOf(i+1) );
		return new_input;
	}
	//------------------------------------------------------------------------------
	public JInput NewIrregular( double []sommets, int nb_sef, double min, double max) throws Throwable
	{
		JInput new_input = new JInput( jnifis.NewIrregularInput(sommets, nb_sef, min, max) );
		new_input.SetName(this.GetName());
		new_input.SetActive(this.GetActive());
		// les SEF sont crees sans nom -> affectation d'un nom par defaut
		for( int i=0 ; i<new_input.GetNbMF() ; i++ )
			((JMF)new_input.GetMF(i)).SetName(LocaleFIS.GetString("MF") + String.valueOf(i+1) );
		return new_input;
	}
	//------------------------------------------------------------------------------
	public void Delete() throws Throwable { jnifis.DeleteInput( ptr ); ptr = 0; }
	//------------------------------------------------------------------------------
	protected long Ptr() { return ptr; }
	//------------------------------------------------------------------------------
	public String GetName() throws Throwable { return jnifis.GetInputName( ptr ); }
	//------------------------------------------------------------------------------
	public boolean GetActive() throws Throwable { return jnifis.GetInputActive( ptr ); }
	//------------------------------------------------------------------------------
	public double[] GetRange() throws Throwable { return jnifis.GetInputRange( ptr ); }
	//------------------------------------------------------------------------------
	public void SetName( String name ) throws Throwable { jnifis.SetInputName(this.ptr, name); }
	//------------------------------------------------------------------------------
	public void SetRange( double[] range ) throws Throwable { jnifis.SetInputRange(this.ptr, range[0], range[1]); }
	//------------------------------------------------------------------------------
	public void SetRange( double range_min, double range_max ) throws Throwable { jnifis.SetInputRange(this.ptr, range_min, range_max); }
	//------------------------------------------------------------------------------
	public void SetActive( boolean active ) throws Throwable { jnifis.SetInputActive(this.ptr, active); }
	//------------------------------------------------------------------------------
	public int GetNbMF() throws Throwable {  return jnifis.GetNbMF( ptr ); }
	//------------------------------------------------------------------------------
	public JMF GetMF( int mf_number ) throws Throwable { return new JMF( jnifis.GetMF(ptr, mf_number) ); }
	//------------------------------------------------------------------------------
	public JMF GetMFGlob() throws Throwable { return new JMF( jnifis.GetMFGlob(this.ptr) ); }
	//------------------------------------------------------------------------------
	public JMF GetMFConc(int rule_number) throws Throwable { return new JMF( jnifis.GetMFConc(ptr,rule_number) ); }
	//------------------------------------------------------------------------------
	public void AddMF( JMF sef ) throws Throwable { jnifis.AddMF(ptr, sef.Ptr()); }
	//------------------------------------------------------------------------------
	public void ReplaceMF( int mf_number, JMF new_sef ) throws Throwable { jnifis.ReplaceMF(ptr, mf_number, new_sef.Ptr()); }
	//------------------------------------------------------------------------------
	public double [] Appartenance() throws Throwable { return jnifis.InputAppartenance(ptr); }
	//------------------------------------------------------------------------------
	public double[] GetBreakPoints() throws Throwable {return jnifis.GetBreakPoints(ptr) ;}
	//------------------------------------------------------------------------------
	public boolean IsSfp() throws Throwable { return jnifis.IsSfp(ptr); }
	//------------------------------------------------------------------------------ 
	public void SetTemplate(double kw, double sw) { jnifis.SetTemplate(ptr, kw, sw);};
	//------------------------------------------------------------------------------ 
	public double GetKernelWeightTemplate() { return jnifis.GetKernelWeightTemplate(ptr);}
	//------------------------------------------------------------------------------ 
	public double GetSupportWeightTemplate() { return jnifis.GetSupportWeightTemplate(ptr);}
	//------------------------------------------------------------------------------
	public void WriteTemplate(String file, double kw, double sw) throws Throwable { jnifis.WriteTemplate(file, kw, sw); }
	//------------------------------------------------------------------------------
	public void ReadTemplate(String file) throws Throwable 
	{
		double [] res = jnifis.ReadTemplate(file);
		this.SetTemplate(res[0], res[1]);
	}
	public double[] PcPe(double[] datas) { return jnifis.PcPe(ptr, datas, datas.length); }
	public void CheckFuzDist() { jnifis.CheckFuzDist(ptr); }
	public double Distance(double x, double y, boolean normalize) { return jnifis.InputDistance(ptr, x, y, normalize); }
}

//------------------------------------------------------------------------------
class DoubleFieldInput extends DoubleField
{
	private JInput Input;
	private double [] Range;
	//------------------------------------------------------------------------------
	public JInput Input() { return this.Input; }
	//------------------------------------------------------------------------------
	public void SetInput( JInput input ) throws Throwable
	{
		this.Input = input;
		Range = this.Input.GetRange();
	}
	//------------------------------------------------------------------------------
	public DoubleFieldInput( JInput input ) throws Throwable { this.SetInput(input); }
	//------------------------------------------------------------------------------
	protected void ColorizeValue( double value )
	{
		try { Range = Input.GetRange(); }
		catch( Throwable t ) {}
		if( (value < Range[0]) || (value > Range[1]) )
			this.setForeground(Color.red);
		else
			this.setForeground(Color.black);
	}
	//------------------------------------------------------------------------------
	public void setValue(double value)
	{
		super.setValue(value);
		ColorizeValue( value );
	}
	//------------------------------------------------------------------------------
	public void actionPerformed(ActionEvent e)
	{
		this.setValue(this.getValue());
		super.actionPerformed(e);
	}
	//------------------------------------------------------------------------------
	public void FocusLost()
	{
		if( this.isEditable() == true )
		{
			this.setValue(this.getValue());
			super.FocusLost();
		}
	}
}


