//***********************************************************************
//
//                              JInputFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import java.lang.*;
import javax.swing.event.*;
import javax.swing.plaf.basic.*;
import org.freehep.util.export.ExportDialog;
import java.awt.print.*;
import ptolemy.plot.*;


// classe de saisie des parametres du SEF
//------------------------------------------------------------------------------
abstract class MFParams
{
	protected JLabel [] jLParams;
	protected DoubleField [] dfParams;


	//------------------------------------------------------------------------------
	public MFParams() {}
	public abstract int Number();
	//------------------------------------------------------------------------------
	public void SetParams( double [] params ) throws Throwable
	{
		if( params.length == this.dfParams.length )
			for( int i=0 ; i<this.dfParams.length ; i++ )
				this.dfParams[i].setValue(params[i]);
	}
	//------------------------------------------------------------------------------
	public double [] GetParams()
	{
		double [] result = new double[this.dfParams.length];
		for( int i=0 ; i<result.length ; i++ )
			result[i] = this.dfParams[i].getValue();
		return result;
	}
	//------------------------------------------------------------------------------
	public void RAZParams() throws Throwable
	{
		for( int i=0 ; i<this.dfParams.length ; i++ )
			this.dfParams[i].setValue(0);
	}
	//------------------------------------------------------------------------------
	public void Translate()
	{
		for( int i=0 ; i<this.dfParams.length ; i++ )
		{
			double sauve = this.dfParams[i].getValue();
			this.dfParams[i].setLocale(LocaleFIS.Locale());
			this.dfParams[i].setValue(sauve);
		}
	}
}


//------------------------------------------------------------------------------
class MFTriangularParams extends MFParams
{
	private JInput input;
	public MFTriangularParams( JPanel panel, JInput input ) throws Throwable
	{
		this.input = input;
		jLParams = new JLabel[3];
		dfParams = new DoubleFieldInput[3];
		for( int i=0 ; i<3 ; i++ )
		{
			jLParams[i] = new JLabel();
			dfParams[i] =  new DoubleFieldInput(input);
		}
		this.RAZParams();
		panel.removeAll();
		panel.add(jLParams[0],        new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
		panel.add(dfParams[0],                new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 60, 0));
		panel.add(jLParams[1],      new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
		panel.add(dfParams[1],        new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 60, 0));
		panel.add(jLParams[2],       new GridBagConstraints(2, 2, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
		panel.add(dfParams[2],           new GridBagConstraints(3, 2, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 60, 0));
		panel.repaint();
		this.Translate();
	}
	//------------------------------------------------------------------------------
	public void Translate()
	{
		super.Translate();
		jLParams[0].setText("S1:");
		jLParams[1].setText("S2:");
		jLParams[2].setText("S3:");
	}
	//------------------------------------------------------------------------------
	public int Number() { return JMF.TRIANGULAR; }
	//------------------------------------------------------------------------------
	public void RAZParams() throws Throwable
	{
		double [] range = input.GetRange();
		dfParams[0].setValue(range[0]);
		dfParams[1].setValue(range[0] + ((range[1] - range[0])/2));
		dfParams[2].setValue(range[1]);
	}
}

//------------------------------------------------------------------------------
class MFSinusParams extends MFParams
{
	protected JInput input;
	public MFSinusParams( JPanel panel, JInput input ) throws Throwable
	{
		this.input = input;
		jLParams = new JLabel[2];
		dfParams = new DoubleFieldInput[2];
		for( int i=0 ; i<2 ; i++ )
		{
			jLParams[i] = new JLabel();
			dfParams[i] =  new DoubleFieldInput(input);
		}
		this.RAZParams();
		panel.removeAll();
		panel.add(jLParams[0],        new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
		panel.add(dfParams[0],                new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 5, 0), 60, 0));
		panel.add(jLParams[1],      new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
		panel.add(dfParams[1],        new GridBagConstraints(3, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 5, 0), 60, 0));
		panel.repaint();
		this.Translate();
	}
	//------------------------------------------------------------------------------
	public void Translate()
	{
		super.Translate();
		jLParams[0].setText("S1:");
		jLParams[1].setText("S2:");
	}
	//------------------------------------------------------------------------------
	public int Number() { return JMF.SINUS; }
	//------------------------------------------------------------------------------
	public void RAZParams() throws Throwable
	{
		double [] range = input.GetRange();
		dfParams[0].setValue(range[0]);
		dfParams[1].setValue(range[1]);
	}
}

class MFSinusParamsInf extends MFSinusParams
{
	public MFSinusParamsInf( JPanel panel, JInput input ) throws Throwable {
		super(panel, input);
	}

	public int Number() { return JMF.SINUSINF; }

	public void RAZParams() throws Throwable {
		double [] range = input.GetRange();
		dfParams[0].setValue(range[0]);
		dfParams[1].setValue(range[0] + ((range[1] - range[0])/2));
	}
}

class MFSinusParamsSup extends MFSinusParams
{
	public MFSinusParamsSup( JPanel panel, JInput input ) throws Throwable {
		super(panel, input);
	}

	public int Number() { return JMF.SINUSSUP; }

	public void RAZParams() throws Throwable {
		double [] range = input.GetRange();
		dfParams[0].setValue(range[0] + ((range[1] - range[0])/2));
		dfParams[1].setValue(range[1]);
	}
}

//------------------------------------------------------------------------------
class MFTrapezoidalParams extends MFParams
{
	private JInput input;
	public MFTrapezoidalParams( JPanel panel, JInput input ) throws Throwable
	{
		this.input = input;
		jLParams = new JLabel[4];
		dfParams = new DoubleFieldInput[4];
		for( int i=0 ; i<4 ; i++ )
		{
			jLParams[i] = new JLabel();
			dfParams[i] =  new DoubleFieldInput(input);
		}
		this.RAZParams();
		panel.removeAll();
		panel.add(jLParams[0],         new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
		panel.add(dfParams[0],                 new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 60, 0));
		panel.add(jLParams[1],         new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
		panel.add(dfParams[1],           new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 60, 0));
		panel.add(jLParams[2],         new GridBagConstraints(3, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
		panel.add(dfParams[2],          new GridBagConstraints(4, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 60, 0));
		panel.add(jLParams[3],    new GridBagConstraints(4, 2, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
		panel.add(dfParams[3],        new GridBagConstraints(5, 2, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 60, 0));
		panel.repaint();
		this.Translate();
	}
	//------------------------------------------------------------------------------
	public void Translate()
	{
		super.Translate();
		jLParams[0].setText("S1:");
		jLParams[1].setText("S2:");
		jLParams[2].setText("S3:");
		jLParams[3].setText("S4:");
	}
	//------------------------------------------------------------------------------
	public int Number() { return JMF.TRAPEZOIDAL; }
	//------------------------------------------------------------------------------
	public void RAZParams() throws Throwable
	{
		double [] range = input.GetRange();
		dfParams[0].setValue(range[0]);
		dfParams[1].setValue(range[0] + ((range[1] - range[0])/3));
		dfParams[2].setValue(range[1] - ((range[1] - range[0])/3));
		dfParams[3].setValue(range[1]);
	}
}



//------------------------------------------------------------------------------
class MFSemiTrapezoidalInfParams extends MFParams
{
	private JInput input;
	public MFSemiTrapezoidalInfParams( JPanel panel, JInput input ) throws Throwable
	{
		this.input = input;
		jLParams = new JLabel[3];
		dfParams = new DoubleFieldInput[3];
		for( int i=0 ; i<3 ; i++ )
		{
			jLParams[i] = new JLabel();
			dfParams[i] =  new DoubleFieldInput(input);
		}
		this.RAZParams();
		this.dfParams[0].setValue(input.GetRange()[0]);
		panel.removeAll();
		panel.add(jLParams[0],        new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
		panel.add(dfParams[0],                new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 60, 0));
		panel.add(jLParams[1],      new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
		panel.add(dfParams[1],        new GridBagConstraints(3, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 60, 0));
		panel.add(jLParams[2],       new GridBagConstraints(3, 2, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
		panel.add(dfParams[2],           new GridBagConstraints(4, 2, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 60, 0));
		panel.repaint();
		this.Translate();
	}
	//------------------------------------------------------------------------------
	public void Translate()
	{
		super.Translate();
		jLParams[0].setText("S1:");
		jLParams[1].setText("S2:");
		jLParams[2].setText("S3:");
	}
	//------------------------------------------------------------------------------
	public int Number() { return JMF.SEMITRAPEZOIDALINF; }
	//------------------------------------------------------------------------------
	public void RAZParams() throws Throwable
	{
		double [] range = input.GetRange();
		dfParams[0].setValue(range[0]);
		dfParams[1].setValue(range[0] + ((range[1] - range[0])/2));
		dfParams[2].setValue(range[1]);
	}
}



//------------------------------------------------------------------------------
class MFSemiTrapezoidalSupParams extends MFParams
{
	private JInput input;
	public MFSemiTrapezoidalSupParams( JPanel panel, JInput input ) throws Throwable
	{
		this.input = input;
		jLParams = new JLabel[3];
		dfParams = new DoubleFieldInput[3];
		for( int i=0 ; i<3 ; i++ )
		{
			jLParams[i] = new JLabel();
			dfParams[i] =  new DoubleFieldInput(input);
		}
		this.RAZParams();
		this.dfParams[2].setValue(input.GetRange()[1]);
		panel.removeAll();
		panel.add(jLParams[0],         new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
		panel.add(dfParams[0],                 new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 60, 0));
		panel.add(jLParams[1],           new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
		panel.add(dfParams[1],           new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 60, 0));
		panel.add(jLParams[2],          new GridBagConstraints(3, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
		panel.add(dfParams[2],             new GridBagConstraints(4, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 60, 0));
		panel.repaint();
		this.Translate();
	}
	//------------------------------------------------------------------------------
	public void Translate()
	{
		super.Translate();
		jLParams[0].setText("S1:");
		jLParams[1].setText("S2:");
		jLParams[2].setText("S3:");
	}
	//------------------------------------------------------------------------------
	public int Number() { return JMF.SEMITRAPEZOIDALSUP; }
	//------------------------------------------------------------------------------
	public void RAZParams() throws Throwable
	{
		double [] range = input.GetRange();
		dfParams[0].setValue(range[0]);
		dfParams[1].setValue(range[0] + ((range[1] - range[0])/2));
		dfParams[2].setValue(range[1]);
	}
}



//------------------------------------------------------------------------------
class MFDiscreteParams extends MFParams
{
	private JLabel jLNumberOfValues = new JLabel();
	private IntegerField ifNumberOfValues = new IntegerField();
	private JInput Input;
	private JPanel Panel;
	private JButton jbModifyNumberOfValues = new JButton();
	//------------------------------------------------------------------------------
	public MFDiscreteParams( JPanel panel, JInput input ) throws Throwable
	{
		this.Input = input;
		this.Panel = panel;
		this.InitParams(1); // une valeur par defaut
		// listener du bouton de modification du nombre de valeurs
		this.jbModifyNumberOfValues.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e)
		{
			if( ifNumberOfValues.getValue() <= 0 )
				MessageFIS.Error(null, LocaleFIS.GetString("Error"), LocaleFIS.GetString("NumberOfValues") + " " + LocaleFIS.GetString("MustBePositive"));
			else
			{
				try
				{
					DoubleField [] old_dfParams = MFDiscreteParams.this.dfParams;
					MFDiscreteParams.this.InitParams(ifNumberOfValues.getValue());
					for( int i=0 ; i<Math.min(old_dfParams.length, MFDiscreteParams.this.dfParams.length) ; i++ )
						MFDiscreteParams.this.dfParams[i].setValue(old_dfParams[i].getValue());
				}
				catch( Throwable t ) { MessageFIS.Error(null, t);}
			}
		}} );
	}
	//------------------------------------------------------------------------------
	public void Translate()
	{
		super.Translate();
		this.jLNumberOfValues.setText(LocaleFIS.GetString("NumberOfValues") + ":");
		this.jbModifyNumberOfValues.setText(LocaleFIS.GetString("Modify"));
		for( int i=0 ; i<this.jLParams.length ; i++ )
			jLParams[i].setText(LocaleFIS.GetString("Value") + " " + Integer.toString(i+1) + ":");
	}
	//------------------------------------------------------------------------------
	public void SetParams( double [] params ) throws Throwable
	{
		this.InitParams(params.length);
		super.SetParams(params);
	}
	//------------------------------------------------------------------------------
	private void InitParams( int number_of_params ) throws Throwable
	{
		// une valeur par defaut
		this.ifNumberOfValues.setValue(number_of_params);
		jLParams = new JLabel[number_of_params];
		dfParams = new DoubleFieldInput[number_of_params];
		for( int i=0 ; i<number_of_params ; i++ )
		{
			jLParams[i] = new JLabel();
			dfParams[i] = new DoubleFieldInput(this.Input);
		}
		this.RAZParams();
		Panel.removeAll();
		Panel.add(jLNumberOfValues,         new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
		Panel.add(ifNumberOfValues,         new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 60, 0));
		Panel.add(jbModifyNumberOfValues,   new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 60, 0));
		for( int i=0 ; i<number_of_params ; i++ )
		{
			Panel.add(jLParams[i],         new GridBagConstraints(0, i+1, 1, 1, 0.0, 0.0
					,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
			Panel.add(dfParams[i],                 new GridBagConstraints(1, i+1, 1, 1, 0.0, 0.0
					,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 5, 0), 60, 0));
		}
		Panel.repaint();
		this.Translate();
	}
	//------------------------------------------------------------------------------
	public int Number() { return JMF.DISCRETE; }
	//------------------------------------------------------------------------------
	public void RAZParams() throws Throwable
	{
		double [] range = this.Input.GetRange();
		double ecart = (range[1] - range[0]) / (this.dfParams.length + 1);
		for( int i=0 ; i<this.dfParams.length ; i++ )
			dfParams[i].setValue(range[0] + ((i+1)*ecart));
	}
}



//------------------------------------------------------------------------------
class MFUniversalParams extends MFParams
{
	public MFUniversalParams( JPanel panel, JInput input ) throws Throwable
	{
		jLParams = new JLabel[2];
		dfParams = new DoubleFieldInput[2];
		for( int i=0 ; i<2 ; i++ )
		{
			jLParams[i] = new JLabel();
			dfParams[i] =  new DoubleFieldInput(input);
			this.dfParams[i].setValue(0);
		}
		this.dfParams[0].setEnabled(false);
		this.dfParams[0].setValue(input.GetRange()[0]);
		this.dfParams[1].setEnabled(false);
		this.dfParams[1].setValue(input.GetRange()[1]);
		panel.removeAll();
		panel.add(jLParams[0],        new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
		panel.add(dfParams[0],                new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 60, 0));
		panel.add(jLParams[1],      new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
		panel.add(dfParams[1],        new GridBagConstraints(3, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 60, 0));
		this.Translate();
	}
	//------------------------------------------------------------------------------
	public void Translate()
	{
		super.Translate();
		jLParams[0].setText("S1:");
		jLParams[1].setText("S2:");
	}
	//------------------------------------------------------------------------------
	public int Number() { return JMF.UNIVERSAL; }
}

//------------------------------------------------------------------------------
class MFDoorParams extends MFUniversalParams
{
	public MFDoorParams( JPanel panel, JInput input ) throws Throwable
	{
		super(panel, input);
		this.dfParams[0].setEnabled(true);
		this.dfParams[1].setEnabled(true);
		double []range = input.GetRange();
		this.dfParams[0].setValue(range[0]+((range[1]-range[0])/3));
		this.dfParams[1].setValue(range[0]+2*((range[1]-range[0])/3));
	}
	//------------------------------------------------------------------------------
	public int Number() { return JMF.DOOR; }
}


//------------------------------------------------------------------------------
class MFGaussianParams extends MFParams
{
	private JInput Input;
	public MFGaussianParams( JPanel panel, JInput input ) throws Throwable
	{
		this.Input = input;
		jLParams = new JLabel[2];
		dfParams = new DoubleField[2];
		jLParams[0] = new JLabel();
		dfParams[0] =  new DoubleFieldInput(input);
		jLParams[1] = new JLabel();
		dfParams[1] =  new DoubleField();
		this.RAZParams();
		panel.removeAll();
		panel.add(jLParams[0],        new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
		panel.add(dfParams[0],                new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 60, 0));
		panel.add(jLParams[1],      new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
		panel.add(dfParams[1],        new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 60, 0));
		panel.repaint();
		this.Translate();
	}
	//------------------------------------------------------------------------------
	public void Translate()
	{
		super.Translate();
		jLParams[0].setText(LocaleFIS.GetString("Mean") + ":");
		jLParams[1].setText(LocaleFIS.GetString("StandardDeviation") + ":");
	}
	//------------------------------------------------------------------------------
	public int Number() { return JMF.GAUSSIAN; }
	//------------------------------------------------------------------------------
	public void RAZParams() throws Throwable
	{
		double [] range = this.Input.GetRange();
		this.dfParams[0].setValue(range[0] + ((range[1] - range[0])/2));
		this.dfParams[1].setValue((range[1] - range[0])/10);
	}
}



/**
 * Titre :
 * Description :
 * Copyright :    Copyright (c) 2001
 * Societe :      CEMAGREF
 * @author
 * @version 1.0
 */


public class JInputFrame extends JFISChildFrame
{
	protected JFIS fis;
	protected JSemaphore Semaphore;
	protected int Number;
	protected JInput Temp;
	private FISListener FISListener;
	private MFParams MFParams = null;
	protected JPanel PanelInput = new JPanel(new GridBagLayout());

	// couleurs d'affichage des SEFs
	public static Color[] colors;

	// menus
	private JMenuBar jMenuBarInput = new JMenuBar();
	// menu MFs
	private JMenu jMenuMFs = new JMenu();
	protected JMenuItem jMenuNewMF = new JMenuItem();
	protected JMenuItem jMenuRemoveMF = new JMenuItem();
	protected JMenuItem jMenuRegularGrid = new JMenuItem();
	protected JMenuItem jMenuIrregularGrid = new JMenuItem();
	// menu range
	private JMenu jMenuRange = new JMenu();
	protected JMenuItem jMenuInputRange = new JMenuItem();
	private JMenuItem jMenuDisplayRange = new JMenuItem();
	// menu print
	private JMenuItem jMenuPrint = new JMenuItem();
	private JMenuItem jMenuExport = new JMenuItem();
	// popup MFs
	protected JPopupMenu jPopupMFs = new JPopupMenu();
	private JMenuItem jPopupNewMF = new JMenuItem();
	protected JMenuItem jPopupRemoveMF = new JMenuItem();
	// popup range
	private JPopupMenu jPopupRange = new JPopupMenu();
	private JMenuItem jPopupDisplayRange = new JMenuItem();

	// fenetre JInputFrame
	// nom
	private JLabel jLabelName = new JLabel();
	protected JTextField jTFName = new JTextField();
	// active
	protected JCheckBox jCBActive = new JCheckBox();
	// panel range
	private JPanel jPanelRange = new JPanel(new GridBagLayout());
	private TitledBorder titledBorderRange;
	private JLabel jLabelLower = new JLabel();
	protected DoubleField dfLower = new DoubleField();
	private JLabel jLabelUpper = new JLabel();
	protected DoubleField dfUpper = new DoubleField();
	// panel MFs
	private TitledBorder titledBorderMFs;
	private JPanel jPanelMFs = new JPanel(new GridBagLayout());
	// panel liste de SEF
	private TitledBorder titledBorderListMF;
	private DefaultListModel jListMFModel = new DefaultListModel();
	protected JList jListMF = new JList(jListMFModel);
	private JScrollPane jScrollPaneListMF = new JScrollPane(jListMF)
	{
		public Dimension getMinimumSize() { return  new Dimension(50,100); }
	};
	private JMFsPanel jGraphMFs;
	// panel parametres SEF
	private TitledBorder titledBorderMFParameters;
	private JPanel jPanelMFParameters = new JPanel(new GridBagLayout())
	{
		public void setEnabled( boolean enabled )
		{
			super.setEnabled(enabled);
			JInputFrame.this.jTFMFName.setEnabled(enabled);
			JInputFrame.this.jCBMFType.setEnabled(enabled);
			JInputFrame.this.jBApplyParameter.setEnabled(enabled);
			JInputFrame.this.jBCancelParameter.setEnabled(enabled);
		}
	};
	private JScrollPane jScrollPaneMFParameters = new JScrollPane(this.jPanelMFParameters) ;
	private JLabel jLName = new JLabel();
	private JTextField jTFMFName = new JTextField();
	private JPanel jPType = new JPanel(new GridBagLayout());
	private JLabel jLType = new JLabel();
	private DefaultComboBoxModel jCBMFTypeModel = new DefaultComboBoxModel()
	{
		// c'est cette methode qui assure la traduction des types de SEF
		public Object getElementAt(int index)
		{
			try { return LocaleFIS.GetString(JMF.MFType()[index]); }
			catch( Throwable t ) { return null; }
		}
	};
	protected JComboBox jCBMFType = new JComboBox(jCBMFTypeModel);
	private JPanel jPValidParameter = new JPanel();
	private JButton jBApplyParameter = new JButton();
	private JButton jBCancelParameter = new JButton();
	private JPanel jPanelMFParams = new JPanel(new GridBagLayout());

	private boolean NewMF = false;    // flag utilise lors de la creation d'un nouveau SEF ou de la saisie d'un SEF
	private Point PopupMenuPosition;  // position du click menu popup

	//------------------------------------------------------------------------------
	protected JInputFrame( JFISFrame parent ) { super( parent ); }
	//------------------------------------------------------------------------------
	public JInputFrame( JFISFrame parent, JFIS fis, int input_number ) throws Throwable
	{
		super(parent);
		this.fis = fis;
		this.Number = input_number;
		this.Temp = this.fis.GetInput(this.Number);
		this.Semaphore = this.fis.GetInputSemaphore(input_number);
	}
	//------------------------------------------------------------------------------
	public int print( java.awt.Graphics g, PageFormat pageFormat, int pageIndex ) throws PrinterException
	{
		java.awt.Graphics2D  g2 = ( java.awt.Graphics2D )g;
		double scalew=1;
		double scaleh=1;
		double pageHeight = pageFormat.getImageableHeight();
		// See how much we need to scale the image to fit in the page's width
		double pageWidth = pageFormat.getImageableWidth();
		if(  getWidth() >= pageWidth ) { scalew =  pageWidth / getWidth(); }
		if(  getHeight() >= pageHeight) { scaleh =  pageWidth / getHeight(); }
		g2.translate( pageFormat.getImageableX(), pageFormat.getImageableY() );
		g2.scale( scalew, scaleh );
		this.PanelInput.print( g2 );
		return Printable.PAGE_EXISTS;
	}


	//------------------------------------------------------------------------------
	protected void InitComponents()
	{
		// fennetre JInputFrame
		this.getContentPane().add(this.PanelInput);
		// panel header
		JPanel jPanelHeader = new JPanel(new GridBagLayout());
		this.PanelInput.add(jPanelHeader, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets( 0, 0, 0, 0), 0, 0));
		// label nom
		jPanelHeader.add(jLabelName, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 10, 5, 5), 0, 0));
		// jTextField nom
		jPanelHeader.add(jTFName, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 5, 0), 97, 0));
		// jCheckBox active
		jPanelHeader.add(jCBActive, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 10, 5, 10), 0, 0));
		// panel range
		titledBorderRange = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"");
		jPanelRange.setBorder(titledBorderRange);
		jPanelRange.add(jLabelLower, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
		jPanelRange.add(dfLower, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 50, 0));
		jPanelRange.add(jLabelUpper, new GridBagConstraints(2, 0, 1, 1, 1.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 10, 5, 5), 0, 0));
		jPanelRange.add(dfUpper, new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 50, 0));
		this.PanelInput.add(jPanelRange,            new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 5, 5, 5), 0, 0));
	}
	//------------------------------------------------------------------------------
	protected void jbInit() throws Throwable
	{
		// menus
		this.setJMenuBar(this.jMenuBarInput);
		// menu MFs
		jMenuBarInput.add(jMenuMFs);
		jMenuMFs.add(jMenuNewMF);
		jMenuMFs.add(jMenuRemoveMF);
		jMenuMFs.addSeparator();
		jMenuMFs.add(jMenuRegularGrid);
		jMenuMFs.add(jMenuIrregularGrid);
		jMenuMFs.addSeparator();
		// menu print
		jMenuMFs.add(jMenuPrint);
		jMenuMFs.add(jMenuExport);
		// menu range
		jMenuBarInput.add(jMenuRange);
		jMenuRange.add(jMenuInputRange);
		jMenuRange.add(jMenuDisplayRange);
		// popup MFs
		jPopupMFs.add(jPopupNewMF);
		jPopupMFs.add(jPopupRemoveMF);
		// popup range
		jPopupRange.add(jPopupDisplayRange);

		this.InitComponents();

		this.dfLower.setEnabled(false);
		this.dfUpper.setEnabled(false);

		// panel MFs
		titledBorderMFs = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"");
		jPanelMFs.setBorder(titledBorderMFs);
		this.AddPanelMFs();
		// panel graphique SEF
		jGraphMFs = new JMFsPanel(this.Temp);
		jGraphMFs.setSize(new Dimension(300,150));

		jPanelMFs.add(jGraphMFs, new GridBagConstraints(0, 0, 2, 1, 1.0, 1.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 5), 0, 0));
		// on utilise les couleurs de la classe Plot
		this.colors = this.jGraphMFs.getColors();
		// panel Liste SEF
		titledBorderListMF = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"");
		jScrollPaneListMF.setBorder(titledBorderListMF);
		jPanelMFs.add(jScrollPaneListMF, new GridBagConstraints(0, 1, 1, 1, 0.0, 1.0
				,GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 20, 0));
		jListMF.setBackground(Color.white);
		jListMF.setCellRenderer(new DefaultListCellRenderer()
		{
			public Component getListCellRendererComponent(JList clist, Object value, int index, boolean isSelected, boolean cellHasFocus)
			{
				super.getListCellRendererComponent( clist, value, index, isSelected, cellHasFocus );
				try
				{
					super.setText(JInputFrame.this.Temp.GetMF(index).GetName());
					// affichage en couleur
					// index peut etre superieur a la taille du tableau couleur
					super.setForeground(JInputFrame.colors[index - ((index/JInputFrame.colors.length)*JInputFrame.colors.length)]);
					// affichage de la selection
					if( isSelected == true )
					{
						super.setBackground(Color.lightGray);
						super.setBorder(BorderFactory.createLineBorder(Color.black, 1));
					}
				}
				catch( Throwable e ) {}   // on n'affiche pas l'erreur
				return this;
			}
		});
		jListMF.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		// panel parametres SEF
		this.jPanelMFParameters.setEnabled(false);
		titledBorderMFParameters = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"");
		this.jScrollPaneMFParameters.setBorder(titledBorderMFParameters);

		jPValidParameter.add(jBApplyParameter,    new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 0, 5, 0), 0, 0));
		jPValidParameter.add(jBCancelParameter,    new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 0, 5, 0), 0, 0));
		jPanelMFParameters.add(jPValidParameter,      new GridBagConstraints(0, 3, 6, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));

		jPanelMFParameters.add(jPType,   new GridBagConstraints(0, 0, 6, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
		jPType.add(jLName,    new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 5, 5, 5), 0, 0));
		jPType.add(jTFMFName,   new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 75, 0));
		jPType.add(jLType,               new GridBagConstraints(2, 0, 1, 1, 1.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 5, 5), 0, 0));
		for( int i=0 ; i<JMF.MFType().length ; i++ )
			this.jCBMFTypeModel.addElement(new String()); // on met des chaine vide dans le modele, c'est la methode getElementAt du modele qui assure le traduction des types de SEF
		this.jCBMFType.setRenderer(new ComboRenderer() { protected boolean isEnabled( int index ) { 
					if(index == JMF.DISCRETE) return JInputFrame.this.isDiscretMFEnabled(index);
					else return JInputFrame.this.isDPossMFEnabled(index); } });
			
		jPType.add(jCBMFType,        new GridBagConstraints(3, 0, 1, 1, 1.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));

		jPanelMFParameters.add(this.jPanelMFParams,      new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));

	}
	//------------------------------------------------------------------------------
	protected boolean isDiscretMFEnabled( int index ) { return true; }
	//------------------------------------------------------------------------------
	protected boolean isDPossMFEnabled( int index )
	{
		try { return ( (index == JMF.POSSIBILITY_DISTRIBUTION) ) ? false : true; }
		catch( Throwable t ) { return false; }
	}	
	//------------------------------------------------------------------------------
	protected void AddPanelMFs()
	{
		this.PanelInput.add(this.jPanelMFs, new GridBagConstraints(0, 2, 1, 1, 1.0, 1.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 5, 0, 5), 0, 0));
		jPanelMFs.add(this.jScrollPaneMFParameters,    new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
	}
	//------------------------------------------------------------------------------
	protected void RemovePanelMFs()
	{
		this.PanelInput.remove(this.jPanelMFs);
	}
	//------------------------------------------------------------------------------
	private void AddPanelMFParameter()
	{
		this.PanelInput.add(this.jScrollPaneMFParameters, new GridBagConstraints(0, 2, 1, 1, 1.0, 1.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 5, 0, 5), 0, 0));
	}
	//------------------------------------------------------------------------------
	private void RemovePanelMFParameter()
	{
		this.PanelInput.remove(this.jScrollPaneMFParameters);
	}
	//------------------------------------------------------------------------------
	public void Show( JInputFrame previous ) throws Throwable
	{
		jbInit();
		// initialisation des objets
		InitObjectsWithTemp();
		// traduction
		Translate();
		// evenements
		Events();
		// ouverture du premier SEF
		if( this.Temp.GetNbMF() != 0 )
			this.jListMF.setSelectedIndex(0);
		else
			this.RemovePanelMFs();
		// affichage
		this.pack();
		if( previous != null )
			this.setLocation(previous.getLocation().x + this.getInsets().top, previous.getLocation().y + this.getInsets().top);
		else
			this.setLocation(this.ChildPosition(this.getSize()));
		if( this.Semaphore != null )
			this.Semaphore.Acquire();
		this.setVisible(true);
		this.repaint();
	}
	//------------------------------------------------------------------------------
	protected void TranslateComponents()
	{
		this.setTitle(LocaleFIS.GetString("Input"));
		jLabelName.setText(LocaleFIS.GetString("Name")+":");
		jCBActive.setText(LocaleFIS.GetString("Active"));
		titledBorderRange.setTitle(LocaleFIS.GetString("Range"));
		jLabelLower.setText(LocaleFIS.GetString("Lower")+":");
		jLabelUpper.setText(LocaleFIS.GetString("Upper")+":");
	}
	//------------------------------------------------------------------------------
	public void Translate() throws Throwable
	{
		// menu MFs
		jMenuMFs.setText(LocaleFIS.GetString("MFs"));
		jMenuNewMF.setText(LocaleFIS.GetString("NewMF"));
		jMenuRemoveMF.setText(LocaleFIS.GetString("RemoveMF"));
		jMenuRegularGrid.setText(LocaleFIS.GetString("RegularGrid"));
		jMenuIrregularGrid.setText(LocaleFIS.GetString("IrregularGrid"));
		// menu range
		jMenuRange.setText(LocaleFIS.GetString("Range"));
		jMenuInputRange.setText(LocaleFIS.GetString("Input"));
		jMenuDisplayRange.setText(LocaleFIS.GetString("Display"));
		// menu print
		jMenuPrint.setText(LocaleFIS.GetString("Print"));
		jMenuExport.setText(LocaleFIS.GetString("Export"));
		// popup MFs
		jPopupNewMF.setText(LocaleFIS.GetString("NewMF"));
		jPopupRemoveMF.setText(LocaleFIS.GetString("Remove"));
		// popup range
		jPopupDisplayRange.setText(LocaleFIS.GetString("DisplayRange"));

		// traduction de la fenetre Input
		this.TranslateComponents();
		titledBorderMFs.setTitle(LocaleFIS.GetString("MFs"));
		titledBorderListMF.setTitle(LocaleFIS.GetString("MF"));
		titledBorderMFParameters.setTitle(LocaleFIS.GetString("MFParameters"));
		jBApplyParameter.setText(LocaleFIS.GetString("Apply"));
		jBCancelParameter.setText(LocaleFIS.GetString("Cancel"));
		jLType.setText(LocaleFIS.GetString("Type")+":");
		jLName.setText(LocaleFIS.GetString("Name")+":");

		// traduction de la disposition
		if( this.MFParams != null )
		{
			this.MFParams.Translate();
			double [] params = this.MFParams.GetParams();
			this.jCBMFType.setSelectedIndex(this.MFParams.Number());
			this.MFParams.SetParams(params);
		}

		// Indicateurs de domaine
		this.dfLower.setLocale(LocaleFIS.Locale());
		this.dfLower.setValue(this.Temp.GetRange()[0]);
		this.dfUpper.setLocale(LocaleFIS.Locale());
		this.dfUpper.setValue(this.Temp.GetRange()[1]);
		this.repaint();
	}

	//------------------------------------------------------------------------------
	protected void InitObjectsWithTemp() throws Throwable
	{
		this.jTFName.setText(this.Temp.GetName());
		this.jCBActive.setSelected(this.Temp.GetActive());
		this.dfLower.setValue(this.Temp.GetRange()[0]);
		this.dfUpper.setValue(this.Temp.GetRange()[1]);
		this.jListMFModel.removeAllElements();
		for( int i=0 ; i<this.Temp.GetNbMF() ; i++ )
			this.jListMFModel.addElement(new String());    // on met des String vide dans la liste, c'est la classe MFListCellRenderer qui affiche le nom des SEFs
		this.jGraphMFs.SetInput(this.Temp);
	}

	//------------------------------------------------------------------------------
	protected void Events()
	{
		// evenements menu
		jMenuMFs.addMenuListener(new MenuListener()
		{
			public void menuSelected(MenuEvent e) { jMenuMFs_menuSelected(e); }
			public void menuDeselected(MenuEvent e) {}
			public void menuCanceled(MenuEvent e) {}
		});
		jMenuNewMF.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuNewMF_actionPerformed(e); } });
		jMenuRemoveMF.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuRemoveMF_actionPerformed(e); } });
		jMenuRegularGrid.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuRegularGrid_actionPerformed(e); } });
		jMenuIrregularGrid.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuIrregularGrid_actionPerformed(e); } });
		jMenuRange.addMenuListener(new MenuListener()
		{
			public void menuSelected(MenuEvent e) { jMenuRange_menuSelected(e); }
			public void menuDeselected(MenuEvent e) {}
			public void menuCanceled(MenuEvent e) {}
		});
		jMenuInputRange.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuInputRange_actionPerformed(e); } });
		jMenuDisplayRange.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuDisplayRange_actionPerformed(e); } });
		jMenuPrint.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuPrint_actionPerformed(e); } });
		jMenuExport.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuExport_actionPerformed(e); } });
		// popups
		jPopupNewMF.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jPopupNewMF_actionPerformed(e); } });
		jPopupRemoveMF.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jPopupRemoveMF_actionPerformed(e); } });
		jPopupDisplayRange.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jPopupDisplayRange_actionPerformed(e); } });

		jTFName.addFocusListener(new FocusAdapter() { public void focusLost(FocusEvent e) { jTFName_focusLost(e); } });
		jTFName.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jTFName_actionPerformed(e); } });
		jCBActive.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jCBActive_actionPerformed(e); } });
		this.jCBMFType.addActionListener(new ComboListener(this.jCBMFType)
		{
			protected boolean isEnabled( int index ) { return JInputFrame.this.isDiscretMFEnabled(index); }
			protected void actionPerformed()
			{
				try
				{
					switch( JInputFrame.this.jCBMFType.getSelectedIndex() )
					{
					case JMF.TRIANGULAR :  JInputFrame.this.MFParams = new MFTriangularParams(JInputFrame.this.jPanelMFParams, JInputFrame.this.Temp); break;
					case JMF.TRAPEZOIDAL : JInputFrame.this.MFParams = new MFTrapezoidalParams(JInputFrame.this.jPanelMFParams, JInputFrame.this.Temp); break;
					case JMF.SEMITRAPEZOIDALINF :  JInputFrame.this.MFParams = new MFSemiTrapezoidalInfParams(JInputFrame.this.jPanelMFParams, JInputFrame.this.Temp); break;
					case JMF.SEMITRAPEZOIDALSUP :  JInputFrame.this.MFParams = new MFSemiTrapezoidalSupParams(JInputFrame.this.jPanelMFParams, JInputFrame.this.Temp); break;
					case JMF.DISCRETE :  JInputFrame.this.MFParams = new MFDiscreteParams(JInputFrame.this.jPanelMFParams, JInputFrame.this.Temp); break;
					case JMF.UNIVERSAL :  JInputFrame.this.MFParams = new MFUniversalParams(JInputFrame.this.jPanelMFParams, JInputFrame.this.Temp); break;
					case JMF.GAUSSIAN :  JInputFrame.this.MFParams = new MFGaussianParams(JInputFrame.this.jPanelMFParams, JInputFrame.this.Temp); break;
					case JMF.DOOR :  JInputFrame.this.MFParams = new MFDoorParams(JInputFrame.this.jPanelMFParams, JInputFrame.this.Temp); break;
					case JMF.SINUS :  JInputFrame.this.MFParams = new MFSinusParams(JInputFrame.this.jPanelMFParams, JInputFrame.this.Temp); break;
					case JMF.SINUSINF :  JInputFrame.this.MFParams = new MFSinusParamsInf(JInputFrame.this.jPanelMFParams, JInputFrame.this.Temp); break;
					case JMF.SINUSSUP :  JInputFrame.this.MFParams = new MFSinusParamsSup(JInputFrame.this.jPanelMFParams, JInputFrame.this.Temp); break;
					}
					JInputFrame.this.jPanelMFParameters.setEnabled(true);
					JInputFrame.this.jPanelMFParameters.revalidate();
				}
				catch( Throwable t ) { MessageFIS.Error( null, t ); }
			}
		} );
		jBApplyParameter.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jBApplyParameter_actionPerformed(e); } });
		jBCancelParameter.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jBCancelParameter_actionPerformed(e); } });
		jListMF.addMouseListener(new MouseAdapter() { public void mousePressed(MouseEvent e) { jListMF_mousePressed(e); }});
		jListMF.addListSelectionListener(new ListSelectionListener() { public void valueChanged(ListSelectionEvent e) { jListMF_valueChanged(e); }});
		jGraphMFs.addMouseListener(new MouseAdapter() { public void mousePressed(MouseEvent e) { jGraphMFs_mousePressed(e); }});
		this.FISListener = new FISListener()
		{
			public void FISClosed() { JInputFrame.this.dispose(); }
			public void InputRemoved( int input_number ) {}
			public void OutputRemoved( int output_number ) {}
			public void InputAdded( int input_number ) {}
			public void OutputAdded( int output_number ) {}
			public void InputReplaced( int input_number ) {}
			public void OutputReplaced( int output_number ) {}
			public void InputActiveChanged( int input_number ) {}
			public void OutputActiveChanged( int output_number ) {}
			public void InputNameChanged( int input_number ) {}
			public void OutputNameChanged( int output_number ) {}
			public void InputRangeChanged( int input_number ) {}
			public void OutputRangeChanged( int output_number ) {}
			public void MFRemovedInInput( int input_number ) {}
			public void MFRemovedInOutput( int output_number ) {}
			public void MFAddedInInput( int input_number ) {}
			public void MFAddedInOutput( int output_number ) {}
			public void MFReplacedInInput( int input_number ) {}
			public void MFReplacedInOutput( int output_number ) {}
			public void OutputDefuzChanged( int output_number ) {}
			public void OutputImpliChanged( int output_number ) {}    
			public void OutputDisjChanged( int output_number ) {}
			public void OutputClassifChanged( int output_number ) {}
			public void OutputDefaultChanged( int output_number ) {}
			public void OutputAlarmThresChanged( int output_number ) {}
			public void RulesModified() {}
			public void ConjunctionChanged() {}
		};
		this.fis.AddFISListener(this.FISListener);
	}
	//------------------------------------------------------------------------------
	public void dispose()
	{
		super.dispose();
		if( this.Semaphore != null )
			this.Semaphore.Release();
		if( this.FISListener != null )
			this.fis.RemoveFISListener(this.FISListener);
	}

	//------------------------------------------------------------------------------
	private void ShowNewMFParameters() throws Throwable
	{
		this.jListMF.clearSelection();  // supprime la selection en cours sur la liste jListMF
		// affecte un nom par defaut "SEF n" n = nombre de SEF dans l'entree + 1
		this.jTFMFName.setText(LocaleFIS.GetString("MF") + String.valueOf(this.Temp.GetNbMF()+1));
		this.jCBMFType.setSelectedIndex(JMF.TRIANGULAR);   // triangle par defaut
		this.MFParams.RAZParams();   // remet a zero les parametres
		this.NewMF = true;
		if( this.Temp.GetNbMF() == 0 )   // on redimensionnne la fenetre seulement si il n'y a aucun SEF
		{
			this.AddPanelMFParameter();
			this.pack();
		}
		this.validate();
	}
	//------------------------------------------------------------------------------
	void jListMF_mousePressed(MouseEvent e)
	{
		// bouton droit
		if (SwingUtilities.isRightMouseButton(e))
		{
			if( jListMF.isSelectionEmpty() == true )    // masque les item du popup non utile
				jPopupRemoveMF.setEnabled(false);
			else
				jPopupRemoveMF.setEnabled(true);
			SwingUtilities.updateComponentTreeUI(jPopupMFs);    // affecte le look actuel au composant jPopupMenuInputs
			jPopupMFs.show(jListMF, e.getX(), e.getY());       // affichage du popup
		}
	}

	//------------------------------------------------------------------------------
	private void NewMF()
	{
		try { this.ShowNewMFParameters(); }
		catch( Throwable t ) { MessageFIS.Error( null, t ); }
	}
	//------------------------------------------------------------------------------
	void jPopupNewMF_actionPerformed(ActionEvent e) { this.NewMF(); }
	//------------------------------------------------------------------------------
	void jBCancelParameter_actionPerformed(ActionEvent e)
	{
		try
		{
			// saisie nouveau SEF
			if( this.NewMF == true )
			{
				this.NewMF = false;
				if( this.Temp.GetNbMF() != 0 )
					this.jListMF.setSelectedIndex(0); // on selectionne le premier SEF
			}

			// si aucun SEF
			if( this.Temp.GetNbMF() == 0 )
			{
				this.RemovePanelMFParameter();
				this.pack();
			}
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	void jBApplyParameter_actionPerformed(ActionEvent e)
	{
		// test si le nom a ete saisi
		if( this.jTFMFName.getText().equals("") )
		{
			MessageFIS.Error( this, LocaleFIS.GetString("Error"), LocaleFIS.GetString("EnterAName") );
			return;
		}
		// creation d'un nouveau SEF
		JMF newMF = null;
		try
		{
			switch( this.jCBMFType.getSelectedIndex() )
			{
			case JMF.TRIANGULAR: newMF = new JMFTriangular(this.jTFMFName.getText(), this.MFParams.GetParams()); break;
			case JMF.TRAPEZOIDAL:  newMF = new JMFTrapezoidal(this.jTFMFName.getText(), this.MFParams.GetParams()); break;
			case JMF.SEMITRAPEZOIDALINF: newMF = new JMFSemiTrapezoidalInf(this.jTFMFName.getText(), this.MFParams.GetParams()); break;
			case JMF.SEMITRAPEZOIDALSUP: newMF = new JMFSemiTrapezoidalSup(this.jTFMFName.getText(), this.MFParams.GetParams()); break;
			case JMF.DISCRETE: newMF = new JMFDiscrete(this.jTFMFName.getText(), this.MFParams.GetParams()); break;
			case JMF.UNIVERSAL: newMF = new JMFUniversal(this.jTFMFName.getText(), this.MFParams.GetParams()); break;
			case JMF.GAUSSIAN: newMF = new JMFGaussian(this.jTFMFName.getText(), this.MFParams.GetParams()); break;
			case JMF.DOOR: newMF = new JMFDoor(this.jTFMFName.getText(), this.MFParams.GetParams()); break;
			case JMF.SINUS: newMF = new JMFSinus(this.jTFMFName.getText(), this.MFParams.GetParams()); break;
			case JMF.SINUSINF: newMF = new JMFSinusInf(this.jTFMFName.getText(), this.MFParams.GetParams()); break;
			case JMF.SINUSSUP: newMF = new JMFSinusSup(this.jTFMFName.getText(), this.MFParams.GetParams()); break;
			}
			// c'est un nouveau SEF
			if( this.NewMF == true )
			{				
				this.NewMF = false;
				boolean pack = false;
				if( this.Temp.GetNbMF() == 0 )
					pack = true;  // on redimensionne la fenetre seulement si aucun SEF dans Temp
				this.AddMF(newMF);
				this.InitObjectsWithTemp(); // reinitialisation de la fenetre
				this.jListMF.setSelectedIndex(this.Temp.GetNbMF()-1); // selectionne le nouveau SEF
				if( pack == true )
				{
					this.RemovePanelMFParameter();
					this.AddPanelMFs();
					this.pack();
				}
			}
			// c'est une modification d'un SEF existant
			else
			{
				this.ReplaceMF(this.jListMF.getSelectedIndex(), newMF);   // remplacement de l'ancien SEf par le nouveau				
			}
			this.jGraphMFs.clear(false);
			this.jGraphMFs.repaint();
			this.repaint();
		}
		catch( Throwable t )
		{
			if( newMF != null )
			{
				try { newMF.Delete(); }    // suppression du nouveau SEF
				catch( Throwable t1 ) { MessageFIS.Error( null, t1 ); }
			}
			MessageFIS.Error( null, t );
			return;
		}
	}
	//------------------------------------------------------------------------------
	protected void ReplaceMF( int mf_number, JMF new_mf ) throws Throwable
	{
		this.Temp.ReplaceMF(mf_number, new_mf);
		this.fis.MFReplacedInInput(this.Number);
	}
	//------------------------------------------------------------------------------
	protected void AddMF( JMF new_mf ) throws Throwable
	{
		this.Temp.AddMF(new_mf);
		this.fis.MFAddedInInput(this.Number);
	}
	//------------------------------------------------------------------------------
	void jListMF_valueChanged(ListSelectionEvent e)
	{
		try
		{
			// verifie que l'element selectionne dans la liste correspond a un SEF
			// cela est utile car pour deselectionner la liste (dans la methode HideMFParameters()) on fait appel a clearSelection qui rement getSelectionIndex a -1
			if( this.jListMF.getSelectedIndex() != -1 )
			{
				JMF mf = this.Temp.GetMF(this.jListMF.getSelectedIndex());
				this.jTFMFName.setText(mf.GetName());
				this.jCBMFType.setSelectedIndex(mf.GetType());
				this.jPanelMFParameters.setEnabled(true);
				this.MFParams.SetParams(mf.GetParams());
			}
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}

	//------------------------------------------------------------------------------
	private void RemoveMF()
	{
		try
		{
			this.RemoveMF(this.jListMF.getSelectedIndex());
			this.InitObjectsWithTemp();
			// ouverture du premier SEF
			if( this.Temp.GetNbMF() != 0 )
				this.jListMF.setSelectedIndex(0);
			else   // on redimensionnne la fenetre seulement si il n'y a aucun SEF
			{
				this.RemovePanelMFs();
				this.pack();
			}
		}
		catch( Throwable t ) { MessageFIS.Error( null, t ); }
	}
	//------------------------------------------------------------------------------
	protected void RemoveMF( int mf_number ) throws Throwable { this.fis.RemoveMFInInput(this.Number, mf_number); }
	//------------------------------------------------------------------------------
	void jPopupRemoveMF_actionPerformed(ActionEvent e) { this.RemoveMF(); }
	//------------------------------------------------------------------------------
	private void RegularGrid()
	{
		try
		{
			if( this.Temp.GetNbMF() != 0 )
				if( MessageFIS.Confirm(this, LocaleFIS.GetString("AllMFsWillBeErased") + "\n" + LocaleFIS.GetString("DoYouWantToContinue")) != JOptionPane.YES_OPTION )
					return;
			JRegularInputFrame jrif = new JRegularInputFrame();
			jrif.setLocation(JFISChildFrame.ChildPosition(this, jrif.getSize()));
			JInput new_input = jrif.Show(this.Temp);
			if( new_input != null )
			{
				boolean pack = false;
				if( this.Temp.GetNbMF() == 0 )
					pack = true;    // on redimensionne la fenetre seulement si aucun SEF dans Temp
				this.Replace(new_input);
				this.Temp = new_input;
				this.jGraphMFs.SetInput(new_input);
				this.InitObjectsWithTemp();
				this.jListMF.setSelectedIndex(0);   // selectionne le premier SEF
				if( pack == true )
				{
					this.AddPanelMFs();
					this.pack();
				}
			}
		}
		catch( Throwable t ) { MessageFIS.Error( null, t ); }
	}
	//------------------------------------------------------------------------------
	private void IrregularGrid()
	{
		try
		{
			//super.jButtonOK_actionPerformed(e);
			if( this.Temp.GetNbMF() != 0 )
				if( MessageFIS.Confirm(this, LocaleFIS.GetString("AllMFsWillBeErased") + "\n" + LocaleFIS.GetString("DoYouWantToContinue")) != JOptionPane.YES_OPTION )
					return;
			JIrregularInputFrame jiif = new JIrregularInputFrame(this.Temp);
			jiif.setLocation(JFISChildFrame.ChildPosition(this, jiif.getSize()));
			JInput new_input = jiif.Show(this.Temp);
			if( new_input != null )
			{
				boolean pack = false;
				if( this.Temp.GetNbMF() == 0 )
					pack = true;    // on redimensionne la fenetre seulement si aucun SEF dans Temp
				this.Replace(new_input);
				this.Temp = new_input;
				this.jGraphMFs.SetInput(new_input);
				this.InitObjectsWithTemp();
				this.jListMF.setSelectedIndex(0);   // selectionne le premier SEF
				if( pack == true )
				{
					this.AddPanelMFs();
					this.pack();
				}
			}
		}
		catch( Throwable t ) { MessageFIS.Error( null, t ); }
	}
	//------------------------------------------------------------------------------
	protected void Replace( JInput new_input ) throws Throwable { this.fis.ReplaceInput(this.Number, new_input); }
	//------------------------------------------------------------------------------
	void jPopupDisplayRange_actionPerformed(ActionEvent e)
	{
		JDisplayRangeFrame jdrf = new JDisplayRangeFrame(this.Temp, this.jGraphMFs.getXRange(), this.RangeType());
		jdrf.setLocation((int)this.PopupMenuPosition.getX(), (int)this.PopupMenuPosition.getY());
		this.jGraphMFs.SetXRange(jdrf.Show());
		this.jGraphMFs.setYRange(0, 1);
	}
	//------------------------------------------------------------------------------
	void jGraphMFs_mousePressed(MouseEvent e)
	{
		if( SwingUtilities.isRightMouseButton(e) )
		{
			SwingUtilities.updateComponentTreeUI(this.jPopupRange);
			this.jPopupRange.show(this.jGraphMFs, e.getX(), e.getY());
			this.PopupMenuPosition = this.jPopupRange.getLocationOnScreen();
		}
	}
	//------------------------------------------------------------------------------
	void jMenuDisplayRange_actionPerformed(ActionEvent e)
	{
		JDisplayRangeFrame jdrf = new JDisplayRangeFrame(this.Temp, this.jGraphMFs.getXRange(), this.RangeType());
		jdrf.setLocation(JFISChildFrame.ChildPosition(this, jdrf.getSize()));
		this.jGraphMFs.SetXRange(jdrf.Show());
		this.jGraphMFs.setYRange(0, 1);
	}
	//------------------------------------------------------------------------------
	protected String RangeType() { return new String("InputRange"); }
	//------------------------------------------------------------------------------
	void jMenuInputRange_actionPerformed(ActionEvent e)
	{
		JRangeFrame jrf = new JRangeFrame(this.Temp, this.RangeType());
		jrf.setLocation(JFISChildFrame.ChildPosition(this, jrf.getSize()));
		double [] new_range = jrf.Show();
		if( new_range != null )
		{
			try
			{
				this.Temp.SetRange(new_range);
				this.RangeChanged();
				this.dfLower.setValue(new_range[0]);
				this.dfUpper.setValue(new_range[1]);
				this.jGraphMFs.SetXRange(new_range);
				// affichage du SEF selectionne
				int index = this.jListMF.getSelectedIndex();
				this.jListMF.clearSelection();
				this.jListMF.setSelectedIndex(index);
			}
			catch( Throwable t ) { MessageFIS.Error( null, t ); }
		}
	}
	//------------------------------------------------------------------------------
	void jMenuPrint_actionPerformed(ActionEvent e)
	{
		PrinterJob job = PrinterJob.getPrinterJob();
		if( job.printDialog() == true )
		{
			job.setPrintable(new Printable()
			{
				public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) throws PrinterException
				{
					if (pageIndex >= 1) { return Printable.NO_SUCH_PAGE; }
					else return JInputFrame.this.print(graphics,pageFormat,pageIndex);
				}
			});
			try { job.print(); }
			catch (PrinterException ex) { ex.printStackTrace(); }
		}
	}
	//------------------------------------------------------------------------------
	void jMenuExport_actionPerformed(ActionEvent e)
	{
		try
		{
			ExportDialog export = new ExportDialog();
			export.showExportDialog( this.jGraphMFs, LocaleFIS.GetString("ExportViewAs"), this.jGraphMFs, "export" );
		}
		catch (Exception ex) { MessageFIS.Error(null, ex); }
	}
	//------------------------------------------------------------------------------
	protected void RangeChanged() { this.fis.InputRangeChanged(this.Number); }
	//------------------------------------------------------------------------------
	void jMenuNewMF_actionPerformed(ActionEvent e) { this.NewMF(); }
	//------------------------------------------------------------------------------
	void jMenuRemoveMF_actionPerformed(ActionEvent e) { this.RemoveMF(); }
	//------------------------------------------------------------------------------
	void jMenuRegularGrid_actionPerformed(ActionEvent e) { this.RegularGrid(); }
	//------------------------------------------------------------------------------
	void jMenuIrregularGrid_actionPerformed(ActionEvent e) { this.IrregularGrid(); }
	//------------------------------------------------------------------------------
	void jMenuMFs_menuSelected(MenuEvent e)
	{
		if( this.jListMF.isSelectionEmpty() == true )
			this.jMenuRemoveMF.setEnabled(false);
		else
			this.jMenuRemoveMF.setEnabled(true);
	}
	//------------------------------------------------------------------------------
	void jCBActive_actionPerformed(ActionEvent e)
	{
		try
		{
			this.Temp.SetActive(this.jCBActive.isSelected());
			this.ActiveChanged();
		}
		catch( Throwable except ) { MessageFIS.Error( null, except ); }
	}
	//------------------------------------------------------------------------------
	protected void ActiveChanged() { this.fis.InputActiveChanged(this.Number); }
	//------------------------------------------------------------------------------
	void jTFName_actionPerformed(ActionEvent e)
	{
		try
		{
			this.Temp.SetName(this.jTFName.getText());
			this.NameChanged();
		}
		catch( Throwable except ) { MessageFIS.Error( null, except ); }
	}
	//------------------------------------------------------------------------------
	void jTFName_focusLost(FocusEvent e)
	{
		try
		{
			this.Temp.SetName(this.jTFName.getText());
			this.NameChanged();
		}
		catch( Throwable except ) { MessageFIS.Error( null, except ); }
	}
	//------------------------------------------------------------------------------
	protected void NameChanged() { this.fis.InputNameChanged(this.Number);}
	//------------------------------------------------------------------------------
	void jMenuRange_menuSelected(MenuEvent e)
	{
		int nb_mfs = 0;
		try { nb_mfs = this.Temp.GetNbMF(); }
		catch( Throwable t ) { MessageFIS.Error(null, t); }
		if( nb_mfs == 0 )
			this.jMenuDisplayRange.setEnabled(false);
		else
			this.jMenuDisplayRange.setEnabled(true);
	}
}



