//***********************************************************************
//
//                              JLinksFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.*;
import java.io.*;
import javax.swing.*;
import java.awt.event.*;

//------------------------------------------------------------------------------
public class JLinksFrame extends JFISDialog
{
  private JFISFrame Parent;
  private JFIS fis;
  private JExtendedDataFile DataFile;
  private JLabel jLabelThres = new JLabel();
  private DoubleField jDFThres = new DoubleField();
  private JCheckBox jCBSort = new JCheckBox();
  private JPanel jPanelValidation = new JPanel(new GridBagLayout());
  private JButton jButtonApply = new JButton();
  private JButton jButtonCancel = new JButton();
  private JLabel jLabelLinksFile = new JLabel();
  private JTextField jTFLinksFile = new JTextField();
  private JButton jButtonSelectLinksFile = new JButton("...");
//------------------------------------------------------------------------------
  public JLinksFrame( JFISFrame parent, JFIS fis, JExtendedDataFile data )
  {
    this.fis = fis;
    this.Parent = parent;
    this.DataFile = data;
    try { jbInit(); }
    catch( Throwable en ) { MessageFIS.Error( null, en ); }
  }
//------------------------------------------------------------------------------
  private void jbInit() throws Exception
  {
    this.getContentPane().setLayout(new GridBagLayout());
    // fichiers liens
    this.jTFLinksFile.setText(this.DataFile.FileName());
    this.jButtonSelectLinksFile.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonSelectLinksFile_actionPerformed(e); } });
    this.getContentPane().add(this.jLabelLinksFile, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.getContentPane().add(this.jTFLinksFile, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 0, 0, 5), 20, 0));
    this.getContentPane().add(this.jButtonSelectLinksFile, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, -10));

    // seuil
    this.getContentPane().add(jLabelThres, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jDFThres.NumberFormat().setMaximumFractionDigits(10);
    this.jDFThres.setValue(1e-6);
    this.getContentPane().add(jDFThres, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));

    // tri
    this.getContentPane().add(jCBSort,  new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, 0));

    // panel validation
    jPanelValidation.add(jButtonApply,  new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    jPanelValidation.add(jButtonCancel,  new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    this.getContentPane().add(jPanelValidation,         new GridBagConstraints(0, 3, 4, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));

    // events
    jButtonApply.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonApply_actionPerformed(e); } });
    jButtonCancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonCancel_actionPerformed(e); } });

    // affichage
    this.Translate();
    this.pack();
    this.setModal(true);
    this.setLocation(JFISChildFrame.ChildPosition(this.Parent, this.getSize()));
    this.setVisible(true);
  }
//------------------------------------------------------------------------------
  void jButtonSelectLinksFile_actionPerformed(ActionEvent e)
  {
  JOpenFileChooser file_chooser = new JOpenFileChooser(MainFIS.config.FISFilePath());
  if( this.jTFLinksFile.getText().equals("") )
    file_chooser.setSelectedFile(new File(this.DataFile.FileName()));
  else
    file_chooser.setSelectedFile(new File(this.jTFLinksFile.getText()));
  file_chooser.setAcceptAllFileFilterUsed(true);
  if( file_chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION )
    this.jTFLinksFile.setText(file_chooser.getSelectedFile().getAbsolutePath());
  }
//------------------------------------------------------------------------------
  private void Translate()
  {
    this.setTitle(LocaleFIS.GetString("Links"));
    this.jLabelLinksFile.setText(LocaleFIS.GetString("PrefixLinksFiles") + ":");
    jLabelThres.setText(LocaleFIS.GetString("Threshold") + ":");
    jCBSort.setText(LocaleFIS.GetString("SortByWeight"));
    jButtonApply.setText(LocaleFIS.GetString("Apply"));
    jButtonCancel.setText(LocaleFIS.GetString("Cancel"));
  }
//------------------------------------------------------------------------------
  void jButtonCancel_actionPerformed(ActionEvent e) { this.dispose(); }
//------------------------------------------------------------------------------
  void jButtonApply_actionPerformed(ActionEvent e)
  {
  try
    {
    this.fis.Links(this.DataFile, this.jDFThres.getValue(), this.jCBSort.isSelected(), this.jTFLinksFile.getText(), null);
    // ouverture des fichiers liens dans une console
    new JFISConsole(this.Parent, this.jTFLinksFile.getText() + "." + jnifis.LinksRulesItemsExtension());
    JFISConsole jfc = new JFISConsole(this.Parent, this.jTFLinksFile.getText() + "." + jnifis.LinksItemsRulesExtension());
    Insets inset = jfc.getInsets();
    jfc.setLocation(jfc.getX()+jfc.getInsets().top, jfc.getY()+jfc.getInsets().top);
    jfc = new JFISConsole(this.Parent, this.jTFLinksFile.getText() + "." + jnifis.LinksRulesLinksExtension());
    jfc.setLocation(jfc.getX()+2*jfc.getInsets().top, jfc.getY()+2*jfc.getInsets().top);
    if( this.jCBSort.isSelected() == true )
      {
      jfc = new JFISConsole(this.Parent, this.jTFLinksFile.getText() + "." + jnifis.LinksRulesSortedExtension());
      jfc.setLocation(jfc.getX()+3*jfc.getInsets().top, jfc.getY()+3*jfc.getInsets().top);
      }
    this.dispose();
    }
  catch( Throwable except ) { MessageFIS.Error( null, except ); }
  }
}