//***********************************************************************
//
//                              JMFsPanel.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.Line2D;
import java.util.ArrayList;

import ptolemy.plot.PlotBox;

//------------------------------------------------------------------------------
//Classe d'affichage des SEFs

public class JMFsPanel extends FISPlot
{
protected JInput Input;
protected ArrayList cursors = new ArrayList();

//------------------------------------------------------------------------------
//public Dimension getPreferredSize() { return new Dimension(400,200); }
//public Dimension getMinimumSize() { return  new Dimension(100,100); }
//------------------------------------------------------------------------------
public JMFsPanel() {
   super();
   this.initYAxis();
   FISPlot.DisableZoom(this);
}
//------------------------------------------------------------------------------
public JMFsPanel( JInput input ) throws Throwable
{
   this.init(input);
   FISPlot.DisableZoom(this);
}
private void initYAxis() {
    this.setYRange(0, 1);
    this.addYTick("0", 0);
    this.addYTick("", 0.5);
    this.addYTick("1", 1);
}
//------------------------------------------------------------------------------
public void init( JInput input ) throws Throwable {
   this.initYAxis();
   this.SetInput(input);
   if( this.Input != null )
       this.SetXRange(this.Input.GetRange());
}
//------------------------------------------------------------------------------
public void SetXRange( double [] x_range ) { this.setXRange(x_range[0], x_range[1]); this.repaint(); }
//------------------------------------------------------------------------------
public void SetInput( JInput input ) throws Throwable
{
this.cursors.removeAll(this.cursors);
this.Input = input;
if( this.Input != null )
    this.SetXRange(this.Input.GetRange());
this.clear(false);
this.repaint();
}
//------------------------------------------------------------------------------
public void paintComponent(Graphics g)
{
try
 {
 // affichage de chaque SEF
 this.clear(false);
 if( this.Input != null )
     for( int i=0 ; i<this.Input.GetNbMF() ; i++ )
         this.Input.GetMF(i).NewMF().Draw(this, i);
 super.paintComponent(g);

 // affichage des curseurs
 for( int i=0 ; i<this.cursors.size() ; i++ ) {
     DrawCursor(g, this, ((Double)this.cursors.get(i)).doubleValue(), this._xMin, this._xMax);
 }
 // affichage des bornes du domaine de l'entree
 if( this.Input != null )
     FISPlot.DrawXInputRange(g, this, this.Input.GetRange(), this._xMin, this._xMax); 
 }
catch( Throwable t ) { MessageFIS.Error( null, t ); }
}
public void addCursor(double value) {
    this.cursors.add(new Double(value));
}
public void setCursor(int cursorIndex, double value) {
    this.cursors.set(cursorIndex, new Double(value));
    this.repaint();
}
// dessine le domaine de l'entree sur l'axe X
private static void DrawCursor( Graphics g, PlotBox pb, double cursor_value, double x_min, double x_max )
  {
  Graphics2D g2d = (Graphics2D)g;
  g2d.setColor(Color.black);
  Rectangle rect = pb.getPlotRectangle();
  if( rect.height <= 0 ) return;
  if( (cursor_value >= x_min) && (cursor_value <= x_max) )
    {
    int x = FISPlot.XtoPixel(pb, cursor_value, x_min, x_max);
    g2d.draw(new Line2D.Double(x, rect.y, x, rect.y+rect.height) );
    }
  }

}
