//***********************************************************************
//
//                              JOptFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.*;
import java.io.*;
import javax.swing.*;

import java.awt.event.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.util.*;

//------------------------------------------------------------------------------
public class JOptFrame extends JFISDialog
{
  private JFISFrame Parent;
  private JExtendedDataFile DataFile;
  private JFIS fis;
  private JPanel jPanelOutput;
  private ButtonGroup jBGOutput;
  private JLabel jLabelMaxOfIter;
  private IntegerField jIFMaxOfIter;
  double StdGaussianNoise = 0.005;
  int MaxOfConstraints;
  int MaxOfFailures;
  private JLabel jLabelDistanceThres;
  private DoubleField jDFDistanceThres;
  private JLabel jLabelBlankThres;
  private DoubleField jDFBlankThres;
  private JCheckBox cNear;
  private JButton jbAdvanced;
  private JButton jButtonApply;
  private JButton jButtonCancel;
  private JButton jButtonAllInputs;
  private JButton jButtonAllOutputs;
  private JButton jButtonAllRules;
  private AbstractButton abNature;
  private AbstractButton abClassif;
  private Vector jVectorKey;
  private Vector jVectorInputs;
  private Vector jVectorOutput;
  private Vector jVectorRules;
  private JPanel jPanelOpt = null;
  private JComponent jCompOpt = null;
  private int SeedValue = 1;
  private double l1 = 0.4, l2 = 0.2, l3 = 0.5;
  private Vector jVectorPFF;
  private boolean SaisieMaxFailures = false;
  private boolean SaisieMaxConstraints = false;
  private boolean InputsSelected = false;
  private boolean OutputSelected = false;
  private boolean RulesSelected = false;
  private JTextField jtfKey;
  private JCheckBox jcbDisplayKey;
  private JLabel jLabelLossOfCoverage = new JLabel();
  private DoubleField jDFLossOfCoverage = new DoubleField();
//------------------------------------------------------------------------------
  public JOptFrame( JFISFrame parent, JFIS fis, JExtendedDataFile data ) throws Throwable
  {
    this.fis = fis;
    this.Parent = parent;
    this.DataFile = data;
    try
      {
      if( this.fis.NbRules() == 0 )
        {
        MessageFIS.Error(this,LocaleFIS.GetString("Error"),LocaleFIS.GetString("OptimizationImpossibleTheFISHasNoRules"));
        return;
        }

      this.jBGOutput = new ButtonGroup();
      this.jPanelOutput = new JPanel(new GridBagLayout());
      this.jPanelOutput.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),LocaleFIS.GetString("OutputToBeOptimized")));
      for( int i=0, x=0, y=0 ; i<this.fis.NbOutputs() ; i++ )
        {
        JOutput output = this.fis.GetOutput(i);
        if( output.GetActive() == true )
          {
          JRadioButton jrb = new JRadioButton(output.GetName());
          jrb.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e)
            {
            try { JOptFrame.this.jbInit(); }
            catch(Throwable t) { MessageFIS.Error(null, t); }
            } });
          this.jBGOutput.add(jrb);
          this.jPanelOutput.add(jrb,  new GridBagConstraints(x, y, 1, 1, 0.0, 0.0
                ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 0, 0), 0, 0));
          //y = (x == 2) ? y+1 : y;
          //x = (x == 2) ? 0 : x+1;
          x++;
          }
        }
      ((JRadioButton)this.jBGOutput.getElements().nextElement()).setSelected(true); // active la premiere sortie
      jbInit();
      this.setLocation(JFISChildFrame.ChildPosition(this.Parent, this.getSize()));
      this.setVisible(true);
      }
    catch(Throwable t) { MessageFIS.Error(null, t); }
  }
//------------------------------------------------------------------------------
  private void jbInit()  throws Throwable
  {
    jLabelMaxOfIter = new JLabel();
    jIFMaxOfIter = new IntegerField();
    jLabelDistanceThres = new JLabel();
    jDFDistanceThres = new DoubleField();
    jLabelBlankThres = new JLabel();
    jDFBlankThres = new DoubleField();
    cNear = new JCheckBox();
    jbAdvanced = new JButton();
    jButtonApply = new JButton();
    jButtonAllInputs = new JButton();
    jButtonAllOutputs = new JButton();
    jButtonAllRules = new JButton();
    jButtonCancel = new JButton();

    this.jVectorKey = new Vector();
    this.jVectorInputs = new Vector();
    this.jVectorOutput = new Vector();
    this.jVectorRules = new Vector();
    this.jVectorPFF = new Vector();
    if( this.jPanelOpt != null )
      this.getContentPane().remove(this.jCompOpt);
    this.jPanelOpt = new JPanel(new GridBagLayout());

    // panel sortie
    jPanelOpt.add(jPanelOutput,  new GridBagConstraints(0, 0, 5, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 0, 5), 0, 0));

    // nombre d'iterations
    this.jIFMaxOfIter.setValue(100);
    /*jPanelOpt.add(this.jLabelMaxOfIter, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jIFMaxOfIter.setValue(100);
    jPanelOpt.add(this.jIFMaxOfIter, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));
*/
    // loss of coverage
    jPanelOpt.add(this.jLabelLossOfCoverage, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
    		,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
	this.jDFLossOfCoverage.setValue(10);
	jPanelOpt.add(this.jDFLossOfCoverage, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));
	jPanelOpt.add(new JLabel("%"), new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 40, 0));
    
    // distance des centres
    jPanelOpt.add(this.jLabelDistanceThres, new GridBagConstraints(3, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jDFDistanceThres.NumberFormat().setMaximumFractionDigits(10);
    this.jDFDistanceThres.setValue(0.000001);
    jPanelOpt.add(this.jDFDistanceThres, new GridBagConstraints(4, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));

    // seuil blanc
    jPanelOpt.add(this.jLabelBlankThres, new GridBagConstraints(3, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jDFBlankThres.setValue(0.1);
    jPanelOpt.add(this.jDFBlankThres, new GridBagConstraints(4, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));

    // infere sans redondance -supprime le 19 mai 2003
    // (a remplacer par disjonction max)
    // checkbox reused for MF max breakpoint constraint
    jPanelOpt.add(this.cNear, new GridBagConstraints(0, 4, 2, 1, 0.0, 0.0
	   ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));

    // options avancees
    jPanelOpt.add(this.jbAdvanced, new GridBagConstraints(2, 4, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jbAdvanced.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jbAdvanced_actionPerformed(e); } });

    // panel selection
    JPanel jPanelSelection = new JPanel(new GridBagLayout());
    jPanelOpt.add(jPanelSelection, new GridBagConstraints(0, 5, 5, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
    //allInputs button
    jButtonAllInputs.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonAllInputs_actionPerformed(e); } });
    jPanelSelection.add(jButtonAllInputs,  new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
    //allOutputs button
    if( this.fis.GetOutput(this.SelectedOutput()).GetNature() == JOutput.FUZZY )
      {
      jButtonAllOutputs.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonAllOutputs_actionPerformed(e); } });
      jPanelSelection.add(jButtonAllOutputs,  new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
              ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
      }
    //allRules button
    jButtonAllRules.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonAllRules_actionPerformed(e); } });
    jPanelSelection.add(jButtonAllRules,  new GridBagConstraints(2, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));

    // panel cle
    JPanel jPanelKey = new JPanel(new GridBagLayout());
    jPanelOpt.add(jPanelKey, new GridBagConstraints(0, 6, 5, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 0, 5), 0, 0));

    // entrees
    final JPanel jPanelInputs = new JPanel(new GridBagLayout());
    JScrollPane jsp = new JScrollPane(jPanelInputs) { public Dimension getMinimumSize() { return new Dimension(0, Math.min(150, jPanelInputs.getMinimumSize().height)); } };
    jPanelKey.add(jsp, new GridBagConstraints(0, 0, 2, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
    int y = 0;
    for( int i=0, x=0 ; i<this.fis.NbInputs() ; i++ )
      {
      JInput input = this.fis.GetInput(i);
      if( input.GetActive() == true )
        {
        JCheckBox jcbPFF = new JCheckBox(LocaleFIS.GetString("PFF"), true);
        jcbPFF.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { JOptFrame.this.jtfKey.setText(JOptFrame.this.GetKey()); } } );
        this.jVectorInputs.add(jcbPFF);
        this.jVectorPFF.add(jcbPFF);
        JPanel jPanelInput = new JPanel(new GridBagLayout());
        jPanelInput.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),input.GetName()));
        jPanelInputs.add(jPanelInput, new GridBagConstraints(x, y, 1, 1, 1.0, 0.0
              ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
        y = (x == 5) ? y+1 : y;
        x = (x == 5) ? 0 : x+1;
        //x++;
        jPanelInput.add(jcbPFF, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
              ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        Vector jVectorMFS = new Vector();
        for( int j=0 ; j<input.GetNbMF() ; j++ )
          {
          JCheckBox jcbMF = new JCheckBox(LocaleFIS.GetString("Optimize") + " " + input.GetMF(j).GetName(), true);
          jcbMF.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { JOptFrame.this.jtfKey.setText(JOptFrame.this.GetKey()); } } );
          this.jVectorInputs.add(jcbMF);
          jVectorMFS.add(jcbMF);
          jPanelInput.add(jcbMF, new GridBagConstraints(0, 1+j, 1, 1, 0.0, 0.0
              ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
          }
        this.jVectorPFF.add(jVectorMFS);
        }
      }
    this.jVectorKey.addAll(this.jVectorInputs);

    // sortie
    JPanel jPanelOutput = new JPanel(new GridBagLayout());
    final JOutput output = this.fis.GetOutput(this.SelectedOutput());
    final int nature = output.GetNature();
    // bouton abstrait nature de la sortie
    abNature = new AbstractButton() { public boolean isSelected() { return (nature == JOutput.FUZZY) ? true : false; } };

    this.jVectorKey.add(abNature);

    if( nature == JOutput.FUZZY ) // sortie floue
      {
      jPanelOutput.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),output.GetName()));
      final JPanel inner_output = new JPanel(new GridBagLayout());
      JScrollPane jsp_output = new JScrollPane(inner_output) { public Dimension getMinimumSize() { return new Dimension(inner_output.getSize().width, Math.min(150, inner_output.getMinimumSize().height)); } };
      jPanelOutput.add(jsp_output, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
      jPanelKey.add(jPanelOutput, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 0, 0, 5), 0, 0));
      // bouton abstrait classif
      final boolean classif = output.GetClassif();
       abClassif = new AbstractButton() { public boolean isSelected() { return classif; } };
      this.jVectorKey.add(abClassif);
      JCheckBox jcbPFF = new JCheckBox(LocaleFIS.GetString("PFF"), true);
      jcbPFF.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { JOptFrame.this.jtfKey.setText(JOptFrame.this.GetKey()); } } );
      this.jVectorOutput.add(jcbPFF);
      this.jVectorPFF.add(jcbPFF);
      inner_output.add(jcbPFF, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
      Vector jVectorMFS = new Vector();
      for( int j=0 ; j<output.GetNbMF() ; j++ )
        {
        JCheckBox jcbMF = new JCheckBox(LocaleFIS.GetString("Optimize") + " " + output.GetMF(j).GetName(), true);
        jcbMF.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { JOptFrame.this.jtfKey.setText(JOptFrame.this.GetKey()); } } );
        this.jVectorOutput.add(jcbMF);
        jVectorMFS.add(jcbMF);
        inner_output.add(jcbMF, new GridBagConstraints(0, 1+j, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        }
      this.jVectorPFF.add(jVectorMFS);
      this.jVectorKey.addAll(this.jVectorOutput);
      }

    // regles
    JPanel jPanelRules = new JPanel(new GridBagLayout());
    jPanelRules.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),LocaleFIS.GetString("Optimize") + " " + LocaleFIS.GetString("Rules").toLowerCase()));
    final JPanel inner_rules = new JPanel(new GridBagLayout());
    JScrollPane jsp_rules = new JScrollPane(inner_rules) { public Dimension getMinimumSize() { return new Dimension(/*Math.min(150, inner_rules.getMinimumSize().width)*/inner_rules.getSize().width, Math.min(150, inner_rules.getMinimumSize().height)); } };
    jPanelRules.add(jsp_rules, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
              ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
    jPanelKey.add(jPanelRules, new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0
              ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 0, 0), 0, 0));

    if( nature == JOutput.CRISP ) // sortie nette
      {
      final JCheckBox jCBVocabularyInFIS = new JCheckBox(LocaleFIS.GetString("OutputVocabularyInFIS"));
      jCBVocabularyInFIS.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { JOptFrame.this.jtfKey.setText(JOptFrame.this.GetKey()); } } );
      this.jVectorKey.add(jCBVocabularyInFIS);
      jCBVocabularyInFIS.setSelected(true);
      inner_rules.add(jCBVocabularyInFIS, new GridBagConstraints(0, 0, 4, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
      this.jVectorKey.add(new AbstractButton() { public boolean isSelected() { return false; } });
      }

    for( int i=0, x=0, yrule=1 ; i<this.fis.NbRules() ; i++ )
      if( this.fis.GetRule(i).GetActive() == true )
        {
        JCheckBox jcbrule = new JCheckBox(LocaleFIS.GetString("Rule")+Integer.toString(i+1), true);
        jcbrule.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { JOptFrame.this.jtfKey.setText(JOptFrame.this.GetKey()); } } );
        this.jVectorRules.add(jcbrule);
        inner_rules.add(jcbrule , new GridBagConstraints(x, yrule, 1, 1, 0.0, 0.0
              ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        yrule = (x == 3) ? yrule+1 : yrule;
        x = (x == 3) ? 0 : x+1;
        }
    this.jVectorKey.addAll(this.jVectorRules);

    // affichage de la cle
    JPanel jpDisplayKey = new JPanel(new GridBagLayout());
    this.jcbDisplayKey = new JCheckBox(LocaleFIS.GetString("DisplayKey"));
    this.jcbDisplayKey.setSelected(true);
    this.jtfKey = new JTextField(this.GetKey());
    this.jtfKey.setEditable(false);
    this.jcbDisplayKey.addActionListener(new ActionListener()
      {
      public void actionPerformed(ActionEvent e) { jtfKey.setVisible(jcbDisplayKey.isSelected()); }
      });
    jpDisplayKey.add(this.jcbDisplayKey, new GridBagConstraints(0, 0, 1, 1, 0.01, 0.0
              ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    jpDisplayKey.add(this.jtfKey, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
              ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 0, 0), 0, 0));
    jPanelKey.add(jpDisplayKey, new GridBagConstraints(0, 2, 2, 1, 1.0, 0.0
              ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 0, 5), 0, 0));

    // panel validation
    JPanel jPanelValidation = new JPanel(new GridBagLayout());
    //apply button
    jButtonApply.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonApply_actionPerformed(e); } });
    jPanelValidation.add(jButtonApply,  new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));

    // cancel button
    jButtonCancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { JOptFrame.this.dispose(); } });
    jPanelValidation.add(jButtonCancel,  new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
    jPanelOpt.add(jPanelValidation, new GridBagConstraints(0, 7, 5, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));

    // selectionne les entrees
    this.jButtonAllInputs_actionPerformed(null);
    this.jtfKey.setText(this.GetKey());

    // affichage
    this.Translate();
    this.setModal(true);
    this.jCompOpt = this.jPanelOpt;
    this.getContentPane().add(this.jCompOpt);
    this.pack();
  }
//------------------------------------------------------------------------------
  private void Translate()
  {
    this.setTitle(LocaleFIS.GetString("Optimization") + " " + LocaleFIS.GetString("SolisWets"));
    this.jLabelMaxOfIter.setText(LocaleFIS.GetString("MaximumNumberOfIterations") + ":");
    this.jLabelLossOfCoverage.setText(LocaleFIS.GetString("LossOfCoverage") + ":");
    this.jLabelDistanceThres.setText(LocaleFIS.GetString("CenterDistanceThreshold") + ":");
    this.jLabelBlankThres.setText(LocaleFIS.GetString("BlankThres") + ":");
    this.cNear.setText(LocaleFIS.GetString("Cnear"));
    this.jbAdvanced.setText(LocaleFIS.GetString("AdvancedParameters"));
    this.jButtonApply.setText(LocaleFIS.GetString("Apply"));
    this.jButtonCancel.setText(LocaleFIS.GetString("Cancel"));
    this.jButtonAllInputs.setText(LocaleFIS.GetString("SelectallInputs"));
    this.jButtonAllOutputs.setText(LocaleFIS.GetString("SelectallOutput"));
    this.jButtonAllRules.setText(LocaleFIS.GetString("SelectallRules"));
   }
//------------------------------------------------------------------------------
  void jbAdvanced_actionPerformed(ActionEvent e)
  {
  final JDialog dialog = new JFISDialog();
  dialog.setTitle(LocaleFIS.GetString("AdvancedParameters"));
  dialog.getContentPane().setLayout(new GridBagLayout());

  // init des valeurs par defaut
  this.InitMaxFailuresConstraints();

  // seed value
  JLabel jLabelSeedValue = new JLabel(LocaleFIS.GetString("SeedValue") + ":");
  final IntegerField jIFSeedValue = new IntegerField();
  jIFSeedValue.setValue(this.SeedValue);
  dialog.getContentPane().add(jLabelSeedValue, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
  dialog.getContentPane().add(jIFSeedValue, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));

  // Solis Wets constants
  JPanel jPanelSolisWets = new JPanel(new GridBagLayout());
  jPanelSolisWets.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),LocaleFIS.GetString("SolisWetsConstants")));
  dialog.getContentPane().add(jPanelSolisWets, new GridBagConstraints(0, 1, 2, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 0, 5), 0, 0));
  final DoubleField jDFl1 = new DoubleField();
  jDFl1.setValue(this.l1);
  jDFl1.setEditable(false);
  final DoubleField jDFl2 = new DoubleField();
  jDFl2.setValue(this.l2);
  jDFl2.setEditable(false);
  final DoubleField jDFl3 = new DoubleField();
  jDFl3.setValue(this.l3);
  jDFl3.setEditable(false);
  jPanelSolisWets.add(new JLabel("l1:"), new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 0), 0, 0));
  jPanelSolisWets.add(jDFl1, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 0), 20, 0));
  jPanelSolisWets.add(new JLabel("l2:"), new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 0), 0, 0));
  jPanelSolisWets.add(jDFl2, new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 0), 20, 0));
  jPanelSolisWets.add(new JLabel("l3:"), new GridBagConstraints(4, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 0), 0, 0));
  jPanelSolisWets.add(jDFl3, new GridBagConstraints(5, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 20, 0));

  // ecart type bruit gaussien
  JLabel jLabelStdGaussianNoise = new JLabel(LocaleFIS.GetString("StdGaussianNoise") + ":");
  final DoubleField jDFStdGaussianNoise = new DoubleField();
  dialog.getContentPane().add(jLabelStdGaussianNoise, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
          ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
  jDFStdGaussianNoise.setValue(this.StdGaussianNoise);
  dialog.getContentPane().add(jDFStdGaussianNoise, new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
          ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));

  // nombre de contraintes
  JLabel jLabelMaxOfConstraints = new JLabel(LocaleFIS.GetString("MaximumNumberOfConstraints") + ":");
  final IntegerField jIFMaxOfConstraints = new IntegerField();
  dialog.getContentPane().add(jLabelMaxOfConstraints, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
          ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
  jIFMaxOfConstraints.setValue(this.MaxOfConstraints);
  dialog.getContentPane().add(jIFMaxOfConstraints, new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0
          ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));

  // nombre d'echecs
  final IntegerField jIFMaxOfFailures = new IntegerField();
  jIFMaxOfFailures.setValue(this.MaxOfFailures);
  /*JLabel jLabelMaxOfFailures = new JLabel(LocaleFIS.GetString("MaximumNumberOfFailures") + ":");
  final IntegerField jIFMaxOfFailures = new IntegerField();
  dialog.getContentPane().add(jLabelMaxOfFailures, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
          ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
  jIFMaxOfFailures.setValue(this.MaxOfFailures);
  dialog.getContentPane().add(jIFMaxOfFailures, new GridBagConstraints(1, 4, 1, 1, 0.0, 0.0
          ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));
*/
  
  // nombre d'iterations
  dialog.getContentPane().add(this.jLabelMaxOfIter, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
	  ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
  dialog.getContentPane().add(this.jIFMaxOfIter, new GridBagConstraints(1, 4, 1, 1, 0.0, 0.0
	  ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));
  
  // panel validation
  JPanel jPanelValidation = new JPanel();
  JButton jbApply = new JButton(LocaleFIS.GetString("Apply"));
  jbApply.addActionListener(new ActionListener()
    {
    public void actionPerformed(ActionEvent e)
      {
      JOptFrame.this.SeedValue = jIFSeedValue.getValue();
      JOptFrame.this.l1 = jDFl1.getValue();
      JOptFrame.this.l2 = jDFl2.getValue();
      JOptFrame.this.l3 = jDFl3.getValue();
      JOptFrame.this.StdGaussianNoise = jDFStdGaussianNoise.getValue();
      if( JOptFrame.this.MaxOfConstraints != jIFMaxOfConstraints.getValue() )
        JOptFrame.this.SaisieMaxConstraints = true;
      JOptFrame.this.MaxOfConstraints = jIFMaxOfConstraints.getValue();
      if( JOptFrame.this.MaxOfFailures != jIFMaxOfFailures.getValue() )
        JOptFrame.this.SaisieMaxFailures = true;
      JOptFrame.this.MaxOfFailures = jIFMaxOfFailures.getValue();
      dialog.dispose();
      }
    });
  JButton jbCancel = new JButton(LocaleFIS.GetString("Cancel"));
  jbCancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { dialog.dispose(); } });
  jPanelValidation.add(jbApply);
  jPanelValidation.add(jbCancel);
  dialog.getContentPane().add(jPanelValidation, new GridBagConstraints(0, 5, 2, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));

  // affichage
  dialog.pack();
  dialog.setModal(true);
  dialog.setResizable(false);
  dialog.setLocation(JFISChildFrame.ChildPosition(JOptFrame.this, dialog.getSize()));
  dialog.setVisible(true);
  }
//------------------------------------------------------------------------------
  private int SelectedOutput() throws Throwable
  {
  Enumeration items = this.jBGOutput.getElements();
  for( int i=0 ; i<this.fis.NbOutputs() ; i++ )
    if( this.fis.GetOutput(i).GetActive() == true )
      if( ((JRadioButton)items.nextElement()).isSelected() )
        return i;
  throw new Exception("NoOutputFound");
  }
//------------------------------------------------------------------------------
  private String GetKey()
  {
  String key = new String();
  for( int i=this.jVectorKey.size()-1 ; i>=0 ; i-- )
    key = key + (((AbstractButton)this.jVectorKey.elementAt(i)).isSelected() ? "1" : "0");
  return key;
  }
//------------------------------------------------------------------------------
  void jButtonApply_actionPerformed(ActionEvent e)
  {
  //System.out.println(this.GetKey());
  try
    {
    this.InitMaxFailuresConstraints();
    JFIS new_fis = this.fis.NewCustomFISOPT(this.DataFile, this.GetKey(), this.jIFMaxOfIter.getValue(), this.StdGaussianNoise, this.MaxOfConstraints, this.MaxOfFailures, this.jDFDistanceThres.getValue(), this.SelectedOutput(), this.cNear.isSelected(), this.SeedValue, this.l1, this.l2, this.l3, 5, this.jDFBlankThres.getValue(), this.jDFLossOfCoverage.getValue() / 100.0);
    if( new_fis != null ) { JFISFrame jf = new JFISFrame(new_fis, this.DataFile, this.Parent); }
    else { MessageFIS.Information(this, LocaleFIS.GetString("NoPerformanceImprovement")); }
    this.dispose();
    }
  catch(Throwable t) { MessageFIS.Error(null, t); }
  }

//------------------------------------------------------------------------------
  void jButtonAllInputs_actionPerformed(ActionEvent e)
    {
    this.InputsSelected = !this.InputsSelected;
    this.OutputSelected = false;
    this.RulesSelected = false;
    for( int i=0;i<=this.jVectorInputs.size()-1 ; i++ )
      {
      ((AbstractButton)this.jVectorInputs.elementAt(i)).setSelected(this.InputsSelected);
      ((AbstractButton)this.jVectorInputs.elementAt(i)).setEnabled(true);
      }
    for( int i=0;i<=this.jVectorOutput.size()-1 ; i++ )
      {
      ((AbstractButton)this.jVectorOutput.elementAt(i)).setSelected(false);
      ((AbstractButton)this.jVectorOutput.elementAt(i)).setEnabled(false);
      }
    for( int i=0;i<=this.jVectorRules.size()-1 ; i++ )
      {
      ((AbstractButton)this.jVectorRules.elementAt(i)).setSelected(false);
      ((AbstractButton)this.jVectorRules.elementAt(i)).setEnabled(false);
      }
    this.jtfKey.setText(this.GetKey());
    }
//------------------------------------------------------------------------------
  void jButtonAllOutputs_actionPerformed(ActionEvent e)
    {
    this.OutputSelected = !this.OutputSelected;
    this.InputsSelected = false;
    this.RulesSelected = false;
    for( int i=0;i<=this.jVectorOutput.size()-1 ; i++ )
      {
      ((AbstractButton)this.jVectorOutput.elementAt(i)).setSelected(this.OutputSelected);
      ((AbstractButton)this.jVectorOutput.elementAt(i)).setEnabled(true);
      }
    for( int i=0;i<=this.jVectorInputs.size()-1 ; i++ )
      {
      ((AbstractButton)this.jVectorInputs.elementAt(i)).setSelected(false);
      ((AbstractButton)this.jVectorInputs.elementAt(i)).setEnabled(false);
      }
    for( int i=0;i<=this.jVectorRules.size()-1 ; i++ )
      {
      ((AbstractButton)this.jVectorRules.elementAt(i)).setSelected(false);
      ((AbstractButton)this.jVectorRules.elementAt(i)).setEnabled(false);
      }
    this.jtfKey.setText(this.GetKey());
    }
//------------------------------------------------------------------------------
  void jButtonAllRules_actionPerformed(ActionEvent e)
    {
    this.RulesSelected = !this.RulesSelected;
    this.InputsSelected = false;
    this.OutputSelected = false;
    for( int i=0;i<=this.jVectorRules.size()-1 ; i++ )
      {
      ((AbstractButton)this.jVectorRules.elementAt(i)).setSelected(this.RulesSelected);
      ((AbstractButton)this.jVectorRules.elementAt(i)).setEnabled(true);
      }
    for( int i=0;i<=this.jVectorInputs.size()-1 ; i++ )
      {
      ((AbstractButton)this.jVectorInputs.elementAt(i)).setSelected(false);
      ((AbstractButton)this.jVectorInputs.elementAt(i)).setEnabled(false);
      }
    for( int i=0;i<=this.jVectorOutput.size()-1 ; i++ )
      {
      ((AbstractButton)this.jVectorOutput.elementAt(i)).setSelected(false);
      ((AbstractButton)this.jVectorOutput.elementAt(i)).setEnabled(false);
      }
    this.jtfKey.setText(this.GetKey());
    }
//------------------------------------------------------------------------------
  private void InitMaxFailuresConstraints()
    {
    /*if( this.SaisieMaxFailures == false )
      this.MaxOfFailures = 10 * this.jIFMaxOfIter.getValue();
    if( this.SaisieMaxConstraints == false )
      {
      int nbMFPFF = 0;
      for( int i=0 ; i<this.jVectorPFF.size() ; i+=2 )
        if( ((AbstractButton)this.jVectorPFF.elementAt(i)).isSelected() == true )
          for( int j=0 ; j<((Vector)this.jVectorPFF.elementAt(i+1)).size() ; j++ )
            if( ((AbstractButton)((Vector)this.jVectorPFF.elementAt(i+1)).elementAt(j)).isSelected() == true )
              nbMFPFF++;
      this.MaxOfConstraints = 10 * this.jIFMaxOfIter.getValue() * (1 + (this.cNear.isSelected() ? 1 : 0) + (this.jDFDistanceThres.getValue() == 1 ? 0 : 1) + nbMFPFF);
      }*/
    this.MaxOfFailures = 1000;
    this.MaxOfConstraints = 1000;
    }
}
