//***********************************************************************
//
//                              JOutput.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.util.Arrays;

//------------------------------------------------------------------------------
public class JOutput extends JInput
{
	//------------------------------------------------------------------------------
	public final static int FUZZY = 0;
	public final static int CRISP = 1;
	public final static String IMPLI = "impli"; //type de regle implicative
	public final static String CONJ = "conj";
	public final static int DEFUZ_FUZZY_AREA = 0;
	public final static int DEFUZ_CRISP_SUGENO = 0;   /**<Defuzification sugeno nette. index de retour de la fonction jnifis#TypeDefuzzificationNette (a modifier pour plus de robustesse) */
	public final static int DEFUZ_CRISP_MAXCRISP = 1; /**<Defuzification maxcrisp nette. index de retour de la fonction jnifis#TypeDefuzzificationNette (a modifier pour plus de robustesse) */

	//------------------------------------------------------------------------------
	public JOutput( long ptr ) { super( ptr ); }
	//------------------------------------------------------------------------------
	public JOutput( JOutput output ) throws Throwable { super( jnifis.NewOutput(output.Ptr()) ); }
	//------------------------------------------------------------------------------
	public JInput NewRegular( int nb_sef, double min, double max) throws Throwable
	{
		JOutput new_output = new JOutput( jnifis.NewRegularOutput(nb_sef, min, max, this.TypeDefuzzification()[this.GetDefuz()], this.GetClassif(), this.GetDefaultValue(), this.TypeDisjunction()[this.GetDisjunction()]) );
		new_output.SetName(this.GetName());
		new_output.SetActive(this.GetActive());
		// les SEF sont crees sans nom -> affectation d'un nom par defaut
		for( int i=0 ; i<new_output.GetNbMF() ; i++ )
			((JMF)new_output.GetMF(i)).SetName(LocaleFIS.GetString("MF") + String.valueOf(i+1) );
		return new_output;
	}
	//------------------------------------------------------------------------------
	public JInput NewIrregular( double []sommets, int nb_sef, double min, double max) throws Throwable
	{
		JOutput new_output = new JOutput( jnifis.NewIrregularOutput(sommets, nb_sef, min, max, this.TypeDefuzzification()[this.GetDefuz()], this.GetClassif(), this.GetDefaultValue(), this.TypeDisjunction()[this.GetDisjunction()]) );
		new_output.SetName(this.GetName());
		new_output.SetActive(this.GetActive());
		// les SEF sont crees sans nom -> affectation d'un nom par defaut
		for( int i=0 ; i<new_output.GetNbMF() ; i++ )
			((JMF)new_output.GetMF(i)).SetName(LocaleFIS.GetString("MF") + String.valueOf(i+1) );
		return new_output;
	}
	//------------------------------------------------------------------------------
	public void ReplaceMFOutput( int mf_number, JMF new_sef ) throws Throwable { jnifis.ReplaceMFOutput(ptr, mf_number, new_sef.Ptr()); }
	//------------------------------------------------------------------------------
	public int GetNature() throws Throwable
	{
		String [] types = this.TypeNature();
		String type = jnifis.GetOutputNature( ptr );
		if( type.equals(types[JOutput.CRISP]) )
			return JOutput.CRISP;
		if( type.equals(types[JOutput.FUZZY]) )
			return JOutput.FUZZY;  
		throw new Exception(LocaleFIS.GetString("UnknownNature") + ": " + type);
	}
	//------------------------------------------------------------------------------
	public boolean GetImplicatif() throws Throwable
	{
		String typeR = jnifis.GetOutputDefuz(this.ptr);
		if (typeR.equals(JOutput.IMPLI))
			return true;
		else return false;
	}
	//------------------------------------------------------------------------------
	public int GetDefuz() throws Throwable
	{
		String defuz = jnifis.GetOutputDefuz( ptr ); //impli de objet c++
		String [] defuzs = this.TypeDefuzzification(); //
		for( int i=0 ; i<defuzs.length ; i++ )
			if( defuz.equals(defuzs[i]) )
				return i;
		throw new Exception(LocaleFIS.GetString("UnknownDefuzzification") + ": " + defuz);
	}
	//------------------------------------------------------------------------------
	public int GetDisjunction() throws Throwable
	{
		String disjunction = jnifis.GetOutputDisjunction( ptr );
		String [] disjs = this.TypeDisjunction();
		for( int i=0 ; i<disjs.length ; i++ )
			if( disjunction.equals(disjs[i]) )
				return i;
		throw new Exception(LocaleFIS.GetString("UnknownDisjunction") + ": " + disjunction);
	}  
	//------------------------------------------------------------------------------
	public void SetDefaultValue( double default_value ) throws Throwable { jnifis.SetOutputDefaultValue( ptr, default_value ); }
	//------------------------------------------------------------------------------
	public double GetDefaultValue() throws Throwable { return jnifis.GetOutputDefaultValue( ptr ); }
	//------------------------------------------------------------------------------
	public boolean GetClassif() throws Throwable { return jnifis.GetOutputClassif( ptr ); }
	//------------------------------------------------------------------------------
	public String [] TypeDefuzzification() throws Throwable
	{
		if(this.GetImplicatif())
		{
			return jnifis.TypeDefuzzificationImplicatif();
		}
		if( this.GetNature() == JOutput.FUZZY )
			return jnifis.TypeDefuzzificationFloue();
		if( this.GetNature() == JOutput.CRISP )
			return jnifis.TypeDefuzzificationNette();
		throw new Exception(LocaleFIS.GetString("UnknownNature") + ": " + this.GetNature());
	}  
	//------------------------------------------------------------------------------
	public String [] TypeDisjunction() throws Throwable
	{
		if(this.GetImplicatif())
		{
			return jnifis.TypeDisjunctionImplicatif();
		}
		if( this.GetNature() == JOutput.FUZZY )
			return jnifis.TypeDisjunctionFloue();
		if( this.GetNature() == JOutput.CRISP )
			return jnifis.TypeDisjunctionNette();
		throw new Exception(LocaleFIS.GetString("UnknownNature") + ": " + this.GetNature());
	}
	//------------------------------------------------------------------------------
	public static String [] TypeNature() throws Throwable
	{ return jnifis.TypeNature(); }
	//------------------------------------------------------------------------------
	static public String [] TypeFuzzyDefuzzification() throws Throwable { return jnifis.TypeDefuzzificationFloue(); }
	//------------------------------------------------------------------------------
	static public String [] TypeFuzzyDisjunction() throws Throwable { return jnifis.TypeDisjunctionFloue(); }
	//------------------------------------------------------------------------------
	public int GetAlarm() throws Throwable { return jnifis.GetAlarm(this.ptr); }
	//------------------------------------------------------------------------------
	public void SetDefuzThres( double thres ) throws Throwable { jnifis.SetDefuzThres(this.ptr, thres); }
	//------------------------------------------------------------------------------
	public double GetDefuzThres() throws Throwable { return jnifis.GetDefuzThres(this.ptr); }
	//------------------------------------------------------------------------------
	/** Renvoie les labels des classes pour une sortie classif
	 * @return tableau trie des labels des classes ou null si la sortie n'est pas classif
	 * */
	public double[] GetClasses() throws Throwable
	{
		double []classes = jnifis.GetClasses(this.ptr);
		Arrays.sort(classes);
		return classes;
	}
	public boolean IsQsp() throws Throwable
	{
		return jnifis.IsQsp(this.ptr);
	}
}



//------------------------------------------------------------------------------
class JOutputFloue extends JOutput
{
	//------------------------------------------------------------------------------
	public JOutputFloue() throws Throwable { super( jnifis.NewOutputFloue() ); }
	//------------------------------------------------------------------------------
	public JOutputFloue( JOutput output ) throws Throwable { super(jnifis.NewOutputFloue(output.Ptr())); }
}

//------------------------------------------------------------------------------
class JOutputNette extends JOutput
{
	//------------------------------------------------------------------------------
	public JOutputNette() throws Throwable { super( jnifis.NewOutputNette() ); }
	//------------------------------------------------------------------------------
	public JOutputNette( JOutput output ) throws Throwable { super(jnifis.NewOutputNette(output.Ptr())); }
}

