//***********************************************************************
//
//                              JOutputFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import java.lang.*;
import javax.swing.event.*;

//------------------------------------------------------------------------------
public class JOutputFrame extends JInputFrame
{
	private JPanel jPanelOutput = new JPanel(new GridBagLayout());
	private JCheckBox jCBClassif = new JCheckBox();
	private JCheckBox jCBImpli = new JCheckBox();
	private ButtonGroup buttonGroupNature = new ButtonGroup();
	private JPanel jPanelNature = new JPanel(new BorderLayout());
	private TitledBorder titledBorderNature;
	private JRadioButton jRBFuzzy = new JRadioButton();
	private JRadioButton jRBCrisp = new JRadioButton();
	private JRadioButton [] jRBNature = new JRadioButton[2];
	private JLabel jLDefaultValue = new JLabel();
	private DoubleFieldInput dfDefaultValue;
	private JLabel jLDeffuzzication = new JLabel();
	private DefaultComboBoxModel jCBDefuzzificationModel = new DefaultComboBoxModel()
	{
		// c'est cette methode qui assure la traduction des types de defuzzification
		public Object getElementAt(int index)
		{
			try { return LocaleFIS.GetString(((JOutput)JOutputFrame.this.Temp).TypeDefuzzification()[index]); }
			catch( Throwable t ) { return null; }
		}
	};
	private JComboBox jCBDefuzzification = new JComboBox(this.jCBDefuzzificationModel);
	private JLabel jlThres = new JLabel();
	private DoubleField dfThres = new DoubleField();
	private JLabel jLDisjunction = new JLabel();
	private DefaultComboBoxModel jCBDisjunctionModel = new DefaultComboBoxModel()
	{
		// c'est cette methode qui assure la traduction des types de defuzzification
		public Object getElementAt(int index)
		{
			try { return LocaleFIS.GetString(((JOutput)JOutputFrame.this.Temp).TypeDisjunction()[index]); }
			catch( Throwable t ) { return null; }
		}
	};
	private JComboBox jCBDisjunction = new JComboBox(this.jCBDisjunctionModel);
	private boolean ModifyType = false;

	//------------------------------------------------------------------------------
	protected JOutputFrame( JFISFrame parent ) { super(parent); }
	//------------------------------------------------------------------------------
	public JOutputFrame( JFISFrame parent, JFIS fis, int output_number ) throws Throwable
	{
		super(parent);
		this.fis = fis;
		this.Number = output_number;
		this.Temp = this.fis.GetOutput(this.Number);
		this.Semaphore = this.fis.GetOutputSemaphore(output_number);
	}
	//------------------------------------------------------------------------------
	protected void jbInit() throws Throwable
	{
		super.jbInit();
		this.jCBActive.setEnabled(false);

		// impli
		jPanelOutput.add(jCBImpli, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
		// classif
		jPanelOutput.add(jCBClassif,        new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
		// nature
		titledBorderNature = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"");
		jPanelNature.setBorder(titledBorderNature);
		buttonGroupNature.add(jRBFuzzy);
		buttonGroupNature.add(jRBCrisp);
		this.jRBNature[JOutput.FUZZY] = this.jRBFuzzy;
		this.jRBNature[JOutput.CRISP] = this.jRBCrisp;
		this.jRBNature[((JOutput)this.Temp).GetNature()].setSelected(true);
		jPanelNature.add(jRBFuzzy,  BorderLayout.NORTH);
		jPanelNature.add(jRBCrisp,  BorderLayout.SOUTH);
		jPanelOutput.add(jPanelNature,      new GridBagConstraints(0, 0, 1, 3, 0.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 5), 0, 0));
		// valeur par defaut
		this.dfDefaultValue = new DoubleFieldInput(this.Temp);
		this.dfDefaultValue.SetAction( new NumberFieldActionListener()
		{
			public void NumberFieldAction()
			{
				try { JOutputFrame.this.fis.SetOutputDefaultValue(JOutputFrame.this.Number, JOutputFrame.this.dfDefaultValue.getValue()); }
				catch( Throwable t ) { MessageFIS.Error(null, t); }
			}
		} );
		jPanelOutput.add(jLDefaultValue,   new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		jPanelOutput.add(dfDefaultValue,     new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 50, 0));
		// deffuzzification
		this.jCBDefuzzification.setRenderer(new ComboRenderer() { protected boolean isEnabled( int index ) { return JOutputFrame.this.isAreaDefuzEnabled(index); } });
		jPanelOutput.add(jLDeffuzzication,  new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		jPanelOutput.add(jCBDefuzzification,   new GridBagConstraints(2, 2, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
		// threshold
		jPanelOutput.add(this.jlThres,  new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		//    this.dfThres.setValue(((JOutput)this.Temp).GetDefuzThres());
		this.dfThres.SetAction( new NumberFieldActionListener()
		{
			public void NumberFieldAction()
			{
				try { ((JFIS)JOutputFrame.this.fis).SetAlarmThres(JOutputFrame.this.Number, JOutputFrame.this.dfThres.getValue());}
				catch( Throwable t ) { MessageFIS.Error(null, t); }
			}
		} );
		jPanelOutput.add(this.dfThres,   new GridBagConstraints(2, 3, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 5, 0), 50, 0));
		// disjunction
		jPanelOutput.add(jLDisjunction,  new GridBagConstraints(1, 4, 1, 1, 0.0, 0.0
				,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		jPanelOutput.add(jCBDisjunction,   new GridBagConstraints(2, 4, 1, 1, 0.0, 0.0
				,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 5, 0), 0, 0));
		// panel sortie
		this.PanelInput.add(jPanelOutput,            new GridBagConstraints(0, 3, 1, 1, 1.0, 0.0
				,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 10, 0, 10), 0, 0));
	}
	//------------------------------------------------------------------------------
	private boolean isAreaDefuzEnabled( int index )
	{
		JOutput temp = (JOutput)this.Temp;
		try
		{
			if( (temp.GetNature() == JOutput.CRISP) | (index != JOutput.DEFUZ_FUZZY_AREA) ) return true;
			if( this.jCBMFType.getSelectedIndex() == JMF.DISCRETE ) return false;
			for (int i = 0; i < temp.GetNbMF(); i++)
			{
				if (temp.GetMF(i).GetType() == JMF.DISCRETE)
					return false;
				/*if(temp.GetMF(i).GetType() == JMF.POSSIBILITY_DISTRIBUTION)
			  return false;*/
			}
			return true;
		}
		catch( Throwable t ) { return false; }
	}
	//------------------------------------------------------------------------------
	protected boolean isDiscretMFEnabled( int index )
	{
		try { return ( ( ((JOutput)this.Temp).GetDefuz() == JOutput.DEFUZ_FUZZY_AREA) & (index == JMF.DISCRETE) )  ? false : true; }
		catch( Throwable t ) { return false; }
	}
	//------------------------------------------------------------------------------
	protected void InitObjectsWithTemp() throws Throwable
	{
		super.InitObjectsWithTemp();
		this.jCBClassif.setSelected(((JOutput)this.Temp).GetClassif());
		this.dfDefaultValue.SetInput(this.Temp);
		this.dfDefaultValue.setValue(((JOutput)this.Temp).GetDefaultValue());
		this.jCBDefuzzificationModel.removeAllElements();
		for( int i=0 ; i<((JOutput)this.Temp).TypeDefuzzification().length ; i++ )
			this.jCBDefuzzificationModel.addElement(new String());  // ajout de chaines vide, c'est la methode getElementAt du modele qui assure la traduction
		this.jCBDefuzzification.setSelectedIndex(((JOutput)this.Temp).GetDefuz());
		this.jCBDisjunctionModel.removeAllElements();
		for( int i=0 ; i<((JOutput)this.Temp).TypeDisjunction().length ; i++ )
			this.jCBDisjunctionModel.addElement(new String());  // ajout de chaines vide, c'est la methode getElementAt du modele qui assure la traduction
		this.jCBDisjunction.setSelectedIndex(((JOutput)this.Temp).GetDisjunction());
		if(this.jRBFuzzy.isSelected()) this.jCBImpli.setSelected(((JOutput)this.Temp).GetImplicatif()); //impli et fuzzy
		else this.jCBImpli.setVisible(false);
		this.dfThres.setValue(((JOutput)this.Temp).GetDefuzThres());
	}
	//------------------------------------------------------------------------------
	public void Translate() throws Throwable
	{
		super.Translate();
		this.setTitle(LocaleFIS.GetString("Output"));
		this.jMenuInputRange.setText(LocaleFIS.GetString("Output"));
		this.jCBClassif.setText(LocaleFIS.GetString("Classif"));
		this.jCBImpli.setText(LocaleFIS.GetString("Impli"));
		this.titledBorderNature.setTitle(LocaleFIS.GetString("Nature"));
		this.jRBFuzzy.setText(LocaleFIS.GetString(JOutput.TypeNature()[JOutput.FUZZY]));
		this.jRBCrisp.setText(LocaleFIS.GetString(JOutput.TypeNature()[JOutput.CRISP]));
		this.jLDefaultValue.setText(LocaleFIS.GetString("DefaultValue")+":");
		this.jLDeffuzzication.setText(LocaleFIS.GetString("Defuzzification")+":");
		this.jlThres.setText(LocaleFIS.GetString("AlarmThreshold")+":");
		this.jLDisjunction.setText(LocaleFIS.GetString("Disjunction")+":");
		this.dfDefaultValue.setLocale(LocaleFIS.Locale());
		this.dfDefaultValue.setValue(((JOutput)this.Temp).GetDefaultValue());
		this.jCBDefuzzification.setSelectedIndex(((JOutput)this.Temp).GetDefuz());
		this.jCBDisjunction.setSelectedIndex(((JOutput)this.Temp).GetDisjunction());
	}
	//------------------------------------------------------------------------------
	protected void Events()
	{
		super.Events();
		jCBClassif.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jCBClassif_actionPerformed(e); } });
		jCBImpli.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jCBImpli_actionPerformed(e); } });
		jRBFuzzy.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jRBFuzzy_actionPerformed(e); } });
		jRBCrisp.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jRBCrisp_actionPerformed(e); } });
		//jCBDefuzzification.addItemListener(new ItemListener() { public void itemStateChanged(ItemEvent e) { jCBDefuzzification_itemStateChanged(e); } });
		this.jCBDefuzzification.addActionListener(new ComboListener(this.jCBDefuzzification)
		{
			protected boolean isEnabled( int index ) { return JOutputFrame.this.isAreaDefuzEnabled(index); }
			protected void actionPerformed()
			{
				try
				{
					int index = JOutputFrame.this.jCBDefuzzification.getSelectedIndex();
					if( index != -1 )
					{
						int mf_selected = JOutputFrame.this.jListMF.getSelectedIndex(); // sauvegarde le SEF selectionnne
						JOutputFrame.this.fis.SetOutputDefuz(JOutputFrame.this.Number, ( (JOutput)JOutputFrame.this.Temp).TypeDefuzzification()[index]);
						JOutputFrame.this.dfThres.setValue(((JOutput)JOutputFrame.this.Temp).GetDefuzThres());
						JOutputFrame.this.jListMF.clearSelection();
						JOutputFrame.this.jListMF.setSelectedIndex(mf_selected); // reaffiche le SEF selectionne
					}
				}
				catch( Throwable t ) { MessageFIS.Error(null, t); }
			}
		} );
		jCBDisjunction.addItemListener(new ItemListener() { public void itemStateChanged(ItemEvent e) { jCBDisjunction_itemStateChanged(e); } });
	}
	//------------------------------------------------------------------------------
	protected void ActiveChanged() { this.fis.OutputActiveChanged(this.Number); }
	//------------------------------------------------------------------------------
	protected void NameChanged() { this.fis.OutputNameChanged(this.Number);}
	//------------------------------------------------------------------------------
	protected void RangeChanged() { this.fis.OutputRangeChanged(this.Number);}
	//------------------------------------------------------------------------------
	protected void Replace( JInput new_input ) throws Throwable { this.fis.ReplaceOutput(this.Number, (JOutput)new_input); }
	//------------------------------------------------------------------------------
	protected void RemoveMF( int mf_number ) throws Throwable { this.fis.RemoveMFInOutput(this.Number, mf_number); }
	//------------------------------------------------------------------------------
	protected void AddMF( JMF new_mf ) throws Throwable
	{
		this.Temp.AddMF(new_mf);
		this.fis.MFAddedInOutput(this.Number);
	}
	//------------------------------------------------------------------------------
	protected void ReplaceMF( int mf_number, JMF new_mf ) throws Throwable
	{
		((JOutput)this.Temp).ReplaceMFOutput(mf_number, new_mf);
		this.fis.MFReplacedInOutput(this.Number);
	}
	//------------------------------------------------------------------------------
	void jMenuMFs_menuSelected(MenuEvent e)
	{
		super.jMenuMFs_menuSelected(e);
		try
		{
			if( ((JOutput)this.Temp).GetNbMF() == 1 )
				this.jMenuRemoveMF.setEnabled(false);
			this.jMenuNewMF.setEnabled(false);
			this.jMenuRegularGrid.setEnabled(false);
			this.jMenuIrregularGrid.setEnabled(false);
			if( ((JOutput)this.Temp).GetNature() == JOutput.FUZZY )
			{
				this.jMenuNewMF.setEnabled(true);
				this.jMenuRegularGrid.setEnabled(true);
				this.jMenuIrregularGrid.setEnabled(true);
			}
		}
		catch( Throwable t ) {}
	}
	//------------------------------------------------------------------------------
	void jListMF_mousePressed(MouseEvent e)
	{
		// bouton droit
		if (SwingUtilities.isRightMouseButton(e))
		{
			try { this.jPopupRemoveMF.setEnabled( ((JOutput)this.Temp).GetNbMF() == 1 ? false : true ); }
			catch( Throwable t ) {}
			SwingUtilities.updateComponentTreeUI(jPopupMFs);    // affecte le look actuel au composant jPopupMenuInputs
			jPopupMFs.show(jListMF, e.getX(), e.getY());       // affichage du popup
		}
	}
	//------------------------------------------------------------------------------
	void jRBFuzzy_actionPerformed(ActionEvent e)
	{
		try
		{
			this.fis.Crisp2Fuz(null, this.Number);
			this.Temp = this.fis.GetOutput(this.Number);
			for( int i=0 ; i<this.Temp.GetNbMF() ; i++ )
				this.Temp.GetMF(i).SetName(LocaleFIS.GetString("MF")+(i+1));
			this.InitObjectsWithTemp();
			if( this.Temp.GetNbMF() != 0 )
			{
				this.AddPanelMFs();
				this.jListMF.setSelectedIndex(0);
				this.pack();
			}
			this.ModifyType = true;
			//type regle implicative possible
			this.jCBImpli.setVisible(true);
		}
		catch( Throwable t ) { MessageFIS.Error( null, t ); this.jRBCrisp.setSelected(true); }
	}
	//------------------------------------------------------------------------------
	void jRBCrisp_actionPerformed(ActionEvent e)
	{
		try
		{
			boolean pack = false;
			if( this.Temp.GetNbMF() != 0 )
			{
				if( MessageFIS.Confirm(this, LocaleFIS.GetString("AllMFsWillBeErased") + "\n" + LocaleFIS.GetString("DoYouWantToContinue")) != JOptionPane.YES_OPTION )
				{
					this.jRBFuzzy.setSelected(true);
					return;
				}
				pack = true;    // redimensionne la fenetre si il y avait des SEF avant le passage en sortie nette
			}
			this.fis.Fuz2Crisp(this.Number);
			this.Temp = this.fis.GetOutput(this.Number);
			this.InitObjectsWithTemp();
			if( pack == true )
			{
				this.RemovePanelMFs();
				this.pack();
			}
			this.ModifyType = true;
			//type regle implicative impossible
			this.jCBImpli.setVisible(false);
		}
		catch( Throwable t ) { MessageFIS.Error( null, t ); }
	}
	//------------------------------------------------------------------------------
	protected String RangeType() { return new String("OutputRange"); }
	//------------------------------------------------------------------------------
	void jCBClassif_actionPerformed(ActionEvent e)
	{
		try
		{
			this.fis.SetOutputClassif(this.Number, this.jCBClassif.isSelected());
			this.dfThres.setValue(((JOutput)this.Temp).GetDefuzThres());
		}
		catch( Throwable t ) { MessageFIS.Error(null, t); }
	}
	//------------------------------------------------------------------------------
	void jCBImpli_actionPerformed(ActionEvent e)
	{
		try
		{	
			boolean transfPart = false;
			if(this.jCBImpli.isSelected()) //Conj2Imp
			{
				if( this.Temp.GetNbMF()>0 )
				{
					if(!this.fis.GetOutput(this.Number).IsQsp())
					{
						if(this.fis.GetOutput(this.Number).IsSfp())
						{
							if(	MessageFIS.Confirm(this, LocaleFIS.GetString("QspPossibleAppliquer") ) == JOptionPane.YES_OPTION )
								transfPart = true;
						}
						else
							if(	MessageFIS.Confirm(this, LocaleFIS.GetString("QspImpossibleContinuer") ) != JOptionPane.YES_OPTION )
							{	
								this.jCBImpli.setSelected(false);
								return;
							}
					}
					if(fis.hasExpertWeights() && (MessageFIS.Confirm(this, LocaleFIS.GetString("ExpertWeightsWillBeLostContinue")) != JOptionPane.YES_OPTION)) {
						this.jCBImpli.setSelected(false);
						return;
					}
					this.fis.Conj2Imp(this.Number,transfPart);
				}

			}
			else //Imp2Conj		
			{				
				if( this.Temp.GetNbMF()>0 )
				{
					if(!this.fis.GetOutput(this.Number).IsSfp()) // non sfp case
					{
						if(this.fis.GetOutput(this.Number).IsQsp()) //qsp case
						{
							if(	MessageFIS.Confirm(this, LocaleFIS.GetString("SfpPossibleAppliquer") ) == JOptionPane.YES_OPTION )
								transfPart = true;//transformation to Sfp required by user
						}
						else // non qsp case

							if(	MessageFIS.Confirm(this, LocaleFIS.GetString("SfpImpossibleContinuer") ) != JOptionPane.YES_OPTION )
							{	
								this.jCBImpli.setSelected(true);
								return;
							}
					}
					// already sfp (no transformation of partition) or 
					this.fis.Imp2Conj(this.Number, transfPart);

				}
			}
			this.Temp = this.fis.GetOutput(this.Number);
			for( int i=0 ; i<this.Temp.GetNbMF() ; i++ )
				this.Temp.GetMF(i).SetName(LocaleFIS.GetString("MF")+(i+1));
			this.InitObjectsWithTemp();
			if( this.Temp.GetNbMF() != 0 )
			{
				this.AddPanelMFs();
				this.jListMF.setSelectedIndex(0);
				this.pack();
			}	
		}
		catch( Throwable t ) { MessageFIS.Error(null, t);  this.jCBImpli.setSelected(false);}
	}
	//------------------------------------------------------------------------------
	/*void jCBDefuzzification_itemStateChanged(ItemEvent e)
      {
      //if( e.getStateChange() == ItemEvent.SELECTED )
      //   {
      try
      {
      int index = this.jCBDefuzzification.getSelectedIndex();
      if( index != -1 )
      {
      this.fis.SetOutputDefuz(this.Number, ( (JOutput)this.Temp).TypeDefuzzification()[index]);
      this.dfThres.setValue(((JOutput)this.Temp).GetDefuzThres());
      }
      }
      catch( Throwable t ) { MessageFIS.Error(null, t); }
      //   }
      }*/
	//------------------------------------------------------------------------------
	void jCBDisjunction_itemStateChanged(ItemEvent e)
	{
		if( e.getStateChange() == ItemEvent.SELECTED )
		{
			try
			{
				int index = this.jCBDisjunction.getSelectedIndex();
				if( index != -1 )
					this.fis.SetOutputDisjunction(this.Number, ((JOutput)this.Temp).TypeDisjunction()[index]);
			}
			catch( Throwable t ) { MessageFIS.Error(null, t); }
		}
	}
}





//------------------------------------------------------------------------------
class ComboRenderer extends JLabel implements ListCellRenderer {
	//------------------------------------------------------------------------------
	public ComboRenderer() {
		setOpaque(true);
		setBorder(new EmptyBorder(1, 1, 1, 1));
	}
	//------------------------------------------------------------------------------
	protected boolean isEnabled( int index ) { return true; }
	//------------------------------------------------------------------------------
	public Component getListCellRendererComponent( JList list,
			Object value, int index, boolean isSelected, boolean cellHasFocus) {
		if (isSelected) {
			setBackground(list.getSelectionBackground());
			setForeground(list.getSelectionForeground());
		} else {
			setBackground(list.getBackground());
			setForeground(list.getForeground());
		}
		if( this.isEnabled(index) == false ) {
			setBackground(list.getBackground());
			setForeground(UIManager.getColor("Label.disabledForeground"));
		}
		setFont(list.getFont());
		setText((value == null) ? "" : value.toString());
		return this;
	}
}



//------------------------------------------------------------------------------
class ComboListener implements ActionListener {
	JComboBox combo;
	Object currentItem;
	//------------------------------------------------------------------------------
	ComboListener(JComboBox combo) {
		this.combo  = combo;
		//combo.setSelectedIndex(0);
		currentItem = combo.getSelectedItem();
	}
	//------------------------------------------------------------------------------
	protected boolean isEnabled( int index ) { return true; }
	//------------------------------------------------------------------------------
	protected void actionPerformed() {}
	//------------------------------------------------------------------------------
	public void actionPerformed(ActionEvent e) {
		Object tempItem = combo.getSelectedItem();
		int index = combo.getSelectedIndex();
		if (!isEnabled( index )) {
			combo.setSelectedItem(currentItem);
		} else {
			currentItem = tempItem;
			this.actionPerformed();
		}
	}
}


