//***********************************************************************
//
//                              JPanel2DInferParameters.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.plaf.basic.BasicArrowButton;

import util.JDoubleMThumbSliderSpinner;
import javax.swing.JPanel;

public class JPanel2DInferParameters extends JFISPanel {

    protected JComboBoxInputs jComboBoxInputX = null;

    private JLabel jLabelAxes = null;

    private JLabel jLabelResolution = null;

    private JLabel jLabelX = null;

    private JLabel jLabelMin = null;

    private JLabel jLabelMax = null;

    private JSpinner jSpinnerXResolution = null;

    private JDoubleMThumbSliderSpinner jDoubleMThumbSliderSpinnerX = null;

    private JSpinner jSpinnerXMin = null;

    private JSpinner jSpinnerXMax = null;

    private SpinnerNumberModel spinnerNumberModelXResolution = null; //  @jve:decl-index=0:visual-constraint="500,6"

    protected JFIS fis = null;

    protected JLabel jLabelOutput = null;

    protected JComboBoxInputs jComboBoxOutput = null;

    private JMFsPanel jPanelMFX = null;

    private int previousSelectedIndexX = -1;

    protected JPanelInputsValue jPanelInputsValue = null;

    private BasicArrowButton jButtonCollapseAxes = null;

    protected BasicArrowButton jButtonCollapseOtherInputs = null;

    protected JPanel jPanelAxesSelection = null;

    /**
     * This is the default constructor
     */
    public JPanel2DInferParameters() {
        super();
        initialize();
    }

    /**
     * Initialisation de l'objet
     * @param fis objet fis
     * @param showCollapseButtons affichage des boutons de reductions des panels
     * @throws Throwable
     */
    public void init(JFIS fis, boolean showCollapseButtons) throws Throwable {
        this.fis = fis;
        this.jPanelInputsValue.init(this.fis);
        this.jComboBoxInputX.setInputs(this.fis.GetInputs());
        this.jComboBoxOutput.setInputs(this.fis.GetOutputs());
        this.jPanelInputsValue.setTitle(LocaleFIS.GetString("OtherInputs"));
        if (showCollapseButtons == true) {
            this.jButtonCollapseOtherInputs.setVisible(this.jPanelInputsValue.isVisible());
            this.jButtonCollapseAxes.setVisible(true);
        }
    }

    /**
     * This method initializes this
     * 
     * @return void
     */
    private void initialize() {
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0D;
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
        gridBagConstraints3.gridx = 1;
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.SOUTHEAST;
        gridBagConstraints3.fill = java.awt.GridBagConstraints.NONE;
        gridBagConstraints3.weightx = 0.0010D;
        gridBagConstraints3.gridy = 1;
        GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.NORTHEAST;
        gridBagConstraints2.weightx = 0.0010D;
        gridBagConstraints2.gridy = 0;
        GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.weightx = 1.0D;
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        this.setLayout(new GridBagLayout());
        this.setSize(new java.awt.Dimension(447, 179));
        this.add(getJPanelAxesSelection(), gridBagConstraints);
        this.add(getJPanelInputsValue(), gridBagConstraints1);
        this.add(getJButtonCollapseAxes(), gridBagConstraints2);
        this.add(getJButtonCollapseOtherInputs(), gridBagConstraints3);
    }

    /**
     * This method initializes jCBInputX	
     * 	
     * @return fis.JComboBoxInputs	
     */
    private JComboBoxInputs getJComboBoxInputX() {
        if (jComboBoxInputX == null) {
            jComboBoxInputX = new JComboBoxInputs();
            jComboBoxInputX.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    int selectedIndex = ((JComboBox) e.getSource()).getSelectedIndex();
                    try {
                        double[] range = fis.GetInput(selectedIndex).GetRange();
                        jDoubleMThumbSliderSpinnerX.setDoubleMinMax(range[0], range[1]);
                        jPanelMFX.SetInput(fis.GetInput(selectedIndex));
                        jPanelMFX.addCursor(range[0]);
                        jPanelMFX.addCursor(range[1]);
                        if (previousSelectedIndexX != -1)
                            jPanelInputsValue.setInputVisible(previousSelectedIndexX, true);
                        jPanelInputsValue.setInputVisible(selectedIndex, false);
                        previousSelectedIndexX = selectedIndex;
                        fireStateChanged();
                    } catch (Throwable e1) {
                        // TODO Bloc catch auto-genere
                        e1.printStackTrace();
                    }
                }
            });
        }
        return jComboBoxInputX;
    }

    /**
     * This method initializes jSXRes	
     * 	
     * @return javax.swing.JSpinner	
     */
    private JSpinner getJSpinnerXRes() {
        if (jSpinnerXResolution == null) {
            jSpinnerXResolution = new JSpinner();
            jSpinnerXResolution.setModel(getSpinnerNumberModelXResolution());
            jSpinnerXResolution.addChangeListener(new javax.swing.event.ChangeListener() {
                public void stateChanged(javax.swing.event.ChangeEvent e) {
                    fireStateChanged();
                }
            });
        }
        return jSpinnerXResolution;
    }

    /**
     * This method initializes jDouble2ThumbSliderSpinner	
     * 	
     * @return util.JDouble2ThumbSliderSpinner	
     */
    private JDoubleMThumbSliderSpinner getJDoubleMThumbSliderSpinner() {
        if (jDoubleMThumbSliderSpinnerX == null) {
            jDoubleMThumbSliderSpinnerX = new JDoubleMThumbSliderSpinner(2);
            jDoubleMThumbSliderSpinnerX.addChangeListener(new javax.swing.event.ChangeListener() {
                public void stateChanged(javax.swing.event.ChangeEvent e) {
                    jPanelMFX.setCursor(0, jDoubleMThumbSliderSpinnerX.getDoubleValue(0));
                    jPanelMFX.setCursor(1, jDoubleMThumbSliderSpinnerX.getDoubleValue(1));
                    fireStateChanged();
                }
            });
        }
        return jDoubleMThumbSliderSpinnerX;
    }

    /**
     * This method initializes jSpinnerXMin	
     * 	
     * @return javax.swing.JSpinner	
     */
    private JSpinner getJSpinnerXMin() {
        if (jSpinnerXMin == null) {
            jSpinnerXMin = this.jDoubleMThumbSliderSpinnerX.getSpinners()[0];
        }
        return jSpinnerXMin;
    }

    /**
     * This method initializes jSpinnerXMax	
     * 	
     * @return javax.swing.JSpinner	
     */
    private JSpinner getJSpinnerXMax() {
        if (jSpinnerXMax == null) {
            jSpinnerXMax = this.jDoubleMThumbSliderSpinnerX.getSpinners()[1];
        }
        return jSpinnerXMax;
    }

    /**
     * This method initializes spinnerNumberModelXResolution	
     * 	
     * @return javax.swing.SpinnerNumberModel	
     */
    private SpinnerNumberModel getSpinnerNumberModelXResolution() {
        if (spinnerNumberModelXResolution == null) {
            spinnerNumberModelXResolution = new SpinnerNumberModel(40, 10, 100, 1);
        }
        return spinnerNumberModelXResolution;
    }

    /**
     * This method initializes jComboBoxOutput	
     * 	
     * @return fis.JComboBoxInputs	
     */
    private JComboBoxInputs getJComboBoxOutput() {
        if (jComboBoxOutput == null) {
            jComboBoxOutput = new JComboBoxInputs();
            jComboBoxOutput.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    fireStateChanged();
                }
            });
        }
        return jComboBoxOutput;
    }

    /**
     * This method initializes JPaneMFX	
     * 	
     * @return fis.JMFsPanel	
     */
    private JMFsPanel getJPaneMFX() {
        if (jPanelMFX == null) {
            jPanelMFX = new JMFsPanel() {

				@Override
				public Dimension getMinimumSize() {
					return new Dimension(100, 50);
				}
			};
            jPanelMFX.setSize(new java.awt.Dimension(100, 50));
            jPanelMFX.setShowTicks(false);
        }
        return jPanelMFX;
    }

    /**
     * This method initializes jPanelInputsValue	
     * 	
     * @return fis.JPanelInputsValue	
     */
    private JPanelInputsValue getJPanelInputsValue() {
        if (jPanelInputsValue == null) {
            jPanelInputsValue = new JPanelInputsValue();
            jPanelInputsValue.addChangeListener(new javax.swing.event.ChangeListener() {
                public void stateChanged(javax.swing.event.ChangeEvent e) {
                    fireStateChanged();
                }
            });
        }
        return jPanelInputsValue;
    }

    public int selectedXInput() {
        return this.jComboBoxInputX.getSelectedIndex();
    }

    public int selectedOutput() {
        return this.jComboBoxOutput.getSelectedIndex();
    }

    public int xResolution() {
        return ((Integer) this.jSpinnerXResolution.getValue()).intValue();
    }

    public double xMin() {
        return this.jDoubleMThumbSliderSpinnerX.getDoubleValue(0);
    }

    public double xMax() {
        return this.jDoubleMThumbSliderSpinnerX.getDoubleValue(1);
    }

    public double[] getInputsValues() {
        return this.jPanelInputsValue.getValues();
    }

    /* (non-Javadoc)
     * @see fis.JFISPanel#fireStateChanged()
     */
    protected void fireStateChanged() {
        if ((this.jComboBoxInputX.getSelectedIndex() != -1) && (this.jComboBoxOutput.getSelectedIndex() != -1))
            super.fireStateChanged();
    }

    /**
     * This method initializes jButtonCollapseAxes	
     * 	
     * @return javax.swing.JButton	
     */
    private BasicArrowButton getJButtonCollapseAxes() {
        if (jButtonCollapseAxes == null) {
            jButtonCollapseAxes = new BasicArrowButton(BasicArrowButton.NORTH) {
                public Dimension getMinimumSize() {
                    return this.getPreferredSize();
                }
            };
            jButtonCollapseAxes.setToolTipText(LocaleFIS.GetString("HideAxesSelection"));
            jButtonCollapseAxes.setVisible(false);
            jButtonCollapseAxes.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    int direction = jButtonCollapseAxes.getDirection();
                    jPanelAxesSelection.setVisible(direction == BasicArrowButton.NORTH ? false : true);
                    jButtonCollapseAxes.setDirection(direction == BasicArrowButton.NORTH ? BasicArrowButton.SOUTH : BasicArrowButton.NORTH);
                    jButtonCollapseAxes.setToolTipText(direction == BasicArrowButton.NORTH ? LocaleFIS.GetString("ShowAxesSelection") : LocaleFIS.GetString("HideAxesSelection"));
                    fireComponentCollapsed();
                }
            });
        }
        return jButtonCollapseAxes;
    }

    /**
     * This method initializes jButtonCollapseOtherInputs	
     * 	
     * @return javax.swing.JButton	
     */
    private BasicArrowButton getJButtonCollapseOtherInputs() {
        if (jButtonCollapseOtherInputs == null) {
            jButtonCollapseOtherInputs = new BasicArrowButton(BasicArrowButton.NORTH) {
                public Dimension getMinimumSize() {
                    return this.getPreferredSize();
                }
            };
            jButtonCollapseOtherInputs.setToolTipText(LocaleFIS.GetString("HideInputs"));
            jButtonCollapseOtherInputs.setVisible(false);
            jButtonCollapseOtherInputs.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    int direction = jButtonCollapseOtherInputs.getDirection();
                    jPanelInputsValue.setVisible(direction == BasicArrowButton.NORTH ? false : true);
                    jButtonCollapseOtherInputs.setDirection(direction == BasicArrowButton.NORTH ? BasicArrowButton.SOUTH : BasicArrowButton.NORTH);
                    jButtonCollapseOtherInputs.setToolTipText(direction == BasicArrowButton.NORTH ? LocaleFIS.GetString("ShowInputs") : LocaleFIS.GetString("HideInputs"));
                    fireComponentCollapsed();
                }
            });
        }
        return jButtonCollapseOtherInputs;
    }

    public interface ComponentCollapsedListener extends CustomListener {
        public void componentCollapsed();
    }

    public void fireComponentCollapsed() {
        Object[] listeners = listenerList.getListenerList();
        for (int i = listeners.length - 2; i >= 0; i -= 2) {
            if (listeners[i] == ComponentCollapsedListener.class) {
                ((ComponentCollapsedListener) listeners[i + 1]).componentCollapsed();
            }
        }
    }

    /**
     * This method initializes jPanelAxesSelection	
     * 	
     * @return javax.swing.JPanel	
     */
    protected JPanel getJPanelAxesSelection() {
        if (jPanelAxesSelection == null) {
            jPanelAxesSelection = new JPanel();
            jPanelAxesSelection.setLayout(new GridBagLayout());
            GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
            gridBagConstraints12.gridx = 5;
            gridBagConstraints12.gridheight = 2;
            gridBagConstraints12.fill = java.awt.GridBagConstraints.NONE;
            gridBagConstraints12.insets = new java.awt.Insets(5, 5, 0, 5);
            gridBagConstraints12.gridy = 1;
            GridBagConstraints gridBagConstraints21 = new GridBagConstraints();
            gridBagConstraints21.fill = java.awt.GridBagConstraints.NONE;
            gridBagConstraints21.gridy = 5;
            gridBagConstraints21.insets = new java.awt.Insets(5, 5, 5, 0);
            gridBagConstraints21.gridx = 1;
            GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
            gridBagConstraints11.gridx = 0;
            gridBagConstraints11.insets = new java.awt.Insets(5, 5, 5, 0);
            gridBagConstraints11.gridy = 5;
            jLabelOutput = new JLabel();
            jLabelOutput.setText("Y");
            GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
            gridBagConstraints9.gridx = 4;
            gridBagConstraints9.insets = new java.awt.Insets(5, 5, 0, 0);
            gridBagConstraints9.ipadx = 20;
            gridBagConstraints9.gridy = 1;
            GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
            gridBagConstraints8.gridx = 3;
            gridBagConstraints8.insets = new java.awt.Insets(5, 5, 0, 0);
            gridBagConstraints8.ipadx = 20;
            gridBagConstraints8.gridy = 1;
            GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
            gridBagConstraints7.fill = java.awt.GridBagConstraints.HORIZONTAL;
            gridBagConstraints7.gridy = 2;
            gridBagConstraints7.gridwidth = 2;
            gridBagConstraints7.insets = new java.awt.Insets(5, 5, 0, 5);
            gridBagConstraints7.gridx = 3;
            GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
            gridBagConstraints6.gridx = 2;
            gridBagConstraints6.gridheight = 4;
            gridBagConstraints6.gridy = 1;
            GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
            gridBagConstraints5.gridx = 4;
            gridBagConstraints5.insets = new java.awt.Insets(5, 5, 0, 0);
            gridBagConstraints5.gridy = 0;
            jLabelMax = new JLabel();
            jLabelMax.setText(LocaleFIS.GetString("Max"));
            GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
            gridBagConstraints4.gridx = 3;
            gridBagConstraints4.insets = new java.awt.Insets(5, 5, 0, 0);
            gridBagConstraints4.gridy = 0;
            jLabelMin = new JLabel();
            jLabelMin.setText(LocaleFIS.GetString("Min"));
            GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
            gridBagConstraints3.gridx = 0;
            gridBagConstraints3.insets = new java.awt.Insets(0, 5, 0, 0);
            gridBagConstraints3.gridheight = 2;
            gridBagConstraints3.gridy = 1;
            jLabelX = new JLabel();
            jLabelX.setText("X");
            GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
            gridBagConstraints2.gridx = 2;
            gridBagConstraints2.insets = new java.awt.Insets(5, 5, 0, 0);
            gridBagConstraints2.gridy = 0;
            jLabelResolution = new JLabel();
            jLabelResolution.setText(LocaleFIS.GetString("Resolution"));
            GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
            gridBagConstraints1.gridx = 1;
            gridBagConstraints1.insets = new java.awt.Insets(5, 5, 0, 0);
            gridBagConstraints1.gridy = 0;
            jLabelAxes = new JLabel();
            jLabelAxes.setText(LocaleFIS.GetString("Axes"));
            GridBagConstraints gridBagConstraints = new GridBagConstraints();
            gridBagConstraints.fill = java.awt.GridBagConstraints.NONE;
            gridBagConstraints.gridy = 1;
            gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 0);
            gridBagConstraints.gridheight = 2;
            gridBagConstraints.gridx = 1;
            jPanelAxesSelection.add(getJDoubleMThumbSliderSpinner(), gridBagConstraints7);
            jPanelAxesSelection.add(getJSpinnerXMin(), gridBagConstraints8);
            jPanelAxesSelection.add(getJSpinnerXMax(), gridBagConstraints9);
            jPanelAxesSelection.add(getJPaneMFX(), gridBagConstraints12);
            jPanelAxesSelection.add(getJComboBoxInputX(), gridBagConstraints);
            jPanelAxesSelection.add(jLabelAxes, gridBagConstraints1);
            jPanelAxesSelection.add(jLabelResolution, gridBagConstraints2);
            jPanelAxesSelection.add(jLabelX, gridBagConstraints3);
            jPanelAxesSelection.add(jLabelMin, gridBagConstraints4);
            jPanelAxesSelection.add(jLabelMax, gridBagConstraints5);
            jPanelAxesSelection.add(getJSpinnerXRes(), gridBagConstraints6);
            jPanelAxesSelection.add(jLabelOutput, gridBagConstraints11);
            jPanelAxesSelection.add(getJComboBoxOutput(), gridBagConstraints21);
        }
        return jPanelAxesSelection;
    }

}
