//***********************************************************************
//
//                              JPanelInputValue.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JLabel;
import javax.swing.JSpinner;

import util.JDoubleMThumbSliderSpinner;

public class JPanelInputValue extends JFISPanel {

    private JLabel jLabelName = null;
    private JMFsPanel jPanelMFs = null;
    private JDoubleMThumbSliderSpinner jDoubleMThumbSliderSpinner = null;
    private JSpinner jSpinner = null;
    private JInput input = null;

    /**
     * This is the default constructor
     */
    public JPanelInputValue() {
        super();
        initialize();
    }
    
    public void init(JInput input) throws Throwable {
        this.input = input;
        double []range = this.input.GetRange();
        this.jLabelName.setText(this.input.GetName());
        this.jDoubleMThumbSliderSpinner.setDoubleMinMax(range[0], range[1]);
        this.jPanelMFs.SetInput(this.input);
        this.jPanelMFs.addCursor(this.jDoubleMThumbSliderSpinner.getDoubleValue(0));
    }
    
    public double getValue() {
        return this.jDoubleMThumbSliderSpinner.getDoubleValue(0);
    }

    /**
     * This method initializes this
     * 
     * @return void
     */
    private void initialize() {
        GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
        gridBagConstraints4.gridx = 1;
        gridBagConstraints4.weightx = 1.0D;
        gridBagConstraints4.ipadx = 30;
        gridBagConstraints4.insets = new java.awt.Insets(0,0,5,0);
        gridBagConstraints4.gridy = 0;
        GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
        gridBagConstraints3.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints3.gridy = 2;
        gridBagConstraints3.weightx = 1.0;
        gridBagConstraints3.gridwidth = 2;
        gridBagConstraints3.insets = new java.awt.Insets(5,0,0,0);
        gridBagConstraints3.gridx = 0;
        GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.weightx = 1.0D;
        gridBagConstraints1.gridy = 1;
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.weightx = 0.0D;
        gridBagConstraints.insets = new java.awt.Insets(0,0,5,5);
        gridBagConstraints.gridy = 0;
        jLabelName = new JLabel();
        jLabelName.setText("name");
        this.setLayout(new GridBagLayout());
        this.setSize(new java.awt.Dimension(100,100));
        this.add(jLabelName, gridBagConstraints);
        this.add(getJMFsPanel(), gridBagConstraints1);
        this.add(getJDoubleMThumbSliderSpinner(), gridBagConstraints3);
        this.add(getJSpinner(), gridBagConstraints4);
    }

    /**
     * This method initializes JMFsPanel	
     * 	
     * @return fis.JMFsPanel	
     */
    private JMFsPanel getJMFsPanel() {
        if (jPanelMFs == null) {
            jPanelMFs = new JMFsPanel();
            jPanelMFs.setShowTicks(false);
            jPanelMFs.setSize(new java.awt.Dimension(100,50));
        }
        return jPanelMFs;
    }

    /**
     * This method initializes jDoubleMThumbSliderSpinner3	
     * 	
     * @return util.JDoubleMThumbSliderSpinner	
     */
    private JDoubleMThumbSliderSpinner getJDoubleMThumbSliderSpinner() {
        if (jDoubleMThumbSliderSpinner == null) {
            jDoubleMThumbSliderSpinner = new JDoubleMThumbSliderSpinner(1);
            jDoubleMThumbSliderSpinner.setPreferredSize(new java.awt.Dimension(50,16));
            jDoubleMThumbSliderSpinner.addChangeListener(new javax.swing.event.ChangeListener() {   
            	public void stateChanged(javax.swing.event.ChangeEvent e) {    
                    jPanelMFs.setCursor(0, jDoubleMThumbSliderSpinner.getDoubleValue(0));
                    fireStateChanged();
            	}
            
            });
        }
        return jDoubleMThumbSliderSpinner;
    }

    /**
     * This method initializes jSpinner	
     * 	
     * @return javax.swing.JSpinner	
     */
    private JSpinner getJSpinner() {
        if (jSpinner == null) {
            jSpinner = this.jDoubleMThumbSliderSpinner.getSpinners()[0];
        }
        return jSpinner;
    }

    /**
     * Modifie l'objet pour selectionner les breakpoints de l'entree
     * @param breakpoints
     * @throws Throwable
     */
    public void setBreakpoints(boolean breakpoints) throws Throwable {
        if( breakpoints == true )
            this.jDoubleMThumbSliderSpinner.setSpinnerListModel(this.input.GetBreakPoints());
        else
            this.jDoubleMThumbSliderSpinner.setDefaultSpinnerModel();
        this.jPanelMFs.setCursor(0, jDoubleMThumbSliderSpinner.getDoubleValue(0));
        this.fireStateChanged();
    }
    
}
