//***********************************************************************
//
//                              JPanelInputsValue.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class JPanelInputsValue extends JFISPanel {

    private JScrollPane jScrollPane = null;

    private JPanel jPanel = null; //  @jve:decl-index=0:visual-constraint="343,30"

    private JCheckBox jCheckBoxBreakpoints = null;

    private JPanelInputValue[] jPanelInputValues = null;

    private JFIS fis = null;

    private boolean blockFireStateChanged = false; // permet de bloquer le fireStateChanged de l'objet

    private int panelOrientation = SwingConstants.HORIZONTAL;

    /**
     * Constructeur par defaut
     */
    public JPanelInputsValue() {
        this(SwingConstants.HORIZONTAL);
    }

    /**
     * Constructeur avec orientation du panel
     * @param panelOrientation orientation du panel SwingConstants.VERTICAL ou SwingConstants.HORIZONTAL
     */
    public JPanelInputsValue(int panelOrientation) {
        super();
        if ((panelOrientation != SwingConstants.VERTICAL) && (panelOrientation != SwingConstants.HORIZONTAL))
            throw new IllegalArgumentException("panelOrientation must be one of: SwingConstants.VERTICAL, SwingConstants.HORIZONTAL");
        this.panelOrientation = panelOrientation;
        initialize();
    }

    public void setInputVisible(int inputNumber, boolean visible) {
        this.jPanelInputValues[inputNumber].setVisible(visible);
        this.setVisible(!(this.getNumberInputsVisible() == 0));
    }

    public double[] getValues() {
        double[] values = new double[this.jPanelInputValues.length];
        for (int i = 0; i < this.jPanelInputValues.length; i++)
            values[i] = this.jPanelInputValues[i].getValue();
        return values;
    }

    public void init(JFIS fis) throws Throwable {
        this.fis = fis;
        this.jPanelInputValues = new JPanelInputValue[this.fis.NbInputs()];
        for (int i = 0; i < this.fis.NbInputs(); i++) {
            JPanelInputValue jpiv = new JPanelInputValue();
            this.jPanelInputValues[i] = jpiv;
            this.jPanel.add(jpiv, new GridBagConstraints(this.panelOrientation == SwingConstants.HORIZONTAL ? i : 0, this.panelOrientation == SwingConstants.HORIZONTAL ? 1 : 1 + i, 1, 1, 0, 0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 0, 5), 0, 0));
            jpiv.init(this.fis.GetInput(i));
            jpiv.addChangeListener(new ChangeListener() {
                public void stateChanged(javax.swing.event.ChangeEvent e) {
                    fireStateChanged();
                }
            });
            if (this.fis.GetInput(i).GetActive() == false)
                jpiv.setVisible(false);
        }
        this.setVisible(!(this.getNumberInputsVisible() == 0));
        
        // redessine les panels apres un mouvement des scrollbars
        this.jScrollPane.getHorizontalScrollBar().getModel().addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent arg0) {
                jScrollPane.repaint();
            }
        });
        this.jScrollPane.getVerticalScrollBar().getModel().addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent arg0) {
                jScrollPane.repaint();
            }
        });
        // modification de la contrainte de positionnement de jCheckBoxBreakpoints en fonction du nombre d'entrees
        GridBagConstraints gbc = ((GridBagLayout) this.jPanel.getLayout()).getConstraints(this.jCheckBoxBreakpoints);
        gbc.gridwidth = this.fis.NbInputs();
        ((GridBagLayout) this.jPanel.getLayout()).setConstraints(this.jCheckBoxBreakpoints, gbc);
    }

    /**
     * This method initializes this
     * 
     * @return void
     */
    private void initialize() {
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.gridx = 0;
        this.setLayout(new GridBagLayout());
        this.setSize(121, 68);
        this.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "inputs", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
        this.add(getJScrollPane(), gridBagConstraints);
    }
    
    private int getNumberInputsVisible() {
        int numberInputsVisible = 0;
        for(int i=0; i<this.jPanelInputValues.length ; i++)
            if(this.jPanelInputValues[i].isVisible() == true)
                numberInputsVisible++;
        return numberInputsVisible;
    }

    /**
     * This method initializes jScrollPane	
     * 	
     * @return javax.swing.JScrollPane	
     */
    private JScrollPane getJScrollPane() {
        if (jScrollPane == null) {
            jScrollPane = new JScrollPane(this.getJPanel());
        }
        return jScrollPane;
    }

    /**
     * This method initializes jPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getJPanel() {
        if (jPanel == null) {
            GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
            gridBagConstraints1.gridx = 0;
            gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints1.gridy = 0;
            jPanel = new JPanel();
            jPanel.setLayout(new GridBagLayout());
            jPanel.add(getJCheckBoxBreakpoints(), gridBagConstraints1);
        }
        return jPanel;
    }

    /* (non-Javadoc)
     * @see javax.swing.JComponent#getMinimumSize()
     */
    public Dimension getMinimumSize() {
        return (this.jPanelInputValues == null || this.jPanelInputValues.length == 0) ? super.getMinimumSize() : new Dimension(50, this.jPanelInputValues[0].getMinimumSize().height + super.getMinimumSize().height + this.jCheckBoxBreakpoints.getMinimumSize().height + 5);
    }

    /**
     * This method initializes jCheckBox	
     * 	
     * @return javax.swing.JCheckBox	
     */
    private JCheckBox getJCheckBoxBreakpoints() {
        if (jCheckBoxBreakpoints == null) {
            jCheckBoxBreakpoints = new JCheckBox();
            jCheckBoxBreakpoints.setText("breakpoints");
            jCheckBoxBreakpoints.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    try {
                        blockFireStateChanged = true;
                        for (int i = 0; i < fis.NbInputs(); i++) {
                            jPanelInputValues[i].setBreakpoints(jCheckBoxBreakpoints.isSelected());
                        }
                    } catch (Throwable e1) {
                        // TODO Bloc catch auto-genere
                        e1.printStackTrace();
                    } finally {
                        blockFireStateChanged = false;
                        fireStateChanged();
                    }
                }
            });
        }
        return jCheckBoxBreakpoints;
    }

    /* (non-Javadoc)
     * @see fis.JFISPanel#fireStateChanged()
     */
    protected void fireStateChanged() {
        if (this.blockFireStateChanged == false)
            super.fireStateChanged();
    }

    /**
     * Modifie le titre du panel
     * @param title titre du panel
     */
    public void setTitle(String title) {
        ((TitledBorder) this.getBorder()).setTitle(title);
    }

}
