//***********************************************************************
//
//                              JPanelOLS.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.border.TitledBorder;

import util.skt.swing.border.ComponentTitledBorder;

public class JPanelOLS extends JPanel {

    private JPanel jPanelFIS = null;

    private JLabel jLabelOutput = null;

    private JComboBoxInputs jComboBoxOutputs = null;

    private JCheckBox jCheckBoxKeepExistingRules = null;

    private JFIS fis = null;

    private JPanel jPanelGeneratePartitions = null;

    private JLabel jLabelStandardDeviation = null;

    private IntegerField integerFieldStandardDeviation = null;

    private JLabel jLabelStandardDeviationUnit = null;

    private JLabel jLabelWarningGeneratePartitions = null;

    private JPanel jPanelFirstPass = null;

    private IntegerField integerFieldMaximumNumberOfRules = null;

    private DoubleField doubleFieldTotalUnexplainedVariance = null;

    private JPanel jPanelSecondPass = null;

    private JLabel jLabelTestDataFile = null;

    private JTextField jTextFieldTestDataFile = null;

    private JButton jButtonTestDataFile = null;

    private JPanelOutputVocabularyReduction jPanelOutputVocabularyReduction = null;

    private JCheckBox jCheckBoxRemoveTemporaryFiles = null;

    private JCheckBox jCheckBoxOutputFuzzification = null;

    private JCheckBox jCheckBoxOutputVocabularyReduction = new JCheckBox(LocaleFIS.GetString("OutputVocabularyReduction"), true);

    private JLabel jLabelConjunction = null;

    private JComboBox jComboBoxConjunction = null;

    private DefaultComboBoxModel defaultComboBoxModelConjunction = null;

    private JRadioButton jRadioButtonGenerateFISWithoutRules = null;

    private JRadioButton jRadioButtonGenerateGaussianPartition = null;

    private ButtonGroup buttonGroupGeneratePartition = null;

    private JLabel jLabelMaximumNumberOfRules = null;

    private JLabel jLabelTotalUnexplainedVariance = null;

    /**
     * This is the default constructor
     * @param fis TODO
     * @param dataFile TODO
     */
    public JPanelOLS(JFIS fis, JDataFile dataFile) throws Throwable {
        super();
        this.fis = fis;
        initialize();
        if (this.fis == null) {
            this.jPanelFIS.setVisible(false);
        } else {
            this.jPanelGeneratePartitions.setVisible(false);
            this.jComboBoxOutputs.setInputs(this.fis.GetOutputs());
        }
        // init les fichiers de donnees
        this.jTextFieldTestDataFile.setText(dataFile.FileName());
        // cache la saisie du seuil dans le panel de reduction du vocabulaire, utilisation du seuil saisie dans la seconde passe
        this.jPanelOutputVocabularyReduction.hideBlankTreshold();
    }

    /**
     * This method initializes this
     * 
     * @return void
     */
    private void initialize() {
        GridBagConstraints gridBagConstraints81 = new GridBagConstraints();
        gridBagConstraints81.gridx = 0;
        gridBagConstraints81.insets = new java.awt.Insets(5, 5, 0, 5);
        gridBagConstraints81.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints81.gridy = 5;
        GridBagConstraints gridBagConstraints61 = new GridBagConstraints();
        gridBagConstraints61.gridx = 0;
        gridBagConstraints61.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints61.insets = new java.awt.Insets(0, 5, 0, 5);
        gridBagConstraints61.gridy = 6;
        GridBagConstraints gridBagConstraints110 = new GridBagConstraints();
        gridBagConstraints110.gridx = 0;
        gridBagConstraints110.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints110.weightx = 1.0D;
        gridBagConstraints110.insets = new java.awt.Insets(5, 2, 0, 2);
        gridBagConstraints110.gridy = 4;
        GridBagConstraints gridBagConstraints51 = new GridBagConstraints();
        gridBagConstraints51.gridx = 0;
        gridBagConstraints51.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints51.weightx = 1.0D;
        gridBagConstraints51.insets = new java.awt.Insets(5, 0, 0, 0);
        gridBagConstraints51.gridy = 3;
        GridBagConstraints gridBagConstraints41 = new GridBagConstraints();
        gridBagConstraints41.gridx = 0;
        gridBagConstraints41.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints41.weightx = 1.0D;
        gridBagConstraints41.insets = new java.awt.Insets(5, 0, 0, 0);
        gridBagConstraints41.gridy = 2;
        GridBagConstraints gridBagConstraints31 = new GridBagConstraints();
        gridBagConstraints31.gridx = 0;
        gridBagConstraints31.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints31.weightx = 1.0D;
        gridBagConstraints31.gridy = 1;
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0D;
        gridBagConstraints.gridy = 0;
        this.setLayout(new GridBagLayout());
        this.setSize(300, 514);
        this.add(getJPanelFIS(), gridBagConstraints);
        this.add(getJPanelGeneratePartitions(), gridBagConstraints31);
        this.add(getJPanelFirstPass(), gridBagConstraints41);
        this.add(getJPanelSecondPass(), gridBagConstraints51);
        this.add(getJPanelOutputVocabularyReduction(), gridBagConstraints110);
        this.add(getJCheckBoxRemoveTemporaryFiles(), gridBagConstraints61);
        this.add(getJCheckBoxOutputFuzzification(), gridBagConstraints81);
    }

    /**
     * This method initializes jPanelFIS	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getJPanelFIS() {
        if (jPanelFIS == null) {
            GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
            gridBagConstraints3.gridx = 0;
            gridBagConstraints3.gridwidth = 2;
            gridBagConstraints3.insets = new java.awt.Insets(0, 5, 0, 5);
            gridBagConstraints3.gridy = 1;
            GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
            gridBagConstraints2.gridx = 0;
            gridBagConstraints2.insets = new java.awt.Insets(5, 5, 0, 0);
            gridBagConstraints2.anchor = java.awt.GridBagConstraints.EAST;
            gridBagConstraints2.gridy = 0;
            GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
            gridBagConstraints1.fill = java.awt.GridBagConstraints.NONE;
            gridBagConstraints1.gridy = 0;
            gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints1.insets = new java.awt.Insets(5, 5, 0, 5);
            gridBagConstraints1.gridx = 1;
            jLabelOutput = new JLabel();
            jLabelOutput.setText(LocaleFIS.GetString("Output") + ":");
            jPanelFIS = new JPanel();
            jPanelFIS.setLayout(new GridBagLayout());
            jPanelFIS.add(jLabelOutput, gridBagConstraints2);
            jPanelFIS.add(getJComboBoxOutputs(), gridBagConstraints1);
            jPanelFIS.add(getJCheckBoxKeepExistingRules(), gridBagConstraints3);
        }
        return jPanelFIS;
    }

    /**
     * This method initializes jComboBoxOutputs	
     * 	
     * @return fis.JComboBoxInputs	
     */
    private JComboBoxInputs getJComboBoxOutputs() {
        if (jComboBoxOutputs == null) {
            jComboBoxOutputs = new JComboBoxInputs() {
                protected boolean isIndexEnabled(int index) throws Throwable {
                    boolean enabled = super.isIndexEnabled(index);
                    if (index != -1) {
                        JOutput output = fis.GetOutput(index);
                        if ((output.GetClassif() == true) && (output.GetNature() == JOutput.CRISP))
                            return false;
                        if (output.GetNature() == JOutput.FUZZY)
                            return false;
                    }
                    return enabled;
                }

                protected void clickOnDisabledItem(int index) {
                    try {
                        if (index != -1) {
                            JOutput output = fis.GetOutput(index);
                            if (((output.GetClassif() == true) && (output.GetNature() == JOutput.CRISP)) || (output.GetNature() == JOutput.FUZZY))
                                MessageFIS.Information(JPanelOLS.this, LocaleFIS.GetString("ApplyOLSOnlyWithNoClassifCrispOutput"));
                        }
                    } catch (Throwable e) {
                        // TODO Bloc catch auto-genere
                        e.printStackTrace();
                    }
                }
            };
        }
        return jComboBoxOutputs;
    }

    /**
     * This method initializes jCheckBoxKeepExistingRules	
     * 	
     * @return javax.swing.JCheckBox	
     */
    private JCheckBox getJCheckBoxKeepExistingRules() {
        if (jCheckBoxKeepExistingRules == null) {
            jCheckBoxKeepExistingRules = new JCheckBox();
            jCheckBoxKeepExistingRules.setText(LocaleFIS.GetString("KeepExistingRules"));
            jCheckBoxKeepExistingRules.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    setEnabledPanelFirstPass(!jCheckBoxKeepExistingRules.isSelected());
                }
            });
        }
        return jCheckBoxKeepExistingRules;
    }

    /**
     * This method initializes jPanelGeneratePartitions	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getJPanelGeneratePartitions() {
        if (jPanelGeneratePartitions == null) {
            GridBagConstraints gridBagConstraints18 = new GridBagConstraints();
            gridBagConstraints18.gridx = 0;
            gridBagConstraints18.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints18.gridwidth = 3;
            gridBagConstraints18.gridy = 1;
            GridBagConstraints gridBagConstraints17 = new GridBagConstraints();
            gridBagConstraints17.gridx = 0;
            gridBagConstraints17.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints17.gridwidth = 3;
            gridBagConstraints17.gridy = 0;
            GridBagConstraints gridBagConstraints16 = new GridBagConstraints();
            gridBagConstraints16.fill = java.awt.GridBagConstraints.NONE;
            gridBagConstraints16.gridy = 3;
            gridBagConstraints16.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints16.insets = new java.awt.Insets(5, 5, 0, 5);
            gridBagConstraints16.gridwidth = 2;
            gridBagConstraints16.gridx = 1;
            GridBagConstraints gridBagConstraints15 = new GridBagConstraints();
            gridBagConstraints15.gridx = 0;
            gridBagConstraints15.anchor = java.awt.GridBagConstraints.EAST;
            gridBagConstraints15.insets = new java.awt.Insets(5, 5, 0, 0);
            gridBagConstraints15.gridy = 3;
            jLabelConjunction = new JLabel();
            jLabelConjunction.setText(LocaleFIS.GetString("Conjunction") + ":");
            GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
            gridBagConstraints7.gridx = 0;
            gridBagConstraints7.gridwidth = 3;
            gridBagConstraints7.insets = new java.awt.Insets(5, 0, 0, 0);
            gridBagConstraints7.gridy = 4;
            jLabelWarningGeneratePartitions = new JLabel();
            jLabelWarningGeneratePartitions.setText(LocaleFIS.GetString("WarningGeneratedPartitionsAreNotFF"));
            GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
            gridBagConstraints6.gridx = 2;
            gridBagConstraints6.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints6.insets = new java.awt.Insets(5, 0, 0, 0);
            gridBagConstraints6.gridy = 2;
            jLabelStandardDeviationUnit = new JLabel();
            jLabelStandardDeviationUnit.setText("% max-min");
            GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
            gridBagConstraints5.fill = java.awt.GridBagConstraints.NONE;
            gridBagConstraints5.gridy = 2;
            gridBagConstraints5.ipadx = 40;
            gridBagConstraints5.insets = new java.awt.Insets(5, 5, 0, 5);
            gridBagConstraints5.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints5.gridx = 1;
            GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
            gridBagConstraints4.gridx = 0;
            gridBagConstraints4.anchor = java.awt.GridBagConstraints.EAST;
            gridBagConstraints4.insets = new java.awt.Insets(5, 0, 0, 0);
            gridBagConstraints4.gridy = 2;
            jLabelStandardDeviation = new JLabel();
            jLabelStandardDeviation.setText(LocaleFIS.GetString("StandardDeviation") + ":");
            jPanelGeneratePartitions = new JPanel();
            jPanelGeneratePartitions.setLayout(new GridBagLayout());
            jPanelGeneratePartitions.setBorder(javax.swing.BorderFactory.createTitledBorder(null, LocaleFIS.GetString("GeneratePartitions"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
            jPanelGeneratePartitions.add(jLabelStandardDeviation, gridBagConstraints4);
            jPanelGeneratePartitions.add(getIntegerFieldStandardDeviation(), gridBagConstraints5);
            jPanelGeneratePartitions.add(jLabelStandardDeviationUnit, gridBagConstraints6);
            jPanelGeneratePartitions.add(jLabelWarningGeneratePartitions, gridBagConstraints7);
            jPanelGeneratePartitions.add(jLabelConjunction, gridBagConstraints15);
            jPanelGeneratePartitions.add(getJComboBoxConjunction(), gridBagConstraints16);
            jPanelGeneratePartitions.add(getJRadioButtonGenerateFISWithoutRules(), gridBagConstraints17);
            jPanelGeneratePartitions.add(getJRadioButtonGenerateGaussianPartition(), gridBagConstraints18);
        }
        return jPanelGeneratePartitions;
    }

    /**
     * This method initializes integerFieldStandardDeviation	
     * 	
     * @return fis.IntegerField	
     */
    private IntegerField getIntegerFieldStandardDeviation() {
        if (integerFieldStandardDeviation == null) {
            integerFieldStandardDeviation = new IntegerField();
            integerFieldStandardDeviation.setValue(5);
        }
        return integerFieldStandardDeviation;
    }

    /**
     * This method initializes jPanelFirstPass	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getJPanelFirstPass() {
        if (jPanelFirstPass == null) {
            GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
            gridBagConstraints9.gridx = 0;
            gridBagConstraints9.anchor = java.awt.GridBagConstraints.EAST;
            gridBagConstraints9.insets = new java.awt.Insets(5, 5, 5, 5);
            gridBagConstraints9.gridy = 1;
            jLabelTotalUnexplainedVariance = new JLabel();
            jLabelTotalUnexplainedVariance.setText(LocaleFIS.GetString("TotalUnexplainedVariance"));
            GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
            gridBagConstraints8.gridx = 0;
            gridBagConstraints8.anchor = java.awt.GridBagConstraints.EAST;
            gridBagConstraints8.insets = new java.awt.Insets(0, 5, 0, 5);
            gridBagConstraints8.gridy = 0;
            jLabelMaximumNumberOfRules = new JLabel();
            jLabelMaximumNumberOfRules.setText(LocaleFIS.GetString("MaximumNumberOfRules"));
            GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
            gridBagConstraints11.fill = java.awt.GridBagConstraints.NONE;
            gridBagConstraints11.gridy = 1;
            gridBagConstraints11.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints11.ipadx = 40;
            gridBagConstraints11.insets = new java.awt.Insets(5, 0, 5, 5);
            gridBagConstraints11.gridx = 1;
            GridBagConstraints gridBagConstraints10 = new GridBagConstraints();
            gridBagConstraints10.fill = java.awt.GridBagConstraints.NONE;
            gridBagConstraints10.gridy = 0;
            gridBagConstraints10.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints10.ipadx = 40;
            gridBagConstraints10.insets = new java.awt.Insets(0, 0, 0, 5);
            gridBagConstraints10.gridx = 1;
            jPanelFirstPass = new JPanel();
            jPanelFirstPass.setLayout(new GridBagLayout());
            jPanelFirstPass.setBorder(javax.swing.BorderFactory.createTitledBorder(null, LocaleFIS.GetString("FirstPass") + " - " + LocaleFIS.GetString("RuleSelection"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
            jPanelFirstPass.add(getIntegerFieldMaximumNumberOfRules(), gridBagConstraints10);
            jPanelFirstPass.add(getDoubleFieldTotalUnexplainedVariance(), gridBagConstraints11);
            jPanelFirstPass.add(jLabelMaximumNumberOfRules, gridBagConstraints8);
            jPanelFirstPass.add(jLabelTotalUnexplainedVariance, gridBagConstraints9);
        }
        return jPanelFirstPass;
    }

    /**
     * This method initializes integerFieldMaximumNumberOfRules	
     * 	
     * @return fis.IntegerField	
     */
    private IntegerField getIntegerFieldMaximumNumberOfRules() {
        if (integerFieldMaximumNumberOfRules == null) {
            integerFieldMaximumNumberOfRules = new IntegerField();
            integerFieldMaximumNumberOfRules.setValue(100);
        }
        return integerFieldMaximumNumberOfRules;
    }

    /**
     * This method initializes doubleFieldTotalUnexplainedVariance	
     * 	
     * @return fis.DoubleField	
     */
    private DoubleField getDoubleFieldTotalUnexplainedVariance() {
        if (doubleFieldTotalUnexplainedVariance == null) {
            doubleFieldTotalUnexplainedVariance = new DoubleField();
            doubleFieldTotalUnexplainedVariance.setValue(0.05D);
        }
        return doubleFieldTotalUnexplainedVariance;
    }

    private void setEnabledPanelFirstPass(boolean enabled) {
        ((TitledBorder) this.jPanelFirstPass.getBorder()).setTitleColor(enabled == false ? UIManager.getColor("Label.disabledForeground") : UIManager.getColor("Label.foreground"));
        this.repaint();
        this.jLabelMaximumNumberOfRules.setEnabled(enabled);
        this.jLabelTotalUnexplainedVariance.setEnabled(enabled);
        this.integerFieldMaximumNumberOfRules.setEnabled(enabled);
        this.doubleFieldTotalUnexplainedVariance.setEnabled(enabled);
    }

    /**
     * This method initializes jPanelSecondPass	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getJPanelSecondPass() {
        if (jPanelSecondPass == null) {
            GridBagConstraints gridBagConstraints14 = new GridBagConstraints();
            gridBagConstraints14.gridx = 2;
            gridBagConstraints14.ipady = -10;
            gridBagConstraints14.insets = new java.awt.Insets(0, 0, 5, 5);
            gridBagConstraints14.gridy = 0;
            GridBagConstraints gridBagConstraints13 = new GridBagConstraints();
            gridBagConstraints13.fill = java.awt.GridBagConstraints.HORIZONTAL;
            gridBagConstraints13.gridy = 0;
            gridBagConstraints13.weightx = 1.0D;
            gridBagConstraints13.insets = new java.awt.Insets(0, 5, 5, 5);
            gridBagConstraints13.ipadx = 40;
            gridBagConstraints13.gridx = 1;
            GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
            gridBagConstraints12.gridx = 0;
            gridBagConstraints12.insets = new java.awt.Insets(0, 5, 0, 0);
            gridBagConstraints12.anchor = java.awt.GridBagConstraints.EAST;
            gridBagConstraints12.gridy = 0;
            jLabelTestDataFile = new JLabel();
            jLabelTestDataFile.setText(LocaleFIS.GetString("TestDataFile") + ":");
            TitledBorder titledBorder = javax.swing.BorderFactory.createTitledBorder(null, LocaleFIS.GetString("SecondPass") + " - " + LocaleFIS.GetString("OptimizeConclusions"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null);
            jPanelSecondPass = new JPanel();
            jPanelSecondPass.setLayout(new GridBagLayout());
            jPanelSecondPass.setBorder(titledBorder);
            jPanelSecondPass.add(jLabelTestDataFile, gridBagConstraints12);
            jPanelSecondPass.add(getJTextFieldTestDataFile(), gridBagConstraints13);
            jPanelSecondPass.add(getJButtonTestDataFile(), gridBagConstraints14);
        }
        return jPanelSecondPass;
    }

    /**
     * This method initializes jTextFieldTestDataFile	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getJTextFieldTestDataFile() {
        if (jTextFieldTestDataFile == null) {
            jTextFieldTestDataFile = new JTextField();
            jTextFieldTestDataFile.setEditable(false);
        }
        return jTextFieldTestDataFile;
    }

    /**
     * This method initializes jButtonTestDataFile	
     * 	
     * @return javax.swing.JButton	
     */
    private JButton getJButtonTestDataFile() {
        if (jButtonTestDataFile == null) {
            jButtonTestDataFile = new JButton();
            jButtonTestDataFile.setText("...");
            jButtonTestDataFile.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    JOpenFileChooser file_chooser = new JOpenFileChooser(MainFIS.config.FISFilePath());
                    if (!jTextFieldTestDataFile.getText().equals(""))
                        file_chooser.setSelectedFile(new File(jTextFieldTestDataFile.getText()));
                    file_chooser.setAcceptAllFileFilterUsed(true);
                    if (file_chooser.showOpenDialog(JPanelOLS.this) == JFileChooser.APPROVE_OPTION)
                        jTextFieldTestDataFile.setText(file_chooser.getSelectedFile().getAbsolutePath());
                }
            });
        }
        return jButtonTestDataFile;
    }

    /**
     * This method initializes jPanelOutputVocabularyReduction	
     * 	
     * @return fis.JPanelOutputVocabularyReduction	
     */
    public JPanelOutputVocabularyReduction getJPanelOutputVocabularyReduction() {
        if (jPanelOutputVocabularyReduction == null) {
            jPanelOutputVocabularyReduction = new JPanelOutputVocabularyReduction();
            ComponentTitledBorder componentBorder = new ComponentTitledBorder(jCheckBoxOutputVocabularyReduction, jPanelOutputVocabularyReduction, BorderFactory.createTitledBorder(""));
            jCheckBoxOutputVocabularyReduction.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    jPanelOutputVocabularyReduction.setEnabled(jCheckBoxOutputVocabularyReduction.isSelected());
                    jCheckBoxOutputFuzzification.setEnabled(jCheckBoxOutputVocabularyReduction.isSelected());
                }
            });
            jPanelOutputVocabularyReduction.setBorder(componentBorder);
        }
        return jPanelOutputVocabularyReduction;
    }

    /**
     * This method initializes jCheckBoxRemoveTemporaryFiles	
     * 	
     * @return javax.swing.JCheckBox	
     */
    private JCheckBox getJCheckBoxRemoveTemporaryFiles() {
        if (jCheckBoxRemoveTemporaryFiles == null) {
            jCheckBoxRemoveTemporaryFiles = new JCheckBox();
            jCheckBoxRemoveTemporaryFiles.setText(LocaleFIS.GetString("RemoveTemporaryFiles"));
            jCheckBoxRemoveTemporaryFiles.setSelected(true);
        }
        return jCheckBoxRemoveTemporaryFiles;
    }

    /**
     * This method initializes jCheckBoxOutputFuzzification	
     * 	
     * @return javax.swing.JCheckBox	
     */
    private JCheckBox getJCheckBoxOutputFuzzification() {
        if (jCheckBoxOutputFuzzification == null) {
            jCheckBoxOutputFuzzification = new JCheckBox();
            jCheckBoxOutputFuzzification.setText(LocaleFIS.GetString("OutputFuzzification"));
        }
        return jCheckBoxOutputFuzzification;
    }

    public int getOutputNumber() {
        return this.jComboBoxOutputs.getSelectedIndex();
    }

    public double getTotalUnexplainedVariance() {
        return doubleFieldTotalUnexplainedVariance.getValue();
    }

    public int getMaximumNumberOfRules() {
        return integerFieldMaximumNumberOfRules.getValue();
    }

    public boolean getReduceVocabulary() {
        return this.jCheckBoxOutputVocabularyReduction.isSelected();
    }

    public boolean getOutputFuzzification() {
        return this.jCheckBoxOutputFuzzification.isSelected();
    }

    public boolean getRemoveTemporaryFiles() {
        return this.jCheckBoxRemoveTemporaryFiles.isSelected();
    }

    public String getTestDataFile() {
        return this.jTextFieldTestDataFile.getText();
    }

    public double getStandardDeviation() {
        return (double) this.integerFieldStandardDeviation.getValue() / 100;
    }

    public String getConjunction() throws Throwable {
        return JFIS.TypeConjunction()[this.jComboBoxConjunction.getSelectedIndex()];
    }

    public boolean keepExistingRules() {
        return this.jCheckBoxKeepExistingRules.isSelected();
    }

    /**
     * This method initializes jComboBoxConjunction	
     * 	
     * @return javax.swing.JComboBox	
     */
    private JComboBox getJComboBoxConjunction() {
        if (jComboBoxConjunction == null) {
            jComboBoxConjunction = new JComboBox();
            jComboBoxConjunction.setModel(this.getDefaultComboBoxModelConjunction());
            try {
                for (int i = 0; i < JFIS.TypeConjunction().length - 1; i++)
                    this.defaultComboBoxModelConjunction.addElement(new String()); // on met des String vide dans le modele, c'est la methode getElementAt qui fait la traduction
            } catch (Throwable e) {
                // TODO Bloc catch auto-genere
                e.printStackTrace();
            }
            jComboBoxConjunction.setSelectedIndex(1);
        }
        return jComboBoxConjunction;
    }

    /**
     * This method initializes defaultComboBoxModelConjunction	
     * 	
     * @return javax.swing.DefaultComboBoxModel	
     */
    private DefaultComboBoxModel getDefaultComboBoxModelConjunction() {
        if (defaultComboBoxModelConjunction == null) {
            defaultComboBoxModelConjunction = new DefaultComboBoxModel() {
                // c'est cette methode qui assure la traduction des types de conjonction
                public Object getElementAt(int index) {
                    try {
                        return LocaleFIS.GetString(JFIS.TypeConjunction()[index]);
                    } catch (Throwable t) {
                        return null;
                    }
                }
            };
        }
        return defaultComboBoxModelConjunction;
    }

    /**
     * This method initializes jRadioButtonGenerateFISWithoutRules	
     * 	
     * @return javax.swing.JRadioButton	
     */
    private JRadioButton getJRadioButtonGenerateFISWithoutRules() {
        if (jRadioButtonGenerateFISWithoutRules == null) {
            jRadioButtonGenerateFISWithoutRules = new JRadioButton();
            jRadioButtonGenerateFISWithoutRules.setText(LocaleFIS.GetString("GenerateFisWithoutRules"));
            jRadioButtonGenerateFISWithoutRules.setSelected(true);
            jRadioButtonGenerateFISWithoutRules.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    setEnabledGenerateGaussianPartition(false);
                    // reduction du vocabulaire de sortie autorise
                    jCheckBoxOutputVocabularyReduction.setSelected(true);
                    jCheckBoxOutputVocabularyReduction.setEnabled(true);
                    jPanelOutputVocabularyReduction.setEnabled(true);
                    jCheckBoxOutputFuzzification.setEnabled(true);
                }
            });
            this.setEnabledGenerateGaussianPartition(false);
            this.getButtonGroupGeneratePartition().add(jRadioButtonGenerateFISWithoutRules);
        }
        return jRadioButtonGenerateFISWithoutRules;
    }

    /**
     * This method initializes jRadioButtonGenerateGaussianPartition	
     * 	
     * @return javax.swing.JRadioButton	
     */
    private JRadioButton getJRadioButtonGenerateGaussianPartition() {
        if (jRadioButtonGenerateGaussianPartition == null) {
            jRadioButtonGenerateGaussianPartition = new JRadioButton();
            jRadioButtonGenerateGaussianPartition.setText(LocaleFIS.GetString("GaussianPartitions"));
            jRadioButtonGenerateGaussianPartition.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    setEnabledGenerateGaussianPartition(true);
                    // pas de reduction du vocabulaire de sortie 
                    jCheckBoxOutputVocabularyReduction.setSelected(false);
                    jCheckBoxOutputVocabularyReduction.setEnabled(false);
                    jPanelOutputVocabularyReduction.setEnabled(false);
                    jCheckBoxOutputFuzzification.setEnabled(false);
                }
            });
            this.getButtonGroupGeneratePartition().add(jRadioButtonGenerateGaussianPartition);
        }
        return jRadioButtonGenerateGaussianPartition;
    }

    /**
     * This method initializes buttonGroupGeneratePartition	
     * 	
     * @return javax.swing.ButtonGroup	
     */
    private ButtonGroup getButtonGroupGeneratePartition() {
        if (buttonGroupGeneratePartition == null) {
            buttonGroupGeneratePartition = new ButtonGroup();
        }
        return buttonGroupGeneratePartition;
    }

    public void setEnabledGenerateGaussianPartition(boolean enabled) {
        this.jLabelStandardDeviation.setEnabled(enabled);
        this.integerFieldStandardDeviation.setEnabled(enabled);
        this.jLabelStandardDeviationUnit.setEnabled(enabled);
        this.jLabelConjunction.setEnabled(enabled);
        this.jComboBoxConjunction.setEnabled(enabled);
        this.jLabelWarningGeneratePartitions.setEnabled(enabled);
    }

    public boolean getGenerateFISWithoutRules() {
        return this.jRadioButtonGenerateFISWithoutRules.isSelected();
    }
}
