//***********************************************************************
//
//                              JPanelGenerateConclusions.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  December 1, 2007
//
//**********************************************************************
package fis;

import javax.swing.JPanel;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JRadioButton;
import java.awt.GridBagConstraints;
import java.util.Enumeration;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;

import util.skt.swing.border.ComponentTitledBorder;
import javax.swing.border.TitledBorder;

public class JPanelOptimizeConclusions extends JPanel {

    private ButtonGroup buttonGroupMethod = null;

    private JCheckBox jCheckBoxOutputVocabularyReduction = null; //  @jve:decl-index=0:visual-constraint="50,250"

    private JPanelOutputVocabularyReduction jPanelOutputVocabularyReduction = null;

	private JPanel jPanelOutputs = null;
	
	private JFIS fis = null;

	private ButtonGroup jbgOutput = new ButtonGroup();  //  @jve:decl-index=0:

    /**
     * This is the default constructor
     */
    public JPanelOptimizeConclusions(JFIS fis) throws Throwable {
        super();
        this.fis = fis;
        initialize();
        for( int i=0 ; i<this.fis.NbOutputs() ; i++ ) {
	        final JOutput output = this.fis.GetOutput(i);
	        JRadioButton jrb = new JRadioButton(output.GetName());
	        this.jbgOutput.add(jrb);
	        
	        if((output.GetNature() == JOutput.CRISP) && (!output.GetClassif())) {
	        	if(this.getSelectedOutput() == -1)
	        		jrb.setSelected(true);
	        } else
	            jrb.setEnabled(false);
	        
	        this.jPanelOutputs.add(jrb, new GridBagConstraints(i%2 == 0 ? 0 : 1, i/2, 1, 1, 1.0, 0.0
	                ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets( 0, 0, 0, 0), 0, 0));
        }
        if(this.getSelectedOutput() == -1)
  		  	throw new Exception(LocaleFIS.GetString("ApplyOLSOnlyWithNoClassifCrispOutput"));
    }

    /**
     * This method initializes this
     * 
     * @return void
     */
    private void initialize() {
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.gridy = 0;
        GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
        gridBagConstraints7.gridx = 0;
        gridBagConstraints7.gridwidth = 2;
        gridBagConstraints7.insets = new java.awt.Insets(5, 0, 0, 0);
        gridBagConstraints7.gridy = 2;
        this.setLayout(new GridBagLayout());
        this.setSize(400, 250);
        this.add(getJPanelOutputVocabularyReduction(), gridBagConstraints7);
        this.add(getJPanelOutputs(), gridBagConstraints);
    }

    /**
     * This method initializes buttonGroupMethod	
     * 	
     * @return javax.swing.ButtonGroup	
     */
    private ButtonGroup getButtonGroupMethod() {
        if (buttonGroupMethod == null) {
            buttonGroupMethod = new ButtonGroup();
        }
        return buttonGroupMethod;
    }

    /**
     * This method initializes jCheckBoxOutputVocabularyReduction	
     * 	
     * @return javax.swing.JCheckBox	
     */
    private JCheckBox getJCheckBoxOutputVocabularyReduction() {
        if (jCheckBoxOutputVocabularyReduction == null) {
            jCheckBoxOutputVocabularyReduction = new JCheckBox();
            jCheckBoxOutputVocabularyReduction.setText(LocaleFIS.GetString("OutputVocabularyReduction"));
            jCheckBoxOutputVocabularyReduction.setSelected(true);
            jCheckBoxOutputVocabularyReduction.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    jPanelOutputVocabularyReduction.setEnabled(jCheckBoxOutputVocabularyReduction.isSelected());
                }
            });
        }
        return jCheckBoxOutputVocabularyReduction;
    }

    /**
     * This method initializes jPanelOutputVocabularyReduction	
     * 	
     * @return fis.JPanelOutputVocabularyReduction	
     */
    public JPanelOutputVocabularyReduction getJPanelOutputVocabularyReduction() {
        if (jPanelOutputVocabularyReduction == null) {
            jPanelOutputVocabularyReduction = new JPanelOutputVocabularyReduction();
            jPanelOutputVocabularyReduction.setBorder(new ComponentTitledBorder(this.getJCheckBoxOutputVocabularyReduction(), jPanelOutputVocabularyReduction, BorderFactory.createTitledBorder("")));
        }
        return jPanelOutputVocabularyReduction;
    }

    public boolean getReduceVocabulary() {
        return this.jCheckBoxOutputVocabularyReduction.isSelected();
    }

	/**
	 * This method initializes jPanelOutputs	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanelOutputs() {
		if (jPanelOutputs == null) {
			jPanelOutputs = new JPanel();
			jPanelOutputs.setLayout(new GridBagLayout());
			jPanelOutputs.setBorder(BorderFactory.createTitledBorder(null, "Outputs", TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, null, null));
		}
		return jPanelOutputs;
	}

	public int getSelectedOutput() {
		Enumeration items = this.jbgOutput.getElements();
		int outputNumber = 0;
		while(items.hasMoreElements()) {
			JRadioButton jrb = (JRadioButton)items.nextElement();
			if(jrb.isSelected())
				return outputNumber;
			outputNumber++;
		}
		return -1;
	}

}
