//***********************************************************************
//
//                              JPanelPlot2DInfer.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JPanel;

public class JPanelPlot2DInfer extends JPanel {

    private FISPlot FISPlot = null;

    private JFIS fis = null;

    private double[] yRange = null;

    /**
     * This is the default constructor
     */
    public JPanelPlot2DInfer() {
        super();
        initialize();
    }

    public void init(JFIS fis) {
        this.fis = fis;
        FISPlot.DisableZoom(this.FISPlot);
    }

    public void setOutput(JOutput output) throws Throwable {
        this.FISPlot.setYLabel(output.GetName());
        this.yRange = output.GetRange();
        this.FISPlot.setYRange(this.yRange[0], this.yRange[1]);
        this.FISPlot.repaint();
    }

    public void setX(String xLabel, double xMin, double xMax) {
        this.FISPlot.setXLabel(xLabel);
        this.FISPlot.setXRange(xMin, xMax);
        this.FISPlot.repaint();
    }

    private void plotInferedValues(double[][] values) {
        // find y values range and adjust FisPlot y range to show all values
        double[] yValuesRange = { java.lang.Double.POSITIVE_INFINITY, java.lang.Double.NEGATIVE_INFINITY };
        for (int i = 0; i < values[0].length; i++) {
            if (values[1][i] < yValuesRange[0])
                yValuesRange[0] = values[1][i];
            if (values[1][i] > yValuesRange[1])
                yValuesRange[1] = values[1][i];
        }
        this.FISPlot.setYRange(Math.min(yValuesRange[0], this.yRange[0]), Math.max(yValuesRange[1], this.yRange[1]));

        // plot values
        this.FISPlot.clear(0);
        for (int i = 0; i < values[0].length; i++)
            this.FISPlot.addPoint(0, values[0][i], values[1][i], true);
    }

    /**
     * TODO a reprendre quand Infer acceptera les entrees inactives
     * @param selectedXInput
     * @param xResolution
     * @param xMin
     * @param xMax
     * @param selectedOutput
     * @param inputsValues
     * @return the Y (output) values array to send to JPanelPlot2DInfer. there's two array, one for x result and another for y result
     * @throws Throwable
     */
    private double[][] computeInfer(int selectedXInput, int xResolution, double xMin, double xMax, int selectedOutput, double[] inputsValues) throws Throwable {
        SurfacePlotParameters spp = new SurfacePlotParameters();
        spp.Xresolution = xResolution;
        spp.Yresolution = -1; //in the parameters object, Y refer to a 3D surface
        //Xmin, Xmax (Ymin & Ymax not used)
        spp.Xmax = xMax;
        spp.Xmin = xMin;
        spp.Ymax = -1;
        spp.Ymin = -1;

        //output number
        spp.output_number = selectedOutput;
        spp.selected_Y_input_number = -1;

        //fixed inputs :
        spp.fixedinputs_numbers = new int[this.fis.NbActiveInputs() - 1];
        spp.fixedinputs_values = new double[this.fis.NbActiveInputs() - 1];

        // modifie la selection selectedXInput en numero d'entree active
        spp.selected_X_input_number = selectedXInput;
        for (int i = 0; i < selectedXInput; i++)
            if (this.fis.GetInput(i).GetActive() == false)
                spp.selected_X_input_number--;

        // rempli les tableaux fixedinputs_numbers et fixedinputs_values avec les valeurs actives (excepte l'entree selectionnee)
        for (int i = 0, active = 0, active_count = 0; i < this.fis.NbInputs(); i++) {
            if (this.fis.GetInput(i).GetActive() == true) //we retrieve all actives values except the X one
            {
                if (i != selectedXInput) {
                    spp.fixedinputs_numbers[active_count] = active;
                    spp.fixedinputs_values[active_count] = inputsValues[i];
                    active_count++;
                }
                active++;
            }
        }

        //SurfacePlotParameters is ready. Now, go to jni
        return this.fis.Infer2DSurface(spp);
    }

    public void showInferedValues(int selectedInput, int resolution, double min, double max, int selectedOutput, double[] inputsValues) throws Throwable {
        this.plotInferedValues(this.computeInfer(selectedInput, resolution, min, max, selectedOutput, inputsValues));
    }

    /* (non-Javadoc)
     * @see javax.swing.JComponent#getMinimumSize()
     */
    public Dimension getMinimumSize() {
        return new Dimension(300, 150);
    }

    /**
     * This method initializes this
     * 
     * @return void
     */
    private void initialize() {
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0D;
        gridBagConstraints.weighty = 1.0D;
        gridBagConstraints.gridx = 0;
        this.setLayout(new GridBagLayout());
        this.setSize(510, 310);
        this.add(getFISPlot(), gridBagConstraints);
    }

    /**
     * This method initializes FISPlot	
     * 	
     * @return fis.FISPlot	
     */
    private FISPlot getFISPlot() {
        if (FISPlot == null) {
            FISPlot = new FISPlot();
        }
        return FISPlot;
    }

}
