//***********************************************************************
//
//                              JPanelSectionInfer.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JSpinner;

import util.JDoubleMThumbSliderSpinner;

/**
 * Panel de parametrage et d'affichage 
 * 
 * @author jean-luc.lablee
 */
public class JPanelSectionInfer extends JFISPanel {

    private JLabel jLabelSelectedInput = null;

    private JLabel jLabelSectionInput = null;

    private JDoubleMThumbSliderSpinner jDoubleMThumbSliderSectionValue = null;

    private JSpinner jSpinnerSectionValue = null;

    private JCheckBox jCheckBoxShowSection = null;

    private JPanelPlot2DInfer jPanelPlot2DInfer = null;

    private JFIS fis = null;

    private JComboBoxInputs jComboBoxSelectedInput = null;

    private JPanel3DInferParameters jp3dip = null;

    /**
     * This is the default constructor
     */
    public JPanelSectionInfer() {
        super();
        initialize();
    }

    /**
     * Initialisation du panel
     * @param fis objet FIS
     * @param jp3dip objet de parametrage 3D fis.JPanel3DInferParameters
     * @throws Throwable
     */
    public void init(JFIS fis, JPanel3DInferParameters jp3dip) throws Throwable {
        this.fis = fis;
        this.jPanelPlot2DInfer.init(this.fis);
        this.jp3dip = jp3dip;
        this.jp3dip.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent e) {
                try {
                    jPanel3DInferParametersStateChanged();
                } catch (Throwable e1) {
                    // TODO Bloc catch auto-genere
                    e1.printStackTrace();
                }
            }
        });
        this.jPanelPlot2DInfer.setOutput(this.fis.GetOutput(this.jp3dip.selectedOutput()));
        this.jComboBoxSelectedInput.setInputs(this.fis.GetInputs());
        this.jPanel3DInferParametersStateChanged();
        fireStateChanged();
    }

    /**
     * Fonction appellee lors du changement d'etat de l'objet de parametrage 3D
     * @throws Throwable
     */
    private void jPanel3DInferParametersStateChanged() throws Throwable {
        // on change la selection de la combobox seulement si l'entree selectionnee n'est plus selectionnee dans JPanel3DInferParameters
        if ((this.jComboBoxSelectedInput.getSelectedIndex() != this.jp3dip.selectedXInput()) && (this.jComboBoxSelectedInput.getSelectedIndex() != this.jp3dip.selectedYInput()))
            this.jComboBoxSelectedInput.setSelectedIndex(this.jp3dip.selectedXInput());
        this.setXJPanelPlot2DInfer();
        this.jPanelPlot2DInfer.setOutput(this.fis.GetOutput(this.jp3dip.selectedOutput()));
        this.setOtherInput();
        this.inferSection();
    }

    /**
     * Initialisation 
     * @throws Throwable
     */
    private void setOtherInput() throws Throwable {
        if (this.jComboBoxSelectedInput.getSelectedIndex() == this.jp3dip.selectedXInput()) {
            this.jLabelSectionInput.setText(this.fis.GetInput(this.jp3dip.selectedYInput()).GetName());
            this.jDoubleMThumbSliderSectionValue.setDoubleMinMax(this.jp3dip.yMin(), this.jp3dip.yMax());
        } else {
            this.jLabelSectionInput.setText(this.fis.GetInput(this.jp3dip.selectedXInput()).GetName());
            this.jDoubleMThumbSliderSectionValue.setDoubleMinMax(this.jp3dip.xMin(), this.jp3dip.xMax());
        }
    }

    /**
     * Initialise l'axe X du panel jPanelPlot2DInfer
     * @throws Throwable
     */
    private void setXJPanelPlot2DInfer() throws Throwable {
        if (this.jComboBoxSelectedInput.getSelectedIndex() == this.jp3dip.selectedXInput())
            this.jPanelPlot2DInfer.setX(this.fis.GetInput(this.jp3dip.selectedXInput()).GetName(), this.jp3dip.xMin(), this.jp3dip.xMax());
        else
            this.jPanelPlot2DInfer.setX(this.fis.GetInput(this.jp3dip.selectedYInput()).GetName(), this.jp3dip.yMin(), this.jp3dip.yMax());
    }

    /**
     * Infere la section en fonction des parametres selectionnes
     * @throws Throwable
     */
    private void inferSection() throws Throwable {
        double[] inputValues = this.jp3dip.getInputsValues();
        if (this.jComboBoxSelectedInput.getSelectedIndex() == this.jp3dip.selectedXInput()) {
            inputValues[this.jp3dip.selectedYInput()] = this.jDoubleMThumbSliderSectionValue.getDoubleValue(0);
            this.jPanelPlot2DInfer.showInferedValues(this.jp3dip.selectedXInput(), this.jp3dip.xResolution(), this.jp3dip.xMin(), this.jp3dip.xMax(), this.jp3dip.selectedOutput(), inputValues);
        } else {
            inputValues[this.jp3dip.selectedXInput()] = this.jDoubleMThumbSliderSectionValue.getDoubleValue(0);
            this.jPanelPlot2DInfer.showInferedValues(this.jp3dip.selectedYInput(), this.jp3dip.yResolution(), this.jp3dip.yMin(), this.jp3dip.yMax(), this.jp3dip.selectedOutput(), inputValues);
        }
    }

    /**
     * This method initializes this
     * 
     * @return void
     */
    private void initialize() {
        GridBagConstraints gridBagConstraints31 = new GridBagConstraints();
        gridBagConstraints31.fill = java.awt.GridBagConstraints.NONE;
        gridBagConstraints31.gridy = 0;
        gridBagConstraints31.weightx = 1.0;
        gridBagConstraints31.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints31.insets = new java.awt.Insets(5, 5, 0, 0);
        gridBagConstraints31.gridx = 1;
        GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
        gridBagConstraints11.gridx = 0;
        gridBagConstraints11.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints11.insets = new java.awt.Insets(5, 0, 0, 0);
        gridBagConstraints11.gridy = 1;
        jLabelSectionInput = new JLabel();
        jLabelSectionInput.setText("OtherInput");
        GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
        gridBagConstraints5.gridx = 0;
        gridBagConstraints5.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints5.gridwidth = 2;
        gridBagConstraints5.weightx = 1.0D;
        gridBagConstraints5.weighty = 1.0D;
        gridBagConstraints5.insets = new java.awt.Insets(5, 5, 5, 5);
        gridBagConstraints5.gridy = 4;
        GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
        gridBagConstraints4.gridx = 0;
        gridBagConstraints4.insets = new java.awt.Insets(5, 5, 0, 5);
        gridBagConstraints4.gridwidth = 2;
        gridBagConstraints4.gridy = 3;
        GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
        gridBagConstraints3.gridx = 1;
        gridBagConstraints3.gridwidth = 1;
        gridBagConstraints3.insets = new java.awt.Insets(5, 5, 0, 0);
        gridBagConstraints3.ipadx = 30;
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints3.gridy = 1;
        GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
        gridBagConstraints2.fill = java.awt.GridBagConstraints.NONE;
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.gridwidth = 2;
        gridBagConstraints2.insets = new java.awt.Insets(5, 5, 0, 5);
        gridBagConstraints2.gridx = 0;
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 0);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.fill = java.awt.GridBagConstraints.NONE;
        gridBagConstraints.weightx = 1.0D;
        gridBagConstraints.gridy = 0;
        jLabelSelectedInput = new JLabel();
        jLabelSelectedInput.setText(LocaleFIS.GetString("SelectInput"));
        this.setLayout(new GridBagLayout());
        this.setSize(new java.awt.Dimension(520, 429));
        this.add(jLabelSelectedInput, gridBagConstraints);
        this.add(getJDoubleMThumbSliderSpinner(), gridBagConstraints2);
        this.add(getJSpinner(), gridBagConstraints3);
        this.add(getJCheckBox(), gridBagConstraints4);
        this.add(getJPanelPlot2DInfer(), gridBagConstraints5);
        this.add(jLabelSectionInput, gridBagConstraints11);
        this.add(getJComboBoxInputs(), gridBagConstraints31);
    }

    /**
     * This method initializes jDoubleMThumbSliderSpinner	
     * 	
     * @return util.JDoubleMThumbSliderSpinner	
     */
    private JDoubleMThumbSliderSpinner getJDoubleMThumbSliderSpinner() {
        if (jDoubleMThumbSliderSectionValue == null) {
            jDoubleMThumbSliderSectionValue = new JDoubleMThumbSliderSpinner(1);
            jDoubleMThumbSliderSectionValue.setMinimumSize(jDoubleMThumbSliderSectionValue.getPreferredSize());
            jDoubleMThumbSliderSectionValue.addChangeListener(new javax.swing.event.ChangeListener() {
                public void stateChanged(javax.swing.event.ChangeEvent e) {
                    try {
                        inferSection();
                        fireStateChanged();
                    } catch (Throwable e1) {
                        // TODO Bloc catch auto-genere
                        e1.printStackTrace();
                    }
                }
            });
        }
        return jDoubleMThumbSliderSectionValue;
    }

    /**
     * This method initializes jSpinner	
     * 	
     * @return javax.swing.JSpinner	
     */
    private JSpinner getJSpinner() {
        if (jSpinnerSectionValue == null) {
            jSpinnerSectionValue = this.jDoubleMThumbSliderSectionValue.getSpinners()[0];
        }
        return jSpinnerSectionValue;
    }

    /**
     * This method initializes jCheckBox	
     * 	
     * @return javax.swing.JCheckBox	
     */
    private JCheckBox getJCheckBox() {
        if (jCheckBoxShowSection == null) {
            jCheckBoxShowSection = new JCheckBox();
            jCheckBoxShowSection.setText(LocaleFIS.GetString("ShowSectionOn3DGraph"));
            jCheckBoxShowSection.setSelected(true);
            jCheckBoxShowSection.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    fireStateChanged();
                }
            });
        }
        return jCheckBoxShowSection;
    }

    /**
     * @return true si la section est visible
     */
    public boolean isSectionVisible() {
        return this.jCheckBoxShowSection.isSelected();
    }

    /**
     * @return true si l'axe selectionne est l'axe x
     */
    public boolean isXAxisSelected() {
        return this.jComboBoxSelectedInput.getSelectedIndex() == this.jp3dip.selectedXInput();
    }

    /**
     * @return la position de la section
     */
    public double getSectionValue() {
        return this.jDoubleMThumbSliderSectionValue.getDoubleValue(0);
    }

    /**
     * @return la position minimum de la section
     */
    public double getSectionMin() {
        return this.jDoubleMThumbSliderSectionValue.getDoubleMin();
    }

    /**
     * @return la position maximum de la section
     */
    public double getSectionMax() {
        return this.jDoubleMThumbSliderSectionValue.getDoubleMax();
    }

    /**
     * This method initializes jPanelPlot2DInfer	
     * 	
     * @return fis.JPanelPlot2DInfer	
     */
    public JPanelPlot2DInfer getJPanelPlot2DInfer() {
        if (jPanelPlot2DInfer == null) {
            jPanelPlot2DInfer = new JPanelPlot2DInfer();
        }
        return jPanelPlot2DInfer;
    }

    /**
     * This method initializes jComboBoxInputs	
     * 	
     * @return fis.JComboBoxInputs	
     */
    private JComboBoxInputs getJComboBoxInputs() {
        if (jComboBoxSelectedInput == null) {
            jComboBoxSelectedInput = new JComboBoxInputs() {
                // surcharge de isIndexEnabled pour n'autoriser que les entrees selectionnees dans jp3dip
                protected boolean isIndexEnabled(int index) throws Throwable {
                    return ((index == jp3dip.selectedXInput()) || (index == jp3dip.selectedYInput()));
                }
            };
            jComboBoxSelectedInput.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    try {
                        int selectedIndex = ((JComboBox) e.getSource()).getSelectedIndex();
                        if (selectedIndex != -1) {
                            setXJPanelPlot2DInfer();
                            inferSection();
                            setOtherInput();
                        }
                        fireStateChanged();
                    } catch (Throwable e1) {
                        // TODO Bloc catch auto-genere
                        e1.printStackTrace();
                    }
                }
            });
        }
        return jComboBoxSelectedInput;
    }
}
