//***********************************************************************
//
//                              JPanelValidationFile.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import javax.swing.*;
import java.awt.*;
import javax.swing.border.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.io.*;

/**
 * Classe de choix du fichier de validation:
 *   - tous les exemples
 *   - exemples actifs
 *   - exemples inactif
 *   - autre fichier a selectionner par une boite de dialogue
 */
public class JPanelValidationFile extends JPanel {

  protected ButtonGroup jbgFile = new ButtonGroup();
  protected JRadioButton jrbAllItems= new JRadioButton();
  protected JRadioButton jrbActiveItems= new JRadioButton();
  protected JRadioButton jrbInactiveItems= new JRadioButton();
  protected JRadioButton jrbOther= new JRadioButton();
  protected TitledBorder jBorder = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"");
  protected JTextField jtfValidationFile = new JTextField();
  protected JButton jbSelectValidationFile = new JButton("...");
  protected JExtendedDataFile data;
//------------------------------------------------------------------------------
  public JPanelValidationFile(JExtendedDataFile data)
  {
  super(new GridBagLayout());
  this.data = data;
  this.setBorder(this.jBorder);
  this.jbgFile.add(this.jrbAllItems);
  this.jbgFile.add(this.jrbActiveItems);
  this.jbgFile.add(this.jrbInactiveItems);
  this.jbgFile.add(this.jrbOther);

  this.add(this.jrbAllItems,    new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
  this.add(this.jrbActiveItems,    new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
  this.add(this.jrbInactiveItems,    new GridBagConstraints(0, 2, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
  this.jrbOther.addChangeListener(new ChangeListener()
    {
    public void stateChanged(ChangeEvent e)
      {
      JPanelValidationFile.this.jtfValidationFile.setEnabled(JPanelValidationFile.this.jrbOther.isSelected());
      JPanelValidationFile.this.jbSelectValidationFile.setEnabled(JPanelValidationFile.this.jrbOther.isSelected());
      }
    });
  this.add(this.jrbOther,    new GridBagConstraints(0, 3, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
  this.jtfValidationFile.setEditable(false);
  this.add(this.jtfValidationFile,     new GridBagConstraints(0, 4, 1, 1, 1.0, 0.0
             ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 5, 5), 0, 0));
  this.jbSelectValidationFile.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jbSelectValidationFile_actionPerformed(e); } });
  this.add(this.jbSelectValidationFile,     new GridBagConstraints(1, 4, 1, 1, 0.0, 0.0
             ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 5), 0, -10));
  this.Translate();
  this.setEnabled(true);
  if( this.jrbActiveItems.isEnabled() == true )
    this.jrbActiveItems.setSelected(true);
  else
    this.jrbAllItems.setSelected(true);
  }
//------------------------------------------------------------------------------
  public void Translate()
  {
  this.jBorder.setTitle(LocaleFIS.GetString("Validation"));
  this.jrbAllItems.setText(LocaleFIS.GetString("AllData"));
  this.jrbActiveItems.setText(LocaleFIS.GetString("ActiveData"));
  this.jrbInactiveItems.setText(LocaleFIS.GetString("InactiveData"));
  this.jrbOther.setText(LocaleFIS.GetString("OtherFile"));
  }
//------------------------------------------------------------------------------
  void jbSelectValidationFile_actionPerformed(ActionEvent e)
  {
  JOpenFileChooser file_chooser = new JOpenFileChooser(MainFIS.config.FISFilePath());
  if( !this.jtfValidationFile.getText().equals("") )
    file_chooser.setSelectedFile(new File(this.jtfValidationFile.getText()));
  file_chooser.setAcceptAllFileFilterUsed(true);
  if( file_chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION )
    this.jtfValidationFile.setText(file_chooser.getSelectedFile().getAbsolutePath());
  }
//------------------------------------------------------------------------------
  public String GetValidationFile() throws Exception
  {
  if( this.jrbAllItems.isSelected() == true )
    return this.data.GetFileName();
  if( this.jrbActiveItems.isSelected() == true )
    return this.data.ActiveFileName();
  if( this.jrbInactiveItems.isSelected() == true )
    return this.data.InactiveFileName();
  if( this.jrbOther.isSelected() == true )
    if( this.jtfValidationFile.getText().equals("") == true )
      throw new Exception(LocaleFIS.GetString("SelectAValidationFile"));
    else
      return this.jtfValidationFile.getText();
  return null;
  }
//------------------------------------------------------------------------------
  public void setEnabled(boolean enabled)
  {
  super.setEnabled(enabled);
  this.jrbAllItems.setEnabled(enabled );
  this.jrbActiveItems.setEnabled(enabled && (this.data.GetActiveCount() != this.data.ItemCount()));
  this.jrbInactiveItems.setEnabled(enabled && (this.data.GetActiveCount() != this.data.ItemCount()));
  this.jrbOther.setEnabled(enabled);
  this.jtfValidationFile.setEnabled(enabled && this.jrbOther.isSelected());
  this.jbSelectValidationFile.setEnabled(enabled && this.jrbOther.isSelected());
  }
}