//***********************************************************************
//
//                              JRangeFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;

//------------------------------------------------------------------------------
public class JRangeFrame extends JFISDialog
{
  private JPanel jPanelDisplayRange = new JPanel();
  private GridBagLayout gridBLDisplayRange = new GridBagLayout();
  private JPanel jPanelSaisie = new JPanel();
  private JPanel jPanelValidation = new JPanel();
  private GridBagLayout gridBagLayoutValidation = new GridBagLayout();
  private JButton jButtonCancel = new JButton();
  private JButton jButtonApply = new JButton();
  private GridBagLayout gridBLSaisie = new GridBagLayout();
  private JLabel jLabelLower = new JLabel();
  private JLabel jLabelUpper = new JLabel();
  private DoubleField dfLower = new DoubleField();
  private DoubleField dfUpper = new DoubleField();
  private double [] InputRange;       // domaine de l'entree
  private double [] NewRange;         // nouveau domaine saisi
//------------------------------------------------------------------------------
  public JRangeFrame( JInput input, String title )
  {
    try
    {
      this.setTitle(LocaleFIS.GetString(title));
      this.InputRange = input.GetRange();
      jbInit();
    }
    catch( Throwable t) { MessageFIS.Error(null, t); }
  }
//------------------------------------------------------------------------------
  private void jbInit() throws Throwable
  {
    jPanelDisplayRange.setLayout(gridBLDisplayRange);
    jPanelValidation.setLayout(gridBagLayoutValidation);
    jButtonCancel.setText(LocaleFIS.GetString("Cancel"));
    jButtonCancel.addActionListener(new java.awt.event.ActionListener() { public void actionPerformed(ActionEvent e) { jButtonCancel_actionPerformed(e); } });
    jButtonApply.setText(LocaleFIS.GetString("Apply"));
    jButtonApply.addActionListener(new java.awt.event.ActionListener() { public void actionPerformed(ActionEvent e) { jButtonApply_actionPerformed(e); } });
    jPanelSaisie.setLayout(gridBLSaisie);

    dfLower.setValue(this.InputRange[0]);
    dfUpper.setValue(this.InputRange[1]);
    this.jLabelLower.setText(LocaleFIS.GetString("Lower")+":");
    this.jLabelUpper.setText(LocaleFIS.GetString("Upper")+":");

    // affichage des objets
    this.getContentPane().add(jPanelDisplayRange);
    jPanelDisplayRange.add(jPanelSaisie,    new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
    jPanelDisplayRange.add(jPanelValidation,   new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 5, 5), 0, 0));
    jPanelValidation.add(jButtonApply,    new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
    jPanelValidation.add(jButtonCancel, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
    jPanelSaisie.add(jLabelLower,         new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
    jPanelSaisie.add(jLabelUpper,         new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
    jPanelSaisie.add(dfLower,   new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 50, 0));
    jPanelSaisie.add(dfUpper,    new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 50, 0));
    this.setModal(true);
    this.setResizable(false);
    this.pack();
  }
//------------------------------------------------------------------------------
  private void jButtonCancel_actionPerformed(ActionEvent e)
  {
  this.NewRange = null;
  this.dispose();
  }
//------------------------------------------------------------------------------
  private double [] ReadRange() throws Exception
  {
  double [] result = new double[2];
  result[0] = dfLower.getValue();
  result[1] = dfUpper.getValue();
  if( result[0] >= result[1] )
    throw new Exception(LocaleFIS.GetString("Upper")+" "+LocaleFIS.GetString("MustBeHigherThan")+" "+LocaleFIS.GetString("Lower"));
  return result;
  }

//------------------------------------------------------------------------------
  private void jButtonApply_actionPerformed(ActionEvent e)
  {
  try
    {
    this.NewRange = this.ReadRange();
    this.dispose();
    }
  catch( Exception ex ) { MessageFIS.Error(null, ex);}
  }
//------------------------------------------------------------------------------
// affiche la boite de dialogue et retourne le nouveau domaine  saisi ou null si cancel
  public double [] Show()
  {
  this.setVisible(true);
  return this.NewRange;
  }
}

