//***********************************************************************
//
//                              JRegularInputFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.util.*;

//------------------------------------------------------------------------------
public class JRegularInputFrame extends JFISDialog
{
  private GridBagLayout gridBagLayoutRegular = new GridBagLayout();
  private JLabel jLNbSEFs = new JLabel();
  protected IntegerField ifNbSEFs = new IntegerField();
  private JPanel jPanelValidation = new JPanel();
  private JButton jButtonCancel = new JButton();
  private JButton jButtonOK = new JButton();
  protected boolean ClickOK = false;
  protected int NbSEF;

//------------------------------------------------------------------------------
  public JRegularInputFrame()
  {
    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }
//------------------------------------------------------------------------------
  protected void setTitle() { super.setTitle(LocaleFIS.GetString("RegularGrid")); }
//------------------------------------------------------------------------------
  private void jbInit() throws Exception
  {
    this.setTitle();
    this.getContentPane().setLayout(gridBagLayoutRegular);
    jLNbSEFs.setText(LocaleFIS.GetString("NumberOfMF")+":");
    jButtonCancel.setText(LocaleFIS.GetString("Cancel"));
    jButtonCancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonCancel_actionPerformed(e); }});
    jButtonOK.setText(LocaleFIS.GetString("Apply"));
    jButtonOK.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonOK_actionPerformed(e); }});
    this.getContentPane().add(jLNbSEFs,    new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.getContentPane().add(ifNbSEFs,     new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 50, 0));
    this.getContentPane().add(jPanelValidation,   new GridBagConstraints(0, 2, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 5, 5, 5), 0, 0));
    jPanelValidation.add(jButtonOK, null);
    jPanelValidation.add(jButtonCancel, null);
    this.setModal(true);
    this.setResizable(false);
    this.pack();
  }

//------------------------------------------------------------------------------
  int ReadNbSEF() throws NumberFormatException { return this.ifNbSEFs.getValue(); }
//------------------------------------------------------------------------------
  void jButtonOK_actionPerformed(ActionEvent e)
  {
  try
    {
    this.NbSEF = ReadNbSEF();
    this.ClickOK = true;
    }
  catch( NumberFormatException nfe )
    {
    MessageFIS.Error( null, nfe );
    return;
    }
  this.dispose();
  }
//------------------------------------------------------------------------------
  void jButtonCancel_actionPerformed(ActionEvent e) { this.dispose(); }
//------------------------------------------------------------------------------
  protected JInput NewInput( JInput input ) throws Throwable { return input.NewRegular( this.NbSEF, input.GetRange()[0], input.GetRange()[1] ); }
//------------------------------------------------------------------------------
// retourne une nouvelle entree reguliere ou null si abandon
// input peut egalement etre de type JOutput
  JInput Show( JInput input ) throws Throwable
  {
  this.ifNbSEFs.setValue(3);  // 3 SEF par defaut
  this.setVisible(true);
  if( this.ClickOK == true )
    return this.NewInput(input);
  return null;
  }
}



//------------------------------------------------------------------------------
class JIrregularInputFrame extends JRegularInputFrame
{
  private boolean saisie_sommets = false;
  private JPanel jpSommets = new JPanel(new GridBagLayout());
  private Vector jvMFs = new Vector();
  private JInput input;
//------------------------------------------------------------------------------
  public JIrregularInputFrame( JInput input )
  {
  super();
  this.input = input;
  }
//------------------------------------------------------------------------------
  protected void setTitle() { super.setTitle(LocaleFIS.GetString("IrregularGrid")); }
//------------------------------------------------------------------------------
  protected JInput NewInput( JInput input ) throws Throwable
  {
  double []sommets = new double[this.jvMFs.size()];
  for( int i=0 ; i<sommets.length ; i++ )
    sommets[i] = ((DoubleField)this.jvMFs.elementAt(i)).getValue();
  return input.NewIrregular( sommets, this.NbSEF, input.GetRange()[0], input.GetRange()[1] );
  }
//------------------------------------------------------------------------------
  void jButtonOK_actionPerformed(ActionEvent e)
  {
  if( this.saisie_sommets == false )
    {
    this.saisie_sommets = true;
    this.ifNbSEFs.setEnabled(false);
    this.getContentPane().add(this.jpSommets,   new GridBagConstraints(0, 1, 2, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 5, 5, 5), 0, 0));
    for( int i=0 ; i<this.ReadNbSEF() ; i++ )
      {
      this.jpSommets.add(new JLabel("S"+(i+1)+":"),   new GridBagConstraints(0, i, 1, 1, 0.0, 0.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
      DoubleField dfSommet = new DoubleField();
      try { dfSommet = new DoubleFieldInput(this.input); }
      catch( Throwable t ) {}
      dfSommet.setValue(0);
      this.jvMFs.add(dfSommet);
      this.jpSommets.add(dfSommet,   new GridBagConstraints(1, i, 1, 1, 0.0, 0.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 40, 0));
      }
    this.pack();
    }
  else
    super.jButtonOK_actionPerformed(e);
  }
}
