//***********************************************************************
//
//                              JRuleFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import javax.swing.*;

import java.awt.*;
import java.util.*;
import javax.swing.border.*;
import java.awt.event.*;
import javax.swing.table.*;
import javax.swing.event.*;
import javax.swing.plaf.basic.*;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.awt.datatransfer.*;
import java.awt.print.*;
import util.sortabletable.*;
import util.*;
import fis.print.*;
//needed classes for exporting graphics
import org.freehep.graphics2d.*;
import org.freehep.graphicsio.*;
import org.freehep.graphicsio.exportchooser.*;
import org.freehep.graphicsio.ps.*;
import org.freehep.graphicsio.png.*;
import org.freehep.graphicsio.jpg.*;
import org.freehep.graphicsio.pdf.*;
import org.freehep.util.export.*;

//------------------------------------------------------------------------------
public class JRuleFrame extends JFISChildFrame implements SortableHeaderListener.SortableObject
{
  private JFIS sif;
  private GridBagLayout gridBLRuleFrame;
  private JSplitPane jSplitRules;
  private JPopupMenu jPopupMenuRules;
  private JMenuItem jPopupNewRule;
  private JMenuItem jPopupRemoveRule;
  private SortableTableModel RuleModel;
  private int NbIn;
  private int NbOut;
  private int NbRule;
  private Vector Title;
  private Vector Data;
  private JSemaphore JRuleFrameOpen;
  private Vector DoubleFieldInputs; // vecteur des champs utilise (utile pour la traduction)
  private JMenuBar jMenuBarRules;
  private JMenu jMenuRules;
  private JMenuItem jMenuNewRule;
  private JMenuItem jMenuRemoveRule;
  private JMenuItem jMenuPrint;
  private JMenuItem jMenuExport;
  private FISListener FISListener = null;
  private boolean ShowInactive = false;
  private boolean showExpertWeights = false;
  private JMenu jMenuDisplay;
  private JCheckBoxMenuItem jMenuDisplayInactive;
  private JCheckBoxMenuItem jMenuDisplayExpertWeights;
  private int sort_col = -1; /**< Memorisation du numero de colonne triees. -1 pour aucune colonne triee */
  private boolean is_ascent; /**< Memorisation ordre de tri */
  private Vector tables_headers; /**< Vecteur contenant les objets JTableHeader des tables triables */
  private JTable inputs_table; /**< Table des entrees */
  private JTable outputs_table; /**< Table des sorties */
  private SortButtonRenderer header_renderer; /**< Renderer des headers de table triable\n les 3 tables actives, entrees et sorties utilisent le meme objet renderer */
  private ListSelectionModel selection_model; /**< Modele de selection des tables\n les 3 tables actives, entrees et sorties utilisent le meme objet */
  private JScrollBar vertical_inputs_bar; /**< ascenseur vertical de la table entrees */
  private JScrollBar horizontal_inputs_bar; /**< ascenseur horizontal de la table entrees */
//------------------------------------------------------------------------------
  public JRuleFrame( JFISFrame parent, JFIS sif, JSemaphore open )
  {
    super(parent);
    this.sif = sif;
    this.JRuleFrameOpen = open;
    try
      {
      this.showExpertWeights = sif.hasExpertWeights() && !sif.hasImplicativeOutput();
      jbInit();
      // affichage de la fenetre
      this.pack();
      this.setLocation(this.ChildPosition(this.getSize()));
      this.setVisible(true);
      this.JRuleFrameOpen.Acquire();
      }
    catch( Throwable t ) { MessageFIS.Error( null, t ); }
  }
//------------------------------------------------------------------------------
  private Object ActionToObject( JOutput output, double action ) throws Throwable
  {
  // sortie nette
  if( output.GetNature() == JOutput.CRISP )
    // retourne un objet double
    return new Double( action );
  // sortie floue
  else
    return new Integer( (int)action );
  }
//------------------------------------------------------------------------------
  private boolean isRuleNumberColumn(int column) {
	  return column == 0;
  }
  
  private boolean isExpertWeightColumn(int column) {
	  return column == 1; 
  }  
 
  private boolean isActiveColumn(int column) {
	  return column == 2;
  }
  
  private boolean editableColumn(int column) {
	  return !isRuleNumberColumn(column) && !isActiveColumn(column);
  }
  
  private boolean isInputColumn(int column) {
	  return (column > 2) && (column < (NbIn + 3));
  }
  
  private int columnToInput(int column) {
	  return column - 3;
  }
 
  private boolean isOutputColumn(int column) {
	  return column >= (NbIn + 3); 
  }

  private int columnToOutput(int column) {
	  return column - NbIn - 3;
  }
  
  private void jbInit() throws Throwable
  {
    this.tables_headers = new Vector();
    gridBLRuleFrame = new GridBagLayout();
    jPopupMenuRules = new JPopupMenu();
    jPopupNewRule = new JMenuItem();
    jPopupRemoveRule = new JMenuItem();
    DoubleFieldInputs = new Vector();
    jMenuBarRules = new JMenuBar();
    jMenuRules = new JMenu();
    jMenuNewRule = new JMenuItem();
    jMenuRemoveRule = new JMenuItem();
    jMenuDisplay = new JMenu();
    jMenuDisplayInactive = new JCheckBoxMenuItem();
    jMenuDisplayExpertWeights = new JCheckBoxMenuItem();
    jMenuPrint = new JMenuItem();
    jMenuExport = new JMenuItem();

    NbIn = this.sif.NbInputs();
    NbOut = this.sif.NbOutputs();
    NbRule = this.sif.NbRules();

    // menus
    this.setJMenuBar(this.jMenuBarRules);
    jMenuBarRules.add(jMenuRules);
      jMenuRules.add(jMenuNewRule);
      jMenuRules.add(jMenuRemoveRule);
      jMenuRules.addSeparator();
      jMenuRules.add(this.jMenuPrint);
      jMenuRules.add(this.jMenuExport);
    jMenuBarRules.add(jMenuDisplay);
      jMenuDisplay.add(jMenuDisplayInactive);
      jMenuDisplayInactive.setSelected(this.ShowInactive);
      jMenuDisplay.add(jMenuDisplayExpertWeights);
      jMenuDisplayExpertWeights.setSelected(showExpertWeights);
      jMenuDisplayExpertWeights.setEnabled(!sif.hasImplicativeOutput());
    jPopupMenuRules.add(jPopupNewRule);
    jPopupMenuRules.add(jPopupRemoveRule);

    Title = new Vector();
    for( int i=0 ; i<NbIn+NbOut+3 ; i++ )
      Title.add(new String());

    Data = new Vector();
    for( int rule=0 ; rule<NbRule ; rule++ )
      {
      JRule jr = (JRule)this.sif.GetRule(rule);
      Vector rule_data = new Vector();
      rule_data.add(new Integer(rule+1));
      rule_data.add(new Double(jr.GetExpertWeight()));
      rule_data.add(new Boolean(jr.GetActive()));
      int [] facteurs = jr.Facteurs();
      for( int input=0 ; input<this.NbIn ; input ++ )
        rule_data.add(new Integer(facteurs[input]));
      double [] actions = jr.Actions();
      for( int output=0 ; output<this.NbOut ; output++ )
        rule_data.add(this.ActionToObject(this.sif.GetOutput(output), actions[output]));
      Data.add( rule_data );
      }

    // modele de table
    RuleModel = new SortableTableModel()
      {
      public Class getColumnClass(int c) { return getValueAt(0, c).getClass(); } // necessaire pour affichage des valeurs boolean en CheckBox
      public boolean isCellEditable(int row, int col)
        {
        if( isRuleNumberColumn(col) ) // la premiere colonne n'est pas editable
          return false;
        try
          {
          // si la regle est inactive
          if( (JRuleFrame.this.sif.GetRule(((Integer)this.getValueAt(row,0)).intValue()-1).GetActive() == false) && editableColumn(col) ) 
            return false;
          if( JRuleFrame.this.ShowInactive == true )
            {
            // si l'entree est inactive
            if( isInputColumn(col) )
              if( JRuleFrame.this.sif.GetInput(columnToInput(col)).GetActive() == false )
                return false;
            // si la sortie est inactive
            if( isOutputColumn(col) )
              if( JRuleFrame.this.sif.GetOutput(columnToOutput(col)).GetActive() == false )
                return false;
            }
          }
        catch( Throwable t ) { return false; }
        return true;
        }
      public void setValueAt(Object data, int row, int column) 
        {
        super.setValueAt(data, row, column);
        int rule = ((Integer)this.getValueAt(row,0)).intValue()-1; // numero de regle dans la premiere colonne
        try
          {
          if( isExpertWeightColumn(column) ) 
        	JRuleFrame.this.sif.SetRuleExpertWeight(rule, ((Double)data).doubleValue());  
          if( isActiveColumn(column) ) // colonne active
            {
            JRuleFrame.this.sif.SetRuleActive(rule, ((Boolean)data).booleanValue());
            JRuleFrame.this.repaint();  // force le renderer de colonne a redessinner
            }
          if( isInputColumn(column) ) // c'est une entree
            JRuleFrame.this.sif.SetRuleProp(rule, columnToInput(column), ((Integer)data).intValue());
          if( isOutputColumn(column) )  // c'est une sortie
            {
            double conc = 0;
            if( data instanceof Double )  // sortie nette
              conc = ((Double)data).doubleValue();
            if( data instanceof Integer ) // sortie floue
              conc = ((Integer)data).doubleValue();
            JRuleFrame.this.sif.SetRuleConc(rule, columnToOutput(column), conc);
            }
          }
        catch( Throwable t ) { MessageFIS.Error(null, t); }
        }
      };
    // affecte les vecteurs de donnees et de titre au modele de table
    RuleModel.setDataVector(Data, Title);

  this.RuleModel.setSorter(new util.sortabletable.TableSorter(this.RuleModel) // modification de la classe de triage
    {
    // surcharge de la fonction de tri de la classe TableSorter pour implementer les regles de tri suivantes:
    // - si 2 cellules sont egales, on tri sur les numeros de regles ascendants (colonne 0)
    // - sur les colonnes d'entrees:
    //     - si une cellule est nulle (pas de SEF), elle est toujours triee en fin de tableau
    public void sort(final int column, final boolean isAscent)
      {
      int n = model.getRowCount();
      int[] indexes = model.getIndexes();

      // affichage test du tri
      /*for( int y=0 ; y<model.getRowCount() ; y++ )
        System.out.print(model.getValueAt(y,column));
      System.out.println("");*/
      for (int i=0; i<n-1; i++)
        {
        int k = i;
        for (int j=i+1; j<n; j++)
          {
          if (isAscent)  // tri ascendant
            {
            if( isInputColumn(column) ) // colonnes d'entrees
              {
              if( this.Zero(column, j, k) == true ) // la cellule j est nulle et k > 0: on force le tri j > k
                k = j;
              else if( (compare(column, j, k) < 0) && (((Number)model.getValueAt(j, column)).doubleValue() != 0) ) // k > j et j != 0
                k = j;
              else if ((compare(column, j, k) == 0) && (compare(0, j, k) < 0)) // les cellules sont egales, tri sur les numeros de regles
                k = j;
              }
            else // colonnes de sorties
              {
              if( compare(column, j, k) < 0 )
                k = j;
              else if ((compare(column, j, k) == 0) && (compare(0, j, k) < 0)) // les cellules sont egales, tri sur les numeros de regles
                k = j;
              }
            }
          else // tri descendant
            {
            if (compare(column, j, k) > 0)
              k = j;
            else if ((compare(column, j, k) == 0) && (compare(0, j, k) < 0)) // les cellules sont egales, tri sur les numeros de regles
              k = j;
            }
          }
        int tmp = indexes[i];
        indexes[i] = indexes[k];
        indexes[k] = tmp;
        // affichage test du tri
        /*for( int y=0 ; y<model.getRowCount() ; y++ )
          System.out.print(model.getValueAt(y,column));
        System.out.println("");*/
        }
      }
    // fonction utilisee par la fonction sort ci-dessus pour implementer le tri particulier des cellules d'entrees nulles
    // return true si row2 est nulle et row1 non nulle, false sinon
    public boolean Zero(int column, int row1, int row2)
      {
      Object o1 = model.getValueAt(row1, column);
      Object o2 = model.getValueAt(row2, column);
      if( (o1 == null) || (o2 == null) ) { return  false; }
      else if( (((Number)o2).doubleValue() == 0) && (((Number)o1).doubleValue() != 0) ) { return true; }
      return false;
      }
    } );

    // renderer header
    this.header_renderer = new SortButtonRenderer()
      {
        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
          {
          Component comp = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
          ((JButton)comp).setEnabled(true);
          int display_inputs = 0; // nombre d'entrees affichees
          try { display_inputs = JRuleFrame.this.ShowInactive == true ? JRuleFrame.this.sif.NbInputs() : JRuleFrame.this.sif.NbActiveInputs(); } catch( Throwable t ) {}
          int col_mod = table.convertColumnIndexToModel(column);
          if( isRuleNumberColumn(col_mod) ) { ((JButton)comp).setText(LocaleFIS.GetString("Rule")); ((JButton)comp).setToolTipText(null); return comp; }
          if( isExpertWeightColumn(col_mod) ) { ((JButton)comp).setText(LocaleFIS.GetString("Weight")); ((JButton)comp).setToolTipText(LocaleFIS.GetString("ExpertWeight")); return comp; }
          if( isActiveColumn(col_mod) ) { ((JButton)comp).setText(LocaleFIS.GetString("Active")); ((JButton)comp).setToolTipText(null); return comp; }
          try
            {
            JInput input = null;
            if( (column == 0) && (display_inputs != 0) && isInputColumn(col_mod) ) // premiere entree
              {
              input = sif.GetInput(columnToInput(col_mod));
              ((JButton)comp).setText(LocaleFIS.GetString("IF") + " " + input.GetName());
              }
            if( (column > 0) && (column < display_inputs) && isInputColumn(col_mod)  )  // entrees suivantes
              {
              input = sif.GetInput(columnToInput(col_mod));
              ((JButton)comp).setText(LocaleFIS.GetString("AND") + " " + input.GetName());
              }
            if( column == 0 && isOutputColumn(col_mod) )  // premiere sortie
              {
              input = sif.GetOutput(columnToOutput(col_mod));
              ((JButton)comp).setText(LocaleFIS.GetString("THEN") + " " + input.GetName());
              }
            if( (column > 0) && isOutputColumn(col_mod) )  // sorties suivantes
              {
              input = sif.GetOutput(columnToOutput(col_mod));
              ((JButton)comp).setText(input.GetName());
              }
            // grise les entrees ou sorties inactives
            if( input != null ) {
            	((JButton)comp).setToolTipText(input.GetName());
              	((JButton)comp).setEnabled( input.GetActive());
            }
            }
          catch( Throwable t ) {}
          return comp;
          }
        };

    // modele de selection
    this.selection_model = new DefaultListSelectionModel();

    // tri des colonnes de la table
    if( this.sort_col != -1 )
      {
      header_renderer.setPressedColumn(this.sort_col);
      header_renderer.setSelectedColumn(this.sort_col);
      if( this.is_ascent == false ) // fait un appel supplementaire a SortButtonRenderer.setSelectedColumn pour mettre la variable SortButtonRenderer.state dans le bon etat
        header_renderer.setSelectedColumn(this.sort_col);
      this.RuleModel.sortByColumn(this.sort_col, this.is_ascent);
      header_renderer.setPressedColumn(-1);
      }


    if( this.jSplitRules != null )
      this.getContentPane().remove(this.jSplitRules);
    JScrollPane input_scroll = this.InputsTable();
    JScrollPane output_scroll = this.OutputsTable();
    // construction du splitPane
    jSplitRules = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, input_scroll, output_scroll )
      {
      public int getDividerSize() { return 2; }
      public Dimension getMinimumSize() { return  new Dimension(300,100); }
      };
    this.jSplitRules.setResizeWeight( (1.0 / (this.inputs_table.getColumnCount() + (this.showExpertWeights ? 3 : 2) + this.outputs_table.getColumnCount())) * (this.inputs_table.getColumnCount() + (this.showExpertWeights ? 3 : 2)));
    // asservi la scrollbar verticale de la table sortie sur la scrollbar verticale de la table entree
    this.vertical_inputs_bar = input_scroll.getVerticalScrollBar();
    this.horizontal_inputs_bar = input_scroll.getHorizontalScrollBar();
    JScrollBar outputs_bar = output_scroll.getVerticalScrollBar();
    this.vertical_inputs_bar.getModel().addChangeListener(new BoundedRangeModelChangeListener(outputs_bar.getModel()));

    this.getContentPane().setLayout(gridBLRuleFrame);

    jPopupNewRule.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jPopupNewRule_actionPerformed(e); } });
    jPopupRemoveRule.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jPopupRemoveRule_actionPerformed(e); } });
    this.getContentPane().add(jSplitRules,     new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(10, 10, 10, 10), -2, 0));
    jMenuPrint.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuPrint_actionPerformed(e); } });
    jMenuExport.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuExport_actionPerformed(e); } });

    // traduction
    this.Translate();

    // evenements
    this.sif.RemoveFISListener(this.FISListener);
    this.FISListener = new FISListener()
      {
      public void FISClosed()                        { JRuleFrame.this.dispose(); }
      public void InputReplaced( int input_number )   { JRuleFrame.this.ReInit(); }
      public void OutputReplaced( int output_number ) { JRuleFrame.this.ReInit(); }
      public void InputRemoved( int input_number )   { JRuleFrame.this.ReInit(); }
      public void OutputRemoved( int output_number ) { JRuleFrame.this.ReInit(); }
      public void InputAdded( int input_number )     { JRuleFrame.this.ReInit(); }
      public void OutputAdded( int output_number )   { JRuleFrame.this.ReInit(); }
      public void InputActiveChanged( int input_number )   { JRuleFrame.this.ReInit(); }
      public void OutputActiveChanged( int output_number ) { JRuleFrame.this.ReInit(); }
      public void InputNameChanged( int input_number )     { JRuleFrame.this.repaint(); }
      public void OutputNameChanged( int output_number )   { JRuleFrame.this.repaint(); }
      public void InputRangeChanged( int input_number )    { JRuleFrame.this.repaint(); }
      public void OutputRangeChanged( int output_number )  { JRuleFrame.this.repaint(); }
      public void MFRemovedInInput( int input_number )   { JRuleFrame.this.ReInit();  }
      public void MFRemovedInOutput( int output_number ) { JRuleFrame.this.ReInit(); }
      public void MFAddedInInput( int input_number )     { JRuleFrame.this.ReInit();  }
      public void MFAddedInOutput( int output_number )   { JRuleFrame.this.ReInit(); }
      public void MFReplacedInInput( int input_number )   { JRuleFrame.this.ReInit(); }
      public void MFReplacedInOutput( int output_number ) { JRuleFrame.this.ReInit(); }
      public void OutputDefuzChanged( int output_number ) {}
      public void OutputImpliChanged( int output_number ) {}
      public void OutputDisjChanged( int output_number ) {}
      public void OutputClassifChanged( int output_number ) {}
      public void OutputDefaultChanged( int output_number ) {}
      public void OutputAlarmThresChanged( int output_number ) {}
      public void RulesModified() {}
      public void ConjunctionChanged() {}
      };
    this.sif.AddFISListener(this.FISListener);
    jMenuRules.addMenuListener(new MenuListener()
      {
      public void menuSelected(MenuEvent e) { jMenuRules_menuSelected(e); }
      public void menuDeselected(MenuEvent e) {}
      public void menuCanceled(MenuEvent e) {}
      });
    jMenuNewRule.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuNewRule_actionPerformed(e); } });
    jMenuRemoveRule.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuRemoveRule_actionPerformed(e); } });
    jMenuDisplayInactive.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuDisplayInactive_actionPerformed(e); } });
    jMenuDisplayExpertWeights.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jMenuDisplayExpertWeights_actionPerformed(e); } });
  }
//------------------------------------------------------------------------------
  public void Translate() throws Throwable
  {
    jMenuRules.setText(LocaleFIS.GetString("Rules"));
    jMenuNewRule.setText(LocaleFIS.GetString("NewRule"));
    jMenuRemoveRule.setText(LocaleFIS.GetString("RemoveRule"));
    jMenuDisplay.setText(LocaleFIS.GetString("Display"));
    jMenuDisplayInactive.setText(LocaleFIS.GetString("InactiveVariables"));
    jMenuDisplayExpertWeights.setText(LocaleFIS.GetString("ExpertWeight"));
    jMenuPrint.setText(LocaleFIS.GetString("Print"));
    jMenuExport.setText(LocaleFIS.GetString("Export"));
    jPopupNewRule.setText(LocaleFIS.GetString("NewRule"));
    jPopupRemoveRule.setText(LocaleFIS.GetString("RemoveRule"));

    this.setTitle(LocaleFIS.GetString("Rules"));
    for( int i=0 ; i<this.DoubleFieldInputs.size() ; i++ )
      {
      DoubleFieldInput df = (DoubleFieldInput)this.DoubleFieldInputs.elementAt(i);
      double value = df.getValue();
      df.setLocale(LocaleFIS.Locale());
      df.setValue(value);
      }
    this.repaint();
  }
//------------------------------------------------------------------------------
  public void dispose()
  {
  super.dispose();
  this.JRuleFrameOpen.Release();
  this.sif.RemoveFISListener(this.FISListener);
  }
//------------------------------------------------------------------------------
  private void ReInit()
  {
  try
    {
    int location = this.jSplitRules.getDividerLocation();
    int vertical_value = this.vertical_inputs_bar.getModel().getValue();
    int horizontal_value = this.horizontal_inputs_bar.getModel().getValue();
    this.showExpertWeights = this.showExpertWeights && !sif.hasImplicativeOutput();
    jbInit();
    this.jSplitRules.setDividerLocation(location);
    this.vertical_inputs_bar.getModel().setValue(vertical_value);
    this.horizontal_inputs_bar.getModel().setValue(horizontal_value);
    this.repaint();
    }
  catch( Throwable t ) { MessageFIS.Error( null, t ); }
  }
//------------------------------------------------------------------------------
  private void SetUpFuzzyColumn( final TableColumn Column, final JInput input ) throws Throwable
  {
  final JComboBox comboBox = new JComboBox();
  if( input instanceof JOutput ) {}
  else comboBox.addItem(new Integer(0));
  for( int i=0 ; i<input.GetNbMF() ; i++ )
    comboBox.addItem(new Integer(i+1));

  // renderer de la combo box
  comboBox.setRenderer(new BasicComboBoxRenderer()
    {
    public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
      {
      super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
      // centrage des elements de la combo box
      super.setHorizontalAlignment(SwingConstants.CENTER);
      if( index == -1 ) return this.getListCellRendererComponent(list, value, comboBox.getSelectedIndex(), isSelected, cellHasFocus);
      if( input instanceof JOutput )
        {
        try { super.setText(input.GetMF(index).GetName()); }
        catch( Throwable t ) {}
        }
      else
        {
        if( index == 0 )
          super.setText(new String());
        else
          {
          try { super.setText(input.GetMF(index-1).GetName()); }
          catch( Throwable t ) {}
          }
        }
      return this;
      }
    });

  // editeur de cellule de la colonne
  Column.setCellEditor(new DefaultCellEditor(comboBox)
    {
    public Object getCellEditorValue() { return new Integer(input instanceof JOutput ? comboBox.getSelectedIndex()+1 : comboBox.getSelectedIndex()); }
    });

  // renderer de la colonne
  Column.setCellRenderer(new DefaultTableCellRenderer()
    {
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
      {
      super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
      if( ((Integer)value).intValue() == 0 )
        this.setText("");
      else
        {
        try { this.setText(((JMF)input.GetMF(((Integer)value).intValue()-1)).GetName()); }
        catch (Throwable t ) { this.setText(""); }
        }
      this.setHorizontalAlignment(SwingConstants.CENTER);
      // grise les colonnes correspondant a des entrees ou sorties inactives
      this.setForeground(Color.black);
      try
        {
        if( JRuleFrame.this.ShowInactive == true )
          {
          int col_mod = table.convertColumnIndexToModel(column);
          if( isInputColumn(col_mod) )
            if( JRuleFrame.this.sif.GetInput(columnToInput(col_mod)).GetActive() == false )
              this.setForeground(Color.gray);
          if( isOutputColumn(col_mod) )
            if( JRuleFrame.this.sif.GetOutput(columnToOutput(col_mod)).GetActive() == false )
              this.setForeground(Color.gray);
          }
        // grise les regles inactives
        if( sif.GetRule(((Integer)table.getModel().getValueAt(row,0)).intValue()-1).GetActive() == false )
          super.setForeground(Color.gray);
        }
      catch( Throwable t ) {}
      return this;
      }
    });
  }
//------------------------------------------------------------------------------
  private void SetUpCrispColumn( final TableColumn Column, JOutput output ) throws Throwable
  {
  final DoubleFieldInput jtf = new DoubleFieldInput(output);
  this.DoubleFieldInputs.addElement(jtf);
  jtf.setHorizontalAlignment(SwingConstants.CENTER);
  jtf.setBorder(BorderFactory.createEmptyBorder());

  // editeur de cellule de la colonne
  Column.setCellEditor(new DefaultCellEditor(jtf)
    {
    public Object getCellEditorValue() { return new Double(jtf.getValue()); }
    public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column)
      {
      Component c = super.getTableCellEditorComponent(table, value, isSelected, row, column);
      jtf.setValue(((Double)value).doubleValue());
      return c;
      }
    } );

  // renderer de la colonne
  Column.setCellRenderer(new DefaultTableCellRenderer()
    {
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
      {
      super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
      try
        {
        this.setForeground(Color.black);
        // affichage en rouge des valeurs hors domaine
        super.setText(jtf.NumberFormat().format(value));
        double [] domain = jtf.Input().GetRange();
        if( (((Double)value).doubleValue() < domain[0]) || (((Double)value).doubleValue() > domain[1]) )
          this.setForeground(Color.red);
        // grise si la sortie est inactive
        if( jtf.Input().GetActive() == false )
          this.setForeground(Color.gray);
        // grise si la regle est inactive
        if( JRuleFrame.this.sif.GetRule(((Integer)table.getModel().getValueAt(row,0)).intValue()-1).GetActive() == false )
          this.setForeground(Color.gray);
        }
      catch( Throwable t ) {}
      this.setHorizontalAlignment(SwingConstants.CENTER);
      return this;
      }
    } );
  }
//------------------------------------------------------------------------------
  private void SetUpOutputColumn( TableColumn Column, JOutput output ) throws Throwable
  {
  if( output.GetNature() == JOutput.CRISP )
    // si la sortie est nette
    SetUpCrispColumn( Column, output ) ;
  else
    // sortie floue
    SetUpFuzzyColumn( Column, output );
  }
//------------------------------------------------------------------------------
  private void NewRule()
  {
  try
    {
    int [] props = new int [this.sif.NbInputs()];
    for( int i=0 ; i<props.length ; i++ )
      props[i] = 0;
    double [] concs = new double [this.sif.NbOutputs()]  ;
    for( int i=0 ; i<concs.length ; i++ )
      concs[i] = 1;
    this.sif.AddRule(this.sif.NewRule(true, props, concs));
    this.ReInit();
    this.vertical_inputs_bar.getModel().setValue(Integer.MAX_VALUE); // force l'ascenseur vertical en bas
    }
  catch( Throwable en ) { MessageFIS.Error( null, en ); }
  }
//------------------------------------------------------------------------------
  void jPopupNewRule_actionPerformed(ActionEvent e) { this.NewRule(); }
//------------------------------------------------------------------------------
  private void RemoveRule()
  {
  try
    {
    this.sif.RemoveRule(((Integer)this.RuleModel.getValueAt(this.inputs_table.getSelectedRow(),0)).intValue()-1);
    this.ReInit();
    }
  catch( Throwable t ) { MessageFIS.Error( null, t ); }
  }
//------------------------------------------------------------------------------
  void jPopupRemoveRule_actionPerformed(ActionEvent e) { this.RemoveRule(); }
//------------------------------------------------------------------------------
  void jMenuRules_menuSelected(MenuEvent e)
  {
  if( this.inputs_table.getSelectedRow() == -1 ) // aucune regle n'est selectionnee
      this.jMenuRemoveRule.setEnabled(false);
    else
      this.jMenuRemoveRule.setEnabled(true);
  }
//------------------------------------------------------------------------------
  void jMenuNewRule_actionPerformed(ActionEvent e) { this.NewRule(); }
//------------------------------------------------------------------------------
  void jMenuRemoveRule_actionPerformed(ActionEvent e) { this.RemoveRule(); }
//------------------------------------------------------------------------------
  void jMenuDisplayInactive_actionPerformed(ActionEvent e)
  {
  this.ShowInactive = this.jMenuDisplayInactive.isSelected();
  this.ReInit();
  }
//------------------------------------------------------------------------------
  void jMenuDisplayExpertWeights_actionPerformed(ActionEvent e)
  {
  this.showExpertWeights = this.jMenuDisplayExpertWeights.isSelected();
  this.ReInit();
  }
//------------------------------------------------------------------------------
  void jMenuPrint_actionPerformed(ActionEvent e)
  {
  try
    {
    JPrintPreviewTable pp = new JPrintPreviewTable(this, new JPrintTable(this.RuleModel));
    pp.Show();
    }
  catch (Exception ex) { MessageFIS.Error(null, ex); }
  }
//------------------------------------------------------------------------------
  void jMenuExport_actionPerformed(ActionEvent e)
  {
  try
    {
    ExportDialog export = new ExportDialog();
    final JPrintTable table = new JPrintTable(this.RuleModel);
    table.setSize(table.getPreferredSize());
    table.getTableHeader().setSize(table.getTableHeader().getPreferredSize());
    JPanel panel = new JPanel()
      {
        public void paint(Graphics g)
          {
          table.getTableHeader().paint(g);
          g.translate(0,table.getTableHeader().getHeight());
          table.paint(g);
          g.setColor(Color.gray);
          g.drawLine(0,0,0,table.getHeight()-1);
          }
      };
    panel.setSize(new Dimension(table.getWidth(), table.getHeight() + table.getTableHeader().getHeight()));
    export.showExportDialog( panel, "Export view as ...", panel, "export" );
    }
  catch (Exception ex) { MessageFIS.Error(null, ex); }
  }
//------------------------------------------------------------------------------
  public void SetSortedColumn( int column ) { this.sort_col = column; }
  public int GetSortedColumn() { return this.sort_col; }
  public void SetSortAscent( boolean is_ascent ) { this.is_ascent = is_ascent; }
  public Vector GetTablesHeaders() { return this.tables_headers; }

//------------------------------------------------------------------------------
/**
 * Classe de table imprimable
 */
  public class JPrintTable extends JTable
  {
  public JPrintTable(TableModel dm) 
    {
    super( dm );
    // modele de colonne
    this.setColumnModel(new DefaultTableColumnModel()
      {
      public void addColumn(TableColumn tc)
        {
        // affichage du numero de regle
        if( isRuleNumberColumn(tc.getModelIndex()) ) { super.addColumn(tc); return; }
        try {
			if( isExpertWeightColumn(tc.getModelIndex()) && sif.hasExpertWeights() ) { super.addColumn(tc); return; }
		} catch (Throwable e) {}
		// suppression affichage active
		if( isActiveColumn(tc.getModelIndex()) ) return;
        // affichage des variables inactives
        if( (JRuleFrame.this.ShowInactive == true) || (InputActive(tc.getModelIndex()) == true) ) { super.addColumn(tc); return; }
        // affichage des sorties inactives
        if( (JRuleFrame.this.ShowInactive == true) || (OutputActive(tc.getModelIndex()) == true) ) { super.addColumn(tc); return; }
        }
      // true si l'entree model_index est active
      private boolean InputActive(int model_index)
        {
        if( isInputColumn(model_index) )
          try { return JRuleFrame.this.sif.GetInput(columnToInput(model_index)).GetActive(); } catch( Throwable t ) { MessageFIS.Error("Error", t); return false;}
        return false;
        }
      // true si la sortie model_index est active
      private boolean OutputActive(int model_index)
        {
        if( isOutputColumn(model_index) )
          try { return JRuleFrame.this.sif.GetOutput(columnToOutput(model_index)).GetActive(); } catch( Throwable t ) { MessageFIS.Error("", t); return false;}
        return false;
        }
      });
    this.createDefaultColumnsFromModel();

    // renderers d'entete
    for( int i = 0; i < this.getColumnCount(); i++ )
      this.getColumnModel().getColumn(i).setHeaderRenderer(new DefaultTableCellRenderer()
        {
        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
          {
          super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
          int col_mod = table.convertColumnIndexToModel(column);
          super.setHorizontalAlignment(SwingConstants.CENTER);
          //super.setBorder(UIManager.getBorder("TableHeader.cellBorder"));
          super.setBorder(new javax.swing.border.LineBorder(Color.black));
          if( isRuleNumberColumn(col_mod) ) { super.setText(LocaleFIS.GetString("Rule")); }
          if( isExpertWeightColumn(col_mod) ) { super.setText(LocaleFIS.GetString("ExpertWeights")); }
          try
            {
            JInput input = null;
            if( isInputColumn(col_mod) )  // entrees
              {
              input = sif.GetInput(columnToInput(col_mod));
              super.setText(input.GetName());
              }
            if( isOutputColumn(col_mod) )  // sorties
              {
              input = sif.GetOutput(columnToOutput(col_mod));
              super.setText(input.GetName());
              }
            // grise les entrees ou sorties inactives
            if( input != null )
              super.setForeground( input.GetActive() == false ? Color.lightGray : Color.black );
            }
          catch( Throwable t ) { MessageFIS.Error("", t);}
          return this;
          }
        });

  // renderers de colonnes
  for( int i = 0; i < this.getColumnCount(); i++ )
    this.getColumnModel().getColumn(i).setCellRenderer( new DefaultTableCellRenderer()
      {
      public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
        {
        super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
        this.setHorizontalAlignment(SwingConstants.CENTER);
        int col_mod = table.convertColumnIndexToModel(column);
        try
          {
          JInput input = null;
          if( isInputColumn(col_mod) )  // affiche les SEFs des entrees
            {
            input = sif.GetInput(columnToInput(col_mod));
            super.setText(((Integer)value).intValue() == 0 ? " " : input.GetMF(((Integer)value).intValue()-1).GetName());
            }
          if( isOutputColumn(col_mod) && (sif.GetOutput(columnToOutput(col_mod)).GetNature() == JOutput.FUZZY) )  // affiche les SEFs des sorties floues
            {
            input = sif.GetOutput(columnToOutput(col_mod));
            super.setText(input.GetMF(((Integer)value).intValue()-1).GetName());
            }
         super.setForeground(Color.black);
         // grise les regles inactives
         if( JRuleFrame.this.sif.GetRule(((Integer)table.getModel().getValueAt(row,0)).intValue()-1).GetActive() == false )
           super.setForeground( Color.lightGray );
          // grise les entrees ou sorties inactives
          if( (input != null) && (input.GetActive() == false) )
            super.setForeground( Color.lightGray );
          }
        catch( Throwable t ) { MessageFIS.Error("", t); }
        return this;
        }
      });

    }
  }
//------------------------------------------------------------------------------
  class JRuleTable extends JTable
  {
  public JRuleTable(TableModel dm, TableColumnModel cm, ListSelectionModel sm)
    {
    super( dm, cm, sm );
    this.createDefaultColumnsFromModel();
    // renderers
    for (int i = 0; i < this.getColumnCount(); i++)
      this.getColumnModel().getColumn(i).setHeaderRenderer(JRuleFrame.this.header_renderer);
    this.getTableHeader().addMouseListener(new SortableHeaderListener(this.getTableHeader(), JRuleFrame.this.header_renderer, JRuleFrame.this)); // listener de header triable
    JRuleFrame.this.tables_headers.add(this.getTableHeader());
    this.getTableHeader().setReorderingAllowed(false);
    //this.getTableHeader().setResizingAllowed(false);
    this.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    this.addMouseListener(new MouseAdapter() { public void mousePressed(MouseEvent e)
      {
      if (SwingUtilities.isRightMouseButton(e))
        {
        if( JRuleTable.this.getSelectedRow() == -1 )    // masque les item du popup non utile
          JRuleFrame.this.jPopupRemoveRule.setEnabled(false);
        else
          JRuleFrame.this.jPopupRemoveRule.setEnabled(true);
        SwingUtilities.updateComponentTreeUI(JRuleFrame.this.jPopupMenuRules);   // affecte le look actuel au composant jPopupMenuInputs
        jPopupMenuRules.show(JRuleTable.this, e.getX(), e.getY());   // affichage du popup
        }
      } });
    this.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE); 
    }
  public Dimension getPreferredScrollableViewportSize() { return new Dimension(this.getPreferredSize().width, this.getPreferredSize().height); }
  }
//------------------------------------------------------------------------------
  private JScrollPane InputsTable() throws Throwable
  {
  this.inputs_table = new JRuleTable(this.RuleModel, new DefaultTableColumnModel()
    {
    public void addColumn(TableColumn tc)
      {
      int index = tc.getModelIndex();
      if( isInputColumn(index) )
        {
        boolean active = false;
        try{ active = JRuleFrame.this.sif.GetInput(columnToInput(index)).GetActive(); } catch( Throwable t ) {}
        if( (JRuleFrame.this.ShowInactive == true) || (active == true) )
          super.addColumn(tc);
        }
      }
    }, this.selection_model);

  for( int i=0 ; i<this.inputs_table.getColumnCount() ;  i++ )
    SetUpFuzzyColumn( this.inputs_table.getColumnModel().getColumn(i), sif.GetInput(columnToInput(this.inputs_table.convertColumnIndexToModel(i))) );

  // construction du panel de retour
  JPanel inputs_panel = new JPanel(new BorderLayout());
  inputs_panel.add(this.inputs_table, BorderLayout.CENTER);
  JScrollPane scroll = new JScrollPane(inputs_panel);
  scroll.setColumnHeaderView(this.inputs_table.getTableHeader());
  JTable active_table = this.ActiveTable();
  JPanel active_panel = new JPanel(new BorderLayout());
  active_panel.add(active_table, BorderLayout.CENTER);
  scroll.setRowHeaderView(active_panel);
  scroll.setCorner(JScrollPane.UPPER_LEFT_CORNER, active_table.getTableHeader());
  return scroll;
  }
//------------------------------------------------------------------------------
  
  private class ActiveTableCellRenderer extends DefaultTableCellRenderer {
	  
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
      {
      super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
      this.setHorizontalAlignment(SwingConstants.CENTER);
      try
        {
        // grise les regles inactives
        if( JRuleFrame.this.sif.GetRule(((Integer)table.getModel().getValueAt(row,0)).intValue()-1).GetActive() == false )
          super.setForeground(Color.gray);
        else
          super.setForeground(Color.black);
        }
      catch( Throwable t ) {}
      return this;
      }
  }
  
  private class ExpertWeightCellRenderer extends ActiveTableCellRenderer {
	  
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
    	super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
    	this.setText(NumberFormat.getNumberInstance().format(value));
    	return this;
    }
  }
  
  private class ExpertWeightCellEditor extends DefaultCellEditor {

	  public ExpertWeightCellEditor(DoubleField field) { 
		  super(field);  
		  field.setHorizontalAlignment(SwingConstants.CENTER);
		  field.setBorder(BorderFactory.createEmptyBorder());
	  }
	  
	  public Object getCellEditorValue() { return new Double(((DoubleField)super.getComponent()).getValue()); }
	  
	  public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
		  DoubleField field = (DoubleField)super.getTableCellEditorComponent(table, value, isSelected, row, column);
		  field.setValue(((Double)value).doubleValue());
		  return field;
	  }

	@Override
	public boolean stopCellEditing() {
		final DoubleField field = (DoubleField)super.getComponent();
		if(field.getValue() <= 0) {
			SwingUtilities.invokeLater(new Runnable() {

				@Override
				public void run() {
					JOptionPane.showMessageDialog(SwingUtilities.getWindowAncestor(field), LocaleFIS.GetString("ExpertWeight") + " " + LocaleFIS.GetString("MustBePositive"), null, JOptionPane.WARNING_MESSAGE);
				}});
			return false;
		}
		return super.stopCellEditing();
	}
  };

  private JTable ActiveTable() throws Throwable
  {
  JTable table = new JRuleTable(this.RuleModel, new DefaultTableColumnModel()
    {
    public void addColumn(TableColumn tc)
      {
        if( isRuleNumberColumn(tc.getModelIndex()) ) {
            super.addColumn(tc);
            tc.setCellRenderer(new ActiveTableCellRenderer());
        }
        if( isExpertWeightColumn(tc.getModelIndex()) && jMenuDisplayExpertWeights.isSelected() ) {
            super.addColumn(tc);
        	tc.setCellRenderer(new ExpertWeightCellRenderer());
        	tc.setCellEditor(new ExpertWeightCellEditor(new DoubleField()));
        }
        if( isActiveColumn(tc.getModelIndex()) )
            super.addColumn(tc);
      }
    }, this.selection_model);
  table.getTableHeader().setResizingAllowed(false);
  return table;
  }
//------------------------------------------------------------------------------
  private JScrollPane OutputsTable() throws Throwable
  {
  this.outputs_table = new JRuleTable(this.RuleModel, new DefaultTableColumnModel()
    {
    public void addColumn(TableColumn tc)
      {
      int index = tc.getModelIndex();
      if( isOutputColumn(index) )
        {
        boolean active = false;
        try{ active = JRuleFrame.this.sif.GetOutput(columnToOutput(index)).GetActive(); } catch( Throwable t ) {}
        if( (JRuleFrame.this.ShowInactive == true) || (active == true) )
          super.addColumn(tc);
        } 
      }
    }, this.selection_model);

  for( int i=0 ; i<this.outputs_table.getColumnCount() ;  i++ )
	if(isOutputColumn(this.outputs_table.convertColumnIndexToModel(i)))
      SetUpOutputColumn( this.outputs_table.getColumnModel().getColumn(i), sif.GetOutput(columnToOutput(this.outputs_table.convertColumnIndexToModel(i))) );

  // construction du panel de retour
  JPanel panel = new JPanel(new BorderLayout());
  panel.add(this.outputs_table, BorderLayout.CENTER);
  JScrollPane scroll = new JScrollPane(panel);
  scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER); // cache la scrollbar verticale
  scroll.setColumnHeaderView(this.outputs_table.getTableHeader());
  return scroll;
  }
//------------------------------------------------------------------------------
  public void setSize(Dimension d)
    {
    super.setSize(d);
    if (this.jSplitRules.getDividerLocation() > d.width )
      this.jSplitRules.setDividerLocation(d.width-100);
    }
}









