//***********************************************************************
//
//                              JSimpleFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.border.*;
import java.util.*;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;

//------------------------------------------------------------------------------
public class JSimpleFrame extends JFISDialog
{
  private JExtendedDataFile DataFile;
  private JFISFrame Parent;
  private JFIS fis;
  private ButtonGroup jbgOutput = new ButtonGroup();
  private JPanel jPanelOutput = new JPanel(new GridBagLayout());
  private JLabel jLabelHomogeneityThreshold = new JLabel();
  private DoubleField jDFHomogeneityThreshold = new DoubleField();
  private JLabel jLabelLossOfCoverage = new JLabel();
  private DoubleField jDFLossOfCoverage = new DoubleField();
  private JLabel jLabelMinMatchingDegree = new JLabel();
  private DoubleField jDFMinMatchingDegree = new DoubleField();
  private JRadioButton jRBLossOfPerformance = new JRadioButton();
  private DoubleField jDFLossOfPerformance = new DoubleField();
  private JRadioButton jRBMaxError = new JRadioButton();
  private DoubleField jDFMaxError = new DoubleField();
  private JCheckBox jCBKeepLastRule = new JCheckBox();
  private JPanel jPanelValidation = new JPanel(new GridBagLayout());
  private JButton jButtonApply = new JButton();
  private JButton jButtonCancel = new JButton();
private JRadioButton jRadioButtonLossOfCoverage = null;  //  @jve:decl-index=0:visual-constraint="455,255"
private JRadioButton jRadioButtonCoverageMin = null;  //  @jve:decl-index=0:visual-constraint="402,187"
private ButtonGroup buttonGroupCoverage = null;  //  @jve:decl-index=0:visual-constraint="472,215"
private DoubleField doubleFieldCoverageMin = null;  //  @jve:decl-index=0:visual-constraint="349,193"
private JCheckBox jCheckBoxRuleRemoval = null;  //  @jve:decl-index=0:visual-constraint="154,232"
private JCheckBox jCheckBoxVariableRemoval = null;  //  @jve:decl-index=0:visual-constraint="425,161"
private JPanelValidationFile jPanelValidationFile = null;
//------------------------------------------------------------------------------
  public JSimpleFrame( JFISFrame parent, JFIS fis, JExtendedDataFile data_file )
  {
    this.Parent = parent;
    this.fis = fis;
    this.DataFile = data_file;
    try { jbInit(); }
    catch(Throwable t) { MessageFIS.Error(null, t); }
  }
//------------------------------------------------------------------------------
  private void jbInit() throws Throwable
  {
    this.getContentPane().setLayout(new GridBagLayout());

    this.jPanelOutput.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),LocaleFIS.GetString("Outputs")));
    boolean first_active = false;
    for( int i=0 ; i<this.fis.NbOutputs() ; i++ )
      {
      JRadioButton jrb = new JRadioButton(this.fis.GetOutput(i).GetName());
      this.jbgOutput.add(jrb);
      if( i == 0 )
        jrb.setSelected(true);
      if( this.fis.GetOutput(i).GetActive() == false )
        jrb.setEnabled(false);
      else
        {
        if( first_active == false )
          {
          jrb.setSelected(true);
          first_active = true;
          }
        }
      this.jPanelOutput.add(jrb, new GridBagConstraints(i%2==0?0:1, i/2, 1, 1, 0.0, 0.0
          ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets( 0, 0, 0, 0), 0, 0));
      }
    this.getContentPane().add(this.jPanelOutput,   new GridBagConstraints(0, 0, 5, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 0, 5), 0, 0));

    this.getContentPane().add(jLabelHomogeneityThreshold,     new GridBagConstraints(0, 10, 2, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jDFHomogeneityThreshold.setValue(0.8);
    this.getContentPane().add(jDFHomogeneityThreshold,   new GridBagConstraints(2, 10, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));

    this.getButtonGroupCoverage().add(this.getJRadioButtonCoverageMin());
    this.getButtonGroupCoverage().add(this.getJRadioButtonLossOfCoverage());
    this.getContentPane().add(this.getJRadioButtonCoverageMin(),     new GridBagConstraints(0, 1, 2, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.getDoubleFieldCoverageMin().setValue(90);
    this.getContentPane().add(this.getDoubleFieldCoverageMin(),   new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));
    this.getContentPane().add(new JLabel("%"),  new GridBagConstraints(3, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, 0));
    this.getContentPane().add(this.getJRadioButtonLossOfCoverage(),     new GridBagConstraints(0, 2, 2, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
    this.jDFLossOfCoverage.setValue(10);
    this.jDFLossOfCoverage.setEnabled(false);
    this.getContentPane().add(jDFLossOfCoverage,   new GridBagConstraints(2, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 40, 0));
    this.getContentPane().add(new JLabel("%"),  new GridBagConstraints(3, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));

    this.getContentPane().add(jLabelMinMatchingDegree,    new GridBagConstraints(0, 11, 2, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jDFMinMatchingDegree.setValue(0.2);
    this.getContentPane().add(jDFMinMatchingDegree,   new GridBagConstraints(2, 11, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));

    ButtonGroup bg = new ButtonGroup();
    bg.add(this.jRBLossOfPerformance);
    this.getContentPane().add(jRBLossOfPerformance,    new GridBagConstraints(0, 5, 2, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
    this.jDFLossOfPerformance.setValue(10);
    this.jDFLossOfPerformance.setEnabled(false);
    this.getContentPane().add(jDFLossOfPerformance,   new GridBagConstraints(2, 5, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 40, 0));
    this.getContentPane().add(new JLabel("%"),  new GridBagConstraints(3, 5, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, 0));
    bg.add(this.jRBMaxError);
    this.getContentPane().add(jRBMaxError,    new GridBagConstraints(0, 6, 2, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
    this.jDFMaxError.setValue(-1);
    this.jDFMaxError.setEnabled(false);
    this.getContentPane().add(jDFMaxError,   new GridBagConstraints(2, 6, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 40, 0));

    this.jCBKeepLastRule.setSelected(true);
    this.getContentPane().add(jCBKeepLastRule,   new GridBagConstraints(0, 7, 3, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, 0));
    this.getContentPane().add(this.getJCheckBoxRuleRemoval(),   new GridBagConstraints(0, 8, 3, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
    this.getContentPane().add(this.getJCheckBoxVariableRemoval(),   new GridBagConstraints(0, 9, 3, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));

    // panel validation file
    jPanelValidationFile = new JPanelValidationFile(this.DataFile);
    this.getContentPane().add(jPanelValidationFile, new GridBagConstraints(0, 12, 5, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
    
    // panel validation
    jPanelValidation.add(jButtonApply,  new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    jPanelValidation.add(jButtonCancel,  new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    this.getContentPane().add(jPanelValidation,          new GridBagConstraints(0, 13, 5, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));

    // evenements
    this.jRBLossOfPerformance.addItemListener(new java.awt.event.ItemListener() { public void itemStateChanged(ItemEvent e) { jRBLossOfPerformance_itemStateChanged(e); } });
    this.jRBMaxError.addItemListener(new java.awt.event.ItemListener() { public void itemStateChanged(ItemEvent e) { jRBMaxError_itemStateChanged(e); } });
    this.jRBLossOfPerformance.setSelected(true);
    jButtonApply.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonApply_actionPerformed(e); } });
    jButtonCancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonCancel_actionPerformed(e); } });

    // affichage
    this.Translate();
    this.pack();
    this.setModal(true);
    this.setLocation(JFISChildFrame.ChildPosition(this.Parent, this.getSize()));
    this.setVisible(true);
  }
//------------------------------------------------------------------------------
  private void Translate()
  {
    this.setTitle(LocaleFIS.GetString("Simple"));
    this.jLabelHomogeneityThreshold.setText(LocaleFIS.GetString("HomogeneityThreshold") + ":");
    this.jLabelLossOfCoverage.setText(LocaleFIS.GetString("LossOfCoverage") + ":");
    this.jLabelMinMatchingDegree.setText(LocaleFIS.GetString("MinimumMatchingDegree") + ":");
    this.jRBLossOfPerformance.setText(LocaleFIS.GetString("LossOfPerformance") + ":");
    this.jRBMaxError.setText(LocaleFIS.GetString("Performance") + " max" + ":");
    this.jCBKeepLastRule.setText(LocaleFIS.GetString("KeepLastRule"));
    jButtonApply.setText(LocaleFIS.GetString("Apply"));
    jButtonCancel.setText(LocaleFIS.GetString("Cancel"));
  }
//------------------------------------------------------------------------------
  void jButtonCancel_actionPerformed(ActionEvent e) { this.dispose(); }
//------------------------------------------------------------------------------
  private int OutputSelected()
  {
  int retour = 0;
  for( Enumeration e = this.jbgOutput.getElements() ; e.hasMoreElements() ; retour++ )
    if( ((AbstractButton)e.nextElement()).isSelected() == true )
      return retour;
  return retour;
  }
//------------------------------------------------------------------------------
  void jButtonApply_actionPerformed(ActionEvent e)
  {
  try
    {
    // teste erreur max positif
    if( (this.jDFMaxError.getValue() <= 0) && (this.jRBMaxError.isSelected() == true) )
      {
      MessageFIS.Error(this, LocaleFIS.GetString("Error"), LocaleFIS.GetString("MaxError") + " "  + LocaleFIS.GetString("MustBePositive"));
      return;
      }
    JFIS new_fis = this.fis.NewFISSIMPLE(this.DataFile, this.DataFile.FileName() + ".simple", this.jDFHomogeneityThreshold.getValue(), this.jRBLossOfPerformance.isSelected() == true ? this.jDFLossOfPerformance.getValue()/100.0 : 0.0, this.OutputSelected(), this.jDFMinMatchingDegree.getValue(), this.jRadioButtonCoverageMin.isSelected() ? (1.0 -(this.doubleFieldCoverageMin.getValue()/100.0)) : 0.0, this.jRadioButtonCoverageMin.isSelected() ? 0.0 : this.jDFLossOfCoverage.getValue()/100.0, this.jCBKeepLastRule.isSelected(),this.jRBMaxError.isSelected() == true ? this.jDFMaxError.getValue() : -1,  this.jCheckBoxRuleRemoval.isSelected(), this.jCheckBoxVariableRemoval.isSelected(), this.jPanelValidationFile.GetValidationFile());
    if( new_fis != null )
      { JFISFrame jff = new JFISFrame(new_fis, this.DataFile, this.Parent); }
    this.dispose();
    }
  catch( Throwable except ) { MessageFIS.Error( null, except ); }
  }
//------------------------------------------------------------------------------
  void jRBLossOfPerformance_itemStateChanged(ItemEvent e)
  {
  if( this.jRBLossOfPerformance.isSelected() == true )
    this.jDFLossOfPerformance.setEnabled(true);
  else
    this.jDFLossOfPerformance.setEnabled(false);
  }
//------------------------------------------------------------------------------
  void jRBMaxError_itemStateChanged(ItemEvent e)
  {
  if( this.jRBMaxError.isSelected() == true )
    this.jDFMaxError.setEnabled(true);
  else
    this.jDFMaxError.setEnabled(false);
  }
/**
 * This method initializes jRadioButtonLossOfCoverage	
 * 	
 * @return javax.swing.JRadioButton	
 */
private JRadioButton getJRadioButtonLossOfCoverage() {
    if (jRadioButtonLossOfCoverage == null) {
        jRadioButtonLossOfCoverage = new JRadioButton();
        jRadioButtonLossOfCoverage.setText(LocaleFIS.GetString("LossOfCoverage"));
        jRadioButtonLossOfCoverage.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent e) {
                doubleFieldCoverageMin.setEnabled(false);
                jDFLossOfCoverage.setEnabled(true);
            }
        });
    }
    return jRadioButtonLossOfCoverage;
}
/**
 * This method initializes jRadioButtonCoverageMin	
 * 	
 * @return javax.swing.JRadioButton	
 */
private JRadioButton getJRadioButtonCoverageMin() {
    if (jRadioButtonCoverageMin == null) {
        jRadioButtonCoverageMin = new JRadioButton();
        jRadioButtonCoverageMin.setSelected(true);
        jRadioButtonCoverageMin.setText(LocaleFIS.GetString("Coverage") + " min");
        jRadioButtonCoverageMin.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent e) {
                doubleFieldCoverageMin.setEnabled(true);
                jDFLossOfCoverage.setEnabled(false);
            }
        });
    }
    return jRadioButtonCoverageMin;
}
/**
 * This method initializes buttonGroupCoverage	
 * 	
 * @return javax.swing.ButtonGroup	
 */
private ButtonGroup getButtonGroupCoverage() {
    if (buttonGroupCoverage == null) {
        buttonGroupCoverage = new ButtonGroup();
    }
    return buttonGroupCoverage;
}
/**
 * This method initializes doubleFieldCoverageMin	
 * 	
 * @return fis.DoubleField	
 */
private DoubleField getDoubleFieldCoverageMin() {
    if (doubleFieldCoverageMin == null) {
        doubleFieldCoverageMin = new DoubleField();
    }
    return doubleFieldCoverageMin;
}
/**
 * This method initializes jCheckBoxRuleRemoval	
 * 	
 * @return javax.swing.JCheckBox	
 */
private JCheckBox getJCheckBoxRuleRemoval() {
    if (jCheckBoxRuleRemoval == null) {
        jCheckBoxRuleRemoval = new JCheckBox();
        jCheckBoxRuleRemoval.setText(LocaleFIS.GetString("RuleRemoval"));
    }
    return jCheckBoxRuleRemoval;
}
/**
 * This method initializes jCheckBoxVariableRemoval	
 * 	
 * @return javax.swing.JCheckBox	
 */
private JCheckBox getJCheckBoxVariableRemoval() {
    if (jCheckBoxVariableRemoval == null) {
        jCheckBoxVariableRemoval = new JCheckBox();
        jCheckBoxVariableRemoval.setText(LocaleFIS.GetString("VariableRemoval"));
    }
    return jCheckBoxVariableRemoval;
}
}