//***********************************************************************
//
//                              JTreeGenerateFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.io.*;
import java.util.*;

//------------------------------------------------------------------------------
public class JTreeGenerateFrame extends JFISDialog
{
  private JFISFrame parent;
  private JFIS fis;
  private JExtendedDataFile data;
  JLabel jLabelTreeFile = new JLabel();
  JLabel jLabelMinSignificantLevel = new JLabel();
  JLabel jLabelLeafMinCard = new JLabel();
  JLabel jLabelAmbiguityThresold = new JLabel();
  JLabel jLabelTreeDepth= new JLabel();
  JLabel jLabelPerfLoss= new JLabel();
  JLabel jLabelMinEDGain = new JLabel();
  JLabel jLabelCovThresh = new JLabel();
  JTextField jTFTreeFile = new JTextField();
  DoubleField jDFMinSignificantLevel = new DoubleField();
  IntegerField jIFLeafMinCard = new IntegerField();
  DoubleField jDFAmbiguityThreshold = new DoubleField();
  DoubleField jDFPerfLoss = new DoubleField();
  DoubleField jDFMinEDGain = new DoubleField();
  DoubleField jDFCovThresh = new DoubleField();
  IntegerField jIFTreeDepth = new IntegerField();
  JCheckBox jCBPrune = new JCheckBox();
  JCheckBox jCBSplit = new JCheckBox();
  JCheckBox jCBRelgain = new JCheckBox();
  JPanel jPanelOutput = new JPanel(new GridBagLayout());
  ButtonGroup jbgOutput = new ButtonGroup();
  protected JPanelValidationFile jpValidationFile;
//------------------------------------------------------------------------------
  public JTreeGenerateFrame(JFISFrame parent, JFIS fis, JExtendedDataFile data)
  {
    this.parent = parent;
    this.fis = fis;
    this.data = data;
    try { jbInit(); }
    catch(Throwable t) { MessageFIS.Error(null, t); }
  }
//------------------------------------------------------------------------------
  private void jbInit() throws Throwable
  {
    this.setTitle(LocaleFIS.GetString("TreeGenerate"));
    this.getContentPane().setLayout(new GridBagLayout());
    JPanel jPanelSaisie = new JPanel(new GridBagLayout());
    JPanel jPanelValidation = new JPanel(new GridBagLayout());
    JButton jButtonApply = new JButton(LocaleFIS.GetString("Apply"));
    JButton jButtonCancel = new JButton(LocaleFIS.GetString("Cancel"));
    jLabelTreeFile.setText(LocaleFIS.GetString("TreeFile") + ":");
    jLabelMinSignificantLevel.setText(LocaleFIS.GetString("MinSignificantLevel") + ":");
    jLabelLeafMinCard.setText(LocaleFIS.GetString("LeafMinCard") + ":");
    jLabelAmbiguityThresold.setText(LocaleFIS.GetString("AmbiguityThreshold") + ":");
    jLabelTreeDepth.setText(LocaleFIS.GetString("TreeDepth")+ ":");
    jLabelPerfLoss.setText(LocaleFIS.GetString("RelPerfLoss")+ ":");
    jLabelMinEDGain.setText(LocaleFIS.GetString("MinEDGain")+ ":");
    jLabelCovThresh.setText(LocaleFIS.GetString("CovThresh")+ ":");
    jCBPrune.setText(LocaleFIS.GetString("Prune"));
    jCBSplit.setText(LocaleFIS.GetString("Split"));
    jCBRelgain.setText(LocaleFIS.GetString("Relgain"));
    jPanelSaisie.add(jLabelTreeFile,    new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
    jPanelSaisie.add(jLabelTreeDepth,    new GridBagConstraints(0, 2, 2, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
    jPanelSaisie.add(jLabelMinSignificantLevel,    new GridBagConstraints(0, 3, 2, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
    jPanelSaisie.add(jLabelLeafMinCard,    new GridBagConstraints(0, 4, 2, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
    jPanelSaisie.add(jLabelAmbiguityThresold,   new GridBagConstraints(0, 5, 2, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
    jPanelSaisie.add(jLabelMinEDGain,   new GridBagConstraints(0, 6, 2, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
    jPanelSaisie.add(jLabelCovThresh,   new GridBagConstraints(0, 7, 2, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));

    this.jTFTreeFile.setText(MainFIS.config.FISFilePath() + System.getProperty("file.separator") + this.fis.GetName() + ".tree");
    jPanelSaisie.add(jTFTreeFile,     new GridBagConstraints(1, 0, 4, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 0), 20, 0));

    // panel sortie
    this.jPanelOutput.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),LocaleFIS.GetString("Outputs")));
    boolean first_active = false;
    for( int i=0 ; i<this.fis.NbOutputs() ; i++ )
      {
      final JOutput output = this.fis.GetOutput(i);
      JRadioButton jrb = new JRadioButton(output.GetName());
      jrb.addItemListener(new java.awt.event.ItemListener() { public void itemStateChanged(ItemEvent e)
        {
        try
          {
          JTreeGenerateFrame.this.jLabelAmbiguityThresold.setEnabled((output.GetNature() == JOutput.FUZZY) || output.GetClassif());
          JTreeGenerateFrame.this.jDFAmbiguityThreshold.setEnabled((output.GetNature() == JOutput.FUZZY) || output.GetClassif());
          }
        catch( Throwable t ) {}
        } } );
      this.jbgOutput.add(jrb);
      if( i == 0 )
        jrb.setSelected(true);
      if( output.GetActive() == false )
        jrb.setEnabled(false);
      else
        {
        if( first_active == false )
          {
          jrb.setSelected(true);
          first_active = true;
          }
        }
      this.jPanelOutput.add(jrb, new GridBagConstraints(i%2==0?0:1, i/2, 1, 1, 1.0, 0.0
          ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets( 0, 0, 0, 0), 0, 0));
      }
    jPanelSaisie.add(this.jPanelOutput,  new GridBagConstraints(0, 1, 4, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 0), 0, 0));

    this.jIFTreeDepth.setValue(this.fis.NbInputs());
    jPanelSaisie.add(jIFTreeDepth,  new GridBagConstraints(2, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 30, 0));

    this.jDFMinSignificantLevel.setValue(0.2);
    jPanelSaisie.add(jDFMinSignificantLevel,  new GridBagConstraints(2, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 30, 0));
    this.jIFLeafMinCard.setValue(Math.min(10, data.GetData().length * 10 / 100));
    jPanelSaisie.add(jIFLeafMinCard,  new GridBagConstraints(2, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 30, 0));
    this.jDFAmbiguityThreshold.setValue(0.1);
    jPanelSaisie.add(jDFAmbiguityThreshold,    new GridBagConstraints(2, 5, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 30, 0));
    this.jDFMinEDGain.setValue(0.001);
    jPanelSaisie.add(jDFMinEDGain,    new GridBagConstraints(2, 6, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 30, 0));
    this.jDFCovThresh.setValue(0.9);
    jPanelSaisie.add(jDFCovThresh,    new GridBagConstraints(2, 7, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 30, 0));

    jPanelSaisie.add(jCBRelgain,  new GridBagConstraints(0, 8, 3, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    jPanelSaisie.add(jCBPrune,    new GridBagConstraints(0, 9, 3, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    jPanelSaisie.add(jCBSplit,    new GridBagConstraints(0, 10, 3, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    this.jCBSplit.setSelected(true);
    jPanelSaisie.add(jLabelPerfLoss,    new GridBagConstraints(0, 11, 2, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
     this.jDFPerfLoss.setValue(0.1);
    jPanelSaisie.add(jDFPerfLoss,  new GridBagConstraints(2, 11, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 30, 0));

    // fichier de validation
    this.jpValidationFile = new JPanelValidationFile(this.data);
    jPanelSaisie.add(this.jpValidationFile,     new GridBagConstraints(0, 12, 3, 1, 1.0, 0.0
             ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));

    this.getContentPane().add(jPanelSaisie, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
    jPanelValidation.add(jButtonApply,     new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    jPanelValidation.add(jButtonCancel,   new GridBagConstraints(10, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    this.getContentPane().add(jPanelValidation,   new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 5, 5), 0, 0));

    // evenements
    this.jCBPrune.addItemListener(new java.awt.event.ItemListener() { public void itemStateChanged(ItemEvent e)
      {
      boolean enable = JTreeGenerateFrame.this.jCBPrune.isSelected();
      JTreeGenerateFrame.this.jCBSplit.setEnabled(enable);
      JTreeGenerateFrame.this.jLabelPerfLoss.setEnabled(enable);
      JTreeGenerateFrame.this.jDFPerfLoss.setEnabled(enable);
      JTreeGenerateFrame.this.jpValidationFile.setEnabled(enable);
      } });
    jButtonApply.addActionListener(new java.awt.event.ActionListener() { public void actionPerformed(ActionEvent e) { jButtonApply_actionPerformed(e); } });
    jButtonCancel.addActionListener(new java.awt.event.ActionListener() { public void actionPerformed(ActionEvent e) { jButtonCancel_actionPerformed(e); } });

    // affichage de la boite de dialogue
    this.jCBPrune.setSelected(true);
    this.pack();
    this.setModal(true);
    this.setLocation(JFISChildFrame.ChildPosition(this.parent, this.getSize()));
    this.setVisible(true);
  }
//------------------------------------------------------------------------------
  private int SelectedOutput() throws Throwable
  {
  Enumeration items = this.jbgOutput.getElements();
  for( int i=0 ; i<this.fis.NbOutputs() ; i++ )
    {
    JRadioButton jrb = (JRadioButton)items.nextElement();
    if( this.fis.GetOutput(i).GetActive() == true )
      if( jrb.isSelected() )
        return i;
    }
  throw new Exception("NoSelectedOutputFound");
  }
//------------------------------------------------------------------------------
  void jButtonCancel_actionPerformed(ActionEvent e) { this.dispose(); }
//------------------------------------------------------------------------------
  void jButtonApply_actionPerformed(ActionEvent e)
  {
  if( (this.jDFMinEDGain.getValue() < 0) || (this.jDFMinEDGain.getValue() > 1) )
    {
    MessageFIS.Error(this, "", "0 <= " + LocaleFIS.GetString("MinEDGain") + " <= 1");
    return;
    }
  try
    {
    boolean relgain = this.jCBRelgain.isSelected();
    boolean prune = this.jCBPrune.isSelected();
    boolean split = this.jCBSplit.isSelected();

    JFIS [] generated_fis = this.fis.NewTreeFIS(this.data, this.jTFTreeFile.getText(), this.jpValidationFile.GetValidationFile(), this.data.FileName() + ".result.fistree", this.data.FileName() + ".perf.fistree", this.jIFTreeDepth.getValue(), this.jDFMinEDGain.getValue(), this.jDFMinSignificantLevel.getValue(), this.jIFLeafMinCard.getValue(), this.jDFAmbiguityThreshold.getValue(), this.SelectedOutput(), prune, split, this.jDFPerfLoss.getValue(), relgain, false, this.jDFCovThresh.getValue());
    JFISFrame jff_tree = new JFISFrame(generated_fis[0], this.data, this.parent);
    //new JFuzzyTreeFrame(this.parent, this.jTFTreeFile.getText());
    new JFuzzyGraphFrame(this.parent, this.jTFTreeFile.getText());

    if( prune == true )
      {
      new JFISFrame(generated_fis[1], this.data, jff_tree);
      //new JFuzzyTreeFrame(this.parent, this.jTFTreeFile.getText()+".pruned");
      new JFuzzyGraphFrame(this.parent, this.jTFTreeFile.getText()+".pruned");
      }
    this.dispose();
    }
  catch( Throwable t ) { MessageFIS.Error(null, t); }
  }
}

