//***********************************************************************
//
//                              JViewVertexFrame.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.*;

import javax.swing.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.Box.Filler;
import javax.swing.border.*;
import java.util.*;
import java.awt.geom.*;
import javax.swing.event.*;

import util.swing.JFormattedTextFieldDouble;

//------------------------------------------------------------------------------
public class JViewVertexFrame extends JFISDialog
{
  private JFISFrame Parent;
  private String VertexFile;
  private String HFPFile;
  private JExtendedDataFile DataFile;
  private Vector CheckBoxs = new Vector();
  private Vector MFs = new Vector();
  private JPanel jPanelValidation = new JPanel(new GridBagLayout());
  private JButton jButtonApply = new JButton();
  private JButton jButtonCancel = new JButton();
  private JFISHFP hfp = null;
  private JVertexFile jvf = null;
  private JTextField jTFHFPFile;
  private JTextField jTFVertexFile;
  private JPanel jpVariables = new JPanel(new GridBagLayout());
//------------------------------------------------------------------------------
  public JViewVertexFrame( JFISFrame parent, String vertex_file, String hfp_file, JExtendedDataFile data_file )
  {
  this.HFPFile = hfp_file;
  this.Parent = parent;
  this.VertexFile = vertex_file;
  this.DataFile = data_file;
  try { jbInit(); }
  catch(Throwable t) { MessageFIS.Error(null, t); }
  }
//------------------------------------------------------------------------------
  private void jbInit() throws Throwable
  {
    // saisie des fichiers HFP et Vertex
    this.HfpVertexChooser();
    if( (this.hfp == null) || (this.jvf == null) ) return;

    this.getContentPane().setLayout(new GridBagLayout());
    // affichage des entrees
    for( int i=0 ; i<jvf.VariableCount() ; i++ )
      {
      JCheckBox jcb = new JCheckBox(this.hfp.GetInput(i).GetName());
      this.CheckBoxs.add(jcb);
      this.jpVariables.add(jcb,  new GridBagConstraints(0, i, 1, 1, 0.0, 0.0
              ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(i==0 ? 5 : 0, 5, 0, 5), 0, 0));
      this.jpVariables.add(new JLabel(LocaleFIS.GetString("MaxNumberOfMF")+":"),  new GridBagConstraints(1, i, 1, 1, 0.0, 0.0
              ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(i==0 ? 5 : 0, 0, 0, 5), 0, 0));
      IntegerField intf = new IntegerField();
      this.MFs.add(intf);
      intf.setValue(this.hfp.GetInput(i).GetNumberOfMF());
      this.jpVariables.add(intf,  new GridBagConstraints(2, i, 1, 1, 0.0, 0.0
              ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(i==0 ? 5 : 0, 0, 0, 5), 40, 0));
      }
    JScrollPane jsp = new JScrollPane(this.jpVariables) { public Dimension getMinimumSize() { return new Dimension(0, Math.min(200, JViewVertexFrame.this.jpVariables.getMinimumSize().height)); } };
    jsp.setBorder(BorderFactory.createEmptyBorder());
    this.getContentPane().add(jsp,  new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
              ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));

    // panel validation
    jPanelValidation.add(jButtonApply,  new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    jPanelValidation.add(jButtonCancel,  new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    this.getContentPane().add(jPanelValidation,         new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));

    // events
    jButtonApply.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonApply_actionPerformed(e); } });
    jButtonCancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { jButtonCancel_actionPerformed(e); } });

    // affichage
    this.Translate();
    this.pack();
    this.setModal(true);
    this.setLocation(JFISChildFrame.ChildPosition(this.Parent, this.getSize()));
    this.setVisible(true);
  }
//------------------------------------------------------------------------------
  private void HfpVertexChooser()
  {

  final JDialog jd = new JFISDialog();
  jd.setTitle(LocaleFIS.GetString("ViewVertices"));
  jd.getContentPane().setLayout(new GridBagLayout());

  // nom du fichier hfp
  JLabel jLabelHFPFile = new JLabel(LocaleFIS.GetString("HFPFile") + ":");
  this.jTFHFPFile = new JTextField(this.HFPFile);
  jTFHFPFile.setEditable(false);
  JButton jButtonSelectHFPFile = new JButton("...");
  jButtonSelectHFPFile.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e)
    {
    JOpenFileChooser file_chooser = new JOpenFileChooser(MainFIS.config.FISFilePath());
    file_chooser.setSelectedFile(new File(jTFHFPFile.getText()));
    file_chooser.setAcceptAllFileFilterUsed(true);
    // ajout du filtre *.hfp
    JFileFilter filter = new JFileFilter(new String("hfp"), LocaleFIS.GetString("HFPFile"));
    file_chooser.addChoosableFileFilter(filter);
    if( file_chooser.showOpenDialog(jd) == JFileChooser.APPROVE_OPTION )
      {
      jTFHFPFile.setText(file_chooser.getSelectedFile().getAbsolutePath());
      jd.pack();
      }
    } });
  jd.getContentPane().add(jLabelHFPFile,   new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
  if( this.HFPFile != null )
    jTFHFPFile.setText(this.HFPFile);
  jd.getContentPane().add(jTFHFPFile,  new GridBagConstraints(1, 0, 3, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 0, 0, 5), 50, 0));
  jd.getContentPane().add(jButtonSelectHFPFile,       new GridBagConstraints(4, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, -10));

  // nom du fichier vertex
  JLabel jLabelVertexFile = new JLabel(LocaleFIS.GetString("VerticesFile") + ":");
  this.jTFVertexFile = new JTextField(this.VertexFile);
  jTFVertexFile.setEditable(false);
  JButton jButtonSelectVertexFile = new JButton("...");
  jButtonSelectVertexFile.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e)
    {
    JOpenFileChooser file_chooser = new JOpenFileChooser(MainFIS.config.FISFilePath());
    file_chooser.setSelectedFile(new File(jTFVertexFile.getText()));
    file_chooser.setAcceptAllFileFilterUsed(true);
    // ajout du filtre *.vertices
    JFileFilter filter = new JFileFilter(new String("vertices"), LocaleFIS.GetString("VerticesFile"));
    file_chooser.addChoosableFileFilter(filter);
    if( file_chooser.showOpenDialog(jd) == JFileChooser.APPROVE_OPTION )
      {
      jTFVertexFile.setText(file_chooser.getSelectedFile().getAbsolutePath());
      jd.pack();
      }
    } });
  jd.getContentPane().add(jLabelVertexFile,   new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
          ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 0, 5), 0, 0));
  if( this.VertexFile != null )
    jTFVertexFile.setText(this.VertexFile);
  jd.getContentPane().add(jTFVertexFile,  new GridBagConstraints(1, 1, 3, 1, 1.0, 0.0
          ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 0, 0, 5), 50, 0));
  jd.getContentPane().add(jButtonSelectVertexFile,    new GridBagConstraints(4, 1, 1, 1, 0.0, 0.0
          ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 0, 5), 0, -10));

  // panel validation
  JPanel jPanelValidation = new JPanel(new GridBagLayout());
  JButton jButtonApply = new JButton();
  JButton jButtonCancel = new JButton();
  jButtonApply.setText(LocaleFIS.GetString("Apply"));
  jButtonCancel.setText(LocaleFIS.GetString("Cancel"));
  jButtonApply.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e)
    {
    if( jTFHFPFile.getText().equals("") )
      {
      JOptionPane.showConfirmDialog(jd, LocaleFIS.GetString("SelectAHFPFile"), null, JOptionPane.CLOSED_OPTION , JOptionPane.INFORMATION_MESSAGE );
      return;
      }
    if( jTFVertexFile.getText().equals("") )
      {
      JOptionPane.showConfirmDialog(jd, LocaleFIS.GetString("SelectAVerticesFile"), null, JOptionPane.CLOSED_OPTION , JOptionPane.INFORMATION_MESSAGE );
      return;
      }
    JViewVertexFrame.this.HFPFile = jTFHFPFile.getText();
    JViewVertexFrame.this.VertexFile = jTFVertexFile.getText();
    try
      {
      JViewVertexFrame.this.hfp = new JFISHFP(JViewVertexFrame.this.DataFile.ActiveFileName(), JViewVertexFrame.this.HFPFile);
      JViewVertexFrame.this.jvf = new JVertexFile(JViewVertexFrame.this.VertexFile);
      }
    catch(Throwable t) { MessageFIS.Error(null, t); }
    jd.dispose();
    } });
  jButtonCancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e)
    {
    JViewVertexFrame.this.hfp = null;
    JViewVertexFrame.this.jvf = null;
    jd.dispose();
    jTFHFPFile.setText(null);
    jTFVertexFile.setText(null);
    } });
  jPanelValidation.add(jButtonApply,  new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
          ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
  jPanelValidation.add(jButtonCancel,  new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
          ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
  jd.getContentPane().add(jPanelValidation,         new GridBagConstraints(0, 12, 5, 1, 1.0, 0.0
          ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));

  jd.pack();
  jd.setModal(true);
  jd.setLocation(JFISChildFrame.ChildPosition(this.Parent, jd.getSize()));
  jd.setVisible(true);
  }
//------------------------------------------------------------------------------
  private void Translate()
  {
    this.setTitle(LocaleFIS.GetString("ViewVertices"));
    jButtonApply.setText(LocaleFIS.GetString("Apply"));
    jButtonCancel.setText(LocaleFIS.GetString("Cancel"));
  }
//------------------------------------------------------------------------------
  void jButtonCancel_actionPerformed(ActionEvent e) { this.dispose(); }
//------------------------------------------------------------------------------
  void jButtonApply_actionPerformed(ActionEvent e)
  {
  try
    {
    for( int i=0 ; i<this.CheckBoxs.size() ; i++ )
      if( ((JCheckBox)this.CheckBoxs.elementAt(i)).isSelected() )
        {
        VertexDataHistogramFrame dhf = new VertexDataHistogramFrame(this.Parent, i, this.DataFile, this.hfp.GetInput(i), this.jvf.getPartitionInfos(i), ((IntegerField)this.MFs.elementAt(i)).getValue());
        }
    this.dispose();
    }
  catch( Throwable except ) { MessageFIS.Error( null, except ); }
  }
//------------------------------------------------------------------------------
  String GetHFPFile() { return this.jTFHFPFile.getText(); }
//------------------------------------------------------------------------------
  String GetVertexFile() { return this.jTFVertexFile.getText(); }
}

class PartitionInfo {
	
	PartitionInfo() {}
	PartitionInfo(double[] values) {
		this.partitionCoefficient = values[0];
		this.partitionEntropy = values[1];
	}
	
	Vector<Double> vertices;
	double partitionCoefficient;
	double partitionEntropy;
}


//------------------------------------------------------------------------------
class JVertexFile
{
  private Vector<Vector<PartitionInfo>> variables = new Vector<Vector<PartitionInfo>>();
//------------------------------------------------------------------------------
JVertexFile( String file_name ) throws Exception
  {
  // creation d'un buffer de lecture du fichier f
  BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(new File(file_name))));
  String buffer = br.readLine();
  while( buffer != null ) // tant que le fichier n'est pas entierement lu
    {
    Vector<PartitionInfo> variable = new Vector<PartitionInfo>();
    int nb_lines = Integer.parseInt(buffer);
    for( int i=0 ; i<nb_lines-1 ; i++ )
      {
      PartitionInfo info = new PartitionInfo();
      info.vertices = new Vector<Double>();
      //Vector v = new Vector();
      buffer = br.readLine();
      StringTokenizer st = new StringTokenizer( buffer, "," );
      if( st.countTokens() != (4 + i) )
        throw new Exception( LocaleFIS.GetString("ErrorInVerticesFile") + " " + file_name);
      info.partitionCoefficient = new Double(st.nextToken());
      info.partitionEntropy = new Double(st.nextToken());
      while( st.countTokens() != 0 )
    	  info.vertices.add(new Double(st.nextToken()));
      variable.add(info);
      }
    this.variables.add(variable);
    buffer = br.readLine();
    }
  }
//------------------------------------------------------------------------------
public int VariableCount() { return this.variables.size(); }
//------------------------------------------------------------------------------
public Vector<PartitionInfo> getPartitionInfos( int variable ) { return this.variables.elementAt(variable); }
}


class JPanelPartitionInfo extends JPanel implements Translatable {
	
	private final JLabel labelPartitionCoefficient = new JLabel();
	private final JLabel labelPartitionEntropy = new JLabel();
	private final JFormattedTextFieldDouble fieldCoefficient = new JFormattedTextFieldDouble();
	private final JFormattedTextFieldDouble fieldEntropy = new JFormattedTextFieldDouble();
	
	public JPanelPartitionInfo(PartitionInfo partitionInfo) {
		add(labelPartitionCoefficient);
		fieldCoefficient.setEditable(false);
		fieldCoefficient.setFocusable(false);
		add(fieldCoefficient);
		add(labelPartitionEntropy);
		fieldEntropy.setEditable(false);
		fieldEntropy.setFocusable(false);
		add(fieldEntropy);
		setPartitionInfo(partitionInfo);
		Translate();
	}

	@Override
	public void Translate() {
		labelPartitionCoefficient.setText(LocaleFIS.GetString("PartitionCoefficient"));
		labelPartitionEntropy.setText(LocaleFIS.GetString("PartitionEntropy"));
	}

	public void setPartitionInfo(PartitionInfo partitionInfo) {
		fieldCoefficient.setValue(partitionInfo.partitionCoefficient);
		fieldEntropy.setValue(partitionInfo.partitionEntropy);
	}
} 

//------------------------------------------------------------------------------
class VertexDataHistogramFrame extends DataHistogramFrame
{
  private JLabel jLabelNumberOfMF = new JLabel();
  private IntegerField jIFNumberOfMF = new IntegerField();
  private JButton jButtonApplyNumberOfMF = new JButton();
  private JInputHFP hfp_input;
  private Vector<PartitionInfo> partitionInfos;
  private Vector Inputs = new Vector();
  private JPanel panel = new JPanel(new GridBagLayout());
  private JScrollPane scrollPane = new JScrollPane(panel);
  private Vector<JPanel> InputPanels = new Vector<JPanel>();
  private Vector<JPanel> infoPanels = new Vector<JPanel>();
  private JPanelPartitionInfo panelPartitionInfo;
//------------------------------------------------------------------------------
  public VertexDataHistogramFrame( JFISFrame parent, int index, JFile data_file, JInputHFP hfp_input, Vector<PartitionInfo> partitionInfos, int number_of_mf ) throws Throwable
  {
    super(parent);
    this.hfp_input = hfp_input;
    this.partitionInfos = partitionInfos;
    super.jbInit( hfp_input.GetName(), index, data_file);

    // ajout saisie nombre de SEF
    JPanel jPanelMumberOfMF = new JPanel(new GridBagLayout());
    jPanelMumberOfMF.add(this.jLabelNumberOfMF,  new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 10, 0, 5), 0, 0));
    this.jIFNumberOfMF.setValue(number_of_mf);
    jPanelMumberOfMF.add(this.jIFNumberOfMF,  new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 5), 40, 0));
    jPanelMumberOfMF.add(this.jButtonApplyNumberOfMF,  new GridBagConstraints(2, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 10), 0, 0));
    System.out.println(scrollPane.getVerticalScrollBar().getWidth());
    final Component glue = Box.createGlue();
    this.jHistogramFramePanel.add(glue,  new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    this.jHistogramFramePanel.add(jPanelMumberOfMF,  new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 0), 0, 0));
    this.jHistogramFramePanel.add(scrollPane,  new GridBagConstraints(0, 2, 2, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
    scrollPane.setBorder(BorderFactory.createEmptyBorder());
    scrollPane.getVerticalScrollBar().addAdjustmentListener(new AdjustmentListener() {
		
		@Override
		public void adjustmentValueChanged(AdjustmentEvent e) {
			glue.setMinimumSize(new Dimension(((JScrollBar)e.getSource()).getWidth(), 0));
			glue.setVisible(((JScrollBar)e.getSource()).isVisible());
		}
	});
    this.jButtonApplyNumberOfMF.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e)
      {
      try { ShowVertices(jIFNumberOfMF.getValue()); repaint(); pack(); }
      catch(Throwable t) { MessageFIS.Error(null, t); }
      } });

    this.ShowVertices(number_of_mf);
    this.Show();
  }
//------------------------------------------------------------------------------
  protected void ApplyNumberOfClasses()
  {
  super.ApplyNumberOfClasses();
  // ajuste les x ranges des panels entree au panel histogram
  for( int i=0 ; i<this.InputPanels.size() ; i++ )
    ((JPanelInput)this.InputPanels.elementAt(i)).setXRange(this.jGraphHistogram.getXRange()[0], this.jGraphHistogram.getXRange()[1]);
  }
//------------------------------------------------------------------------------
  private void ShowVertices( int number_of_mf ) throws Throwable
  {
  // retire les panels precedents et detruit les entrees precedentes
  for( int i=0 ; i<this.InputPanels.size() ; i++ )
    this.panel.remove((Component)this.InputPanels.elementAt(i));
  for( int i=0 ; i<this.Inputs.size() ; i++ )
    ((JInput)this.Inputs.elementAt(i)).Delete();
  this.InputPanels = new Vector<JPanel>();
  this.Inputs = new Vector();
  for( int i=0 ; i<this.infoPanels.size() ; i++ )
	    this.panel.remove((Component)this.infoPanels.elementAt(i));
  this.infoPanels = new Vector<JPanel>();

  if( this.partitionInfos.size() < 1 )
    return;
  if( number_of_mf > this.hfp_input.GetNumberOfMF() )
    number_of_mf = this.hfp_input.GetNumberOfMF();
  if( number_of_mf > this.partitionInfos.size() )
    number_of_mf = this.partitionInfos.size()+1;
  if( number_of_mf < 2 )
    number_of_mf = 2;
  this.jIFNumberOfMF.setValue(number_of_mf);
  for( int i=0 ; i<number_of_mf-1 ; i++ )
    {
    double [] range = this.hfp_input.GetRange();
    Vector<Double> vdata = this.partitionInfos.elementAt(i).vertices;
    double [] data = new double[vdata.size()];
    for( int j=0 ; j<vdata.size() ; j++ )
      data[j] = ((Double)vdata.elementAt(j)).doubleValue();
    JInput new_input = new JInput(data, range[0], range[1], true );
    this.Inputs.add(new_input);
    JPanelInput jpi = new JPanelInput(new_input)
      {
      public Dimension getPreferredSize() { return new Dimension(10,50); }
      protected void AddYTicks() { this.addYTick("", Double.MAX_VALUE ); }  // ajoute un tick invisible
      public void PlotRectangle( Rectangle rect )
        {
        this.listener_rect = rect;
        // ajuste la hauteur du rectangle a la hauteur du panel
        this.listener_rect.y = 0;
        this.listener_rect.height = this.getSize().height-1;
        this.repaint();
        }
      };
    jpi.setGrid(false);
    jpi.addXTick("",Double.MAX_VALUE);
    jpi.setXRange(this.jGraphHistogram.getXRange()[0], this.jGraphHistogram.getXRange()[1]);
    this.jGraphHistogram.AddPlotRectangleListener(jpi);
    this.InputPanels.add(jpi);
    panelPartitionInfo = new JPanelPartitionInfo(this.partitionInfos.elementAt(i));
    this.infoPanels.add(panelPartitionInfo);
    panel.add(jpi,  new GridBagConstraints(0, 2+i*2, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 5, 0, 5), 0, 0));
    panel.add(panelPartitionInfo,  new GridBagConstraints(0, 2+i*2+1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
    }
  this.pack();
  }
//------------------------------------------------------------------------------
  public void Translate()
  {
  super.Translate();
  this.jLabelNumberOfMF.setText(LocaleFIS.GetString("NumberOfMF")+":");
  this.jButtonApplyNumberOfMF.setText(LocaleFIS.GetString("Apply"));
  this.panelPartitionInfo.Translate();
  }
//------------------------------------------------------------------------------
  protected HistogramPanel HistogramPanel() throws Throwable { return new HistogramInputPanel(this.hfp_input); }
}


