//***********************************************************************
//
//                              LookFIS.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

//------------------------------------------------------------------------------
public class LookFIS
{
  // tableau des styles installes
  private UIManager.LookAndFeelInfo[] lafi;
  // style par defaut
  private static String default_laf;

//------------------------------------------------------------------------------
  public LookFIS()
  {
  // lecture des styles installes
  lafi = UIManager.getInstalledLookAndFeels();
  // style par defaut
  default_laf = UIManager.getLookAndFeel().getClass().getName();
  }

//------------------------------------------------------------------------------
  public int Length()
  {
  return lafi.length;
  }

//------------------------------------------------------------------------------
  public UIManager.LookAndFeelInfo LookAndFeelInfo( int i )
  {
  return lafi[i];
  }

//------------------------------------------------------------------------------
  public static String DefaultLookAndFeel()
  {
  return default_laf;
  }

//------------------------------------------------------------------------------
  public void ChangeLook( Component c, String laf )
  {
  try {
    // installe le nouveau style
    UIManager.setLookAndFeel(laf);
    UpdateLookAndFeel( c ); }
  catch( Exception e ) {
    // l'installation du nouveau style n'a pas reussie
    MessageFIS.Error(LocaleFIS.GetString("CannotInstallLook&Feel"), e);
    try {
      // installation du style par defaut
      UIManager.setLookAndFeel(default_laf);
      UpdateLookAndFeel( c ); }
    catch( Exception ed ) {
      // l'installation du style par defaut n'a pas reussie
      MessageFIS.Error(LocaleFIS.GetString("CannotInstallDefaultLook&Feel"), e);
      System.exit(-1);
      }
    }
  }

//------------------------------------------------------------------------------
  public static void UpdateLookAndFeel( Component c )
  {
  SwingUtilities.updateComponentTreeUI(c);
  }
}




//------------------------------------------------------------------------------
// classe JRadioButtonMenuItemLook specifique a la gestion des menus dynamiques de style
class JRadioButtonMenuItemLook extends JRadioButtonMenuItem
{
  String name;        // nom du style
  String class_name;  // nom de la classe style

  public JRadioButtonMenuItemLook( UIManager.LookAndFeelInfo lafi )
    {
    name = lafi.getName();
    class_name = lafi.getClassName();
    setText( name );

    // active le bouton du menu correspondant au look actuel
    if( class_name.equalsIgnoreCase(UIManager.getLookAndFeel().getClass().getName()) == true )
      setSelected(true);
    }
}

