//***********************************************************************
//
//                              NumberField.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import javax.swing.*;
import javax.swing.text.*;
import java.util.Locale;
import java.text.*;
import java.awt.event.*;

//------------------------------------------------------------------------------
interface NumberFieldActionListener
{
  public void NumberFieldAction();
}


//------------------------------------------------------------------------------
abstract public class NumberField extends JTextField implements ActionListener
{
  private NumberFieldActionListener action = null;
//------------------------------------------------------------------------------
  public NumberField()
  {
  this.addActionListener(this);

  FocusAdapter fa = new FocusAdapter()
    {
    public void focusLost(FocusEvent e) { FocusLost(); }
    };
  this.addFocusListener(fa);
  }
//------------------------------------------------------------------------------
  public void FocusLost()
  {
  if( action != null )
    action.NumberFieldAction();
  }
//------------------------------------------------------------------------------
  public void SetAction( NumberFieldActionListener action ) { this.action = action; }
//------------------------------------------------------------------------------
  protected Document createDefaultModel() { return new NumberFieldDocument(); }
//------------------------------------------------------------------------------
  abstract protected boolean ValidCharacter( char c );
//------------------------------------------------------------------------------
  protected class NumberFieldDocument extends PlainDocument
    {
    public void insertString(int offs, String str, AttributeSet a) throws BadLocationException
      {
      char[] source = str.toCharArray();
      char[] result = new char[source.length];
      int j = 0;

      for( int i=0 ; i<result.length ; i++ )
        if( ValidCharacter(source[i]) )
          result[j++] = source[i];
      super.insertString(offs, new String(result, 0, j), a);
      }
    }
//------------------------------------------------------------------------------
  public void actionPerformed(ActionEvent e)
    {
    if( action != null )
      action.NumberFieldAction();
    }
}








