package fis.data.distance.input.preview;

import static com.google.common.primitives.Doubles.max;
import static com.google.common.primitives.Doubles.min;
import static com.humaorie.dollar.Dollar.$;
import static java.lang.Math.abs;
import fis.JInput;

public class DefaultPreview implements Preview {

	private final JInput input;
	private final double[] distances;

	public DefaultPreview(JInput input) throws Throwable {
		this.input = new JInput(input);
		this.input.CheckFuzDist();
		this.distances = computeDistances();
	}

	private double[] computeDistances() {
		double[] distancePixels = new double[DISTANCE_COUNT * DISTANCE_COUNT];
		for (int x : $(DISTANCE_COUNT))
			for (int y : $(DISTANCE_COUNT))
				distancePixels[x + (y * DISTANCE_COUNT)] = distance((double) x / (double) (DISTANCE_COUNT - 1), (double) y / (double) (DISTANCE_COUNT - 1));
		return distancePixels;
	}

	private double distance(double x, double y) {
		return fuzzyDistance(x, y) - euclideanDistance(x, y);
	}

	private double fuzzyDistance(double x, double y) {
		return input.Distance(x, y, true);
	}

	private double euclideanDistance(double x, double y) {
		return abs(x - y);
	}

	@Override
	public double[] getDistances() {
		return distances;
	}

	@Override
	public double getDistanceMin() {
		return min(distances);
	}

	@Override
	public double getDistanceMax() {
		return max(distances);
	}
}
