package fis.data.distance.jni;

import static com.google.common.collect.Iterables.transform;
import static com.google.common.collect.Lists.newArrayList;
import static com.google.common.primitives.Doubles.toArray;

import java.io.File;

import com.google.common.base.Function;

import fis.JExtendedDataFile;
import fis.JFIS;
import fis.data.distance.Distance;
import fis.data.distance.input.Input;

public class DefaultJNIDistance implements JNIDistance {

	private final JFIS fis;
	private final JExtendedDataFile data;
	private final Distance distance;

	public DefaultJNIDistance(JFIS fis, JExtendedDataFile data, Distance distance) {
		this.fis = fis;
		this.data = data;
		this.distance = distance;
	}

	@Override
	public long getFISPtr() {
		return fis.getPtr();
	}

	@Override
	public String getDataFileName() {
		return data.GetFileName();
	}

	@Override
	public boolean isNormalize() {
		return distance.isNormalize();
	}

	@Override
	public double[] getInputDistances() {
		return toArray(newArrayList(transform(distance.getInputs(), toOrdinal())));
	}

	private static Function<Input, Double> toOrdinal() {
		return new Function<Input, Double>() {

			@Override
			public Double apply(Input input) {
				return input.getDistance().ordinal();
			}
		};
	}

	@Override
	public File getOutputFile() {
		return distance.getOutputFile();
	}

	@Override
	public double getPower() {
		return distance.getMinkowski().getPower();
	}

	@Override
	public String getOutputFilePath() {
		return getOutputFile().getPath();
	}
}
