package fis.data.sample.classif;

import static fis.util.JFISUtil.getFISIndex;
import static fis.util.JFISUtil.hasOutput;
import static fis.util.JFISUtil.isOutputClassif;

import java.util.Map;

import util.jgoodies.binding.beans.ModelCloner;

import com.jgoodies.binding.beans.Model;
import com.rits.cloning.Cloner;
import com.rits.cloning.IFastCloner;

import fis.JExtendedDataFile;
import fis.JFIS;

@SuppressWarnings("serial")
public final class ClassifSampleModel extends Model implements ClassifSample {

	private static final int NO_CLASSIF_VARIABLE = -1;
	private static final int DEFAULT_OUTPUT_INDEX = 0;

	private boolean classif = false;
	private int classifVariable = NO_CLASSIF_VARIABLE;
	private double precision = DEFAULT_PRECISION;

	private final JExtendedDataFile data;
	private final JFIS fis;

	public ClassifSampleModel(JExtendedDataFile data, JFIS fis) {
		this.data = data;
		this.fis = fis;
		setClassif(isDefaultFISClassif());
	}

	private boolean isDefaultFISClassif() {
		return fis != null ? isOutputClassif(fis, DEFAULT_OUTPUT_INDEX) : false;
	}

	@Override
	public void setClassif(boolean classif) {
		firePropertyChange(PROPERTY_CLASSIF, this.classif, this.classif = classif);
		setClassifVariable(isClassif() ? getDefaultClassifVariable() : NO_CLASSIF_VARIABLE);
	}

	private int getDefaultClassifVariable() {
		return fis != null && hasOutput(fis) ? getDefaultFisClassifVariable() : getDefaultDataClassifVariable();
	}

	private int getDefaultFisClassifVariable() {
		return getFISIndex(fis, DEFAULT_OUTPUT_INDEX);
	}

	private int getDefaultDataClassifVariable() {
		return data.VariableCount() - 1;
	}

	@Override
	public boolean isClassif() {
		return classif;
	}

	@Override
	public void setClassifVariable(int classifVariable) {
		firePropertyChange(PROPERTY_CLASSIF_VARIABLE, this.classifVariable, this.classifVariable = classifVariable);
	}

	@Override
	public int getClassifVariable() {
		return classifVariable;
	}

	@Override
	public void setPrecision(double precision) {
		firePropertyChange(PROPERTY_PRECISION, this.precision, this.precision = precision);
	}

	@Override
	public double getPrecision() {
		return precision;
	}

	public static class FastCloner implements IFastCloner {

		@Override
		public Object clone(Object t, Cloner cloner, Map<Object, Object> clones) throws IllegalAccessException {
			return getCloner().deepClone(t);
		}

		private Cloner getCloner() {
			Cloner cloner = new ModelCloner();
			cloner.dontCloneInstanceOf(JExtendedDataFile.class);
			cloner.dontCloneInstanceOf(JFIS.class);
			return cloner;
		}
	}
}
