package fis.data.sample.generator.block;

import java.beans.PropertyChangeEvent;

import util.jgoodies.binding.beans.ModelCloner;

import com.jgoodies.binding.beans.Model;
import com.rits.cloning.Cloner;

import fis.JExtendedDataFile;
import fis.JFIS;
import fis.data.sample.block.BlockSample;
import fis.data.sample.block.BlockSampleChangeSupportDecorator;
import fis.data.sample.block.BlockSampleModel;
import fis.data.sample.classif.ClassifSample;
import fis.data.sample.classif.ClassifSampleChangeSupportDecorator;
import fis.data.sample.classif.ClassifSampleModel;
import fis.data.sample.seed.Seed;
import fis.data.sample.seed.SeedChangeSupportDecorator;
import fis.data.sample.seed.SeedModel;

@SuppressWarnings("serial")
public class BlockSampleGeneratorModel extends Model implements BlockSampleGenerator {

	private final BlockSample blockSample;
	private final ClassifSample classifSample;
	private final Seed seed;;
	private BlockSampleChangeSupportDecorator blockSampleChangeSupportDecorator;
	private ClassifSampleChangeSupportDecorator classifSampleChangeSupportDecorator;
	private SeedChangeSupportDecorator seedChangeSupportDecorator;

	public BlockSampleGeneratorModel(JExtendedDataFile data, JFIS fis) {
		blockSample = new BlockSampleModel();
		classifSample = new ClassifSampleModel(data, fis);
		seed = new SeedModel();
	}

	public BlockSampleGenerator clone() {
		return getCloner().deepClone(this);
	}

	private Cloner getCloner() {
		Cloner cloner = new ModelCloner();
		cloner.registerFastCloner(BlockSampleModel.class, new BlockSampleModel.FastCloner());
		cloner.registerFastCloner(ClassifSampleModel.class, new ClassifSampleModel.FastCloner());
		cloner.registerFastCloner(SeedModel.class, new SeedModel.FastCloner());
		cloner.nullInsteadOfClone(BlockSampleChangeSupportDecorator.class);
		cloner.nullInsteadOfClone(ClassifSampleChangeSupportDecorator.class);
		cloner.nullInsteadOfClone(SeedChangeSupportDecorator.class);
		return cloner;
	}

	public void release() {
		getBlockSampleChangeSupportDecorator().release();
		getClassifSampleChangeSupportDecorator().release();
		getSeedChangeSupportDecorator().release();
	}

	private BlockSampleChangeSupportDecorator getBlockSampleChangeSupportDecorator() {
		return blockSampleChangeSupportDecorator != null ? blockSampleChangeSupportDecorator : (blockSampleChangeSupportDecorator = new BlockSampleChangeSupportDecorator(blockSample) {

			@Override
			public void beanChange(PropertyChangeEvent evt) {
				firePropertyChange(PROPERTY_BLOCK_SAMPLE_CHANGED, null, evt);
			}
		});
	}

	private ClassifSampleChangeSupportDecorator getClassifSampleChangeSupportDecorator() {
		return classifSampleChangeSupportDecorator != null ? classifSampleChangeSupportDecorator : (classifSampleChangeSupportDecorator = new ClassifSampleChangeSupportDecorator(classifSample) {

			@Override
			public void beanChange(PropertyChangeEvent evt) {
				firePropertyChange(PROPERTY_CLASSIF_CHANGED, null, evt);
			}
		});
	}

	private SeedChangeSupportDecorator getSeedChangeSupportDecorator() {
		return seedChangeSupportDecorator != null ? seedChangeSupportDecorator : (seedChangeSupportDecorator = new SeedChangeSupportDecorator(seed) {

			@Override
			public void beanChange(PropertyChangeEvent evt) {
				firePropertyChange(PROPERTY_SEED_CHANGED, null, evt);
			}
		});
	}

	@Override
	public BlockSample getBlockSample() {
		return getBlockSampleChangeSupportDecorator();
	}

	@Override
	public ClassifSample getClassifSample() {
		return getClassifSampleChangeSupportDecorator();
	}

	@Override
	public Seed getSeed() {
		return getSeedChangeSupportDecorator();
	}
}
