package fis.data.sample.generator.pair;

import java.beans.PropertyChangeEvent;

import util.jgoodies.binding.beans.ModelCloner;

import com.jgoodies.binding.beans.Model;
import com.rits.cloning.Cloner;

import fis.JExtendedDataFile;
import fis.JFIS;
import fis.data.sample.classif.ClassifSample;
import fis.data.sample.classif.ClassifSampleChangeSupportDecorator;
import fis.data.sample.classif.ClassifSampleModel;
import fis.data.sample.pair.PairSample;
import fis.data.sample.pair.PairSampleChangeSupportDecorator;
import fis.data.sample.pair.PairSampleModel;
import fis.data.sample.seed.Seed;
import fis.data.sample.seed.SeedChangeSupportDecorator;
import fis.data.sample.seed.SeedModel;

@SuppressWarnings("serial")
public class PairSampleGeneratorModel extends Model implements PairSampleGenerator {

	private final PairSample pairSample;
	private final ClassifSample classifSample;
	private final Seed seed;
	private PairSampleChangeSupportDecorator pairSampleChangeSupportDecorator;
	private ClassifSampleChangeSupportDecorator classifSampleChangeSupportDecorator;
	private SeedChangeSupportDecorator seedChangeSupportDecorator;

	public PairSampleGeneratorModel(JExtendedDataFile data, JFIS fis) {
		pairSample = new PairSampleModel();
		classifSample = new ClassifSampleModel(data, fis);
		seed = new SeedModel();
	}

	public PairSampleGenerator clone() {
		return getCloner().deepClone(this);
	}

	private Cloner getCloner() {
		Cloner cloner = new ModelCloner();
		cloner.registerFastCloner(PairSampleModel.class, new PairSampleModel.FastCloner());
		cloner.registerFastCloner(ClassifSampleModel.class, new ClassifSampleModel.FastCloner());
		cloner.registerFastCloner(SeedModel.class, new SeedModel.FastCloner());
		cloner.nullInsteadOfClone(PairSampleChangeSupportDecorator.class);
		cloner.nullInsteadOfClone(ClassifSampleChangeSupportDecorator.class);
		cloner.nullInsteadOfClone(SeedChangeSupportDecorator.class);
		return cloner;
	}

	public void release() {
		getPairSampleChangeSupportDecorator().release();
		getClassifSampleChangeSupportDecorator().release();
		getSeedChangeSupportDecorator().release();
	}

	private PairSampleChangeSupportDecorator getPairSampleChangeSupportDecorator() {
		return pairSampleChangeSupportDecorator != null ? pairSampleChangeSupportDecorator : (pairSampleChangeSupportDecorator = new PairSampleChangeSupportDecorator(pairSample) {

			@Override
			public void beanChange(PropertyChangeEvent evt) {
				firePropertyChange(PROPERTY_PAIR_SAMPLE_CHANGED, null, evt);
			}
		});
	}

	private ClassifSampleChangeSupportDecorator getClassifSampleChangeSupportDecorator() {
		return classifSampleChangeSupportDecorator != null ? classifSampleChangeSupportDecorator : (classifSampleChangeSupportDecorator = new ClassifSampleChangeSupportDecorator(classifSample) {

			@Override
			public void beanChange(PropertyChangeEvent evt) {
				firePropertyChange(PROPERTY_CLASSIF_CHANGED, null, evt);
			}
		});
	}

	private SeedChangeSupportDecorator getSeedChangeSupportDecorator() {
		return seedChangeSupportDecorator != null ? seedChangeSupportDecorator : (seedChangeSupportDecorator = new SeedChangeSupportDecorator(seed) {

			@Override
			public void beanChange(PropertyChangeEvent evt) {
				firePropertyChange(PROPERTY_SEED_CHANGED, null, evt);
			}
		});
	}

	@Override
	public PairSample getPairSample() {
		return getPairSampleChangeSupportDecorator();
	}

	@Override
	public ClassifSample getClassifSample() {
		return getClassifSampleChangeSupportDecorator();
	}

	@Override
	public Seed getSeed() {
		return getSeedChangeSupportDecorator();
	}

/*	@Override
	public boolean isReplace() {
		return true;
	}

	@Override
	public boolean isTruncate() {
		return false;
	}*/
}
