package fis.optimization.crossvalidation;

import java.beans.PropertyChangeEvent;

import util.jgoodies.binding.beans.ModelCloner;

import com.jgoodies.binding.beans.Model;
import com.rits.cloning.Cloner;

import fis.JExtendedDataFile;
import fis.JFIS;
import fis.data.sample.classif.ClassifSample;
import fis.data.sample.classif.ClassifSampleChangeSupportDecorator;
import fis.data.sample.classif.ClassifSampleModel;
import fis.data.sample.pair.PairSample;
import fis.data.sample.pair.PairSampleChangeSupportDecorator;
import fis.data.sample.pair.PairSampleModel;

@SuppressWarnings("serial")
public final class CrossValidationModel extends Model implements CrossValidation {

	private boolean enabled = DEFAULT_ENABLED;
	private final PairSample pairSample;
	private final ClassifSample classifSample;
	private PairSampleChangeSupportDecorator pairSampleChangeSupportDecorator;
	private ClassifSampleChangeSupportDecorator classifSampleChangeSupportDecorator;

	public CrossValidationModel(JExtendedDataFile data, JFIS fis) {
		pairSample = new PairSampleModel();
		classifSample = new ClassifSampleModel(data, fis);
	}

	@Override
	public CrossValidation clone() {
		return getCloner().deepClone(this);
	}

	private Cloner getCloner() {
		Cloner cloner = new ModelCloner();
		cloner.registerFastCloner(PairSampleModel.class, new PairSampleModel.FastCloner());
		cloner.registerFastCloner(ClassifSampleModel.class, new ClassifSampleModel.FastCloner());
		cloner.nullInsteadOfClone(PairSampleChangeSupportDecorator.class);
		cloner.nullInsteadOfClone(ClassifSampleChangeSupportDecorator.class);
		return cloner;
	}

	public void release() {
		getPairSampleChangeSupportDecorator().release();
		getClassifSampleChangeSupportDecorator().release();
	}

	private PairSampleChangeSupportDecorator getPairSampleChangeSupportDecorator() {
		return pairSampleChangeSupportDecorator != null ? pairSampleChangeSupportDecorator : (pairSampleChangeSupportDecorator = new PairSampleChangeSupportDecorator(pairSample) {

			@Override
			public void beanChange(PropertyChangeEvent evt) {
				firePropertyChange(PROPERTY_PAIR_SAMPLE_CHANGED, null, evt);
			}
		});
	}

	private ClassifSampleChangeSupportDecorator getClassifSampleChangeSupportDecorator() {
		return classifSampleChangeSupportDecorator != null ? classifSampleChangeSupportDecorator : (classifSampleChangeSupportDecorator = new ClassifSampleChangeSupportDecorator(classifSample) {

			@Override
			public void beanChange(PropertyChangeEvent evt) {
				firePropertyChange(PROPERTY_CLASSIF_CHANGED, null, evt);
			}
		});
	}

	public void setEnabled(boolean enabled) {
		firePropertyChange(PROPERTY_ENABLED, this.enabled, this.enabled = enabled);
	}

	public boolean isEnabled() {
		return enabled;
	}

	@Override
	public PairSample getPairSample() {
		return getPairSampleChangeSupportDecorator();
	}

	@Override
	public ClassifSample getClassifSample() {
		return getClassifSampleChangeSupportDecorator();
	}
}
