//***********************************************************************
//
//                              JPrintPreview.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis.print;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.print.*;
import javax.swing.*;
import javax.swing.border.*;
import fis.*;

/**
 * A fairly generic class that provides a print preview capability
 */
public class JPrintPreview extends JFISDialog
{
    protected int               fPageWidth;
    protected int               fPageHeight;
    protected Printable         fTarget;
    protected JComboBox         fComboBoxScale;
    protected PreviewContainer  fPanelPreview;
    protected ActionListener    fPrintListener;
    protected int scale = 50;
    protected PrinterJob prnJob;
    protected PageFormat pageFormat;
    protected JScrollPane ps = null;
    protected Component parent;
    protected JPanel toolbar;

    public JPrintPreview( Component parent, Printable target)
    {
        super();
        this.parent = parent;
        this.Init( target );
    }

    protected JPrintPreview( Component parent )
    {
        super();
        this.parent = parent;
    }

    protected void Init( Printable target )
    {
        fTarget = target;

        this.ToolbarConstruct();
        getContentPane().add( toolbar, BorderLayout.NORTH);

        this.prnJob = PrinterJob.getPrinterJob();
        this.pageFormat = prnJob.defaultPage();
        this.setTitle(LocaleFIS.GetString("PrintPreview") + " : " + JPrintPreview.this.prnJob.getPrintService().getName());
        this.ShowPreview();

        this.pack();
        this.setModal(true);
        this.setLocation(JFISChildFrame.ChildPosition(parent, this.getSize()));
        //this.show();
	this.setVisible(true);
      }

    protected void ToolbarConstruct()
    {

        toolbar = new JPanel(new GridBagLayout());
        JButton buttonPrint = new JButton( LocaleFIS.GetString("Print"));
        fPrintListener = new ActionListener() {
            public void actionPerformed( ActionEvent e)
            {
                try
                {
                    JPrintPreview.this.prnJob.setPrintService(JPrintPreview.this.prnJob.getPrintService());
                    JPrintPreview.this.prnJob.setPrintable( fTarget, JPrintPreview.this.pageFormat );
                    setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                    JPrintPreview.this.prnJob.print();
                    setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                    dispose();
                }
                catch ( PrinterException ex) { MessageFIS.Error(null, ex ); }
            }
        };
        buttonPrint.addActionListener( fPrintListener);
        toolbar.add( buttonPrint,    new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));

        JButton buttonSetup = new JButton( LocaleFIS.GetString("PrintSetup"));
        ActionListener lst = new ActionListener() {
            public void actionPerformed( ActionEvent e)
            {
            Thread runner = new Thread() {
               public void run()
                 {
                 JPrintPreview.this.pageFormat = JPrintPreview.this.prnJob.pageDialog(JPrintPreview.this.pageFormat);
                 setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                 JPrintPreview.this.setTitle(LocaleFIS.GetString("PrintPreview") + " : " + JPrintPreview.this.prnJob.getPrintService().getName());
                 JPrintPreview.this.ShowPreview();
                 JPrintPreview.this.pack();
                 setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                 } };
            runner.start();
            }
        };
        buttonSetup.addActionListener( lst);
        toolbar.add( buttonSetup,    new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 5, 5), 0, 0));

        String[] scales = { "10 %", "25 %", "50 %", "100 %" };
        fComboBoxScale = new JComboBox( scales);
        fComboBoxScale.setSelectedItem( "50 %");

        lst = new ActionListener() {
            public void actionPerformed( ActionEvent e)
            {
                Thread runner = new Thread() {
                    public void run()
                    {
                        setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                        String str = fComboBoxScale.getSelectedItem().toString();
                        if ( str.endsWith( "%")) str = str.substring( 0, str.length()-1);
                        str = str.trim();
                        int sc = 0;

                        try
                        {
                            sc = Integer.parseInt( str);
                        }
                        catch ( NumberFormatException ex)
                        {
                            return;
                        }

                        int w = ( int)( fPageWidth*sc/100);
                        int h = ( int)( fPageHeight*sc/100);

                        Component[] comps = fPanelPreview.getComponents();
                        for ( int k=0; k<comps.length; k++)
                        {
                            if ( !( comps[k] instanceof PagePreview)) continue;
                            PagePreview pp = ( PagePreview)comps[k];
                            pp.setScaledSize( w, h);
                        }
                        fPanelPreview.doLayout();
                        fPanelPreview.getParent().getParent().validate();
                        setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                    }
                };
                runner.start();
            }

        };

        fComboBoxScale.addActionListener( lst);
        fComboBoxScale.setEditable( true);
        toolbar.add( fComboBoxScale,    new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 5, 5), 0, 0));

    }

    protected void ShowPreview()
    {
        if( this.ps != null )
          this.getContentPane().remove(this.ps);

        fPanelPreview = new PreviewContainer();
        if ( pageFormat.getHeight()==0 || pageFormat.getWidth()==0)
        {
            MessageFIS.Error(this, LocaleFIS.GetString("Error"), "Unable to determine default page size");
            return;
        }

        setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        fPageWidth = ( int)( pageFormat.getWidth());
        fPageHeight = ( int)( pageFormat.getHeight());

        int pageIndex = 0;
        int w = ( int)( fPageWidth*scale/100);
        int h = ( int)( fPageHeight*scale/100);

        try
        {
            while ( true)
            {
                BufferedImage img = new BufferedImage( fPageWidth,
                    fPageHeight, BufferedImage.TYPE_INT_RGB);
                Graphics2D g = img.createGraphics();
                g.setColor( Color.white);
                g.fillRect( 0, 0, fPageWidth, fPageHeight);
                if ( fTarget.print( g, pageFormat, pageIndex) != Printable.PAGE_EXISTS) break;
                PagePreview pp = new PagePreview( w, h, img);
                fPanelPreview.add( pp);
                pageIndex++;
            }
        }
        catch ( PrinterException e) { MessageFIS.Error(null, e ); }
        finally { setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)); }

        this.ps = new JScrollPane( fPanelPreview)
          {
          public Dimension getMinimumSize() { return  new Dimension(100,100); }
          };
        getContentPane().add( this.ps, BorderLayout.CENTER);
    }

    class PreviewContainer extends JPanel
    {
        protected int H_GAP = 16;
        protected int V_GAP = 10;

        public Dimension getPreferredSize()
        {
            int n = getComponentCount();
            if ( n == 0) return new Dimension( H_GAP, V_GAP);

            Component comp = getComponent( 0);
            Dimension dc = comp.getPreferredSize();
            int w = dc.width;
            int h = dc.height;

            Dimension dp = getParent().getSize();
            int nCol = Math.max( ( dp.width-H_GAP)/( w+H_GAP), 1);
            int nRow = n/nCol;
            if ( nRow*nCol < n) nRow++;

            int ww = nCol*( w+H_GAP) + H_GAP;
            int hh = nRow*( h+V_GAP) + V_GAP;
            Insets ins = getInsets();

            return new Dimension( ww+ins.left+ins.right,
                hh+ins.top+ins.bottom);
        }

        public Dimension getMaximumSize()
        {
            return getPreferredSize();
        }

        public Dimension getMinimumSize()
        {
            return getPreferredSize();
        }

        public void doLayout()
        {
            Insets ins = getInsets();
            int x = ins.left + H_GAP;
            int y = ins.top + V_GAP;

            int n = getComponentCount();
            if ( n == 0) return;

            Component comp = getComponent( 0);
            Dimension dc = comp.getPreferredSize();
            int w = dc.width;
            int h = dc.height;

            Dimension dp = getParent().getSize();
            int nCol = Math.max( ( dp.width-H_GAP)/( w+H_GAP), 1);
            int nRow = n/nCol;

            if ( nRow*nCol < n) nRow++;

            int index = 0;
            for ( int k = 0; k<nRow; k++)
            {
                for ( int m = 0; m<nCol; m++)
                {
                    if ( index >= n) return;
                    comp = getComponent( index++);
                    comp.setBounds( x, y, w, h);
                    x += w+H_GAP;
                }
                y += h+V_GAP;
                x = ins.left + H_GAP;
            }
        }
    }

    class PagePreview extends JPanel
    {
        protected int m_w;
        protected int m_h;
        protected Image m_source;
        protected Image m_img;

        public PagePreview( int w, int h, Image source)
        {
            m_w = w;
            m_h = h;
            m_source= source;
            m_img = m_source.getScaledInstance( m_w, m_h,
                Image.SCALE_SMOOTH);
            m_img.flush();
            setBackground( Color.white);
            setBorder( new MatteBorder( 1, 1, 2, 2, Color.black));
        }

        public void setScaledSize( int w, int h)
        {
            m_w = w;
            m_h = h;
            m_img = m_source.getScaledInstance( m_w, m_h,
                Image.SCALE_SMOOTH);
            repaint();
        }

        public Dimension getPreferredSize()
        {
            Insets ins = getInsets();
            return new Dimension( m_w+ins.left+ins.right,
                m_h+ins.top+ins.bottom);
        }

        public Dimension getMaximumSize()
        {
            return getPreferredSize();
        }

        public Dimension getMinimumSize()
        {
            return getPreferredSize();
        }

        public void paint( Graphics g)
        {
            g.setColor( getBackground());
            g.fillRect( 0, 0, getWidth(), getHeight());
            g.drawImage( m_img, 0, 0, this);
            paintBorder( g);
        }
    }
}

