//***********************************************************************
//
//                              JPrintableTable.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis.print;

import javax.swing.*;
import javax.swing.table.*;

import java.awt.print.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.Dimension;

public class JPrintableTable implements Printable {

    JTable ppTable;
    JTableHeader tableHeader;
    int [] subTableSplit = null;
    //boolean pageinfoCalculated=false;
    int totalNumPages=0;
    int prevPageIndex = 0;
    int subPageIndex = 0;
    int subTableSplitSize = 0;
    double tableHeightOnFullPage, headerHeight;
    double pageWidth, pageHeight;
    int fontHeight, fontDesent;
    double tableHeight, rowHeight;
    PageFormat pageFormat;


    public JPrintableTable( JTable ppTable ) {

	this.ppTable = ppTable;
        this.ppTable.setSize(this.ppTable.getPreferredSize());
    }

	public int print(Graphics g, PageFormat pageFormat, int pageIndex) throws PrinterException {

	Graphics2D g2=(Graphics2D)g;

        if( (this.pageFormat == null) || (this.pageFormat.equals(pageFormat) == false))
	/*if(!pageinfoCalculated)*/ {
		getPageInfo(g, pageFormat);
        this.pageFormat = pageFormat;
	}

	g2.setColor(Color.black);
	if(pageIndex>=totalNumPages) {
		return NO_SUCH_PAGE;
	}
	if (prevPageIndex != pageIndex) {
		subPageIndex++;
		if( subPageIndex == subTableSplitSize -1) {
			subPageIndex=0;
		}
	}

	g2.translate(pageFormat.getImageableX(), pageFormat.getImageableY());

	int rowIndex = pageIndex/ (subTableSplitSize -1);

	printTablePart(g2, pageFormat, rowIndex, subPageIndex);
	prevPageIndex= pageIndex;

	return Printable.PAGE_EXISTS;
	}


    public void getPageInfo(Graphics g, PageFormat pageFormat) {

        subTableSplit = null;
        subTableSplitSize = 0;
        subPageIndex = 0;
        prevPageIndex = 0;

        fontHeight=g.getFontMetrics().getHeight();
        fontDesent=g.getFontMetrics().getDescent();

        tableHeader = ppTable.getTableHeader();
        tableHeader.setSize(tableHeader.getPreferredSize());
        double headerWidth = tableHeader.getWidth();
        headerHeight = tableHeader.getHeight() /*+ ppTable.getRowMargin()*/;

        pageHeight = pageFormat.getImageableHeight();
        pageWidth =  pageFormat.getImageableWidth();

        double tableWidth = ppTable.getColumnModel().getTotalColumnWidth();
        tableHeight = ppTable.getHeight();
        rowHeight = ppTable.getRowHeight() /*+ ppTable.getRowMargin()*/;

        tableHeightOnFullPage = (int)(pageHeight - headerHeight - fontHeight*2);
        tableHeightOnFullPage = Math.floor(tableHeightOnFullPage/rowHeight) * rowHeight;

        TableColumnModel tableColumnModel = tableHeader.getColumnModel();
        int columns = tableColumnModel.getColumnCount();
        //int columnMargin = tableColumnModel.getColumnMargin();

        int [] temp = new int[columns];
        int columnIndex = 0;
        temp[0] = 0;
        int columnWidth;
        int length = 0;
        subTableSplitSize = 0;
        while ( columnIndex < columns ) {

           columnWidth = tableColumnModel.getColumn(columnIndex).getWidth();

           if ( length + columnWidth /*+ columnMargin*/ > pageWidth ) {
              temp[subTableSplitSize+1] = temp[subTableSplitSize] + length;
              length = columnWidth;
              subTableSplitSize++;
            }
            else {
               length += columnWidth /*+ columnMargin*/;
            }
            columnIndex++;
        } //while

        if ( length > 0 )  {  // if are more columns left, part page
           temp[subTableSplitSize+1] = temp[subTableSplitSize] + length;
           subTableSplitSize++;
        }

        subTableSplitSize++;
        subTableSplit = new int[subTableSplitSize];
        for ( int i=0; i < subTableSplitSize; i++ ) {
           subTableSplit[i]= temp[i];
        }
        totalNumPages = (int)(tableHeight/tableHeightOnFullPage);
        if ( tableHeight%tableHeightOnFullPage >= rowHeight ) { // at least 1 more row left
            totalNumPages++;
        }

        totalNumPages *= (subTableSplitSize-1);
        //pageinfoCalculated = true;
    }

    public void printTablePart(Graphics2D g2, PageFormat pageFormat, int rowIndex, int columnIndex) {

        String pageNumber = "Page: "+(rowIndex+1);
        if ( subTableSplitSize > 1 ) {
                pageNumber += "-" + (columnIndex+1);
	}

        int pageLeft = subTableSplit[columnIndex];
        int pageRight = subTableSplit[columnIndex + 1];

        int pageWidth =  pageRight-pageLeft;
	// page number message
        g2.drawString(pageNumber,  pageWidth/2-35, (int)(pageHeight - fontHeight));

        double clipHeight = Math.min(tableHeightOnFullPage, tableHeight - rowIndex*tableHeightOnFullPage);

        g2.translate(-subTableSplit[columnIndex], 0);
        g2.setClip(pageLeft ,0, pageWidth, (int)headerHeight);

        tableHeader.paint(g2);   // draw the header on every page
        g2.translate(0, headerHeight);
        g2.translate(0,  -tableHeightOnFullPage*rowIndex);

        // cut table image and draw on the page

        g2.setClip(pageLeft, (int)tableHeightOnFullPage*rowIndex, pageWidth, (int)clipHeight);
        ppTable.paint(g2);

        double pageTop =  tableHeightOnFullPage*rowIndex - headerHeight;
        double pageBottom = pageTop +  clipHeight + headerHeight;
        g2.setColor(Color.gray);
        g2.drawLine(pageLeft, (int)pageTop, pageLeft, (int)(pageTop + clipHeight + headerHeight));
    }



}
