package fis.swing.data.distance;

import static fis.LocaleFIS.GetString;
import static javax.swing.JOptionPane.OK_CANCEL_OPTION;
import static javax.swing.JOptionPane.PLAIN_MESSAGE;
import static javax.swing.JOptionPane.showOptionDialog;
import static util.swing.SwingUtilities.getWindowAncestor;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;

import util.jgoodies.validation.ValidationResultAdapter;
import util.swing.dialog.JOptionPaneException;

import com.jgoodies.binding.value.ValueHolder;

import fis.JExtendedDataFile;
import fis.JFIS;
import fis.LocaleFIS;
import fis.data.distance.DistanceModel;

@SuppressWarnings("serial")
public class DistanceAction extends AbstractAction {

	private static final String LABEL_DISTANCE = "Distance";

	private final ValueHolder fisHolder;
	private final ValueHolder dataHolder;

	public DistanceAction(ValueHolder fisHolder, ValueHolder dataHolder) {
		super(LocaleFIS.GetString(LABEL_DISTANCE));
		this.fisHolder = fisHolder;
		this.dataHolder = dataHolder;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		try {
			DistanceModel distanceModel = new DistanceModel((JFIS) fisHolder.getValue(), (JExtendedDataFile) dataHolder.getValue());
			DistanceAdapter distanceAdapter = new DistanceAdapter((JFIS) fisHolder.getValue(), (JExtendedDataFile) dataHolder.getValue(), distanceModel);
			JPanelDistance panelDistance = new JPanelDistance();
			panelDistance.bind(distanceAdapter);
			ValidationResultAdapter validationResultAdapter = new ValidationResultAdapter(panelDistance, distanceAdapter);
			showOptionDialog(getWindowAncestor(e.getSource()), panelDistance, GetString(LABEL_DISTANCE), OK_CANCEL_OPTION, PLAIN_MESSAGE, null, distanceAdapter.getOptions(), distanceAdapter.getInitialOption());
			validationResultAdapter.release();
			distanceAdapter.release();
			distanceModel.release();
		} catch (Throwable throwable) {
			JOptionPaneException.show(getWindowAncestor(e.getSource()), throwable);
		}
	}
}
