package fis.swing.data.distance;

import static com.google.common.collect.ImmutableList.copyOf;
import static com.google.common.collect.Iterables.transform;
import static fis.data.distance.Distance.PROPERTY_NORMALIZE;
import static fis.data.distance.Distance.PROPERTY_OUTPUT_FILE;
import static fis.util.JFISUtil.iterableInputs;
import static util.swing.dialog.JOptionPaneAction.createOkCancelActions;

import java.util.List;

import javax.swing.Action;

import util.jgoodies.validation.BeanValidationSupport;
import util.jgoodies.validation.DefaultBeanValidationSupport;
import util.swing.DefaultActionSupport;
import util.swing.DefaultOptionSupport;
import util.swing.OptionSupport;
import util.swing.action.performable.PerformableActionDecorator;
import util.swing.algorithm.distance.minkowski.MinkowskiAdapter;
import util.swing.io.SaveAsAction;

import com.google.common.base.Function;
import com.jgoodies.binding.PresentationModel;
import com.jgoodies.binding.value.ValueModel;
import com.jgoodies.validation.ValidationResultModel;

import edu.rice.cs.plt.iter.IterUtil;
import edu.rice.cs.plt.tuple.Pair;
import fis.JExtendedDataFile;
import fis.JFIS;
import fis.JInput;
import fis.data.distance.Distance;
import fis.data.distance.DistancePerformableAction;
import fis.data.distance.DistanceValidator;
import fis.data.distance.input.Input;
import fis.data.distance.jni.DefaultJNIDistance;
import fis.swing.data.distance.input.InputAdapter;

@SuppressWarnings("serial")
public class DistanceAdapter extends PresentationModel<Distance> implements BeanValidationSupport, OptionSupport {

	private final JFIS fis;
	private final JExtendedDataFile data;
	private MinkowskiAdapter minkowskiAdapter;
	private List<InputAdapter> inputAdapters;
	private final DefaultBeanValidationSupport<Distance> beanValidationSupport;
	private final OptionSupport optionSupport;

	public DistanceAdapter(JFIS fis, JExtendedDataFile data, Distance distance) {
		super(distance);
		this.fis = fis;
		this.data = data;
		beanValidationSupport = new DefaultBeanValidationSupport<Distance>(getBean(), new DistanceValidator(), new DefaultActionSupport(createActions(createOkCancelActions())));
		optionSupport = new DefaultOptionSupport(beanValidationSupport);
	}

	public MinkowskiAdapter getMinkowskiAdapter() {
		return minkowskiAdapter != null ? minkowskiAdapter : (minkowskiAdapter = new MinkowskiAdapter(getBean().getMinkowski()));
	}

	public Iterable<InputAdapter> getInputAdapters() {
		return inputAdapters != null ? inputAdapters : (inputAdapters = copyOf(toInputAdapters(getBean().getInputs(), iterableInputs(fis), getModel(PROPERTY_NORMALIZE))));
	}

	private static Iterable<InputAdapter> toInputAdapters(Iterable<Input> inputs, Iterable<JInput> jinputs, ValueModel normalizeModel) {
		return transform(IterUtil.zip(inputs, jinputs), toInputAdapter(normalizeModel));
	}

	private static Function<Pair<Input, JInput>, InputAdapter> toInputAdapter(final ValueModel normalizeModel) {
		return new Function<Pair<Input, JInput>, InputAdapter>() {

			@Override
			public InputAdapter apply(Pair<Input, JInput> pair) {
				return new InputAdapter(pair.first(), pair.second(), normalizeModel);
			}
		};
	}

	@Override
	public Object getInitialOption() {
		return optionSupport.getInitialOption();
	}

	@Override
	public Object[] getOptions() {
		return optionSupport.getOptions();
	}

	@Override
	public ValidationResultModel getValidationResultModel() {
		return beanValidationSupport.getValidationResultModel();
	}

	private Action[] createActions(Action[] actions) {
		return new Action[] { new PerformableActionDecorator(actions[0], new DistancePerformableAction(new DefaultJNIDistance(fis, data, getBean()))), actions[1] };
	}

	@Override
	public void release() {
		super.release();
		getMinkowskiAdapter().release();
		for (InputAdapter inputAdapter : getInputAdapters())
			inputAdapter.release();
		beanValidationSupport.release();
	}

	public Action getOutputFileAction() {
		return new SaveAsAction(getModel(PROPERTY_OUTPUT_FILE));
	}
}
