package fis.swing.data.distance.input.preview;

import static fis.LocaleFIS.GetString;
import static fis.data.distance.input.preview.Preview.DISTANCE_COUNT;
import static fis.util.JInputUtil.getInputName;
import static java.awt.image.DataBuffer.TYPE_DOUBLE;
import static java.lang.String.format;
import static javax.media.jai.RasterFactory.createBandedRaster;

import java.awt.image.Raster;
import java.awt.image.WritableRaster;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.Plot;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYBlockRenderer;
import org.jfree.chart.title.Title;
import org.jfree.data.xy.XYDataset;

import fis.JInput;
import fis.data.distance.input.preview.Preview;
import fis.swing.data.distance.input.preview.jfree.PreviewAxis;
import fis.swing.data.distance.input.preview.jfree.PreviewDataset;
import fis.swing.data.distance.input.preview.jfree.PreviewLegend;
import fis.swing.data.distance.input.preview.jfree.PreviewRenderer;

public class PreviewAdapter {

	private final Preview preview;
	private final JInput input;
	private JFreeChart previewChart;
	private XYBlockRenderer renderer;

	public PreviewAdapter(Preview preview, JInput input) {
		this.preview = preview;
		this.input = input;
	}

	public JFreeChart getPreviewChart() {
		if (previewChart == null) {
			previewChart = new JFreeChart(getPreviewPlot());
			previewChart.setTitle(format("%s: %s(%s - %s)", getInputName(input), GetString("Difference"), GetString("FPBased"), GetString("Euclidean")));
			previewChart.removeLegend();
			previewChart.addSubtitle(getPreviewLegend());
		}
		return previewChart;
	}

	private Title getPreviewLegend() {
		return new PreviewLegend(preview, getPreviewRenderer().getPaintScale());
	}

	private Plot getPreviewPlot() {
		return new XYPlot(getPreviewDataset(), getDomainAxis(), getRangeAxis(), getPreviewRenderer());
	}

	private XYBlockRenderer getPreviewRenderer() {
		return renderer != null ? renderer : (renderer = new PreviewRenderer(preview, input));
	}

	private XYDataset getPreviewDataset() {
		return new PreviewDataset(getPreviewRaster(), input);
	}

	private ValueAxis getRangeAxis() {
		return new PreviewAxis(input);
	}

	private ValueAxis getDomainAxis() {
		return new PreviewAxis(input);
	}

	private Raster getPreviewRaster() {
		WritableRaster distanceRaster = createBandedRaster(TYPE_DOUBLE, DISTANCE_COUNT, DISTANCE_COUNT, 1, null);
		distanceRaster.setPixels(0, 0, DISTANCE_COUNT, DISTANCE_COUNT, preview.getDistances());
		return distanceRaster;
	}
}
