package fis.swing.data.distance.input.preview.jfree;

import static java.lang.Math.max;
import static java.lang.Math.min;

import java.awt.Color;
import java.awt.Paint;

import org.jfree.chart.renderer.PaintScale;

import fis.data.distance.input.preview.Preview;

public class PreviewColorPaintScale implements PaintScale {

	private final double min;
	private final double max;

	public PreviewColorPaintScale(Preview preview) {
		this(getMin(preview), getMax(preview));
	}

	public PreviewColorPaintScale(double min, double max) {
		this.min = min;
		this.max = max;
	}

	private static double getMin(Preview preview) {
		return getMin(preview.getDistanceMin(), preview.getDistanceMax());
	}

	private static double getMax(Preview preview) {
		return getMax(preview.getDistanceMin(), preview.getDistanceMax());
	}

	private static double getMin(double min, double max) {
		return isEmptyRange(min, max) ? 0 : min;
	}

	private static double getMax(double min, double max) {
		return isEmptyRange(min, max) ? 1 : max;
	}

	protected static boolean isEmptyRange(double min, double max) {
		return (max - min) < 1e-10;
	}

	@Override
	public double getLowerBound() {
		return min;
	}

	@Override
	public double getUpperBound() {
		return max;
	}

	@Override
	public Paint getPaint(double value) {
		return reverseColor(getColorPaint(min(max(value, min), max)));
	}

	private Color reverseColor(Color color) {
		return new Color(255 - color.getRed(), 255 - color.getGreen(), 255 - color.getBlue());
	}

	private Color getColorPaint(double value) {
		return value > 0 ? getBluePaint(value) : getRedPaint(value);
	}

	private Color getRedPaint(double value) {
		return new Color(0, getRed(value), getRed(value), 255);
	}

	private Color getBluePaint(double value) {
		return new Color(getBlue(value), getBlue(value), 0, 255);
	}

	private int getRed(double value) {
		return value < 0 ? (int) (value / min * 255.0) : 0;
	}

	private int getBlue(double value) {
		return value > 0 ? (int) (value / max * 255.0) : 0;
	}
}
