package fis.swing.data.sample.generator.block;

import static util.swing.dialog.JOptionPaneAction.createCancelAction;
import static util.swing.dialog.JOptionPaneAction.createOkAction;

import javax.swing.Action;

import util.jgoodies.binding.BufferedPresentationModel;
import util.jgoodies.binding.value.TriggerableActionSupport;
import util.jgoodies.validation.BeanValidationSupport;
import util.jgoodies.validation.DefaultBeanValidationSupport;
import util.swing.DefaultOptionSupport;
import util.swing.OptionSupport;
import util.swing.action.throwable.ThrowableActionDecorator;

import com.jgoodies.validation.ValidationResultModel;

import fis.JExtendedDataFile;
import fis.JFIS;
import fis.data.sample.generator.SampleGeneratorPerformableAction;
import fis.data.sample.generator.block.BlockSampleGenerator;
import fis.data.sample.generator.block.BlockSampleGeneratorValidator;
import fis.data.sample.generator.jni.block.DefaultJNIBlockSampleGenerator;
import fis.swing.data.sample.block.BlockSampleAdapter;
import fis.swing.data.sample.classif.ClassifSampleAdapter;
import fis.swing.data.sample.seed.SeedAdapter;

@SuppressWarnings("serial")
public class BlockSampleGeneratorAdapter extends BufferedPresentationModel<BlockSampleGenerator> implements BeanValidationSupport, OptionSupport {

	private BlockSampleAdapter blockSampleAdapter;
	private ClassifSampleAdapter classifSampleAdapter;
	private SeedAdapter seedAdapter;
	private final JExtendedDataFile data;
	private final JFIS fis;
	private final TriggerableActionSupport triggerableActionSupport;
	private final DefaultBeanValidationSupport<BlockSampleGenerator> beanValidationSupport;
	private final OptionSupport optionSupport;

	public BlockSampleGeneratorAdapter(BlockSampleGenerator blockSampleGenerator, JExtendedDataFile data, JFIS fis) {
		super(blockSampleGenerator);
		this.data = data;
		this.fis = fis;
		triggerableActionSupport = new TriggerableActionSupport(getTriggerChannel(), createActions());
		beanValidationSupport = new DefaultBeanValidationSupport<BlockSampleGenerator>(getBufferedBean(), new BlockSampleGeneratorValidator(), triggerableActionSupport);
		optionSupport = new DefaultOptionSupport(beanValidationSupport);
		connectProperties(getBlockSampleAdapter(), getBufferedBean().getBlockSample());
		connectProperties(getClassifSampleAdapter(), getBufferedBean().getClassifSample());
		connectProperties(getSeedAdapter(), getBufferedBean().getSeed());
	}

	@Override
	public ValidationResultModel getValidationResultModel() {
		return beanValidationSupport.getValidationResultModel();
	}

	@Override
	public void release() {
		super.release();
		getBlockSampleAdapter().release();
		getClassifSampleAdapter().release();
		getSeedAdapter().release();
	}

	public BlockSampleAdapter getBlockSampleAdapter() {
		return blockSampleAdapter != null ? blockSampleAdapter : (blockSampleAdapter = new BlockSampleAdapter(getBean().getBlockSample(), getTriggerChannel()));
	}

	public ClassifSampleAdapter getClassifSampleAdapter() {
		return classifSampleAdapter != null ? classifSampleAdapter : (classifSampleAdapter = new ClassifSampleAdapter(getBean().getClassifSample(), getTriggerChannel(), data, fis));
	}

	public SeedAdapter getSeedAdapter() {
		return seedAdapter != null ? seedAdapter : (seedAdapter = new SeedAdapter(getBean().getSeed(), getTriggerChannel()));
	}

	@Override
	public Object[] getOptions() {
		return optionSupport.getOptions();
	}

	private Action[] createActions() {
		return new Action[] { new ThrowableActionDecorator(createOkAction(), new SampleGeneratorPerformableAction(new DefaultJNIBlockSampleGenerator(data, getBean()))), createCancelAction() };
	}

	public Object getInitialOption() {
		return optionSupport.getInitialOption();
	}
}
