package fis.swing.data.sample.generator.pair;

import static util.swing.dialog.JOptionPaneAction.createCancelAction;
import static util.swing.dialog.JOptionPaneAction.createOkAction;

import javax.swing.Action;

import util.jgoodies.binding.BufferedPresentationModel;
import util.jgoodies.binding.value.TriggerableActionSupport;
import util.jgoodies.validation.BeanValidationSupport;
import util.jgoodies.validation.DefaultBeanValidationSupport;
import util.swing.DefaultOptionSupport;
import util.swing.OptionSupport;
import util.swing.action.throwable.ThrowableActionDecorator;

import com.jgoodies.validation.ValidationResultModel;

import fis.JExtendedDataFile;
import fis.JFIS;
import fis.data.sample.generator.SampleGeneratorPerformableAction;
import fis.data.sample.generator.jni.pair.DefaultJNIPairSampleGenerator;
import fis.data.sample.generator.pair.PairSampleGenerator;
import fis.data.sample.generator.pair.PairSampleGeneratorValidator;
import fis.swing.data.sample.classif.ClassifSampleAdapter;
import fis.swing.data.sample.pair.PairSampleAdapter;
import fis.swing.data.sample.seed.SeedAdapter;

@SuppressWarnings("serial")
public class PairSampleGeneratorAdapter extends BufferedPresentationModel<PairSampleGenerator> implements BeanValidationSupport, OptionSupport {

	private PairSampleAdapter pairSampleAdapter;
	private ClassifSampleAdapter classifSampleAdapter;
	private SeedAdapter seedAdapter;
	private final JExtendedDataFile data;
	private final JFIS fis;
	private final TriggerableActionSupport triggerableActionSupport;
	private final DefaultBeanValidationSupport<PairSampleGenerator> beanValidationSupport;
	private final OptionSupport optionSupport;

	public PairSampleGeneratorAdapter(PairSampleGenerator pairSampleGenerator, JExtendedDataFile data, JFIS fis) {
		super(pairSampleGenerator);
		this.data = data;
		this.fis = fis;
		triggerableActionSupport = new TriggerableActionSupport(getTriggerChannel(), createActions());
		beanValidationSupport = new DefaultBeanValidationSupport<PairSampleGenerator>(getBufferedBean(), new PairSampleGeneratorValidator(), triggerableActionSupport);
		optionSupport = new DefaultOptionSupport(beanValidationSupport);
		connectProperties(getPairSampleAdapter(), getBufferedBean().getPairSample());
		connectProperties(getClassifSampleAdapter(), getBufferedBean().getClassifSample());
		connectProperties(getSeedAdapter(), getBufferedBean().getSeed());
	}

	@Override
	public void release() {
		super.release();
		getPairSampleAdapter().release();
		getClassifSampleAdapter().release();
		getSeedAdapter().release();
		beanValidationSupport.release();
	}

	@Override
	public ValidationResultModel getValidationResultModel() {
		return beanValidationSupport.getValidationResultModel();
	}

	public PairSampleAdapter getPairSampleAdapter() {
		return pairSampleAdapter != null ? pairSampleAdapter : (pairSampleAdapter = new PairSampleAdapter(getBean().getPairSample(), getTriggerChannel()));
	}

	public ClassifSampleAdapter getClassifSampleAdapter() {
		return classifSampleAdapter != null ? classifSampleAdapter : (classifSampleAdapter = new ClassifSampleAdapter(getBean().getClassifSample(), getTriggerChannel(), data, fis));
	}

	public SeedAdapter getSeedAdapter() {
		return seedAdapter != null ? seedAdapter : (seedAdapter = new SeedAdapter(getBean().getSeed(), getTriggerChannel()));
	}

	@Override
	public Object[] getOptions() {
		return optionSupport.getOptions();
	}

	private Action[] createActions() {
		return new Action[] { new ThrowableActionDecorator(createOkAction(), new SampleGeneratorPerformableAction(new DefaultJNIPairSampleGenerator(data, getBean()))), createCancelAction() };
	}

	@Override
	public Object getInitialOption() {
		return optionSupport.getInitialOption();
	}
}
