package fis.swing.optimization.crossvalidation.parameters;

import util.jgoodies.binding.BufferedPresentationModel;
import util.jgoodies.validation.BeanValidationSupport;
import util.jgoodies.validation.DefaultBeanValidationSupport;
import util.swing.DefaultOptionSupport;
import util.swing.OptionSupport;

import com.jgoodies.validation.ValidationResultModel;

import fis.JExtendedDataFile;
import fis.JFIS;
import fis.optimization.crossvalidation.CrossValidation;
import fis.optimization.crossvalidation.CrossValidationValidator;
import fis.swing.data.sample.classif.ClassifSampleAdapter;
import fis.swing.data.sample.pair.PairSampleAdapter;

@SuppressWarnings("serial")
public class CrossValidationParameterAdapter extends BufferedPresentationModel<CrossValidation> implements BeanValidationSupport, OptionSupport {

	private PairSampleAdapter pairSampleAdapter;
	private ClassifSampleAdapter classifSampleAdapter;
	private final JExtendedDataFile data;
	private final JFIS fis;
	private final DefaultBeanValidationSupport<CrossValidation> beanValidationSupport;
	private final OptionSupport optionSupport;

	public CrossValidationParameterAdapter(CrossValidation crossValidation, JExtendedDataFile data, JFIS fis) {
		super(crossValidation);
		this.data = data;
		this.fis = fis;
		this.beanValidationSupport = new DefaultBeanValidationSupport<CrossValidation>(getBufferedBean(), new CrossValidationValidator(), this);
		this.optionSupport = new DefaultOptionSupport(this.beanValidationSupport);
		connectProperties(getPairSampleAdapter(), getBufferedBean().getPairSample());
		connectProperties(getClassifSampleAdapter(), getBufferedBean().getClassifSample());
	}

	@Override
	public ValidationResultModel getValidationResultModel() {
		return beanValidationSupport.getValidationResultModel();
	}

	@Override
	public void release() {
		getPairSampleAdapter().release();
		getClassifSampleAdapter().release();
		beanValidationSupport.release();
		super.release();
	}

	public PairSampleAdapter getPairSampleAdapter() {
		return pairSampleAdapter != null ? pairSampleAdapter : (pairSampleAdapter = new PairSampleAdapter(getBean().getPairSample(), getTriggerChannel()));
	}

	public ClassifSampleAdapter getClassifSampleAdapter() {
		return classifSampleAdapter != null ? classifSampleAdapter : (classifSampleAdapter = new ClassifSampleAdapter(getBean().getClassifSample(), getTriggerChannel(), data, fis));
	}

	@Override
	public Object getInitialOption() {
		return optionSupport.getInitialOption();
	}

	@Override
	public Object[] getOptions() {
		return optionSupport.getOptions();
	}
}
