package fis.util;

import static com.google.common.collect.Iterables.transform;
import static com.humaorie.dollar.Dollar.$;
import static java.util.Collections.emptyList;

import com.google.common.base.Function;
import com.google.common.collect.Range;
import com.google.common.collect.Ranges;

import fis.JFIS;
import fis.JInput;
import fis.JOutput;

public class JFISUtil {

	public static JInput getFISInput(JFIS fis, int index) {
		try {
			return index < fis.NbInputs() ? fis.GetInput(index) : fis.GetOutput(index - fis.NbInputs());
		} catch (Throwable e) {
			return null;
		}
	}

	public static JOutput getFISOutput(JFIS fis, int index) {
		try {
			return index >= fis.NbInputs() ? fis.GetOutput(index - fis.NbInputs()) : null;
		} catch (Throwable e) {
			return null;
		}
	}

	public static boolean isFISOutputNature(JFIS fis, int index, int nature) {
		try {
			return getFISOutput(fis, index).GetNature() == nature;
		} catch (Throwable e) {
			return false;
		}
	}

	public static String getFISInputName(JFIS fis, int index) {
		try {
			return getFISInput(fis, index).GetName();
		} catch (Throwable e) {
			return null;
		}
	}

	public static int getFISIndex(JFIS fis, int outputIndex) {
		try {
			return fis.NbInputs() + outputIndex;
		} catch (Throwable e) {
			return 0;
		}
	}

	public static boolean hasOutput(JFIS fis) {
		try {
			return fis.NbOutputs() != 0;
		} catch (Throwable e) {
			return false;
		}
	}

	public static boolean isOutputClassif(JFIS fis, int outputIndex) {
		try {
			return hasOutput(fis) && fis.GetOutput(outputIndex).GetClassif();
		} catch (Throwable e) {
			return false;
		}
	}

	public static Range<Integer> outputIndexRange(JFIS fis) {
		try {
			return Ranges.closed(fis.NbInputs(), fis.NbInputs() + fis.NbActiveOutputs() - 1);
		} catch (Throwable e) {
			return Ranges.closed(0, 0);
		}
	}

	public static boolean isOutput(JFIS fis, int index) {
		return outputIndexRange(fis).contains(index);
	}

	public static Iterable<JInput> iterableInputs(JFIS fis) {
		try {
			return transform($(fis.NbInputs()), toInput(fis));
		} catch (Throwable e) {
			return emptyList();
		}
	}

	private static Function<Integer, JInput> toInput(final JFIS fis) {
		return new Function<Integer, JInput>() {

			@Override
			public JInput apply(Integer inputIndex) {
				try {
					return fis.GetInput(inputIndex);
				} catch (Throwable e) {
					return null;
				}
			}
		};
	}
}
