//***********************************************************************
//
//                              ExtendedPlot3D.java
//
// Author(s) : Mathieu GRELIER
// FISPRO Version : 3.0 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Contact : fispro@ensam.inra.fr
// Last modification date:  December 25, 2005
//
//**********************************************************************
package org.freehep.j3d.plot;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Enumeration;

import javax.media.j3d.AmbientLight;
import javax.media.j3d.Appearance;
import javax.media.j3d.Background;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.ColoringAttributes;
import javax.media.j3d.DirectionalLight;
import javax.media.j3d.GraphicsContext3D;
import javax.media.j3d.Group;
import javax.media.j3d.ImageComponent;
import javax.media.j3d.ImageComponent2D;
import javax.media.j3d.J3DGraphics2D;
import javax.media.j3d.Node;
import javax.media.j3d.Raster;
import javax.media.j3d.Shape3D;
import javax.media.j3d.TransformGroup;
import javax.vecmath.Color3f;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGEncodeParam;
import com.sun.image.codec.jpeg.JPEGImageEncoder;
import com.sun.j3d.utils.universe.SimpleUniverse;

public abstract class ExtendedPlot3D extends Plot3D implements PickAndClickProcessable {
    protected Node plot;

    protected BranchGroup scene;

    private int postSwapCount_ = 0;

    private boolean recording = false;

    private int mouse_posx, mouse_posy;

    private String recordFileNamePrefix = null;

    private ExtendedPlot3DToolTip toolTip = null;

    ExtendedPlot3D() {
        super();
        this.toolTip = new ExtendedPlot3DToolTip(this);
        this.addMouseMotionListener(new MouseMotionListener() {
            public void mouseDragged(MouseEvent e) {}

            public void mouseMoved(MouseEvent e) {
                toolTip.hideToolTip();
            }
        });
    }

    protected void init() {
        plot = createPlot();
        Background background = new Background(new Color3f(Color.white));
        background.setApplicationBounds(new BoundingSphere()) ;
        ((Group)plot).addChild(background);

        scene = defineMouseBehaviour(plot);
        scene.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
        scene.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
        scene.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
        // if you want to detach the branchgroup at runtime
        scene.setCapability(BranchGroup.ALLOW_DETACH);
        setPickCapability(); //plot type dependant
        setupLights(scene); // Surface plot wants an extra light and overrides this method
        scene.compile(); //Java3D optimization

        universe = new SimpleUniverse(this);
        universe.getViewingPlatform().setNominalViewingTransform();
        universe.addBranchGraph(scene);

        if (parallelProjection) {
            setProjectionPolicy(universe, parallelProjection);
        }

        init = true;
    }

    protected Node applyShape3DColor(Shape3D shape3D, Color color) {
        Appearance appearance = new Appearance();
        appearance.setCapability(Appearance.ALLOW_COLORING_ATTRIBUTES_WRITE);
        appearance.setColoringAttributes(new ColoringAttributes(new Color3f(color), ColoringAttributes.FASTEST));
        shape3D.setAppearance(appearance);
        return shape3D;
    }
    
    public void resetView() {
        Enumeration childs = this.scene.getAllChildren();
        while (childs.hasMoreElements() == true) {
            Object obj = childs.nextElement();
            if (obj.getClass() == TransformGroup.class) {
                ((TransformGroup) obj).setTransform(this.createDefaultOrientation());
                return;
            }
        }
    }

    protected BranchGroup defineMouseBehaviour(Node scene) {
        BranchGroup bg = super.defineMouseBehaviour(scene);
        PickAndClickBehavior pc_behavior = new PickAndClickBehavior(this, this, bg);
        pc_behavior.setSchedulingBounds(new BoundingSphere());
        bg.addChild(pc_behavior);
        return bg;
    }

    public void setMousePosition(int x_pos, int y_pos) //the mouse position on this Canvas3D
    {
        this.mouse_posx = x_pos;
        this.mouse_posy = y_pos;
    }

    protected void setupLights(BranchGroup root) {
        DirectionalLight lightD = new DirectionalLight();
        lightD.setDirection(new Vector3f(0.0f, -0.7f, -0.7f));
        lightD.setInfluencingBounds(getDefaultBounds());
        root.addChild(lightD);

        //  This second light is added for the Surface Plot, so you
        //  can see the "under" surface
        DirectionalLight lightD1 = new DirectionalLight();
        lightD1.setDirection(new Vector3f(0.0f, 0.7f, 0.7f));
        lightD1.setInfluencingBounds(getDefaultBounds());
        root.addChild(lightD1);

        AmbientLight lightA = new AmbientLight();
        lightA.setInfluencingBounds(getDefaultBounds());
        root.addChild(lightA);
    }

    //image recording capability
    private BufferedImage getImage() {
        GraphicsContext3D ctx = getGraphicsContext3D();
        // The raster components need all be set!
        Raster ras = new Raster(new Point3f(-1.0f, -1.0f, -1.0f), Raster.RASTER_COLOR, 0, 0, this.getWidth(), this.getHeight(), new ImageComponent2D(ImageComponent.FORMAT_RGB, new BufferedImage(this.getWidth(), this.getHeight(), BufferedImage.TYPE_INT_RGB)), null);

        ctx.readRaster(ras);

        // Now strip out the image info
        return ras.getImage().getImage();
    }

    public void postSwap() {

        if (recording) {
            try {
                BufferedImage img = getImage();
                NumberFormat formatter = new DecimalFormat("000000000");
                String s = formatter.format(postSwapCount_);
                FileOutputStream out = new FileOutputStream(this.recordFileNamePrefix + s + ".jpg");
                JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(out);
                JPEGEncodeParam param = encoder.getDefaultJPEGEncodeParam(img);
                param.setQuality(0.9f, false); // 90% qualith JPEG
                encoder.setJPEGEncodeParam(param);
                encoder.encode(img);
                out.close();
            } catch (IOException e) {
                System.out.println(e.getMessage());
            }
            postSwapCount_++;
        }
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.Canvas3D#postRender()
     */
    public void postRender() {
        super.postRender();
        J3DGraphics2D g = getGraphics2D();
        this.toolTip.paint(g);
        g.flush(true);
    }

    /**
     * Record jpegs images from the 3D plot evolution<p>
     * each time the plot rendering is refreshed, postSwap() method is called. This method was overridden to record to disk the corresponding new image converted in jpeg<p>
     * So a group of indexed jpegs images can be obtained in a folder<p>
     * It can be converted in a mpeg or other, for example using Linux Mplayer+mencoder
     * For ex, with mencoder the command line is :
     * ->Creating an MPEG-4 file from all the JPEG files in the current directory:
     * mencoder mf://*.jpg -mf w=800:h=600:fps=25:type=jpg -ovc lavc -lavcopts vcodec=mpeg4 -oac copy -o output.avi<p>
     * this can be played on windows...
     * @param recording recording flag
     * @param recordFileNamePrefix file name preffix
     */
    public void record(boolean recording, String recordFileNamePrefix) {
        this.recording = recording;
        this.recordFileNamePrefix = recordFileNamePrefix;
    }

    /**
     * @return Renvoie toolTip.
     */
    public ExtendedPlot3DToolTip getToolTip() {
        return toolTip;
    }

    /**
     * @return Renvoie mouse_posx.
     */
    public int getMousePosX() {
        return mouse_posx;
    }

    /**
     * @return Renvoie mouse_posy.
     */
    public int getMousePosY() {
        return mouse_posy;
    }

    public Dimension getPreferredSize() {
        return new Dimension(400, 400);
    }

    /* (non-Javadoc)
     * @see java.awt.Component#getMinimumSize()
     */
    public Dimension getMinimumSize() {
        return new Dimension(200, 200);
    }

    /* (non-Javadoc)
     * @see javax.media.j3d.Canvas3D#paint(java.awt.Graphics)
     */
    public void paint(Graphics g) {
        super.paint(g);
        // This is a regression in JDK 1.4 (see bug ID Bug ID 4374079) (http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4374079) It only manifests itself when running the application with the javaw command, and you don't see it when running under the normal java command.
        // There are two workarounds for this problem - make sure you always use the java command or create your own extended class that extends Canvas3D paint() and call Toolkit.sync()
        Toolkit.getDefaultToolkit().sync();
    }

    /* (non-Javadoc)
     * @see java.awt.Component#print(java.awt.Graphics)
     */
    public void print(Graphics g) {
        g.drawImage(this.getImage(), 0, 0, this);
    }
}
