//***********************************************************************
//
//                              ExtendedPlot3DToolTip.java
//
// Author(s) : Mathieu GRELIER, Jean-luc LABLEE
// FISPRO Version : 3.0 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Contact : fispro@ensam.inra.fr
// Last modification date:  December 25, 2005
//
//**********************************************************************
package org.freehep.j3d.plot;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.UIManager;

public class ExtendedPlot3DToolTip extends Canvas {

    private ExtendedPlot3D canvas = null; //specific container

    private boolean isPaintable = false;

    private Vector strings = null;

    private Vector tip_lines = null;

    private int HORIZONTAL_ENLARGE = 10;

    private int VERTICAL_OFFSET = 1;

    private int possible_number_of_lines;

    public ExtendedPlot3DToolTip(ExtendedPlot3D canvas) {
        this.canvas = canvas;
    }

    public void setToolTipText(String text) {
        this.isPaintable = true;
        this.ExtractSubStrings(text);
        this.canvas.paint(getGraphics());
    }

    public void hideToolTip() {
        if (this.isPaintable == true) {
            isPaintable = false;
            // delai d'effacement
            (new Timer()).schedule(new TimerTask() {
                public void run() {
                    if (isPaintable == false)
                        canvas.paint(getGraphics());
                }
            }, 500);
        }
    }

    public void paint(Graphics g) {
        if (this.isPaintable == true) {
            g.setColor(Color.black);
            g.drawRect(this.getLocation().x, this.getLocation().y, getSize().width, getSize().height);
            g.setColor((Color) UIManager.get("ToolTip.background"));
            g.fillRect(this.getLocation().x + 1, this.getLocation().y + 1, getSize().width - 1, ((getSize().height - 1)));
            g.setColor(Color.black);
            for (int i = 0; i < possible_number_of_lines; i++) {
                String s = (String) strings.get(i);
                if ((s != null) && (s.equals("") == false))
                    g.drawString(s, this.getLocation().x + 3, this.getLocation().y + 1 + ((((getSize().height) / possible_number_of_lines) * (i + 1)) - VERTICAL_OFFSET));
            }
        }
    }

    private void ExtractSubStrings(String raw_tip) {
        // Compile regular expression
        String patternStr = "<n>";
        Pattern pattern = Pattern.compile(patternStr);
        // Determine if pattern exists in input
        Matcher matcher = pattern.matcher(raw_tip);
        boolean manylinesFound = matcher.find(); // true
        strings = new Vector();
        if (manylinesFound) {
            String substr;
            int substring_start = 0;
            int substring_end;
            int match_start;
            int match_end;
            while (manylinesFound) {
                // Get indices of matching string
                match_start = matcher.start();
                match_end = matcher.end();
                substring_end = match_start;
                substr = raw_tip.substring(substring_start, substring_end);
                strings.add(substr);
                substring_start = match_end + 1;
                // Find the next occurrence
                manylinesFound = matcher.find(); // true
            }
            //extract the last part of the string
            substr = raw_tip.substring(substring_start, raw_tip.length()); //-1 is important for this last part
            strings.add(substr);
        } else
            strings.add(raw_tip);
        tip_lines = strings;
        createToolTip();
    }

    private void createToolTip() {
        FontMetrics fm = getFontMetrics(this.canvas.getFont());
        boolean placeit_in_upper_part = false;
        boolean lower_part = false;
        if (this.canvas.getMousePosY() > (this.canvas.getHeight() / 2)) //if there is more space in the panel above the mouse cursor
            lower_part = true;
        if ((this.canvas.getMousePosY() - (fm.getHeight() * (this.tip_lines.size()))) < 0 && (this.canvas.getMousePosY() + (fm.getHeight() * this.tip_lines.size())) > (this.canvas.getHeight())) //if the tooltip height doesn't fit in the window ; theres's two lines added at the beginning of the tip
        {
            int possible_height;
            if (lower_part) {
                possible_height = this.canvas.getMousePosY(); //we keep this maximum possible height for the tooltip
                placeit_in_upper_part = true;
            } else
                possible_height = this.canvas.getHeight() - this.canvas.getMousePosY(); //else we take the space under the mouse cursor
            //now calculate the possible number of lines
            possible_number_of_lines = (int) Math.floor((double) possible_height / (double) fm.getHeight());
            this.tip_lines.add(possible_number_of_lines - 1, new String("...")); //replace the last lines with dots to evocate that it lacks space
        } else {
            possible_number_of_lines = this.tip_lines.size();
            if (lower_part)
                placeit_in_upper_part = true;
        }

        int max_length_string = 0;
        for (int i = 0; i < possible_number_of_lines; i++) {
            if (((String) this.tip_lines.get(i)).length() >= ((String) this.tip_lines.get(max_length_string)).length())
                max_length_string = i;
        }

        setSize(fm.stringWidth((String) (this.tip_lines.get(max_length_string))) + HORIZONTAL_ENLARGE, ((fm.getHeight() * possible_number_of_lines) - 10));

        //diagonal offset can be used here
        if (placeit_in_upper_part) {
            if (this.canvas.getMousePosX() - (this.getWidth()) < this.canvas.getLocation().x)
                setLocation(this.canvas.getMousePosX(), this.canvas.getMousePosY() - (this.getHeight()));
            else
                setLocation(this.canvas.getMousePosX() - (this.getWidth()), this.canvas.getMousePosY() - (this.getHeight()));
        } else {
            if (this.canvas.getMousePosX() - (this.getWidth()) < this.canvas.getLocation().x)
                setLocation(this.canvas.getMousePosX(), this.canvas.getMousePosY());
            else
                setLocation(this.canvas.getMousePosX() - (this.getWidth()), this.canvas.getMousePosY());

        }
    }

}
