//***********************************************************************
//
//                              OutputInferBinned2DData.java
//
// Author(s) : Mathieu GRELIER
// FISPRO Version : 3.0 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Contact : fispro@ensam.inra.fr
// Last modification date:  December 25, 2005
//
//**********************************************************************
package org.freehep.j3d.plot;

import java.io.*;
import javax.vecmath.Point3f;
import javax.vecmath.Point3d;

/**
 * Description : implementation of freehep's Binned2DData interface for SurfacePlot2
 */
public class OutputInferBinned2DData implements Binned2DData {
    private int xBins;

    private int yBins;

    private Rainbow rainbow;

    private float Xmin, Xmax, Ymin, Ymax, Zmin, Zmax;

    private double local_xmin = -0.5d, local_ymin = -0.5d, local_xmax = 0.5d, local_ymax = 0.5d;

    private float[][] data;

    public OutputInferBinned2DData(double[] inferdata, double Xmin, double Xmax, double Ymin, double Ymax) throws IOException {
        this.Xmin = (float) Xmin;
        this.Xmax = (float) Xmax;
        this.Ymin = (float) Ymin;
        this.Ymax = (float) Ymax;
        xBins = (int) inferdata[0];
        yBins = (int) inferdata[1];
        data = new float[xBins][yBins];
        int count = 2;
        float zmin = (float) inferdata[2];
        float zmax = zmin; //to calculate zmin and zmax while filling the data array
        for (int i = 0; i < xBins; i++)
            for (int j = yBins - 1; j >= 0; j--) { // invert y axis
                data[i][j] = (float) inferdata[count];
                if (data[i][j] < zmin)
                    zmin = data[i][j];
                if (data[i][j] > zmax)
                    zmax = data[i][j];

                count++;
            }
        if (zmin == zmax) //possible case -> it causes a crash
        {
            //arbitrary choices to avoid it
            zmin = 0f;
            zmax = 1f;
        }
        this.Zmin = zmin;
        this.Zmax = zmax;
        rainbow = new Rainbow(Zmin, Zmax);

    }

    public int xBins() {
        return xBins;
    }

    public int yBins() {
        return yBins;
    }

    public float xMin() {
        return Xmin;
    }

    public float xMax() {
        return Xmax;
    }

    public float yMin() {
        return Ymin;
    }

    public float yMax() {
        return Ymax;
    }

    public float zMin() {
        return Zmin;
    }

    public float zMax() {
        return Zmax;
    }

    public float zAt(int xIndex, int yIndex) {
        return data[xIndex][yIndex];
    }

    public void setLocalMinsAnsMax(double local_xmin, double local_xmax, double local_ymin, double local_ymax) {
        //set x and y mins and maxs in the 3D space, OutputInferBinned2DDAta need to know
        //that to be able to convert a point in the 3D coordinates into an original data
        //value ; these values are the ones used by the plot builder
        this.local_xmin = local_xmin;
        this.local_xmax = local_xmax;
        this.local_ymin = local_ymin;
        this.local_ymax = local_ymax;
    }

    public Point3f localPointToInferedPoint(Point3d local_point) {
        double xbin = Math.floor((local_point.x - local_xmin) / ((local_xmax - local_xmin) / this.xBins));
        double ybin = Math.floor((local_point.y - local_ymin) / ((local_ymax - local_ymin) / this.yBins));
        float xAt = (float) (this.Xmin + (((this.Xmax - this.Xmin) / (this.xBins - 1)) * xbin));
        float yAt = (float) (this.Ymax - (((this.Ymax - this.Ymin) / (this.yBins - 1)) * ybin)); // invert y axis
        float zAt = this.zAt((int) xbin, (int) ybin);
        Point3f p = new Point3f(xAt, yAt, zAt);
        return p;
    }

    public javax.vecmath.Color3b colorAt(int xIndex, int yIndex) {
        return rainbow.colorFor(zAt(xIndex, yIndex));
    }
}
