//***********************************************************************
//
//                              PickAndClickBehavior.java
//
// Author(s) : Mathieu GRELIER
// FISPRO Version : 3.0 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Contact : fispro@ensam.inra.fr
// Last modification date:  December 25, 2005
//
//**********************************************************************
package org.freehep.j3d.plot;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.picking.PickResult;

/**
 * Description : an additional mouse behavior for ExtendedPlot3D class of this package
 */
public class PickAndClickBehavior extends Behavior {

    private WakeupCriterion[] mouseEvents;

    private WakeupOr mouseCriterion;

    private int x, y;

    private BranchGroup branchGroup;

    private PickAndClickProcessable processable;

    private Canvas3D canvas;

    private PickRay pickRay = new PickRay();

    private PickResult pickresult;

    private SceneGraphPath sceneGraphPath;

    private boolean now_picked = false;

    private boolean debug = false;
    
    public PickAndClickBehavior(PickAndClickProcessable processable, Canvas3D canvas, BranchGroup branchGroup) {
        //necessary referencings
        this.processable = processable;
        this.canvas = canvas;
        this.branchGroup = branchGroup;
    }

    public void initialize() {
        //the events we are going to handle
        mouseEvents = new WakeupCriterion[3];
        mouseEvents[0] = new WakeupOnAWTEvent(MouseEvent.MOUSE_MOVED);
        mouseEvents[1] = new WakeupOnAWTEvent(MouseEvent.MOUSE_CLICKED);
        mouseEvents[2] = new WakeupOnAWTEvent(MouseEvent.MOUSE_DRAGGED);
        mouseCriterion = new WakeupOr(mouseEvents);
        wakeupOn(mouseCriterion);
    }

    public void processStimulus(Enumeration criteria) { //this defines what we are going to do
        WakeupCriterion wakeup;
        AWTEvent[] event;
        int id;

        while (criteria.hasMoreElements()) {
            wakeup = (WakeupCriterion) criteria.nextElement();
            if (wakeup instanceof WakeupOnAWTEvent) {
                event = ((WakeupOnAWTEvent) wakeup).getAWTEvent();
                for (int i = 0; i < event.length; i++) {
                    id = event[i].getID();
                    if (id == MouseEvent.MOUSE_MOVED) { //we try to pick a the plot

                        //PickRay creation from the mouse position
                        x = ((MouseEvent) event[i]).getX();
                        y = ((MouseEvent) event[i]).getY();
                        processable.setMousePosition(x, y);

                        Point3d eyePos = new Point3d();
                        canvas.getCenterEyeInImagePlate(eyePos);

                        Point3d mousePos = new Point3d();
                        canvas.getPixelLocationInImagePlate(x, y, mousePos);

                        Transform3D transform3D = new Transform3D();
                        canvas.getImagePlateToVworld(transform3D);

                        transform3D.transform(eyePos);
                        transform3D.transform(mousePos);

                        Vector3d direction = new Vector3d(mousePos);
                        direction.sub(eyePos);
                        pickRay.set(eyePos, direction);
                        //use this pickRay to pick the plot
                        sceneGraphPath = branchGroup.pickClosest(pickRay);
                        
                        if (sceneGraphPath != null) {//there is a pick hit
                            this.now_picked = true;//the mouse is over a pickable shape
                            pickresult = new PickResult(sceneGraphPath, pickRay);//this stores info about the hit
                            if (pickresult.getObject() instanceof Shape3D) { //we pick only Shape3D instances
                                try {
                                    if (pickresult.numIntersections() != 0) 
                                        processable.processIntersection(pickresult);
                                } catch (CapabilityNotSetException e) {
                                    if (debug)
                                        System.out.println(e.getMessage());
                                }

                                //Normally, there should not be such exceptions since all the potentially
                                //pickable objects other than the plot contents (items or surface) are set
                                //to be non pickable
                            }
                        } else //no pick
                        {
                            this.now_picked = false;
                            this.pickresult = null;
                            processable.processNoIntersection();

                        }
                    }

                    else
                        if (id == MouseEvent.MOUSE_CLICKED) {//to be able to left or right click the plot
                            processable.processNoIntersection();
                            if (this.now_picked && pickresult.numIntersections() != 0) //no action if there is no pick at this moment or if there is no intersection
                            {
                                if ((((MouseEvent) event[i]).getModifiers() & InputEvent.BUTTON1_MASK) != 0 && (((MouseEvent) event[i]).getClickCount() == 2)) //right click
                                {
                                    processable.processLeftDoubleClick(pickresult);
                                }

                                if ((((MouseEvent) event[i]).getModifiers() & InputEvent.BUTTON1_MASK) != 0) //left click
                                    processable.processLeftClick(pickresult);
                                else
                                    if ((((MouseEvent) event[i]).getModifiers() & InputEvent.BUTTON2_MASK) != 0) //center click
                                        processable.processCenterClick(pickresult);
                                    else
                                        if ((((MouseEvent) event[i]).getModifiers() & InputEvent.BUTTON3_MASK) != 0) //right click
                                            processable.processRightClick(pickresult);

                            }
                        } else
                            if (id == MouseEvent.MOUSE_DRAGGED) { //the drag behavior is handled by
                                //the standard mouse behaviors that must already be added (see freehep's Plot3D in init() method). Here, we want to remove the
                                //tool tip -if it is displayed- from the plot when there is a mouse translate,
                                //mouse rotate or mouse zoom (Fispro plots implement processNoIntersectioncalling removeToolTip
                                this.now_picked = false;
                                this.pickresult = null;
                                processable.processNoIntersection();
                            }
                }
            }
        }
        wakeupOn(mouseCriterion); //necessary re-initialization
    }
}
