//***********************************************************************
//
//                              SurfaceBuilder2.java
//
// Author(s) : Mathieu GRELIER
// FISPRO Version : 3.0 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Contact : fispro@ensam.inra.fr
// Last modification date:  December 25, 2005
//
//**********************************************************************
package org.freehep.j3d.plot;

import javax.media.j3d.Appearance;
import javax.media.j3d.Geometry;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.LineArray;
import javax.media.j3d.Material;
import javax.media.j3d.Node;
import javax.media.j3d.PointArray;
import javax.media.j3d.PolygonAttributes;
import javax.media.j3d.Shape3D;
import javax.media.j3d.Switch;
import javax.vecmath.Color3b;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import com.sun.j3d.utils.geometry.GeometryInfo;
import com.sun.j3d.utils.geometry.NormalGenerator;
import com.sun.j3d.utils.geometry.Stripifier;

/**
 * Description : Adapted from freehep's SurfaceBuilder class (Joy Kyriakopulos)
 * build 2 geometries for SurfacePlot2 (surface or points)
 */
public class SurfaceBuilder2 extends AbstractPlotBuilder {
    private boolean drawSurface = true;

    private static final Vector3f xnormal = new Vector3f(1f, 0, 0);

    private int numWireLines = 600;

    private int popBins = 0;

    private static final int fullPlotChild = 0;

    private Shape3D shape;

    private Shape3D pointShape;

    private Switch targetSwitch;

    private PointArray point;

    private int bcur;

    private boolean shapeIsSurface = false;

    public Node buildContent(NormalizedBinned2DData data) {

        // Build switch node to switch between full plot and sparse plot
        targetSwitch = new Switch();
        targetSwitch.setCapability(Switch.ALLOW_SWITCH_WRITE);

        // Create shape and geometry for full plot
        if (drawSurface) {
            shape = createShape();
            shapeIsSurface = true;
            shape.setCapability(shape.ALLOW_GEOMETRY_WRITE);
            shape.setCapability(Shape3D.ALLOW_GEOMETRY_READ);
            shape.setCapability(shape.ALLOW_APPEARANCE_WRITE);

            //   set the correct capabilities to allow info retrieving when plot is picked
            GeometryArray geom = (GeometryArray) (buildGeometry(data)); //it is a triangle strip array
            geom.setCapability(GeometryArray.ALLOW_COORDINATE_READ);
            geom.setCapability(GeometryArray.ALLOW_COUNT_READ);
            geom.setCapability(GeometryArray.ALLOW_FORMAT_READ);

            shape.setGeometry(geom); // draw lego as full plot
        } else {
            pointShape = createPointShape();
            shapeIsSurface = false;
            pointShape.setCapability(shape.ALLOW_GEOMETRY_WRITE);
            pointShape.setCapability(Shape3D.ALLOW_GEOMETRY_READ);
            popBins = calcPopBins(data); // buildWireGeometry won't calculate # populated bins
            int[] binInc = { 1, 1 };
            pointShape.setGeometry(buildPointGeometry(data, binInc)); // draw full "wire frame" plot
        }

        pointShape = createPointShape();
        pointShape.setCapability(shape.ALLOW_GEOMETRY_WRITE);

        targetSwitch.addChild(shape); // full plot should be added first
        targetSwitch.addChild(pointShape); // sparse added second

        //If necessary, create geometry for a sparsified plot
        targetSwitch.setUserData(Boolean.FALSE);

        targetSwitch.setWhichChild(fullPlotChild); // full plot displayed by default
        return targetSwitch;

    }

    public void updatePlot(NormalizedBinned2DData data) {
        if (this.drawSurface) {
            //   set again the correct capabilities to allow info retrieving when plot is picked
            GeometryArray geom = (GeometryArray) (buildGeometry(data)); //it is a triangle strip array
            geom.setCapability(GeometryArray.ALLOW_COORDINATE_READ);
            geom.setCapability(GeometryArray.ALLOW_COUNT_READ);
            geom.setCapability(GeometryArray.ALLOW_FORMAT_READ);

            shape.setGeometry(geom); // draw lego as full plot
        } else
            shape.setGeometry(buildPointGeometry(data));
    }

    public void setDrawSurface(boolean b) {
        drawSurface = b;
    }

    private Geometry buildGeometry(NormalizedBinned2DData data) {

        int nXbins = data.xBins();
        int nYbins = data.yBins();
        float xBinWidth = 1.f / nXbins;
        float yBinWidth = 1.f / nYbins;
        float x, y, z;
        int i, j, k, l;

        Point3d bcoord[] = new Point3d[((int) nXbins - 1) * ((int) nYbins - 1) * 4];
        Color3b bcolor[] = new Color3b[((int) nXbins - 1) * ((int) nYbins - 1) * 4];

        for (i = 0; i < ((int) nXbins - 1) * ((int) nYbins - 1) * 4; i++) {
            bcoord[i] = new Point3d();
            bcolor[i] = new Color3b();
        }

        // Fill bcoord array with points that compose the surface
        int bcur = 0;
        for (k = 0, x = -.5f; k < (int) nXbins - 1; k++, x += xBinWidth)
            for (l = 0, y = -.5f; l < (int) nYbins - 1; l++, y += yBinWidth) {

                // Point x,y
                bcoord[bcur].x = x + xBinWidth / 2.f;
                bcoord[bcur].y = y + yBinWidth / 2.f;
                bcoord[bcur].z = data.zAt(k, l);
                bcolor[bcur] = data.colorAt(k, l);
                bcur++;

                // Next point in y direction
                bcoord[bcur].x = x + xBinWidth / 2.f;
                bcoord[bcur].y = y + 1.5f * xBinWidth;
                bcoord[bcur].z = data.zAt(k, l + 1);
                bcolor[bcur] = data.colorAt(k, l + 1);
                bcur++;

                // Next point diagonally
                bcoord[bcur].x = x + 1.5f * xBinWidth;
                bcoord[bcur].y = y + 1.5f * yBinWidth;
                bcoord[bcur].z = data.zAt(k + 1, l + 1);
                bcolor[bcur] = data.colorAt(k + 1, l + 1);
                bcur++;

                // Next point in x direction
                bcoord[bcur].x = x + 1.5f * xBinWidth;
                bcoord[bcur].y = y + yBinWidth / 2.f;
                bcoord[bcur].z = data.zAt(k + 1, l);
                bcolor[bcur] = data.colorAt(k + 1, l);
                bcur++;
            }

        // We make a GeometryInfo object so that normals can be generated
        // for us and geometry stripified for us.  J3D documentation
        // says it's best to do these two steps in this order.

        GeometryInfo geom = new GeometryInfo(GeometryInfo.QUAD_ARRAY);

        geom.setCoordinates(bcoord);
        geom.setColors(bcolor);
        NormalGenerator ng = new NormalGenerator();
        ng.generateNormals(geom);

        // Make normals conform to our "handedness" of z direction
        // i.e. change their sign
        Vector3f normals[] = geom.getNormals();
        for (i = 0; i < normals.length; ++i) {
            normals[i].x = -normals[i].x;
            normals[i].y = -normals[i].y;
            normals[i].z = -normals[i].z;
        }
        geom.setNormals(normals);

        Stripifier st = new Stripifier();
        st.stripify(geom);
        geom.recomputeIndices();

        return geom.getGeometryArray();
    }

    private Geometry buildPointGeometry(NormalizedBinned2DData data) {
        // Compute x,y factors: will be > 1 if data is too large and
        // plot needs to be sparsified (i.e. made more sparse)
        int[] wireBinInc = calcXYfactors(data.xBins(), data.yBins());
        return buildPointGeometry(data, wireBinInc);
    }

    private Geometry buildPointGeometry(NormalizedBinned2DData data, int[] wireBinInc) {

        int nXbins = data.xBins();
        int nYbins = data.yBins();

        // Create the coordinate and color arrays

        bcur = 0;
        int maxpoints = ((nXbins / wireBinInc[0] + 1) * (nYbins / wireBinInc[1] + 1)) * 3 * 2 + 8; // An over estimate
        point = new PointArray(maxpoints, LineArray.COORDINATES + LineArray.NORMALS + LineArray.COLOR_3);

        float xBinWidth = (float) wireBinInc[0] / (float) (nXbins);
        float yBinWidth = (float) wireBinInc[1] / (float) (nYbins);
        ;
        float x = -.5f;

        // Go through data, drawing a vertical line instead of a bin

        for (int k = 0; k < nXbins; k += wireBinInc[0], x += xBinWidth) {
            float y = -.5f;
            for (int l = 0; l < nYbins; l += wireBinInc[1], y += yBinWidth) {
                float z = data.zAt(k, l);
                Color3b curColor = data.colorAt(k, l);
                if (z != 0) { // skip drawing if no height
                    drawVPoint(x + xBinWidth / 2.f, y + yBinWidth / 2.f, 0.f, z, curColor, xnormal);
                }
            }
        }
        return point;
    }

    private void drawVPoint(float x, float y, float z1, float z2, Color3b lineColor, Vector3f normal) {

        point.setCoordinate(bcur, new Point3f(x, y, z2));
        point.setColor(bcur, lineColor);
        point.setNormal(bcur, normal);
        bcur++;

    }

    Shape3D createShape() {
        Shape3D surface = new Shape3D();
        surface.setAppearance(createMaterialAppearance());

        return surface;
    }

    Shape3D createPointShape() {
        Shape3D PointShape = new Shape3D();
        PointShape.setAppearance(createPointAppearance());

        return PointShape;
    }

    private Appearance createMaterialAppearance() {
        Appearance materialAppear = new Appearance();
        PolygonAttributes polyAttrib = new PolygonAttributes();
        polyAttrib.setCullFace(PolygonAttributes.CULL_NONE);
        materialAppear.setPolygonAttributes(polyAttrib);

        Material material = new Material();
        // set diffuse color to red (this color will only be used
        //     if lighting disabled - per-vertex color overrides)
        material.setDiffuseColor(new Color3f(1.0f, 0.0f, 0.0f));
        materialAppear.setMaterial(material);

        return materialAppear;
    }

    private Appearance createPointAppearance() {
        Appearance materialAppear = new Appearance();
        Material material = new Material();
        // set diffuse color to red (this color will only be used
        //     if lighting disabled - per-vertex color overrides)
        material.setEmissiveColor(new Color3f(1.0f, 1.0f, 1.0f));
        material.setShininess(128);
        materialAppear.setMaterial(material);

        return materialAppear;
    }

    private int[] calcXYfactors(int nXbins, int nYbins) {
        int[] binInc = { 1, 1 };
        if (nXbins * nYbins > numWireLines) {
            double xyBinRatio = (double) nXbins / (double) nYbins;
            binInc[0] = (int) (nXbins / (Math.sqrt((double) numWireLines) / xyBinRatio));
            binInc[1] = (int) (nYbins / (Math.sqrt((double) numWireLines) * xyBinRatio));
            if (binInc[0] < 1)
                binInc[0] = 1;
            if (binInc[1] < 1)
                binInc[1] = 1;
        }
        return binInc;
    }

    private int calcPopBins(NormalizedBinned2DData data) {
        int nXbins = data.xBins();
        int nYbins = data.yBins();
        int numPopBins = 0;
        float x = -1.f / nXbins - .5f;
        for (int k = -1; k < nXbins; k++, x += 1.f / nXbins) {
            float y = -1.f / nYbins - .5f;
            for (int l = -1; l < nYbins; l++, y += 1.f / nYbins) {
                float z = data.zAt(k, l);
                if (z != 0)
                    ++numPopBins; // keep track of how many bins != 0
            }
        }
        return numPopBins;
    }

}
