//***********************************************************************
//
//                              SurfacePlot2.java
//
// Author(s) : Mathieu GRELIER
// FISPRO Version : 3.0 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Contact : fispro@ensam.inra.fr
// Last modification date:  December 25, 2005
//
//**********************************************************************
package org.freehep.j3d.plot;

import java.awt.Color;
import java.util.Enumeration;

import javax.media.j3d.Appearance;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.DirectionalLight;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.Group;
import javax.media.j3d.Light;
import javax.media.j3d.LineArray;
import javax.media.j3d.LineAttributes;
import javax.media.j3d.Node;
import javax.media.j3d.PolygonAttributes;
import javax.media.j3d.QuadArray;
import javax.media.j3d.Shape3D;
import javax.media.j3d.TransparencyAttributes;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import com.sun.j3d.utils.picking.PickIntersection;
import com.sun.j3d.utils.picking.PickResult;
import com.sun.j3d.utils.picking.PickTool;

/**
 * Description : A 3D surface plot extending freehep's abstract plot builder,
 * alternative to freehep's SurfacePlot.
 * Characteristics :
 * 2D section plane can be shown as a transparent square parallel to the XY basis
 * and default value zone plane can be added, as a empty square with dashed lines,
 * parallel to the Y axis. It can switch between two geometries, one with the surface,
 * the other just with points to accelerate rendering when parameters are continuously
 * changed, ex with sliders.
 * Based on SurfacePlot class from freehep j3DPlot library
 */
public class SurfacePlot2 extends ExtendedPlot3D {
    private OutputInferBinned2DData data;

    private SurfaceBuilder2 builder;

    private Group g;

    private Group extraLightsGroup;

    private ExtendedXAxisBuilder xAxis = null;

    private ExtendedYAxisBuilder yAxis = null;

    private ExtendedZAxisBuilder zAxis = null;

    private QuadArray CarreDefaut;

    private QuadArray CarreSection;

    private Appearance app1;

    private PolygonAttributes polyAttrib1;

    private TransparencyAttributes transpAttrib;

    private boolean isSectionVisible;

    private double xmin;

    private double xmax;

    private double ymin;

    private double ymax;

    private double zmin;

    private double zmax;

    private boolean drawSurface = true;

    private String[] Axislabels = { "x", "y", "z" };

    private float initial_sectionvalue = 0.5f; //invisible

    public float initial_defaultzonevalue = 0f; // position initiale du carre pointille (0=invisible)

    public boolean writeimg_;

    /**
     * 
     */
    public SurfacePlot2() {
        super();
    }

    public SurfacePlot2(String[] Axislabels) {
        super();
        this.Axislabels = Axislabels;
    }

    public void setAxes(String[] Axislabels) {
        this.Axislabels = Axislabels;
    }

    protected void setupLights(BranchGroup root) {
        super.setupLights(root);
        extraLightsGroup = new Group();

        //4 extra lights for the point surface plot. Modifiable.
        //Add here your extra lights definitions and add them to the extra lights group
        DirectionalLight lightD2 = new DirectionalLight();
        lightD2.setDirection(new Vector3f(1.0f, 1.f, 0f));
        lightD2.setInfluencingBounds(getDefaultBounds());
        lightD2.setUserData(new String("D2"));
        lightD2.setCapability(Light.ALLOW_STATE_READ);
        lightD2.setCapability(Light.ALLOW_STATE_WRITE);
        ;
        extraLightsGroup.addChild(lightD2);

        DirectionalLight lightD3 = new DirectionalLight();
        lightD3.setDirection(new Vector3f(-1.0f, -1.0f, 0f));
        lightD3.setInfluencingBounds(getDefaultBounds());
        lightD3.setUserData(new String("D3"));
        lightD3.setCapability(Light.ALLOW_STATE_READ);
        lightD3.setCapability(Light.ALLOW_STATE_WRITE);
        ;
        extraLightsGroup.addChild(lightD3);

        DirectionalLight lightD4 = new DirectionalLight();
        lightD4.setDirection(new Vector3f(1f, 0.0f, 0f));
        lightD4.setInfluencingBounds(getDefaultBounds());
        lightD4.setUserData(new String("D4"));
        lightD4.setCapability(Light.ALLOW_STATE_READ);
        lightD4.setCapability(Light.ALLOW_STATE_WRITE);
        ;
        extraLightsGroup.addChild(lightD4);

        DirectionalLight lightD5 = new DirectionalLight();
        lightD5.setDirection(new Vector3f(-1f, 0.0f, 0f));
        lightD5.setInfluencingBounds(getDefaultBounds());
        lightD5.setUserData(new String("D5"));
        lightD5.setCapability(Light.ALLOW_STATE_READ);
        lightD5.setCapability(Light.ALLOW_STATE_WRITE);
        ;
        extraLightsGroup.addChild(lightD5);
        extraLightsGroup.setCapability(Group.ALLOW_CHILDREN_READ);
        extraLightsGroup.setCapability(Group.ALLOW_CHILDREN_WRITE);

        root.addChild(extraLightsGroup);

        enableExtraLights(false);

    }

    protected void enableExtraLights(boolean enable) //enable/disable the extra lights needed for the point surface plot
    {
        int extraLightsNodeNb = scene.indexOfChild(extraLightsGroup);
        for (Enumeration e = ((Group) (this.scene.getChild(extraLightsNodeNb))).getAllChildren(); e.hasMoreElements();) {
            ((DirectionalLight) (e.nextElement())).setEnable(enable);
        }
    }

    public void addDefaultValueIndicator(double min_value) {
        QuadArray CarreDefaut = new QuadArray(4, LineArray.COORDINATES);
        CarreDefaut.setCoordinate(0, new Point3f(-0.5f, -0.5f, 0.2f));
        CarreDefaut.setCoordinate(1, new Point3f(0.5f, -0.5f, 0.2f));
        CarreDefaut.setCoordinate(2, new Point3f(0.5f, 0.5f, 0.2f));
        CarreDefaut.setCoordinate(3, new Point3f(-0.5f, 0.5f, 0.2f));
        ((Group) plot).addChild(new Shape3D(CarreDefaut));
    }

    public void setData(OutputInferBinned2DData data, String[] Axislabels, boolean donotCalculateAxis) {
        this.data = data;
        if (init) {
            builder.updatePlot(new NormalizedBinned2DData(data));

            if (!donotCalculateAxis) {

                if (!drawSurface)
                    this.enableExtraLights(false);

                if (data.xMin() != xmin || data.xMax() != xmax) {
                    xmin = data.xMin();
                    xmax = data.xMax();
                    xAxis.createLabelsNTicks(xmin, xmax);
                    xAxis.apply();
                }
                if (data.yMin() != ymin || data.yMax() != ymax) {
                    ymin = data.yMin();
                    ymax = data.yMax();
                    yAxis.createLabelsNTicks(ymin, ymax);
                    yAxis.apply();
                }
                if (data.zMin() != zmin || data.zMax() != zmax) {
                    zmin = data.zMin();
                    zmax = data.zMax();
                    zAxis.createLabelsNTicks(zmin, zmax);
                    zAxis.apply();
                }

                if (Axislabels != null)
                    this.Axislabels = Axislabels; //update the axis labels if necessary (an input has changed)
                if (xAxis != null && yAxis != null && zAxis != null) {

                    setXAxisLabel(this.Axislabels[0]);
                    setYAxisLabel(this.Axislabels[1]);
                    setZAxisLabel(this.Axislabels[2]);

                }

            } else {
                if (!drawSurface)
                    this.enableExtraLights(true); // point Surface plot needs extra lights to be more visible

            }
        }
    }

    public String getXAxisLabel() {
        return Axislabels[0];
    }

    public void setXAxisLabel(String s) {
        xAxis.setLabel(s);
        xAxis.apply();
    }

    public String getYAxisLabel() {
        return Axislabels[1];
    }

    public void setYAxisLabel(String s) {
        yAxis.setLabel(s);
        yAxis.apply();
    }

    public String getZAxisLabel() {
        return Axislabels[2];
    }

    public void setZAxisLabel(String s) {
        zAxis.setLabel(s);
        zAxis.apply();
    }

    protected Node createPlot() {

        builder = new SurfaceBuilder2();
        Node box = builder.buildOutsideBox();
        box.setPickable(false);
        plot = builder.buildContent(new NormalizedBinned2DData(data));
        plot.setPickable(true); //we want only the plot to be pickable
        plot.setCapability(Node.ENABLE_PICK_REPORTING);
        plot.setCapability(Shape3D.ALLOW_GEOMETRY_READ);
        plot.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);

        xAxis = new ExtendedXAxisBuilder(); //the non-pickability of the axis is handled
        yAxis = new ExtendedYAxisBuilder(); //after the creation of the ticks (see apply)
        zAxis = new ExtendedZAxisBuilder();

        xAxis.createLabelsNTicks(data.xMin(), data.xMax());
        yAxis.createLabelsNTicks(data.yMin(), data.yMax());
        zAxis.createLabelsNTicks(data.zMin(), data.zMax());

        setXAxisLabel(this.Axislabels[0]);
        setYAxisLabel(this.Axislabels[1]);
        setZAxisLabel(this.Axislabels[2]);

        xAxis.apply();
        xAxis.setAxisNonPickable();
        yAxis.apply();
        yAxis.setAxisNonPickable();
        zAxis.apply();
        zAxis.setAxisNonPickable();

        //default values zone level indicator
        CarreDefaut = new QuadArray(4, LineArray.COORDINATES);
        CarreDefaut.setCapability(GeometryArray.ALLOW_COORDINATE_READ);
        CarreDefaut.setCapability(GeometryArray.ALLOW_COORDINATE_WRITE);
        setDefaultzoneCoordinates(initial_defaultzonevalue);
        Appearance app = new Appearance();
        PolygonAttributes polyAttrib = new PolygonAttributes();
        polyAttrib.setCullFace(PolygonAttributes.CULL_NONE);
        polyAttrib.setPolygonMode(PolygonAttributes.POLYGON_LINE);
        LineAttributes lineAttrib = new LineAttributes();
        lineAttrib.setLinePattern(LineAttributes.PATTERN_DASH);
        app.setPolygonAttributes(polyAttrib);
        app.setLineAttributes(lineAttrib);
        Shape3D default_level = new Shape3D(CarreDefaut, app);
        default_level.setPickable(false);

        //section indicator
        CarreSection = new QuadArray(4, LineArray.COORDINATES);
        CarreSection.setCapability(GeometryArray.ALLOW_COORDINATE_READ);
        CarreSection.setCapability(GeometryArray.ALLOW_COORDINATE_WRITE);
        setSectionCoordinates(initial_sectionvalue, 0);
        app1 = new Appearance();
        app1.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_READ);
        app1.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE);
        app1.setCapability(Appearance.ALLOW_POLYGON_ATTRIBUTES_READ);
        app1.setCapability(Appearance.ALLOW_POLYGON_ATTRIBUTES_WRITE);
        polyAttrib1 = new PolygonAttributes();
        polyAttrib1.setCapability(PolygonAttributes.ALLOW_MODE_READ);
        polyAttrib1.setCapability(PolygonAttributes.ALLOW_MODE_WRITE);
        transpAttrib = new TransparencyAttributes();
        transpAttrib.setCapability(TransparencyAttributes.ALLOW_MODE_READ);
        transpAttrib.setCapability(TransparencyAttributes.ALLOW_MODE_WRITE);
        transpAttrib.setCapability(TransparencyAttributes.ALLOW_VALUE_READ);
        transpAttrib.setCapability(TransparencyAttributes.ALLOW_VALUE_WRITE);
        polyAttrib1.setCullFace(PolygonAttributes.CULL_NONE);
        transpAttrib.setTransparency(0.7f);
        setSectionVisible(false);
        app1.setPolygonAttributes(polyAttrib1);
        app1.setTransparencyAttributes(transpAttrib);
        Shape3D section_level = new Shape3D(CarreSection, app1);
        section_level.setPickable(false);

        g = new Group();
        g.addChild(applyShape3DColor((Shape3D)box, Color.black));
        g.addChild(plot);
        g.addChild(xAxis.getNode());
        g.addChild(yAxis.getNode());
        g.addChild(zAxis.getNode());
        g.addChild(default_level);
        g.addChild(section_level);
        return g;
    }

    public void setDrawSurface(boolean b) {
        drawSurface = b;
        if (drawSurface)
            this.enableExtraLights(false);
        else
            this.enableExtraLights(true);
        if (builder != null) {
            builder.setDrawSurface(b);
            if (data != null) {
                this.setData(data, null, !b); //if draw surface is set on, when this method is invoked to refresh the plot, tha axis must be calculated
            }
        }
    }

    public void setDefaultzoneCoordinates(float defautZoneLevel) //called by the parameters frame
    {
        if (CarreDefaut != null) {

            CarreDefaut.setCoordinate(0, new Point3f(-0.5f, -0.5f, defautZoneLevel));
            CarreDefaut.setCoordinate(1, new Point3f(0.5f, -0.5f, defautZoneLevel));
            CarreDefaut.setCoordinate(2, new Point3f(0.5f, 0.5f, defautZoneLevel));
            CarreDefaut.setCoordinate(3, new Point3f(-0.5f, 0.5f, defautZoneLevel));
        }
    }

    public void setSectionCoordinates(float sectionLevel, int whichXorYaxis) //called by the parameters frame, whichXorYaxis=0 for X axis, 1 for Y
    {
        if (whichXorYaxis == 0) {
            CarreSection.setCoordinate(0, new Point3f(-0.5f, sectionLevel, 0f));
            CarreSection.setCoordinate(1, new Point3f(0.5f, sectionLevel, 0f));
            CarreSection.setCoordinate(2, new Point3f(0.5f, sectionLevel, 1f));
            CarreSection.setCoordinate(3, new Point3f(-0.5f, sectionLevel, 1f));
        } else {
            CarreSection.setCoordinate(0, new Point3f(sectionLevel, -0.5f, 0f));
            CarreSection.setCoordinate(1, new Point3f(sectionLevel, 0.5f, 0f));
            CarreSection.setCoordinate(2, new Point3f(sectionLevel, 0.5f, 1f));
            CarreSection.setCoordinate(3, new Point3f(sectionLevel, -0.5f, 1f));
        }

    }

    public void setSectionVisible(boolean setVisible) {
        if (setVisible) {
            polyAttrib1.setPolygonMode(PolygonAttributes.POLYGON_FILL);
            transpAttrib.setTransparencyMode(TransparencyAttributes.BLENDED);
            app1.setPolygonAttributes(polyAttrib1);
            app1.setTransparencyAttributes(transpAttrib);
            isSectionVisible = true;
        } else {
            polyAttrib1.setPolygonMode(PolygonAttributes.POLYGON_LINE);
            transpAttrib.setTransparencyMode(TransparencyAttributes.NONE);
            app1.setPolygonAttributes(polyAttrib1);
            app1.setTransparencyAttributes(transpAttrib);

            isSectionVisible = false;

        }
    }

    public boolean isSectionVisible() {
        return isSectionVisible;
    }

    public void setPickCapability() {
        for (Enumeration e = this.g.getAllChildren(); e.hasMoreElements();) {
            Object el = e.nextElement();
            try {
                if (el instanceof Shape3D) {
                    PickTool.setCapabilities(((Shape3D) (el)), PickTool.INTERSECT_FULL); // set the pick capability for the items to allow picking at the specified detail level : here we want all the info to be retrieved
                }
            } catch (Throwable t) {
                System.out.println(t.getMessage());
            }
        }
    }

    public void setMousePosition(int x, int y) {
        super.setMousePosition(x, y);
    }

    public void processIntersection(PickResult result) {
    //can be defined somewhere else, ex. in an anonymous class
    }

    public void processNoIntersection() {
    //can be defined somewhere else, ex. in an anonymous class
    }

    public void processLeftClick(PickResult result) {
    //can be defined somewhere else, ex. in an anonymous class
    }

    public void processRightClick(PickResult result) {
    //can be defined somewhere else, ex. in an anonymous class
    }

    public void processCenterClick(PickResult result) {
    //can be defined in an anonymous class
    }

    public void processLeftDoubleClick(PickResult result) {
    //can be defined somewhere else, ex. in an anonymous class
    }

    public Point3f getPickIntersectionPoint(PickResult result) {
        double distance = Double.MAX_VALUE;
        PickIntersection intersection = null;
        for( int i=0 ; i < result.numIntersections() ; i++ ) {
            if( result.getIntersection(i).getDistance() < distance ) {
                distance = result.getIntersection(i).getDistance();
                intersection = result.getIntersection(i);
            }
        }
        return this.data.localPointToInferedPoint(intersection.getClosestVertexCoordinates());
    }
}
