//***********************************************************************
//
//                              View3DItemBuilder.java
//
// Author(s) : Mathieu GRELIER
// FISPRO Version : 3.0 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Contact : fispro@ensam.inra.fr
// Last modification date:  December 25, 2005
//
//**********************************************************************
package org.freehep.j3d.plot;

import java.io.IOException;

import javax.media.j3d.Appearance;
import javax.media.j3d.Geometry;
import javax.media.j3d.Material;
import javax.media.j3d.Node;
import javax.media.j3d.PolygonAttributes;
import javax.media.j3d.QuadArray;
import javax.media.j3d.Shape3D;
import javax.vecmath.Color3b;
import javax.vecmath.Color3f;
import javax.vecmath.Point3f;

import com.sun.j3d.utils.geometry.GeometryInfo;
import com.sun.j3d.utils.geometry.NormalGenerator;

public class View3DItemBuilder extends AbstractPlotBuilder {
    private ViewNormalizedUnbinned3DData norm_data;

    private int current_item;

    private Shape3D shape;

    private float c1;

    private float c2;

    private float scale_factor;

    private Color3b[] cb_tmp;

    public View3DItemBuilder(ViewUnbinned3DData raw_data, float shift) throws IOException {
        this.norm_data = new ViewNormalizedUnbinned3DData(raw_data, shift);
        this.calculate_scale_factor(); // not used
    }

    public Node buildContent(int item_number, boolean active) {
        this.current_item = item_number;
        shape = createShape();
        shape.setCapability(Shape3D.ALLOW_GEOMETRY_READ);
        shape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
        QuadArray geom = (QuadArray) buildGeometry(this.norm_data.pointAt(current_item), active);
        geom.setCapability(QuadArray.ALLOW_COLOR_WRITE);
        geom.setCapability(QuadArray.ALLOW_COORDINATE_READ);
        geom.setCapability(QuadArray.ALLOW_COORDINATE_WRITE);
        geom.setCapability(QuadArray.ALLOW_COUNT_READ);
        geom.setCapability(QuadArray.ALLOW_COUNT_WRITE);

        shape.setGeometry(geom);
        return shape;
    }

    private void calculate_scale_factor() {
        //use this method to to modify the item's scale according to their number
        this.scale_factor = 1f;
    }

    public void setCurrentItem(int new_current_item) // used so that the builder refers to a particular item
    {
        this.current_item = new_current_item;
    }

    public void updateData(ViewUnbinned3DData new_raw_data, float shift) {
        this.norm_data = new ViewNormalizedUnbinned3DData(new_raw_data, shift);
    }

    Shape3D createShape() {
        Shape3D surface = new Shape3D();
        //timeStamp.print("geometry set");
        surface.setAppearance(createMaterialAppearance());
        return surface;
    }

    private Appearance createMaterialAppearance() {
        Appearance materialAppear = new Appearance();
        PolygonAttributes polyAttrib = new PolygonAttributes();
        polyAttrib.setCullFace(PolygonAttributes.CULL_NONE);
        materialAppear.setPolygonAttributes(polyAttrib);

        Material material = new Material();
        // set diffuse color to red (this color will only be used
        //     if lighting disabled - per-vertex color overrides)
        material.setDiffuseColor(new Color3f(1.0f, 0.0f, 0.0f));
        materialAppear.setMaterial(material);

        return materialAppear;
    }

    private Geometry buildGeometry(Point3f point_coord, boolean active) {
        point_coord.x = point_coord.x - 0.5f;
        point_coord.y = point_coord.y - 0.5f;
        //timeStamp.print("Starting CrossBuilder.buildContent()");
        QuadArray quad = new QuadArray(72, QuadArray.COORDINATES | QuadArray.COLOR_3 | QuadArray.NORMALS);
        Point3f[] pf;
        pf = this.setitem_coords(point_coord);
        quad.setCoordinates(0, pf);
        this.setItemColor(quad, active);
        GeometryInfo geom = new GeometryInfo(GeometryInfo.QUAD_ARRAY);
        geom.setCoordinates(pf);
        geom.setColors(cb_tmp);
        NormalGenerator ng = new NormalGenerator();
        ng.generateNormals(geom);
        //timeStamp.print("finished");
        return geom.getGeometryArray();
    }

    private Point3f[] setitem_coords(Point3f point_coord) {
        //c1 and c2 determines the item's thickness ; change the two float values to modify the item's shape
        c1 = (0.003f * scale_factor);
        c2 = (0.0175f * scale_factor);
        Point3f[] pf = new Point3f[72];
        pf[0] = new Point3f((point_coord.x - c1), (point_coord.y - c1), (point_coord.z - c2));
        pf[1] = new Point3f((point_coord.x + c1), (point_coord.y - c1), (point_coord.z - c2));
        pf[2] = new Point3f((point_coord.x + c1), (point_coord.y - c1), (point_coord.z + c2));
        pf[3] = new Point3f((point_coord.x - c1), (point_coord.y - c1), (point_coord.z + c2));
        pf[4] = new Point3f((point_coord.x - c1), (point_coord.y + c1), (point_coord.z - c2));
        pf[5] = new Point3f((point_coord.x + c1), (point_coord.y + c1), (point_coord.z - c2));
        pf[6] = new Point3f((point_coord.x + c1), (point_coord.y + c1), (point_coord.z + c2));
        pf[7] = new Point3f((point_coord.x - c1), (point_coord.y + c1), (point_coord.z + c2));
        pf[8] = new Point3f((point_coord.x - c1), (point_coord.y - c1), (point_coord.z - c2));
        pf[9] = new Point3f((point_coord.x - c1), (point_coord.y + c1), (point_coord.z - c2));
        pf[10] = new Point3f((point_coord.x - c1), (point_coord.y + c1), (point_coord.z + c2));
        pf[11] = new Point3f((point_coord.x - c1), (point_coord.y - c1), (point_coord.z + c2));
        pf[12] = new Point3f((point_coord.x + c1), (point_coord.y - c1), (point_coord.z - c2));
        pf[13] = new Point3f((point_coord.x + c1), (point_coord.y + c1), (point_coord.z - c2));
        pf[14] = new Point3f((point_coord.x + c1), (point_coord.y + c1), (point_coord.z + c2));
        pf[15] = new Point3f((point_coord.x + c1), (point_coord.y - c1), (point_coord.z + c2));
        pf[16] = new Point3f((point_coord.x - c1), (point_coord.y - c1), (point_coord.z - c2));
        pf[17] = new Point3f((point_coord.x + c1), (point_coord.y - c1), (point_coord.z - c2));
        pf[18] = new Point3f((point_coord.x + c1), (point_coord.y + c1), (point_coord.z - c2));
        pf[19] = new Point3f((point_coord.x - c1), (point_coord.y + c1), (point_coord.z - c2));
        pf[20] = new Point3f((point_coord.x - c1), (point_coord.y - c1), (point_coord.z + c2));
        pf[21] = new Point3f((point_coord.x + c1), (point_coord.y - c1), (point_coord.z + c2));
        pf[22] = new Point3f((point_coord.x + c1), (point_coord.y + c1), (point_coord.z + c2));
        pf[23] = new Point3f((point_coord.x - c1), (point_coord.y + c1), (point_coord.z + c2));
        pf[24] = new Point3f((point_coord.x - c1), (point_coord.y - c2), (point_coord.z - c1));
        pf[25] = new Point3f((point_coord.x + c1), (point_coord.y - c2), (point_coord.z - c1));
        pf[26] = new Point3f((point_coord.x + c1), (point_coord.y - c2), (point_coord.z + c1));
        pf[27] = new Point3f((point_coord.x - c1), (point_coord.y - c2), (point_coord.z + c1));
        pf[28] = new Point3f((point_coord.x - c1), (point_coord.y + c2), (point_coord.z - c1));
        pf[29] = new Point3f((point_coord.x + c1), (point_coord.y + c2), (point_coord.z - c1));
        pf[30] = new Point3f((point_coord.x + c1), (point_coord.y + c2), (point_coord.z + c1));
        pf[31] = new Point3f((point_coord.x - c1), (point_coord.y + c2), (point_coord.z + c1));
        pf[32] = new Point3f((point_coord.x - c1), (point_coord.y - c2), (point_coord.z - c1));
        pf[33] = new Point3f((point_coord.x + c1), (point_coord.y - c2), (point_coord.z - c1));
        pf[34] = new Point3f((point_coord.x + c1), (point_coord.y + c2), (point_coord.z - c1));
        pf[35] = new Point3f((point_coord.x - c1), (point_coord.y + c2), (point_coord.z - c1));
        pf[36] = new Point3f((point_coord.x - c1), (point_coord.y - c2), (point_coord.z + c1));
        pf[37] = new Point3f((point_coord.x + c1), (point_coord.y - c2), (point_coord.z + c1));
        pf[38] = new Point3f((point_coord.x + c1), (point_coord.y + c2), (point_coord.z + c1));
        pf[39] = new Point3f((point_coord.x - c1), (point_coord.y + c2), (point_coord.z + c1));
        pf[40] = new Point3f((point_coord.x - c1), (point_coord.y - c2), (point_coord.z - c1));
        pf[41] = new Point3f((point_coord.x - c1), (point_coord.y + c2), (point_coord.z - c1));
        pf[42] = new Point3f((point_coord.x - c1), (point_coord.y + c2), (point_coord.z + c1));
        pf[43] = new Point3f((point_coord.x - c1), (point_coord.y - c2), (point_coord.z + c1));
        pf[44] = new Point3f((point_coord.x + c1), (point_coord.y - c2), (point_coord.z - c1));
        pf[45] = new Point3f((point_coord.x + c1), (point_coord.y + c2), (point_coord.z - c1));
        pf[46] = new Point3f((point_coord.x + c1), (point_coord.y + c2), (point_coord.z + c1));
        pf[47] = new Point3f((point_coord.x + c1), (point_coord.y - c2), (point_coord.z + c1));
        pf[48] = new Point3f((point_coord.x - c2), (point_coord.y - c1), (point_coord.z - c1));
        pf[49] = new Point3f((point_coord.x - c2), (point_coord.y + c1), (point_coord.z - c1));
        pf[50] = new Point3f((point_coord.x - c2), (point_coord.y + c1), (point_coord.z + c1));
        pf[51] = new Point3f((point_coord.x - c2), (point_coord.y - c1), (point_coord.z + c1));
        pf[52] = new Point3f((point_coord.x + c2), (point_coord.y - c1), (point_coord.z - c1));
        pf[53] = new Point3f((point_coord.x + c2), (point_coord.y + c1), (point_coord.z - c1));
        pf[54] = new Point3f((point_coord.x + c2), (point_coord.y + c1), (point_coord.z + c1));
        pf[55] = new Point3f((point_coord.x + c2), (point_coord.y - c1), (point_coord.z + c1));
        pf[56] = new Point3f((point_coord.x - c2), (point_coord.y - c1), (point_coord.z - c1));
        pf[57] = new Point3f((point_coord.x + c2), (point_coord.y - c1), (point_coord.z - c1));
        pf[58] = new Point3f((point_coord.x + c2), (point_coord.y - c1), (point_coord.z + c1));
        pf[59] = new Point3f((point_coord.x - c2), (point_coord.y - c1), (point_coord.z + c1));
        pf[60] = new Point3f((point_coord.x - c2), (point_coord.y + c1), (point_coord.z - c1));
        pf[61] = new Point3f((point_coord.x + c2), (point_coord.y + c1), (point_coord.z - c1));
        pf[62] = new Point3f((point_coord.x + c2), (point_coord.y + c1), (point_coord.z + c1));
        pf[63] = new Point3f((point_coord.x - c2), (point_coord.y + c1), (point_coord.z + c1));
        pf[64] = new Point3f((point_coord.x - c2), (point_coord.y - c1), (point_coord.z - c1));
        pf[65] = new Point3f((point_coord.x + c2), (point_coord.y - c1), (point_coord.z - c1));
        pf[66] = new Point3f((point_coord.x + c2), (point_coord.y + c1), (point_coord.z - c1));
        pf[67] = new Point3f((point_coord.x - c2), (point_coord.y + c1), (point_coord.z - c1));
        pf[68] = new Point3f((point_coord.x - c2), (point_coord.y - c1), (point_coord.z + c1));
        pf[69] = new Point3f((point_coord.x + c2), (point_coord.y - c1), (point_coord.z + c1));
        pf[70] = new Point3f((point_coord.x + c2), (point_coord.y + c1), (point_coord.z + c1));
        pf[71] = new Point3f((point_coord.x - c2), (point_coord.y + c1), (point_coord.z + c1));

        return pf;
    }

    public void updateItemPos(Shape3D item, int pos, boolean active) {
        Point3f point_coord = this.norm_data.pointAt(pos);
        item.setGeometry(this.buildGeometry(point_coord, active));
    }

    public void setItemColor(Geometry geom_array, boolean setActive) {
        QuadArray geom = (QuadArray) geom_array;
        cb_tmp = new Color3b[geom.getVertexCount()];
        Point3f p = new Point3f(); //to retrieve the item's z coordinate
        geom.getCoordinate(0, p);
        if (setActive) {
            for (int i = 0; i < geom.getVertexCount(); i++)
                cb_tmp[i] = norm_data.colorAt(current_item);
        } else {
            byte b = (byte) 127; // 127 ;
            for (int i = 0; i < geom.getVertexCount(); i++)
                cb_tmp[i] = new Color3b(b, b, b);
        }
        geom.setColors(0, cb_tmp);

    }
} //end of View3DItemBuilder class

