//***********************************************************************
//
//                              View3DScatterPlot.java
//
// Author(s) : Mathieu GRELIER
// FISPRO Version : 3.0 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Contact : fispro@ensam.inra.fr
// Last modification date:  December 25, 2005
//
//**********************************************************************
package org.freehep.j3d.plot;

import java.awt.Color;
import java.util.Enumeration;

import javax.media.j3d.BranchGroup;
import javax.media.j3d.Group;
import javax.media.j3d.Node;
import javax.media.j3d.Shape3D;
import javax.vecmath.Point3f;

import com.sun.j3d.utils.picking.PickResult;
import com.sun.j3d.utils.picking.PickTool;

public class View3DScatterPlot extends ExtendedPlot3D {

    private ViewUnbinned3DData data;

    private View3DItemBuilder builder;

    public Node[] plot;

    private ExtendedXAxisBuilder xAxis = null;

    private ExtendedYAxisBuilder yAxis = null;

    private ExtendedZAxisBuilder zAxis = null;

    private String[] Axislabels = { "x", "y", "z" };

    private double xmin;

    private double xmax;

    private double ymin;

    private double ymax;

    private double zmin;

    private double zmax;

    private float shift = 0.1f; // axes enlargment factor, set here

    float xshift; // x axe enlargment factor

    float yshift; // y axe enlargment factor

    float zshift; // z axe enlargment factor

    public Group items;

    public Group g;

    public View3DScatterPlot() //standard 3Dplot
    {
        super();
    }

    public View3DScatterPlot(String[] Axislabels) //standard 3Dplot
    {
        super();
        this.Axislabels = Axislabels;
    }

    public void setAxes(String[] Axislabels) {
        this.Axislabels = Axislabels;
    }

    public void setData(ViewUnbinned3DData data) {
        this.data = data;
        //shift to avoid having min and max points exactly on the planes (axes are max-min*shift% larger)
        xshift = ((data.xMax() - data.xMin()) * shift);
        yshift = ((data.yMax() - data.yMin()) * shift);
        zshift = ((data.zMax() - data.zMin()) * shift);

        if (init) {
            if (data.xMin() != xmin || data.xMax() != xmax) {
                xmin = data.xMin();
                xmax = data.xMax();
                xAxis.createLabelsNTicks((data.xMin() - xshift), (data.xMax() + xshift));
                xAxis.apply();
            }
            if (data.yMin() != ymin || data.yMax() != ymax) {
                ymin = data.yMin();
                ymax = data.yMax();
                yAxis.createLabelsNTicks((data.yMin() - yshift), (data.yMax() + yshift));
                yAxis.apply();
            }
            if (data.zMin() != zmin || data.zMax() != zmax) {
                zmin = data.zMin();
                zmax = data.zMax();
                zAxis.createLabelsNTicks((data.zMin() - zshift), (data.zMax() + zshift));
                zAxis.apply();
            }

            if (xAxis != null && yAxis != null && zAxis != null) {
                setXAxisLabel(this.Axislabels[0]);
                setYAxisLabel(this.Axislabels[1]);
                setZAxisLabel(this.Axislabels[2]);
            }

            builder.updateData(this.data, this.shift);
            Node item_i;
            for (int i = 0; i < plot.length; i++) {
                item_i = plot[i];
                builder.updateItemPos(((Shape3D) item_i), i, data.getActiveAt(i));
            }
        }
    }

    public String getXAxisLabel() {
        return Axislabels[0];
    }

    public void setXAxisLabel(String s) {
        xAxis.setLabel(s);
        xAxis.apply();
    }

    public String getYAxisLabel() {
        return Axislabels[1];
    }

    public void setYAxisLabel(String s) {
        yAxis.setLabel(s);
        yAxis.apply();
    }

    public String getZAxisLabel() {
        return Axislabels[2];
    }

    public void setZAxisLabel(String s) {
        zAxis.setLabel(s);
        zAxis.apply();
    }

    protected Node createPlot() {
        try {
            builder = new View3DItemBuilder(this.data, this.shift);
        } catch (Throwable t) {
            System.out.println(t.getMessage());
        }
        Node box = builder.buildOutsideBox();
        box.setPickable(false);
        plot = new Node[data.getNPoints()];
        for (int item_i = 0; item_i < plot.length; item_i++) {
            plot[item_i] = builder.buildContent(item_i, data.getActiveAt(item_i));
            plot[item_i].setUserData(new Integer(item_i)); //user data item number starts at 0
            plot[item_i].setCapability(BranchGroup.ALLOW_DETACH);
            plot[item_i].setPickable(true);
            plot[item_i].setCapability(Node.ENABLE_PICK_REPORTING);
            plot[item_i].setCapability(Shape3D.ALLOW_GEOMETRY_READ);
        }

        xAxis = new ExtendedXAxisBuilder(); //the non-pickability of the axis is handled
        yAxis = new ExtendedYAxisBuilder(); //after the creation of the ticks (see apply)
        zAxis = new ExtendedZAxisBuilder();

        //shift to avoid having min and max points exactly on the planes (axes are max-min*shift% larger)
        xAxis.createLabelsNTicks((data.xMin() - xshift), (data.xMax() + xshift));
        yAxis.createLabelsNTicks((data.yMin() - yshift), (data.yMax() + yshift));
        zAxis.createLabelsNTicks((data.zMin() - zshift), (data.zMax() + zshift));

        xAxis.setLabel(this.Axislabels[0]);
        yAxis.setLabel(this.Axislabels[1]);
        zAxis.setLabel(this.Axislabels[2]);

        //after all the ticks are set, we retrieve the axis maingroup and its Shape3D nodes to set them to non pickable nodes
        //it is an approach accomodating with the AxisBuilder class' original design
        xAxis.apply();
        xAxis.setAxisNonPickable();
        yAxis.apply();
        yAxis.setAxisNonPickable();
        zAxis.apply();
        zAxis.setAxisNonPickable();

        g = new Group();
        g.addChild(applyShape3DColor((Shape3D)box, Color.black));
        this.items = new Group();
        for (int i = 0; i < plot.length; i++) {
            items.addChild(plot[i]);
        }
        g.addChild(items); // this position is important in the method setPickingCapability (child # = 1)
        g.addChild(xAxis.getNode());
        g.addChild(yAxis.getNode());
        g.addChild(zAxis.getNode());
        return g;
    }

    public void setPickCapability() {
        //we have to retrieve the items group and set the pick capability for each one (different from the pickable property ; here we set the pick's detail level)
        //the test is based on the fact that items is a Group node with 'item number' children,
        //assuming that there won't be other nodes with this characteristic
        for (Enumeration e = this.g.getAllChildren(); e.hasMoreElements();) { // Branch group child # for items must correspond to the adding order in createPlot method
            Object el = e.nextElement();
            if (el instanceof Group && ((Group) el).numChildren() == this.data.getNPoints()) //items identification
            {
                for (Enumeration e1 = ((Group) el).getAllChildren(); e1.hasMoreElements();) {
                    PickTool.setCapabilities(((Shape3D) (e1.nextElement())), PickTool.INTERSECT_FULL); // set the pick capability for the items to allow picking at the specified detail level : here wa want all the info to be retrieved
                }
            }
        }

    }

    public void processIntersection(PickResult result) {
    //can be defined somewhere else, ex. in an anonymous class
    }

    public void processNoIntersection() {
    //can be defined somewhere else, ex. in an anonymous class
    }

    public void processLeftClick(PickResult result) {
    //can be defined somewhere else, ex. in an anonymous class
    }

    public void processRightClick(PickResult result) {
    //can be defined somewhere else, ex. in an anonymous class
    }

    public void processCenterClick(PickResult result) {
    //can be defined somewhere else, ex. in an anonymous class
    }

    public void processLeftDoubleClick(PickResult result) {
    //can be defined somewhere else, ex. in an anonymous class
    }

    public Point3f getItemData(int item_number) //read only acces to one item data
    {
        Point3f p = new Point3f();
        p.x = this.data.pointAt(item_number).x;
        p.y = this.data.pointAt(item_number).y;
        p.z = this.data.pointAt(item_number).z;
        return p;
    }

    public void setItemActive(int it, boolean setActive) {
        Node item_i;
        item_i = plot[it];
        builder.setCurrentItem(it);
        builder.setItemColor(((Shape3D) item_i).getGeometry(), setActive);
    }

}
