//***********************************************************************
//
//                              ViewNormalizedUnbinned3DData.java
//
// Author(s) : Mathieu GRELIER
// FISPRO Version : 3.0 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Contact : fispro@ensam.inra.fr
// Last modification date:  December 25, 2005
//
//**********************************************************************
package org.freehep.j3d.plot;

import javax.vecmath.Color3b;
import javax.vecmath.Point3f;

public class ViewNormalizedUnbinned3DData implements Unbinned3DData {
    private Rainbow rainbow;

    private double[] xdata;

    private double[] ydata;

    private double[] zdata;

    private float z_min, z_max;

    private float ori_xmin, ori_xmax, ori_ymin, ori_ymax, ori_zmin, ori_zmax;

    private float norm_x, norm_y, norm_z;

    private int npoints;

    public ViewNormalizedUnbinned3DData(ViewUnbinned3DData data, float shift) {

        this.xdata = data.getxdata();
        this.ydata = data.getydata();
        this.zdata = data.getzdata();
        this.z_min = data.zMin();
        this.z_max = data.zMax();

        //this.shift = shift;
        float xshift = ((data.xMax() - data.xMin()) * shift);
        float yshift = ((data.yMax() - data.yMin()) * shift);
        float zshift = ((data.zMax() - data.zMin()) * shift);
        //shift to avoid having min and max points exactly on the planes (axes are max-min*shift% larger)
        this.ori_xmin = (data.xMin() - xshift);
        this.ori_xmax = (data.xMax() + xshift);
        this.ori_ymin = (data.yMin() - yshift);
        this.ori_ymax = (data.yMax() + yshift);
        this.ori_zmin = (data.zMin() - zshift);
        this.ori_zmax = (data.zMax() + zshift);
        this.npoints = data.getNPoints();
        rainbow = new Rainbow(0, 1); //normalized colors
    }

    public float xMin() {
        return 0f;
    }

    public float xMax() {
        return 1f;
    }

    public float yMin() {
        return 0f;
    }

    public float yMax() {
        return 1f;
    }

    public float zMin() {
        return 0f;

    }

    public float zMax() {
        return 1f;
    }

    /**
     * The number of points in the scatter plot
     */
    public int getNPoints() {
        return this.npoints;
    }

    /**
     * The x,y,z normalized coordinate of the specified point
     */
    public Point3f pointAt(int index) {
        norm_x = ((float) xdata[index] - this.ori_xmin) / (this.ori_xmax - this.ori_xmin);
        norm_y = (this.ori_ymax - (float) ydata[index]) / (this.ori_ymax - this.ori_ymin); // invert y axis
        norm_z = ((float) zdata[index] - this.ori_zmin) / (this.ori_zmax - this.ori_zmin);
        Point3f point = new Point3f(norm_x, norm_y, norm_z);
        return point;
    }

    /**
     * Get the Color of the specified point ; z data min and max are used as references
     */
    public Color3b colorAt(int index) {
        return rainbow.colorFor((zdata[index] - this.z_min) / (this.z_max - this.z_min));
    }

    public Point3f denormalizePoint(Point3f p) {
        p.x = (p.x * (this.ori_xmax - this.ori_xmin)) + this.ori_xmin;
        p.y =  this.ori_ymax - (p.y * (this.ori_ymax - this.ori_ymin));  // invert y axis
        p.z = (p.z * (this.ori_zmax - this.ori_zmin)) + this.ori_zmin;
        return p;
    }
}
