//***********************************************************************
//
//                              ViewUnbinned3DData.java
//
// Author(s) : Mathieu GRELIER
// FISPRO Version : 3.0 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Contact : fispro@ensam.inra.fr
// Last modification date:  December 25, 2005
//
//**********************************************************************
package org.freehep.j3d.plot;

import javax.vecmath.Color3b;
import javax.vecmath.Point3f;

public class ViewUnbinned3DData implements Unbinned3DData {
    private Rainbow rainbow;

    private double[] xdata;

    private double[] ydata;

    private double[] zdata;

    private boolean[] active_array = null;

    private float xmin, xmax, ymin, ymax, zmin, zmax;

    public ViewUnbinned3DData(double[] xdata_r, double[] ydata_r, double[] zdata_r) {
        init(xdata_r, ydata_r, zdata_r);
    }

    public ViewUnbinned3DData(double[] xdata_r, double[] ydata_r, double[] zdata_r, boolean[] active_array) {
        if (!(active_array.length > xdata_r.length && active_array.length > ydata_r.length && active_array.length > zdata_r.length)) {
            this.active_array = active_array;
        }
        init(xdata_r, ydata_r, zdata_r);

    }

    private void init(double[] xdata_r, double[] ydata_r, double[] zdata_r) {
        int max_length;
        max_length = (xdata_r.length >= ydata_r.length) ? xdata_r.length : ydata_r.length;
        max_length = (zdata_r.length >= max_length) ? zdata_r.length : max_length;
        this.xdata = new double[max_length];
        this.ydata = new double[max_length];
        this.zdata = new double[max_length];
        if (active_array == null) {
            active_array = new boolean[max_length];
            for (int i = 0; i < max_length; i++)
                active_array[i] = true;
        }

        for (int i = 0; i < max_length; i++) {
            this.xdata[i] = 0;
            this.ydata[i] = 0;
            this.zdata[i] = 0;
        }

        for (int i = 0; i < xdata_r.length; i++)
            this.xdata[i] = xdata_r[i];
        for (int i = 0; i < ydata_r.length; i++)
            this.ydata[i] = ydata_r[i];
        for (int i = 0; i < zdata_r.length; i++)
            this.zdata[i] = zdata_r[i];

        CalculateMinsAndMaxs();

    }

    private void CalculateMinsAndMaxs() {
        xmin = (float) this.xdata[0];
        ;
        for (int i = 1; i < this.xdata.length; i++) {
            if (xdata[i] < xmin)
                xmin = (float) xdata[i];
        }
        xmax = (float) this.xdata[0];
        ;
        for (int i = 1; i < this.xdata.length; i++) {
            if (xdata[i] > xmax)
                xmax = (float) xdata[i];
        }
        ymin = (float) this.ydata[0];
        ;
        for (int i = 1; i < this.ydata.length; i++) {
            if (ydata[i] < ymin)
                ymin = (float) ydata[i];
        }

        ymax = (float) this.ydata[0];
        ;
        for (int i = 1; i < this.ydata.length; i++) {
            if (ydata[i] > ymax)
                ymax = (float) ydata[i];
        }
        zmin = (float) this.zdata[0];
        ;
        for (int i = 1; i < this.zdata.length; i++) {
            if (zdata[i] < zmin)
                zmin = (float) zdata[i];
        }

        zmax = (float) this.zdata[0];
        ;
        for (int i = 1; i < this.zdata.length; i++) {
            if (zdata[i] > zmax)
                zmax = (float) zdata[i];
        }

        this.rainbow = new Rainbow(zmin, zmax);

    }

    /**
     * Axis minimum on the X Axis
     */
    public float xMin() {
        return xmin;
    }

    /**
     * Axis maximum on the X Axis
     */
    public float xMax() {
        return xmax;
    }

    /**
     * Axis minimum on the Y Axis
     */
    public float yMin() {
        return ymin;
    }

    /**
     * Axis maximum on  the Y Axis
     */
    public float yMax() {
        return ymax;
    }

    /**
     * Minimum data value on the Z Axis
     */
    public float zMin() {
        return zmin;

    }

    /**
     * Maximum data value on the Z Axis
     */
    public float zMax() {
        return zmax;
    }

    /**
     * The number of points in the scatter plot
     */
    public int getNPoints() {
        return (xdata.length > ydata.length) ? ((ydata.length > zdata.length) ? zdata.length : ydata.length) : ((xdata.length > zdata.length) ? zdata.length : xdata.length);
    }

    public boolean getActiveAt(int index) {
        if (this.active_array != null && index >= 0 && index < active_array.length)
            return this.active_array[index];
        else
            return true;
    }

    /**
     * The x,y,z coordinate of the specified point
     */
    public Point3f pointAt(int index) {
        Point3f point = new Point3f((float) xdata[index], (float) ydata[index], (float) zdata[index]);
        return point;
    }

    /**
     * Get the Color of the specified point in the non-normalized range ; use the ViewNormalizedUnbinned3DData's rainbow rather
     */
    public Color3b colorAt(int index) {
        return rainbow.colorFor(zdata[index]);
    }

    public double[] getxdata() {
        return xdata;
    }

    public double[] getydata() {
        return ydata;
    }

    public double[] getzdata() {
        return zdata;
    }
}
