package util;

import javax.swing.event.*;
import javax.swing.*;

/**
 * Synchronize the data models of any two JComponents that use a
 *  BoundedRangeModel ( such as a JScrollBar, JSlider or ProgressBar).
 *
 * @see javax.swing.BoundedRangeModel
 * @see javax.swing.event.ChangeListener
 * @Author  R. Kevin Cole
 */
public class BoundedRangeModelChangeListener implements  ChangeListener
{
 private BoundedRangeModel myModel;

 /**
  * @param   model  This model is forced to move in synchronization
  *  to this ChangeListener's event-source.
  */
 public BoundedRangeModelChangeListener( BoundedRangeModel model )
 {
  myModel = model;
 }


// - begin - implementation of ChangeListener
//
 /**
  *  Envoked when the target of the listener has changed its state.
  */
 public void stateChanged( ChangeEvent e )
 {
  BoundedRangeModel sourceModel = (BoundedRangeModel) e.getSource();

  int sourceDiff  = sourceModel.getMaximum() - sourceModel.getMinimum();
  int destDiff  = myModel.getMaximum() - myModel.getMinimum();
  int destValue = sourceModel.getValue();

  if( (sourceDiff != destDiff) && (sourceDiff != 0) )
   destValue   = (destDiff * sourceModel.getValue())/sourceDiff;

  myModel.setValue( myModel.getMinimum() + destValue );
 }
//
// - end   - implementation of ChangeListener

}


