//***********************************************************************
//
//                              JDoubleMThumbSliderSpinner.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.0 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Contact : fispro@ensam.inra.fr
// Last modification date:  December 25, 2005
//
//**********************************************************************
package util;

import javax.swing.JComponent;
import javax.swing.JSpinner;
import javax.swing.SpinnerListModel;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


/**
 * Classe slider  plusieurs curseurs avec valeurs en double plus spinners synchroniss<p>
 * Possibilit de changer le modle du spinner (seulement avec un objet contenant 1 curseur) en SpinnerListModel avec la fonction setSpinnerListModel(), retour au modele par defaut avec la fonction setDefaultSpinnerModel()
 * 
 * @author jean-luc.lablee
 */
public class JDoubleMThumbSliderSpinner extends JDoubleMThumbSlider {
    protected JSpinner[] spinners = new JSpinner[this.thumbNum]; // objets spinners

    protected boolean spinnerValueIsAdjusting = false; // spinner en ajustement

    /**
     *  Construit un slider  2 curseurs et 2 spinners
     */
    public JDoubleMThumbSliderSpinner() {
        super();
        this.init();
    }

    /**
     * Construit un slider  plusieurs curseurs et spinners
     * @param thumbNum nombre de curseurs et spinners
     */
    public JDoubleMThumbSliderSpinner(int thumbNum) {
        super(thumbNum);
        this.init();
    }

    /**
     * Fonction d'init de la classe
     */
    private void init() {
        for (int i = 0; i < this.thumbNum; i++) {
            final int fi = i;
            // creation des editeurs de spinner pour fixer le pattern du DecimalFormat du JSpinner.NumberEditor
            spinners[i] = new JSpinner() {
                protected JComponent createEditor(SpinnerModel model) {
                    if (model instanceof SpinnerListModel)
                        return super.createEditor(model);
                    else
                        return new JSpinner.NumberEditor(this, JDoubleRound.getDecimalFormatPattern());
                }
            };
            // listeners pour ajuster les curseurs sur les spinners
            spinners[i].addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    // ajuste le minimum et maximum des spinners voisins
                    if (thumbCrossing == false) {
                        if (fi != thumbNum - 1)
                            ((SpinnerNumberModel) spinners[fi + 1].getModel()).setMinimum(new Double(((Double) spinners[fi].getValue()).doubleValue() + ((Double) ((SpinnerNumberModel) spinners[fi].getModel()).getStepSize()).doubleValue()));
                        if (fi != 0)
                            ((SpinnerNumberModel) spinners[fi - 1].getModel()).setMaximum(new Double(((Double) spinners[fi].getValue()).doubleValue() - ((Double) ((SpinnerNumberModel) spinners[fi].getModel()).getStepSize()).doubleValue()));
                    }
                    // ajuste le slider min
                    spinnerValueIsAdjusting = true;
                    if (((Double) spinners[fi].getValue()).doubleValue() != getDoubleValue(fi)) {
                        JDoubleMThumbSliderSpinner.super.setDoubleValue(fi, ((Double) spinners[fi].getValue()).doubleValue());
                        sliderStateChanged(fi);
                    }
                }
            });
        }
        // valeurs par defaut
        this.setDoubleMinMax(0, 1);
    }

    /* (non-Javadoc)
     * @see util.JDoubleMThumbSlider#sliderStateChanged(int)
     */
    protected void sliderStateChanged(int thumbIndex) {
        if (spinners[thumbIndex].getModel() instanceof SpinnerListModel) {
            Double oldValue = (Double) spinners[thumbIndex].getValue();
            double newValue = this.modelValueToDoubleValue(this.getValueAt(thumbIndex));
            spinners[thumbIndex].setValue(new Double(newValue));
            if ((!this.getValueIsAdjusting() && this.getSpinnerValueIsAdjusting()) || (oldValue != spinners[thumbIndex].getValue()))
                this.fireLocalChangeEvent();
        } else {
            super.sliderStateChanged(thumbIndex);
            spinners[thumbIndex].setValue(new Double(getDoubleValue(thumbIndex)));
            // getValueIsAdjusting est gr par util.JDoubleMThumbSlider#sliderStateChanged(int)
            if (!this.getValueIsAdjusting() && this.getSpinnerValueIsAdjusting())
                this.fireLocalChangeEvent();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see util.JDoubleMThumbSlider#setDoubleMinMax(double, double)
     */
    public void setDoubleMinMax(double min, double max) {
        super.setDoubleMinMax(min, max);
        for (int i = 0; i < this.thumbNum; i++) {
            spinners[i].setModel(new JSpinnerDoubleModel(this.thumbNum == 1 ? this.doubleMin + ((this.doubleMax - this.doubleMin) / 2) : this.doubleMin + (i * ((this.doubleMax - this.doubleMin) / (this.thumbNum - 1))), this.doubleMin, this.doubleMax, (this.doubleMax - this.doubleMin) / this.resolution, true));
            this.setDoubleValue(i, ((Double) spinners[i].getValue()).doubleValue());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see util.JDoubleMThumbSlider#setDoubleValue(int, double)
     */
    public void setDoubleValue(int index, double value) {
        super.setDoubleValue(index, value);
        spinners[index].setValue(new Double(value));
    }

    /**
     * Renvoie le tableau des spinners
     * @return tableau des spinners
     */
    public JSpinner[] getSpinners() {
        return spinners;

    }

    /**
     * Spinner en cours d'ajustement
     * @return true si le spinner a t ajust
     */
    private boolean getSpinnerValueIsAdjusting() {
        boolean returnValue = this.spinnerValueIsAdjusting;
        this.spinnerValueIsAdjusting = false;
        return returnValue;
    }

    /**
     * Modifie le modle de slction du spinner en SpinnerListModel<p>
     * Cette fonction est utilisable uniquement avec un objet  1 curseur
     * @param values valeurs du modle SpinnerListModel
     */
    public void setSpinnerListModel(double[] values) {
        if (this.thumbNum != 1)
            return;
        Double[] modelValues = new Double[values.length];
        for (int i = 0; i < modelValues.length; i++)
            modelValues[i] = new Double(JDoubleRound.round(values[i]));
        this.spinners[0].setModel(new SpinnerListModel(modelValues) {
            public void setValue(Object elt) {
                if (elt instanceof String) // appui sur les boutons du JSpinner (genere un setValue avec une String)
                    super.setValue(elt);
                else { // setValue appel depuis sliderStateChanged (avec un Double)
                    // recherche la valeur la plus proche de elt
                    Double[] values = (Double[]) this.getList().toArray();
                    int nearestIndex = 0;
                    double minDistance = Double.MAX_VALUE;
                    for (int i = 0; i < values.length; i++) {
                        double distance = Math.abs(((Double) elt).doubleValue() - values[i].doubleValue());
                        if (distance < minDistance) {
                            minDistance = distance;
                            nearestIndex = i;
                        }
                    }
                    super.setValue(values[nearestIndex]); // setValue avec la valeur la plus proche de elt
                }
            }
        });
        ((JSpinner.DefaultEditor) this.spinners[0].getEditor()).getTextField().setEditable(false);
        super.setDoubleValue(0, ((Double) spinners[0].getValue()).doubleValue());
    }

    /**
     * Rinitialise le modle du spinner en JSpinnerDoubleModel (modle par dfaut  la construction de l'objet)<p>
     * Cette fonction est utilisable uniquement avec un objet  1 curseur
     */
    public void setDefaultSpinnerModel() {
        if (this.thumbNum != 1)
            return;
        double value = this.getDoubleValue(0);
        spinners[0].setModel(new JSpinnerDoubleModel(this.doubleMin + ((this.doubleMax - this.doubleMin) / 2), this.doubleMin, this.doubleMax, (this.doubleMax - this.doubleMin) / this.resolution, true));
        this.setDoubleValue(0, value);
    }
}
