//***********************************************************************
//
//                              JDoubleRound.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.0 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Contact : fispro@ensam.inra.fr
// Last modification date:  December 25, 2005
//
//**********************************************************************
package util;

import java.text.DecimalFormat;
import java.text.ParseException;

/**
 * Classe de gestion des arrondis pour les doubles
 * @author jean-luc.lablee
 */
public class JDoubleRound {

    private static String decimalFormatPattern = new String("#.#########"); // pattern d'arrondi a 10e-9

    private static DecimalFormat df = new DecimalFormat(decimalFormatPattern);

    /**
     * Arrondi
     * @param value valeur  arrondir
     * @return valeur arrondi
     */
    public static double round(double value) {
        try {
            return df.parse(df.format(value)).doubleValue();
        } catch (ParseException e) {
            return value;
        }
    }

    /**
     * Arrondi avec un nombre de dcimales
     * @param value valeur  arrondir
     * @param decimalDigitsNumber nombre de decimales
     * @return
     */
    public static double round(double value, int decimalDigitsNumber) {
        try {
            String pattern = new String("#.");
            for (int i = 0; i < decimalDigitsNumber; i++)
                pattern += "#";
            DecimalFormat df = new DecimalFormat(pattern);
            return df.parse(df.format(value)).doubleValue();
        } catch (ParseException e) {
            return value;
        }
    }
    /**
     * Renvoie le pattern d'arrondi
     * @return pattern d'arrondi
     */
    public static String getDecimalFormatPattern() {
        return decimalFormatPattern;
    }
}
