//***********************************************************************
//
//                              JSpinnerDoubleModel.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.0 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Contact : fispro@ensam.inra.fr
// Last modification date:  December 25, 2005
//
//**********************************************************************
package util;

import javax.swing.SpinnerNumberModel;

/**
 * Modle de spinner pour des valeurs de type double <p>
 * Il est ncessaire d'arrondir les valeurs dans la fonction setValue pour viter des venements multiple lors de l'appui sur les boutons du spinner. 
 * c'est un problme de comparaison entre 2 valeurs de type double, avec un arrondi ca fonctionne
 * @author jean-luc.lablee
 */
public class JSpinnerDoubleModel extends SpinnerNumberModel {

    private boolean minimumChanged = false; // changement du minimum du modle

    private boolean maximumChanged = false; // changement du maximum du modle

    // permet de bloquer l'apppel  fireStateChanged() lors du changement de minimum ou maximum du modle
    private boolean noFireStateChangedOnMinMaxModification = false;

    /**
     * Construit un objet modle de spinner double
     * @param value valeur du modle
     * @param minimum valeur minimum du modle
     * @param maximum valeur maximum du modle
     * @param stepSize pas du modle
     */
    public JSpinnerDoubleModel(double value, double minimum, double maximum, double stepSize) {
        super(JDoubleRound.round(value), JDoubleRound.round(minimum), JDoubleRound.round(maximum), JDoubleRound.round(stepSize));
    }

    /**
     * Construit un objet modle de spinner double avec possibilit de bloquer l'vnement changement d'tat lors des changement du minimum et du maximum avec setMinimum() et setMaximum()
     * @param value valeur du modle
     * @param minimum valeur minimum du modle
     * @param maximum valeur maximum du modle
     * @param noFireStateChangedOnMinMaxModification true pour bloquer l'vnement changement d'tat
     */
    public JSpinnerDoubleModel(double value, double minimum, double maximum, double stepSize, boolean noFireStateChangedOnMinMaxModification) {
        super(JDoubleRound.round(value), JDoubleRound.round(minimum), JDoubleRound.round(maximum), JDoubleRound.round(stepSize));
        this.noFireStateChangedOnMinMaxModification = noFireStateChangedOnMinMaxModification;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.SpinnerModel#setValue(java.lang.Object)
     */
    public void setValue(Object value) {
        super.setValue(new Double(JDoubleRound.round(((Double) value).doubleValue())));
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.SpinnerNumberModel#setMaximum(java.lang.Comparable)
     */
    public void setMaximum(Comparable maximum) {
        this.maximumChanged = true;
        super.setMaximum(maximum);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.SpinnerNumberModel#setMinimum(java.lang.Comparable)
     */
    public void setMinimum(Comparable minimum) {
        this.minimumChanged = true;
        super.setMinimum(minimum);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.AbstractSpinnerModel#fireStateChanged()
     */
    protected void fireStateChanged() {
        // bloque l'appel  fireStateChanged() si noFireStateChangedOnMinMaxModification true
        if (this.noFireStateChangedOnMinMaxModification == true) {
            if (this.minimumChanged == true) {
                this.minimumChanged = false;
                return;
            }
            if (this.maximumChanged == true) {
                this.maximumChanged = false;
                return;
            }
        }
        super.fireStateChanged();
    }

}
