package util.jfree.data.xy;

import java.awt.image.Raster;

import org.jfree.data.DomainOrder;
import org.jfree.data.general.DatasetChangeListener;
import org.jfree.data.general.DatasetGroup;
import org.jfree.data.xy.XYZDataset;

public class RasterDataset implements XYZDataset {

	private final Raster raster;

	public RasterDataset(Raster raster) {
		this.raster = raster;
	}

	@Override
	public DomainOrder getDomainOrder() {
		return DomainOrder.NONE;
	}

	@Override
	public int getItemCount(int series) {
		return raster.getWidth() * raster.getHeight();
	}

	@Override
	public Number getX(int series, int item) {
		return new Double(getXValue(series, item));
	}

	@Override
	public double getXValue(int series, int item) {
		return getItemToX(item);
	}

	private int getItemToX(int item) {
		return item - ((item / raster.getWidth()) * raster.getWidth());
	}

	@Override
	public Number getY(int series, int item) {
		return new Double(getYValue(series, item));
	}

	@Override
	public double getYValue(int series, int item) {
		return getItemToY(item);
	}

	private int getItemToY(int item) {
		return item / raster.getWidth();
	}

	@Override
	public int getSeriesCount() {
		return raster.getNumBands();
	}

	@Override
	public Comparable<?> getSeriesKey(int series) {
		return series;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public int indexOf(Comparable seriesKey) {
		return (Integer) seriesKey;
	}

	@Override
	public void addChangeListener(DatasetChangeListener listener) {}

	@Override
	public void removeChangeListener(DatasetChangeListener listener) {}

	@Override
	public DatasetGroup getGroup() {
		return null;
	}

	@Override
	public void setGroup(DatasetGroup group) {}

	@Override
	public Number getZ(int series, int item) {
		return new Double(getZValue(series, item));
	}

	@Override
	public double getZValue(int series, int item) {
		return raster.getSampleDouble(getItemToX(item), getItemToY(item), series);
	}
}
