package util.jgoodies.binding;

import static com.jgoodies.binding.beans.PropertyConnector.connectAndUpdate;
import static java.beans.Introspector.getBeanInfo;
import static java.util.Arrays.asList;
import static util.swing.dialog.JOptionPaneAction.createOkCancelActions;

import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;

import javax.swing.Action;

import util.bean.BeanCloneable;
import util.jgoodies.binding.value.TriggerableActionSupport;
import util.swing.ActionSupport;
import util.swing.DefaultOptionSupport;
import util.swing.OptionSupport;

import com.google.common.base.Predicate;
import com.google.common.collect.Iterables;
import com.jgoodies.binding.PresentationModel;

@SuppressWarnings("serial")
public class BufferedPresentationModel<B extends BeanCloneable<B>> extends PresentationModel<B> implements OptionSupport, ActionSupport {

	private final B bufferedBean;
	private final ActionSupport actionSupport;
	private final OptionSupport optionSupport;

	public BufferedPresentationModel(B bean) {
		super(bean);
		bufferedBean = bean.clone();
		actionSupport = new TriggerableActionSupport(getTriggerChannel(), createOkCancelActions());
		optionSupport = new DefaultOptionSupport(actionSupport);
		connectProperties(this, bufferedBean);
	}

	protected B getBufferedBean() {
		return bufferedBean;
	}

	public static <B> void connectProperties(PresentationModel<B> adapter, B bean) {
		try {
			connectProperties(adapter, bean, getBeanInfo(bean.getClass()).getPropertyDescriptors());
		} catch (IntrospectionException e) {}
	}

	private static <B> void connectProperties(PresentationModel<B> adapter, B bean, PropertyDescriptor[] propertyDescriptors) {
		connectProperties(adapter, bean, Iterables.filter(asList(propertyDescriptors), new Predicate<PropertyDescriptor>() {

			@Override
			public boolean apply(PropertyDescriptor propertyDescriptor) {
				return propertyDescriptor.getReadMethod() != null && propertyDescriptor.getWriteMethod() != null;
			}
		}));
	}

	private static <B> void connectProperties(PresentationModel<B> adapter, B bean, Iterable<PropertyDescriptor> propertyDescriptors) {
		for (PropertyDescriptor propertyDescriptor : propertyDescriptors)
			connectProperty(adapter, bean, propertyDescriptor.getName());
	}

	private static <B> void connectProperty(PresentationModel<B> adapter, B bean, String propertyName) {
		connectAndUpdate(adapter.getBufferedModel(propertyName), bean, propertyName);
	}

	@Override
	public Action[] getActions() {
		return actionSupport.getActions();
	}

	@Override
	public Object[] getOptions() {
		return optionSupport.getOptions();
	}

	@Override
	public Object getInitialOption() {
		return optionSupport.getInitialOption();
	}
}
