package util.jgoodies.binding.value;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import net.sf.jga.fn.BinaryFunctor;

import com.jgoodies.binding.value.AbstractValueModel;
import com.jgoodies.binding.value.ValueModel;

@SuppressWarnings("serial")
public class BinaryConverter<T1, T2, R> extends AbstractValueModel {

	private final BinaryFunctor<T1, T2, R> functor;
	private final ValueModel model1;
	private final ValueModel model2;
	private final Class<T1> binding1;
	private final Class<T2> binding2;
	private final ValueChangeListener valueChangeListener = new ValueChangeListener();
	private Object value;

	public BinaryConverter(BinaryFunctor<T1, T2, R> functor, ValueModel model1, ValueModel model2, Class<T1> binding1, Class<T2> binding2) {
		this.functor = functor;
		this.model1 = model1;
		this.model2 = model2;
		this.binding1 = binding1;
		this.binding2 = binding2;
		this.value = getValue(getValue1(), getValue2());
		addValueChangeListener(getModel1());
		addValueChangeListener(getModel2());
	}

	private BinaryFunctor<T1, T2, R> getFunctor() {
		return functor;
	}

	private ValueModel getModel1() {
		return model1;
	}

	private ValueModel getModel2() {
		return model2;
	}

	public Class<T1> getBinding1() {
		return binding1;
	}

	public Class<T2> getBinding2() {
		return binding2;
	}

	public T1 getValue1() {
		return getBinding1().cast(getModel1().getValue());
	}

	public T2 getValue2() {
		return getBinding2().cast(getModel2().getValue());
	}

	private ValueChangeListener getValueChangeListener() {
		return valueChangeListener;
	}

	private void addValueChangeListener(ValueModel valueModel) {
		valueModel.addValueChangeListener(getValueChangeListener());
	}

	@Override
	public Object getValue() {
		return value;
	}

	private Object getValue(T1 value1, T2 value2) {
		return getFunctor().fn(value1, value2);
	}

	@Override
	public void setValue(Object newValue) {
		throw new UnsupportedOperationException();
	}

	public void release() {
		removeValueChangeListener(getModel1());
		removeValueChangeListener(getModel2());
	}

	private void removeValueChangeListener(ValueModel valueModel) {
		valueModel.removeValueChangeListener(getValueChangeListener());
	}

	public static <T1, T2, R> BinaryConverter<T1, T2, R> binaryConverter(BinaryFunctor<T1, T2, R> functor, ValueModel model1, ValueModel model2, Class<T1> binding1, Class<T2> binding2) {
		return new BinaryConverter<T1, T2, R>(functor, model1, model2, binding1, binding2);
	}

	public static <T> BinaryConverter<T, T, T> binaryConverter(BinaryFunctor<T, T, T> functor, ValueModel model1, ValueModel model2, Class<T> binding) {
		return new BinaryConverter<T, T, T>(functor, model1, model2, binding, binding);
	}

	class ValueChangeListener implements PropertyChangeListener {

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			fireValueChange(/*getValue()*/null, value = getValue(getValue1(), getValue2()));
		}
	}
}
