package util.jgoodies.validation;

import static util.jgoodies.validation.ValidatableAction.createOkValidatableCancelActions;

import java.beans.PropertyChangeEvent;

import javax.swing.Action;

import util.jgoodies.common.bean.ObservableBean2ChangeSupport;
import util.swing.ActionSupport;
import util.swing.DefaultActionSupport;

import com.jgoodies.common.bean.ObservableBean2;
import com.jgoodies.validation.ValidationResultModel;
import com.jgoodies.validation.Validator;
import com.jgoodies.validation.util.DefaultValidationResultModel;

public final class DefaultBeanValidationSupport<T extends ObservableBean2> implements BeanValidationSupport, ActionSupport {

	private final ValidationResultModel validationResultModel = new DefaultValidationResultModel();
	private final T bean;
	private final Validator<T> validator;
	private final ActionSupport actionSupport;
	private final ObservableBean2ChangeSupport changeSupport;

	public DefaultBeanValidationSupport(T bean, Validator<T> validator, ActionSupport actionSupport) {
		this.bean = bean;
		this.validator = validator;
		this.actionSupport = new DefaultActionSupport(createValidationActions(actionSupport.getActions()));
		this.changeSupport = new ObservableBean2ChangeSupport(bean) {

			@Override
			public void beanChange(PropertyChangeEvent evt) {
				updateValidationResult();
			}
		};
		updateValidationResult();
	}

	public void release() {
		changeSupport.release();
	}

	private void updateValidationResult() {
		validationResultModel.setResult(validator.validate(bean));
	}

	@Override
	public ValidationResultModel getValidationResultModel() {
		return validationResultModel;
	}

	@Override
	public Action[] getActions() {
		return actionSupport.getActions();
	}

	private Action[] createValidationActions(Action[] actions) {
		return createOkValidatableCancelActions(validationResultModel, actions);
	}
}
