package util.jgoodies.validation;

import static com.jgoodies.validation.view.ValidationComponentUtils.updateComponentTreeSeverityBackground;

import java.awt.Container;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import com.jgoodies.validation.ValidationResult;
import com.jgoodies.validation.ValidationResultModel;

public final class ValidationResultAdapter {

	private final Container container;
	private final ValidationResultModel validationResultModel;
	private final ValidationListener validationListener = new ValidationListener();

	public ValidationResultAdapter(Container container, BeanValidationSupport beanValidationSupport) {
		this.container = container;
		this.validationResultModel = beanValidationSupport.getValidationResultModel();
		initEventHandling();
		updateValidationResult();
	}

	private void initEventHandling() {
		validationResultModel.addPropertyChangeListener(ValidationResultModel.PROPERTY_RESULT, validationListener);
	}

	private void updateValidationResult() {
		updateComponentTreeValidation(validationResultModel.getResult());
	}

	public void release() {
		validationResultModel.removePropertyChangeListener(ValidationResultModel.PROPERTY_RESULT, validationListener);
	}

	private void updateComponentTreeValidation(ValidationResult result) {
		updateComponentTreeSeverityBackground(container, result);
	}

	private final class ValidationListener implements PropertyChangeListener {

		public void propertyChange(PropertyChangeEvent evt) {
			updateComponentTreeValidation((ValidationResult) evt.getNewValue());
		}
	}
}
