package util.jgoodies.validation.range;

import static com.google.common.collect.Ranges.closed;
import static net.sf.jga.fn.arithmetic.ArithmeticFactory.getArithmetic;
import net.sf.jga.fn.arithmetic.Arithmetic;

import com.google.common.collect.Range;
import com.google.common.collect.Ranges;
import com.jgoodies.validation.Severity;
import com.jgoodies.validation.ValidationMessage;
import com.jgoodies.validation.Validator;

public class RangeUtils {

	public static <T extends Number & Comparable<T>> Validator<T> percentValidator(String name, Severity severity, Class<T> classType) {
		return percentValidator(name, severity, null, classType);
	}

	public static <T extends Number & Comparable<T>> Validator<T> percentValidator(String name, Severity severity, Object key, Class<T> classType) {
		return new RangeValidator<T>(percentRange(classType), percentValidationMessage(name, severity, key));
	}

	private static ValidationMessage percentValidationMessage(String name, Severity severity, Object key) {
		return new RangeValidationMessage(name, closed(0, 100), severity, key);
	}

	private static <T extends Number & Comparable<T>> Range<T> percentRange(Class<T> classType) {
		return percentRange(getArithmetic(classType));
	}

	private static <T extends Number & Comparable<T>> Range<T> percentRange(Arithmetic<T> arithmetic) {
		return closed(arithmetic.zero(), arithmetic.one());
	}

	public static <T extends Comparable<T>> Validator<T> atLeastValidator(String name, Severity severity, T endpoint) {
		return atLeastValidator(name, severity, null, endpoint);
	}

	public static <T extends Comparable<T>> Validator<T> atLeastValidator(String name, Severity severity, Object key, T endpoint) {
		return new RangeValidator<T>(name, Ranges.atLeast(endpoint), severity, key);
	}
}
