package util.jgoodies.validation.range;

import static com.jgoodies.validation.ValidationResult.EMPTY;
import static util.jgoodies.validation.ValidationUtils.validationResult;

import com.google.common.collect.Range;
import com.jgoodies.validation.Severity;
import com.jgoodies.validation.ValidationMessage;
import com.jgoodies.validation.ValidationResult;
import com.jgoodies.validation.Validator;

public class RangeValidator<T extends Comparable<T>> implements Validator<T> {

	private final Range<T> range;
	private final ValidationMessage validationMessage;

	public RangeValidator(String name, Range<T> range, Severity severity) {
		this(name, range, severity, null);
	}

	public RangeValidator(String name, Range<T> range, Severity severity, Object key) {
		this(range, new RangeValidationMessage(name, range, severity, key));
	}

	public RangeValidator(Range<T> range, ValidationMessage validationMessage) {
		this.range = range;
		this.validationMessage = validationMessage;
	}

	@Override
	public ValidationResult validate(T validationTarget) {
		return range.contains(validationTarget) ? EMPTY : validationResult(validationMessage);
	}
}
