package util.swing;

import static java.lang.String.format;

import java.awt.Toolkit;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.NumberFormat;
import java.text.ParseException;

import javax.swing.JFormattedTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultFormatterFactory;
import javax.swing.text.NumberFormatter;

import util.swing.text.DocumentFilter;

@SuppressWarnings("serial")
public class JFormattedTextFieldDouble extends JFormattedTextField {

	//private static final DecimalFormatSymbols decimalFormatSymbols = ((DecimalFormat) NumberFormat.getInstance()).getDecimalFormatSymbols();

	
	private DecimalFormatSymbols getDecimalFormatSymbols() {
		return ((DecimalFormat) NumberFormat.getInstance()).getDecimalFormatSymbols();
	}
	
	public JFormattedTextFieldDouble() {
		NumberFormatter numberFormatter = new NumberFormatter() {


			@Override
			public String valueToString(Object value) throws ParseException {
				if(value == null)
					return "";
				double doubleValue = ((Number) value).doubleValue();
				if(((doubleValue < 0.01) || (doubleValue > 1000)) && (doubleValue != 0))
					return removeNegativeZero(new DecimalFormat("0.###E0").format(value));
				else
					return removeNegativeZero(new DecimalFormat("0.####").format(value));
			}

			private String removeNegativeZero(String value) {
				return value.replaceAll(removeNegativeZeroRegex(), "");
			}
			
			private String removeNegativeZeroRegex() {
				return format("%s%c%s", "^-(?=0(", getDecimalFormatSymbols().getDecimalSeparator(), "0*)?$)");
			}

			@Override
			protected DocumentFilter getDocumentFilter() {
				return new DocumentFilter(new DocumentFilter.FilterFunctor() {

					@Override
					public Boolean fn(Character arg) {
						boolean allowedCharacter = Character.isDigit(arg) || allowMinusSign(arg) || allowDecimalSeparator(arg) || allowExponent(arg);
						if(!allowedCharacter)
							Toolkit.getDefaultToolkit().beep();
						return allowedCharacter;
					}

					private boolean startPosition() {
						return getCaretPosition() == 0;
					}

					private boolean allowExponent(Character arg) {
						return !startPosition() && isExponentCharacter(arg) && !getText().contains(String.valueOf(exponentCharacter()));
					}

					private boolean isExponentCharacter(Character character) {
						return character == exponentCharacter();
					}

					private char exponentCharacter() {
						return 'E';
					}

					private boolean allowDecimalSeparator(Character arg) {
						return !startPosition() && (arg == getDecimalFormatSymbols().getDecimalSeparator()) && (Character.isDigit(lastCharacter()) && !getText().contains(String.valueOf(getDecimalFormatSymbols().getDecimalSeparator())));
					}

					private Character lastCharacter() {
						return getText().isEmpty() ? null : getText().charAt(getText().length() - 1);
					}

					private boolean afterExponent() {
						return lastCharacter() == exponentCharacter();
					}

					private boolean allowMinusSign(Character arg) {
						return arg != getDecimalFormatSymbols().getMinusSign() ? false : startPosition() || afterExponent();
					}
				}) {

					@Override
					public void insertString(FilterBypass fb, int offset, String string, AttributeSet attr) throws BadLocationException {
						super.insertString(fb, offset, transformString(string), attr);
					}

					@Override
					public void replace(FilterBypass fb, int offset, int length, String text, AttributeSet attrs) throws BadLocationException {
						super.replace(fb, offset, length, transformString(text), attrs);
					}

					private String transformString(String string) {
						return string.replace('.', getDecimalFormatSymbols().getDecimalSeparator()).replace('e', 'E');
					}
				};
			}
		};
		numberFormatter.setValueClass(Double.class);
		this.setFormatterFactory(new DefaultFormatterFactory(numberFormatter));
	}
}
