package util.swing;

import java.awt.Toolkit;
import java.text.DecimalFormat;
import java.text.NumberFormat;

import javax.swing.JFormattedTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultFormatterFactory;
import javax.swing.text.NumberFormatter;

import util.swing.text.DocumentFilter;

@SuppressWarnings("serial")
public class JFormattedTextFieldInteger extends JFormattedTextField {

	private static final DecimalFormat decimalFormat = (DecimalFormat) NumberFormat.getInstance();

	public JFormattedTextFieldInteger() {
		NumberFormatter numberFormatter = new NumberFormatter() {

			@Override
			protected DocumentFilter getDocumentFilter() {
				return new DocumentFilter(new DocumentFilter.FilterFunctor() {

					@Override
					public Boolean fn(Character arg) {
						boolean allowedCharacter = Character.isDigit(arg) || allowMinusSign(arg) || allowExponent(arg);
						if(!allowedCharacter)
							Toolkit.getDefaultToolkit().beep();
						return allowedCharacter;
					}

					private boolean startPosition() {
						return getCaretPosition() == 0;
					}

					private boolean allowExponent(Character arg) {
						return !startPosition() && isExponentCharacter(arg) && !getText().contains(String.valueOf(exponentCharacter()));
					}

					private boolean isExponentCharacter(Character character) {
						return character == exponentCharacter();
					}

					private char exponentCharacter() {
						return 'E';
					}

					private boolean allowMinusSign(Character arg) {
						return startPosition() && (arg == decimalFormat.getDecimalFormatSymbols().getMinusSign());
					}
				}) {

					@Override
					public void insertString(FilterBypass fb, int offset, String string, AttributeSet attr) throws BadLocationException {
						super.insertString(fb, offset, transformString(string), attr);
					}

					@Override
					public void replace(FilterBypass fb, int offset, int length, String text, AttributeSet attrs) throws BadLocationException {
						super.replace(fb, offset, length, transformString(text), attrs);
					}

					private String transformString(String string) {
						return string.replace('e', 'E');
					}
				};
			}
		};
		numberFormatter.setValueClass(Integer.class);
		this.setFormatterFactory(new DefaultFormatterFactory(numberFormatter));
	}
}
