package util.swing;

import java.awt.Component;
import java.awt.Container;
import java.awt.Window;

import javax.swing.InputMap;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.KeyStroke;
import javax.swing.UIManager;

public class SwingUtilities {

	public static <T extends Component> T getComponentInContainer(Container container, Class<T> classType) {
		for(Component component : container.getComponents()) {
			if(classType.isInstance(component))
				return classType.cast(component);
			else if(component instanceof Container) {
				T t = getComponentInContainer((Container) component, classType);
				if(t != null)
					return t;
			}
		}
		return null;
	}

	public static <T extends Component> T getComponentInHierarchy(Container container, Class<T> classType) {
		for(; container != null; container = container.getParent()) {
			T component = getComponentInContainer(container, classType);
			if(component != null)
				return component;
		}
		return null;
	}

	public static Window getWindowAncestor(Object object) {
		return object instanceof Component ? getWindowAncestor((Component) object) : null;
	}

	public static Window getWindowAncestor(Component component) {
		if(component instanceof Window)
			return (Window) component;
		if(component instanceof JMenuItem) {
			Component invoker = ((JPopupMenu) component.getParent()).getInvoker();
			while(invoker.getParent() instanceof JPopupMenu)
				invoker = ((JPopupMenu) invoker.getParent()).getInvoker();
			return javax.swing.SwingUtilities.getWindowAncestor(invoker);
		}
		return javax.swing.SwingUtilities.getWindowAncestor(component);
	}

	public static void configureButtonFocus() {
		UIManager.put("Button.defaultButtonFollowsFocus", Boolean.TRUE);
		InputMap im = (InputMap) UIManager.getDefaults().get("Button.focusInputMap");
		Object pressedAction = im.get(KeyStroke.getKeyStroke("pressed SPACE"));
		Object releasedAction = im.get(KeyStroke.getKeyStroke("released SPACE"));
		im.put(KeyStroke.getKeyStroke("pressed ENTER"), pressedAction);
		im.put(KeyStroke.getKeyStroke("released ENTER"), releasedAction);
	}
}
