package util.swing;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Window;

public class WindowUtilities {

	public static void setCursor(Object object, Cursor cursor) {
		if(object instanceof Component)
			setCursor((Component) object, cursor);
	}

	public static void setCursor(Component component, Cursor cursor) {
		if(component != null) {
			Window window = component instanceof Window ? (Window) component : SwingUtilities.getWindowAncestor(component);
			if(window != null)
				window.setCursor(cursor);
		}
	}

	public static void setCursor(Object object, int type) {
		if(object instanceof Component)
			setCursor((Component) object, type);
	}

	public static void setCursor(Component component, int type) {
		setCursor(component, Cursor.getPredefinedCursor(type));
	}

	public static void sizeWindowOnScreen(Window window, double ratio) {
		sizeWindowOnScreen(0, window, ratio, ratio);
	}

	public static void sizeWindowOnScreen(int screen, Window window, double ratio) {
		sizeWindowOnScreen(screen, window, ratio, ratio);
	}

	public static void sizeWindowOnScreen(Window window, double horizontalRatio, double verticalRatio) {
		sizeWindowOnScreen(0, window, horizontalRatio, verticalRatio);
	}

	public static void sizeWindowOnScreen(int screen, Window window, double horizontalRatio, double verticalRatio) {
		sizeWindowOnScreen(window, horizontalRatio, verticalRatio, getScreenBounds(screen));
	}

	private static void sizeWindowOnScreen(Window window, double horizontalRatio, double verticalRatio, Rectangle bounds) {
		window.setSize(new Dimension((int) (bounds.width * horizontalRatio), (int) (bounds.height * verticalRatio)));
	}

	private static Rectangle getScreenBounds(int screen) {
		GraphicsDevice[] gs = GraphicsEnvironment.getLocalGraphicsEnvironment().getScreenDevices();
		return gs[screen < gs.length ? screen : 0].getDefaultConfiguration().getBounds();
	}

	static public void centreWindowOnScreen(Window window) {
		centreWindowOnScreen(0, window);
	}

	static public void centreWindowOnScreen(int screen, Window window) {
		centreWindowOnScreen(window, getScreenBounds(screen));
	}

	private static void centreWindowOnScreen(Window window, Rectangle screenBounds) {
		positionWindowOnScreen(window, getWindowBounds(window, screenBounds));
	}

	private static void positionWindowOnScreen(Window window, Rectangle windowBounds) {
		window.setBounds(windowBounds);
	}

	private static Rectangle getWindowBounds(Window window, Rectangle screenBounds) {
		return new Rectangle(getWindowPoint(window, screenBounds), getWindowDimension(window));
	}

	private static Dimension getWindowDimension(Window window) {
		return new Dimension(window.getWidth(), window.getHeight());
	}

	private static Point getWindowPoint(Window window, Rectangle screenBounds) {
		return new Point(getWindowX(window, screenBounds), getWindowY(window, screenBounds));
	}

	private static int getWindowX(Window window, Rectangle screenBounds) {
		return screenBounds.x + (screenBounds.width / 2) - (window.getWidth() / 2);
	}

	private static int getWindowY(Window window, Rectangle screenBounds) {
		return screenBounds.y + (screenBounds.height / 2) - (window.getHeight() / 2);
	}

	static public void positionWindowOnScreen(Window window, int x, int y) {
		positionWindowOnScreen(0, window, x, y);
	}

	static public void positionWindowOnScreen(int screen, Window window, int x, int y) {
		window.setBounds(x, y, window.getWidth(), window.getHeight());
	}

	public static void centreWindowOnScreen(int screen, Window window, double ratio) {
		centreWindowOnScreen(screen, window, ratio, ratio);
	}

	private static void centreWindowOnScreen(int screen, Window window, double horizontalRatio, double verticalRatio) {
		sizeWindowOnScreen(screen, window, horizontalRatio, verticalRatio);
		centreWindowOnScreen(screen, window);
	}

	public static void centreWindowOnScreen(Window window, double ratio) {
		centreWindowOnScreen(window, ratio, ratio);
	}

	private static void centreWindowOnScreen(Window window, double horizontalRatio, double verticalRatio) {
		centreWindowOnScreen(0, window, horizontalRatio, verticalRatio);
	}
}
