package util.swing.dialog;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.prefs.Preferences;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;


public class JOptionPaneException {

	private static final String SHOW_DETAILS = "show.details"; 
	
	public static int show(Component parent, Throwable throwable) {
		return show(parent, throwable, null, null);
	}
	
	public static int show(Component parent, Throwable throwable, String message) {
		return show(parent, throwable, message, null);
	}
	
	public static int show(Component parent, Throwable throwable, String message, String title) {
		Preferences preferences = Preferences.userNodeForPackage(JOptionPaneException.class); 
		if(preferences.getBoolean(SHOW_DETAILS, true) == true)
			return showDetails(parent, throwable, message, title);
		else
			return JOptionPane.showOptionDialog(parent, getMessage(throwable, message), title, JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE, null, null, null);
	}
	
	private static int showDetails(Component parent, final Throwable throwable, String message, String title) {
		
		// throwable trace 
		StringWriter trace = new StringWriter();
		throwable.printStackTrace(new PrintWriter(trace));

		// text component to display throwable detail
		JTextArea textArea = new JTextArea(trace.toString(), 20, 40);
		textArea.setEditable(false);
		final JScrollPane scrollPane = new JScrollPane(textArea);
		scrollPane.setBorder(BorderFactory.createTitledBorder("Exception details"));
		scrollPane.setVisible(false);

		// details button
		JCheckBox details = new JCheckBox("Show details");
		details.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				throwable.printStackTrace();
				scrollPane.setVisible(!scrollPane.isVisible());
				SwingUtilities.windowForComponent(scrollPane).pack();
			}
		});

		final JButtonOptionPane ok = new JButtonOptionPane("Ok");
		return JOptionPane.showOptionDialog(parent, new Object[] { getMessage(throwable, message), scrollPane }, title, JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE, null, new Object[] { ok/*, details*/ }, ok);
	}

	private static String getMessage(Throwable throwable, String message) {
		return message != null ? message : ((throwable.getMessage() != null) && (throwable.getMessage().isEmpty() == false)) ? throwable.getMessage() : throwable.getClass().getName();
	}
	
}
