package util.swing.dialog.validation;

import java.awt.Color;
import java.awt.Component;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JOptionPane;
import javax.swing.JTextArea;
import javax.swing.UIManager;

//import util.resources.messages.Messages;

import com.jgoodies.validation.ValidationMessage;
import com.jgoodies.validation.ValidationResult;

public class JOptionPaneValidationResult {

	public static int show(Component parent, ValidationResult validationResult) {
		return show(parent, validationResult, null);
	}

	public static int show(Component parent, ValidationResult validationResult, String title) {
		return JOptionPane.showOptionDialog(parent, createReportTextArea(validationResult), getTitle(title, validationResult), JOptionPane.DEFAULT_OPTION, getMessageType(validationResult), null, null, null);
	}

	private static String getTitle(String title, ValidationResult validationResult) {
		return title != null ? title : getValidationResultTitle(validationResult);
	}

	private static String getValidationResultTitle(ValidationResult validationResult) {
		return validationResult.hasErrors() ? /*Messages.getString*/("Error.text") : (validationResult.hasWarnings() ? /*Messages.getString*/("Warning.text") : null);
	}

	private static int getMessageType(ValidationResult validationResult) {
		return validationResult.hasErrors() ? JOptionPane.ERROR_MESSAGE : JOptionPane.WARNING_MESSAGE;
	}

	private static JTextArea createReportTextArea(ValidationResult validationResult) {
		JTextArea area = new JTextArea();
		area.setEditable(false);
		area.setFocusable(false);
		area.setOpaque(false);
		area.setBackground(new Color(0, 0, 0, 0));
		area.setBorder(BorderFactory.createEmptyBorder());
		area.setText(getMessagesText(validationResult.hasErrors() ? validationResult.getErrors() : validationResult.getWarnings()));
		area.setCaretPosition(0);
		area.setFont(UIManager.getFont("Label.font"));
		return area;
	}

	private static String getMessagesText(List<ValidationMessage> messages) {
		if(messages.isEmpty())
			return "OK";
		StringBuilder builder = new StringBuilder();
		for(ValidationMessage message : messages) {
			if(builder.length() > 0)
				builder.append("\n");
			builder.append(message.formattedText());
		}
		return builder.toString();
	}
}
