//***********************************************************************
//
//                              FuzzyGraph.java
//
// Author(s) : Anne Tireau
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.*;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Map;
import java.lang.Math;
import java.awt.Dimension;

import java.awt.BasicStroke;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import java.awt.Color;

import org.jgraph.JGraph;
import org.jgraph.graph.*;

//------------------------------------------------------------------------------
public class FuzzyGraph extends JGraph
{
	private int NbClass;					// Nombre de classes de l'arbre
	private FuzzyCell Root;					// noeud racine de l'arbre
	protected Vector <FuzzyCell> Roots;          // vecteur de cellules
	protected Vector <String> params ;			//vecteur de parametres
	Vector VariablesGroups = new Vector();    // vecteur des groupes de variables
	boolean DisplayAllCell = true;           // Affiche tous les noeuds 
	private boolean Classif = false;          // arbre classif
	private boolean Regression = false;       // arbre rgression
	private boolean FuzzyOutput = false;      // arbre flou
	final static int LIGNE = 70;	//hauteur entre 2 lignes
	final static int LARGEUR_SEP = 20;	//separation entre 2 cellules
	public final static int FONTSIZE = 10;	//taille par defaut de la police du graph
	private DefaultGraphModel Model;		// Model de larbre
	private GraphLayoutCache View;		      // vue du graph
	private int DeepGraph = 0;				//profondeur de l'arbre
	private Map nested = null;	//table d'attributs du graph
	public final static Font FONTGRAPH = new Font(GraphConstants.FONT,4,FuzzyGraph.FONTSIZE); //police par defaut du graph
	protected Object[] cellsVisibles = null; //tableau de cellule visible
	
	/*
	 * Lecture du fichier texte de l'arbre et construction des cellules et des artes
	 */
	public FuzzyGraph (String file_name) throws FileNotFoundException, IOException, Exception
	{
		// vecteur de cellules
		this.Roots = new Vector<FuzzyCell>();
		// model
		this.Model = new DefaultGraphModel(this.Roots,null);
		this.setModel(this.Model);
		// vue
		this.View = new GraphLayoutCache(this.Model, new MyCellViewFactory(),true);
		this.setGraphLayoutCache(this.View);
		// lecture du fichier
		File f = new File(file_name);
		BufferedReader br = new BufferedReader( new InputStreamReader( new FileInputStream(f) ) );
		// entte du fichier
		String buffer = br.readLine();
		this.params	 = new Vector<String>(); // paramtres sur premire ligne du fichier
	    
	    try
	      {
		    StringTokenizer st = new StringTokenizer( buffer, "," ); // decompose le buffer (sparateur ,)
		    this.NbClass = Integer.parseInt(st.nextToken().trim()); // lecture du nombre de classes
		    int class_number = 1;
		    while( st.countTokens() != 0 )
		    {
		    	String class_name = new String(st.nextToken().trim());
		    	// noms des classes (affecte un nom par dfaut si celui-ci n'est pas prsent dans le fichier arbre)
		    	this.params.add(class_name.equals("") == true ? new String("Class "+class_number) : class_name);  
		    	class_number++;
		    }
	      }
	    catch( NumberFormatException e ) { throw new Exception( LocaleFIS.GetString("ErrorInTreeFile") + ": " + file_name + "\n" + LocaleFIS.GetString("UnexpectedValueForNumberOfClass") + " : " + buffer); }
	    
        // determination du type d'arbre
        if( this.NbClass < 0 )    // arbre flou
        {
        	this.FuzzyOutput = true;
        	this.NbClass = Math.abs(this.NbClass);
        }
        else if( this.NbClass == 0 )  // arbre rgression
        	this.Regression = true;
        else if( this.NbClass > 0 )   // arbre classification
        	this.Classif = true;
                       
        this.nested = new Hashtable();
        if( (buffer = br.readLine()) != null )
        {
        try
          {        	
        	Map mapRoot = new Hashtable();
        	// init de la racine de l'arbre
	        this.Root = CreateCell(buffer,mapRoot);
	        this.Root.InitVariablesGroups(this.VariablesGroups, this.params);  // Init des noms des variables affichables
	        this.Root.setDeep(0);
	        this.nested.put(this.Root, mapRoot);                    	
	      	// autres cellules de l'arbre 
	        if((buffer = br.readLine()) != null)	        	
	        	this.CreateCellH(this.Root, buffer, br);
         }
        catch( Exception e ) { throw new Exception(LocaleFIS.GetString("ErrorInTreeFile") + ": " + file_name + "\n" + e.getMessage() ); }
        }
      	br.close(); // fermeture du fichier arbre;

      	this.setAntiAliased(true);
      	this.setGridColor(Color.WHITE);
      	
      	this.View.setVisible(this.Roots.toArray(), true);
      	this.View.setAutoSizeOnValueChange(true); //permet auto-size si la valeur des cellules change
      	
		this.cellsVisibles = this.Roots.toArray().clone();
		this.View.edit(this.nested);//pour les feuilles
		//mise en place du userObject
		this.InfoGraph();
	}

	//	------------------------------------------------------------------------------	
	// Fonctions de modification du graphe depuis l'interface
	//	------------------------------------------------------------------------------		
	/**
	 * Change l'echelle d'un graphe
	 */
	public void setScaleGraph(Double val)
	{
		// entree : nouvelle valeur du graphe (val en %)
		this.setScale(val/100);
	}
	/**
	 * Recuperation de la taille de la police utilisee pour le graph 
	 */
	public int getFontSizeGraph()
	{
		Font f = GraphConstants.getFont(this.nested);
		return f.getSize();
	}
	/**
	 * Changement de la taille de la police utilisee pour le graphe 
	 * Attention le changement de taille necessite un edit,
	 * donc perte des attibuts visibles des cellules
	 */	
	public void setFontSizeGraph(int taille)
	{
		Font font = new Font(GraphConstants.FONT,4,taille);
		Map map = new Hashtable();
		GraphConstants.setFont(map, font);
		this.View.edit(this.getRoots(), map);
		//objets visibles
		this.reappliqueVisibility();
		//placement des cellules
		this.placeCell();
	}
	/**
	 *  Fonction qui rend invisible toutes les cellules et parcours 
	 *  l'arbre en rendant visible les cellules de profondeur ok
	 *  - met  jours l'attribut cellsVisibles afin de pouvoir rappliquer 
	 *  l'invisibilit aprs un edit du graphe
	 */
	public void DisplayDeepGraph(int profondeur)
	{
		//modification de l'attribut du graphe
		this.setDeepGraph(profondeur); 
		// rend invisible toutes les cellules
		this.View.setVisible(this.getRoots(), false);
		//this.placeCell();
		this.setVisibleCell(this.Root, profondeur);
		// sav cellule visible dans l'attribut cellsVisibles du graphe
		this.cellsVisibles = this.View.getVisibleCells(this.Roots.toArray()).clone();
		// placement des cellules
		this.placeCell();
	}
	/**
	 *	Rend Visible une cellule
	 *	Fonction recursive
	 */		
	public void setVisibleCell(FuzzyCell cell,int profondeur)
	{		
		if(cell.Deep()<=profondeur) 
		{
			this.View.setVisible(cell, true);
					
			if(!cell.estFeuille())
			{
				//pour tous les ports de la cellule
				for (int i=0; i<cell.getChildCount();i++)
				{
					// port de la cellule
					DefaultPort p = (DefaultPort)cell.getChildAt(i);
					Iterator it = p.edges();
					//pour chaque lien
					while(it.hasNext())
					{
						DefaultEdge e = (DefaultEdge)it.next();
						FuzzyCell f = (FuzzyCell)this.Model.getParent(this.Model.getTarget(e));
						if(f.Deep()>cell.Deep()) this.setVisibleCell(f,profondeur);
					}
				}
			}
		}
	}
	/**
	 * Fonction qui permet, aprs un edit (perte de la propriete visible des cellules)
	 * de remettre attribut visible  true des cellules qui doivent tre vu
	 * Utilisation : dans le cas o on demande un affichage par profondeur de l'arbre
	 */
	public void reappliqueVisibility()
	{
		if(this.cellsVisibles != null && this.getRoots().length != this.cellsVisibles.length) 	
		{
			this.View.setVisible(this.getRoots(), false);
			this.View.setVisible(this.cellsVisibles, true);
		}
	}
	/**
	 *	Affiche les informations dans les vertex selon le choix des utilisateurs 
	 * 	Attention : ensemble des cellules visibles
	 */	
	public void InfoGraph()
	{				
		for (int i=0; i<this.Model.getRootCount();i++)
		{
			Object c =  this.Model.getRootAt(i);
			if(DefaultGraphModel.isVertex(this.Model, c)) 
			{
				String info = new String();
				FuzzyCell cell = (FuzzyCell)c;
				
				for(int j=0; j<this.VariablesGroups.size();j++)
				{
					DisplayVariableGroup dv = (DisplayVariableGroup)this.VariablesGroups.elementAt(j);
					if(dv.Display() && (this.DisplayAllCell || (!this.DisplayAllCell && cell.estFeuille())))
						{
							if(dv.VariablesNamesSize()==1)
								{
									info = new String(info+LocaleFIS.GetString(dv.VariableName(0))+" : ");
									info = new String(info+cell.Variables.elementAt(j+2)+"<br>");
								}
							else if(dv.VariablesNamesSize()>1) //cas des groupes de valeurs (classEffectif)
								{
									info = new String(info+LocaleFIS.GetString(dv.GroupName())+" : ");
									String vals = new String();
									for (int k=0; k<dv.VariablesNamesSize()-1; k++) //sauf la dernire valeur (pour eviter /)
									{
										vals = new String(vals+cell.Variables.elementAt(j+2+k)+"/");
									}
									info = new String(info+vals+cell.Variables.elementAt(dv.VariablesNamesSize()+2+j-1)+"<br>");
								}	
						}	
				}
				// ajout du split
				if(!cell.estFeuille() && info.length()!=0) info = new String(info+"<hr><center>"+cell.Split()+"</center>");
				else if(!cell.estFeuille() && info.length()==0) info = new String("<center>"+ cell.Split()+"</center>");
					else if(cell.estFeuille() && info.length()==0) info = new String("<center>"+LocaleFIS.GetString("Feuille")+"</center>");		
				// construction du label et assignation au userObject de la cellule
				info = new String("<html>"+info+"</html>");
				cell.setUserObject(info);
			}
		}
		// edit pour reaffichage avec bon userObject
		this.View.edit(this.nested);
		// maj des cellules invisibles
		this.reappliqueVisibility();
		// replacement des cellules
		this.placeCell();
	}
	/**
	 * Creation d'une cellule en fonction du type de graphe 
	 */
	private FuzzyCell CreateCell (String buffer, Map map) throws Exception
	{
		FuzzyCell new_cell = null;
		if( this.Classif == true )
			new_cell = new FuzzyCellClassif( buffer, this.NbClass, map);  // creation d'un noeud Classif
		if( this.Regression == true )
			new_cell = new FuzzyCellRegression( buffer, map); // creation d'un noeud Regression
		if( this.FuzzyOutput == true )
			new_cell = new FuzzyCellFuzzyOutput( buffer, this.NbClass, map);  // creation d'un noeud flou
		this.Roots.add(new_cell);   // ajout du noeud dans le vecteur des noeuds
		return new_cell;
	}
	/**
	 * Fonction qui met construit les celulles et les liens  partir du fichier de donnees
	 * Fonction rcursive
	 */	
	protected void CreateCellH(FuzzyCell pere, String buffer, BufferedReader br) throws Exception
	{
		Map map = new Hashtable();
		FuzzyCell fils = CreateCell (buffer, map);
		//verification de la parente
		if(pere.Cell() != fils.Parent())
			{
				pere = findParent(pere);
				while(pere.Cell() != fils.Parent())
					pere = findParent(pere);
			}
		this.nested.put(fils, map);
		createEdge(pere,fils);
		
		if(fils.estFeuille())
		{
			if( (buffer = br.readLine()) != null)
				this.CreateCellH(pere, buffer, br);
		}
		else 
			{
				buffer = br.readLine();
				this.CreateCellH(fils, buffer, br);
			}
	}
	/**
	 * Retourne le pere d'une cellule
	 */
	protected FuzzyCell findParent(FuzzyCell fils)
	{
		for(int i=0;i<fils.getChildCount();i++)
		{
			DefaultPort p = (DefaultPort)fils.getChildAt(i);
			Iterator it = p.edges();
			//pour chaque lien
			while(it.hasNext())
			{
				DefaultEdge e = (DefaultEdge)it.next();
				FuzzyCell f = (FuzzyCell)this.Model.getParent(this.Model.getSource(e));
				if(f.Cell()==fils.Parent()) return (FuzzyCell)f;
			}		
		}
		return null;
	}
	/**
	 * Creation du lien entre cellule
	 */
	protected void createEdge(FuzzyCell pere, FuzzyCell fils)
	{
		//ajout/maj des ports au pere et au fils
		DefaultPort source = new DefaultPort();
		pere.add(source);
		source.setParent(pere);
		DefaultPort target = new DefaultPort();
		fils.add(target);
		target.setParent(fils);				
		//creation du pont
		DefaultEdge edge = new DefaultEdge();
		edge.setSource(pere.getChildAt(0));
		edge.setTarget(fils.getChildAt(0));
		GraphConstants.setLineEnd(edge.getAttributes(), GraphConstants.ARROW_CLASSIC);
		GraphConstants.setEndFill(edge.getAttributes(), true);
		GraphConstants.setAutoSize(edge.getAttributes(), true);
		Font font = new Font(GraphConstants.FONT,1,FuzzyGraph.FONTSIZE);
		GraphConstants.setFont(edge.getAttributes(), font);
		GraphConstants.setOpaque(edge.getAttributes(), true);
		GraphConstants.setLineColor(edge.getAttributes(), new Color(10, 79, 104));
		//GraphConstants.setMoveable(edge.getAttributes(), false);
		GraphConstants.setDisconnectable(edge.getAttributes(), false);
		//mise  jour des attributs des cellules
		fils.setDeep(pere.Deep()+1);
		if(this.getDeepGraph()<fils.Deep()) this.setDeepGraph(fils.Deep());
		pere.setSplit((String)fils.Variables.elementAt(0));
		edge.setUserObject(fils.Variables.elementAt(1));
		
		//ajout de l'edge au model
		this.View.insertEdge(edge, source, target);	
	}
	/**
	 * Retourne un vecteur de cellule de mme profondeur
	 */
	public void findBrothersSameDeep(Vector<FuzzyCell> brothers, FuzzyCell cell, int profondeur)
	{
		if(cell.Deep() == profondeur) brothers.add(cell);
		else
		{
			for(int i=0;i<cell.getChildCount();i++)
			{
				// port de la cellule
				DefaultPort p = (DefaultPort)cell.getChildAt(i);
				Iterator it = p.edges();
				//pour chaque lien
				while(it.hasNext())
				{					
					DefaultEdge e = (DefaultEdge)it.next();
					FuzzyCell f = (FuzzyCell)this.Model.getParent(this.Model.getTarget(e));
					if(f.Deep()>cell.Deep()) findBrothersSameDeep(brothers, f, profondeur);						
				}			
			}
		}
	}
	/**
	 * Pour chaque nsemble de frere, calcul de la largeur maximum
	 * retourne la plus grande
	 */
	public double getLargeurMax(int deep)
	{
		double largeur = 0;
		
		for (int i=0; i<=deep; i++)
		{		
			Vector<FuzzyCell> brothers = new Vector <FuzzyCell>();
			findBrothersSameDeep(brothers,  this.Root, i);
			double largTmp = 0;
			double lg = 0;
			for(int j=0;j<brothers.size();j++)
			{
				lg = this.getCellBounds(brothers.elementAt(j)).getWidth();
				largTmp = largTmp + lg + LARGEUR_SEP;
			}
			largeur = Math.max(largeur,largTmp);
		}
		return largeur;
	}
	// ------------------------------------------------------------------------------
	public int NbClass() { return this.NbClass; }
	// ------------------------------------------------------------------------------
	public boolean Classif() { return this.Classif; }
	// ------------------------------------------------------------------------------
	public boolean Regression() { return this.Regression; }
	// ------------------------------------------------------------------------------
	public boolean FuzzyOutput() { return this.FuzzyOutput; }	
	/**
	 * Change l'attribut profondeur de l'arbre 
	 */
	protected void setDeepGraph(int deep) { this.DeepGraph = deep; }
	// ------------------------------------------------------------------------------	  
	public int getDeepGraph() { return this.DeepGraph; }		
	// ------------------------------------------------------------------------------
	protected void placeCell()
	{
		//visibilite
		Object[] cells = this.View.getVisibleCells(this.getRoots());
		
		// position des cellules
		  double posY = LIGNE; // debut du graph
		  int deep = this.getDeepGraph();
		  double largeurMax = this.getLargeurMax(deep)+200; // largeur la plus large de l'tage
		  FuzzyCell cell = null;
		  
		  for (int d=0; d<=deep; d++)
		  {	
			  Vector<FuzzyCell> fuzzyBrothers = new Vector <FuzzyCell>();
			  this.findBrothersSameDeep(fuzzyBrothers, this.Root, d); //vecteur de cell de mme profondeur 
			  int nbBrothers = fuzzyBrothers.size(); //nb freres pour un palier
			  double maxHauteur = 0;
			  for(int b=0; b<nbBrothers; b++)
			  {
				  Map map =  new Hashtable();
				  cell = (FuzzyCell)fuzzyBrothers.elementAt(b);
				  double largeur = this.getCellBounds(cell).getWidth();
				  double hauteur = this.getCellBounds(cell).getHeight();
				  maxHauteur = Math.max(maxHauteur, hauteur);
				  double zone = (largeurMax/nbBrothers);
				  double posX = (zone/2)-(largeur/2)+(zone*b); 
				  Rectangle2D.Double rect = new Rectangle2D.Double(posX,posY,largeur,hauteur);
				  GraphConstants.setBounds(map, rect);
				  GraphConstants.setBorderColor(map, new Color(10, 79, 104));
				  nested.put(cell,map);
			  }
			  //maj des hauteurs...
			  posY = posY + LIGNE + maxHauteur; 
			  //position ordre
			  this.View.toFront(fuzzyBrothers.toArray());
		  }	
		  this.View.edit(this.nested); //edit de la diposition
		  if(this.getRoots().length != cells.length) 	
		  {
			  this.View.setVisible(this.getRoots(), false);
			  this.View.setVisible(cells, true);
		  }
		  
	  } 
}//fin class


/**
*  
*/
class FuzzyCell extends DefaultGraphCell
{
	private int NumCell;                 // numero du noeud
	private int ParentCell;               // numero du noeud parent
	protected int Deep;			      // profondeur du noeud
	private int Feuille;			// feuille de l'arbre 0 non - 1 oui	
	protected Vector Variables = new Vector();  // vecteur des variables du noeud
	protected StringTokenizer st;
	private String split;
	// ------------------------------------------------------------------------------
	public FuzzyCell( String buffer , Map map) throws Exception
	{
		this.split = null;
		st = new StringTokenizer(buffer, "," ); // decompose le buffer
	    if( st.countTokens() < 7 )
	        throw new Exception( LocaleFIS.GetString("UnexpectedFormat") + " : " + buffer );
	    
	    // lecture du noeud (partie commune  tous les types de noeuds)
	    this.NumCell = Integer.parseInt(st.nextToken().trim());  // lecture du numero du noeud
	    // variable et SEF
	    if(this.NumCell == 1)
	    {
		    this.Variables.add("Root");  // lecture du label variable
		    this.Variables.add(new String(" "));  // lecture label sef	  
		    st.nextToken();
		    st.nextToken();
	    }
	    else
	    {
	    	this.Variables.add(new String( st.nextToken().trim()));  // lecture du label variable
	    	this.Variables.add(new String( st.nextToken().trim()));  // lecture label sef
	    }
	    // pre
	    this.ParentCell = Integer.parseInt(st.nextToken().trim());  // lecture du numero du noeud parent
	    this.Feuille = (Integer.parseInt(st.nextToken().trim()));// saute le label feuille
	    this.Variables.add( new Integer(st.nextToken().trim()) ); // lecture du nombre d'exemples
	    this.Variables.add( (Math.floor((new Double(st.nextToken().trim())*100+0.5))/100 ));  // lecture entropie
	    
	    this.setUserObject(this.NumCell);  
	    GraphConstants.setBounds(map,new Rectangle2D.Double(20, 20, 40, 20));
	    GraphConstants.setOpaque(map, true);
	    GraphConstants.setBorderColor(map, Color.BLACK);
	    GraphConstants.setInset(map, 5);
	    GraphConstants.setAutoSize(map, true);	 
	    GraphConstants.setFont(map, FuzzyGraph.FONTGRAPH);
	    GraphConstants.setBackground(map, new Color(199, 215, 225));
	}
	// ------------------------------------------------------------------------------
	public int Parent()
	{
		return this.ParentCell;
	}
	// ------------------------------------------------------------------------------
	public int Cell() { return this.NumCell;}	
	// ------------------------------------------------------------------------------
	protected int Deep() { return this.Deep;}
	// ------------------------------------------------------------------------------
	protected void setDeep(int d) {	this.Deep = d;}	
	// ------------------------------------------------------------------------------
	// rempli le vecteur display_variables avec le nom des variables
	public void InitVariablesGroups( Vector display_variables, Vector params )
	{
		DisplayVariableGroup dvg = new DisplayVariableGroup(new String("Samples"));
		dvg.Display(false);
		display_variables.add(dvg);
	}
	// ------------------------------------------------------------------------------
	public void setSplit(String split) { this.split = split;}
	// ------------------------------------------------------------------------------
	public String Split() { return this.split;}
	// ------------------------------------------------------------------------------
	public boolean estFeuille()
	{
		if(this.Feuille == 0) return false;
		return true;
	}
}

/**
*  classe noeud classif
*/
class FuzzyCellClassif extends FuzzyCell
{
	private int nb_class; // nombre de classes pour la classif
	//------------------------------------------------------------------------------
	public FuzzyCellClassif(  String buffer, int nb_class, Map map) throws Exception
	 {
	 super( buffer, map);
	 this.nb_class = nb_class;
	
	 // lecture du noeud
	 this.Variables.add(new Integer(st.nextToken().trim()) );  // lecture de la classe majoritaire
	 for( int i=0 ; i<nb_class ; i++ )
	   this.Variables.add((Math.floor((new Double(st.nextToken().trim())*100+0.5))/100 ));  // lecture des effectifs par classe
	 }
	//------------------------------------------------------------------------------
	//rempli le vecteur display_variables avec le nom des variables
	public void InitVariablesGroups( Vector display_variables, Vector params )
 {
	super.InitVariablesGroups(display_variables, params);
	DisplayVariableGroup dvg = new DisplayVariableGroup(new String("Entropie"));
	dvg.Display(false);
	display_variables.add(dvg);
	DisplayVariableGroup dvg2 = new DisplayVariableGroup(new String("MajorityClass"), new String("Majority\nClass"));
	dvg2.Display(false);
	display_variables.add(dvg2);
	// les effectifs par classe sont grs par un groupe de variables ClassEffectif
	DisplayVariableGroup new_dv = new DisplayVariableGroup(new String("ClassPourcentage"), (String)params.elementAt(0));
	new_dv.Display(false);
	for( int i=1 ; i<this.nb_class ; i++ )
		new_dv.AddVariableName(Integer.toString(i+1));
	display_variables.add(new_dv);
 }
}

/**
 *  classe fispro regression
*/
class FuzzyCellRegression extends FuzzyCell
{
	public FuzzyCellRegression( String buffer , Map map ) throws Exception
	 {
		 super( buffer, map );
		
		 // lecture du noeud
		 this.Variables.add((Math.floor((new Double(st.nextToken().trim())*100+0.5))/100 ) ); // lecture moyenne
		 this.Variables.add((Math.floor((new Double(st.nextToken().trim())*100+0.5))/100 ) ); // lecture ecart-type
	 }
	//------------------------------------------------------------------------------
	//rempli le vecteur display_variables avec le nom des variables
	public void InitVariablesGroups( Vector display_variables, Vector params )
	 {
		super.InitVariablesGroups(display_variables, params);
		DisplayVariableGroup dvg = new DisplayVariableGroup(new String("Deviance"));
		dvg.Display(false);
		display_variables.add(dvg);
		DisplayVariableGroup dvg2 = new DisplayVariableGroup(new String("Mean"));
		dvg2.Display(false);
		display_variables.add(dvg2);
		DisplayVariableGroup dvg3 = new DisplayVariableGroup(new String("STD"));
		dvg3.Display(false);
		display_variables.add(dvg3);
	 }
}

/**
 *  classe noeud flou
 */
class FuzzyCellFuzzyOutput extends FuzzyCell
{
	private int nb_class; // nombre de classe (en fait nombre de sef dans ce cas)
	//------------------------------------------------------------------------------
	public FuzzyCellFuzzyOutput(  String buffer, int nb_class , Map map) throws Exception
	 {
		 super( buffer,map);
		 this.nb_class = nb_class;		
		 // lecture du noeud
		 this.Variables.add(new String(st.nextToken().trim()));  // lecture sef majoritaire
		 for( int i=0 ; i<nb_class ; i++ )
			 this.Variables.add( (Math.floor((new Double(st.nextToken().trim())*100+0.5))/100 )); // lecture de l'effectif par sef
 }
	//------------------------------------------------------------------------------
	//rempli le vecteur display_variables avec le nom des variables
	public void InitVariablesGroups( Vector display_variables, Vector params )
	 {
		super.InitVariablesGroups(display_variables, params);
		DisplayVariableGroup dvg = new DisplayVariableGroup(new String("Entropie"));
		dvg.Display(false);
		display_variables.add(dvg);
		// le nom de la variable du groupe MajorityClass est affich sur 2 lignes
		DisplayVariableGroup dvg2 = new DisplayVariableGroup(new String("MajorityClass"), new String("Majority\nClass"));
		dvg2.Display(false);
		display_variables.add(dvg2);
		// les effectifs par sef sont grs par un groupe de variables ClassEffectif
		DisplayVariableGroup new_dv = new DisplayVariableGroup(new String("ClassPourcentage"), (String)params.elementAt(0));
		new_dv.Display(false);
		for( int i=1 ; i<this.nb_class ; i++ )
			new_dv.AddVariableName((String)params.elementAt(i));
		display_variables.add(new_dv);
	 }
}

/**
 *  
 */
class MyCellViewFactory extends DefaultCellViewFactory
{
	protected EdgeView createEdgeView(Object cell) { return new EdgeView(cell); }
	   
	protected PortView createPortView(Object cell) { return new PortView(cell); }

	protected VertexView createVertexView(Object cell) 
	{
	      if(((FuzzyCell)cell).estFeuille()) return new JGraphEllipseView(cell); 	  
	      else return new VertexView(cell);
	} 
}


/**
 * @author Gaudenz Alder
 * package com.jgraph.example.mycellview;  
 */
class JGraphEllipseView extends VertexView 
{
	public static transient JGraphEllipseRenderer renderer = new JGraphEllipseRenderer();

	public JGraphEllipseView() {super();}

	public JGraphEllipseView(Object cell) {super(cell);	}

	/**
	 * Returns the intersection of the bounding rectangle and the
	 * straight line between the source and the specified point p.
	 * The specified point is expected not to intersect the bounds.
	 */
	public Point2D getPerimeterPoint(EdgeView edge, Point2D source, Point2D p) {
		Rectangle2D r = getBounds();

		double x = r.getX();
		double y = r.getY();
		double a = (r.getWidth() + 1) / 2;
		double b = (r.getHeight() + 1) / 2;

		// x0,y0 - center of ellipse
		double x0 = x + a;
		double y0 = y + b;

		// x1, y1 - point
		double x1 = p.getX();
		double y1 = p.getY();

		// calculate straight line equation through point and ellipse center
		// y = d * x + h
		double dx = x1 - x0;
		double dy = y1 - y0;

		if (dx == 0)
			return new Point((int) x0, (int) (y0 + b * dy / Math.abs(dy)));

		double d = dy / dx;
		double h = y0 - d * x0;

		// calculate intersection
		double e = a * a * d * d + b * b;
		double f = -2 * x0 * e;
		double g = a * a * d * d * x0 * x0 + b * b * x0 * x0 - a * a * b * b;

		double det = Math.sqrt(f * f - 4 * e * g);

		// two solutions (perimeter points)
		double xout1 = (-f + det) / (2 * e);
		double xout2 = (-f - det) / (2 * e);
		double yout1 = d * xout1 + h;
		double yout2 = d * xout2 + h;

		double dist1Squared = Math.pow((xout1 - x1), 2)
				+ Math.pow((yout1 - y1), 2);
		double dist2Squared = Math.pow((xout2 - x1), 2)
				+ Math.pow((yout2 - y1), 2);

		// correct solution
		double xout, yout;

		if (dist1Squared < dist2Squared) {
			xout = xout1;
			yout = yout1;
		} else {
			xout = xout2;
			yout = yout2;
		}

		return getAttributes().createPoint(xout, yout);
	}

	public CellViewRenderer getRenderer() {return renderer;}

	public static class JGraphEllipseRenderer extends VertexRenderer {

		/**
		 * Return a slightly larger preferred size than for a rectangle.
		 */
		public Dimension getPreferredSize() {
			Dimension d = super.getPreferredSize();
			d.width += d.width / 8;
			d.height += d.height / 2;
			return d;
		}

		/**
		 */
		public void paint(Graphics g) {
			int b = borderWidth;
			Graphics2D g2 = (Graphics2D) g;
			Dimension d = getSize();
			boolean tmp = selected;
			if (super.isOpaque()) {
				g.setColor(super.getBackground());
				if (gradientColor != null && !preview) {
					setOpaque(false);
					g2.setPaint(new GradientPaint(0, 0, getBackground(),
							getWidth(), getHeight(), gradientColor, true));
				}
				g.fillOval(b - 1, b - 1, d.width - b, d.height - b);
			}
			try {
				setBorder(null);
				setOpaque(false);
				selected = false;
				super.paint(g);
			} finally {
				selected = tmp;
			}
			if (bordercolor != null) {
				g.setColor(bordercolor);
				g2.setStroke(new BasicStroke(b));
				g.drawOval(b - 1, b - 1, d.width - b, d.height - b);
			}
			if (selected) {
				g2.setStroke(GraphConstants.SELECTION_STROKE);
				g.setColor(highlightColor);
				g.drawOval(b - 1, b - 1, d.width - b, d.height - b);
			}
		}
	}
}

