//***********************************************************************
//
//                              FuzzyTreeTableModel.java
//
// Author(s) : Jean-luc LABLEE
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//
//**********************************************************************
package fis;

import java.util.*;
import util.treetable.*;
import javax.swing.table.*;

//------------------------------------------------------------------------------
public class FuzzyTreeTableModel extends AbstractTreeTableModel
{
  protected FuzzyTree fuzzy_tree;
//------------------------------------------------------------------------------
  public FuzzyTreeTableModel( FuzzyTree fuzzy_tree )
  {
  super(fuzzy_tree.getModel().getRoot());
  this.fuzzy_tree = fuzzy_tree;
  }
//------------------------------------------------------------------------------
  protected Object[] getChildren(Object node)
  {
  FuzzyNode fn = (FuzzyNode)node;
  Object [] objs = new Object[fn.getChildCount()];
  for( int i=0 ; i<objs.length ; i++ )
    objs[i] = fn.getChildAt(i);
  return objs;
  }
//------------------------------------------------------------------------------
  public int getChildCount(Object node)
  {
  Object[] children = getChildren(node);
  return (children == null) ? 0 : children.length;
  }
//------------------------------------------------------------------------------
  public Object getChild(Object node, int i) { return getChildren(node)[i]; }
//------------------------------------------------------------------------------
// retourne le nombre de colonnes
  public int getColumnCount()
  {
  int count = 1;  // la premiere colonne est toujours affichee
  for( int i=0 ; i<this.fuzzy_tree.VariablesGroups.size() ; i++ ) // parmis tous les groupes de variable
    {
    DisplayVariableGroup dvg = (DisplayVariableGroup)this.fuzzy_tree.VariablesGroups.elementAt(i);
    if( dvg.Display() == true ) // si le groupe de variable est affichable
      count += dvg.VariablesNamesSize();  // ajout du groupe au nombre de colonnes
    }
  return count;
  }
//------------------------------------------------------------------------------
// retourne l enom des colonnes
  public String getColumnName(int column)
  {
  if( column == 0 ) // la premiere colonne n'a pas de nom
    return new String();

  int column_index = 0;
  for( int i=0 ; i<this.fuzzy_tree.VariablesGroups.size() ; i++ ) // parmis tous les groupes de variables
    {
    DisplayVariableGroup dvg = (DisplayVariableGroup)this.fuzzy_tree.VariablesGroups.elementAt(i);
    if( dvg.Display() == true ) // si le groupe est affichable
      {
      for( int j=0 ; j<dvg.VariablesNamesSize() ; j++ )
        {
        column_index++;
        if( column_index == column )
          return dvg.VariableName(j); // retourne le nom de la colonne
        }
      }
    }
  return new String();
  }
//------------------------------------------------------------------------------
  public Class getColumnClass(int column)
  {
  if( column == 0 )   // seule la premiere colonne est de type TreeTable
    return TreeTableModel.class;
  return null;
  }
//------------------------------------------------------------------------------
// retourne la valeur de la colonne column
  public Object getValueAt(Object node, int column)
  {
  int variable_index = 0;
  int column_index = 0;
  // si le noeud n'est pas une feuille et qu'on n'affiche que les feuilles
  if( (((FuzzyNode)node).isLeaf() == false) && (this.fuzzy_tree.DisplayAllNodes == false) )
    return new String();  // retourne une chaine vide
  else
    for( int i=0 ; i<this.fuzzy_tree.VariablesGroups.size() ; i++ ) // parmis tous les groupes de variables
      {
      DisplayVariableGroup dvg = (DisplayVariableGroup)this.fuzzy_tree.VariablesGroups.elementAt(i);
      for( int j=0 ; j<dvg.VariablesNamesSize() ; j++ )
        {
        variable_index++;
        if( dvg.Display() == true )
          column_index++;
        if( column_index == column )
          return ((FuzzyNode)node).Variables.elementAt(variable_index); // retourne la variable
        }
      }
  return new String();
  }
}



